=== modified file '.bzrignore'
--- .bzrignore	2005-10-03 16:53:39 +0000
+++ .bzrignore	2005-11-25 04:36:54 +0000
@@ -19,4 +19,3 @@
 ./ID
 ./tags
 .bzr-shelf
-bzrlib/plugins/*

=== modified file '.rsyncexclude'
--- .rsyncexclude	2005-05-05 06:38:18 +0000
+++ .rsyncexclude	2005-11-22 04:30:14 +0000
@@ -15,3 +15,9 @@
 /tmp
 # do want this after all
 + CHANGELOG
+/build
+test*.tmp
+.*.swp
+*.orig
+.*.orig
+.bzr-shelf*

=== added file 'BRANCH.TODO'
--- BRANCH.TODO	1970-01-01 00:00:00 +0000
+++ BRANCH.TODO	2006-01-10 06:27:04 +0000
@@ -0,0 +1,3 @@
+# This file is for listing TODOs for branches that are being worked on.
+# It should ALWAYS be empty in the mainline or in integration branches.
+# 

=== modified file 'HACKING'
--- HACKING	2005-10-06 22:15:52 +0000
+++ HACKING	2006-01-05 22:30:59 +0000
@@ -21,7 +21,11 @@
   core improvements should be tested closer to the code that is doing the
   work. Command line level tests should be placed in 'blackbox.py'.
 
-* Before fixing a bug, write a test case so that it does not regress.
+* Try to practice Test-Driven Development.  before fixing a bug, write a
+  test case so that it does not regress.  Similarly for adding a new
+  feature: write a test case for a small version of the new feature before
+  starting on the code itself.  Check the test fails on the old code, then
+  add the feature or fix and check it passes.
 
 * Exceptions should be defined inside bzrlib.errors, so that we can
   see the whole tree at a glance.
@@ -34,18 +38,55 @@
 * Module names should always be given fully-qualified,
   i.e. ``bzrlib.hashcache`` not just ``hashcache``.
 
+* Commands should return non-zero when they encounter circumstances that
+  the user should really pay attention to - which includes trivial shell
+  pipelines.
+
+  Recommended values are 
+    0- OK, 
+    1- Conflicts in merge-like operations, or changes are present in
+       diff-like operations. 
+    2- Unrepresentable diff changes (i.e. binary files that we cannot show 
+       a diff of).
+    3- An error or exception has occurred.
+
 Evolving interfaces
 -------------------
 
-If you change the behaviour of an API in an incompatible way, please
-be sure to change its name as well. For instance, if I add a keyword
-parameter to branch.commit - that's fine. On the other hand, if I add
-a keyword parameter to branch.commit which is a *required* transaction
-object, I should rename the api - i.e. to 'branch.commit_transaction'.
-
-This will prevent users of the old api getting surprising results. 
-Instead, they will get an Attribute error as the api is missing, and
-will know to update their code. If in doubt, just ask on #bzr.
+We have a commitment to 6 months API stability - any supported symbol in a
+release of bzr MUST NOT be altered in any way that would result in
+breaking existing code that uses it. That means that method names,
+parameter ordering, parameter names, variable and attribute names etc must
+not be changed without leaving a 'deprecated forwarder' behind. This even
+applies to modules and classes.
+
+If you wish to change the behaviour of a supported API in an incompatible
+way, you need to change its name as well. For instance, if I add a optional keyword
+parameter to branch.commit - that's fine. On the other hand, if I add a
+keyword parameter to branch.commit which is a *required* transaction
+object, I should rename the API - i.e. to 'branch.commit_transaction'. 
+
+When renaming such supported API's, be sure to leave a deprecated_method (or
+_function or ...) behind which forwards to the new API. See the
+bzrlib.symbol_versioning module for decorators that take care of the
+details for you - such as updating the docstring, and issuing a warning
+when the old api is used.
+
+For unsupported API's, it does not hurt to follow this discipline, but its
+not required. Minimally though, please try to rename things so that
+callers will at least get an AttributeError rather than weird results.
+
+
+Standard parameter types
+------------------------
+
+There are some common requirements in the library: some parameters need to be
+unicode safe, some need byte strings, and so on. At the moment we have
+only codified one specific pattern: Parameters that need to be unicode
+should be check via 'bzrlib.osutils.safe_unicode'. This will coerce the
+input into unicode in a consistent fashion, allowing trivial strings to be
+used for programmer convenience, but not performing unpredictably in the
+presence of different locales.
 
 Documentation
 =============
@@ -53,6 +94,9 @@
 If you change the behaviour of a command, please update its docstring
 in bzrlib/commands.py.  This is displayed by the 'bzr help' command.
 
+NEWS file
+---------
+
 If you make a user-visible change, please add a note to the NEWS file.
 The description should be written to make sense to someone who's just
 a user of bzr, not a developer: new functions or classes shouldn't be
@@ -60,6 +104,20 @@
 bugs should be listed.  See the existing entries for an idea of what
 should be done.
 
+Within each release, entries in the news file should have the most
+user-visible changes first.  So the order should be approximately:
+
+ * changes to existing behaviour - the highest priority because the 
+   user's existing knowledge is incorrect
+ * new features - should be brought to their attention
+ * bug fixes - may be of interest if the bug was affecting them, and
+   should include the bug number if any
+ * major documentation changes
+ * changes to internal interfaces
+
+People who made significant contributions to each change are listed in
+parenthesis.  This can include reporting bugs (particularly with good
+details or reproduction recipes), submitting patches, etc.
 
 API documentation
 -----------------
@@ -119,6 +177,28 @@
 (not ``x2y`` as occurs in some old code.)
 
 
+Destructors
+-----------
+
+Python destructors (``__del__``) work differently to those of other
+languages.  In particular, bear in mind that destructors may be called
+immediately when the object apparently becomes unreferenced, or at some
+later time, or possibly never at all.  Therefore we have restrictions on
+what can be done inside them.
+
+ 0. Never use a __del__ method without asking Martin/Robert first.
+
+ 1. Never rely on a ``__del__`` method running.  If there is code that
+    must run, do it from a ``finally`` block instead.
+
+ 2. Never ``import`` from inside a ``__del__`` method, or you may crash the
+    interpreter!!
+
+ 3. In some places we raise a warning from the destructor if the object
+    has not been cleaned up or closed.  This is considered OK: the warning
+    may not catch every case but it's still useful sometimes.
+
+
 Writing output
 ==============
 
@@ -178,7 +258,10 @@
 Errors and exceptions
 =====================
 
-[write me]
+Errors are handled through Python exceptions.  They can represent user
+errors, environmental errors or program bugs.  Sometimes we can't be sure
+at the time it's raised which case applies.  See bzrlib/errors.py for 
+details on the error-handling practices.
 
 
 Jargon
@@ -188,3 +271,49 @@
     Integer identifier for a revision on the main line of a branch.
     Revision 0 is always the null revision; others are 1-based
     indexes into the branch's revision history.
+
+
+Merge/review process
+====================
+
+If you'd like to propose a change, please post to the
+bazaar-ng@lists.canonical.com list with a patch, bzr changeset, or link to a
+branch.  Please put '[patch]' in the subject so we can pick them out, and
+include some text explaining the change.  Remember to put an update to the NEWS
+file in your diff, if it makes any changes visible to users or plugin
+developers.  Please include a diff against mainline if you're giving a link to
+a branch.
+
+Please indicate if you think the code is ready to merge, or if it's just a
+draft or for discussion.  If you want comments from many developers rather than
+to be merged, you can put '[rfc]' in the subject lines.
+
+Anyone is welcome to review code.  There are broadly three gates for
+code to get in:
+
+ * Doesn't reduce test coverage: if it adds new methods or commands,
+   there should be tests for them.  There is a good test framework
+   and plenty of examples to crib from, but if you are having trouble
+   working out how to test something feel free to post a draft patch
+   and ask for help.
+
+ * Doesn't reduce design clarity, such as by entangling objects
+   we're trying to separate.  This is mostly something the more
+   experienced reviewers need to help check.
+
+ * Improves bugs, features, speed, or code simplicity.
+
+Code that goes in should pass all three.
+
+If you read a patch please reply and say so.  We can use a numeric scale
+of -1, -0, +0, +1, meaning respectively "really don't want it in current
+form", "somewhat uncomfortable", "ok with me", and "please put it in".
+Anyone can "vote".   (It's not really voting, just a terse expression.)
+
+If something gets say two +1 votes from core reviewers, and no
+vetos, then it's OK to come in.  Any of the core developers can bring it
+into their integration branch, which I'll merge regularly.  (If you do
+so, please reply and say so.)
+
+
+:: vim:tw=74:ai

=== added file 'INSTALL'
--- INSTALL	1970-01-01 00:00:00 +0000
+++ INSTALL	2005-10-19 07:04:01 +0000
@@ -0,0 +1,26 @@
+bzr install instructions
+************************
+
+Dependencies
+------------
+
+bzr requires Python2.4 and cElementTree:
+
+  http://effbot.org/zone/element-index.htm
+
+If you wish to access branches over sftp, you will need paramiko and
+pycrypto:
+
+  http://www.lag.net/paramiko/
+
+Installation
+------------
+
+To install bzr as a user, run
+
+  python setup.py install --home ~ 
+
+To install system-wide, run (as root)
+
+  python setup.py install
+

=== modified file 'Makefile'
--- Makefile	2005-09-15 13:34:01 +0000
+++ Makefile	2006-01-06 12:18:04 +0000
@@ -1,14 +1,18 @@
-all: executables
-
-executables:
-	chmod u+x ./bzr
-	chmod u+x ./setup.py
+all: 
 
 check:
-	./bzr selftest
-
-clean: executables
+	./bzr selftest $(tests)
+	@echo "Running all tests with no locale."
+	LC_CTYPE= LANG=C LC_ALL= ./bzr selftest $(tests)
+
+check-msgeditor:
+	./bzr --no-plugins selftest -v msgeditor
+
+clean: 
 	./setup.py clean
+	-find . -name "*.pyc" | xargs rm
+	-find . -name "*.pyo" | xargs rm
+	rm -rf test????.tmp
 
 .PHONY: all
 
@@ -17,3 +21,6 @@
 tag_files=./bzr ./bzrlib/*py ./bzrlib/selftest/*.py
 TAGS: $(tag_files)
 	ctags-exuberant -e $(tag_files)
+
+tutorial.html: tutorial.txt
+	rest2html tutorial.txt > tutorial.html

=== modified file 'NEWS'
--- NEWS	2005-10-06 13:08:14 +0000
+++ NEWS	2006-01-10 04:08:39 +0000
@@ -1,5 +1,520 @@
-bzr 0.1 2005-10-06
-
+bzr 0.7rc1 2006-01-09
+
+  CHANGES:
+
+    * .bzrignore is excluded from exports, on the grounds that it's a bzr 
+      internal-use file and may not be wanted.  (Jamie Wilkinson)
+
+    * The "bzr directories" command were removed in favor of the new
+      --kind option to the "bzr inventory" command.  To list all 
+      versioned directories, now use "bzr inventory --kind directory".  
+      (Johan Rydberg)
+
+    * Under Windows configuration directory is now %APPDATA%\bazaar\2.0
+      by default. (John Arbash Meinel)
+
+    * The parent of Bzr configuration directory can be set by BZR_HOME
+      environment variable. Now the path for it is searched in BZR_HOME, then
+      in HOME. Under Windows the order is: BZR_HOME, APPDATA (usually
+      points to C:\Documents and Settings\User Name\Application Data), HOME.
+      (John Arbash Meinel)
+
+    * Plugins with the same name in different directories in the bzr plugin
+      path are no longer loaded: only the first successfully loaded one is
+      used. (Robert Collins)
+
+    * Use systems' external ssh command to open connections if possible.  
+      This gives better integration with user settings such as ProxyCommand.
+      (James Henstridge)
+
+    * Sftp paths can now be relative, or local, according to the IETF 
+      draft spec.  Paths now take the form:
+      sftp://user:pass@host:port/relative/path
+      or
+      sftp://user:pass@host:port/%2Fabsolute/path
+      or
+      sftp://user:pass@host:port//absolute/path
+
+    * Permissions on files underneath .bzr/ are inherited from the .bzr 
+      directory. So for a shared repository, simply doing 'chmod -R g+w .bzr/'
+      will mean that future file will be created with group write permissions.
+
+    * configure.in and config.guess are no longer in the builtin default 
+      ignore list.
+
+    * '.sw[nop]' pattern ignored, to ignore vim swap files for nameless
+      files.  (John Arbash Meinel, Martin Pool)
+
+  IMPROVEMENTS:
+
+    * "bzr INIT dir" now initializes the specified directory, and creates 
+      it if it does not exist.  (John Arbash Meinel)
+
+    * New remerge command (Aaron Bentley)
+
+    * Better zsh completion script.  (Steve Borho)
+
+    * 'bzr diff' now returns 1 when there are changes in the working 
+      tree. (Robert Collins)
+
+    * 'bzr push' now exists and can push changes to a remote location. 
+      This uses the transport infrastructure, and can store the remote
+      location in the ~/.bazaar/branches.conf configuration file.
+      (Robert Collins)
+
+    * Test directories are only kept if the test fails and the user requests
+      that they be kept.
+
+    * Tweaks to short log printing
+
+    * Added branch nicks, new nick command, printing them in log output. 
+      (Aaron Bentley)
+
+    * If $BZR_PDB is set, pop into the debugger when an uncaught exception 
+      occurs.  (Martin Pool)
+
+    * Accept 'bzr resolved' (an alias for 'bzr resolve'), as this is
+      the same as Subversion.  (Martin Pool)
+
+    * New ftp transport support (on ftplib), for ftp:// and aftp:// 
+      URLs.  (Daniel Silverstone)
+
+    * Commit editor temporary files now start with 'bzr_log.', to allow 
+      text editors to match the file name and set up appropriate modes or 
+      settings.  (Magnus Therning)
+
+    * Improved performance when integrating changes from a remote weave.  
+      (Goffredo Baroncelli)
+
+    * Sftp will attempt to cache the connection, so it is more likely that
+      a connection will be reused, rather than requiring multiple password
+      requests.
+
+    * bzr revno now takes an optional argument indicating the branch whose
+      revno should be printed.  (Michael Ellerman)
+
+    * bzr cat defaults to printing the last version of the file.  
+      (#3632, Matthieu Moy)
+
+    * New global option 'bzr --lsprof COMMAND' runs bzr under the lsprof 
+      profiler.  (Denys Duchier)
+
+    * Faster commits by reading only the headers of affected weave files. 
+      (Denys Duchier)
+
+    * 'bzr add' now takes a --dry-run parameter which shows you what would be
+      added, but doesn't actually add anything. (Michael Ellerman)
+
+    * bzr add now lists how many files were ignored per glob.  add --verbose
+      lists the specific files.  (Aaron Bentley)
+
+  BUG FIXES:
+
+    * SFTP can walk up to the root path without index errors. (Robert Collins)
+
+    * Fix bugs in running bzr with 'python -O'.  (Martin Pool)
+
+    * Error when run with -OO
+
+    * Fix bug in reporting http errors that don't have an http error code.
+      (Martin Pool)
+
+    * Handle more cases of pipe errors in display commands
+
+    * Change status to 3 for all errors
+
+    * Files that are added and unlinked before committing are completely
+      ignored by diff and status
+
+    * Stores with some compressed texts and some uncompressed texts are now
+      able to be used. (John A Meinel)
+
+    * Fix for bzr pull failing sometimes under windows
+
+    * Fix for sftp transport under windows when using interactive auth
+
+    * Show files which are both renamed and modified as such in 'bzr 
+      status' output.  (#4503, Daniel Silverstone)
+
+    * Make annotate cope better with revisions committed without a valid 
+      email address.  (Marien Zwart)
+
+    * Fix representation of tab characters in commit messages.  (Harald 
+      Meland)
+
+    * List of plugin directories in BZR_PLUGIN_PATH environment variable is
+      now parsed properly under Windows. (Alexander Belchenko)
+
+    * Show number of revisions pushed/pulled/merged. (Robey Pointer)
+
+    * Keep a cached copy of the basis inventory to speed up operations 
+      that need to refer to it.  (Johan Rydberg, Martin Pool)
+
+    * Fix bugs in bzr status display of non-ascii characters.  (Martin 
+      Pool)
+
+    * Remove Makefile.in from default ignore list.  (#6413, Tollef Fog 
+      Heen, Martin Pool)
+
+  TESTING:
+
+    * Fix selftest asking for passwords when there are no SFTP keys.  
+      (Robey Pointer, Jelmer Vernooij) 
+
+    * Fix selftest run with 'python -O'.  (Martin Pool)
+
+    * Fix HTTP tests under Windows. (John Arbash Meinel)
+
+    * Make tests work even if HOME is not set (Aaron Bentley)
+
+    * Updated build_tree to use fixed line-endings for tests which read 
+      the file cotents and compare. Make some tests use this to pass under
+      Windows. (John Arbash Meinel)
+
+    * Skip stat and symlink tests under Windows. (Alexander Belchenko)
+
+    * Delay in selftest/testhashcash is now issued under win32 and Cygwin.
+      (John Arbash Meinel)
+
+    * Use terminal width to align verbose test output.  (Martin Pool)
+
+    * Blackbox tests are maintained within the bzrlib.tests.blackbox directory.
+      If adding a new test script please add that to
+      bzrlib.tests.blackbox.__init__. (Robert Collins)
+
+    * Much better error message if one of the test suites can't be 
+      imported.  (Martin Pool)
+
+    * Make check now runs the test suite twice - once with the default locale,
+      and once with all locales forced to C, to expose bugs. This is not 
+      trivially done within python, so for now its only triggered by running
+      Make check. Integrators and packagers who wish to check for full 
+      platform support should run 'make check' to test the source.
+      (Robert Collins)
+
+    * Tests can now run TestSkipped if they can't execute for any reason.
+      (Martin Pool) (NB: TestSkipped should only be raised for correctable
+      reasons - see the wiki spec ImprovingBzrTestSuite).
+
+    * Test sftp with relative, absolute-in-homedir and absolute-not-in-homedir
+      paths for the transport tests. Introduce blackbox remote sftp tests that
+      test the same permutations. (Robert Collins, Robey Pointer)
+
+    * Transport implementation tests are now independent of the local file
+      system, which allows tests for esoteric transports, and for features
+      not available in the local file system. They also repeat for variations
+      on the URL scheme that can introduce issues in the transport code,
+      see bzrlib.transport.TransportTestProviderAdapter() for this.
+      (Robert Collins).
+
+    * TestCase.build_tree uses the transport interface to build trees, pass
+      in a transport parameter to give it an existing connection.
+      (Robert Collins).
+
+  INTERNALS:
+
+    * WorkingTree.pull has been split across Branch and WorkingTree,
+      to allow Branch only pulls. (Robert Collins)
+
+    * commands.display_command now returns the result of the decorated 
+      function. (Robert Collins)
+
+    * LocationConfig now has a set_user_option(key, value) call to save
+      a setting in its matching location section (a new one is created
+      if needed). (Robert Collins)
+
+    * Branch has two new methods, get_push_location and set_push_location
+      to respectively, get and set the push location. (Robert Collins)
+
+    * commands.register_command now takes an optional flag to signal that
+      the registrant is planning to decorate an existing command. When 
+      given multiple plugins registering a command is not an error, and
+      the original command class (whether built in or a plugin based one) is
+      returned to the caller. There is a new error 'MustUseDecorated' for
+      signalling when a wrapping command should switch to the original
+      version. (Robert Collins)
+
+    * Some option parsing errors will raise 'BzrOptionError', allowing 
+      granular detection for decorating commands. (Robert Collins).
+
+    * Branch.read_working_inventory has moved to
+      WorkingTree.read_working_inventory. This necessitated changes to
+      Branch.get_root_id, and a move of Branch.set_inventory to WorkingTree
+      as well. To make it clear that a WorkingTree cannot always be obtained
+      Branch.working_tree() will raise 'errors.NoWorkingTree' if one cannot
+      be obtained. (Robert Collins)
+
+    * All pending merges operations from Branch are now on WorkingTree.
+      (Robert Collins)
+
+    * The follow operations from Branch have moved to WorkingTree:
+      add()
+      commit()
+      move()
+      rename_one()
+      unknowns()
+      (Robert Collins)
+
+    * bzrlib.add.smart_add_branch is now smart_add_tree. (Robert Collins)
+
+    * New "rio" serialization format, similar to rfc-822. (Martin Pool)
+
+    * Rename selftests to `bzrlib.tests.test_foo`.  (John A Meinel, Martin 
+      Pool)
+
+    * bzrlib.plugin.all_plugins has been changed from an attribute to a 
+      query method. (Robert Collins)
+ 
+    * New options to read only the table-of-contents of a weave.  
+      (Denys Duchier)
+
+    * Raise NoSuchFile when someone tries to add a non-existant file.
+      (Michael Ellerman)
+
+    * Simplify handling of DivergedBranches in cmd_pull().
+      (Michael Ellerman)
+		   
+    * Deprecated functions and methods can now be marked as such using the 
+      bzrlib.symbol_versioning module. Marked method have their docstring
+      updated and will issue a DeprecationWarning using the warnings module
+      when they are used. (Robert Collins)
+
+    * bzrlib.osutils.safe_unicode now exists to provide parameter coercion
+      for functions that need unicode strings. (Robert Collins)
+
+bzr 0.6 2005-10-28
+
+  IMPROVEMENTS:
+  
+    * pull now takes --verbose to show you what revisions are added or removed
+      (John A Meinel)
+
+    * merge now takes a --show-base option to include the base text in
+      conflicts.
+      (Aaron Bentley)
+
+    * The config files are now read using ConfigObj, so '=' should be used as
+      a separator, not ':'.
+      (Aaron Bentley)
+
+    * New 'bzr commit --strict' option refuses to commit if there are 
+      any unknown files in the tree.  To commit, make sure all files are 
+      either ignored, added, or deleted.  (Michael Ellerman)
+
+    * The config directory is now ~/.bazaar, and there is a single file 
+      ~/.bazaar/bazaar.conf storing email, editor and other preferences.
+      (Robert Collins)
+
+    * 'bzr add' no longer takes a --verbose option, and a --quiet option
+      has been added that suppresses all output.
+
+    * Improved zsh completion support in contrib/zsh, from Clint
+      Adams.
+
+    * Builtin 'bzr annotate' command, by Martin Pool with improvements from 
+      Goffredo Baroncelli.
+    
+    * 'bzr check' now accepts -v for verbose reporting, and checks for
+      ghosts in the branch. (Robert Collins)
+
+    * New command 're-sign' which will regenerate the gpg signature for 
+      a revision. (Robert Collins)
+
+    * If you set check_signatures=require for a path in 
+      ~/.bazaar/branches.conf then bzr will invoke your
+      gpg_signing_command (defaults to gpg) and record a digital signature
+      of your commit. (Robert Collins)
+
+    * New sftp transport, based on Paramiko.  (Robey Pointer)
+
+    * 'bzr pull' now accepts '--clobber' which will discard local changes
+      and make this branch identical to the source branch. (Robert Collins)
+
+    * Just give a quieter warning if a plugin can't be loaded, and 
+      put the details in .bzr.log.  (Martin Pool)
+
+    * 'bzr branch' will now set the branch-name to the last component of the
+      output directory, if one was supplied.
+
+    * If the option 'post_commit' is set to one (or more) python function
+      names (must be in the bzrlib namespace), then they will be invoked
+      after the commit has completed, with the branch and revision_id as
+      parameters. (Robert Collins)
+
+    * Merge now has a retcode of 1 when conflicts occur. (Robert Collins)
+
+    * --merge-type weave is now supported for file contents.  Tree-shape
+      changes are still three-way based.  (Martin Pool, Aaron Bentley)
+
+    * 'bzr check' allows the first revision on revision-history to have
+      parents - something that is expected for cheap checkouts, and occurs
+      when conversions from baz do not have all history.  (Robert Collins).
+
+   * 'bzr merge' can now graft unrelated trees together, if your specify
+     0 as a base. (Aaron Bentley)
+
+   * 'bzr commit branch' and 'bzr commit branch/file1 branch/file2' now work
+     (Aaron Bentley)
+
+    * Add '.sconsign*' to default ignore list.  (Alexander Belchenko)
+
+   * 'bzr merge --reprocess' minimizes conflicts
+
+  TESTING:
+
+    * The 'bzr selftest --pattern' option for has been removed, now 
+      test specifiers on the command line can be simple strings, or 
+      regexps, or both. (Robert Collins)
+
+    * Passing -v to selftest will now show the time each test took to 
+      complete, which will aid in analysing performance regressions and
+      related questions. (Robert Collins)
+
+    * 'bzr selftest' runs all tests, even if one fails, unless '--one'
+      is given. (Martin Pool)
+
+    * There is a new method for TestCaseInTempDir, assertFileEqual, which
+      will check that a given content is equal to the content of the named
+      file. (Robert Collins)
+
+    * Fix test suite's habit of leaving many temporary log files in $TMPDIR.
+      (Martin Pool)
+
+  INTERNALS:
+
+    * New 'testament' command and concept for making gpg-signatures 
+      of revisions that are not tied to a particular internal
+      representation.  (Martin Pool).
+
+    * Per-revision properties ('revprops') as key-value associated 
+      strings on each revision created when the revision is committed.
+      Intended mainly for the use of external tools.  (Martin Pool).
+
+    * Config options have moved from bzrlib.osutils to bzrlib.config.
+      (Robert Collins)
+
+    * Improved command line option definitions allowing explanations
+      for individual options, among other things.  Contributed by 
+      Magnus Therning.
+
+    * Config options have moved from bzrlib.osutils to bzrlib.config.
+      Configuration is now done via the config.Config interface:
+      Depending on whether you have a Branch, a Location or no information
+      available, construct a *Config, and use its signature_checking,
+      username and user_email methods. (Robert Collins)
+
+    * Plugins are now loaded under bzrlib.plugins, not bzrlib.plugin, and
+      they are made available for other plugins to use. You should not 
+      import other plugins during the __init__ of your plugin though, as 
+      no ordering is guaranteed, and the plugins directory is not on the
+      python path. (Robert Collins)
+
+    * Branch.relpath has been moved to WorkingTree.relpath. WorkingTree no
+      no longer takes an inventory, rather it takes an option branch
+      parameter, and if None is given will open the branch at basedir 
+      implicitly. (Robert Collins)
+
+    * Cleaner exception structure and error reporting.  Suggested by 
+      Scott James Remnant.  (Martin Pool)
+
+    * Branch.remove has been moved to WorkingTree, which has also gained
+      lock_read, lock_write and unlock methods for convenience. (Robert
+      Collins)
+
+    * Two decorators, needs_read_lock and needs_write_lock have been added
+      to the branch module. Use these to cause a function to run in a
+      read or write lock respectively. (Robert Collins)
+
+    * Branch.open_containing now returns a tuple (Branch, relative-path),
+      which allows direct access to the common case of 'get me this file
+      from its branch'. (Robert Collins)
+
+    * Transports can register using register_lazy_transport, and they 
+      will be loaded when first used.  (Martin Pool)
+
+    * 'pull' has been factored out of the command as WorkingTree.pull().
+      A new option to WorkingTree.pull has been added, clobber, which will
+      ignore diverged history and pull anyway.
+      (Robert Collins)
+
+    * config.Config has a 'get_user_option' call that accepts an option name.
+      This will be looked up in branches.conf and bazaar.conf as normal.
+      It is intended that this be used by plugins to support options - 
+      options of built in programs should have specific methods on the config.
+      (Robert Collins)
+
+    * merge.merge_inner now has tempdir as an optional parameter. (Robert
+      Collins)
+
+    * Tree.kind is not recorded at the top level of the hierarchy, as it was
+      missing on EmptyTree, leading to a bug with merge on EmptyTrees.
+      (Robert Collins)
+
+    * WorkingTree.__del__ has been removed, it was non deterministic and not 
+      doing what it was intended to. See WorkingTree.__init__ for a comment
+      about future directions. (Robert Collins/Martin Pool)
+
+    * bzrlib.transport.http has been modified so that only 404 urllib errors
+      are returned as NoSuchFile. Other exceptions will propogate as normal.
+      This allows debuging of actual errors. (Robert Collins)
+
+    * bzrlib.transport.Transport now accepts *ONLY* url escaped relative paths
+      to apis like 'put', 'get' and 'has'. This is to provide consistent
+      behaviour - it operates on url's only. (Robert Collins)
+
+    * Transports can register using register_lazy_transport, and they 
+      will be loaded when first used.  (Martin Pool)
+
+    * 'merge_flex' no longer calls conflict_handler.finalize(), instead that
+      is called by merge_inner. This is so that the conflict count can be 
+      retrieved (and potentially manipulated) before returning to the caller
+      of merge_inner. Likewise 'merge' now returns the conflict count to the
+      caller. (Robert Collins)
+
+    * 'revision.revision_graph can handle having only partial history for
+      a revision - that is no revisions in the graph with no parents.
+      (Robert Collins).
+
+    * New builtins.branch_files uses the standard file_list rules to produce
+      a branch and a list of paths, relative to that branch (Aaron Bentley)
+
+    * New TestCase.addCleanup facility.
+
+    * New bzrlib.version_info tuple (similar to sys.version_info), which can
+      be used by programs importing bzrlib.
+
+  BUG FIXES:
+
+    * Better handling of branches in directories with non-ascii names. 
+      (Joel Rosdahl, Panagiotis Papadakos)
+
+    * Upgrades of trees with no commits will not fail due to accessing
+      [-1] in the revision-history. (Andres Salomon)
+
+
+bzr 0.1.1 2005-10-12
+
+  BUG FIXES:
+
+    * Fix problem in pulling over http from machines that do not 
+      allow directories to be listed.
+
+    * Avoid harmless warning about invalid hash cache after 
+      upgrading branch format.
+
+  PERFORMANCE: 
+  
+    * Avoid some unnecessary http operations in branch and pull.
+
+
+bzr 0.1 2005-10-11
+
+  NOTES:
+
+    * 'bzr branch' over http initially gives a very high estimate
+      of completion time but it should fall as the first few 
+      revisions are pulled in.  branch is still slow on 
+      high-latency connections.
 
   BUG FIXES:
   
@@ -9,6 +524,14 @@
     * Locking is now done with fcntl.lockf which works with NFS
       file systems. Contributed by Harald Meland.
 
+    * When a merge encounters a file that has been deleted on
+      one side and modified on the other, the old contents are
+      written out to foo.BASE and foo.SIDE, where SIDE is this
+      or OTHER. Contributed by Aaron Bentley.
+
+    * Export was choosing incorrect file paths for the content of
+      the tarball, this has been fixed by Aaron Bentley.
+
     * Commit will no longer commit without a log message, an 
       error is returned instead. Contributed by Jelmer Vernooij.
 
@@ -16,6 +539,20 @@
       parent directories that are added but not listed will be 
       automatically included. Suggested by Michael Ellerman.
 
+    * bzr commit and upgrade did not correctly record new revisions
+      for files with only a change to their executable status.
+      bzr will correct this when it encounters it. Fixed by
+      Robert Collins
+
+    * HTTP tests now force off the use of http_proxy for the duration.
+      Contributed by Gustavo Niemeyer.
+
+    * Fix problems in merging weave-based branches that have 
+      different partial views of history.
+
+    * Symlink support: working with symlinks when not in the root of a 
+      bzr tree was broken, patch from Scott James Remnant.
+
 
   IMPROVEMENTS:
 
@@ -41,6 +578,19 @@
       merge conflicts respectively. A conflicted tree cannot be committed
       in. Contributed by Aaron Bentley.
 
+    * 'rm' is now an alias for 'remove'.
+
+    * Stores now split out their content in a single byte prefixed hash,
+      dropping the density of files per directory by 256. Contributed by
+      Gustavo Niemeyer.
+
+    * 'bzr diff -r branch:URL' will now perform a diff between two branches.
+      Contributed by Robert Collins.
+
+    * 'bzr log' with the default formatter will show merged revisions,
+      indented to the right. Initial implementation contributed by Gustavo
+      Niemeyer, made incremental by Robert Collins.
+
 
   INTERNALS:
 
@@ -50,6 +600,10 @@
     * InventoryEntry is now an abstract base class, use one of the
       concrete InventoryDirectory etc classes instead.
 
+    * Branch raises an UnsupportedFormatError when it detects a 
+      bzr branch it cannot understand. This allows for precise
+      handling of such circumstances.
+
 
   TESTING:
 

=== modified file 'NEWS.developers'
--- NEWS.developers	2005-10-06 06:31:16 +0000
+++ NEWS.developers	2005-10-13 07:30:57 +0000
@@ -1,4 +1,9 @@
-changes in weave branch:
+DEVELOPMENT HEAD
+
+ * Properties can be set on Revisions by passing 'revprops=' to
+   Branch.commit(), and retrieved by looking at Revision.properties.
+
+bzr 0.1
 
  * Remove RevisionReference class; Revision.parent_ids is now simply a
    list of their ids and parent_sha1s is a list of their corresponding

=== modified file 'README'
--- README	2005-07-22 23:32:00 +0000
+++ README	2005-11-18 08:10:07 +0000
@@ -1,57 +1,32 @@
-*********************************
-Release notes for Bazaar-NG 0.0.5
-*********************************
-
-mbp@sourcefrog.net, May 2005, Canberra
-
-
-
-Caveats
--------
-
-
-
-* Don't use this for critical data; at the very least keep separate
-  regular snapshots of your tree.
-
-
-Dependencies
-------------
-
-This is mostly developed on Linux (Ubuntu); it should work on Unix,
-Windows, or OS X with relatively little trouble.
-
-The only dependency is Python 2.4.
-
-You may optionally install cElementTree to speed up some operations.
-This is strongly recommended if you are dealing with large trees.
-
-This package includes copies of two Python libraries, elementtree and
-urlgrabber.
-
-If you are building .deb or .rpm packages for bzr, please don't
-include these libraries but instead package them separately and depend
-on that.
-
-
-Installation
-------------
-
-The best way to install bzr is to symlink the ``bzr`` command onto a
-directory on your path.  For example::
-
-  ln -s ~/work/bzr/bzr ~/bin/bzr
-
-If you use a symlink for this, Python will be able to automatically
-find the bzr libraries.  Otherwise you must ensure they are listed on
-your $PYTHONPATH.
-
-After installing, please run the test suite to identify any problems
-on your platform::
-
-  ./bzr selftest
-
-If you use the setup.py script then bzr will be installed into the
-specified path.  In this case you must install ElementTree and
-urlgrabber separately.
-
+*******************************
+Release notes for Bazaar-NG 0.6
+*******************************
+
+bazaar-ng is a distributed/decentralized version control system.
+
+Features include:
+
+ * simple to use
+
+ * runs on Unix and Windows
+
+ * branches can be hosted on an http server with no special software
+
+ * full directory versioning including tracking subdirectories, 
+   file renames, symlinks and execute-bits
+   
+ * friendly interface that should be familiar to users of cvs, 
+   svn or other systems
+
+ * Python interface for extending and embedding, including a 
+   plugin interface
+
+ * strong integrity protection, including gpg-signing
+
+ * vibrant and active community
+
+For usage instructions see 
+http://bazaar.canonical.com/IntroductionToBzr
+or run "bzr help"
+
+For installation instructions see ./INSTALL

=== modified file 'TODO'
--- TODO	2005-09-19 02:52:24 +0000
+++ TODO	2005-11-23 01:14:09 +0000
@@ -17,13 +17,6 @@
 Small things
 ------------
 
-* ``bzr status`` should show some description of patches that are merged but 
-  not yet committed.  Preferably in a very compact format so that they
-  just fit on a single line; this could also be used for say ``log
-  --tiny``::
-
-  2005-11-05      mbp  Flowers for stephane
-
 * Fix tests so that import errors caused by modules don't produce false reports
   that the tests themselves don't exist.
 
@@ -59,8 +52,6 @@
 
 * -r option should take a revision-id as well as a revno.
 
-* allow ``bzr st -r 300`` to show a summary of changes since then.
-
 * ``bzr info`` should count only people with distinct email addresses as
   different committers.  (Or perhaps only distinct userids?)
 
@@ -91,15 +82,6 @@
 * ``bzr inventory -r REV`` and perhaps unify this with ``bzr ls``,
   giving options to display ids, types, etc.
 
-* Split BzrError into various more specific subclasses for different
-  errors people might want to catch.
-
-* If the export destination ends in '.tar', '.tar.gz', etc then create
-  a tarball instead of a directory.  (Need to actually make a
-  temporary directory and then tar that up.)
-
-  http://www.gelato.unsw.edu.au/archives/git/0504/2194.html
-
 * RemoteBranch could maintain a cache either in memory or on disk.  We
   know more than an external cache might about which files are
   immutable and which can vary.  On the other hand, it's much simpler
@@ -145,10 +127,6 @@
   methods return object, but what we really want is the raw XML, which
   can be popped into our own store.  That needs to be refactored.
 
-* ``bzr status FOO`` where foo is ignored should say so.
-
-* ``bzr mkdir A...`` should just create and add A.
-
 * Guard against repeatedly merging any particular patch.
 
 * More options for diff:
@@ -169,8 +147,6 @@
 
   - and, of course, tests for all this
 
-* ``bzr add`` with no arguments should probably be the same as ``bzr add .``
-
 * Reproducible performance benchmark to measure whether performance is 
   getting better or worse.
 
@@ -208,14 +184,9 @@
 
 * BZRDIR should be in branch.py not __init__.py.
 
-* ``status `` should accept a -r option to show changes relative to a revision,
-  or between revisions
-
 Medium things
 -------------
 
-* ~/.bzr.log is not written anymore for some reason.
-
 * merge should add all revision and inventory XML to the local store.
 
 * check should give a warning for revisions that are named in the
@@ -228,9 +199,6 @@
   just get the id for the selected files, look up their location and
   diff just those files.  No need to traverse the entire inventories.
 
-* ``bzr status DIR`` or ``bzr diff DIR`` should report on all changes
-  under that directory.
-
 * Fix up Inventory objects to represent root object as an entry.
 
 * Don't convert entire entry from ElementTree to an object when it is
@@ -313,13 +281,6 @@
   - Class that describes the state of a working tree so we can just
     assert it's equal.
 
-* There are too many methods on Branch() that really manipulate the
-  WorkingTree.  They should be moved across.
-
-  Also there are some methods which are duplicated on Tree and
-  Inventory objects, and it should be made more clear which ones are
-  proxies and which ones behave differently, and how.
-
 * Try using XSLT to add some formatting to REST-generated HTML.  Or
   maybe write a small Python program that specifies a header and foot
   for the pages and calls into the docutils libraries.
@@ -341,7 +302,7 @@
 
   - Hold the ElementTree in memory in the Inventory object and work
     directly on that, rather than converting into Python objects every
-    time it is read in.  Probably still exposoe it through some kind of
+    time it is read in.  Probably still expose it through some kind of
     object interface though, but perhaps that should just be a proxy
     for the elements.
 
@@ -353,7 +314,7 @@
 
 * stat cache should perhaps only stat files as necessary, rather than
   doing them all up-front.  On the other hand, that disallows the
-  opimization of stating them in inode order.
+  optimization of stating them in inode order.
 
 * It'd be nice to pipeline multiple HTTP requests.  Often we can
   predict what will be wanted in future: all revisions, or all texts
@@ -364,8 +325,6 @@
 
 * Paranoid mode where we never trust SHA-1 matches.
 
-* Don't commit if there are no changes unless forced.
-
 * --dry-run mode for commit?  (Or maybe just run with
   check-command=false?)
 
@@ -377,14 +336,12 @@
 * XML attributes might have trouble with filenames containing \n and
   \r.  Do we really want to support this?  I think perhaps not.
 
-* Remember execute bits, so that exports will work OK.
-
 * Unify smart_add and plain Branch.add(); perhaps smart_add should
   just build a list of files to add and pass that to the regular add
   function.
 
 * Function to list a directory, saying in which revision each file was
-  last modified.  Useful for web and gui interfaces, and slow to
+  last modified.  Useful for web and GUI interfaces, and slow to
   compute one file at a time.
   
   This will be done when we track file texts by referring to the
@@ -395,21 +352,11 @@
 * Perhaps attempts to get locks should timeout after some period of
   time, or at least display a progress message.
 
-* Split out upgrade functionality from check command into a separate
-  ``bzr upgrade``.
-
 * Don't pass around command classes but rather pass objects.  This'd
   make it cleaner to construct objects wrapping external commands.
 
 * Track all merged-in revisions in a versioned add-only metafile.
 
-* ``pull --clobber`` should discard any local changes not present
-  remotely.  Not generally what you want, but possibly useful when
-  you're just mirroring another branch and want to keep tracking it
-  even when they e.g. uncommit or make similar non-forward movements.
-  Also for push I suppose.  Clobber may not be the best name, maybe
-  ``--destroy``?
-
 * ``uncommit`` command that removes a revision from the end of the
   revision-history; just doing this is enough to remove the commit,
   and a new commit will automatically be made against the

=== modified file 'build-api'
--- build-api	2005-05-16 02:19:13 +0000
+++ build-api	2005-11-18 08:10:07 +0000
@@ -1,2 +1,3 @@
+#!/bin/bash
 mkdir -p api
 PYTHONPATH=$PWD epydoc -o api/html --docformat restructuredtext bzr bzrlib

=== modified file 'bzr' (properties changed: -x to +x)
--- bzr	2005-09-29 05:06:14 +0000
+++ bzr	2006-01-13 06:31:42 +0000
@@ -15,16 +15,17 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-
+"""Bazaar-NG -- a free distributed version-control tool"""
 import os, sys
-
+if __doc__ is None:
+    print "bzr does not support python -OO."
+    sys.exit(2)
 try:
     version_info = sys.version_info
 except AttributeError:
     version_info = 1, 5 # 1.5 or older
 
-
-REINVOKE = "__BZR_REINVOKE"    
+REINVOKE = "__BZR_REINVOKE"
 NEED_VERS = (2, 4)
 KNOWN_PYTHONS = ('python2.4',)
 
@@ -54,6 +55,12 @@
                      "\n")
     raise
 
+if bzrlib.version_info[:3] != (0, 8, 0):
+    sys.stderr.write("bzr: WARNING: bzrlib version doesn't match the bzr program.\n"
+            "This may indicate an installation problem.\n"
+            "bzrlib from %s is version %r\n"
+            % (bzrlib.__path__, bzrlib.version_info))
+
 if __name__ == '__main__':
     bzrlib.trace.enable_default_logging()
     sys.exit(bzrlib.commands.main(sys.argv))

=== renamed file 'bzr-man.py' => 'bzr_man.py'
--- bzr-man.py	2005-09-28 04:58:18 +0000
+++ bzr_man.py	2005-12-08 12:01:09 +0000
@@ -21,30 +21,6 @@
 
 import os, sys
 
-try:
-    version_info = sys.version_info
-except AttributeError:
-    version_info = 1, 5 # 1.5 or older
-
-
-REINVOKE = "__BZR_REINVOKE"    
-NEED_VERS = (2, 3)
-
-if version_info < NEED_VERS:
-    if not os.environ.has_key(REINVOKE):
-        # mutating os.environ doesn't work in old Pythons
-        os.putenv(REINVOKE, "1")
-        for python in 'python2.4', 'python2.3':
-            try:
-                os.execvp(python, [python] + sys.argv)
-            except OSError:
-                pass
-    print >>sys.stderr, "bzr-man.py: error: cannot find a suitable python interpreter"
-    print >>sys.stderr, "  (need %d.%d or later)" % NEED_VERS
-    sys.exit(1)
-if hasattr(os, "unsetenv"):
-    os.unsetenv(REINVOKE)
-
 import bzrlib, bzrlib.help
 
 #>>> code taken from bzr (C) Canonical
@@ -126,7 +102,7 @@
         bzrcmd = self.params["bzrcmd"]
         outfile.write('.SH "COMMAND OVERVIEW"\n')
         for (command,usage,descr) in self.command_usage:
-            outfile.write('.TP\n.B "%s %s"\n%s\n\n' % (bzrcmd, usage, descr))
+            outfile.write('.TP\n.B "%s %s"\n%s\n' % (bzrcmd, usage, descr))
 
 
 class HelpReader:
@@ -196,7 +172,6 @@
 Path where
 .B "%(bzrcmd)s"
 is to look for external command.
-
 .TP
 .I "BZREMAIL"
 E-Mail address of the user. Overrides
@@ -204,14 +179,12 @@
 .IR "EMAIL" .
 Example content:
 .I "John Doe <john@example.com>"
-
 .TP
 .I "EMAIL"
 E-Mail address of the user. Overridden by the content of the file
 .I "~/.bzr.conf/email"
 and of the environment variable
 .IR "BZREMAIL" .
-
 .SH "FILES"
 .TP
 .I "~/.bzr.conf/"
@@ -222,7 +195,6 @@
 .I "EMAIL"
 environment variable. Example content:
 .I "John Doe <john@example.com>"
-
 .SH "SEE ALSO"
 .UR http://www.bazaar-ng.org/
 .BR http://www.bazaar-ng.org/,
@@ -230,7 +202,11 @@
 .BR http://www.bazaar-ng.org/doc/
 """
 
-def main():
+def main(args=[]):
+    """ main function
+    :param  args:   command-line arguments (sys.argv[1:])
+    :type   args:   list
+    """
     t = time.time()
     tt = time.gmtime(t)
     params = \
@@ -245,8 +221,8 @@
     clp.end_parse()
 
     filename = "bzr.1"
-    if len(sys.argv) == 2:
-        filename = sys.argv[1]
+    if len(args) == 1:
+        filename = args[0]
     if filename == "-":
         outfile = sys.stdout
     else:
@@ -266,7 +242,7 @@
 
 
 if __name__ == '__main__':
-    main()
+    main(sys.argv[1:])
 
 
 #>>> code by HUN

=== modified file 'bzrlib/__init__.py'
--- bzrlib/__init__.py	2005-10-06 12:34:22 +0000
+++ bzrlib/__init__.py	2006-01-13 06:31:42 +0000
@@ -18,32 +18,87 @@
 
 BZRDIR = ".bzr"
 
-DEFAULT_IGNORE = ['.bzr.log',
-                  '*~', '#*#', '*$', '.#*',
-                  '.*.sw[nop]', '.*.tmp',
-                  '*.tmp', '*.bak', '*.BAK', '*.orig',
-                  '*.o', '*.obj', '*.a', '*.py[oc]', '*.so', '*.exe', '*.elc', 
-                  '{arch}', 'CVS', 'CVS.adm', '.svn', '_darcs', 'SCCS', 'RCS',
+# please keep these sorted (in C locale order) to aid merging
+DEFAULT_IGNORE = [
+                  '#*#',
+                  '*$',
                   '*,v',
+                  '*.BAK',
+                  '*.a',
+                  '*.bak',
+                  '*.elc',
+                  '*.exe',
+                  '*.la',
+                  '*.lo',
+                  '*.o',
+                  '*.o',
+                  '*.obj',
+                  '*.orig',
+                  '*.py[oc]',
+                  '*.so',
+                  '*.tmp',
+                  '.*.tmp',
+                  '*~',
+                  '.#*',
+                  '.*.sw[nop]',
+                  '.sw[nop]',    # vim editing nameless file
+                  '.DS_Store',
+                  '.arch-ids',
+                  '.arch-inventory',
+                  '.bzr.log',
+                  '.del-*',
+                  '.git',
+                  '.jamdeps'
+                  '.libs',
+                  '.make.state',
+                  '.sconsign*',
+                  '.svn',
+                  '.tmp*',
                   'BitKeeper',
-                  '.git',
-                  'TAGS', '.make.state', '.sconsign', '.tmp*',
-                  '.del-*',
-                  '.DS_Store',]
+                  'CVS',
+                  'CVS.adm',
+                  'RCS',
+                  'SCCS',
+                  'TAGS',
+                  '_darcs',
+                  'aclocal.m4',
+                  'autom4te*',
+                  'config.h',
+                  'config.h.in',
+                  'config.log',
+                  'config.status',
+                  'config.sub',
+                  'stamp-h',
+                  'stamp-h.in',
+                  'stamp-h1',
+                  '{arch}',
+                  ]
 
 IGNORE_FILENAME = ".bzrignore"
 
 import os
-import locale
+import sys
+if sys.platform == 'darwin':
+    # work around egregious python 2.4 bug
+    sys.platform = 'posix'
+    import locale
+    sys.platform = 'darwin'
+else:
+    import locale
 user_encoding = locale.getpreferredencoding() or 'ascii'
 del locale
 
-__copyright__ = "Copyright 2005 Canonical Development Ltd."
-__version__ = '0.1rc1'
-
-
+__copyright__ = "Copyright 2005,06 Canonical Development Ltd."
+__version__ = version_string = '0.8pre'
+# same format as sys.version_info
+version_info = (0, 8, 0, 'pre', 0)
+
+
+from bzrlib.symbol_versioning import deprecated_function, zero_seven
+
+@deprecated_function(zero_seven)
 def get_bzr_revision():
-    """If bzr is run from a branch, return (revno,revid) or None"""
+    """If bzr is run from a branch, return (revno,revid) or None."""
     import bzrlib.errors
     from bzrlib.branch import Branch
     
@@ -58,5 +113,5 @@
         return None
     
 def test_suite():
-    import selftest
-    return selftest.test_suite()
+    import tests
+    return tests.test_suite()

=== modified file 'bzrlib/add.py'
--- bzrlib/add.py	2005-10-03 02:10:02 +0000
+++ bzrlib/add.py	2005-12-18 04:08:46 +0000
@@ -14,14 +14,19 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+from os.path import dirname
+
+import bzrlib.errors as errors
 from bzrlib.inventory import InventoryEntry
 from bzrlib.trace import mutter, note, warning
 from bzrlib.errors import NotBranchError
-from bzrlib.branch import Branch
-from bzrlib.osutils import quotefn
-from os.path import dirname
+from bzrlib.branch import is_control_file
+import bzrlib.osutils
+from bzrlib.workingtree import WorkingTree
 
 def glob_expand_for_win32(file_list):
+    if not file_list:
+        return
     import glob
     expanded_file_list = []
     for possible_glob in file_list:
@@ -36,26 +41,37 @@
     return expanded_file_list
 
 
-def add_reporter_null(path, kind, entry):
-    """Absorb add reports and do nothing."""
-    pass
-
-def add_reporter_print(path, kind, entry):
-    """Print a line to stdout for each file that's added."""
-    print "added", quotefn(path)
-    
 def _prepare_file_list(file_list):
     """Prepare a file list for use by smart_add_*."""
     import sys
     if sys.platform == 'win32':
         file_list = glob_expand_for_win32(file_list)
     if not file_list:
-        file_list = ['.']
+        file_list = [u'.']
     file_list = list(file_list)
     return file_list
 
 
-def smart_add(file_list, recurse=True, reporter=add_reporter_null):
+def add_action_null(inv, path, kind):
+    """Absorb add actions and do nothing."""
+    pass
+
+def add_action_print(inv, path, kind):
+    """Print a line to stdout for each file that would be added."""
+    print "added", bzrlib.osutils.quotefn(path)
+
+def add_action_add(inv, path, kind):
+    """Add each file to the given inventory. Produce no output."""
+    entry = inv.add_path(path, kind=kind)
+    mutter("added %r kind %r file_id={%s}" % (path, kind, entry.file_id))
+
+def add_action_add_and_print(inv, path, kind):
+    """Add each file to the given inventory, and print a line to stdout."""
+    add_action_add(inv, path, kind)
+    add_action_print(inv, path, kind)
+
+
+def smart_add(file_list, recurse=True, action=add_action_add):
     """Add files to version, optionally recursing into directories.
 
     This is designed more towards DWIM for humans than API simplicity.
@@ -64,37 +80,39 @@
     Returns the number of files added.
     """
     file_list = _prepare_file_list(file_list)
-    b = Branch.open_containing(file_list[0])
-    return smart_add_branch(b, file_list, recurse, reporter)
+    tree = WorkingTree.open_containing(file_list[0])[0]
+    return smart_add_tree(tree, file_list, recurse, action)
 
-        
-def smart_add_branch(branch, file_list, recurse=True, reporter=add_reporter_null):
+def smart_add_tree(tree, file_list, recurse=True, action=add_action_add):
     """Add files to version, optionally recursing into directories.
 
     This is designed more towards DWIM for humans than API simplicity.
     For the specific behaviour see the help for cmd_add().
 
-    This yields a sequence of (path, kind, file_id) for added files.
+    This calls reporter with each (path, kind, file_id) of added files.
 
     Returns the number of files added.
     """
-    import os
+    import os, errno
     from bzrlib.errors import BadFileKindError, ForbiddenFileError
-    import bzrlib.branch
-
     assert isinstance(recurse, bool)
 
     file_list = _prepare_file_list(file_list)
     user_list = file_list[:]
-    inv = branch.read_working_inventory()
-    tree = branch.working_tree()
-    count = 0
+    inv = tree.read_working_inventory()
+    added = []
+    ignored = {}
 
     for f in file_list:
-        rf = branch.relpath(f)
-        af = branch.abspath(rf)
+        rf = tree.relpath(f)
+        af = tree.abspath(rf)
 
-        kind = bzrlib.osutils.file_kind(af)
+        try:
+            kind = bzrlib.osutils.file_kind(af)
+        except OSError, e:
+            if hasattr(e, 'errno') and e.errno == errno.ENOENT:
+                raise errors.NoSuchFile(rf)
+            raise
 
         if not InventoryEntry.versionable_kind(kind):
             if f in user_list:
@@ -103,65 +121,69 @@
                 warning("skipping %s (can't add file of kind '%s')", f, kind)
                 continue
 
-        mutter("smart add of %r, abs=%r" % (f, af))
+        mutter("smart add of %r, abs=%r", f, af)
         
-        if bzrlib.branch.is_control_file(af):
+        if is_control_file(af):
             raise ForbiddenFileError('cannot add control file %s' % f)
             
         versioned = (inv.path2id(rf) != None)
 
         if kind == 'directory':
             try:
-                sub_branch = Branch.open(af)
+                sub_branch = WorkingTree(af)
                 sub_tree = True
             except NotBranchError:
                 sub_tree = False
+            except errors.UnsupportedFormatError:
+                sub_tree = True
         else:
             sub_tree = False
 
         if rf == '':
-            mutter("branch root doesn't need to be added")
+            mutter("tree root doesn't need to be added")
             sub_tree = False
         elif versioned:
-            mutter("%r is already versioned" % f)
+            mutter("%r is already versioned", f)
         elif sub_tree:
-            mutter("%r is a bzr tree" %f)
+            mutter("%r is a bzr tree", f)
         else:
-            count += __add_one(branch, inv, rf, kind, reporter)
+            added.extend(__add_one(tree, inv, rf, kind, action))
 
         if kind == 'directory' and recurse and not sub_tree:
             for subf in os.listdir(af):
-                subp = os.path.join(rf, subf)
+                subp = bzrlib.osutils.pathjoin(rf, subf)
                 if subf == bzrlib.BZRDIR:
-                    mutter("skip control directory %r" % subp)
-                elif tree.is_ignored(subp):
-                    mutter("skip ignored sub-file %r" % subp)
+                    mutter("skip control directory %r", subp)
                 else:
-                    mutter("queue to add sub-file %r" % subp)
-                    file_list.append(branch.abspath(subp))
-
-
-    mutter('added %d entries', count)
+                    ignore_glob = tree.is_ignored(subp)
+                    if ignore_glob is not None:
+                        mutter("skip ignored sub-file %r", subp)
+                        if ignore_glob not in ignored:
+                            ignored[ignore_glob] = []
+                        ignored[ignore_glob].append(subp)
+                    else:
+                        mutter("queue to add sub-file %r", subp)
+                        file_list.append(tree.abspath(subp))
+
+
+    mutter('added %d entries', len(added))
     
-    if count > 0:
-        branch._write_inventory(inv)
-
-    return count
-
-def __add_one(branch, inv, path, kind, reporter):
+    if len(added) > 0:
+        tree._write_inventory(inv)
+
+    return added, ignored
+
+def __add_one(tree, inv, path, kind, action):
     """Add a file or directory, automatically add unversioned parents."""
 
     # Nothing to do if path is already versioned.
-    # This is safe from infinite recursion because the branch root is
+    # This is safe from infinite recursion because the tree root is
     # always versioned.
     if inv.path2id(path) != None:
-        return 0
+        return []
 
     # add parent
-    count = __add_one(branch, inv, dirname(path), 'directory', reporter)
-
-    entry = inv.add_path(path, kind=kind)
-    mutter("added %r kind %r file_id={%s}" % (path, kind, entry.file_id))
-    reporter(path, kind, entry)
-
-    return count + 1
+    added = __add_one(tree, inv, dirname(path), 'directory', action)
+    action(inv, path, kind)
+
+    return added + [path]

=== modified file 'bzrlib/annotate.py'
--- bzrlib/annotate.py	2005-09-30 14:27:41 +0000
+++ bzrlib/annotate.py	2005-11-24 01:41:11 +0000
@@ -16,40 +16,70 @@
 
 """File annotate based on weave storage"""
 
+# TODO: Choice of more or less verbose formats:
+# 
+# interposed: show more details between blocks of modified lines
+
+# TODO: Show which revision caused a line to merge into the parent
+
+# TODO: perhaps abbreviate timescales depending on how recent they are
+# e.g. "3:12 Tue", "13 Oct", "Oct 2005", etc.  
+
 import sys
 import os
+import time
 
 import bzrlib.weave
-
-def annotate_file(branch, rev_id, file_id, to_file=None):
+from bzrlib.config import extract_email_address
+from bzrlib.errors import BzrError
+
+
+def annotate_file(branch, rev_id, file_id, verbose=False, full=False,
+        to_file=None):
     if to_file is None:
         to_file = sys.stdout
+
+    prevanno=''
+    annotation = list(_annotate_file(branch, rev_id, file_id))
+    max_origin_len = max(len(origin) for origin in set(x[1] for x in annotation))
+    for (revno_str, author, date_str, line_rev_id, text ) in annotation:
+        if verbose:
+            anno = '%5s %-*s %8s ' % (revno_str, max_origin_len, author, date_str)
+        else:
+            anno = "%5s %-7s " % ( revno_str, author[:7] )
+
+        if anno.lstrip() == "" and full: anno = prevanno
+        print >>to_file, '%s| %s' % (anno, text)
+        prevanno=anno
+
+def _annotate_file(branch, rev_id, file_id ):
+
     rh = branch.revision_history()
-    w = branch.weave_store.get_weave(file_id)
+    w = branch.weave_store.get_weave(file_id, branch.get_transaction())
     last_origin = None
     for origin, text in w.annotate_iter(rev_id):
         text = text.rstrip('\r\n')
         if origin == last_origin:
-            print '      | %s' % (text)
+            (revno_str, author, date_str) = ('','','')
         else:
             last_origin = origin
             line_rev_id = w.idx_to_name(origin)
+            if not branch.has_revision(line_rev_id):
+                (revno_str, author, date_str) = ('?','?','?')
+            else:
+                if line_rev_id in rh:
+                    revno_str = str(rh.index(line_rev_id) + 1)
+                else:
+                    revno_str = 'merge'
+            rev = branch.get_revision(line_rev_id)
+            tz = rev.timezone or 0
+            date_str = time.strftime('%Y%m%d', 
+                                     time.gmtime(rev.timestamp + tz))
+            # a lazy way to get something like the email address
+            # TODO: Get real email address
+            author = rev.committer
             try:
-                revno = rh.index(line_rev_id) + 1
-                print '%5d | %s' % (revno, text)
-            except ValueError:
-                print 'merge | %s' % (text)
-
-
-
-if __name__ == '__main__':
-    from bzrlib.branch import find_branch
-    from bzrlib.trace import enable_default_logging
-
-    enable_default_logging()
-    b = find_branch(sys.argv[1])
-    rp = b.relpath(sys.argv[1])
-    tree = b.revision_tree(b.last_revision())
-    file_id = tree.inventory.path2id(rp)
-    file_version = tree.inventory[file_id].revision
-    annotate_file(b, file_version, file_id, sys.stdout)
+                author = extract_email_address(author)
+            except BzrError:
+                pass        # use the whole name
+        yield (revno_str, author, date_str, line_rev_id, text)

=== modified file 'bzrlib/atomicfile.py'
--- bzrlib/atomicfile.py	2005-09-29 20:34:25 +0000
+++ bzrlib/atomicfile.py	2005-12-18 04:08:46 +0000
@@ -17,7 +17,8 @@
 
 
 from warnings import warn
-from osutils import rename
+from bzrlib.osutils import rename
+import errno
 
 class AtomicFile(object):
     """A file that does an atomic-rename to move into place.
@@ -30,7 +31,7 @@
     An encoding can be specified; otherwise the default is ascii.
     """
 
-    def __init__(self, filename, mode='wb', encoding=None):
+    def __init__(self, filename, mode='wb', encoding=None, new_mode=None):
         if mode != 'wb' and mode != 'wt':
             raise ValueError("invalid AtomicFile mode %r" % mode)
 
@@ -47,6 +48,7 @@
 
         self.write = self.f.write
         self.closed = False
+        self._new_mode = new_mode
 
 
     def __repr__(self):
@@ -65,6 +67,15 @@
         self.f.close()
         self.f = None
         
+        try:
+            if self._new_mode is None:
+                self._new_mode = os.lstat(self.realfilename).st_mode
+        except OSError, e:
+            if e.errno != errno.ENOENT:
+                raise
+        else:
+            os.chmod(self.tmpfilename, self._new_mode)
+
         rename(self.tmpfilename, self.realfilename)
 
 

=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2005-10-06 07:31:42 +0000
+++ bzrlib/branch.py	2006-01-03 02:07:06 +0000
@@ -15,6 +15,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 
+import shutil
 import sys
 import os
 import errno
@@ -23,32 +24,38 @@
 
 
 import bzrlib
-from bzrlib.inventory import InventoryEntry
 import bzrlib.inventory as inventory
 from bzrlib.trace import mutter, note
-from bzrlib.osutils import (isdir, quotefn, compact_date, rand_bytes, 
-                            rename, splitpath, sha_file, appendpath, 
-                            file_kind)
+from bzrlib.osutils import (isdir, quotefn,
+                            rename, splitpath, sha_file,
+                            file_kind, abspath, normpath, pathjoin)
+import bzrlib.errors as errors
 from bzrlib.errors import (BzrError, InvalidRevisionNumber, InvalidRevisionId,
                            NoSuchRevision, HistoryMissing, NotBranchError,
                            DivergedBranches, LockError, UnlistableStore,
-                           UnlistableBranch, NoSuchFile)
+                           UnlistableBranch, NoSuchFile, NotVersionedError,
+                           NoWorkingTree)
 from bzrlib.textui import show_status
-from bzrlib.revision import Revision
+from bzrlib.revision import (Revision, is_ancestor, get_intervening_revisions,
+                             NULL_REVISION)
+
 from bzrlib.delta import compare_trees
 from bzrlib.tree import EmptyTree, RevisionTree
 from bzrlib.inventory import Inventory
 from bzrlib.store import copy_all
-from bzrlib.store.compressed_text import CompressedTextStore
 from bzrlib.store.text import TextStore
 from bzrlib.store.weave import WeaveStore
+from bzrlib.testament import Testament
+import bzrlib.transactions as transactions
 from bzrlib.transport import Transport, get_transport
 import bzrlib.xml5
 import bzrlib.ui
+from config import TreeConfig
 
 
 BZR_BRANCH_FORMAT_4 = "Bazaar-NG branch, format 0.0.4\n"
 BZR_BRANCH_FORMAT_5 = "Bazaar-NG branch, format 5\n"
+BZR_BRANCH_FORMAT_6 = "Bazaar-NG branch, format 6\n"
 ## TODO: Maybe include checks for common corruption of newlines, etc?
 
 
@@ -61,53 +68,28 @@
     # XXX: leave this here for about one release, then remove it
     raise NotImplementedError('find_branch() is not supported anymore, '
                               'please use one of the new branch constructors')
-def _relpath(base, path):
-    """Return path relative to base, or raise exception.
-
-    The path may be either an absolute path or a path relative to the
-    current working directory.
-
-    Lifted out of Branch.relpath for ease of testing.
-
-    os.path.commonprefix (python2.4) has a bad bug that it works just
-    on string prefixes, assuming that '/u' is a prefix of '/u2'.  This
-    avoids that problem."""
-    rp = os.path.abspath(path)
-
-    s = []
-    head = rp
-    while len(head) >= len(base):
-        if head == base:
-            break
-        head, tail = os.path.split(head)
-        if tail:
-            s.insert(0, tail)
-    else:
-        raise NotBranchError("path %r is not within branch %r" % (rp, base))
-
-    return os.sep.join(s)
-        
-
-def find_branch_root(t):
-    """Find the branch root enclosing the transport's base.
-
-    t is a Transport object.
-
-    It is not necessary that the base of t exists.
-
-    Basically we keep looking up until we find the control directory or
-    run into the root.  If there isn't one, raises NotBranchError.
-    """
-    orig_base = t.base
-    while True:
-        if t.has(bzrlib.BZRDIR):
-            return t
-        new_t = t.clone('..')
-        if new_t.base == t.base:
-            # reached the root, whatever that may be
-            raise NotBranchError('%s is not in a branch' % orig_base)
-        t = new_t
-
+
+
+def needs_read_lock(unbound):
+    """Decorate unbound to take out and release a read lock."""
+    def decorated(self, *args, **kwargs):
+        self.lock_read()
+        try:
+            return unbound(self, *args, **kwargs)
+        finally:
+            self.unlock()
+    return decorated
+
+
+def needs_write_lock(unbound):
+    """Decorate unbound to take out and release a write lock."""
+    def decorated(self, *args, **kwargs):
+        self.lock_write()
+        try:
+            return unbound(self, *args, **kwargs)
+        finally:
+            self.unlock()
+    return decorated
 
 ######################################################################
 # branch objects
@@ -128,30 +110,42 @@
         """Open a branch which may be of an old format.
         
         Only local branches are supported."""
-        return _Branch(get_transport(base), relax_version_check=True)
+        return BzrBranch(get_transport(base), relax_version_check=True)
         
     @staticmethod
     def open(base):
         """Open an existing branch, rooted at 'base' (url)"""
         t = get_transport(base)
         mutter("trying to open %r with transport %r", base, t)
-        return _Branch(t)
+        return BzrBranch(t)
 
     @staticmethod
     def open_containing(url):
         """Open an existing branch which contains url.
         
         This probes for a branch at url, and searches upwards from there.
+
+        Basically we keep looking up until we find the control directory or
+        run into the root.  If there isn't one, raises NotBranchError.
+        If there is one, it is returned, along with the unused portion of url.
         """
         t = get_transport(url)
-        t = find_branch_root(t)
-        return _Branch(t)
+        while True:
+            try:
+                return BzrBranch(t), t.relpath(url)
+            except NotBranchError, e:
+                mutter('not a branch in: %r %s', t.base, e)
+            new_t = t.clone('..')
+            if new_t.base == t.base:
+                # reached the root, whatever that may be
+                raise NotBranchError(path=url)
+            t = new_t
 
     @staticmethod
     def initialize(base):
         """Create a new branch, rooted at 'base' (url)"""
         t = get_transport(base)
-        return _Branch(t, init=True)
+        return BzrBranch(t, init=True)
 
     def setup_caching(self, cache_root):
         """Subclasses that care about caching should override this, and set
@@ -159,8 +153,341 @@
         """
         self.cache_root = cache_root
 
-
-class _Branch(Branch):
+    def _get_nick(self):
+        cfg = self.tree_config()
+        return cfg.get_option(u"nickname", default=self.base.split('/')[-2])
+
+    def _set_nick(self, nick):
+        cfg = self.tree_config()
+        cfg.set_option(nick, "nickname")
+        assert cfg.get_option("nickname") == nick
+
+    nick = property(_get_nick, _set_nick)
+        
+    def push_stores(self, branch_to):
+        """Copy the content of this branches store to branch_to."""
+        raise NotImplementedError('push_stores is abstract')
+
+    def get_transaction(self):
+        """Return the current active transaction.
+
+        If no transaction is active, this returns a passthrough object
+        for which all data is immediately flushed and no caching happens.
+        """
+        raise NotImplementedError('get_transaction is abstract')
+
+    def lock_write(self):
+        raise NotImplementedError('lock_write is abstract')
+        
+    def lock_read(self):
+        raise NotImplementedError('lock_read is abstract')
+
+    def unlock(self):
+        raise NotImplementedError('unlock is abstract')
+
+    def abspath(self, name):
+        """Return absolute filename for something in the branch
+        
+        XXX: Robert Collins 20051017 what is this used for? why is it a branch
+        method and not a tree method.
+        """
+        raise NotImplementedError('abspath is abstract')
+
+    def controlfilename(self, file_or_path):
+        """Return location relative to branch."""
+        raise NotImplementedError('controlfilename is abstract')
+
+    def controlfile(self, file_or_path, mode='r'):
+        """Open a control file for this branch.
+
+        There are two classes of file in the control directory: text
+        and binary.  binary files are untranslated byte streams.  Text
+        control files are stored with Unix newlines and in UTF-8, even
+        if the platform or locale defaults are different.
+
+        Controlfiles should almost never be opened in write mode but
+        rather should be atomically copied and replaced using atomicfile.
+        """
+        raise NotImplementedError('controlfile is abstract')
+
+    def put_controlfile(self, path, f, encode=True):
+        """Write an entry as a controlfile.
+
+        :param path: The path to put the file, relative to the .bzr control
+                     directory
+        :param f: A file-like or string object whose contents should be copied.
+        :param encode:  If true, encode the contents as utf-8
+        """
+        raise NotImplementedError('put_controlfile is abstract')
+
+    def put_controlfiles(self, files, encode=True):
+        """Write several entries as controlfiles.
+
+        :param files: A list of [(path, file)] pairs, where the path is the directory
+                      underneath the bzr control directory
+        :param encode:  If true, encode the contents as utf-8
+        """
+        raise NotImplementedError('put_controlfiles is abstract')
+
+    def get_root_id(self):
+        """Return the id of this branches root"""
+        raise NotImplementedError('get_root_id is abstract')
+
+    def set_root_id(self, file_id):
+        raise NotImplementedError('set_root_id is abstract')
+
+    def print_file(self, file, revision_id):
+        """Print `file` to stdout."""
+        raise NotImplementedError('print_file is abstract')
+
+    def append_revision(self, *revision_ids):
+        raise NotImplementedError('append_revision is abstract')
+
+    def set_revision_history(self, rev_history):
+        raise NotImplementedError('set_revision_history is abstract')
+
+    def has_revision(self, revision_id):
+        """True if this branch has a copy of the revision.
+
+        This does not necessarily imply the revision is merge
+        or on the mainline."""
+        raise NotImplementedError('has_revision is abstract')
+
+    def get_revision_xml(self, revision_id):
+        raise NotImplementedError('get_revision_xml is abstract')
+
+    def get_revision(self, revision_id):
+        """Return the Revision object for a named revision"""
+        raise NotImplementedError('get_revision is abstract')
+
+    def get_revision_delta(self, revno):
+        """Return the delta for one revision.
+
+        The delta is relative to its mainline predecessor, or the
+        empty tree for revision 1.
+        """
+        assert isinstance(revno, int)
+        rh = self.revision_history()
+        if not (1 <= revno <= len(rh)):
+            raise InvalidRevisionNumber(revno)
+
+        # revno is 1-based; list is 0-based
+
+        new_tree = self.revision_tree(rh[revno-1])
+        if revno == 1:
+            old_tree = EmptyTree()
+        else:
+            old_tree = self.revision_tree(rh[revno-2])
+
+        return compare_trees(old_tree, new_tree)
+
+    def get_revision_sha1(self, revision_id):
+        """Hash the stored value of a revision, and return it."""
+        raise NotImplementedError('get_revision_sha1 is abstract')
+
+    def get_ancestry(self, revision_id):
+        """Return a list of revision-ids integrated by a revision.
+        
+        This currently returns a list, but the ordering is not guaranteed:
+        treat it as a set.
+        """
+        raise NotImplementedError('get_ancestry is abstract')
+
+    def get_inventory(self, revision_id):
+        """Get Inventory object by hash."""
+        raise NotImplementedError('get_inventory is abstract')
+
+    def get_inventory_xml(self, revision_id):
+        """Get inventory XML as a file object."""
+        raise NotImplementedError('get_inventory_xml is abstract')
+
+    def get_inventory_sha1(self, revision_id):
+        """Return the sha1 hash of the inventory entry."""
+        raise NotImplementedError('get_inventory_sha1 is abstract')
+
+    def get_revision_inventory(self, revision_id):
+        """Return inventory of a past revision."""
+        raise NotImplementedError('get_revision_inventory is abstract')
+
+    def revision_history(self):
+        """Return sequence of revision hashes on to this branch."""
+        raise NotImplementedError('revision_history is abstract')
+
+    def revno(self):
+        """Return current revision number for this branch.
+
+        That is equivalent to the number of revisions committed to
+        this branch.
+        """
+        return len(self.revision_history())
+
+    def last_revision(self):
+        """Return last patch hash, or None if no history."""
+        ph = self.revision_history()
+        if ph:
+            return ph[-1]
+        else:
+            return None
+
+    def missing_revisions(self, other, stop_revision=None, diverged_ok=False):
+        """Return a list of new revisions that would perfectly fit.
+        
+        If self and other have not diverged, return a list of the revisions
+        present in other, but missing from self.
+
+        >>> from bzrlib.commit import commit
+        >>> bzrlib.trace.silent = True
+        >>> br1 = ScratchBranch()
+        >>> br2 = ScratchBranch()
+        >>> br1.missing_revisions(br2)
+        []
+        >>> commit(br2, "lala!", rev_id="REVISION-ID-1")
+        >>> br1.missing_revisions(br2)
+        [u'REVISION-ID-1']
+        >>> br2.missing_revisions(br1)
+        []
+        >>> commit(br1, "lala!", rev_id="REVISION-ID-1")
+        >>> br1.missing_revisions(br2)
+        []
+        >>> commit(br2, "lala!", rev_id="REVISION-ID-2A")
+        >>> br1.missing_revisions(br2)
+        [u'REVISION-ID-2A']
+        >>> commit(br1, "lala!", rev_id="REVISION-ID-2B")
+        >>> br1.missing_revisions(br2)
+        Traceback (most recent call last):
+        DivergedBranches: These branches have diverged.  Try merge.
+        """
+        self_history = self.revision_history()
+        self_len = len(self_history)
+        other_history = other.revision_history()
+        other_len = len(other_history)
+        common_index = min(self_len, other_len) -1
+        if common_index >= 0 and \
+            self_history[common_index] != other_history[common_index]:
+            raise DivergedBranches(self, other)
+
+        if stop_revision is None:
+            stop_revision = other_len
+        else:
+            assert isinstance(stop_revision, int)
+            if stop_revision > other_len:
+                raise bzrlib.errors.NoSuchRevision(self, stop_revision)
+        return other_history[self_len:stop_revision]
+    
+    def update_revisions(self, other, stop_revision=None):
+        """Pull in new perfect-fit revisions."""
+        raise NotImplementedError('update_revisions is abstract')
+
+    def pullable_revisions(self, other, stop_revision):
+        raise NotImplementedError('pullable_revisions is abstract')
+        
+    def revision_id_to_revno(self, revision_id):
+        """Given a revision id, return its revno"""
+        if revision_id is None:
+            return 0
+        history = self.revision_history()
+        try:
+            return history.index(revision_id) + 1
+        except ValueError:
+            raise bzrlib.errors.NoSuchRevision(self, revision_id)
+
+    def get_rev_id(self, revno, history=None):
+        """Find the revision id of the specified revno."""
+        if revno == 0:
+            return None
+        if history is None:
+            history = self.revision_history()
+        elif revno <= 0 or revno > len(history):
+            raise bzrlib.errors.NoSuchRevision(self, revno)
+        return history[revno - 1]
+
+    def revision_tree(self, revision_id):
+        """Return Tree for a revision on this branch.
+
+        `revision_id` may be None for the null revision, in which case
+        an `EmptyTree` is returned."""
+        raise NotImplementedError('revision_tree is abstract')
+
+    def working_tree(self):
+        """Return a `Tree` for the working copy if this is a local branch."""
+        raise NotImplementedError('working_tree is abstract')
+
+    def pull(self, source, overwrite=False):
+        raise NotImplementedError('pull is abstract')
+
+    def basis_tree(self):
+        """Return `Tree` object for last revision.
+
+        If there are no revisions yet, return an `EmptyTree`.
+        """
+        return self.revision_tree(self.last_revision())
+
+    def rename_one(self, from_rel, to_rel):
+        """Rename one file.
+
+        This can change the directory or the filename or both.
+        """
+        raise NotImplementedError('rename_one is abstract')
+
+    def move(self, from_paths, to_name):
+        """Rename files.
+
+        to_name must exist as a versioned directory.
+
+        If to_name exists and is a directory, the files are moved into
+        it, keeping their old names.  If it is a directory, 
+
+        Note that to_name is only the last component of the new name;
+        this doesn't change the directory.
+
+        This returns a list of (from_path, to_path) pairs for each
+        entry that is moved.
+        """
+        raise NotImplementedError('move is abstract')
+
+    def get_parent(self):
+        """Return the parent location of the branch.
+
+        This is the default location for push/pull/missing.  The usual
+        pattern is that the user can override it by specifying a
+        location.
+        """
+        raise NotImplementedError('get_parent is abstract')
+
+    def get_push_location(self):
+        """Return the None or the location to push this branch to."""
+        raise NotImplementedError('get_push_location is abstract')
+
+    def set_push_location(self, location):
+        """Set a new push location for this branch."""
+        raise NotImplementedError('set_push_location is abstract')
+
+    def set_parent(self, url):
+        raise NotImplementedError('set_parent is abstract')
+
+    def check_revno(self, revno):
+        """\
+        Check whether a revno corresponds to any revision.
+        Zero (the NULL revision) is considered valid.
+        """
+        if revno != 0:
+            self.check_real_revno(revno)
+            
+    def check_real_revno(self, revno):
+        """\
+        Check whether a revno corresponds to a real revision.
+        Zero (the NULL revision) is considered invalid
+        """
+        if revno < 1 or revno > self.revno():
+            raise InvalidRevisionNumber(revno)
+        
+    def sign_revision(self, revision_id, gpg_strategy):
+        raise NotImplementedError('sign_revision is abstract')
+
+    def store_revision_signature(self, gpg_strategy, plaintext, revision_id):
+        raise NotImplementedError('store_revision_signature is abstract')
+
+class BzrBranch(Branch):
     """A branch stored in the actual filesystem.
 
     Note that it's "local" in the context of the filesystem; it doesn't
@@ -185,6 +512,10 @@
     _lock_count = None
     _lock = None
     _inventory_weave = None
+    # If set to False (by a plugin, etc) BzrBranch will not set the
+    # mode on created files or directories
+    _set_file_mode = True
+    _set_dir_mode = True
     
     # Map some sort of prefix into a namespace
     # stuff like "revno:10", "revid:", etc.
@@ -192,7 +523,7 @@
     REVISION_NAMESPACES = {}
 
     def push_stores(self, branch_to):
-        """Copy the content of this branches store to branch_to."""
+        """See Branch.push_stores."""
         if (self._branch_format != branch_to._branch_format
             or self._branch_format != 4):
             from bzrlib.fetch import greedy_fetch
@@ -216,8 +547,6 @@
         """Create new branch object at a particular location.
 
         transport -- A Transport object, defining how to access files.
-                (If a string, transport.transport() will be used to
-                create a Transport object)
         
         init -- If True, create new control files in a previously
              unversioned directory.  If False, the branch must already
@@ -237,25 +566,23 @@
         if init:
             self._make_control()
         self._check_format(relax_version_check)
+        self._find_modes()
 
-        def get_store(name, compressed=True):
-            # FIXME: This approach of assuming stores are all entirely compressed
-            # or entirely uncompressed is tidy, but breaks upgrade from 
-            # some existing branches where there's a mixture; we probably 
-            # still want the option to look for both.
-            relpath = self._rel_controlfilename(name)
-            if compressed:
-                store = CompressedTextStore(self._transport.clone(relpath))
-            else:
-                store = TextStore(self._transport.clone(relpath))
-            #if self._transport.should_cache():
-            #    cache_path = os.path.join(self.cache_root, name)
-            #    os.mkdir(cache_path)
-            #    store = bzrlib.store.CachedStore(store, cache_path)
+        def get_store(name, compressed=True, prefixed=False):
+            relpath = self._rel_controlfilename(unicode(name))
+            store = TextStore(self._transport.clone(relpath),
+                              dir_mode=self._dir_mode,
+                              file_mode=self._file_mode,
+                              prefixed=prefixed,
+                              compressed=compressed)
             return store
-        def get_weave(name):
-            relpath = self._rel_controlfilename(name)
-            ws = WeaveStore(self._transport.clone(relpath))
+
+        def get_weave(name, prefixed=False):
+            relpath = self._rel_controlfilename(unicode(name))
+            ws = WeaveStore(self._transport.clone(relpath),
+                            prefixed=prefixed,
+                            dir_mode=self._dir_mode,
+                            file_mode=self._file_mode)
             if self._transport.should_cache():
                 ws.enable_cache = True
             return ws
@@ -265,17 +592,22 @@
             self.text_store = get_store('text-store')
             self.revision_store = get_store('revision-store')
         elif self._branch_format == 5:
-            self.control_weaves = get_weave([])
-            self.weave_store = get_weave('weaves')
-            self.revision_store = get_store('revision-store', compressed=False)
+            self.control_weaves = get_weave(u'')
+            self.weave_store = get_weave(u'weaves')
+            self.revision_store = get_store(u'revision-store', compressed=False)
+        elif self._branch_format == 6:
+            self.control_weaves = get_weave(u'')
+            self.weave_store = get_weave(u'weaves', prefixed=True)
+            self.revision_store = get_store(u'revision-store', compressed=False,
+                                            prefixed=True)
+        self.revision_store.register_suffix('sig')
+        self._transaction = None
 
     def __str__(self):
         return '%s(%r)' % (self.__class__.__name__, self._transport.base)
 
-
     __repr__ = __str__
 
-
     def __del__(self):
         if self._lock_mode or self._lock:
             # XXX: This should show something every time, and be suitable for
@@ -291,7 +623,6 @@
         # should never expect their __del__ function to run.
         if hasattr(self, 'cache_root') and self.cache_root is not None:
             try:
-                import shutil
                 shutil.rmtree(self.cache_root)
             except:
                 pass
@@ -302,10 +633,33 @@
             return self._transport.base
         return None
 
-    base = property(_get_base)
-
+    base = property(_get_base, doc="The URL for the root of this branch.")
+
+    def _finish_transaction(self):
+        """Exit the current transaction."""
+        if self._transaction is None:
+            raise errors.LockError('Branch %s is not in a transaction' %
+                                   self)
+        transaction = self._transaction
+        self._transaction = None
+        transaction.finish()
+
+    def get_transaction(self):
+        """See Branch.get_transaction."""
+        if self._transaction is None:
+            return transactions.PassThroughTransaction()
+        else:
+            return self._transaction
+
+    def _set_transaction(self, new_transaction):
+        """Set a new active transaction."""
+        if self._transaction is not None:
+            raise errors.LockError('Branch %s is in a transaction already.' %
+                                   self)
+        self._transaction = new_transaction
 
     def lock_write(self):
+        #mutter("lock write: %s (%s)", self, self._lock_count)
         # TODO: Upgrade locking to support using a Transport,
         # and potentially a remote locking protocol
         if self._lock_mode:
@@ -318,9 +672,10 @@
                     self._rel_controlfilename('branch-lock'))
             self._lock_mode = 'w'
             self._lock_count = 1
-
+            self._set_transaction(transactions.PassThroughTransaction())
 
     def lock_read(self):
+        #mutter("lock read: %s (%s)", self, self._lock_count)
         if self._lock_mode:
             assert self._lock_mode in ('r', 'w'), \
                    "invalid lock mode %r" % self._lock_mode
@@ -330,50 +685,40 @@
                     self._rel_controlfilename('branch-lock'))
             self._lock_mode = 'r'
             self._lock_count = 1
+            self._set_transaction(transactions.ReadOnlyTransaction())
+            # 5K may be excessive, but hey, its a knob.
+            self.get_transaction().set_cache_size(5000)
                         
     def unlock(self):
+        #mutter("unlock: %s (%s)", self, self._lock_count)
         if not self._lock_mode:
             raise LockError('branch %r is not locked' % (self))
 
         if self._lock_count > 1:
             self._lock_count -= 1
         else:
+            self._finish_transaction()
             self._lock.unlock()
             self._lock = None
             self._lock_mode = self._lock_count = None
 
     def abspath(self, name):
-        """Return absolute filename for something in the branch"""
+        """See Branch.abspath."""
         return self._transport.abspath(name)
 
-    def relpath(self, path):
-        """Return path relative to this branch of something inside it.
-
-        Raises an error if path is not in this branch."""
-        return self._transport.relpath(path)
-
-
     def _rel_controlfilename(self, file_or_path):
-        if isinstance(file_or_path, basestring):
-            file_or_path = [file_or_path]
-        return [bzrlib.BZRDIR] + file_or_path
+        if not isinstance(file_or_path, basestring):
+            file_or_path = u'/'.join(file_or_path)
+        if file_or_path == '':
+            return bzrlib.BZRDIR
+        return bzrlib.transport.urlescape(bzrlib.BZRDIR + u'/' + file_or_path)
 
     def controlfilename(self, file_or_path):
-        """Return location relative to branch."""
+        """See Branch.controlfilename."""
         return self._transport.abspath(self._rel_controlfilename(file_or_path))
 
-
     def controlfile(self, file_or_path, mode='r'):
-        """Open a control file for this branch.
-
-        There are two classes of file in the control directory: text
-        and binary.  binary files are untranslated byte streams.  Text
-        control files are stored with Unix newlines and in UTF-8, even
-        if the platform or locale defaults are different.
-
-        Controlfiles should almost never be opened in write mode but
-        rather should be atomically copied and replaced using atomicfile.
-        """
+        """See Branch.controlfile."""
         import codecs
 
         relpath = self._rel_controlfilename(file_or_path)
@@ -384,6 +729,10 @@
         elif mode == 'wb':
             raise BzrError("Branch.controlfile(mode='wb') is not supported, use put_controlfiles")
         elif mode == 'r':
+            # XXX: Do we really want errors='replace'?   Perhaps it should be
+            # an error, or at least reported, if there's incorrectly-encoded
+            # data inside a file.
+            # <https://launchpad.net/products/bzr/+bug/3823>
             return codecs.getreader('utf-8')(self._transport.get(relpath), errors='replace')
         elif mode == 'w':
             raise BzrError("Branch.controlfile(mode='w') is not supported, use put_controlfiles")
@@ -391,22 +740,11 @@
             raise BzrError("invalid controlfile mode %r" % mode)
 
     def put_controlfile(self, path, f, encode=True):
-        """Write an entry as a controlfile.
-
-        :param path: The path to put the file, relative to the .bzr control
-                     directory
-        :param f: A file-like or string object whose contents should be copied.
-        :param encode:  If true, encode the contents as utf-8
-        """
+        """See Branch.put_controlfile."""
         self.put_controlfiles([(path, f)], encode=encode)
 
     def put_controlfiles(self, files, encode=True):
-        """Write several entries as controlfiles.
-
-        :param files: A list of [(path, file)] pairs, where the path is the directory
-                      underneath the bzr control directory
-        :param encode:  If true, encode the contents as utf-8
-        """
+        """See Branch.put_controlfiles."""
         import codecs
         ctrl_files = []
         for path, f in files:
@@ -417,7 +755,25 @@
                     f = codecs.getwriter('utf-8')(f, errors='replace')
             path = self._rel_controlfilename(path)
             ctrl_files.append((path, f))
-        self._transport.put_multi(ctrl_files)
+        self._transport.put_multi(ctrl_files, mode=self._file_mode)
+
+    def _find_modes(self, path=None):
+        """Determine the appropriate modes for files and directories."""
+        try:
+            if path is None:
+                path = self._rel_controlfilename('')
+            st = self._transport.stat(path)
+        except errors.TransportNotPossible:
+            self._dir_mode = 0755
+            self._file_mode = 0644
+        else:
+            self._dir_mode = st.st_mode & 07777
+            # Remove the sticky and execute bits for files
+            self._file_mode = self._dir_mode & ~07111
+        if not self._set_dir_mode:
+            self._dir_mode = None
+        if not self._set_file_mode:
+            self._file_mode = None
 
     def _make_control(self):
         from bzrlib.inventory import Inventory
@@ -435,11 +791,16 @@
         bzrlib.weavefile.write_weave_v5(Weave(), sio)
         empty_weave = sio.getvalue()
 
-        dirs = [[], 'revision-store', 'weaves']
+        cfn = self._rel_controlfilename
+        # Since we don't have a .bzr directory, inherit the
+        # mode from the root directory
+        self._find_modes(u'.')
+
+        dirs = ['', 'revision-store', 'weaves']
         files = [('README', 
             "This is a Bazaar-NG control directory.\n"
             "Do not change any files in this directory.\n"),
-            ('branch-format', BZR_BRANCH_FORMAT_5),
+            ('branch-format', BZR_BRANCH_FORMAT_6),
             ('revision-history', ''),
             ('branch-name', ''),
             ('branch-lock', ''),
@@ -448,8 +809,7 @@
             ('inventory.weave', empty_weave),
             ('ancestry.weave', empty_weave)
         ]
-        cfn = self._rel_controlfilename
-        self._transport.mkdir_multi([cfn(d) for d in dirs])
+        self._transport.mkdir_multi([cfn(d) for d in dirs], mode=self._dir_mode)
         self.put_controlfiles(files)
         mutter('created control directory in ' + self._transport.base)
 
@@ -465,280 +825,89 @@
         try:
             fmt = self.controlfile('branch-format', 'r').read()
         except NoSuchFile:
-            raise NotBranchError(self.base)
+            raise NotBranchError(path=self.base)
         mutter("got branch format %r", fmt)
-        if fmt == BZR_BRANCH_FORMAT_5:
+        if fmt == BZR_BRANCH_FORMAT_6:
+            self._branch_format = 6
+        elif fmt == BZR_BRANCH_FORMAT_5:
             self._branch_format = 5
         elif fmt == BZR_BRANCH_FORMAT_4:
             self._branch_format = 4
 
         if (not relax_version_check
-            and self._branch_format != 5):
-            raise BzrError('sorry, branch format %r not supported' % fmt,
+            and self._branch_format not in (5, 6)):
+            raise errors.UnsupportedFormatError(
+                           'sorry, branch format %r not supported' % fmt,
                            ['use a different bzr version',
                             'or remove the .bzr directory'
                             ' and "bzr init" again'])
 
+    @needs_read_lock
     def get_root_id(self):
-        """Return the id of this branches root"""
-        inv = self.read_working_inventory()
+        """See Branch.get_root_id."""
+        inv = self.get_inventory(self.last_revision())
         return inv.root.file_id
 
-    def set_root_id(self, file_id):
-        inv = self.read_working_inventory()
-        orig_root_id = inv.root.file_id
-        del inv._byid[inv.root.file_id]
-        inv.root.file_id = file_id
-        inv._byid[inv.root.file_id] = inv.root
-        for fid in inv:
-            entry = inv[fid]
-            if entry.parent_id in (None, orig_root_id):
-                entry.parent_id = inv.root.file_id
-        self._write_inventory(inv)
-
-    def read_working_inventory(self):
-        """Read the working inventory."""
-        self.lock_read()
-        try:
-            # ElementTree does its own conversion from UTF-8, so open in
-            # binary.
-            f = self.controlfile('inventory', 'rb')
-            return bzrlib.xml5.serializer_v5.read_inventory(f)
-        finally:
-            self.unlock()
-            
-
-    def _write_inventory(self, inv):
-        """Update the working inventory.
-
-        That is to say, the inventory describing changes underway, that
-        will be committed to the next revision.
-        """
-        from cStringIO import StringIO
-        self.lock_write()
-        try:
-            sio = StringIO()
-            bzrlib.xml5.serializer_v5.write_inventory(inv, sio)
-            sio.seek(0)
-            # Transport handles atomicity
-            self.put_controlfile('inventory', sio)
-        finally:
-            self.unlock()
-        
-        mutter('wrote working inventory')
-            
-    inventory = property(read_working_inventory, _write_inventory, None,
-                         """Inventory for the working copy.""")
-
-    def add(self, files, ids=None):
-        """Make files versioned.
-
-        Note that the command line normally calls smart_add instead,
-        which can automatically recurse.
-
-        This puts the files in the Added state, so that they will be
-        recorded by the next commit.
-
-        files
-            List of paths to add, relative to the base of the tree.
-
-        ids
-            If set, use these instead of automatically generated ids.
-            Must be the same length as the list of files, but may
-            contain None for ids that are to be autogenerated.
-
-        TODO: Perhaps have an option to add the ids even if the files do
-              not (yet) exist.
-
-        TODO: Perhaps yield the ids and paths as they're added.
-        """
-        # TODO: Re-adding a file that is removed in the working copy
-        # should probably put it back with the previous ID.
-        if isinstance(files, basestring):
-            assert(ids is None or isinstance(ids, basestring))
-            files = [files]
-            if ids is not None:
-                ids = [ids]
-
-        if ids is None:
-            ids = [None] * len(files)
-        else:
-            assert(len(ids) == len(files))
-
-        self.lock_write()
-        try:
-            inv = self.read_working_inventory()
-            for f,file_id in zip(files, ids):
-                if is_control_file(f):
-                    raise BzrError("cannot add control file %s" % quotefn(f))
-
-                fp = splitpath(f)
-
-                if len(fp) == 0:
-                    raise BzrError("cannot add top-level %r" % f)
-
-                fullpath = os.path.normpath(self.abspath(f))
-
-                try:
-                    kind = file_kind(fullpath)
-                except OSError:
-                    # maybe something better?
-                    raise BzrError('cannot add: not a regular file, symlink or directory: %s' % quotefn(f))
-
-                if not InventoryEntry.versionable_kind(kind):
-                    raise BzrError('cannot add: not a versionable file ('
-                                   'i.e. regular file, symlink or directory): %s' % quotefn(f))
-
-                if file_id is None:
-                    file_id = gen_file_id(f)
-                inv.add_path(f, kind=kind, file_id=file_id)
-
-                mutter("add file %s file_id:{%s} kind=%r" % (f, file_id, kind))
-
-            self._write_inventory(inv)
-        finally:
-            self.unlock()
-            
-
-    def print_file(self, file, revno):
-        """Print `file` to stdout."""
-        self.lock_read()
-        try:
-            tree = self.revision_tree(self.get_rev_id(revno))
-            # use inventory as it was in that revision
-            file_id = tree.inventory.path2id(file)
-            if not file_id:
-                raise BzrError("%r is not present in revision %s" % (file, revno))
-            tree.print_file(file_id)
-        finally:
-            self.unlock()
-
-
-    def remove(self, files, verbose=False):
-        """Mark nominated files for removal from the inventory.
-
-        This does not remove their text.  This does not run on 
-
-        TODO: Refuse to remove modified files unless --force is given?
-
-        TODO: Do something useful with directories.
-
-        TODO: Should this remove the text or not?  Tough call; not
-        removing may be useful and the user can just use use rm, and
-        is the opposite of add.  Removing it is consistent with most
-        other tools.  Maybe an option.
-        """
-        ## TODO: Normalize names
-        ## TODO: Remove nested loops; better scalability
-        if isinstance(files, basestring):
-            files = [files]
-
-        self.lock_write()
-
-        try:
-            tree = self.working_tree()
-            inv = tree.inventory
-
-            # do this before any modifications
-            for f in files:
-                fid = inv.path2id(f)
-                if not fid:
-                    raise BzrError("cannot remove unversioned file %s" % quotefn(f))
-                mutter("remove inventory entry %s {%s}" % (quotefn(f), fid))
-                if verbose:
-                    # having remove it, it must be either ignored or unknown
-                    if tree.is_ignored(f):
-                        new_status = 'I'
-                    else:
-                        new_status = '?'
-                    show_status(new_status, inv[fid].kind, quotefn(f))
-                del inv[fid]
-
-            self._write_inventory(inv)
-        finally:
-            self.unlock()
-
-    # FIXME: this doesn't need to be a branch method
-    def set_inventory(self, new_inventory_list):
-        from bzrlib.inventory import Inventory, InventoryEntry
-        inv = Inventory(self.get_root_id())
-        for path, file_id, parent, kind in new_inventory_list:
-            name = os.path.basename(path)
-            if name == "":
-                continue
-            # fixme, there should be a factory function inv,add_?? 
-            if kind == 'directory':
-                inv.add(inventory.InventoryDirectory(file_id, name, parent))
-            elif kind == 'file':
-                inv.add(inventory.InventoryFile(file_id, name, parent))
-            elif kind == 'symlink':
-                inv.add(inventory.InventoryLink(file_id, name, parent))
+    @needs_read_lock
+    def print_file(self, file, revision_id):
+        """See Branch.print_file."""
+        tree = self.revision_tree(revision_id)
+        # use inventory as it was in that revision
+        file_id = tree.inventory.path2id(file)
+        if not file_id:
+            try:
+                revno = self.revision_id_to_revno(revision_id)
+            except errors.NoSuchRevision:
+                # TODO: This should not be BzrError,
+                # but NoSuchFile doesn't fit either
+                raise BzrError('%r is not present in revision %s' 
+                                % (file, revision_id))
             else:
-                raise BzrError("unknown kind %r" % kind)
-        self._write_inventory(inv)
-
-    def unknowns(self):
-        """Return all unknown files.
-
-        These are files in the working directory that are not versioned or
-        control files or ignored.
-        
-        >>> b = ScratchBranch(files=['foo', 'foo~'])
-        >>> list(b.unknowns())
-        ['foo']
-        >>> b.add('foo')
-        >>> list(b.unknowns())
-        []
-        >>> b.remove('foo')
-        >>> list(b.unknowns())
-        ['foo']
-        """
-        return self.working_tree().unknowns()
-
-
+                raise BzrError('%r is not present in revision %s'
+                                % (file, revno))
+        tree.print_file(file_id)
+
+    @needs_write_lock
     def append_revision(self, *revision_ids):
+        """See Branch.append_revision."""
         for revision_id in revision_ids:
             mutter("add {%s} to revision-history" % revision_id)
-        self.lock_write()
+        rev_history = self.revision_history()
+        rev_history.extend(revision_ids)
+        self.set_revision_history(rev_history)
+
+    @needs_write_lock
+    def set_revision_history(self, rev_history):
+        """See Branch.set_revision_history."""
+        old_revision = self.last_revision()
+        new_revision = rev_history[-1]
+        self.put_controlfile('revision-history', '\n'.join(rev_history))
         try:
-            rev_history = self.revision_history()
-            rev_history.extend(revision_ids)
-            self.put_controlfile('revision-history', '\n'.join(rev_history))
-        finally:
-            self.unlock()
+            self.working_tree().set_last_revision(new_revision, old_revision)
+        except NoWorkingTree:
+            mutter('Unable to set_last_revision without a working tree.')
 
     def has_revision(self, revision_id):
-        """True if this branch has a copy of the revision.
-
-        This does not necessarily imply the revision is merge
-        or on the mainline."""
+        """See Branch.has_revision."""
         return (revision_id is None
-                or revision_id in self.revision_store)
+                or self.revision_store.has_id(revision_id))
 
-    def get_revision_xml_file(self, revision_id):
-        """Return XML file object for revision object."""
+    @needs_read_lock
+    def _get_revision_xml_file(self, revision_id):
         if not revision_id or not isinstance(revision_id, basestring):
-            raise InvalidRevisionId(revision_id)
-
-        self.lock_read()
+            raise InvalidRevisionId(revision_id=revision_id, branch=self)
         try:
-            try:
-                return self.revision_store[revision_id]
-            except (IndexError, KeyError):
-                raise bzrlib.errors.NoSuchRevision(self, revision_id)
-        finally:
-            self.unlock()
-
-    #deprecated
-    get_revision_xml = get_revision_xml_file
+            return self.revision_store.get(revision_id)
+        except (IndexError, KeyError):
+            raise bzrlib.errors.NoSuchRevision(self, revision_id)
 
     def get_revision_xml(self, revision_id):
-        return self.get_revision_xml_file(revision_id).read()
-
+        """See Branch.get_revision_xml."""
+        return self._get_revision_xml_file(revision_id).read()
 
     def get_revision(self, revision_id):
-        """Return the Revision object for a named revision"""
-        xml_file = self.get_revision_xml_file(revision_id)
+        """See Branch.get_revision."""
+        xml_file = self._get_revision_xml_file(revision_id)
 
         try:
             r = bzrlib.xml5.serializer_v5.read_revision(xml_file)
@@ -750,29 +919,8 @@
         assert r.revision_id == revision_id
         return r
 
-    def get_revision_delta(self, revno):
-        """Return the delta for one revision.
-
-        The delta is relative to its mainline predecessor, or the
-        empty tree for revision 1.
-        """
-        assert isinstance(revno, int)
-        rh = self.revision_history()
-        if not (1 <= revno <= len(rh)):
-            raise InvalidRevisionNumber(revno)
-
-        # revno is 1-based; list is 0-based
-
-        new_tree = self.revision_tree(rh[revno-1])
-        if revno == 1:
-            old_tree = EmptyTree()
-        else:
-            old_tree = self.revision_tree(rh[revno-2])
-
-        return compare_trees(old_tree, new_tree)
-
     def get_revision_sha1(self, revision_id):
-        """Hash the stored value of a revision, and return it."""
+        """See Branch.get_revision_sha1."""
         # In the future, revision entries will be signed. At that
         # point, it is probably best *not* to include the signature
         # in the revision hash. Because that lets you re-sign
@@ -782,521 +930,217 @@
         return bzrlib.osutils.sha_file(self.get_revision_xml_file(revision_id))
 
     def get_ancestry(self, revision_id):
-        """Return a list of revision-ids integrated by a revision.
-        
-        This currently returns a list, but the ordering is not guaranteed:
-        treat it as a set.
-        """
+        """See Branch.get_ancestry."""
         if revision_id is None:
             return [None]
-        w = self.control_weaves.get_weave('inventory')
+        w = self._get_inventory_weave()
         return [None] + map(w.idx_to_name,
                             w.inclusions([w.lookup(revision_id)]))
 
-    def get_inventory_weave(self):
-        return self.control_weaves.get_weave('inventory')
+    def _get_inventory_weave(self):
+        return self.control_weaves.get_weave('inventory',
+                                             self.get_transaction())
 
     def get_inventory(self, revision_id):
-        """Get Inventory object by hash."""
+        """See Branch.get_inventory."""
         xml = self.get_inventory_xml(revision_id)
         return bzrlib.xml5.serializer_v5.read_inventory_from_string(xml)
 
     def get_inventory_xml(self, revision_id):
-        """Get inventory XML as a file object."""
+        """See Branch.get_inventory_xml."""
         try:
             assert isinstance(revision_id, basestring), type(revision_id)
-            iw = self.get_inventory_weave()
+            iw = self._get_inventory_weave()
             return iw.get_text(iw.lookup(revision_id))
         except IndexError:
             raise bzrlib.errors.HistoryMissing(self, 'inventory', revision_id)
 
     def get_inventory_sha1(self, revision_id):
-        """Return the sha1 hash of the inventory entry
-        """
+        """See Branch.get_inventory_sha1."""
         return self.get_revision(revision_id).inventory_sha1
 
     def get_revision_inventory(self, revision_id):
-        """Return inventory of a past revision."""
+        """See Branch.get_revision_inventory."""
         # TODO: Unify this with get_inventory()
         # bzr 0.0.6 and later imposes the constraint that the inventory_id
         # must be the same as its revision, so this is trivial.
         if revision_id == None:
-            return Inventory(self.get_root_id())
+            # This does not make sense: if there is no revision,
+            # then it is the current tree inventory surely ?!
+            # and thus get_root_id() is something that looks at the last
+            # commit on the branch, and the get_root_id is an inventory check.
+            raise NotImplementedError
+            # return Inventory(self.get_root_id())
         else:
             return self.get_inventory(revision_id)
 
+    @needs_read_lock
     def revision_history(self):
-        """Return sequence of revision hashes on to this branch."""
-        self.lock_read()
-        try:
-            return [l.rstrip('\r\n') for l in
-                    self.controlfile('revision-history', 'r').readlines()]
-        finally:
-            self.unlock()
-
-    def common_ancestor(self, other, self_revno=None, other_revno=None):
-        """
-        >>> from bzrlib.commit import commit
-        >>> sb = ScratchBranch(files=['foo', 'foo~'])
-        >>> sb.common_ancestor(sb) == (None, None)
-        True
-        >>> commit(sb, "Committing first revision", verbose=False)
-        >>> sb.common_ancestor(sb)[0]
-        1
-        >>> clone = sb.clone()
-        >>> commit(sb, "Committing second revision", verbose=False)
-        >>> sb.common_ancestor(sb)[0]
-        2
-        >>> sb.common_ancestor(clone)[0]
-        1
-        >>> commit(clone, "Committing divergent second revision", 
-        ...               verbose=False)
-        >>> sb.common_ancestor(clone)[0]
-        1
-        >>> sb.common_ancestor(clone) == clone.common_ancestor(sb)
-        True
-        >>> sb.common_ancestor(sb) != clone.common_ancestor(clone)
-        True
-        >>> clone2 = sb.clone()
-        >>> sb.common_ancestor(clone2)[0]
-        2
-        >>> sb.common_ancestor(clone2, self_revno=1)[0]
-        1
-        >>> sb.common_ancestor(clone2, other_revno=1)[0]
-        1
-        """
-        my_history = self.revision_history()
-        other_history = other.revision_history()
-        if self_revno is None:
-            self_revno = len(my_history)
-        if other_revno is None:
-            other_revno = len(other_history)
-        indices = range(min((self_revno, other_revno)))
-        indices.reverse()
-        for r in indices:
-            if my_history[r] == other_history[r]:
-                return r+1, my_history[r]
-        return None, None
-
-
-    def revno(self):
-        """Return current revision number for this branch.
-
-        That is equivalent to the number of revisions committed to
-        this branch.
-        """
-        return len(self.revision_history())
-
-
-    def last_revision(self):
-        """Return last patch hash, or None if no history.
-        """
-        ph = self.revision_history()
-        if ph:
-            return ph[-1]
-        else:
-            return None
-
-
-    def missing_revisions(self, other, stop_revision=None, diverged_ok=False):
-        """Return a list of new revisions that would perfectly fit.
-        
-        If self and other have not diverged, return a list of the revisions
-        present in other, but missing from self.
-
-        >>> from bzrlib.commit import commit
-        >>> bzrlib.trace.silent = True
-        >>> br1 = ScratchBranch()
-        >>> br2 = ScratchBranch()
-        >>> br1.missing_revisions(br2)
-        []
-        >>> commit(br2, "lala!", rev_id="REVISION-ID-1")
-        >>> br1.missing_revisions(br2)
-        [u'REVISION-ID-1']
-        >>> br2.missing_revisions(br1)
-        []
-        >>> commit(br1, "lala!", rev_id="REVISION-ID-1")
-        >>> br1.missing_revisions(br2)
-        []
-        >>> commit(br2, "lala!", rev_id="REVISION-ID-2A")
-        >>> br1.missing_revisions(br2)
-        [u'REVISION-ID-2A']
-        >>> commit(br1, "lala!", rev_id="REVISION-ID-2B")
-        >>> br1.missing_revisions(br2)
-        Traceback (most recent call last):
-        DivergedBranches: These branches have diverged.
-        """
-        # FIXME: If the branches have diverged, but the latest
-        # revision in this branch is completely merged into the other,
-        # then we should still be able to pull.
-        self_history = self.revision_history()
-        self_len = len(self_history)
-        other_history = other.revision_history()
-        other_len = len(other_history)
-        common_index = min(self_len, other_len) -1
-        if common_index >= 0 and \
-            self_history[common_index] != other_history[common_index]:
-            raise DivergedBranches(self, other)
-
-        if stop_revision is None:
-            stop_revision = other_len
-        else:
-            assert isinstance(stop_revision, int)
-            if stop_revision > other_len:
-                raise bzrlib.errors.NoSuchRevision(self, stop_revision)
-        return other_history[self_len:stop_revision]
+        """See Branch.revision_history."""
+        transaction = self.get_transaction()
+        history = transaction.map.find_revision_history()
+        if history is not None:
+            mutter("cache hit for revision-history in %s", self)
+            return list(history)
+        history = [l.rstrip('\r\n') for l in
+                self.controlfile('revision-history', 'r').readlines()]
+        transaction.map.add_revision_history(history)
+        # this call is disabled because revision_history is 
+        # not really an object yet, and the transaction is for objects.
+        # transaction.register_clean(history, precious=True)
+        return list(history)
 
     def update_revisions(self, other, stop_revision=None):
-        """Pull in new perfect-fit revisions."""
+        """See Branch.update_revisions."""
         from bzrlib.fetch import greedy_fetch
-        from bzrlib.revision import get_intervening_revisions
         if stop_revision is None:
             stop_revision = other.last_revision()
+        ### Should this be checking is_ancestor instead of revision_history?
+        if (stop_revision is not None and 
+            stop_revision in self.revision_history()):
+            return
         greedy_fetch(to_branch=self, from_branch=other,
                      revision=stop_revision)
-        pullable_revs = self.missing_revisions(
-            other, other.revision_id_to_revno(stop_revision))
-        if pullable_revs:
-            greedy_fetch(to_branch=self,
-                         from_branch=other,
-                         revision=pullable_revs[-1])
+        pullable_revs = self.pullable_revisions(other, stop_revision)
+        if len(pullable_revs) > 0:
             self.append_revision(*pullable_revs)
-    
 
-    def commit(self, *args, **kw):
-        from bzrlib.commit import Commit
-        Commit().commit(self, *args, **kw)
-    
-    def revision_id_to_revno(self, revision_id):
-        """Given a revision id, return its revno"""
-        if revision_id is None:
-            return 0
-        history = self.revision_history()
+    def pullable_revisions(self, other, stop_revision):
+        """See Branch.pullable_revisions."""
+        other_revno = other.revision_id_to_revno(stop_revision)
         try:
-            return history.index(revision_id) + 1
-        except ValueError:
-            raise bzrlib.errors.NoSuchRevision(self, revision_id)
-
-    def get_rev_id(self, revno, history=None):
-        """Find the revision id of the specified revno."""
-        if revno == 0:
-            return None
-        if history is None:
-            history = self.revision_history()
-        elif revno <= 0 or revno > len(history):
-            raise bzrlib.errors.NoSuchRevision(self, revno)
-        return history[revno - 1]
-
+            return self.missing_revisions(other, other_revno)
+        except DivergedBranches, e:
+            try:
+                pullable_revs = get_intervening_revisions(self.last_revision(),
+                                                          stop_revision, self)
+                assert self.last_revision() not in pullable_revs
+                return pullable_revs
+            except bzrlib.errors.NotAncestor:
+                if is_ancestor(self.last_revision(), stop_revision, self):
+                    return []
+                else:
+                    raise e
+        
     def revision_tree(self, revision_id):
-        """Return Tree for a revision on this branch.
-
-        `revision_id` may be None for the null revision, in which case
-        an `EmptyTree` is returned."""
+        """See Branch.revision_tree."""
         # TODO: refactor this to use an existing revision object
         # so we don't need to read it in twice.
-        if revision_id == None:
+        if revision_id == None or revision_id == NULL_REVISION:
             return EmptyTree()
         else:
             inv = self.get_revision_inventory(revision_id)
-            return RevisionTree(self.weave_store, inv, revision_id)
+            return RevisionTree(self, inv, revision_id)
 
+    def basis_tree(self):
+        """See Branch.basis_tree."""
+        try:
+            revision_id = self.revision_history()[-1]
+            xml = self.working_tree().read_basis_inventory(revision_id)
+            inv = bzrlib.xml5.serializer_v5.read_inventory_from_string(xml)
+            return RevisionTree(self, inv, revision_id)
+        except (IndexError, NoSuchFile, NoWorkingTree), e:
+            return self.revision_tree(self.last_revision())
 
     def working_tree(self):
-        """Return a `Tree` for the working copy."""
+        """See Branch.working_tree."""
         from bzrlib.workingtree import WorkingTree
-        # TODO: In the future, WorkingTree should utilize Transport
-        # RobertCollins 20051003 - I don't think it should - working trees are
-        # much more complex to keep consistent than our careful .bzr subset.
-        # instead, we should say that working trees are local only, and optimise
-        # for that.
-        return WorkingTree(self._transport.base, self.read_working_inventory())
-
-
-    def basis_tree(self):
-        """Return `Tree` object for last revision.
-
-        If there are no revisions yet, return an `EmptyTree`.
-        """
-        return self.revision_tree(self.last_revision())
-
-
-    def rename_one(self, from_rel, to_rel):
-        """Rename one file.
-
-        This can change the directory or the filename or both.
-        """
-        self.lock_write()
-        try:
-            tree = self.working_tree()
-            inv = tree.inventory
-            if not tree.has_filename(from_rel):
-                raise BzrError("can't rename: old working file %r does not exist" % from_rel)
-            if tree.has_filename(to_rel):
-                raise BzrError("can't rename: new working file %r already exists" % to_rel)
-
-            file_id = inv.path2id(from_rel)
-            if file_id == None:
-                raise BzrError("can't rename: old name %r is not versioned" % from_rel)
-
-            if inv.path2id(to_rel):
-                raise BzrError("can't rename: new name %r is already versioned" % to_rel)
-
-            to_dir, to_tail = os.path.split(to_rel)
-            to_dir_id = inv.path2id(to_dir)
-            if to_dir_id == None and to_dir != '':
-                raise BzrError("can't determine destination directory id for %r" % to_dir)
-
-            mutter("rename_one:")
-            mutter("  file_id    {%s}" % file_id)
-            mutter("  from_rel   %r" % from_rel)
-            mutter("  to_rel     %r" % to_rel)
-            mutter("  to_dir     %r" % to_dir)
-            mutter("  to_dir_id  {%s}" % to_dir_id)
-
-            inv.rename(file_id, to_dir_id, to_tail)
-
-            from_abs = self.abspath(from_rel)
-            to_abs = self.abspath(to_rel)
-            try:
-                rename(from_abs, to_abs)
-            except OSError, e:
-                raise BzrError("failed to rename %r to %r: %s"
-                        % (from_abs, to_abs, e[1]),
-                        ["rename rolled back"])
-
-            self._write_inventory(inv)
-        finally:
-            self.unlock()
-
-
-    def move(self, from_paths, to_name):
-        """Rename files.
-
-        to_name must exist as a versioned directory.
-
-        If to_name exists and is a directory, the files are moved into
-        it, keeping their old names.  If it is a directory, 
-
-        Note that to_name is only the last component of the new name;
-        this doesn't change the directory.
-
-        This returns a list of (from_path, to_path) pairs for each
-        entry that is moved.
-        """
-        result = []
-        self.lock_write()
-        try:
-            ## TODO: Option to move IDs only
-            assert not isinstance(from_paths, basestring)
-            tree = self.working_tree()
-            inv = tree.inventory
-            to_abs = self.abspath(to_name)
-            if not isdir(to_abs):
-                raise BzrError("destination %r is not a directory" % to_abs)
-            if not tree.has_filename(to_name):
-                raise BzrError("destination %r not in working directory" % to_abs)
-            to_dir_id = inv.path2id(to_name)
-            if to_dir_id == None and to_name != '':
-                raise BzrError("destination %r is not a versioned directory" % to_name)
-            to_dir_ie = inv[to_dir_id]
-            if to_dir_ie.kind not in ('directory', 'root_directory'):
-                raise BzrError("destination %r is not a directory" % to_abs)
-
-            to_idpath = inv.get_idpath(to_dir_id)
-
-            for f in from_paths:
-                if not tree.has_filename(f):
-                    raise BzrError("%r does not exist in working tree" % f)
-                f_id = inv.path2id(f)
-                if f_id == None:
-                    raise BzrError("%r is not versioned" % f)
-                name_tail = splitpath(f)[-1]
-                dest_path = appendpath(to_name, name_tail)
-                if tree.has_filename(dest_path):
-                    raise BzrError("destination %r already exists" % dest_path)
-                if f_id in to_idpath:
-                    raise BzrError("can't move %r to a subdirectory of itself" % f)
-
-            # OK, so there's a race here, it's possible that someone will
-            # create a file in this interval and then the rename might be
-            # left half-done.  But we should have caught most problems.
-
-            for f in from_paths:
-                name_tail = splitpath(f)[-1]
-                dest_path = appendpath(to_name, name_tail)
-                result.append((f, dest_path))
-                inv.rename(inv.path2id(f), to_dir_id, name_tail)
-                try:
-                    rename(self.abspath(f), self.abspath(dest_path))
-                except OSError, e:
-                    raise BzrError("failed to rename %r to %r: %s" % (f, dest_path, e[1]),
-                            ["rename rolled back"])
-
-            self._write_inventory(inv)
-        finally:
-            self.unlock()
-
-        return result
-
-
-    def revert(self, filenames, old_tree=None, backups=True):
-        """Restore selected files to the versions from a previous tree.
-
-        backups
-            If true (default) backups are made of files before
-            they're renamed.
-        """
-        from bzrlib.errors import NotVersionedError, BzrError
-        from bzrlib.atomicfile import AtomicFile
-        from bzrlib.osutils import backup_file
-        
-        inv = self.read_working_inventory()
-        if old_tree is None:
-            old_tree = self.basis_tree()
-        old_inv = old_tree.inventory
-
-        nids = []
-        for fn in filenames:
-            file_id = inv.path2id(fn)
-            if not file_id:
-                raise NotVersionedError("not a versioned file", fn)
-            if not old_inv.has_id(file_id):
-                raise BzrError("file not present in old tree", fn, file_id)
-            nids.append((fn, file_id))
-            
-        # TODO: Rename back if it was previously at a different location
-
-        # TODO: If given a directory, restore the entire contents from
-        # the previous version.
-
-        # TODO: Make a backup to a temporary file.
-
-        # TODO: If the file previously didn't exist, delete it?
-        for fn, file_id in nids:
-            backup_file(fn)
-            
-            f = AtomicFile(fn, 'wb')
-            try:
-                f.write(old_tree.get_file(file_id).read())
-                f.commit()
-            finally:
-                f.close()
-
-
-    def pending_merges(self):
-        """Return a list of pending merges.
-
-        These are revisions that have been merged into the working
-        directory but not yet committed.
-        """
-        cfn = self._rel_controlfilename('pending-merges')
-        if not self._transport.has(cfn):
-            return []
-        p = []
-        for l in self.controlfile('pending-merges', 'r').readlines():
-            p.append(l.rstrip('\n'))
-        return p
-
-
-    def add_pending_merge(self, *revision_ids):
-        # TODO: Perhaps should check at this point that the
-        # history of the revision is actually present?
-        p = self.pending_merges()
-        updated = False
-        for rev_id in revision_ids:
-            if rev_id in p:
-                continue
-            p.append(rev_id)
-            updated = True
-        if updated:
-            self.set_pending_merges(p)
-
-    def set_pending_merges(self, rev_list):
-        self.lock_write()
-        try:
-            self.put_controlfile('pending-merges', '\n'.join(rev_list))
-        finally:
-            self.unlock()
-
+        if self._transport.base.find('://') != -1:
+            raise NoWorkingTree(self.base)
+        return WorkingTree(self.base, branch=self)
+
+    @needs_write_lock
+    def pull(self, source, overwrite=False):
+        """See Branch.pull."""
+        source.lock_read()
+        try:
+            old_count = len(self.revision_history())
+            try:
+                self.update_revisions(source)
+            except DivergedBranches:
+                if not overwrite:
+                    raise
+            if overwrite:
+                self.set_revision_history(source.revision_history())
+            new_count = len(self.revision_history())
+            return new_count - old_count
+        finally:
+            source.unlock()
 
     def get_parent(self):
-        """Return the parent location of the branch.
-
-        This is the default location for push/pull/missing.  The usual
-        pattern is that the user can override it by specifying a
-        location.
-        """
+        """See Branch.get_parent."""
         import errno
         _locs = ['parent', 'pull', 'x-pull']
         for l in _locs:
             try:
                 return self.controlfile(l, 'r').read().strip('\n')
-            except IOError, e:
-                if e.errno != errno.ENOENT:
-                    raise
+            except NoSuchFile:
+                pass
         return None
 
-
+    def get_push_location(self):
+        """See Branch.get_push_location."""
+        config = bzrlib.config.BranchConfig(self)
+        push_loc = config.get_user_option('push_location')
+        return push_loc
+
+    def set_push_location(self, location):
+        """See Branch.set_push_location."""
+        config = bzrlib.config.LocationConfig(self.base)
+        config.set_user_option('push_location', location)
+
+    @needs_write_lock
     def set_parent(self, url):
+        """See Branch.set_parent."""
         # TODO: Maybe delete old location files?
         from bzrlib.atomicfile import AtomicFile
-        self.lock_write()
+        f = AtomicFile(self.controlfilename('parent'))
         try:
-            f = AtomicFile(self.controlfilename('parent'))
-            try:
-                f.write(url + '\n')
-                f.commit()
-            finally:
-                f.close()
+            f.write(url + '\n')
+            f.commit()
         finally:
-            self.unlock()
-
-    def check_revno(self, revno):
-        """\
-        Check whether a revno corresponds to any revision.
-        Zero (the NULL revision) is considered valid.
-        """
-        if revno != 0:
-            self.check_real_revno(revno)
-            
-    def check_real_revno(self, revno):
-        """\
-        Check whether a revno corresponds to a real revision.
-        Zero (the NULL revision) is considered invalid
-        """
-        if revno < 1 or revno > self.revno():
-            raise InvalidRevisionNumber(revno)
-        
-        
-        
-
-
-class ScratchBranch(_Branch):
+            f.close()
+
+    def tree_config(self):
+        return TreeConfig(self)
+
+    def sign_revision(self, revision_id, gpg_strategy):
+        """See Branch.sign_revision."""
+        plaintext = Testament.from_revision(self, revision_id).as_short_text()
+        self.store_revision_signature(gpg_strategy, plaintext, revision_id)
+
+    @needs_write_lock
+    def store_revision_signature(self, gpg_strategy, plaintext, revision_id):
+        """See Branch.store_revision_signature."""
+        self.revision_store.add(StringIO(gpg_strategy.sign(plaintext)), 
+                                revision_id, "sig")
+
+
+class ScratchBranch(BzrBranch):
     """Special test class: a branch that cleans up after itself.
 
     >>> b = ScratchBranch()
     >>> isdir(b.base)
     True
     >>> bd = b.base
-    >>> b.destroy()
+    >>> b._transport.__del__()
     >>> isdir(bd)
     False
     """
-    def __init__(self, files=[], dirs=[], base=None):
+
+    def __init__(self, files=[], dirs=[], transport=None):
         """Make a test branch.
 
         This creates a temporary directory and runs init-tree in it.
 
         If any files are listed, they are created in the working copy.
         """
-        from tempfile import mkdtemp
-        init = False
-        if base is None:
-            base = mkdtemp()
-            init = True
-        if isinstance(base, basestring):
-            base = get_transport(base)
-        _Branch.__init__(self, base, init=init)
+        if transport is None:
+            transport = bzrlib.transport.local.ScratchTransport()
+            super(ScratchBranch, self).__init__(transport, init=True)
+        else:
+            super(ScratchBranch, self).__init__(transport)
+
         for d in dirs:
             self._transport.mkdir(d)
             
@@ -1314,36 +1158,16 @@
         ...   orig.base == clone.base
         ...
         False
-        >>> os.path.isfile(os.path.join(clone.base, "file1"))
+        >>> os.path.isfile(pathjoin(clone.base, "file1"))
         True
         """
         from shutil import copytree
-        from tempfile import mkdtemp
+        from bzrlib.osutils import mkdtemp
         base = mkdtemp()
         os.rmdir(base)
         copytree(self.base, base, symlinks=True)
-        return ScratchBranch(base=base)
-
-    def __del__(self):
-        self.destroy()
-
-    def destroy(self):
-        """Destroy the test branch, removing the scratch directory."""
-        from shutil import rmtree
-        try:
-            if self.base:
-                mutter("delete ScratchBranch %s" % self.base)
-                rmtree(self.base)
-        except OSError, e:
-            # Work around for shutil.rmtree failing on Windows when
-            # readonly files are encountered
-            mutter("hit exception in destroying ScratchBranch: %s" % e)
-            for root, dirs, files in os.walk(self.base, topdown=False):
-                for name in files:
-                    os.chmod(os.path.join(root, name), 0700)
-            rmtree(self.base)
-        self._transport = None
-
+        return ScratchBranch(
+            transport=bzrlib.transport.local.ScratchTransport(base))
     
 
 ######################################################################
@@ -1352,7 +1176,7 @@
 
 def is_control_file(filename):
     ## FIXME: better check
-    filename = os.path.normpath(filename)
+    filename = normpath(filename)
     while filename != '':
         head, tail = os.path.split(filename)
         ## mutter('check %r for control file' % ((head, tail), ))
@@ -1362,40 +1186,3 @@
             break
         filename = head
     return False
-
-
-
-def gen_file_id(name):
-    """Return new file id.
-
-    This should probably generate proper UUIDs, but for the moment we
-    cope with just randomness because running uuidgen every time is
-    slow."""
-    import re
-    from binascii import hexlify
-    from time import time
-
-    # get last component
-    idx = name.rfind('/')
-    if idx != -1:
-        name = name[idx+1 : ]
-    idx = name.rfind('\\')
-    if idx != -1:
-        name = name[idx+1 : ]
-
-    # make it not a hidden file
-    name = name.lstrip('.')
-
-    # remove any wierd characters; we don't escape them but rather
-    # just pull them out
-    name = re.sub(r'[^\w.]', '', name)
-
-    s = hexlify(rand_bytes(8))
-    return '-'.join((name, compact_date(time()), s))
-
-
-def gen_root_id():
-    """Return a new tree-root file id."""
-    return gen_file_id('TREE_ROOT')
-
-

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2005-10-06 13:08:14 +0000
+++ bzrlib/builtins.py	2006-01-03 15:10:11 +0000
@@ -14,18 +14,64 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-
+"""builtin bzr commands"""
+
+# DO NOT change this to cStringIO - it results in control files 
+# written as UCS4
+# FIXIT! (Only deal with byte streams OR unicode at any one layer.)
+# RBC 20051018
+
+from StringIO import StringIO
 import sys
 import os
 
 import bzrlib
+from bzrlib import BZRDIR
+from bzrlib.commands import Command, display_command
+from bzrlib.branch import Branch
+from bzrlib.revision import common_ancestor
+import bzrlib.errors as errors
+from bzrlib.errors import (BzrError, BzrCheckError, BzrCommandError, 
+                           NotBranchError, DivergedBranches, NotConflicted,
+                           NoSuchFile, NoWorkingTree, FileInWrongBranch)
+from bzrlib.option import Option
+from bzrlib.revisionspec import RevisionSpec
 import bzrlib.trace
-from bzrlib.trace import mutter, note, log_error, warning
-from bzrlib.errors import BzrError, BzrCheckError, BzrCommandError, NotBranchError
-from bzrlib.branch import Branch
-from bzrlib import BZRDIR
-from bzrlib.commands import Command
-
+from bzrlib.trace import mutter, note, log_error, warning, is_quiet
+from bzrlib.workingtree import WorkingTree
+from bzrlib.log import show_one_log
+
+
+def tree_files(file_list, default_branch=u'.'):
+    try:
+        return internal_tree_files(file_list, default_branch)
+    except FileInWrongBranch, e:
+        raise BzrCommandError("%s is not in the same branch as %s" %
+                             (e.path, file_list[0]))
+
+def internal_tree_files(file_list, default_branch=u'.'):
+    """\
+    Return a branch and list of branch-relative paths.
+    If supplied file_list is empty or None, the branch default will be used,
+    and returned file_list will match the original.
+    """
+    if file_list is None or len(file_list) == 0:
+        return WorkingTree.open_containing(default_branch)[0], file_list
+    tree = WorkingTree.open_containing(file_list[0])[0]
+    new_list = []
+    for filename in file_list:
+        try:
+            new_list.append(tree.relpath(filename))
+        except errors.PathNotChild:
+            raise FileInWrongBranch(tree.branch, filename)
+    return tree, new_list
+
+
+# TODO: Make sure no commands unconditionally use the working directory as a
+# branch.  If a filename argument is used, the first of them should be used to
+# specify the branch.  (Perhaps this can be factored out into some kind of
+# Argument class, representing a file in a branch, where the first occurrence
+# opens the branch?)
 
 class cmd_status(Command):
     """Display status summary.
@@ -66,26 +112,19 @@
     If a revision argument is given, the status is calculated against
     that revision, or between two revisions if two are provided.
     """
-    # XXX: FIXME: bzr status should accept a -r option to show changes
-    # relative to a revision, or between revisions
-
+    
+    # TODO: --no-recurse, --recurse options
+    
     takes_args = ['file*']
-    takes_options = ['all', 'show-ids']
+    takes_options = ['all', 'show-ids', 'revision']
     aliases = ['st', 'stat']
     
+    @display_command
     def run(self, all=False, show_ids=False, file_list=None, revision=None):
-        if file_list:
-            b = Branch.open_containing(file_list[0])
-            file_list = [b.relpath(x) for x in file_list]
-            # special case: only one path was given and it's the root
-            # of the branch
-            if file_list == ['']:
-                file_list = None
-        else:
-            b = Branch.open_containing('.')
+        tree, file_list = tree_files(file_list)
             
         from bzrlib.status import show_status
-        show_status(b, show_unchanged=all, show_ids=show_ids,
+        show_status(tree.branch, show_unchanged=all, show_ids=show_ids,
                     specific_files=file_list, revision=revision)
 
 
@@ -100,30 +139,32 @@
     takes_args = ['revision_id?']
     takes_options = ['revision']
     
+    @display_command
     def run(self, revision_id=None, revision=None):
-        from bzrlib.revisionspec import RevisionSpec
 
         if revision_id is not None and revision is not None:
             raise BzrCommandError('You can only supply one of revision_id or --revision')
         if revision_id is None and revision is None:
             raise BzrCommandError('You must supply either --revision or a revision_id')
-        b = Branch.open_containing('.')
+        b = WorkingTree.open_containing(u'.')[0].branch
         if revision_id is not None:
-            sys.stdout.write(b.get_revision_xml_file(revision_id).read())
+            sys.stdout.write(b.get_revision_xml(revision_id))
         elif revision is not None:
             for rev in revision:
                 if rev is None:
                     raise BzrCommandError('You cannot specify a NULL revision.')
                 revno, rev_id = rev.in_history(b)
-                sys.stdout.write(b.get_revision_xml_file(rev_id).read())
+                sys.stdout.write(b.get_revision_xml(rev_id))
     
 
 class cmd_revno(Command):
     """Show current revision number.
 
     This is equal to the number of revisions on this branch."""
-    def run(self):
-        print Branch.open_containing('.').revno()
+    takes_args = ['location?']
+    @display_command
+    def run(self, location=u'.'):
+        print Branch.open_containing(location)[0].revno()
 
 
 class cmd_revision_info(Command):
@@ -132,8 +173,8 @@
     hidden = True
     takes_args = ['revision_info*']
     takes_options = ['revision']
+    @display_command
     def run(self, revision=None, revision_info_list=[]):
-        from bzrlib.revisionspec import RevisionSpec
 
         revs = []
         if revision is not None:
@@ -144,7 +185,7 @@
         if len(revs) == 0:
             raise BzrCommandError('You must supply a revision identifier')
 
-        b = Branch.open_containing('.')
+        b = WorkingTree.open_containing(u'.')[0].branch
 
         for rev in revs:
             revinfo = rev.in_history(b)
@@ -176,15 +217,40 @@
     implicitly add the parent, and so on up to the root. This means
     you should never need to explictly add a directory, they'll just
     get added when you add a file in the directory.
+
+    --dry-run will show which files would be added, but not actually 
+    add them.
     """
     takes_args = ['file*']
-    takes_options = ['verbose', 'no-recurse']
-    
-    def run(self, file_list, verbose=False, no_recurse=False):
-        # verbose currently has no effect
-        from bzrlib.add import smart_add, add_reporter_print
-        smart_add(file_list, not no_recurse, add_reporter_print)
-
+    takes_options = ['no-recurse', 'dry-run', 'verbose']
+
+    def run(self, file_list, no_recurse=False, dry_run=False, verbose=False):
+        import bzrlib.add
+
+        if dry_run:
+            if is_quiet():
+                # This is pointless, but I'd rather not raise an error
+                action = bzrlib.add.add_action_null
+            else:
+                action = bzrlib.add.add_action_print
+        elif is_quiet():
+            action = bzrlib.add.add_action_add
+        else:
+            action = bzrlib.add.add_action_add_and_print
+
+        added, ignored = bzrlib.add.smart_add(file_list, not no_recurse, 
+                                              action)
+        if len(ignored) > 0:
+            for glob in sorted(ignored.keys()):
+                match_len = len(ignored[glob])
+                if verbose:
+                    for path in ignored[glob]:
+                        print "ignored %s matching \"%s\"" % (path, glob)
+                else:
+                    print "ignored %d file(s) matching \"%s\"" % (match_len,
+                                                              glob)
+            print "If you wish to add some of these files, please add them"\
+                " by name."
 
 
 class cmd_mkdir(Command):
@@ -195,13 +261,10 @@
     takes_args = ['dir+']
 
     def run(self, dir_list):
-        b = None
-        
         for d in dir_list:
             os.mkdir(d)
-            if not b:
-                b = Branch.open_containing(d)
-            b.add([d])
+            wt, dd = WorkingTree.open_containing(d)
+            wt.add([dd])
             print 'added', d
 
 
@@ -210,26 +273,37 @@
     takes_args = ['filename']
     hidden = True
     
+    @display_command
     def run(self, filename):
-        print Branch.open_containing(filename).relpath(filename)
-
+        tree, relpath = WorkingTree.open_containing(filename)
+        print relpath
 
 
 class cmd_inventory(Command):
-    """Show inventory of the current working copy or a revision."""
-    takes_options = ['revision', 'show-ids']
+    """Show inventory of the current working copy or a revision.
+
+    It is possible to limit the output to a particular entry
+    type using the --kind option.  For example; --kind file.
+    """
+    takes_options = ['revision', 'show-ids', 'kind']
     
-    def run(self, revision=None, show_ids=False):
-        b = Branch.open_containing('.')
+    @display_command
+    def run(self, revision=None, show_ids=False, kind=None):
+        if kind and kind not in ['file', 'directory', 'symlink']:
+            raise BzrCommandError('invalid kind specified')
+        tree = WorkingTree.open_containing(u'.')[0]
         if revision is None:
-            inv = b.read_working_inventory()
+            inv = tree.read_working_inventory()
         else:
             if len(revision) > 1:
                 raise BzrCommandError('bzr inventory --revision takes'
                     ' exactly one revision identifier')
-            inv = b.get_revision_inventory(revision[0].in_history(b).rev_id)
+            inv = tree.branch.get_revision_inventory(
+                revision[0].in_history(tree.branch).rev_id)
 
         for path, entry in inv.entries():
+            if kind and kind != entry.kind:
+                continue
             if show_ids:
                 print '%-50s %s' % (path, entry.file_id)
             else:
@@ -246,10 +320,9 @@
     """
     takes_args = ['source$', 'dest']
     def run(self, source_list, dest):
-        b = Branch.open_containing('.')
-
+        tree, source_list = tree_files(source_list)
         # TODO: glob expansion on windows?
-        b.move([b.relpath(s) for s in source_list], b.relpath(dest))
+        tree.move(source_list, tree.relpath(dest))
 
 
 class cmd_rename(Command):
@@ -263,15 +336,14 @@
 
     See also the 'move' command, which moves files into a different
     directory without changing their name.
-
-    TODO: Some way to rename multiple files without invoking bzr for each
-    one?"""
+    """
+    # TODO: Some way to rename multiple files without invoking 
+    # bzr for each one?"""
     takes_args = ['from_name', 'to_name']
     
     def run(self, from_name, to_name):
-        b = Branch.open_containing('.')
-        b.rename_one(b.relpath(from_name), b.relpath(to_name))
-
+        tree, (from_name, to_name) = tree_files((from_name, to_name))
+        tree.rename_one(from_name, to_name)
 
 
 class cmd_mv(Command):
@@ -291,78 +363,171 @@
     def run(self, names_list):
         if len(names_list) < 2:
             raise BzrCommandError("missing file argument")
-        b = Branch.open_containing(names_list[0])
-
-        rel_names = [b.relpath(x) for x in names_list]
+        tree, rel_names = tree_files(names_list)
         
         if os.path.isdir(names_list[-1]):
             # move into existing directory
-            for pair in b.move(rel_names[:-1], rel_names[-1]):
+            for pair in tree.move(rel_names[:-1], rel_names[-1]):
                 print "%s => %s" % pair
         else:
             if len(names_list) != 2:
                 raise BzrCommandError('to mv multiple files the destination '
                                       'must be a versioned directory')
-            b.rename_one(rel_names[0], rel_names[1])
+            tree.rename_one(rel_names[0], rel_names[1])
             print "%s => %s" % (rel_names[0], rel_names[1])
             
     
-
-
 class cmd_pull(Command):
     """Pull any changes from another branch into the current one.
 
-    If the location is omitted, the last-used location will be used.
-    Both the revision history and the working directory will be
-    updated.
+    If there is no default location set, the first pull will set it.  After
+    that, you can omit the location to use the default.  To change the
+    default, use --remember.
 
     This command only works on branches that have not diverged.  Branches are
     considered diverged if both branches have had commits without first
     pulling from the other.
 
     If branches have diverged, you can use 'bzr merge' to pull the text changes
-    from one into the other.
+    from one into the other.  Once one branch has merged, the other should
+    be able to pull it again.
+
+    If you want to forget your local changes and just update your branch to
+    match the remote one, use --overwrite.
     """
-    takes_options = ['remember']
+    takes_options = ['remember', 'overwrite', 'verbose']
     takes_args = ['location?']
 
-    def run(self, location=None, remember=False):
+    def run(self, location=None, remember=False, overwrite=False, verbose=False):
         from bzrlib.merge import merge
-        import tempfile
         from shutil import rmtree
         import errno
-        
-        br_to = Branch.open_containing('.')
-        stored_loc = br_to.get_parent()
+        # FIXME: too much stuff is in the command class        
+        tree_to = WorkingTree.open_containing(u'.')[0]
+        stored_loc = tree_to.branch.get_parent()
         if location is None:
             if stored_loc is None:
                 raise BzrCommandError("No pull location known or specified.")
             else:
                 print "Using saved location: %s" % stored_loc
                 location = stored_loc
-        cache_root = tempfile.mkdtemp()
-        from bzrlib.errors import DivergedBranches
-        br_from = Branch.open_containing(location)
-        location = br_from.base
-        old_revno = br_to.revno()
-        try:
-            from bzrlib.errors import DivergedBranches
-            br_from = Branch.open(location)
-            br_from.setup_caching(cache_root)
-            location = br_from.base
-            old_revno = br_to.revno()
+
+        br_from = Branch.open(location)
+        br_to = tree_to.branch
+
+        old_rh = br_to.revision_history()
+        count = tree_to.pull(br_from, overwrite)
+
+        if br_to.get_parent() is None or remember:
+            br_to.set_parent(location)
+        note('%d revision(s) pulled.' % (count,))
+
+        if verbose:
+            new_rh = tree_to.branch.revision_history()
+            if old_rh != new_rh:
+                # Something changed
+                from bzrlib.log import show_changed_revisions
+                show_changed_revisions(tree_to.branch, old_rh, new_rh)
+
+
+class cmd_push(Command):
+    """Push this branch into another branch.
+    
+    The remote branch will not have its working tree populated because this
+    is both expensive, and may not be supported on the remote file system.
+    
+    Some smart servers or protocols *may* put the working tree in place.
+
+    If there is no default push location set, the first push will set it.
+    After that, you can omit the location to use the default.  To change the
+    default, use --remember.
+
+    This command only works on branches that have not diverged.  Branches are
+    considered diverged if the branch being pushed to is not an older version
+    of this branch.
+
+    If branches have diverged, you can use 'bzr push --overwrite' to replace
+    the other branch completely.
+    
+    If you want to ensure you have the different changes in the other branch,
+    do a merge (see bzr help merge) from the other branch, and commit that
+    before doing a 'push --overwrite'.
+    """
+    takes_options = ['remember', 'overwrite', 
+                     Option('create-prefix', 
+                            help='Create the path leading up to the branch '
+                                 'if it does not already exist')]
+    takes_args = ['location?']
+
+    def run(self, location=None, remember=False, overwrite=False,
+            create_prefix=False, verbose=False):
+        # FIXME: Way too big!  Put this into a function called from the
+        # command.
+        import errno
+        from shutil import rmtree
+        from bzrlib.transport import get_transport
+        
+        tree_from = WorkingTree.open_containing(u'.')[0]
+        br_from = tree_from.branch
+        stored_loc = tree_from.branch.get_push_location()
+        if location is None:
+            if stored_loc is None:
+                raise BzrCommandError("No push location known or specified.")
+            else:
+                print "Using saved location: %s" % stored_loc
+                location = stored_loc
+        try:
+            br_to = Branch.open(location)
+        except NotBranchError:
+            # create a branch.
+            transport = get_transport(location).clone('..')
+            if not create_prefix:
+                try:
+                    transport.mkdir(transport.relpath(location))
+                except NoSuchFile:
+                    raise BzrCommandError("Parent directory of %s "
+                                          "does not exist." % location)
+            else:
+                current = transport.base
+                needed = [(transport, transport.relpath(location))]
+                while needed:
+                    try:
+                        transport, relpath = needed[-1]
+                        transport.mkdir(relpath)
+                        needed.pop()
+                    except NoSuchFile:
+                        new_transport = transport.clone('..')
+                        needed.append((new_transport,
+                                       new_transport.relpath(transport.base)))
+                        if new_transport.base == transport.base:
+                            raise BzrCommandError("Could not creeate "
+                                                  "path prefix.")
+            br_to = Branch.initialize(location)
+        old_rh = br_to.revision_history()
+        try:
             try:
-                br_to.update_revisions(br_from)
-            except DivergedBranches:
-                raise BzrCommandError("These branches have diverged."
-                    "  Try merge.")
-                
-            merge(('.', -1), ('.', old_revno), check_clean=False)
-            if stored_loc is None or remember:
-                br_to.set_parent(location)
-        finally:
-            rmtree(cache_root)
+                tree_to = br_to.working_tree()
+            except NoWorkingTree:
+                # TODO: This should be updated for branches which don't have a
+                # working tree, as opposed to ones where we just couldn't 
+                # update the tree.
+                warning('Unable to update the working tree of: %s' % (br_to.base,))
+                count = br_to.pull(br_from, overwrite)
+            else:
+                count = tree_to.pull(br_from, overwrite)
+        except DivergedBranches:
+            raise BzrCommandError("These branches have diverged."
+                                  "  Try a merge then push with overwrite.")
+        if br_from.get_push_location() is None or remember:
+            br_from.set_push_location(location)
+        note('%d revision(s) pushed.' % (count,))
 
+        if verbose:
+            new_rh = br_to.revision_history()
+            if old_rh != new_rh:
+                # Something changed
+                from bzrlib.log import show_changed_revisions
+                show_changed_revisions(br_to, old_rh, new_rh)
 
 
 class cmd_branch(Command):
@@ -384,27 +549,25 @@
 
     def run(self, from_location, to_location=None, revision=None, basis=None):
         from bzrlib.clone import copy_branch
-        import tempfile
         import errno
         from shutil import rmtree
-        cache_root = tempfile.mkdtemp()
-        try:
-            if revision is None:
-                revision = [None]
-            elif len(revision) > 1:
-                raise BzrCommandError(
-                    'bzr branch --revision takes exactly 1 revision value')
-            try:
-                br_from = Branch.open(from_location)
-            except OSError, e:
-                if e.errno == errno.ENOENT:
-                    raise BzrCommandError('Source location "%s" does not'
-                                          ' exist.' % to_location)
-                else:
-                    raise
-            br_from.setup_caching(cache_root)
+        if revision is None:
+            revision = [None]
+        elif len(revision) > 1:
+            raise BzrCommandError(
+                'bzr branch --revision takes exactly 1 revision value')
+        try:
+            br_from = Branch.open(from_location)
+        except OSError, e:
+            if e.errno == errno.ENOENT:
+                raise BzrCommandError('Source location "%s" does not'
+                                      ' exist.' % to_location)
+            else:
+                raise
+        br_from.lock_read()
+        try:
             if basis is not None:
-                basis_branch = Branch.open_containing(basis)
+                basis_branch = WorkingTree.open_containing(basis)[0].branch
             else:
                 basis_branch = None
             if len(revision) == 1 and revision[0] is not None:
@@ -413,6 +576,9 @@
                 revision_id = None
             if to_location is None:
                 to_location = os.path.basename(from_location.rstrip("/\\"))
+                name = None
+            else:
+                name = os.path.basename(to_location) + '\n'
             try:
                 os.mkdir(to_location)
             except OSError, e:
@@ -428,27 +594,34 @@
                 copy_branch(br_from, to_location, revision_id, basis_branch)
             except bzrlib.errors.NoSuchRevision:
                 rmtree(to_location)
-                msg = "The branch %s has no revision %d." % (from_location, revision[0])
+                msg = "The branch %s has no revision %s." % (from_location, revision[0])
                 raise BzrCommandError(msg)
             except bzrlib.errors.UnlistableBranch:
+                rmtree(to_location)
                 msg = "The branch %s cannot be used as a --basis"
+                raise BzrCommandError(msg)
+            branch = Branch.open(to_location)
+            if name:
+                name = StringIO(name)
+                branch.put_controlfile('branch-name', name)
+            note('Branched %d revision(s).' % branch.revno())
         finally:
-            rmtree(cache_root)
+            br_from.unlock()
 
 
 class cmd_renames(Command):
     """Show list of renamed files.
-
-    TODO: Option to show renames between two historical versions.
-
-    TODO: Only show renames under dir, rather than in the whole branch.
     """
+    # TODO: Option to show renames between two historical versions.
+
+    # TODO: Only show renames under dir, rather than in the whole branch.
     takes_args = ['dir?']
 
-    def run(self, dir='.'):
-        b = Branch.open_containing(dir)
-        old_inv = b.basis_tree().inventory
-        new_inv = b.read_working_inventory()
+    @display_command
+    def run(self, dir=u'.'):
+        tree = WorkingTree.open_containing(dir)[0]
+        old_inv = tree.branch.basis_tree().inventory
+        new_inv = tree.read_working_inventory()
 
         renames = list(bzrlib.tree.find_renames(old_inv, new_inv))
         renames.sort()
@@ -460,10 +633,10 @@
     """Show statistical information about a branch."""
     takes_args = ['branch?']
     
+    @display_command
     def run(self, branch=None):
         import info
-
-        b = Branch.open_containing(branch)
+        b = WorkingTree.open_containing(branch)[0].branch
         info.show_info(b)
 
 
@@ -475,10 +648,11 @@
     """
     takes_args = ['file+']
     takes_options = ['verbose']
+    aliases = ['rm']
     
     def run(self, file_list, verbose=False):
-        b = Branch.open_containing(file_list[0])
-        b.remove([b.relpath(f) for f in file_list], verbose=verbose)
+        tree, file_list = tree_files(file_list)
+        tree.remove(file_list, verbose=verbose)
 
 
 class cmd_file_id(Command):
@@ -490,9 +664,10 @@
     """
     hidden = True
     takes_args = ['filename']
+    @display_command
     def run(self, filename):
-        b = Branch.open_containing(filename)
-        i = b.inventory.path2id(b.relpath(filename))
+        tree, relpath = WorkingTree.open_containing(filename)
+        i = tree.inventory.path2id(relpath)
         if i == None:
             raise BzrError("%r is not a versioned file" % filename)
         else:
@@ -506,10 +681,11 @@
     starting at the branch root."""
     hidden = True
     takes_args = ['filename']
+    @display_command
     def run(self, filename):
-        b = Branch.open_containing(filename)
-        inv = b.inventory
-        fid = inv.path2id(b.relpath(filename))
+        tree, relpath = WorkingTree.open_containing(filename)
+        inv = tree.inventory
+        fid = inv.path2id(relpath)
         if fid == None:
             raise BzrError("%r is not a versioned file" % filename)
         for fip in inv.get_idpath(fid):
@@ -519,30 +695,25 @@
 class cmd_revision_history(Command):
     """Display list of revision ids on this branch."""
     hidden = True
+    @display_command
     def run(self):
-        for patchid in Branch.open_containing('.').revision_history():
+        branch = WorkingTree.open_containing(u'.')[0].branch
+        for patchid in branch.revision_history():
             print patchid
 
 
 class cmd_ancestry(Command):
     """List all revisions merged into this branch."""
     hidden = True
+    @display_command
     def run(self):
-        b = find_branch('.')
+        tree = WorkingTree.open_containing(u'.')[0]
+        b = tree.branch
+        # FIXME. should be tree.last_revision
         for revision_id in b.get_ancestry(b.last_revision()):
             print revision_id
 
 
-class cmd_directories(Command):
-    """Display list of versioned directories in this branch."""
-    def run(self):
-        for name, ie in Branch.open_containing('.').read_working_inventory().directories():
-            if name == '':
-                print '.'
-            else:
-                print name
-
-
 class cmd_init(Command):
     """Make a directory into a versioned branch.
 
@@ -552,12 +723,24 @@
     Recipe for importing a tree of files:
         cd ~/project
         bzr init
-        bzr add -v .
+        bzr add .
         bzr status
         bzr commit -m 'imported project'
     """
-    def run(self):
-        Branch.initialize('.')
+    takes_args = ['location?']
+    def run(self, location=None):
+        from bzrlib.branch import Branch
+        if location is None:
+            location = u'.'
+        else:
+            # The path has to exist to initialize a
+            # branch inside of it.
+            # Just using os.mkdir, since I don't
+            # believe that we want to create a bunch of
+            # locations if the user supplies an extended path
+            if not os.path.exists(location):
+                os.mkdir(location)
+        Branch.initialize(location)
 
 
 class cmd_diff(Command):
@@ -566,77 +749,82 @@
     If files are listed, only the changes in those files are listed.
     Otherwise, all changes for the tree are listed.
 
-    TODO: Allow diff across branches.
-
-    TODO: Option to use external diff command; could be GNU diff, wdiff,
-          or a graphical diff.
-
-    TODO: Python difflib is not exactly the same as unidiff; should
-          either fix it up or prefer to use an external diff.
-
-    TODO: If a directory is given, diff everything under that.
-
-    TODO: Selected-file diff is inefficient and doesn't show you
-          deleted files.
-
-    TODO: This probably handles non-Unix newlines poorly.
-
     examples:
         bzr diff
         bzr diff -r1
         bzr diff -r1..2
     """
+    # TODO: Allow diff across branches.
+    # TODO: Option to use external diff command; could be GNU diff, wdiff,
+    #       or a graphical diff.
+
+    # TODO: Python difflib is not exactly the same as unidiff; should
+    #       either fix it up or prefer to use an external diff.
+
+    # TODO: If a directory is given, diff everything under that.
+
+    # TODO: Selected-file diff is inefficient and doesn't show you
+    #       deleted files.
+
+    # TODO: This probably handles non-Unix newlines poorly.
     
     takes_args = ['file*']
     takes_options = ['revision', 'diff-options']
     aliases = ['di', 'dif']
 
+    @display_command
     def run(self, revision=None, file_list=None, diff_options=None):
         from bzrlib.diff import show_diff
-
-        if file_list:
-            b = Branch.open_containing(file_list[0])
-            file_list = [b.relpath(f) for f in file_list]
-            if file_list == ['']:
-                # just pointing to top-of-tree
-                file_list = None
-        else:
-            b = Branch.open_containing('.')
-
+        try:
+            tree, file_list = internal_tree_files(file_list)
+            b = None
+            b2 = None
+        except FileInWrongBranch:
+            if len(file_list) != 2:
+                raise BzrCommandError("Files are in different branches")
+
+            b, file1 = Branch.open_containing(file_list[0])
+            b2, file2 = Branch.open_containing(file_list[1])
+            if file1 != "" or file2 != "":
+                # FIXME diff those two files. rbc 20051123
+                raise BzrCommandError("Files are in different branches")
+            file_list = None
         if revision is not None:
+            if b2 is not None:
+                raise BzrCommandError("Can't specify -r with two branches")
             if len(revision) == 1:
-                show_diff(b, revision[0], specific_files=file_list,
-                          external_diff_options=diff_options)
+                return show_diff(tree.branch, revision[0], specific_files=file_list,
+                                 external_diff_options=diff_options)
             elif len(revision) == 2:
-                show_diff(b, revision[0], specific_files=file_list,
-                          external_diff_options=diff_options,
-                          revision2=revision[1])
+                return show_diff(tree.branch, revision[0], specific_files=file_list,
+                                 external_diff_options=diff_options,
+                                 revision2=revision[1])
             else:
                 raise BzrCommandError('bzr diff --revision takes exactly one or two revision identifiers')
         else:
-            show_diff(b, None, specific_files=file_list,
-                      external_diff_options=diff_options)
-
-        
+            if b is not None:
+                return show_diff(b, None, specific_files=file_list,
+                                 external_diff_options=diff_options, b2=b2)
+            else:
+                return show_diff(tree.branch, None, specific_files=file_list,
+                                 external_diff_options=diff_options)
 
 
 class cmd_deleted(Command):
     """List files deleted in the working tree.
-
-    TODO: Show files deleted since a previous revision, or between two revisions.
     """
+    # TODO: Show files deleted since a previous revision, or
+    # between two revisions.
+    # TODO: Much more efficient way to do this: read in new
+    # directories with readdir, rather than stating each one.  Same
+    # level of effort but possibly much less IO.  (Or possibly not,
+    # if the directories are very large...)
+    @display_command
     def run(self, show_ids=False):
-        b = Branch.open_containing('.')
-        old = b.basis_tree()
-        new = b.working_tree()
-
-        ## TODO: Much more efficient way to do this: read in new
-        ## directories with readdir, rather than stating each one.  Same
-        ## level of effort but possibly much less IO.  (Or possibly not,
-        ## if the directories are very large...)
-
+        tree = WorkingTree.open_containing(u'.')[0]
+        old = tree.branch.basis_tree()
         for path, ie in old.inventory.iter_entries():
-            if not new.has_id(ie.file_id):
+            if not tree.has_id(ie.file_id):
                 if show_ids:
                     print '%-50s %s' % (path, ie.file_id)
                 else:
@@ -646,11 +834,12 @@
 class cmd_modified(Command):
     """List files modified in working tree."""
     hidden = True
+    @display_command
     def run(self):
         from bzrlib.delta import compare_trees
 
-        b = Branch.open_containing('.')
-        td = compare_trees(b.basis_tree(), b.working_tree())
+        tree = WorkingTree.open_containing(u'.')[0]
+        td = compare_trees(tree.branch.basis_tree(), tree)
 
         for path, id, kind, text_modified, meta_modified in td.modified:
             print path
@@ -660,10 +849,10 @@
 class cmd_added(Command):
     """List files added in working tree."""
     hidden = True
+    @display_command
     def run(self):
-        b = Branch.open_containing('.')
-        wt = b.working_tree()
-        basis_inv = b.basis_tree().inventory
+        wt = WorkingTree.open_containing(u'.')[0]
+        basis_inv = wt.branch.basis_tree().inventory
         inv = wt.inventory
         for file_id in inv:
             if file_id in basis_inv:
@@ -681,29 +870,35 @@
     The root is the nearest enclosing directory with a .bzr control
     directory."""
     takes_args = ['filename?']
+    @display_command
     def run(self, filename=None):
         """Print the branch root."""
-        b = Branch.open_containing(filename)
-        print b.base
+        tree = WorkingTree.open_containing(filename)[0]
+        print tree.basedir
 
 
 class cmd_log(Command):
     """Show log of this branch.
 
-    To request a range of logs, you can use the command -r begin:end
-    -r revision requests a specific revision, -r :end or -r begin: are
+    To request a range of logs, you can use the command -r begin..end
+    -r revision requests a specific revision, -r ..end or -r begin.. are
     also valid.
-
-    --message allows you to give a regular expression, which will be evaluated
-    so that only matching entries will be displayed.
     """
 
     # TODO: Make --revision support uuid: and hash: [future tag:] notation.
 
     takes_args = ['filename?']
-    takes_options = ['forward', 'timezone', 'verbose', 'show-ids', 'revision',
-                     'long', 'message', 'short',]
-    
+    takes_options = [Option('forward', 
+                            help='show from oldest to newest'),
+                     'timezone', 'verbose', 
+                     'show-ids', 'revision',
+                     'line', 'long', 
+                     Option('message',
+                            help='show revisions whose message matches this regexp',
+                            type=str),
+                     'short',
+                     ]
+    @display_command
     def run(self, filename=None, timezone='original',
             verbose=False,
             show_ids=False,
@@ -711,21 +906,35 @@
             revision=None,
             message=None,
             long=False,
-            short=False):
+            short=False,
+            line=False):
         from bzrlib.log import log_formatter, show_log
         import codecs
-
+        assert message is None or isinstance(message, basestring), \
+            "invalid message argument %r" % message
         direction = (forward and 'forward') or 'reverse'
         
         if filename:
-            b = Branch.open_containing(filename)
-            fp = b.relpath(filename)
-            if fp:
-                file_id = b.read_working_inventory().path2id(fp)
+            # might be a tree:
+            tree = None
+            try:
+                tree, fp = WorkingTree.open_containing(filename)
+                b = tree.branch
+                if fp != '':
+                    inv = tree.read_working_inventory()
+            except NotBranchError:
+                pass
+            if tree is None:
+                b, fp = Branch.open_containing(filename)
+                if fp != '':
+                    inv = b.get_inventory(b.last_revision())
+            if fp != '':
+                file_id = inv.path2id(fp)
             else:
                 file_id = None  # points to branch root
         else:
-            b = Branch.open_containing('.')
+            tree, relpath = WorkingTree.open_containing(u'.')
+            b = tree.branch
             file_id = None
 
         if revision is None:
@@ -739,21 +948,19 @@
         else:
             raise BzrCommandError('bzr log --revision takes one or two values.')
 
-        if rev1 == 0:
-            rev1 = None
-        if rev2 == 0:
-            rev2 = None
+        # By this point, the revision numbers are converted to the +ve
+        # form if they were supplied in the -ve form, so we can do
+        # this comparison in relative safety
+        if rev1 > rev2:
+            (rev2, rev1) = (rev1, rev2)
 
-        mutter('encoding log as %r' % bzrlib.user_encoding)
+        mutter('encoding log as %r', bzrlib.user_encoding)
 
         # use 'replace' so that we don't abort if trying to write out
         # in e.g. the default C locale.
         outf = codecs.getwriter(bzrlib.user_encoding)(sys.stdout, errors='replace')
 
-        if not short:
-            log_format = 'long'
-        else:
-            log_format = 'short'
+        log_format = get_log_format(long=long, short=short, line=line)
         lf = log_formatter(log_format,
                            show_ids=show_ids,
                            to_file=outf,
@@ -768,6 +975,15 @@
                  end_revision=rev2,
                  search=message)
 
+def get_log_format(long=False, short=False, line=False, default='long'):
+    log_format = default
+    if long:
+        log_format = 'long'
+    if short:
+        log_format = 'short'
+    if line:
+        log_format = 'line'
+    return log_format
 
 
 class cmd_touching_revisions(Command):
@@ -776,52 +992,88 @@
     A more user-friendly interface is "bzr log FILE"."""
     hidden = True
     takes_args = ["filename"]
+    @display_command
     def run(self, filename):
-        b = Branch.open_containing(filename)
-        inv = b.read_working_inventory()
-        file_id = inv.path2id(b.relpath(filename))
+        tree, relpath = WorkingTree.open_containing(filename)
+        b = tree.branch
+        inv = tree.read_working_inventory()
+        file_id = inv.path2id(relpath)
         for revno, revision_id, what in bzrlib.log.find_touching_revisions(b, file_id):
             print "%6d %s" % (revno, what)
 
 
 class cmd_ls(Command):
     """List files in a tree.
-
-    TODO: Take a revision or remote path and list that tree instead.
     """
+    # TODO: Take a revision or remote path and list that tree instead.
     hidden = True
-    def run(self, revision=None, verbose=False):
-        b = Branch.open_containing('.')
-        if revision == None:
-            tree = b.working_tree()
-        else:
-            tree = b.revision_tree(revision.in_history(b).rev_id)
+    takes_options = ['verbose', 'revision',
+                     Option('non-recursive',
+                            help='don\'t recurse into sub-directories'),
+                     Option('from-root',
+                            help='Print all paths from the root of the branch.'),
+                     Option('unknown', help='Print unknown files'),
+                     Option('versioned', help='Print versioned files'),
+                     Option('ignored', help='Print ignored files'),
+
+                     Option('null', help='Null separate the files'),
+                    ]
+    @display_command
+    def run(self, revision=None, verbose=False, 
+            non_recursive=False, from_root=False,
+            unknown=False, versioned=False, ignored=False,
+            null=False):
+
+        if verbose and null:
+            raise BzrCommandError('Cannot set both --verbose and --null')
+        all = not (unknown or versioned or ignored)
+
+        selection = {'I':ignored, '?':unknown, 'V':versioned}
+
+        tree, relpath = WorkingTree.open_containing(u'.')
+        if from_root:
+            relpath = u''
+        elif relpath:
+            relpath += '/'
+        if revision is not None:
+            tree = tree.branch.revision_tree(
+                revision[0].in_history(tree.branch).rev_id)
         for fp, fc, kind, fid, entry in tree.list_files():
-            if verbose:
-                kindch = entry.kind_character()
-                print '%-8s %s%s' % (fc, fp, kindch)
-            else:
-                print fp
-
+            if fp.startswith(relpath):
+                fp = fp[len(relpath):]
+                if non_recursive and '/' in fp:
+                    continue
+                if not all and not selection[fc]:
+                    continue
+                if verbose:
+                    kindch = entry.kind_character()
+                    print '%-8s %s%s' % (fc, fp, kindch)
+                elif null:
+                    sys.stdout.write(fp)
+                    sys.stdout.write('\0')
+                    sys.stdout.flush()
+                else:
+                    print fp
 
 
 class cmd_unknowns(Command):
     """List unknown files."""
+    @display_command
     def run(self):
         from bzrlib.osutils import quotefn
-        for f in Branch.open_containing('.').unknowns():
+        for f in WorkingTree.open_containing(u'.')[0].unknowns():
             print quotefn(f)
 
 
-
 class cmd_ignore(Command):
     """Ignore a command or pattern.
 
     To remove patterns from the ignore list, edit the .bzrignore file.
 
     If the pattern contains a slash, it is compared to the whole path
-    from the branch root.  Otherwise, it is comapred to only the last
-    component of the path.
+    from the branch root.  Otherwise, it is compared to only the last
+    component of the path.  To match a file only in the root directory,
+    prepend './'.
 
     Ignore patterns are case-insensitive on case-insensitive systems.
 
@@ -831,14 +1083,15 @@
         bzr ignore ./Makefile
         bzr ignore '*.class'
     """
+    # TODO: Complain if the filename is absolute
     takes_args = ['name_pattern']
     
     def run(self, name_pattern):
         from bzrlib.atomicfile import AtomicFile
         import os.path
 
-        b = Branch.open_containing('.')
-        ifn = b.abspath('.bzrignore')
+        tree, relpath = WorkingTree.open_containing(u'.')
+        ifn = tree.abspath('.bzrignore')
 
         if os.path.exists(ifn):
             f = open(ifn, 'rt')
@@ -863,21 +1116,21 @@
         finally:
             f.close()
 
-        inv = b.working_tree().inventory
+        inv = tree.inventory
         if inv.path2id('.bzrignore'):
             mutter('.bzrignore is already versioned')
         else:
             mutter('need to make new .bzrignore file versioned')
-            b.add(['.bzrignore'])
-
+            tree.add(['.bzrignore'])
 
 
 class cmd_ignored(Command):
     """List ignored files and the patterns that matched them.
 
     See also: bzr ignore"""
+    @display_command
     def run(self):
-        tree = Branch.open_containing('.').working_tree()
+        tree = WorkingTree.open_containing(u'.')[0]
         for path, file_class, kind, file_id, entry in tree.list_files():
             if file_class != 'I':
                 continue
@@ -895,13 +1148,14 @@
     hidden = True
     takes_args = ['revno']
     
+    @display_command
     def run(self, revno):
         try:
             revno = int(revno)
         except ValueError:
             raise BzrCommandError("not a valid revision-number: %r" % revno)
 
-        print Branch.open_containing('.').get_rev_id(revno)
+        print WorkingTree.open_containing(u'.')[0].branch.get_rev_id(revno)
 
 
 class cmd_export(Command):
@@ -914,38 +1168,37 @@
     is found exports to a directory (equivalent to --format=dir).
 
     Root may be the top directory for tar, tgz and tbz2 formats. If none
-    is given, the top directory will be the root name of the file."""
-    # TODO: list known exporters
+    is given, the top directory will be the root name of the file.
+
+    Note: export of tree with non-ascii filenames to zip is not supported.
+
+    Supported formats       Autodetected by extension
+    -----------------       -------------------------
+         dir                            -
+         tar                          .tar
+         tbz2                    .tar.bz2, .tbz2
+         tgz                      .tar.gz, .tgz
+         zip                          .zip
+    """
     takes_args = ['dest']
     takes_options = ['revision', 'format', 'root']
     def run(self, dest, revision=None, format=None, root=None):
         import os.path
-        b = Branch.open_containing('.')
+        from bzrlib.export import export
+        tree = WorkingTree.open_containing(u'.')[0]
+        b = tree.branch
         if revision is None:
+            # should be tree.last_revision  FIXME
             rev_id = b.last_revision()
         else:
             if len(revision) != 1:
                 raise BzrError('bzr export --revision takes exactly 1 argument')
             rev_id = revision[0].in_history(b).rev_id
         t = b.revision_tree(rev_id)
-        arg_root, ext = os.path.splitext(os.path.basename(dest))
-        if ext in ('.gz', '.bz2'):
-            new_root, new_ext = os.path.splitext(arg_root)
-            if new_ext == '.tar':
-                arg_root = new_root
-                ext = new_ext + ext
-        if root is None:
-            root = arg_root
-        if not format:
-            if ext in (".tar",):
-                format = "tar"
-            elif ext in (".tar.gz", ".tgz"):
-                format = "tgz"
-            elif ext in (".tar.bz2", ".tbz2"):
-                format = "tbz2"
-            else:
-                format = "dir"
-        t.export(dest, format, root)
+        try:
+            export(t, dest, format, root)
+        except errors.NoSuchExportFormat, e:
+            raise BzrCommandError('Unsupported export format: %s' % e.format)
 
 
 class cmd_cat(Command):
@@ -954,18 +1207,30 @@
     takes_options = ['revision']
     takes_args = ['filename']
 
+    @display_command
     def run(self, filename, revision=None):
-        if revision is None:
-            raise BzrCommandError("bzr cat requires a revision number")
-        elif len(revision) != 1:
+        if revision is not None and len(revision) != 1:
             raise BzrCommandError("bzr cat --revision takes exactly one number")
-        b = Branch.open_containing('.')
-        b.print_file(b.relpath(filename), revision[0].in_history(b).revno)
+        tree = None
+        try:
+            tree, relpath = WorkingTree.open_containing(filename)
+            b = tree.branch
+        except NotBranchError:
+            pass
+
+        if tree is None:
+            b, relpath = Branch.open_containing(filename)
+        if revision is None:
+            revision_id = b.last_revision()
+        else:
+            revision_id = revision[0].in_history(b).rev_id
+        b.print_file(relpath, revision_id)
 
 
 class cmd_local_time_offset(Command):
     """Show the offset in seconds from GMT to local time."""
     hidden = True    
+    @display_command
     def run(self):
         print bzrlib.osutils.local_time_offset()
 
@@ -983,37 +1248,51 @@
     A selected-file commit may fail in some cases where the committed
     tree would be invalid, such as trying to commit a file in a
     newly-added directory that is not itself committed.
-
-    TODO: Run hooks on tree to-be-committed, and after commit.
-
-    TODO: Strict commit that fails if there are unknown or deleted files.
     """
+    # TODO: Run hooks on tree to-be-committed, and after commit.
+
+    # TODO: Strict commit that fails if there are deleted files.
+    #       (what does "deleted files" mean ??)
+
+    # TODO: Give better message for -s, --summary, used by tla people
+
+    # XXX: verbose currently does nothing
+
     takes_args = ['selected*']
-    takes_options = ['message', 'file', 'verbose', 'unchanged']
+    takes_options = ['message', 'verbose', 
+                     Option('unchanged',
+                            help='commit even if nothing has changed'),
+                     Option('file', type=str, 
+                            argname='msgfile',
+                            help='file containing commit message'),
+                     Option('strict',
+                            help="refuse to commit if there are unknown "
+                            "files in the working tree."),
+                     ]
     aliases = ['ci', 'checkin']
 
-    # TODO: Give better message for -s, --summary, used by tla people
-
-    # XXX: verbose currently does nothing
-    
     def run(self, message=None, file=None, verbose=True, selected_list=None,
-            unchanged=False):
-        from bzrlib.errors import PointlessCommit, ConflictsInTree
-        from bzrlib.msgeditor import edit_commit_message
+            unchanged=False, strict=False):
+        from bzrlib.errors import (PointlessCommit, ConflictsInTree,
+                StrictCommitFailed)
+        from bzrlib.msgeditor import edit_commit_message, \
+                make_commit_message_template
         from bzrlib.status import show_status
-        from cStringIO import StringIO
-
-        b = Branch.open_containing('.')
-        if selected_list:
-            selected_list = [b.relpath(s) for s in selected_list]
-
-            
+        from tempfile import TemporaryFile
+        import codecs
+
+        # TODO: Need a blackbox test for invoking the external editor; may be
+        # slightly problematic to run this cross-platform.
+
+        # TODO: do more checks that the commit will succeed before 
+        # spending the user's valuable time typing a commit message.
+        #
+        # TODO: if the commit *does* happen to fail, then save the commit 
+        # message to a temporary file where it can be recovered
+        tree, selected_list = tree_files(selected_list)
         if message is None and not file:
-            catcher = StringIO()
-            show_status(b, specific_files=selected_list,
-                        to_file=catcher)
-            message = edit_commit_message(catcher.getvalue())
-
+            template = make_commit_message_template(tree, selected_list)
+            message = edit_commit_message(template)
             if message is None:
                 raise BzrCommandError("please specify a commit message"
                                       " with either --message or --file")
@@ -1028,9 +1307,8 @@
                 raise BzrCommandError("empty commit message specified")
             
         try:
-            b.commit(message,
-                     specific_files=selected_list,
-                     allow_pointless=unchanged)
+            tree.commit(message, specific_files=selected_list,
+                        allow_pointless=unchanged, strict=strict)
         except PointlessCommit:
             # FIXME: This should really happen before the file is read in;
             # perhaps prepare the commit; get the message; then actually commit
@@ -1039,6 +1317,10 @@
         except ConflictsInTree:
             raise BzrCommandError("Conflicts detected in working tree.  "
                 'Use "bzr conflicts" to list, "bzr resolve FILE" to resolve.')
+        except StrictCommitFailed:
+            raise BzrCommandError("Commit refused because there are unknown "
+                                  "files in the working tree.")
+        note('Committed revision %d.' % (tree.branch.revno(),))
 
 
 class cmd_check(Command):
@@ -1047,12 +1329,17 @@
     This command checks various invariants about the branch storage to
     detect data corruption or bzr bugs.
     """
-    takes_args = ['dir?']
+    takes_args = ['branch?']
+    takes_options = ['verbose']
 
-    def run(self, dir='.'):
+    def run(self, branch=None, verbose=False):
         from bzrlib.check import check
-
-        check(Branch.open_containing(dir))
+        if branch is None:
+            tree = WorkingTree.open_containing()[0]
+            branch = tree.branch
+        else:
+            branch = Branch.open(branch)
+        check(branch, verbose)
 
 
 class cmd_scan_cache(Command):
@@ -1060,7 +1347,7 @@
     def run(self):
         from bzrlib.hashcache import HashCache
 
-        c = HashCache('.')
+        c = HashCache(u'.')
         c.read()
         c.scan()
             
@@ -1086,7 +1373,7 @@
     """
     takes_args = ['dir?']
 
-    def run(self, dir='.'):
+    def run(self, dir=u'.'):
         from bzrlib.upgrade import upgrade
         upgrade(dir)
 
@@ -1095,17 +1382,36 @@
     """Show bzr user id."""
     takes_options = ['email']
     
+    @display_command
     def run(self, email=False):
         try:
-            b = bzrlib.branch.Branch.open_containing('.')
+            b = WorkingTree.open_containing(u'.')[0].branch
+            config = bzrlib.config.BranchConfig(b)
         except NotBranchError:
-            b = None
+            config = bzrlib.config.GlobalConfig()
         
         if email:
-            print bzrlib.osutils.user_email(b)
-        else:
-            print bzrlib.osutils.username(b)
-
+            print config.user_email()
+        else:
+            print config.username()
+
+class cmd_nick(Command):
+    """\
+    Print or set the branch nickname.  
+    If unset, the tree root directory name is used as the nickname
+    To print the current nickname, execute with no argument.  
+    """
+    takes_args = ['nickname?']
+    def run(self, nickname=None):
+        branch = Branch.open_containing(u'.')[0]
+        if nickname is None:
+            self.printme(branch)
+        else:
+            branch.nick = nickname
+
+    @display_command
+    def printme(self, branch):
+        print branch.nick 
 
 class cmd_selftest(Command):
     """Run internal test suite.
@@ -1113,17 +1419,24 @@
     This creates temporary test directories in the working directory,
     but not existing data is affected.  These directories are deleted
     if the tests pass, or left behind to help in debugging if they
-    fail.
+    fail and --keep-output is specified.
     
     If arguments are given, they are regular expressions that say
-    which tests should run."""
+    which tests should run.
+    """
     # TODO: --list should give a list of all available tests
     hidden = True
-    takes_args = ['testnames*']
-    takes_options = ['verbose', 'pattern']
-    def run(self, testnames_list=None, verbose=False, pattern=".*"):
+    takes_args = ['testspecs*']
+    takes_options = ['verbose', 
+                     Option('one', help='stop when one test fails'),
+                     Option('keep-output', 
+                            help='keep output directories when tests fail')
+                    ]
+
+    def run(self, testspecs_list=None, verbose=False, one=False,
+            keep_output=False):
         import bzrlib.ui
-        from bzrlib.selftest import selftest
+        from bzrlib.tests import selftest
         # we don't want progress meters from the tests to go to the
         # real output; and we don't want log messages cluttering up
         # the real logs.
@@ -1131,9 +1444,14 @@
         bzrlib.trace.info('running tests...')
         try:
             bzrlib.ui.ui_factory = bzrlib.ui.SilentUIFactory()
+            if testspecs_list is not None:
+                pattern = '|'.join(testspecs_list)
+            else:
+                pattern = ".*"
             result = selftest(verbose=verbose, 
                               pattern=pattern,
-                              testnames=testnames_list)
+                              stop_on_failure=one, 
+                              keep_output=keep_output)
             if result:
                 bzrlib.trace.info('tests passed')
             else:
@@ -1143,12 +1461,31 @@
             bzrlib.ui.ui_factory = save_ui
 
 
+def _get_bzr_branch():
+    """If bzr is run from a branch, return Branch or None"""
+    import bzrlib.errors
+    from bzrlib.branch import Branch
+    from bzrlib.osutils import abspath
+    from os.path import dirname
+    
+    try:
+        branch = Branch.open(dirname(abspath(dirname(__file__))))
+        return branch
+    except bzrlib.errors.BzrError:
+        return None
+    
+
 def show_version():
     print "bzr (bazaar-ng) %s" % bzrlib.__version__
     # is bzrlib itself in a branch?
-    bzrrev = bzrlib.get_bzr_revision()
-    if bzrrev:
-        print "  (bzr checkout, revision %d {%s})" % bzrrev
+    branch = _get_bzr_branch()
+    if branch:
+        rh = branch.revision_history()
+        revno = len(rh)
+        print "  bzr checkout, revision %d" % (revno,)
+        print "  nick: %s" % (branch.nick,)
+        if rh:
+            print "  revid: %s" % (rh[-1],)
     print bzrlib.__copyright__
     print "http://bazaar-ng.org/"
     print
@@ -1159,30 +1496,32 @@
 
 class cmd_version(Command):
     """Show version of bzr."""
+    @display_command
     def run(self):
         show_version()
 
 class cmd_rocks(Command):
     """Statement of optimism."""
     hidden = True
+    @display_command
     def run(self):
         print "it sure does!"
 
 
 class cmd_find_merge_base(Command):
     """Find and print a base revision for merging two branches.
-
-    TODO: Options to specify revisions on either side, as if
-          merging only part of the history.
     """
+    # TODO: Options to specify revisions on either side, as if
+    #       merging only part of the history.
     takes_args = ['branch', 'other']
     hidden = True
     
+    @display_command
     def run(self, branch, other):
         from bzrlib.revision import common_ancestor, MultipleRevisionSources
         
-        branch1 = Branch.open_containing(branch)
-        branch2 = Branch.open_containing(other)
+        branch1 = Branch.open_containing(branch)[0]
+        branch2 = Branch.open_containing(other)[0]
 
         history_1 = branch1.revision_history()
         history_2 = branch2.revision_history()
@@ -1237,16 +1576,18 @@
     --force is given.
     """
     takes_args = ['branch?']
-    takes_options = ['revision', 'force', 'merge-type']
+    takes_options = ['revision', 'force', 'merge-type', 'reprocess',
+                     Option('show-base', help="Show base revision text in "
+                            "conflicts")]
 
-    def run(self, branch=None, revision=None, force=False, 
-            merge_type=None):
+    def run(self, branch=None, revision=None, force=False, merge_type=None,
+            show_base=False, reprocess=False):
         from bzrlib.merge import merge
         from bzrlib.merge_core import ApplyMerge3
         if merge_type is None:
             merge_type = ApplyMerge3
         if branch is None:
-            branch = Branch.open_containing('.').get_parent()
+            branch = WorkingTree.open_containing(u'.')[0].branch.get_parent()
             if branch is None:
                 raise BzrCommandError("No merge location known or specified.")
             else:
@@ -1257,19 +1598,27 @@
         else:
             if len(revision) == 1:
                 base = [None, None]
-                other = [branch, revision[0].in_history(branch).revno]
+                other_branch = Branch.open_containing(branch)[0]
+                revno = revision[0].in_history(other_branch).revno
+                other = [branch, revno]
             else:
                 assert len(revision) == 2
                 if None in revision:
                     raise BzrCommandError(
                         "Merge doesn't permit that revision specifier.")
-                b = Branch.open(branch)
+                b = Branch.open_containing(branch)[0]
 
                 base = [branch, revision[0].in_history(b).revno]
                 other = [branch, revision[1].in_history(b).revno]
 
         try:
-            merge(other, base, check_clean=(not force), merge_type=merge_type)
+            conflict_count = merge(other, base, check_clean=(not force),
+                                   merge_type=merge_type, reprocess=reprocess,
+                                   show_base=show_base)
+            if conflict_count != 0:
+                return 1
+            else:
+                return 0
         except bzrlib.errors.AmbiguousBase, e:
             m = ("sorry, bzr can't determine the right merge base yet\n"
                  "candidates are:\n  "
@@ -1280,6 +1629,66 @@
             log_error(m)
 
 
+class cmd_remerge(Command):
+    """Redo a merge.
+    """
+    takes_args = ['file*']
+    takes_options = ['merge-type', 'reprocess',
+                     Option('show-base', help="Show base revision text in "
+                            "conflicts")]
+
+    def run(self, file_list=None, merge_type=None, show_base=False,
+            reprocess=False):
+        from bzrlib.merge import merge_inner, transform_tree
+        from bzrlib.merge_core import ApplyMerge3
+        if merge_type is None:
+            merge_type = ApplyMerge3
+        tree, file_list = tree_files(file_list)
+        tree.lock_write()
+        try:
+            pending_merges = tree.pending_merges() 
+            if len(pending_merges) != 1:
+                raise BzrCommandError("Sorry, remerge only works after normal"
+                                      + " merges.  Not cherrypicking or"
+                                      + "multi-merges.")
+            base_revision = common_ancestor(tree.branch.last_revision(), 
+                                            pending_merges[0], tree.branch)
+            base_tree = tree.branch.revision_tree(base_revision)
+            other_tree = tree.branch.revision_tree(pending_merges[0])
+            interesting_ids = None
+            if file_list is not None:
+                interesting_ids = set()
+                for filename in file_list:
+                    file_id = tree.path2id(filename)
+                    interesting_ids.add(file_id)
+                    if tree.kind(file_id) != "directory":
+                        continue
+                    
+                    for name, ie in tree.inventory.iter_entries(file_id):
+                        interesting_ids.add(ie.file_id)
+            transform_tree(tree, tree.branch.basis_tree(), interesting_ids)
+            if file_list is None:
+                restore_files = list(tree.iter_conflicts())
+            else:
+                restore_files = file_list
+            for filename in restore_files:
+                try:
+                    restore(tree.abspath(filename))
+                except NotConflicted:
+                    pass
+            conflicts =  merge_inner(tree.branch, other_tree, base_tree, 
+                                     interesting_ids = interesting_ids, 
+                                     other_rev_id=pending_merges[0], 
+                                     merge_type=merge_type, 
+                                     show_base=show_base,
+                                     reprocess=reprocess)
+        finally:
+            tree.unlock()
+        if conflicts > 0:
+            return 1
+        else:
+            return 0
+
 class cmd_revert(Command):
     """Reverse all changes since the last commit.
 
@@ -1292,26 +1701,25 @@
     aliases = ['merge-revert']
 
     def run(self, revision=None, no_backup=False, file_list=None):
-        from bzrlib.merge import merge
+        from bzrlib.merge import merge_inner
         from bzrlib.commands import parse_spec
-
         if file_list is not None:
             if len(file_list) == 0:
                 raise BzrCommandError("No files specified")
+        else:
+            file_list = []
         if revision is None:
             revno = -1
+            tree = WorkingTree.open_containing(u'.')[0]
+            # FIXME should be tree.last_revision
+            rev_id = tree.branch.last_revision()
         elif len(revision) != 1:
             raise BzrCommandError('bzr revert --revision takes exactly 1 argument')
         else:
-            b = Branch.open_containing('.')
-            revno = revision[0].in_history(b).revno
-        merge(('.', revno), parse_spec('.'),
-              check_clean=False,
-              ignore_zero=True,
-              backup_files=not no_backup,
-              file_list=file_list)
-        if not file_list:
-            Branch.open_containing('.').set_pending_merges([])
+            tree, file_list = tree_files(file_list)
+            rev_id = revision[0].in_history(tree.branch).rev_id
+        tree.revert(file_list, tree.branch.revision_tree(rev_id),
+                                not no_backup)
 
 
 class cmd_assert_fail(Command):
@@ -1329,6 +1737,7 @@
     takes_args = ['topic?']
     aliases = ['?']
     
+    @display_command
     def run(self, topic=None, long=False):
         import help
         if topic is None and long:
@@ -1344,6 +1753,7 @@
     aliases = ['s-c']
     hidden = True
     
+    @display_command
     def run(self, context=None):
         import shellcomplete
         shellcomplete.shellcomplete(context)
@@ -1358,55 +1768,88 @@
     def run(self, from_branch, to_branch):
         from bzrlib.fetch import Fetcher
         from bzrlib.branch import Branch
-        from_b = Branch(from_branch)
-        to_b = Branch(to_branch)
-        Fetcher(to_b, from_b)
-        
+        from_b = Branch.open(from_branch)
+        to_b = Branch.open(to_branch)
+        from_b.lock_read()
+        try:
+            to_b.lock_write()
+            try:
+                Fetcher(to_b, from_b)
+            finally:
+                to_b.unlock()
+        finally:
+            from_b.unlock()
 
 
 class cmd_missing(Command):
-    """What is missing in this branch relative to other branch.
-    """
-    # TODO: rewrite this in terms of ancestry so that it shows only
-    # unmerged things
-    
-    takes_args = ['remote?']
-    aliases = ['mis', 'miss']
-    # We don't have to add quiet to the list, because 
-    # unknown options are parsed as booleans
-    takes_options = ['verbose', 'quiet']
-
-    def run(self, remote=None, verbose=False, quiet=False):
-        from bzrlib.errors import BzrCommandError
-        from bzrlib.missing import show_missing
-
-        if verbose and quiet:
-            raise BzrCommandError('Cannot pass both quiet and verbose')
-
-        b = Branch.open_containing('.')
-        parent = b.get_parent()
-        if remote is None:
-            if parent is None:
+    """Show unmerged/unpulled revisions between two branches.
+
+    OTHER_BRANCH may be local or remote."""
+    takes_args = ['other_branch?']
+    takes_options = [Option('reverse', 'Reverse the order of revisions'),
+                     Option('mine-only', 
+                            'Display changes in the local branch only'),
+                     Option('theirs-only', 
+                            'Display changes in the remote branch only'), 
+                     'line',
+                     'long', 
+                     'short',
+                     'show-ids',
+                     'verbose'
+                     ]
+
+    def run(self, other_branch=None, reverse=False, mine_only=False,
+            theirs_only=False, long=True, short=False, line=False, 
+            show_ids=False, verbose=False):
+        from bzrlib.missing import find_unmerged, iter_log_data
+        from bzrlib.log import log_formatter
+        local_branch = bzrlib.branch.Branch.open_containing(u".")[0]
+        parent = local_branch.get_parent()
+        if other_branch is None:
+            other_branch = parent
+            if other_branch is None:
                 raise BzrCommandError("No missing location known or specified.")
-            else:
-                if not quiet:
-                    print "Using last location: %s" % parent
-                remote = parent
-        elif parent is None:
-            # We only update parent if it did not exist, missing
-            # should not change the parent
-            b.set_parent(remote)
-        br_remote = Branch.open_containing(remote)
-        return show_missing(b, br_remote, verbose=verbose, quiet=quiet)
+            print "Using last location: " + local_branch.get_parent()
+        remote_branch = bzrlib.branch.Branch.open(other_branch)
+        local_extra, remote_extra = find_unmerged(local_branch, remote_branch)
+        log_format = get_log_format(long=long, short=short, line=line)
+        lf = log_formatter(log_format, sys.stdout,
+                           show_ids=show_ids,
+                           show_timezone='original')
+        if reverse is False:
+            local_extra.reverse()
+            remote_extra.reverse()
+        if local_extra and not theirs_only:
+            print "You have %d extra revision(s):" % len(local_extra)
+            for data in iter_log_data(local_extra, local_branch, verbose):
+                lf.show(*data)
+            printed_local = True
+        else:
+            printed_local = False
+        if remote_extra and not mine_only:
+            if printed_local is True:
+                print "\n\n"
+            print "You are missing %d revision(s):" % len(remote_extra)
+            for data in iter_log_data(remote_extra, remote_branch, verbose):
+                lf.show(*data)
+        if not remote_extra and not local_extra:
+            status_code = 0
+            print "Branches are up to date."
+        else:
+            status_code = 1
+        if parent is None and other_branch is not None:
+            local_branch.set_parent(other_branch)
+        return status_code
 
 
 class cmd_plugins(Command):
     """List plugins"""
     hidden = True
+    @display_command
     def run(self):
         import bzrlib.plugin
         from inspect import getdoc
-        for plugin in bzrlib.plugin.all_plugins:
+        for name, plugin in bzrlib.plugin.all_plugins().items():
             if hasattr(plugin, '__path__'):
                 print plugin.__path__[0]
             elif hasattr(plugin, '__file__'):
@@ -1419,3 +1862,166 @@
                 print '\t', d.split('\n')[0]
 
 
+class cmd_testament(Command):
+    """Show testament (signing-form) of a revision."""
+    takes_options = ['revision', 'long']
+    takes_args = ['branch?']
+    @display_command
+    def run(self, branch=u'.', revision=None, long=False):
+        from bzrlib.testament import Testament
+        b = WorkingTree.open_containing(branch)[0].branch
+        b.lock_read()
+        try:
+            if revision is None:
+                rev_id = b.last_revision()
+            else:
+                rev_id = revision[0].in_history(b).rev_id
+            t = Testament.from_revision(b, rev_id)
+            if long:
+                sys.stdout.writelines(t.as_text_lines())
+            else:
+                sys.stdout.write(t.as_short_text())
+        finally:
+            b.unlock()
+
+
+class cmd_annotate(Command):
+    """Show the origin of each line in a file.
+
+    This prints out the given file with an annotation on the left side
+    indicating which revision, author and date introduced the change.
+
+    If the origin is the same for a run of consecutive lines, it is 
+    shown only at the top, unless the --all option is given.
+    """
+    # TODO: annotate directories; showing when each file was last changed
+    # TODO: annotate a previous version of a file
+    # TODO: if the working copy is modified, show annotations on that 
+    #       with new uncommitted lines marked
+    aliases = ['blame', 'praise']
+    takes_args = ['filename']
+    takes_options = [Option('all', help='show annotations on all lines'),
+                     Option('long', help='show date in annotations'),
+                     ]
+
+    @display_command
+    def run(self, filename, all=False, long=False):
+        from bzrlib.annotate import annotate_file
+        tree, relpath = WorkingTree.open_containing(filename)
+        branch = tree.branch
+        branch.lock_read()
+        try:
+            file_id = tree.inventory.path2id(relpath)
+            tree = branch.revision_tree(branch.last_revision())
+            file_version = tree.inventory[file_id].revision
+            annotate_file(branch, file_version, file_id, long, all, sys.stdout)
+        finally:
+            branch.unlock()
+
+
+class cmd_re_sign(Command):
+    """Create a digital signature for an existing revision."""
+    # TODO be able to replace existing ones.
+
+    hidden = True # is this right ?
+    takes_args = ['revision_id?']
+    takes_options = ['revision']
+    
+    def run(self, revision_id=None, revision=None):
+        import bzrlib.config as config
+        import bzrlib.gpg as gpg
+        if revision_id is not None and revision is not None:
+            raise BzrCommandError('You can only supply one of revision_id or --revision')
+        if revision_id is None and revision is None:
+            raise BzrCommandError('You must supply either --revision or a revision_id')
+        b = WorkingTree.open_containing(u'.')[0].branch
+        gpg_strategy = gpg.GPGStrategy(config.BranchConfig(b))
+        if revision_id is not None:
+            b.sign_revision(revision_id, gpg_strategy)
+        elif revision is not None:
+            if len(revision) == 1:
+                revno, rev_id = revision[0].in_history(b)
+                b.sign_revision(rev_id, gpg_strategy)
+            elif len(revision) == 2:
+                # are they both on rh- if so we can walk between them
+                # might be nice to have a range helper for arbitrary
+                # revision paths. hmm.
+                from_revno, from_revid = revision[0].in_history(b)
+                to_revno, to_revid = revision[1].in_history(b)
+                if to_revid is None:
+                    to_revno = b.revno()
+                if from_revno is None or to_revno is None:
+                    raise BzrCommandError('Cannot sign a range of non-revision-history revisions')
+                for revno in range(from_revno, to_revno + 1):
+                    b.sign_revision(b.get_rev_id(revno), gpg_strategy)
+            else:
+                raise BzrCommandError('Please supply either one revision, or a range.')
+
+
+class cmd_uncommit(bzrlib.commands.Command):
+    """Remove the last committed revision.
+
+    By supplying the --all flag, it will not only remove the entry 
+    from revision_history, but also remove all of the entries in the
+    stores.
+
+    --verbose will print out what is being removed.
+    --dry-run will go through all the motions, but not actually
+    remove anything.
+    
+    In the future, uncommit will create a changeset, which can then
+    be re-applied.
+    """
+    takes_options = ['all', 'verbose', 'revision',
+                    Option('dry-run', help='Don\'t actually make changes'),
+                    Option('force', help='Say yes to all questions.')]
+    takes_args = ['location?']
+    aliases = []
+
+    def run(self, location=None, all=False,
+            dry_run=False, verbose=False,
+            revision=None, force=False):
+        from bzrlib.branch import Branch
+        from bzrlib.log import log_formatter
+        import sys
+        from bzrlib.uncommit import uncommit
+
+        if location is None:
+            location = u'.'
+        b, relpath = Branch.open_containing(location)
+
+        if revision is None:
+            revno = b.revno()
+            rev_id = b.last_revision()
+        else:
+            revno, rev_id = revision[0].in_history(b)
+        if rev_id is None:
+            print 'No revisions to uncommit.'
+
+        for r in range(revno, b.revno()+1):
+            rev_id = b.get_rev_id(r)
+            lf = log_formatter('short', to_file=sys.stdout,show_timezone='original')
+            lf.show(r, b.get_revision(rev_id), None)
+
+        if dry_run:
+            print 'Dry-run, pretending to remove the above revisions.'
+            if not force:
+                val = raw_input('Press <enter> to continue')
+        else:
+            print 'The above revision(s) will be removed.'
+            if not force:
+                val = raw_input('Are you sure [y/N]? ')
+                if val.lower() not in ('y', 'yes'):
+                    print 'Canceled'
+                    return 0
+
+        uncommit(b, remove_files=all,
+                dry_run=dry_run, verbose=verbose,
+                revno=revno)
+
+
+# these get imported and then picked up by the scan for cmd_*
+# TODO: Some more consistent way to split command definitions across files;
+# we do need to load at least some information about them to know of 
+# aliases.
+from bzrlib.conflicts import cmd_resolve, cmd_conflicts, restore

=== modified file 'bzrlib/changeset.py'
--- bzrlib/changeset.py	2005-10-03 19:50:36 +0000
+++ bzrlib/changeset.py	2006-01-09 21:23:09 +0000
@@ -13,31 +13,38 @@
 #    You should have received a copy of the GNU General Public License
 #    along with this program; if not, write to the Free Software
 #    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Represent and apply a changeset.
+
+Conflicts in applying a changeset are represented as exceptions.
+
+This only handles the in-memory objects representing changesets, which are
+primarily used by the merge code. 
+"""
+
 import os.path
 import errno
-import patch
 import stat
-from bzrlib.trace import mutter
-from bzrlib.osutils import rename
+from shutil import rmtree
+from itertools import izip
+
+from bzrlib.trace import mutter, warning
+from bzrlib.osutils import rename, sha_file, pathjoin, mkdtemp
 import bzrlib
-
-# XXX: mbp: I'm not totally convinced that we should handle conflicts
-# as part of changeset application, rather than only in the merge
-# operation.
-
-"""Represent and apply a changeset
-
-Conflicts in applying a changeset are represented as exceptions.
-"""
+from bzrlib.errors import BzrCheckError
 
 __docformat__ = "restructuredtext"
 
+
 NULL_ID = "!NULL"
 
+
 class OldFailedTreeOp(Exception):
     def __init__(self):
         Exception.__init__(self, "bzr-tree-change contains files from a"
                            " previous failed merge operation.")
+
+
 def invert_dict(dict):
     newdict = {}
     for (key,value) in dict.iteritems():
@@ -45,56 +52,59 @@
     return newdict
 
        
-class ChangeUnixPermissions(object):
+class ChangeExecFlag(object):
     """This is two-way change, suitable for file modification, creation,
     deletion"""
-    def __init__(self, old_mode, new_mode):
-        self.old_mode = old_mode
-        self.new_mode = new_mode
+    def __init__(self, old_exec_flag, new_exec_flag):
+        self.old_exec_flag = old_exec_flag
+        self.new_exec_flag = new_exec_flag
 
-    def apply(self, filename, conflict_handler, reverse=False):
-        if not reverse:
-            from_mode = self.old_mode
-            to_mode = self.new_mode
-        else:
-            from_mode = self.new_mode
-            to_mode = self.old_mode
+    def apply(self, filename, conflict_handler):
+        from_exec_flag = self.old_exec_flag
+        to_exec_flag = self.new_exec_flag
         try:
-            current_mode = os.stat(filename).st_mode &0777
+            current_exec_flag = bool(os.stat(filename).st_mode & 0111)
         except OSError, e:
             if e.errno == errno.ENOENT:
-                if conflict_handler.missing_for_chmod(filename) == "skip":
+                if conflict_handler.missing_for_exec_flag(filename) == "skip":
                     return
                 else:
-                    current_mode = from_mode
+                    current_exec_flag = from_exec_flag
 
-        if from_mode is not None and current_mode != from_mode:
-            if conflict_handler.wrong_old_perms(filename, from_mode, 
-                                                current_mode) != "continue":
+        if from_exec_flag is not None and current_exec_flag != from_exec_flag:
+            if conflict_handler.wrong_old_exec_flag(filename,
+                        from_exec_flag, current_exec_flag) != "continue":
                 return
 
-        if to_mode is not None:
+        if to_exec_flag is not None:
+            current_mode = os.stat(filename).st_mode
+            if to_exec_flag:
+                umask = os.umask(0)
+                os.umask(umask)
+                to_mode = current_mode | (0100 & ~umask)
+                # Enable x-bit for others only if they can read it.
+                if current_mode & 0004:
+                    to_mode |= 0001 & ~umask
+                if current_mode & 0040:
+                    to_mode |= 0010 & ~umask
+            else:
+                to_mode = current_mode & ~0111
             try:
                 os.chmod(filename, to_mode)
             except IOError, e:
                 if e.errno == errno.ENOENT:
-                    conflict_handler.missing_for_chmod(filename)
+                    conflict_handler.missing_for_exec_flag(filename)
 
     def __eq__(self, other):
-        if not isinstance(other, ChangeUnixPermissions):
-            return False
-        elif self.old_mode != other.old_mode:
-            return False
-        elif self.new_mode != other.new_mode:
-            return False
-        else:
-            return True
+        return (isinstance(other, ChangeExecFlag) and
+                self.old_exec_flag == other.old_exec_flag and
+                self.new_exec_flag == other.new_exec_flag)
 
     def __ne__(self, other):
         return not (self == other)
 
 
-def dir_create(filename, conflict_handler, reverse):
+def dir_create(filename, conflict_handler, reverse=False):
     """Creates the directory, or deletes it if reverse is true.  Intended to be
     used with ReplaceContents.
 
@@ -136,7 +146,10 @@
         """
         self.target = contents
 
-    def __call__(self, filename, conflict_handler, reverse):
+    def __repr__(self):
+        return "SymlinkCreate(%s)" % self.target
+
+    def __call__(self, filename, conflict_handler, reverse=False):
         """Creates or destroys the symlink.
 
         :param filename: The name of the symlink to create
@@ -165,6 +178,7 @@
     def __ne__(self, other):
         return not (self == other)
 
+
 class FileCreate(object):
     """Create or delete a file (for use with ReplaceContents)"""
     def __init__(self, contents):
@@ -189,7 +203,7 @@
     def __ne__(self, other):
         return not (self == other)
 
-    def __call__(self, filename, conflict_handler, reverse):
+    def __call__(self, filename, conflict_handler, reverse=False):
         """Create or delete a file
 
         :param filename: The name of the file to create
@@ -221,12 +235,71 @@
                 if conflict_handler.missing_for_rm(filename, undo) == "skip":
                     return
 
-                    
-
-def reversed(sequence):
-    max = len(sequence) - 1
-    for i in range(len(sequence)):
-        yield sequence[max - i]
+
+class TreeFileCreate(object):
+    """Create or delete a file (for use with ReplaceContents)"""
+    def __init__(self, tree, file_id):
+        """Constructor
+
+        :param contents: The contents of the file to write
+        :type contents: str
+        """
+        self.tree = tree
+        self.file_id = file_id
+
+    def __repr__(self):
+        return "TreeFileCreate(%s)" % self.file_id
+
+    def __eq__(self, other):
+        if not isinstance(other, TreeFileCreate):
+            return False
+        return self.tree.get_file_sha1(self.file_id) == \
+            other.tree.get_file_sha1(other.file_id)
+
+    def __ne__(self, other):
+        return not (self == other)
+
+    def write_file(self, filename):
+        outfile = file(filename, "wb")
+        for line in self.tree.get_file(self.file_id):
+            outfile.write(line)
+
+    def same_text(self, filename):
+        in_file = file(filename, "rb")
+        return sha_file(in_file) == self.tree.get_file_sha1(self.file_id)
+
+    def __call__(self, filename, conflict_handler, reverse=False):
+        """Create or delete a file
+
+        :param filename: The name of the file to create
+        :type filename: str
+        :param reverse: Delete the file instead of creating it
+        :type reverse: bool
+        """
+        if not reverse:
+            try:
+                self.write_file(filename)
+            except IOError, e:
+                if e.errno == errno.ENOENT:
+                    if conflict_handler.missing_parent(filename)=="continue":
+                        self.write_file(filename)
+                else:
+                    raise
+
+        else:
+            try:
+                if not self.same_text(filename):
+                    direction = conflict_handler.wrong_old_contents(filename,
+                        self.tree.get_file(self.file_id).read())
+                    if  direction != "continue":
+                        return
+                os.unlink(filename)
+            except IOError, e:
+                if e.errno != errno.ENOENT:
+                    raise
+                if conflict_handler.missing_for_rm(filename, undo) == "skip":
+                    return
+
 
 class ReplaceContents(object):
     """A contents-replacement framework.  It allows a file/directory/symlink to
@@ -264,20 +337,14 @@
     def __ne__(self, other):
         return not (self == other)
 
-    def apply(self, filename, conflict_handler, reverse=False):
+    def apply(self, filename, conflict_handler):
         """Applies the FileReplacement to the specified filename
 
         :param filename: The name of the file to apply changes to
         :type filename: str
-        :param reverse: If true, apply the change in reverse
-        :type reverse: bool
         """
-        if not reverse:
-            undo = self.old_contents
-            perform = self.new_contents
-        else:
-            undo = self.new_contents
-            perform = self.old_contents
+        undo = self.old_contents
+        perform = self.new_contents
         mode = None
         if undo is not None:
             try:
@@ -291,46 +358,30 @@
                     return
             undo(filename, conflict_handler, reverse=True)
         if perform is not None:
-            perform(filename, conflict_handler, reverse=False)
+            perform(filename, conflict_handler)
             if mode is not None:
                 os.chmod(filename, mode)
 
-class ApplySequence(object):
-    def __init__(self, changes=None):
-        self.changes = []
-        if changes is not None:
-            self.changes.extend(changes)
-
-    def __eq__(self, other):
-        if not isinstance(other, ApplySequence):
-            return False
-        elif len(other.changes) != len(self.changes):
-            return False
-        else:
-            for i in range(len(self.changes)):
-                if self.changes[i] != other.changes[i]:
-                    return False
-            return True
-
-    def __ne__(self, other):
-        return not (self == other)
-
-    
-    def apply(self, filename, conflict_handler, reverse=False):
-        if not reverse:
-            iter = self.changes
-        else:
-            iter = reversed(self.changes)
-        for change in iter:
-            change.apply(filename, conflict_handler, reverse)
+    def is_creation(self):
+        return self.new_contents is not None and self.old_contents is None
+
+    def is_deletion(self):
+        return self.old_contents is not None and self.new_contents is None
 
 
 class Diff3Merge(object):
+    history_based = False
     def __init__(self, file_id, base, other):
         self.file_id = file_id
         self.base = base
         self.other = other
 
+    def is_creation(self):
+        return False
+
+    def is_deletion(self):
+        return False
+
     def __eq__(self, other):
         if not isinstance(other, Diff3Merge):
             return False
@@ -340,32 +391,41 @@
     def __ne__(self, other):
         return not (self == other)
 
-    def apply(self, filename, conflict_handler, reverse=False):
-        new_file = filename+".new"
-        base_file = self.base.readonly_path(self.file_id)
-        other_file = self.other.readonly_path(self.file_id)
-        if not reverse:
+    def dump_file(self, temp_dir, name, tree):
+        out_path = pathjoin(temp_dir, name)
+        out_file = file(out_path, "wb")
+        in_file = tree.get_file(self.file_id)
+        for line in in_file:
+            out_file.write(line)
+        return out_path
+
+    def apply(self, filename, conflict_handler):
+        import bzrlib.patch
+        temp_dir = mkdtemp(prefix="bzr-")
+        try:
+            new_file = filename+".new"
+            base_file = self.dump_file(temp_dir, "base", self.base)
+            other_file = self.dump_file(temp_dir, "other", self.other)
             base = base_file
             other = other_file
-        else:
-            base = other_file
-            other = base_file
-        status = patch.diff3(new_file, filename, base, other)
-        if status == 0:
-            os.chmod(new_file, os.stat(filename).st_mode)
-            rename(new_file, filename)
-            return
-        else:
-            assert(status == 1)
-            def get_lines(filename):
-                my_file = file(filename, "rb")
-                lines = my_file.readlines()
-                my_file.close()
-                return lines
-            base_lines = get_lines(base)
-            other_lines = get_lines(other)
-            conflict_handler.merge_conflict(new_file, filename, base_lines, 
-                                            other_lines)
+            status = bzrlib.patch.diff3(new_file, filename, base, other)
+            if status == 0:
+                os.chmod(new_file, os.stat(filename).st_mode)
+                rename(new_file, filename)
+                return
+            else:
+                assert(status == 1)
+                def get_lines(filename):
+                    my_file = file(filename, "rb")
+                    lines = my_file.readlines()
+                    my_file.close()
+                    return lines
+                base_lines = get_lines(base)
+                other_lines = get_lines(other)
+                conflict_handler.merge_conflict(new_file, filename, base_lines, 
+                                                other_lines)
+        finally:
+            rmtree(temp_dir)
 
 
 def CreateDir():
@@ -376,6 +436,7 @@
     """
     return ReplaceContents(None, dir_create)
 
+
 def DeleteDir():
     """Convenience function to delete a directory.
 
@@ -384,6 +445,7 @@
     """
     return ReplaceContents(dir_create, None)
 
+
 def CreateFile(contents):
     """Convenience fucntion to create a file.
     
@@ -394,6 +456,7 @@
     """
     return ReplaceContents(None, FileCreate(contents))
 
+
 def DeleteFile(contents):
     """Convenience fucntion to delete a file.
     
@@ -404,7 +467,8 @@
     """
     return ReplaceContents(FileCreate(contents), None)
 
-def ReplaceFileContents(old_contents, new_contents):
+
+def ReplaceFileContents(old_tree, new_tree, file_id):
     """Convenience fucntion to replace the contents of a file.
     
     :param old_contents: The contents of the file to replace 
@@ -414,7 +478,9 @@
     :return: A ReplaceContents that will replace the contents of a file a file 
     :rtype: `ReplaceContents`
     """
-    return ReplaceContents(FileCreate(old_contents), FileCreate(new_contents))
+    return ReplaceContents(TreeFileCreate(old_tree, file_id), 
+                           TreeFileCreate(new_tree, file_id))
+
 
 def CreateSymlink(target):
     """Convenience fucntion to create a symlink.
@@ -426,6 +492,7 @@
     """
     return ReplaceContents(None, SymlinkCreate(target))
 
+
 def DeleteSymlink(target):
     """Convenience fucntion to delete a symlink.
     
@@ -436,6 +503,7 @@
     """
     return ReplaceContents(SymlinkCreate(target), None)
 
+
 def ChangeTarget(old_target, new_target):
     """Convenience fucntion to change the target of a symlink.
     
@@ -479,6 +547,7 @@
         msg = 'Child of !NULL is named "%s", not "./.".' % name
         InvalidEntry.__init__(self, entry, msg)
 
+
 class NullIDAssigned(InvalidEntry):
     """The id !NULL was assigned to a real entry"""
     def __init__(self, entry):
@@ -490,6 +559,7 @@
         msg = '"!NULL" id assigned to a file "%s".' % entry.path
         InvalidEntry.__init__(self, entry, msg)
 
+
 class ParentIDIsSelf(InvalidEntry):
     """An entry is marked as its own parent"""
     def __init__(self, entry):
@@ -502,6 +572,7 @@
             (entry.path, entry.id)
         InvalidEntry.__init__(self, entry, msg)
 
+
 class ChangesetEntry(object):
     """An entry the changeset"""
     def __init__(self, id, parent, path):
@@ -526,16 +597,18 @@
         if self.id  == self.parent:
             raise ParentIDIsSelf(self)
 
-    def __str__(self):
+    def __repr__(self):
         return "ChangesetEntry(%s)" % self.id
 
+    __str__ = __repr__
+
     def __get_dir(self):
         if self.path is None:
             return None
         return os.path.dirname(self.path)
 
     def __set_dir(self, dir):
-        self.path = os.path.join(dir, os.path.basename(self.path))
+        self.path = pathjoin(dir, os.path.basename(self.path))
 
     dir = property(__get_dir, __set_dir)
     
@@ -545,7 +618,7 @@
         return os.path.basename(self.path)
 
     def __set_name(self, name):
-        self.path = os.path.join(os.path.dirname(self.path), name)
+        self.path = pathjoin(os.path.dirname(self.path), name)
 
     name = property(__get_name, __set_name)
 
@@ -555,7 +628,7 @@
         return os.path.dirname(self.new_path)
 
     def __set_new_dir(self, dir):
-        self.new_path = os.path.join(dir, os.path.basename(self.new_path))
+        self.new_path = pathjoin(dir, os.path.basename(self.new_path))
 
     new_dir = property(__get_new_dir, __set_new_dir)
 
@@ -565,7 +638,7 @@
         return os.path.basename(self.new_path)
 
     def __set_new_name(self, name):
-        self.new_path = os.path.join(os.path.dirname(self.new_path), name)
+        self.new_path = pathjoin(os.path.dirname(self.new_path), name)
 
     new_name = property(__get_new_name, __set_new_name)
 
@@ -577,23 +650,26 @@
 
         return (self.parent != self.new_parent or self.name != self.new_name)
 
-    def is_deletion(self, reverse):
+    def is_deletion(self, reverse=False):
         """Return true if applying the entry would delete a file/directory.
 
         :param reverse: if true, the changeset is being applied in reverse
         :rtype: bool
         """
-        return ((self.new_parent is None and not reverse) or 
-                (self.parent is None and reverse))
+        return self.is_creation(not reverse)
 
-    def is_creation(self, reverse):
+    def is_creation(self, reverse=False):
         """Return true if applying the entry would create a file/directory.
 
         :param reverse: if true, the changeset is being applied in reverse
         :rtype: bool
         """
-        return ((self.parent is None and not reverse) or 
-                (self.new_parent is None and reverse))
+        if self.contents_change is None:
+            return False
+        if reverse:
+            return self.contents_change.is_deletion()
+        else:
+            return self.contents_change.is_creation()
 
     def is_creation_or_deletion(self):
         """Return true if applying the entry would create or delete a 
@@ -601,7 +677,7 @@
 
         :rtype: bool
         """
-        return self.parent is None or self.new_parent is None
+        return self.is_creation() or self.is_deletion()
 
     def get_cset_path(self, mod=False):
         """Determine the path of the entry according to the changeset.
@@ -627,77 +703,60 @@
                 return None
             return self.path
 
-    def summarize_name(self, reverse=False):
+    def summarize_name(self):
         """Produce a one-line summary of the filename.  Indicates renames as
         old => new, indicates creation as None => new, indicates deletion as
         old => None.
 
-        :param changeset: The changeset to get paths from
-        :type changeset: `Changeset`
-        :param reverse: If true, reverse the names in the output
-        :type reverse: bool
         :rtype: str
         """
         orig_path = self.get_cset_path(False)
         mod_path = self.get_cset_path(True)
-        if orig_path is not None:
+        if orig_path and orig_path.startswith('./'):
             orig_path = orig_path[2:]
-        if mod_path is not None:
+        if mod_path and mod_path.startswith('./'):
             mod_path = mod_path[2:]
         if orig_path == mod_path:
             return orig_path
         else:
-            if not reverse:
-                return "%s => %s" % (orig_path, mod_path)
-            else:
-                return "%s => %s" % (mod_path, orig_path)
-
-
-    def get_new_path(self, id_map, changeset, reverse=False):
+            return "%s => %s" % (orig_path, mod_path)
+
+    def get_new_path(self, id_map, changeset):
         """Determine the full pathname to rename to
 
         :param id_map: The map of ids to filenames for the tree
         :type id_map: Dictionary
         :param changeset: The changeset to get data from
         :type changeset: `Changeset`
-        :param reverse: If true, we're applying the changeset in reverse
-        :type reverse: bool
         :rtype: str
         """
-        mutter("Finding new path for %s" % self.summarize_name())
-        if reverse:
-            parent = self.parent
-            to_dir = self.dir
-            from_dir = self.new_dir
-            to_name = self.name
-            from_name = self.new_name
-        else:
-            parent = self.new_parent
-            to_dir = self.new_dir
-            from_dir = self.dir
-            to_name = self.new_name
-            from_name = self.name
+        mutter("Finding new path for %s", self.summarize_name())
+        parent = self.new_parent
+        to_dir = self.new_dir
+        from_dir = self.dir
+        to_name = self.new_name
+        from_name = self.name
 
         if to_name is None:
             return None
 
         if parent == NULL_ID or parent is None:
-            if to_name != '.':
+            if to_name != u'.':
                 raise SourceRootHasName(self, to_name)
             else:
-                return '.'
-        if from_dir == to_dir:
+                return u'.'
+        parent_entry = changeset.entries.get(parent)
+        if parent_entry is None:
             dir = os.path.dirname(id_map[self.id])
         else:
-            mutter("path, new_path: %r %r" % (self.path, self.new_path))
-            parent_entry = changeset.entries[parent]
-            dir = parent_entry.get_new_path(id_map, changeset, reverse)
+            mutter("path, new_path: %r %r", self.path, self.new_path)
+            dir = parent_entry.get_new_path(id_map, changeset)
         if from_name == to_name:
             name = os.path.basename(id_map[self.id])
         else:
             name = to_name
             assert(from_name is None or from_name == os.path.basename(id_map[self.id]))
-        return os.path.join(dir, name)
+        return pathjoin(dir, name)
 
     def is_boring(self):
         """Determines whether the entry does nothing
@@ -716,20 +775,19 @@
         else:
             return True
 
-    def apply(self, filename, conflict_handler, reverse=False):
+    def apply(self, filename, conflict_handler):
         """Applies the file content and/or metadata changes.
 
         :param filename: the filename of the entry
         :type filename: str
-        :param reverse: If true, apply the changes in reverse
-        :type reverse: bool
         """
-        if self.is_deletion(reverse) and self.metadata_change is not None:
-            self.metadata_change.apply(filename, conflict_handler, reverse)
+        if self.is_deletion() and self.metadata_change is not None:
+            self.metadata_change.apply(filename, conflict_handler)
         if self.contents_change is not None:
-            self.contents_change.apply(filename, conflict_handler, reverse)
-        if not self.is_deletion(reverse) and self.metadata_change is not None:
-            self.metadata_change.apply(filename, conflict_handler, reverse)
+            self.contents_change.apply(filename, conflict_handler)
+        if not self.is_deletion() and self.metadata_change is not None:
+            self.metadata_change.apply(filename, conflict_handler)
+
 
 class IDPresent(Exception):
     def __init__(self, id):
@@ -738,6 +796,7 @@
         Exception.__init__(self, msg)
         self.id = id
 
+
 class Changeset(object):
     """A set of changes to apply"""
     def __init__(self):
@@ -749,23 +808,8 @@
             raise IDPresent(entry.id)
         self.entries[entry.id] = entry
 
-def my_sort(sequence, key, reverse=False):
-    """A sort function that supports supplying a key for comparison
-    
-    :param sequence: The sequence to sort
-    :param key: A callable object that returns the values to be compared
-    :param reverse: If true, sort in reverse order
-    :type reverse: bool
-    """
-    def cmp_by_key(entry_a, entry_b):
-        if reverse:
-            tmp=entry_a
-            entry_a = entry_b
-            entry_b = tmp
-        return cmp(key(entry_a), key(entry_b))
-    sequence.sort(cmp_by_key)
 
-def get_rename_entries(changeset, inventory, reverse):
+def get_rename_entries(changeset, inventory):
     """Return a list of entries that will be renamed.  Entries are sorted from
     longest to shortest source path and from shortest to longest target path.
 
@@ -773,13 +817,11 @@
     :type changeset: `Changeset`
     :param inventory: The source of current tree paths for the given ids
     :type inventory: Dictionary
-    :param reverse: If true, the changeset is being applied in reverse
-    :type reverse: bool
     :return: source entries and target entries as a tuple
     :rtype: (List, List)
     """
     source_entries = [x for x in changeset.entries.itervalues() 
-                      if x.needs_rename()]
+                      if x.needs_rename() or x.is_creation_or_deletion()]
     # these are done from longest path to shortest, to avoid deleting a
     # parent before its children are deleted/renamed 
     def longest_to_shortest(entry):
@@ -788,22 +830,23 @@
             return 0
         else:
             return len(path)
-    my_sort(source_entries, longest_to_shortest, reverse=True)
+    source_entries.sort(None, longest_to_shortest, True)
 
     target_entries = source_entries[:]
     # These are done from shortest to longest path, to avoid creating a
     # child before its parent has been created/renamed
     def shortest_to_longest(entry):
-        path = entry.get_new_path(inventory, changeset, reverse)
+        path = entry.get_new_path(inventory, changeset)
         if path is None:
             return 0
         else:
             return len(path)
-    my_sort(target_entries, shortest_to_longest)
+    target_entries.sort(None, shortest_to_longest)
     return (source_entries, target_entries)
 
+
 def rename_to_temp_delete(source_entries, inventory, dir, temp_dir, 
-                          conflict_handler, reverse):
+                          conflict_handler):
     """Delete and rename entries as appropriate.  Entries are renamed to temp
     names.  A map of id -> temp name (or None, for deletions) is returned.
 
@@ -813,38 +856,39 @@
     :type inventory: Dictionary
     :param dir: The directory to apply changes to
     :type dir: str
-    :param reverse: Apply changes in reverse
-    :type reverse: bool
     :return: a mapping of id to temporary name
     :rtype: Dictionary
     """
     temp_name = {}
     for i in range(len(source_entries)):
         entry = source_entries[i]
-        if entry.is_deletion(reverse):
-            path = os.path.join(dir, inventory[entry.id])
-            entry.apply(path, conflict_handler, reverse)
+        if entry.is_deletion():
+            path = pathjoin(dir, inventory[entry.id])
+            entry.apply(path, conflict_handler)
             temp_name[entry.id] = None
 
-        else:
-            to_name = os.path.join(temp_dir, str(i))
+        elif entry.needs_rename():
+            if entry.is_creation():
+                continue
+            to_name = pathjoin(temp_dir, str(i))
             src_path = inventory.get(entry.id)
             if src_path is not None:
-                src_path = os.path.join(dir, src_path)
+                src_path = pathjoin(dir, src_path)
                 try:
                     rename(src_path, to_name)
                     temp_name[entry.id] = to_name
                 except OSError, e:
                     if e.errno != errno.ENOENT:
                         raise
-                    if conflict_handler.missing_for_rename(src_path) == "skip":
+                    if conflict_handler.missing_for_rename(src_path, to_name) \
+                        == "skip":
                         continue
 
     return temp_name
 
 
 def rename_to_new_create(changed_inventory, target_entries, inventory, 
-                         changeset, dir, conflict_handler, reverse):
+                         changeset, dir, conflict_handler):
     """Rename entries with temp names to their final names, create new files.
 
     :param changed_inventory: A mapping of id to temporary name
@@ -855,30 +899,33 @@
     :type changeset: `Changeset`
     :param dir: The directory to apply changes to
     :type dir: str
-    :param reverse: If true, apply changes in reverse
-    :type reverse: bool
     """
     for entry in target_entries:
-        new_tree_path = entry.get_new_path(inventory, changeset, reverse)
+        new_tree_path = entry.get_new_path(inventory, changeset)
         if new_tree_path is None:
             continue
-        new_path = os.path.join(dir, new_tree_path)
+        new_path = pathjoin(dir, new_tree_path)
         old_path = changed_inventory.get(entry.id)
         if bzrlib.osutils.lexists(new_path):
             if conflict_handler.target_exists(entry, new_path, old_path) == \
                 "skip":
                 continue
-        if entry.is_creation(reverse):
-            entry.apply(new_path, conflict_handler, reverse)
+        if entry.is_creation():
+            entry.apply(new_path, conflict_handler)
             changed_inventory[entry.id] = new_tree_path
-        else:
+        elif entry.needs_rename():
+            if entry.is_deletion():
+                continue
             if old_path is None:
                 continue
             try:
+                mutter('rename %s to final name %s', old_path, new_path)
                 rename(old_path, new_path)
                 changed_inventory[entry.id] = new_tree_path
             except OSError, e:
-                raise Exception ("%s is missing" % new_path)
+                raise BzrCheckError('failed to rename %s to %s for changeset entry %s: %s'
+                        % (old_path, new_path, entry, e))
+
 
 class TargetExists(Exception):
     def __init__(self, entry, target):
@@ -887,6 +934,7 @@
         self.entry = entry
         self.target = target
 
+
 class RenameConflict(Exception):
     def __init__(self, id, this_name, base_name, other_name):
         msg = """Trees all have different names for a file
@@ -899,6 +947,7 @@
         self.base_name = base_name
         self_other_name = other_name
 
+
 class MoveConflict(Exception):
     def __init__(self, id, this_parent, base_parent, other_parent):
         msg = """The file is in different directories in every tree
@@ -911,25 +960,12 @@
         self.base_parent = base_parent
         self_other_parent = other_parent
 
+
 class MergeConflict(Exception):
     def __init__(self, this_path):
         Exception.__init__(self, "Conflict applying changes to %s" % this_path)
         self.this_path = this_path
 
-class MergePermissionConflict(Exception):
-    def __init__(self, this_path, base_path, other_path):
-        this_perms = os.stat(this_path).st_mode & 0755
-        base_perms = os.stat(base_path).st_mode & 0755
-        other_perms = os.stat(other_path).st_mode & 0755
-        msg = """Conflicting permission for %s
-this: %o
-base: %o
-other: %o
-        """ % (this_path, this_perms, base_perms, other_perms)
-        self.this_path = this_path
-        self.base_path = base_path
-        self.other_path = other_path
-        Exception.__init__(self, msg)
 
 class WrongOldContents(Exception):
     def __init__(self, filename):
@@ -937,13 +973,15 @@
         self.filename = filename
         Exception.__init__(self, msg)
 
-class WrongOldPermissions(Exception):
-    def __init__(self, filename, old_perms, new_perms):
-        msg = "Permission missmatch on %s:\n" \
-        "Expected 0%o, got 0%o." % (filename, old_perms, new_perms)
+
+class WrongOldExecFlag(Exception):
+    def __init__(self, filename, old_exec_flag, new_exec_flag):
+        msg = "Executable flag missmatch on %s:\n" \
+        "Expected %s, got %s." % (filename, old_exec_flag, new_exec_flag)
         self.filename = filename
         Exception.__init__(self, msg)
 
+
 class RemoveContentsConflict(Exception):
     def __init__(self, filename):
         msg = "Conflict deleting %s, which has different contents in BASE"\
@@ -951,6 +989,7 @@
         self.filename = filename
         Exception.__init__(self, msg)
 
+
 class DeletingNonEmptyDirectory(Exception):
     def __init__(self, filename):
         msg = "Trying to remove dir %s while it still had files" % filename
@@ -964,13 +1003,15 @@
         Exception.__init__(self, msg)
         self.filename = filename
 
-class MissingPermsFile(Exception):
+
+class MissingForSetExec(Exception):
     def __init__(self, filename):
         msg = "Attempt to change permissions on  %s, which does not exist" %\
             filename
         Exception.__init__(self, msg)
         self.filename = filename
 
+
 class MissingForRm(Exception):
     def __init__(self, filename):
         msg = "Attempt to remove missing path %s" % filename
@@ -979,17 +1020,30 @@
 
 
 class MissingForRename(Exception):
-    def __init__(self, filename):
-        msg = "Attempt to move missing path %s" % (filename)
+    def __init__(self, filename, to_path):
+        msg = "Attempt to move missing path %s to %s" % (filename, to_path)
         Exception.__init__(self, msg)
         self.filename = filename
 
+
 class NewContentsConflict(Exception):
     def __init__(self, filename):
         msg = "Conflicting contents for new file %s" % (filename)
         Exception.__init__(self, msg)
 
 
+class WeaveMergeConflict(Exception):
+    def __init__(self, filename):
+        msg = "Conflicting contents for file %s" % (filename)
+        Exception.__init__(self, msg)
+
+
+class ThreewayContentsConflict(Exception):
+    def __init__(self, filename):
+        msg = "Conflicting contents for file %s" % (filename)
+        Exception.__init__(self, msg)
+
+
 class MissingForMerge(Exception):
     def __init__(self, filename):
         msg = "The file %s was modified, but does not exist in this tree"\
@@ -1027,17 +1081,14 @@
         os.unlink(new_file)
         raise MergeConflict(this_path)
 
-    def permission_conflict(self, this_path, base_path, other_path):
-        raise MergePermissionConflict(this_path, base_path, other_path)
-
     def wrong_old_contents(self, filename, expected_contents):
         raise WrongOldContents(filename)
 
     def rem_contents_conflict(self, filename, this_contents, base_contents):
         raise RemoveContentsConflict(filename)
 
-    def wrong_old_perms(self, filename, old_perms, new_perms):
-        raise WrongOldPermissions(filename, old_perms, new_perms)
+    def wrong_old_exec_flag(self, filename, old_exec_flag, new_exec_flag):
+        raise WrongOldExecFlag(filename, old_exec_flag, new_exec_flag)
 
     def rmdir_non_empty(self, filename):
         raise DeletingNonEmptyDirectory(filename)
@@ -1048,14 +1099,14 @@
     def patch_target_missing(self, filename, contents):
         raise PatchTargetMissing(filename)
 
-    def missing_for_chmod(self, filename):
-        raise MissingPermsFile(filename)
+    def missing_for_exec_flag(self, filename):
+        raise MissingForExecFlag(filename)
 
     def missing_for_rm(self, filename, change):
         raise MissingForRm(filename)
 
-    def missing_for_rename(self, filename):
-        raise MissingForRename(filename)
+    def missing_for_rename(self, filename, to_path):
+        raise MissingForRename(filename, to_path)
 
     def missing_for_merge(self, file_id, other_path):
         raise MissingForMerge(other_path)
@@ -1063,11 +1114,18 @@
     def new_contents_conflict(self, filename, other_contents):
         raise NewContentsConflict(filename)
 
+    def weave_merge_conflict(self, filename, weave, other_i, out_file):
+        raise WeaveMergeConflict(filename)
+ 
+    def threeway_contents_conflict(self, filename, this_contents,
+                                   base_contents, other_contents):
+        raise ThreewayContentsConflict(filename)
+
     def finalize(self):
         pass
 
-def apply_changeset(changeset, inventory, dir, conflict_handler=None, 
-                    reverse=False):
+
+def apply_changeset(changeset, inventory, dir, conflict_handler=None):
     """Apply a changeset to a directory.
 
     :param changeset: The changes to perform
@@ -1076,14 +1134,12 @@
     :type inventory: Dictionary
     :param dir: The path of the directory to apply the changes to
     :type dir: str
-    :param reverse: If true, apply the changes in reverse
-    :type reverse: bool
     :return: The mapping of the changed entries
     :rtype: Dictionary
     """
     if conflict_handler is None:
         conflict_handler = ExceptionConflictHandler()
-    temp_dir = os.path.join(dir, "bzr-tree-change")
+    temp_dir = pathjoin(dir, "bzr-tree-change")
     try:
         os.mkdir(temp_dir)
     except OSError, e:
@@ -1099,51 +1155,29 @@
     
     #apply changes that don't affect filenames
     for entry in changeset.entries.itervalues():
-        if not entry.is_creation_or_deletion():
-            path = os.path.join(dir, inventory[entry.id])
-            entry.apply(path, conflict_handler, reverse)
+        if not entry.is_creation_or_deletion() and not entry.is_boring():
+            if entry.id not in inventory:
+                warning("entry {%s} no longer present, can't be updated",
+                        entry.id)
+                continue
+            path = pathjoin(dir, inventory[entry.id])
+            entry.apply(path, conflict_handler)
 
     # Apply renames in stages, to minimize conflicts:
     # Only files whose name or parent change are interesting, because their
     # target name may exist in the source tree.  If a directory's name changes,
     # that doesn't make its children interesting.
-    (source_entries, target_entries) = get_rename_entries(changeset, inventory,
-                                                          reverse)
+    (source_entries, target_entries) = get_rename_entries(changeset, inventory)
 
     changed_inventory = rename_to_temp_delete(source_entries, inventory, dir,
-                                              temp_dir, conflict_handler,
-                                              reverse)
+                                              temp_dir, conflict_handler)
 
     rename_to_new_create(changed_inventory, target_entries, inventory,
-                         changeset, dir, conflict_handler, reverse)
+                         changeset, dir, conflict_handler)
     os.rmdir(temp_dir)
     return changed_inventory
 
 
-def apply_changeset_tree(cset, tree, reverse=False):
-    r_inventory = {}
-    for entry in tree.source_inventory().itervalues():
-        inventory[entry.id] = entry.path
-    new_inventory = apply_changeset(cset, r_inventory, tree.root,
-                                    reverse=reverse)
-    new_entries, remove_entries = \
-        get_inventory_change(inventory, new_inventory, cset, reverse)
-    tree.update_source_inventory(new_entries, remove_entries)
-
-
-def get_inventory_change(inventory, new_inventory, cset, reverse=False):
-    new_entries = {}
-    remove_entries = []
-    for entry in cset.entries.itervalues():
-        if entry.needs_rename():
-            new_path = entry.get_new_path(inventory, cset)
-            if new_path is None:
-                remove_entries.append(entry.id)
-            else:
-                new_entries[new_path] = entry.id
-    return new_entries, remove_entries
-
-
 def print_changeset(cset):
     """Print all non-boring changeset entries
     
@@ -1156,138 +1190,20 @@
         print entry.id
         print entry.summarize_name(cset)
 
-class CompositionFailure(Exception):
-    def __init__(self, old_entry, new_entry, problem):
-        msg = "Unable to conpose entries.\n %s" % problem
-        Exception.__init__(self, msg)
-
-class IDMismatch(CompositionFailure):
-    def __init__(self, old_entry, new_entry):
-        problem = "Attempt to compose entries with different ids: %s and %s" %\
-            (old_entry.id, new_entry.id)
-        CompositionFailure.__init__(self, old_entry, new_entry, problem)
-
-def compose_changesets(old_cset, new_cset):
-    """Combine two changesets into one.  This works well for exact patching.
-    Otherwise, not so well.
-
-    :param old_cset: The first changeset that would be applied
-    :type old_cset: `Changeset`
-    :param new_cset: The second changeset that would be applied
-    :type new_cset: `Changeset`
-    :return: A changeset that combines the changes in both changesets
-    :rtype: `Changeset`
-    """
-    composed = Changeset()
-    for old_entry in old_cset.entries.itervalues():
-        new_entry = new_cset.entries.get(old_entry.id)
-        if new_entry is None:
-            composed.add_entry(old_entry)
-        else:
-            composed_entry = compose_entries(old_entry, new_entry)
-            if composed_entry.parent is not None or\
-                composed_entry.new_parent is not None:
-                composed.add_entry(composed_entry)
-    for new_entry in new_cset.entries.itervalues():
-        if not old_cset.entries.has_key(new_entry.id):
-            composed.add_entry(new_entry)
-    return composed
-
-def compose_entries(old_entry, new_entry):
-    """Combine two entries into one.
-
-    :param old_entry: The first entry that would be applied
-    :type old_entry: ChangesetEntry
-    :param old_entry: The second entry that would be applied
-    :type old_entry: ChangesetEntry
-    :return: A changeset entry combining both entries
-    :rtype: `ChangesetEntry`
-    """
-    if old_entry.id != new_entry.id:
-        raise IDMismatch(old_entry, new_entry)
-    output = ChangesetEntry(old_entry.id, old_entry.parent, old_entry.path)
-
-    if (old_entry.parent != old_entry.new_parent or 
-        new_entry.parent != new_entry.new_parent):
-        output.new_parent = new_entry.new_parent
-
-    if (old_entry.path != old_entry.new_path or 
-        new_entry.path != new_entry.new_path):
-        output.new_path = new_entry.new_path
-
-    output.contents_change = compose_contents(old_entry, new_entry)
-    output.metadata_change = compose_metadata(old_entry, new_entry)
-    return output
-
-def compose_contents(old_entry, new_entry):
-    """Combine the contents of two changeset entries.  Entries are combined
-    intelligently where possible, but the fallback behavior returns an 
-    ApplySequence.
-
-    :param old_entry: The first entry that would be applied
-    :type old_entry: `ChangesetEntry`
-    :param new_entry: The second entry that would be applied
-    :type new_entry: `ChangesetEntry`
-    :return: A combined contents change
-    :rtype: anything supporting the apply(reverse=False) method
-    """
-    old_contents = old_entry.contents_change
-    new_contents = new_entry.contents_change
-    if old_entry.contents_change is None:
-        return new_entry.contents_change
-    elif new_entry.contents_change is None:
-        return old_entry.contents_change
-    elif isinstance(old_contents, ReplaceContents) and \
-        isinstance(new_contents, ReplaceContents):
-        if old_contents.old_contents == new_contents.new_contents:
-            return None
-        else:
-            return ReplaceContents(old_contents.old_contents,
-                                   new_contents.new_contents)
-    elif isinstance(old_contents, ApplySequence):
-        output = ApplySequence(old_contents.changes)
-        if isinstance(new_contents, ApplySequence):
-            output.changes.extend(new_contents.changes)
-        else:
-            output.changes.append(new_contents)
-        return output
-    elif isinstance(new_contents, ApplySequence):
-        output = ApplySequence((old_contents.changes,))
-        output.extend(new_contents.changes)
-        return output
-    else:
-        return ApplySequence((old_contents, new_contents))
-
-def compose_metadata(old_entry, new_entry):
-    old_meta = old_entry.metadata_change
-    new_meta = new_entry.metadata_change
-    if old_meta is None:
-        return new_meta
-    elif new_meta is None:
-        return old_meta
-    elif isinstance(old_meta, ChangeUnixPermissions) and \
-        isinstance(new_meta, ChangeUnixPermissions):
-        return ChangeUnixPermissions(old_meta.old_mode, new_meta.new_mode)
-    else:
-        return ApplySequence(old_meta, new_meta)
-
-
-def changeset_is_null(changeset):
-    for entry in changeset.entries.itervalues():
-        if not entry.is_boring():
-            return False
-    return True
-
-class UnsuppportedFiletype(Exception):
-    def __init__(self, full_path, stat_result):
-        msg = "The file \"%s\" is not a supported filetype." % full_path
+
+class UnsupportedFiletype(Exception):
+    def __init__(self, kind, full_path):
+        msg = "The file \"%s\" is a %s, which is not a supported filetype." \
+            % (full_path, kind)
         Exception.__init__(self, msg)
         self.full_path = full_path
-        self.stat_result = stat_result
+        self.kind = kind
+
 
 def generate_changeset(tree_a, tree_b, interesting_ids=None):
     return ChangesetGenerator(tree_a, tree_b, interesting_ids)()
 
+
 class ChangesetGenerator(object):
     def __init__(self, tree_a, tree_b, interesting_ids=None):
         object.__init__(self)
@@ -1329,7 +1245,7 @@
     def get_entry(self, file_id, tree):
         if not tree.has_or_had_id(file_id):
             return None
-        return tree.tree.inventory[file_id]
+        return tree.inventory[file_id]
 
     def get_entry_parent(self, entry):
         if entry is None:
@@ -1378,7 +1294,6 @@
             return self.make_entry(id, only_interesting=False)
         else:
             return cs_entry
-        
 
     def make_entry(self, id, only_interesting=True):
         cs_entry = self.make_basic_entry(id, only_interesting)
@@ -1386,12 +1301,7 @@
         if cs_entry is None:
             return None
 
-        full_path_a = self.tree_a.readonly_path(id)
-        full_path_b = self.tree_b.readonly_path(id)
-        stat_a = self.lstat(full_path_a)
-        stat_b = self.lstat(full_path_b)
-
-        cs_entry.metadata_change = self.make_mode_change(stat_a, stat_b)
+        cs_entry.metadata_change = self.make_exec_flag_change(id)
 
         if id in self.tree_a and id in self.tree_b:
             a_sha1 = self.tree_a.get_file_sha1(id)
@@ -1399,66 +1309,47 @@
             if None not in (a_sha1, b_sha1) and a_sha1 == b_sha1:
                 return cs_entry
 
-        cs_entry.contents_change = self.make_contents_change(full_path_a,
-                                                             stat_a, 
-                                                             full_path_b, 
-                                                             stat_b)
+        cs_entry.contents_change = self.make_contents_change(id)
         return cs_entry
 
-    def make_mode_change(self, stat_a, stat_b):
-        mode_a = None
-        if stat_a is not None and not stat.S_ISLNK(stat_a.st_mode):
-            mode_a = stat_a.st_mode & 0777
-        mode_b = None
-        if stat_b is not None and not stat.S_ISLNK(stat_b.st_mode):
-            mode_b = stat_b.st_mode & 0777
-        if mode_a == mode_b:
-            return None
-        return ChangeUnixPermissions(mode_a, mode_b)
-
-    def make_contents_change(self, full_path_a, stat_a, full_path_b, stat_b):
-        if stat_a is None and stat_b is None:
-            return None
-        if None not in (stat_a, stat_b) and stat.S_ISDIR(stat_a.st_mode) and\
-            stat.S_ISDIR(stat_b.st_mode):
-            return None
-        if None not in (stat_a, stat_b) and stat.S_ISREG(stat_a.st_mode) and\
-            stat.S_ISREG(stat_b.st_mode):
-            if stat_a.st_ino == stat_b.st_ino and \
-                stat_a.st_dev == stat_b.st_dev:
-                return None
-
-        a_contents = self.get_contents(stat_a, full_path_a)
-        b_contents = self.get_contents(stat_b, full_path_b)
+    def make_exec_flag_change(self, file_id):
+        exec_flag_a = exec_flag_b = None
+        if file_id in self.tree_a and self.tree_a.kind(file_id) == "file":
+            exec_flag_a = self.tree_a.is_executable(file_id)
+
+        if file_id in self.tree_b and self.tree_b.kind(file_id) == "file":
+            exec_flag_b = self.tree_b.is_executable(file_id)
+
+        if exec_flag_a == exec_flag_b:
+            return None
+        return ChangeExecFlag(exec_flag_a, exec_flag_b)
+
+    def make_contents_change(self, file_id):
+        a_contents = get_contents(self.tree_a, file_id)
+        b_contents = get_contents(self.tree_b, file_id)
         if a_contents == b_contents:
             return None
         return ReplaceContents(a_contents, b_contents)
 
-    def get_contents(self, stat_result, full_path):
-        if stat_result is None:
-            return None
-        elif stat.S_ISREG(stat_result.st_mode):
-            return FileCreate(file(full_path, "rb").read())
-        elif stat.S_ISDIR(stat_result.st_mode):
-            return dir_create
-        elif stat.S_ISLNK(stat_result.st_mode):
-            return SymlinkCreate(os.readlink(full_path))
-        else:
-            raise UnsupportedFiletype(full_path, stat_result)
 
-    def lstat(self, full_path):
-        stat_result = None
-        if full_path is not None:
-            try:
-                stat_result = os.lstat(full_path)
-            except OSError, e:
-                if e.errno != errno.ENOENT:
-                    raise
-        return stat_result
+def get_contents(tree, file_id):
+    """Return the appropriate contents to create a copy of file_id from tree"""
+    if file_id not in tree:
+        return None
+    kind = tree.kind(file_id)
+    if kind == "file":
+        return TreeFileCreate(tree, file_id)
+    elif kind in ("directory", "root_directory"):
+        return dir_create
+    elif kind == "symlink":
+        return SymlinkCreate(tree.get_symlink_target(file_id))
+    else:
+        raise UnsupportedFiletype(kind, tree.id2path(file_id))
 
 
 def full_path(entry, tree):
-    return os.path.join(tree.root, entry.path)
+    return pathjoin(tree.basedir, entry.path)
+
 
 def new_delete_entry(entry, tree, inventory, delete):
     if entry.path == "":
@@ -1476,10 +1367,8 @@
     status = os.lstat(full_path)
     if stat.S_ISDIR(file_stat.st_mode):
         action = dir_create
-    
-
-
-        
+
+
 # XXX: Can't we unify this with the regular inventory object
 class Inventory(object):
     def __init__(self, inventory):
@@ -1514,7 +1403,9 @@
             return None
         directory = self.get_dir(id)
         if directory == '.':
-            directory = './.'
+            directory = u'./.'
         if directory is None:
             return NULL_ID
         return self.get_rinventory().get(directory)
+
+

=== modified file 'bzrlib/check.py'
--- bzrlib/check.py	2005-09-30 02:54:51 +0000
+++ bzrlib/check.py	2005-12-19 20:31:01 +0000
@@ -25,6 +25,8 @@
 # TODO: Check revision, inventory and entry objects have all 
 # required fields.
 
+# TODO: Get every revision in the revision-store even if they're not
+# referenced by history and make sure they're all valid.
 
 import bzrlib.ui
 from bzrlib.trace import note, warning
@@ -32,52 +34,68 @@
 from bzrlib.trace import mutter
 from bzrlib.errors import BzrCheckError, NoSuchRevision
 from bzrlib.inventory import ROOT_ID
-from bzrlib.branch import gen_root_id
 
 
 class Check(object):
     """Check a branch"""
+
     def __init__(self, branch):
         self.branch = branch
-        branch.lock_read()
-        try:
-            branch.weave_store.enable_cache = True
-            branch.control_weaves.enable_cache = True
-            self.run()
-        finally:
-            branch.unlock()
-            branch.weave_store.enable_cache = False
-            branch.control_weaves.enable_cache = False
-
-
-    def run(self):
-        branch = self.branch
-
-
         self.checked_text_cnt = 0
         self.checked_rev_cnt = 0
+        self.ghosts = []
         self.repeated_text_cnt = 0
+        self.missing_parent_links = {}
         self.missing_inventory_sha_cnt = 0
         self.missing_revision_cnt = 0
         # maps (file-id, version) -> sha1
         self.checked_texts = {}
-
-        history = branch.revision_history()
-        revno = 0
-        revcount = len(history)
-
-        last_rev_id = None
+        self.checked_weaves = {}
+
+    def check(self):
+        self.branch.lock_read()
         self.progress = bzrlib.ui.ui_factory.progress_bar()
-        for rev_id in history:
-            self.progress.update('checking revision', revno, revcount)
-            revno += 1
-            self.check_one_rev(rev_id, last_rev_id)
-            last_rev_id = rev_id
-        self.progress.clear()
-        self.report_results()
-
-
-    def report_results(self):
+        try:
+            self.progress.update('retrieving inventory', 0, 0)
+            # do not put in init, as it should be done with progess,
+            # and inside the lock.
+            self.inventory_weave = self.branch._get_inventory_weave()
+            self.history = self.branch.revision_history()
+            if not len(self.history):
+                # nothing to see here
+                return
+            self.plan_revisions()
+            revno = 0
+            self.check_weaves()
+            while revno < len(self.planned_revisions):
+                rev_id = self.planned_revisions[revno]
+                self.progress.update('checking revision', revno,
+                                     len(self.planned_revisions))
+                revno += 1
+                self.check_one_rev(rev_id)
+        finally:
+            self.progress.clear()
+            self.branch.unlock()
+
+    def plan_revisions(self):
+        if not self.branch.revision_store.listable():
+            self.planned_revisions = self.branch.get_ancestry(self.history[-1])
+            self.planned_revisions.remove(None)
+            # FIXME progress bars should support this more nicely.
+            self.progress.clear()
+            print ("Checking reachable history -"
+                   " for a complete check use a local branch.")
+            return
+        
+        self.planned_revisions = set(self.branch.revision_store)
+        inventoried = set(self.inventory_weave.names())
+        awol = self.planned_revisions - inventoried
+        if len(awol) > 0:
+            raise BzrCheckError('Stored revisions missing from inventory'
+                '{%s}' % ','.join([f for f in awol]))
+        self.planned_revisions = list(self.planned_revisions)
+
+    def report_results(self, verbose):
         note('checked branch %s format %d',
              self.branch.base, 
              self.branch._branch_format)
@@ -85,15 +103,28 @@
         note('%6d revisions', self.checked_rev_cnt)
         note('%6d unique file texts', self.checked_text_cnt)
         note('%6d repeated file texts', self.repeated_text_cnt)
+        note('%6d weaves', len(self.checked_weaves))
         if self.missing_inventory_sha_cnt:
-            note('%d revisions are missing inventory_sha1',
+            note('%6d revisions are missing inventory_sha1',
                  self.missing_inventory_sha_cnt)
         if self.missing_revision_cnt:
-            note('%d revisions are mentioned but not present',
+            note('%6d revisions are mentioned but not present',
                  self.missing_revision_cnt)
-
-
-    def check_one_rev(self, rev_id, last_rev_id):
+        if len(self.ghosts):
+            note('%6d ghost revisions', len(self.ghosts))
+            if verbose:
+                for ghost in self.ghosts:
+                    note('      %s', ghost)
+        if len(self.missing_parent_links):
+            note('%6d revisions missing parents in ancestry', 
+                 len(self.missing_parent_links))
+            if verbose:
+                for link, linkers in self.missing_parent_links.items():
+                    note('      %s should be in the ancestry for:', link)
+                    for linker in linkers:
+                        note('       * %s', linker)
+
+    def check_one_rev(self, rev_id):
         """Check one revision.
 
         rev_id - the one to check
@@ -101,26 +132,49 @@
         last_rev_id - the previous one on the mainline, if any.
         """
 
-        # mutter('    revision {%s}' % rev_id)
+        # mutter('    revision {%s}', rev_id)
         branch = self.branch
-        rev = branch.get_revision(rev_id)
+        try:
+            rev_history_position = self.history.index(rev_id)
+        except ValueError:
+            rev_history_position = None
+        last_rev_id = None
+        if rev_history_position:
+            rev = branch.get_revision(rev_id)
+            if rev_history_position > 0:
+                last_rev_id = self.history[rev_history_position - 1]
+        else:
+            rev = branch.get_revision(rev_id)
+                
         if rev.revision_id != rev_id:
             raise BzrCheckError('wrong internal revision id in revision {%s}'
                                 % rev_id)
 
         # check the previous history entry is a parent of this entry
         if rev.parent_ids:
-            if last_rev_id is None:
-                raise BzrCheckError("revision {%s} has %d parents, but is the "
-                                    "start of the branch"
-                                    % (rev_id, len(rev.parent_ids)))
-            for parent_id in rev.parent_ids:
-                if parent_id == last_rev_id:
-                    break
-            else:
-                raise BzrCheckError("previous revision {%s} not listed among "
-                                    "parents of {%s}"
-                                    % (last_rev_id, rev_id))
+            if last_rev_id is not None:
+                for parent_id in rev.parent_ids:
+                    if parent_id == last_rev_id:
+                        break
+                else:
+                    raise BzrCheckError("previous revision {%s} not listed among "
+                                        "parents of {%s}"
+                                        % (last_rev_id, rev_id))
+            for parent in rev.parent_ids:
+                if not parent in self.planned_revisions:
+                    missing_links = self.missing_parent_links.get(parent, [])
+                    missing_links.append(rev_id)
+                    self.missing_parent_links[parent] = missing_links
+                    # list based so somewhat slow,
+                    # TODO have a planned_revisions list and set.
+                    if self.branch.has_revision(parent):
+                        missing_ancestry = self.branch.get_ancestry(parent)
+                        for missing in missing_ancestry:
+                            if (missing is not None 
+                                and missing not in self.planned_revisions):
+                                self.planned_revisions.append(missing)
+                    else:
+                        self.ghosts.append(rev_id)
         elif last_rev_id:
             raise BzrCheckError("revision {%s} has no parents listed "
                                 "but preceded by {%s}"
@@ -133,10 +187,28 @@
                     ' value in revision {%s}' % rev_id)
         else:
             missing_inventory_sha_cnt += 1
-            mutter("no inventory_sha1 on revision {%s}" % rev_id)
+            mutter("no inventory_sha1 on revision {%s}", rev_id)
         self._check_revision_tree(rev_id)
         self.checked_rev_cnt += 1
 
+    def check_weaves(self):
+        """Check all the weaves we can get our hands on.
+        """
+        n_weaves = 1
+        weave_ids = []
+        if self.branch.weave_store.listable():
+            weave_ids = list(self.branch.weave_store)
+            n_weaves = len(weave_ids)
+        self.progress.update('checking weave', 0, n_weaves)
+        self.inventory_weave.check(progress_bar=self.progress)
+        for i, weave_id in enumerate(weave_ids):
+            self.progress.update('checking weave', i, n_weaves)
+            w = self.branch.weave_store.get_weave(weave_id,
+                    self.branch.get_transaction())
+            # No progress here, because it looks ugly.
+            w.check()
+            self.checked_weaves[weave_id] = True
+
     def _check_revision_tree(self, rev_id):
         tree = self.branch.revision_tree(rev_id)
         inv = tree.inventory
@@ -159,6 +231,8 @@
             seen_names[path] = True
 
 
-def check(branch):
+def check(branch, verbose):
     """Run consistency checks on a branch."""
-    Check(branch)
+    checker = Check(branch)
+    checker.check()
+    checker.report_results(verbose)

=== modified file 'bzrlib/clone.py'
--- bzrlib/clone.py	2005-10-06 06:31:33 +0000
+++ bzrlib/clone.py	2005-11-29 01:12:50 +0000
@@ -52,6 +52,7 @@
 from bzrlib.branch import Branch
 from bzrlib.trace import mutter, note
 from bzrlib.store import copy_all
+from bzrlib.errors import InvalidRevisionId
 
 def copy_branch(branch_from, to_location, revision=None, basis_branch=None):
     """Copy branch_from into the existing directory to_location.
@@ -75,35 +76,39 @@
     assert isinstance(to_location, basestring)
     if basis_branch is not None:
         note("basis_branch is not supported for fast weave copy yet.")
-    if not (branch_from.weave_store.listable()
-            and branch_from.revision_store.listable()):
-        return copy_branch_slower(branch_from, to_location, revision,
-                                  basis_branch)
-    history = _get_truncated_history(branch_from, revision)
-    if not bzrlib.osutils.lexists(to_location):
-        os.mkdir(to_location)
-    branch_to = Branch.initialize(to_location)
-    mutter("copy branch from %s to %s", branch_from, branch_to)
-    branch_to.set_root_id(branch_from.get_root_id())
-    branch_to.append_revision(*history)
-    _copy_control_weaves(branch_from, branch_to)
-    _copy_text_weaves(branch_from, branch_to)
-    _copy_revision_store(branch_from, branch_to)
-    build_working_dir(to_location)
-    branch_to.set_parent(branch_from.base)
-    mutter("copied")
-    return branch_to
-
-
-def _get_truncated_history(branch_from, revision):
+    branch_from.lock_read()
+    try:
+        if not (branch_from.weave_store.listable()
+                and branch_from.revision_store.listable()):
+            return copy_branch_slower(branch_from, to_location, revision,
+                                      basis_branch)
+        history = _get_truncated_history(branch_from, revision)
+        if not bzrlib.osutils.lexists(to_location):
+            os.mkdir(to_location)
+        branch_to = Branch.initialize(to_location)
+        mutter("copy branch from %s to %s", branch_from, branch_to)
+        branch_to.working_tree().set_root_id(branch_from.get_root_id())
+        _copy_control_weaves(branch_from, branch_to)
+        _copy_text_weaves(branch_from, branch_to)
+        _copy_revision_store(branch_from, branch_to)
+        branch_to.set_parent(branch_from.base)
+        # must be done *after* history is copied across
+        branch_to.append_revision(*history)
+        build_working_dir(to_location)
+        mutter("copied")
+        return branch_to
+    finally:
+        branch_from.unlock()
+
+
+def _get_truncated_history(branch_from, revision_id):
     history = branch_from.revision_history()
-    if revision is None:
+    if revision_id is None:
         return history
     try:
-        idx = history.index(revision)
+        idx = history.index(revision_id)
     except ValueError:
-        raise InvalidRevisionId('revision {%s} is not on the mainline of %s' 
-                                % (revision, branch_from))
+        raise InvalidRevisionId(revision_id=revision, branch=branch_from)
     return history[:idx+1]
 
 def _copy_text_weaves(branch_from, branch_to):
@@ -147,7 +152,7 @@
     mutter("copy branch from %s to %s", branch_from, br_to)
     if basis_branch is not None:
         basis_branch.push_stores(br_to)
-    br_to.set_root_id(branch_from.get_root_id())
+    br_to.working_tree().set_root_id(branch_from.get_root_id())
     if revision is None:
         revision = branch_from.last_revision()
     br_to.update_revisions(branch_from, stop_revision=revision)

=== modified file 'bzrlib/commands.py'
--- bzrlib/commands.py	2005-10-06 13:08:14 +0000
+++ bzrlib/commands.py	2005-12-08 06:18:00 +0000
@@ -31,18 +31,24 @@
 import os
 from warnings import warn
 from inspect import getdoc
+import errno
 
 import bzrlib
 import bzrlib.trace
-from bzrlib.trace import mutter, note, log_error, warning
-from bzrlib.errors import BzrError, BzrCheckError, BzrCommandError, NotBranchError
+from bzrlib.trace import mutter, note, log_error, warning, be_quiet
+from bzrlib.errors import (BzrError, 
+                           BzrCheckError,
+                           BzrCommandError,
+                           BzrOptionError,
+                           NotBranchError)
 from bzrlib.revisionspec import RevisionSpec
 from bzrlib import BZRDIR
+from bzrlib.option import Option
 
 plugin_cmds = {}
 
 
-def register_command(cmd):
+def register_command(cmd, decorate=False):
     "Utility function to help register a command"
     global plugin_cmds
     k = cmd.__name__
@@ -53,6 +59,12 @@
     if not plugin_cmds.has_key(k_unsquished):
         plugin_cmds[k_unsquished] = cmd
         mutter('registered plugin command %s', k_unsquished)      
+        if decorate and k_unsquished in builtin_command_names():
+            return _builtin_commands()[k_unsquished]
+    elif decorate:
+        result = plugin_cmds[k_unsquished]
+        plugin_cmds[k_unsquished] = cmd
+        return result
     else:
         log_error('Two plugins defined the same command: %r' % k)
         log_error('Not loading the one in %r' % sys.modules[cmd.__module__])
@@ -67,74 +79,6 @@
     return cmd[4:].replace('_','-')
 
 
-def _parse_revision_str(revstr):
-    """This handles a revision string -> revno.
-
-    This always returns a list.  The list will have one element for
-    each revision.
-
-    >>> _parse_revision_str('234')
-    [<RevisionSpec_int 234>]
-    >>> _parse_revision_str('234..567')
-    [<RevisionSpec_int 234>, <RevisionSpec_int 567>]
-    >>> _parse_revision_str('..')
-    [<RevisionSpec None>, <RevisionSpec None>]
-    >>> _parse_revision_str('..234')
-    [<RevisionSpec None>, <RevisionSpec_int 234>]
-    >>> _parse_revision_str('234..')
-    [<RevisionSpec_int 234>, <RevisionSpec None>]
-    >>> _parse_revision_str('234..456..789') # Maybe this should be an error
-    [<RevisionSpec_int 234>, <RevisionSpec_int 456>, <RevisionSpec_int 789>]
-    >>> _parse_revision_str('234....789') # Error?
-    [<RevisionSpec_int 234>, <RevisionSpec None>, <RevisionSpec_int 789>]
-    >>> _parse_revision_str('revid:test@other.com-234234')
-    [<RevisionSpec_revid revid:test@other.com-234234>]
-    >>> _parse_revision_str('revid:test@other.com-234234..revid:test@other.com-234235')
-    [<RevisionSpec_revid revid:test@other.com-234234>, <RevisionSpec_revid revid:test@other.com-234235>]
-    >>> _parse_revision_str('revid:test@other.com-234234..23')
-    [<RevisionSpec_revid revid:test@other.com-234234>, <RevisionSpec_int 23>]
-    >>> _parse_revision_str('date:2005-04-12')
-    [<RevisionSpec_date date:2005-04-12>]
-    >>> _parse_revision_str('date:2005-04-12 12:24:33')
-    [<RevisionSpec_date date:2005-04-12 12:24:33>]
-    >>> _parse_revision_str('date:2005-04-12T12:24:33')
-    [<RevisionSpec_date date:2005-04-12T12:24:33>]
-    >>> _parse_revision_str('date:2005-04-12,12:24:33')
-    [<RevisionSpec_date date:2005-04-12,12:24:33>]
-    >>> _parse_revision_str('-5..23')
-    [<RevisionSpec_int -5>, <RevisionSpec_int 23>]
-    >>> _parse_revision_str('-5')
-    [<RevisionSpec_int -5>]
-    >>> _parse_revision_str('123a')
-    Traceback (most recent call last):
-      ...
-    BzrError: No namespace registered for string: '123a'
-    >>> _parse_revision_str('abc')
-    Traceback (most recent call last):
-      ...
-    BzrError: No namespace registered for string: 'abc'
-    """
-    import re
-    old_format_re = re.compile('\d*:\d*')
-    m = old_format_re.match(revstr)
-    revs = []
-    if m:
-        warning('Colon separator for revision numbers is deprecated.'
-                ' Use .. instead')
-        for rev in revstr.split(':'):
-            if rev:
-                revs.append(RevisionSpec(int(rev)))
-            else:
-                revs.append(RevisionSpec(None))
-    else:
-        for x in revstr.split('..'):
-            if not x:
-                revs.append(RevisionSpec(None))
-            else:
-                revs.append(RevisionSpec(x))
-    return revs
-
-
 def _builtin_commands():
     import bzrlib.builtins
     r = {}
@@ -224,15 +168,24 @@
         List of argument forms, marked with whether they are optional,
         repeated, etc.
 
+		Examples:
+
+		['to_location', 'from_branch?', 'file*']
+
+		'to_location' is required
+		'from_branch' is optional
+		'file' can be specified 0 or more times
+
     takes_options
-        List of options that may be given for this command.
+        List of options that may be given for this command.  These can
+        be either strings, referring to globally-defined options,
+        or option objects.  Retrieve through options().
 
     hidden
         If true, this command isn't advertised.  This is typically
         for commands intended for expert users.
     """
     aliases = []
-    
     takes_args = []
     takes_options = []
 
@@ -243,23 +196,31 @@
         if self.__doc__ == Command.__doc__:
             warn("No help message set for %r" % self)
 
+    def options(self):
+        """Return dict of valid options for this command.
+
+        Maps from long option name to option object."""
+        r = dict()
+        r['help'] = Option.OPTIONS['help']
+        for o in self.takes_options:
+            if not isinstance(o, Option):
+                o = Option.OPTIONS[o]
+            r[o.name] = o
+        return r
 
     def run_argv(self, argv):
         """Parse command line and run."""
-        args, opts = parse_args(argv)
-
+        args, opts = parse_args(self, argv)
         if 'help' in opts:  # e.g. bzr add --help
             from bzrlib.help import help_on_command
             help_on_command(self.name())
             return 0
-
-        # check options are reasonable
-        allowed = self.takes_options
+        # XXX: This should be handled by the parser
+        allowed_names = self.options().keys()
         for oname in opts:
-            if oname not in allowed:
+            if oname not in allowed_names:
                 raise BzrCommandError("option '--%s' is not allowed for command %r"
                                       % (oname, self.name()))
-
         # mix arguments and options into one dictionary
         cmdargs = _match_argform(self.name(), self.takes_args, args)
         cmdopts = {}
@@ -270,7 +231,6 @@
         all_cmd_args.update(cmdopts)
 
         return self.run(**all_cmd_args)
-
     
     def run(self):
         """Actually run the command.
@@ -326,118 +286,64 @@
         parsed = [spec, None]
     return parsed
 
-
-# list of all available options; the rhs can be either None for an
-# option that takes no argument, or a constructor function that checks
-# the type.
-OPTIONS = {
-    'all':                    None,
-    'basis':                  str,
-    'diff-options':           str,
-    'help':                   None,
-    'file':                   unicode,
-    'force':                  None,
-    'format':                 unicode,
-    'forward':                None,
-    'message':                unicode,
-    'no-recurse':             None,
-    'profile':                None,
-    'revision':               _parse_revision_str,
-    'short':                  None,
-    'show-ids':               None,
-    'timezone':               str,
-    'verbose':                None,
-    'version':                None,
-    'email':                  None,
-    'unchanged':              None,
-    'update':                 None,
-    'long':                   None,
-    'root':                   str,
-    'no-backup':              None,
-    'pattern':                str,
-    'remember':               None,
-    }
-
-SHORT_OPTIONS = {
-    'F':                      'file', 
-    'h':                      'help',
-    'm':                      'message',
-    'r':                      'revision',
-    'v':                      'verbose',
-    'l':                      'long',
-}
-
-
-def parse_args(argv):
+def parse_args(command, argv):
     """Parse command line.
     
     Arguments and options are parsed at this level before being passed
     down to specific command handlers.  This routine knows, from a
     lookup table, something about the available options, what optargs
     they take, and which commands will accept them.
-
-    >>> parse_args('--help'.split())
-    ([], {'help': True})
-    >>> parse_args('help -- --invalidcmd'.split())
-    (['help', '--invalidcmd'], {})
-    >>> parse_args('--version'.split())
-    ([], {'version': True})
-    >>> parse_args('status --all'.split())
-    (['status'], {'all': True})
-    >>> parse_args('commit --message=biter'.split())
-    (['commit'], {'message': u'biter'})
-    >>> parse_args('log -r 500'.split())
-    (['log'], {'revision': [<RevisionSpec_int 500>]})
-    >>> parse_args('log -r500..600'.split())
-    (['log'], {'revision': [<RevisionSpec_int 500>, <RevisionSpec_int 600>]})
-    >>> parse_args('log -vr500..600'.split())
-    (['log'], {'verbose': True, 'revision': [<RevisionSpec_int 500>, <RevisionSpec_int 600>]})
-    >>> parse_args('log -rrevno:500..600'.split()) #the r takes an argument
-    (['log'], {'revision': [<RevisionSpec_revno revno:500>, <RevisionSpec_int 600>]})
     """
+    # TODO: chop up this beast; make it a method of the Command
     args = []
     opts = {}
 
+    cmd_options = command.options()
     argsover = False
     while argv:
         a = argv.pop(0)
-        if not argsover and a[0] == '-':
+        if argsover:
+            args.append(a)
+            continue
+        elif a == '--':
+            # We've received a standalone -- No more flags
+            argsover = True
+            continue
+        if a[0] == '-':
             # option names must not be unicode
             a = str(a)
             optarg = None
             if a[1] == '-':
-                if a == '--':
-                    # We've received a standalone -- No more flags
-                    argsover = True
-                    continue
-                mutter("  got option %r" % a)
+                mutter("  got option %r", a)
                 if '=' in a:
                     optname, optarg = a[2:].split('=', 1)
                 else:
                     optname = a[2:]
-                if optname not in OPTIONS:
-                    raise BzrError('unknown long option %r' % a)
+                if optname not in cmd_options:
+                    raise BzrOptionError('unknown long option %r for command %s'
+                        % (a, command.name()))
             else:
                 shortopt = a[1:]
-                if shortopt in SHORT_OPTIONS:
+                if shortopt in Option.SHORT_OPTIONS:
                     # Multi-character options must have a space to delimit
                     # their value
-                    optname = SHORT_OPTIONS[shortopt]
+                    # ^^^ what does this mean? mbp 20051014
+                    optname = Option.SHORT_OPTIONS[shortopt].name
                 else:
                     # Single character short options, can be chained,
                     # and have their value appended to their name
                     shortopt = a[1:2]
-                    if shortopt not in SHORT_OPTIONS:
+                    if shortopt not in Option.SHORT_OPTIONS:
                         # We didn't find the multi-character name, and we
                         # didn't find the single char name
                         raise BzrError('unknown short option %r' % a)
-                    optname = SHORT_OPTIONS[shortopt]
+                    optname = Option.SHORT_OPTIONS[shortopt].name
 
                     if a[2:]:
                         # There are extra things on this option
                         # see if it is the value, or if it is another
                         # short option
-                        optargfn = OPTIONS[optname]
+                        optargfn = Option.OPTIONS[optname].type
                         if optargfn is None:
                             # This option does not take an argument, so the
                             # next entry is another short option, pack it back
@@ -448,11 +354,15 @@
                             # into the array
                             optarg = a[2:]
             
+                if optname not in cmd_options:
+                    raise BzrOptionError('unknown short option %r for command'
+                        ' %s' % (shortopt, command.name()))
             if optname in opts:
                 # XXX: Do we ever want to support this, e.g. for -r?
                 raise BzrError('repeated option %r' % a)
                 
-            optargfn = OPTIONS[optname]
+            option_obj = cmd_options[optname]
+            optargfn = option_obj.type
             if optargfn:
                 if optarg == None:
                     if not argv:
@@ -466,12 +376,9 @@
                 opts[optname] = True
         else:
             args.append(a)
-
     return args, opts
 
 
-
-
 def _match_argform(cmd, takes_args, args):
     argdict = {}
 
@@ -540,6 +447,13 @@
         os.remove(pfname)
 
 
+def apply_lsprofiled(the_callable, *args, **kwargs):
+    from bzrlib.lsprof import profile
+    ret,stats = profile(the_callable,*args,**kwargs)
+    stats.sort()
+    stats.pprint()
+    return ret
+
 def run_bzr(argv):
     """Execute a command.
 
@@ -562,14 +476,14 @@
         other behaviour.)
 
     --profile
-        Run under the Python profiler.
+        Run under the Python hotshot profiler.
+
+    --lsprof
+        Run under the Python lsprof profiler.
     """
-    # Load all of the transport methods
-    import bzrlib.transport.local, bzrlib.transport.http
-    
     argv = [a.decode(bzrlib.user_encoding) for a in argv]
 
-    opt_profile = opt_no_plugins = opt_builtin = False
+    opt_lsprof = opt_profile = opt_no_plugins = opt_builtin = False
 
     # --no-plugins is handled specially at a very early stage. We need
     # to load plugins before doing other command parsing so that they
@@ -578,12 +492,16 @@
     for a in argv:
         if a == '--profile':
             opt_profile = True
+        elif a == '--lsprof':
+            opt_lsprof = True
         elif a == '--no-plugins':
             opt_no_plugins = True
         elif a == '--builtin':
             opt_builtin = True
+        elif a in ('--quiet', '-q'):
+            be_quiet()
         else:
-            break
+            continue
         argv.remove(a)
 
     if (not argv) or (argv[0] == '--help'):
@@ -607,19 +525,45 @@
 
     cmd_obj = get_cmd_object(cmd, plugins_override=not opt_builtin)
 
-    if opt_profile:
-        ret = apply_profiled(cmd_obj.run_argv, argv)
-    else:
-        ret = cmd_obj.run_argv(argv)
-    return ret or 0
+    try:
+        if opt_lsprof:
+            ret = apply_lsprofiled(cmd_obj.run_argv, argv)
+        elif opt_profile:
+            ret = apply_profiled(cmd_obj.run_argv, argv)
+        else:
+            ret = cmd_obj.run_argv(argv)
+        return ret or 0
+    finally:
+        # reset, in case we may do other commands later within the same process
+        be_quiet(False)
+
+def display_command(func):
+    """Decorator that suppresses pipe/interrupt errors."""
+    def ignore_pipe(*args, **kwargs):
+        try:
+            result = func(*args, **kwargs)
+            sys.stdout.flush()
+            return result
+        except IOError, e:
+            if not hasattr(e, 'errno'):
+                raise
+            if e.errno != errno.EPIPE:
+                raise
+            pass
+        except KeyboardInterrupt:
+            pass
+    return ignore_pipe
 
 
 def main(argv):
     import bzrlib.ui
+    from bzrlib.ui.text import TextUIFactory
+    ## bzrlib.trace.enable_default_logging()
     bzrlib.trace.log_startup(argv)
-    bzrlib.ui.ui_factory = bzrlib.ui.TextUIFactory()
-
-    return run_bzr_catch_errors(argv[1:])
+    bzrlib.ui.ui_factory = TextUIFactory()
+    ret = run_bzr_catch_errors(argv[1:])
+    mutter("return code %d", ret)
+    return ret
 
 
 def run_bzr_catch_errors(argv):
@@ -629,31 +573,22 @@
         finally:
             # do this here inside the exception wrappers to catch EPIPE
             sys.stdout.flush()
-    except BzrCommandError, e:
-        # command line syntax error, etc
-        log_error(str(e))
-        return 1
-    except BzrError, e:
-        bzrlib.trace.log_exception()
-        return 1
-    except AssertionError, e:
-        bzrlib.trace.log_exception('assertion failed: ' + str(e))
-        return 3
-    except KeyboardInterrupt, e:
-        bzrlib.trace.log_exception('interrupted')
-        return 2
     except Exception, e:
+        # used to handle AssertionError and KeyboardInterrupt
+        # specially here, but hopefully they're handled ok by the logger now
         import errno
         if (isinstance(e, IOError) 
             and hasattr(e, 'errno')
             and e.errno == errno.EPIPE):
             bzrlib.trace.note('broken pipe')
-            return 2
+            return 3
         else:
-            ## import pdb
-            ## pdb.pm()
             bzrlib.trace.log_exception()
-            return 2
+            if os.environ.get('BZR_PDB'):
+                print '**** entering debugger'
+                import pdb
+                pdb.post_mortem(sys.exc_traceback)
+            return 3
 
 if __name__ == '__main__':
     sys.exit(main(sys.argv))

=== modified file 'bzrlib/commit.py'
--- bzrlib/commit.py	2005-10-06 13:08:14 +0000
+++ bzrlib/commit.py	2005-12-08 07:35:29 +0000
@@ -56,6 +56,12 @@
 # merges from, then it should still be reported as newly added
 # relative to the basis revision.
 
+# TODO: Do checks that the tree can be committed *before* running the 
+# editor; this should include checks for a pointless commit and for 
+# unknown or missing files.
+
+# TODO: If commit fails, leave the message in a file somewhere.
+
 
 import os
 import re
@@ -66,23 +72,27 @@
 from binascii import hexlify
 from cStringIO import StringIO
 
-from bzrlib.osutils import (local_time_offset, username,
-                            rand_bytes, compact_date, user_email,
+from bzrlib.atomicfile import AtomicFile
+from bzrlib.osutils import (local_time_offset,
+                            rand_bytes, compact_date,
                             kind_marker, is_inside_any, quotefn,
                             sha_string, sha_strings, sha_file, isdir, isfile,
                             split_lines)
-from bzrlib.branch import gen_file_id
+import bzrlib.config
 from bzrlib.errors import (BzrError, PointlessCommit,
                            HistoryMissing,
-                           ConflictsInTree
+                           ConflictsInTree,
+                           StrictCommitFailed
                            )
+import bzrlib.gpg as gpg
 from bzrlib.revision import Revision
+from bzrlib.testament import Testament
 from bzrlib.trace import mutter, note, warning
 from bzrlib.xml5 import serializer_v5
 from bzrlib.inventory import Inventory, ROOT_ID
 from bzrlib.weave import Weave
 from bzrlib.weavefile import read_weave, write_weave_v5
-from bzrlib.atomicfile import AtomicFile
+from bzrlib.workingtree import WorkingTree
 
 
 def commit(*args, **kwargs):
@@ -114,6 +124,7 @@
     def missing(self, path):
         pass
 
+
 class ReportCommitToLog(NullCommitReporter):
 
     def snapshot_change(self, change, path):
@@ -131,6 +142,7 @@
     def missing(self, path):
         note('missing %s', path)
 
+
 class Commit(object):
     """Task of committing a new revision.
 
@@ -144,12 +156,16 @@
             working inventory.
     """
     def __init__(self,
-                 reporter=None):
+                 reporter=None,
+                 config=None):
         if reporter is not None:
             self.reporter = reporter
         else:
             self.reporter = NullCommitReporter()
-
+        if config is not None:
+            self.config = config
+        else:
+            self.config = None
         
     def commit(self,
                branch, message,
@@ -159,7 +175,9 @@
                specific_files=None,
                rev_id=None,
                allow_pointless=True,
-               verbose=False):
+               strict=False,
+               verbose=False,
+               revprops=None):
         """Commit working copy as a new revision.
 
         timestamp -- if not None, seconds-since-epoch for a
@@ -175,6 +193,11 @@
 
         allow_pointless -- If true (default), commit even if nothing
             has changed and no merges are recorded.
+
+        strict -- If true, don't allow a commit if the working tree
+            contains unknown files.
+
+        revprops -- Properties for new revision
         """
         mutter('preparing to commit')
 
@@ -183,19 +206,31 @@
         self.rev_id = rev_id
         self.specific_files = specific_files
         self.allow_pointless = allow_pointless
+        self.revprops = {'branch-nick': branch.nick}
+        if revprops:
+            self.revprops.update(revprops)
+        self.work_tree = WorkingTree(branch.base, branch)
+
+        if strict:
+            # raise an exception as soon as we find a single unknown.
+            for unknown in self.work_tree.unknowns():
+                raise StrictCommitFailed()
 
         if timestamp is None:
             self.timestamp = time.time()
         else:
             self.timestamp = long(timestamp)
             
+        if self.config is None:
+            self.config = bzrlib.config.BranchConfig(self.branch)
+
         if rev_id is None:
-            self.rev_id = _gen_revision_id(self.branch, self.timestamp)
+            self.rev_id = _gen_revision_id(self.config, self.timestamp)
         else:
             self.rev_id = rev_id
 
         if committer is None:
-            self.committer = username(self.branch)
+            self.committer = self.config.username()
         else:
             assert isinstance(committer, basestring), type(committer)
             self.committer = committer
@@ -205,13 +240,14 @@
         else:
             self.timezone = int(timezone)
 
-        assert isinstance(message, basestring), type(message)
+        if isinstance(message, str):
+            message = message.decode(bzrlib.user_encoding)
+        assert isinstance(message, unicode), type(message)
         self.message = message
         self._escape_commit_message()
 
         self.branch.lock_write()
         try:
-            self.work_tree = self.branch.working_tree()
             self.work_inv = self.work_tree.inventory
             self.basis_tree = self.branch.basis_tree()
             self.basis_inv = self.basis_tree.inventory
@@ -236,9 +272,17 @@
 
             self._record_inventory()
             self._make_revision()
+            self.work_tree.set_pending_merges([])
+            self.branch.append_revision(self.rev_id)
             self.reporter.completed(self.branch.revno()+1, self.rev_id)
-            self.branch.append_revision(self.rev_id)
-            self.branch.set_pending_merges([])
+            if self.config.post_commit() is not None:
+                hooks = self.config.post_commit().split(' ')
+                # this would be nicer with twisted.python.reflect.namedAny
+                for hook in hooks:
+                    result = eval(hook + '(branch, rev_id)',
+                                  {'branch':self.branch,
+                                   'bzrlib':bzrlib,
+                                   'rev_id':self.rev_id})
         finally:
             self.branch.unlock()
 
@@ -248,7 +292,8 @@
         self.inv_sha1 = sha_string(inv_text)
         s = self.branch.control_weaves
         s.add_text('inventory', self.rev_id,
-                   split_lines(inv_text), self.present_parents)
+                   split_lines(inv_text), self.present_parents,
+                   self.branch.get_transaction())
 
     def _escape_commit_message(self):
         """Replace xml-incompatible control characters."""
@@ -256,14 +301,8 @@
         # represented in well-formed XML; escape characters that
         # aren't listed in the XML specification
         # (http://www.w3.org/TR/REC-xml/#NT-Char).
-        if isinstance(self.message, unicode):
-            char_pattern = u'[^\x09\x0A\x0D\u0020-\uD7FF\uE000-\uFFFD]'
-        else:
-            # Use a regular 'str' as pattern to avoid having re.subn
-            # return 'unicode' results.
-            char_pattern = '[^x09\x0A\x0D\x20-\xFF]'
         self.message, escape_count = re.subn(
-            char_pattern,
+            u'[^\x09\x0A\x0D\u0020-\uD7FF\uE000-\uFFFD]+',
             lambda match: match.group(0).encode('unicode_escape'),
             self.message)
         if escape_count:
@@ -271,7 +310,7 @@
 
     def _gather_parents(self):
         """Record the parents of a merge for merge detection."""
-        pending_merges = self.branch.pending_merges()
+        pending_merges = self.work_tree.pending_merges()
         self.parents = []
         self.parent_invs = []
         self.present_parents = []
@@ -301,11 +340,16 @@
                             committer=self.committer,
                             message=self.message,
                             inventory_sha1=self.inv_sha1,
-                            revision_id=self.rev_id)
+                            revision_id=self.rev_id,
+                            properties=self.revprops)
         self.rev.parent_ids = self.parents
         rev_tmp = StringIO()
         serializer_v5.write_revision(self.rev, rev_tmp)
         rev_tmp.seek(0)
+        if self.config.signature_needed():
+            plaintext = Testament(self.rev, self.new_inv).as_short_text()
+            self.branch.store_revision_signature(gpg.GPGStrategy(self.config),
+                                                 plaintext, self.rev_id)
         self.branch.revision_store.add(rev_tmp, self.rev_id)
         mutter('new revision_id is {%s}', self.rev_id)
 
@@ -332,7 +376,7 @@
             deleted_ids.sort(reverse=True)
             for path, file_id in deleted_ids:
                 del self.work_inv[file_id]
-            self.branch._write_inventory(self.work_inv)
+            self.work_tree._write_inventory(self.work_inv)
 
     def _store_snapshot(self):
         """Pass over inventory and record a snapshot.
@@ -347,10 +391,12 @@
         for path, ie in self.new_inv.iter_entries():
             previous_entries = ie.find_previous_heads(
                 self.parent_invs, 
-                self.weave_store.get_weave_or_empty(ie.file_id))
+                self.weave_store.get_weave_prelude_or_empty(ie.file_id,
+                    self.branch.get_transaction()))
             if ie.revision is None:
                 change = ie.snapshot(self.rev_id, path, previous_entries,
-                                     self.work_tree, self.weave_store)
+                                     self.work_tree, self.weave_store,
+                                     self.branch.get_transaction())
             else:
                 change = "unchanged"
             self.reporter.snapshot_change(change, path)
@@ -410,8 +456,8 @@
             if file_id not in self.new_inv:
                 self.reporter.deleted(self.basis_inv.id2path(file_id))
 
-def _gen_revision_id(branch, when):
+def _gen_revision_id(config, when):
     """Return new revision-id."""
-    s = '%s-%s-' % (user_email(branch), compact_date(when))
+    s = '%s-%s-' % (config.user_email(), compact_date(when))
     s += hexlify(rand_bytes(8))
     return s

=== added file 'bzrlib/config.py'
--- bzrlib/config.py	1970-01-01 00:00:00 +0000
+++ bzrlib/config.py	2006-01-02 22:37:32 +0000
@@ -0,0 +1,563 @@
+# Copyright (C) 2005 by Canonical Ltd
+#   Authors: Robert Collins <robert.collins@canonical.com>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Configuration that affects the behaviour of Bazaar.
+
+Currently this configuration resides in ~/.bazaar/bazaar.conf
+and ~/.bazaar/branches.conf, which is written to by bzr.
+
+In bazaar.conf the following options may be set:
+[DEFAULT]
+editor=name-of-program
+email=Your Name <your@email.address>
+check_signatures=require|ignore|check-available(default)
+create_signatures=always|never|when-required(default)
+gpg_signing_command=name-of-program
+
+in branches.conf, you specify the url of a branch and options for it.
+Wildcards may be used - * and ? as normal in shell completion. Options
+set in both bazaar.conf and branches.conf are overriden by the branches.conf
+setting.
+[/home/robertc/source]
+recurse=False|True(default)
+email= as above
+check_signatures= as abive 
+create_signatures= as above.
+
+explanation of options
+----------------------
+editor - this option sets the pop up editor to use during commits.
+email - this option sets the user id bzr will use when committing.
+check_signatures - this option controls whether bzr will require good gpg
+                   signatures, ignore them, or check them if they are 
+                   present.
+create_signatures - this option controls whether bzr will always create 
+                    gpg signatures, never create them, or create them if the
+                    branch is configured to require them.
+                    NB: This option is planned, but not implemented yet.
+"""
+
+
+import errno
+import os
+import sys
+from fnmatch import fnmatch
+import re
+
+import bzrlib
+import bzrlib.errors as errors
+from bzrlib.osutils import pathjoin
+from bzrlib.trace import mutter
+import bzrlib.util.configobj.configobj as configobj
+from StringIO import StringIO
+
+CHECK_IF_POSSIBLE=0
+CHECK_ALWAYS=1
+CHECK_NEVER=2
+
+
+class ConfigObj(configobj.ConfigObj):
+
+    def get_bool(self, section, key):
+        val = self[section][key].lower()
+        if val in ('1', 'yes', 'true', 'on'):
+            return True
+        elif val in ('0', 'no', 'false', 'off'):
+            return False
+        else:
+            raise ValueError("Value %r is not boolean" % val)
+
+    def get_value(self, section, name):
+        # Try [] for the old DEFAULT section.
+        if section == "DEFAULT":
+            try:
+                return self[name]
+            except KeyError:
+                pass
+        return self[section][name]
+
+
+class Config(object):
+    """A configuration policy - what username, editor, gpg needs etc."""
+
+    def get_editor(self):
+        """Get the users pop up editor."""
+        raise NotImplementedError
+
+    def _get_signature_checking(self):
+        """Template method to override signature checking policy."""
+
+    def _get_user_option(self, option_name):
+        """Template method to provide a user option."""
+        return None
+
+    def get_user_option(self, option_name):
+        """Get a generic option - no special process, no default."""
+        return self._get_user_option(option_name)
+
+    def gpg_signing_command(self):
+        """What program should be used to sign signatures?"""
+        result = self._gpg_signing_command()
+        if result is None:
+            result = "gpg"
+        return result
+
+    def _gpg_signing_command(self):
+        """See gpg_signing_command()."""
+        return None
+
+    def __init__(self):
+        super(Config, self).__init__()
+
+    def post_commit(self):
+        """An ordered list of python functions to call.
+
+        Each function takes branch, rev_id as parameters.
+        """
+        return self._post_commit()
+
+    def _post_commit(self):
+        """See Config.post_commit."""
+        return None
+
+    def user_email(self):
+        """Return just the email component of a username."""
+        return extract_email_address(self.username())
+
+    def username(self):
+        """Return email-style username.
+    
+        Something similar to 'Martin Pool <mbp@sourcefrog.net>'
+        
+        $BZREMAIL can be set to override this, then
+        the concrete policy type is checked, and finally
+        $EMAIL is examined.
+        If none is found, a reasonable default is (hopefully)
+        created.
+    
+        TODO: Check it's reasonably well-formed.
+        """
+        v = os.environ.get('BZREMAIL')
+        if v:
+            return v.decode(bzrlib.user_encoding)
+    
+        v = self._get_user_id()
+        if v:
+            return v
+        
+        v = os.environ.get('EMAIL')
+        if v:
+            return v.decode(bzrlib.user_encoding)
+
+        name, email = _auto_user_id()
+        if name:
+            return '%s <%s>' % (name, email)
+        else:
+            return email
+
+    def signature_checking(self):
+        """What is the current policy for signature checking?."""
+        policy = self._get_signature_checking()
+        if policy is not None:
+            return policy
+        return CHECK_IF_POSSIBLE
+
+    def signature_needed(self):
+        """Is a signature needed when committing ?."""
+        policy = self._get_signature_checking()
+        if policy == CHECK_ALWAYS:
+            return True
+        return False
+
+
+class IniBasedConfig(Config):
+    """A configuration policy that draws from ini files."""
+
+    def _get_parser(self, file=None):
+        if self._parser is not None:
+            return self._parser
+        if file is None:
+            input = self._get_filename()
+        else:
+            input = file
+        try:
+            self._parser = ConfigObj(input)
+        except configobj.ConfigObjError, e:
+            raise errors.ParseConfigError(e.errors, e.config.filename)
+        return self._parser
+
+    def _get_section(self):
+        """Override this to define the section used by the config."""
+        return "DEFAULT"
+
+    def _get_signature_checking(self):
+        """See Config._get_signature_checking."""
+        policy = self._get_user_option('check_signatures')
+        if policy:
+            return self._string_to_signature_policy(policy)
+
+    def _get_user_id(self):
+        """Get the user id from the 'email' key in the current section."""
+        return self._get_user_option('email')
+
+    def _get_user_option(self, option_name):
+        """See Config._get_user_option."""
+        try:
+            return self._get_parser().get_value(self._get_section(),
+                                                option_name)
+        except KeyError:
+            pass
+
+    def _gpg_signing_command(self):
+        """See Config.gpg_signing_command."""
+        return self._get_user_option('gpg_signing_command')
+
+    def __init__(self, get_filename):
+        super(IniBasedConfig, self).__init__()
+        self._get_filename = get_filename
+        self._parser = None
+        
+    def _post_commit(self):
+        """See Config.post_commit."""
+        return self._get_user_option('post_commit')
+
+    def _string_to_signature_policy(self, signature_string):
+        """Convert a string to a signing policy."""
+        if signature_string.lower() == 'check-available':
+            return CHECK_IF_POSSIBLE
+        if signature_string.lower() == 'ignore':
+            return CHECK_NEVER
+        if signature_string.lower() == 'require':
+            return CHECK_ALWAYS
+        raise errors.BzrError("Invalid signatures policy '%s'"
+                              % signature_string)
+
+
+class GlobalConfig(IniBasedConfig):
+    """The configuration that should be used for a specific location."""
+
+    def get_editor(self):
+        return self._get_user_option('editor')
+
+    def __init__(self):
+        super(GlobalConfig, self).__init__(config_filename)
+
+
+class LocationConfig(IniBasedConfig):
+    """A configuration object that gives the policy for a location."""
+
+    def __init__(self, location):
+        super(LocationConfig, self).__init__(branches_config_filename)
+        self._global_config = None
+        self.location = location
+
+    def _get_global_config(self):
+        if self._global_config is None:
+            self._global_config = GlobalConfig()
+        return self._global_config
+
+    def _get_section(self):
+        """Get the section we should look in for config items.
+
+        Returns None if none exists. 
+        TODO: perhaps return a NullSection that thunks through to the 
+              global config.
+        """
+        sections = self._get_parser()
+        location_names = self.location.split('/')
+        if self.location.endswith('/'):
+            del location_names[-1]
+        matches=[]
+        for section in sections:
+            section_names = section.split('/')
+            if section.endswith('/'):
+                del section_names[-1]
+            names = zip(location_names, section_names)
+            matched = True
+            for name in names:
+                if not fnmatch(name[0], name[1]):
+                    matched = False
+                    break
+            if not matched:
+                continue
+            # so, for the common prefix they matched.
+            # if section is longer, no match.
+            if len(section_names) > len(location_names):
+                continue
+            # if path is longer, and recurse is not true, no match
+            if len(section_names) < len(location_names):
+                try:
+                    if not self._get_parser().get_bool(section, 'recurse'):
+                        continue
+                except KeyError:
+                    pass
+            matches.append((len(section_names), section))
+        if not len(matches):
+            return None
+        matches.sort(reverse=True)
+        return matches[0][1]
+
+    def _gpg_signing_command(self):
+        """See Config.gpg_signing_command."""
+        command = super(LocationConfig, self)._gpg_signing_command()
+        if command is not None:
+            return command
+        return self._get_global_config()._gpg_signing_command()
+
+    def _get_user_id(self):
+        user_id = super(LocationConfig, self)._get_user_id()
+        if user_id is not None:
+            return user_id
+        return self._get_global_config()._get_user_id()
+
+    def _get_user_option(self, option_name):
+        """See Config._get_user_option."""
+        option_value = super(LocationConfig, 
+                             self)._get_user_option(option_name)
+        if option_value is not None:
+            return option_value
+        return self._get_global_config()._get_user_option(option_name)
+
+    def _get_signature_checking(self):
+        """See Config._get_signature_checking."""
+        check = super(LocationConfig, self)._get_signature_checking()
+        if check is not None:
+            return check
+        return self._get_global_config()._get_signature_checking()
+
+    def _post_commit(self):
+        """See Config.post_commit."""
+        hook = self._get_user_option('post_commit')
+        if hook is not None:
+            return hook
+        return self._get_global_config()._post_commit()
+
+    def set_user_option(self, option, value):
+        """Save option and its value in the configuration."""
+        # FIXME: RBC 20051029 This should refresh the parser and also take a
+        # file lock on branches.conf.
+        conf_dir = os.path.dirname(self._get_filename())
+        ensure_config_dir_exists(conf_dir)
+        location = self.location
+        if location.endswith('/'):
+            location = location[:-1]
+        if (not location in self._get_parser() and
+            not location + '/' in self._get_parser()):
+            self._get_parser()[location]={}
+        elif location + '/' in self._get_parser():
+            location = location + '/'
+        self._get_parser()[location][option]=value
+        self._get_parser().write()
+
+
+class BranchConfig(Config):
+    """A configuration object giving the policy for a branch."""
+
+    def _get_location_config(self):
+        if self._location_config is None:
+            self._location_config = LocationConfig(self.branch.base)
+        return self._location_config
+
+    def _get_user_id(self):
+        """Return the full user id for the branch.
+    
+        e.g. "John Hacker <jhacker@foo.org>"
+        This is looked up in the email controlfile for the branch.
+        """
+        try:
+            return (self.branch.controlfile("email", "r") 
+                    .read()
+                    .decode(bzrlib.user_encoding)
+                    .rstrip("\r\n"))
+        except errors.NoSuchFile, e:
+            pass
+        
+        return self._get_location_config()._get_user_id()
+
+    def _get_signature_checking(self):
+        """See Config._get_signature_checking."""
+        return self._get_location_config()._get_signature_checking()
+
+    def _get_user_option(self, option_name):
+        """See Config._get_user_option."""
+        return self._get_location_config()._get_user_option(option_name)
+
+    def _gpg_signing_command(self):
+        """See Config.gpg_signing_command."""
+        return self._get_location_config()._gpg_signing_command()
+        
+    def __init__(self, branch):
+        super(BranchConfig, self).__init__()
+        self._location_config = None
+        self.branch = branch
+
+    def _post_commit(self):
+        """See Config.post_commit."""
+        return self._get_location_config()._post_commit()
+
+
+def ensure_config_dir_exists(path=None):
+    """Make sure a configuration directory exists.
+    This makes sure that the directory exists.
+    On windows, since configuration directories are 2 levels deep,
+    it makes sure both the directory and the parent directory exists.
+    """
+    if path is None:
+        path = config_dir()
+    if not os.path.isdir(path):
+        if sys.platform == 'win32':
+            parent_dir = os.path.dirname(path)
+            if not os.path.isdir(parent_dir):
+                mutter('creating config parent directory: %r', parent_dir)
+            os.mkdir(parent_dir)
+        mutter('creating config directory: %r', path)
+        os.mkdir(path)
+
+
+def config_dir():
+    """Return per-user configuration directory.
+
+    By default this is ~/.bazaar/
+    
+    TODO: Global option --config-dir to override this.
+    """
+    base = os.environ.get('BZR_HOME', None)
+    if sys.platform == 'win32':
+        if base is None:
+            base = os.environ.get('APPDATA', None)
+        if base is None:
+            base = os.environ.get('HOME', None)
+        if base is None:
+            raise BzrError('You must have one of BZR_HOME, APPDATA, or HOME set')
+        return pathjoin(base, 'bazaar', '2.0')
+    else:
+        # cygwin, linux, and darwin all have a $HOME directory
+        if base is None:
+            base = os.path.expanduser("~")
+        return pathjoin(base, ".bazaar")
+
+
+def config_filename():
+    """Return per-user configuration ini file filename."""
+    return pathjoin(config_dir(), 'bazaar.conf')
+
+
+def branches_config_filename():
+    """Return per-user configuration ini file filename."""
+    return pathjoin(config_dir(), 'branches.conf')
+
+
+def _auto_user_id():
+    """Calculate automatic user identification.
+
+    Returns (realname, email).
+
+    Only used when none is set in the environment or the id file.
+
+    This previously used the FQDN as the default domain, but that can
+    be very slow on machines where DNS is broken.  So now we simply
+    use the hostname.
+    """
+    import socket
+
+    # XXX: Any good way to get real user name on win32?
+
+    try:
+        import pwd
+        uid = os.getuid()
+        w = pwd.getpwuid(uid)
+        gecos = w.pw_gecos.decode(bzrlib.user_encoding)
+        username = w.pw_name.decode(bzrlib.user_encoding)
+        comma = gecos.find(',')
+        if comma == -1:
+            realname = gecos
+        else:
+            realname = gecos[:comma]
+        if not realname:
+            realname = username
+
+    except ImportError:
+        import getpass
+        realname = username = getpass.getuser().decode(bzrlib.user_encoding)
+
+    return realname, (username + '@' + socket.gethostname())
+
+
+def extract_email_address(e):
+    """Return just the address part of an email string.
+    
+    That is just the user@domain part, nothing else. 
+    This part is required to contain only ascii characters.
+    If it can't be extracted, raises an error.
+    
+    >>> extract_email_address('Jane Tester <jane@test.com>')
+    "jane@test.com"
+    """
+    m = re.search(r'[\w+.-]+@[\w+.-]+', e)
+    if not m:
+        raise errors.BzrError("%r doesn't seem to contain "
+                              "a reasonable email address" % e)
+    return m.group(0)
+
+class TreeConfig(object):
+    """Branch configuration data associated with its contents, not location"""
+    def __init__(self, branch):
+        self.branch = branch
+
+    def _get_config(self):
+        try:
+            obj = ConfigObj(self.branch.controlfile('branch.conf',
+                                                    'rb').readlines())
+            obj.decode('UTF-8')
+        except errors.NoSuchFile:
+            obj = ConfigObj()
+        return obj
+
+    def get_option(self, name, section=None, default=None):
+        self.branch.lock_read()
+        try:
+            obj = self._get_config()
+            try:
+                if section is not None:
+                    obj[section]
+                result = obj[name]
+            except KeyError:
+                result = default
+        finally:
+            self.branch.unlock()
+        return result
+
+    def set_option(self, value, name, section=None):
+        """Set a per-branch configuration option"""
+        self.branch.lock_write()
+        try:
+            cfg_obj = self._get_config()
+            if section is None:
+                obj = cfg_obj
+            else:
+                try:
+                    obj = cfg_obj[section]
+                except KeyError:
+                    cfg_obj[section] = {}
+                    obj = cfg_obj[section]
+            obj[name] = value
+            cfg_obj.encode('UTF-8')
+            out_file = StringIO(''.join([l+'\n' for l in cfg_obj.write()]))
+            out_file.seek(0)
+            self.branch.put_controlfile('branch.conf', out_file, encode=False)
+        finally:
+            self.branch.unlock()

=== renamed file 'bzrlib/plugins/conflicts.py' => 'bzrlib/conflicts.py'
--- bzrlib/plugins/conflicts.py	2005-10-06 04:20:44 +0000
+++ bzrlib/conflicts.py	2005-12-01 23:52:16 +0000
@@ -14,13 +14,23 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+# TODO: Move this into builtins
+
+# TODO: 'bzr resolve' should accept a directory name and work from that 
+# point down
+
+# TODO: bzr revert should resolve; even when reverting the whole tree
+# or particular directories
+
+import os
+import errno
+
 import bzrlib.status
 from bzrlib.branch import Branch
-from bzrlib.errors import BzrCommandError
+from bzrlib.errors import BzrCommandError, NotConflicted
 from bzrlib.commands import register_command
 from bzrlib.workingtree import CONFLICT_SUFFIXES
-import os
-import errno
+from bzrlib.osutils import rename
 
 class cmd_conflicts(bzrlib.commands.Command):
     """List files with conflicts.
@@ -28,14 +38,13 @@
     files.)
     """
     def run(self):
-        for path in Branch.open_containing('.').working_tree().iter_conflicts():
+        for path in Branch.open_containing(u'.')[0].working_tree().iter_conflicts():
             print path
 
-register_command(cmd_conflicts)
-
 class cmd_resolve(bzrlib.commands.Command):
     """Mark a conflict as resolved.
     """
+    aliases = ['resolved']
     takes_args = ['file*']
     takes_options = ['all']
     def run(self, file_list=None, all=False):
@@ -43,7 +52,7 @@
             if not all:
                 raise BzrCommandError(
                     "command 'resolve' needs one or more FILE, or --all")
-            tree = Branch.open_containing('.').working_tree()
+            tree = Branch.open_containing(u'.')[0].working_tree()
             file_list = list(tree.abspath(f) for f in tree.iter_conflicts())
         else:
             if all:
@@ -64,5 +73,30 @@
                     print "%s does not exist" % filename
                 else:
                     print "%s is not conflicted" % filename
-                        
-register_command(cmd_resolve)
+
+def restore(filename):
+    """\
+    Restore a conflicted file to the state it was in before merging.
+    Only text restoration supported at present.
+    """
+    conflicted = False
+    try:
+        rename(filename + ".THIS", filename)
+        conflicted = True
+    except OSError, e:
+        if e.errno != errno.ENOENT:
+            raise
+    try:
+        os.unlink(filename + ".BASE")
+        conflicted = True
+    except OSError, e:
+        if e.errno != errno.ENOENT:
+            raise
+    try:
+        os.unlink(filename + ".OTHER")
+        conflicted = True
+    except OSError, e:
+        if e.errno != errno.ENOENT:
+            raise
+    if not conflicted:
+        raise NotConflicted(filename)

=== modified file 'bzrlib/delta.py'
--- bzrlib/delta.py	2005-10-03 03:30:02 +0000
+++ bzrlib/delta.py	2005-11-16 13:46:41 +0000
@@ -114,10 +114,15 @@
             print >>to_file, 'added:'
             show_list(self.added)
 
+        extra_modified = []
+
         if self.renamed:
             print >>to_file, 'renamed:'
             for (oldpath, newpath, fid, kind,
                  text_modified, meta_modified) in self.renamed:
+                if text_modified or meta_modified:
+                    extra_modified.append((newpath, fid, kind,
+                                           text_modified, meta_modified))
                 if meta_modified:
                     newpath += '*'
                 if show_ids:
@@ -125,9 +130,10 @@
                 else:
                     print >>to_file, '  %s => %s' % (oldpath, newpath)
                     
-        if self.modified:
+        if self.modified or extra_modified:
             print >>to_file, 'modified:'
             show_list(self.modified)
+            show_list(extra_modified)
             
         if show_unchanged and self.unchanged:
             print >>to_file, 'unchanged:'
@@ -219,7 +225,7 @@
 
     mutter('start looking for new files')
     for file_id in new_inv:
-        if file_id in old_inv:
+        if file_id in old_inv or file_id not in new_tree:
             continue
         kind = new_inv.get_file_kind(file_id)
         if kind == 'root_directory':

=== modified file 'bzrlib/diff.py'
--- bzrlib/diff.py	2005-10-03 05:16:47 +0000
+++ bzrlib/diff.py	2005-11-11 17:46:11 +0000
@@ -22,7 +22,7 @@
 # invoke callbacks on an object.  That object can either accumulate a
 # list, write them out directly, etc etc.
 
-def internal_diff(old_label, oldlines, new_label, newlines, to_file):
+def internal_diff(old_filename, oldlines, new_filename, newlines, to_file):
     import difflib
     
     # FIXME: difflib is wrong if there is no trailing newline.
@@ -42,7 +42,8 @@
         return
 
     ud = difflib.unified_diff(oldlines, newlines,
-                              fromfile=old_label, tofile=new_label)
+                              fromfile=old_filename+'\t', 
+                              tofile=new_filename+'\t')
 
     ud = list(ud)
     # work-around for difflib being too smart for its own good
@@ -62,7 +63,7 @@
     print >>to_file
 
 
-def external_diff(old_label, oldlines, new_label, newlines, to_file,
+def external_diff(old_filename, oldlines, new_filename, newlines, to_file,
                   diff_opts):
     """Display a diff by calling out to the external diff program."""
     import sys
@@ -97,9 +98,9 @@
         if not diff_opts:
             diff_opts = []
         diffcmd = ['diff',
-                   '--label', old_label,
+                   '--label', old_filename+'\t',
                    oldtmpf.name,
-                   '--label', new_label,
+                   '--label', new_filename+'\t',
                    newtmpf.name]
 
         # diff only allows one style to be specified; they don't override.
@@ -140,15 +141,15 @@
         oldtmpf.close()                 # and delete
         newtmpf.close()
 
-def show_diff(b, revision, specific_files, external_diff_options=None,
-              revision2=None, output=None):
+def show_diff(b, from_spec, specific_files, external_diff_options=None,
+              revision2=None, output=None, b2=None):
     """Shortcut for showing the diff to the working tree.
 
     b
         Branch.
 
     revision
-        None for each, or otherwise the old revision to compare against.
+        None for 'basis tree', or otherwise the old revision to compare against.
     
     The more general form is show_diff_trees(), where the caller
     supplies any two trees.
@@ -157,18 +158,24 @@
         import sys
         output = sys.stdout
 
-    if revision is None:
-        old_tree = b.basis_tree()
+    if from_spec is None:
+        if b2 is None:
+            old_tree = b.basis_tree()
+        else:
+            old_tree = b.working_tree()
     else:
-        old_tree = b.revision_tree(revision.in_history(b).rev_id)
+        old_tree = b.revision_tree(from_spec.in_history(b).rev_id)
 
     if revision2 is None:
-        new_tree = b.working_tree()
+        if b2 is None:
+            new_tree = b.working_tree()
+        else:
+            new_tree = b2.working_tree()
     else:
         new_tree = b.revision_tree(revision2.in_history(b).rev_id)
 
-    show_diff_trees(old_tree, new_tree, output, specific_files,
-                    external_diff_options)
+    return show_diff_trees(old_tree, new_tree, output, specific_files,
+                           external_diff_options)
 
 
 
@@ -207,17 +214,21 @@
     delta = compare_trees(old_tree, new_tree, want_unchanged=False,
                           specific_files=specific_files)
 
+    has_changes = 0
     for path, file_id, kind in delta.removed:
+        has_changes = 1
         print >>to_file, '=== removed %s %r' % (kind, path)
         old_tree.inventory[file_id].diff(diff_file, old_label + path, old_tree,
                                          DEVNULL, None, None, to_file)
     for path, file_id, kind in delta.added:
+        has_changes = 1
         print >>to_file, '=== added %s %r' % (kind, path)
         new_tree.inventory[file_id].diff(diff_file, new_label + path, new_tree,
                                          DEVNULL, None, None, to_file, 
                                          reverse=True)
     for (old_path, new_path, file_id, kind,
          text_modified, meta_modified) in delta.renamed:
+        has_changes = 1
         prop_str = get_prop_change(meta_modified)
         print >>to_file, '=== renamed %s %r => %r%s' % (
                           kind, old_path, new_path, prop_str)
@@ -225,12 +236,14 @@
                                     new_label, new_path, new_tree,
                                     text_modified, kind, to_file, diff_file)
     for path, file_id, kind, text_modified, meta_modified in delta.modified:
+        has_changes = 1
         prop_str = get_prop_change(meta_modified)
         print >>to_file, '=== modified %s %r%s' % (kind, path, prop_str)
         if text_modified:
             _maybe_diff_file_or_symlink(old_label, path, old_tree, file_id,
                                         new_label, path, new_tree,
                                         True, kind, to_file, diff_file)
+    return has_changes
     
 
 def get_prop_change(meta_modified):

=== modified file 'bzrlib/errors.py'
--- bzrlib/errors.py	2005-10-06 13:08:14 +0000
+++ bzrlib/errors.py	2006-01-05 22:30:59 +0000
@@ -1,4 +1,4 @@
-# -*- coding: UTF-8 -*-
+# (C) 2005 Canonical
 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -14,13 +14,59 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-
-__copyright__ = "Copyright (C) 2005 Canonical Ltd."
-__author__ = "Martin Pool <mbp@canonical.com>"
-
-
-######################################################################
-# exceptions 
+"""Exceptions for bzr, and reporting of them.
+
+Exceptions are caught at a high level to report errors to the user, and
+might also be caught inside the program.  Therefore it needs to be
+possible to convert them to a meaningful string, and also for them to be
+interrogated by the program.
+
+Exceptions are defined such that the arguments given to the constructor
+are stored in the object as properties of the same name.  When the
+object is printed as a string, the doc string of the class is used as
+a format string with the property dictionary available to it.
+
+This means that exceptions can used like this:
+
+>>> import sys
+>>> try:
+...   raise NotBranchError(path='/foo/bar')
+... except:
+...   print sys.exc_type
+...   print sys.exc_value
+...   path = getattr(sys.exc_value, 'path')
+...   if path is not None:
+...     print path
+bzrlib.errors.NotBranchError
+Not a branch: /foo/bar
+/foo/bar
+
+Therefore:
+
+ * create a new exception class for any class of error that can be
+   usefully distinguished.
+
+ * the printable form of an exception is generated by the base class
+   __str__ method
+
+Exception strings should start with a capital letter and not have a final
+fullstop.
+"""
+
+# based on Scott James Remnant's hct error classes
+
+# TODO: is there any value in providing the .args field used by standard
+# python exceptions?   A list of values with no names seems less useful 
+# to me.
+
+# TODO: Perhaps convert the exception to a string at the moment it's 
+# constructed to make sure it will succeed.  But that says nothing about
+# exceptions that are never raised.
+
+# TODO: Convert all the other error classes here to BzrNewError, and eliminate
+# the old one.
+
+
 class BzrError(StandardError):
     def __str__(self):
         # XXX: Should we show the exception class in 
@@ -29,7 +75,7 @@
         ## n = self.__class__.__name__ + ': '
         n = ''
         if len(self.args) == 1:
-            return n + self.args[0]
+            return str(self.args[0])
         elif len(self.args) == 2:
             # further explanation or suggestions
             try:
@@ -40,63 +86,179 @@
             return n + `self.args`
 
 
-class BzrCheckError(BzrError):
-    pass
-
-
-class InvalidRevisionNumber(BzrError):
+class BzrNewError(BzrError):
+    """bzr error"""
+    # base classes should override the docstring with their human-
+    # readable explanation
+
+    def __init__(self, **kwds):
+        for key, value in kwds.items():
+            setattr(self, key, value)
+
     def __str__(self):
-        return 'invalid revision number: %r' % self.args[0]
-
-
-class InvalidRevisionId(BzrError):
-    pass
+        try:
+            return self.__doc__ % self.__dict__
+        except (NameError, ValueError, KeyError), e:
+            return 'Unprintable exception %s: %s' \
+                % (self.__class__.__name__, str(e))
+
+
+class BzrCheckError(BzrNewError):
+    """Internal check failed: %(message)s"""
+    def __init__(self, message):
+        BzrNewError.__init__(self)
+        self.message = message
+
+
+class InvalidEntryName(BzrNewError):
+    """Invalid entry name: %(name)s"""
+    def __init__(self, name):
+        BzrNewError.__init__(self)
+        self.name = name
+
+
+class InvalidRevisionNumber(BzrNewError):
+    """Invalid revision number %(revno)d"""
+    def __init__(self, revno):
+        BzrNewError.__init__(self)
+        self.revno = revno
+
+
+class InvalidRevisionId(BzrNewError):
+    """Invalid revision-id {%(revision_id)s} in %(branch)s"""
+    def __init__(self, revision_id, branch):
+        BzrNewError.__init__(self)
+        self.revision_id = revision_id
+        self.branch = branch
+
+
+class NoWorkingTree(BzrNewError):
+    """No WorkingTree exists for %s(base)."""
+    
+    def __init__(self, base):
+        BzrNewError.__init__(self)
+        self.base = base
 
 
 class BzrCommandError(BzrError):
     # Error from malformed user command
+    # This is being misused as a generic exception
+    # pleae subclass. RBC 20051030
+    #
+    # I think it's a waste of effort to differentiate between errors that
+    # are not intended to be caught anyway.  UI code need not subclass
+    # BzrCommandError, and non-UI code should not throw a subclass of
+    # BzrCommandError.  ADHB 20051211
     def __str__(self):
         return self.args[0]
 
 
-class NotBranchError(BzrError):
-    """Specified path is not in a branch"""
+class BzrOptionError(BzrCommandError):
+    """Some missing or otherwise incorrect option was supplied."""
+
+    
+class StrictCommitFailed(Exception):
+    """Commit refused because there are unknowns in the tree."""
+
+
+class PathError(BzrNewError):
+    """Generic path error: %(path)r%(extra)s)"""
+    def __init__(self, path, extra=None):
+        BzrNewError.__init__(self)
+        self.path = path
+        if extra:
+            self.extra = ': ' + str(extra)
+        else:
+            self.extra = ''
+
+
+class NoSuchFile(PathError):
+    """No such file: %(path)r%(extra)s"""
+
+
+class FileExists(PathError):
+    """File exists: %(path)r%(extra)s"""
+
+
+class PermissionDenied(PathError):
+    """Permission denied: %(path)r%(extra)s"""
+
+
+class PathNotChild(BzrNewError):
+    """Path %(path)r is not a child of path %(base)r%(extra)s"""
+    def __init__(self, path, base, extra=None):
+        BzrNewError.__init__(self)
+        self.path = path
+        self.base = base
+        if extra:
+            self.extra = ': ' + str(extra)
+        else:
+            self.extra = ''
+
+
+class NotBranchError(BzrNewError):
+    """Not a branch: %(path)s"""
+    def __init__(self, path):
+        BzrNewError.__init__(self)
+        self.path = path
+
+
+class FileInWrongBranch(BzrNewError):
+    """File %(path)s in not in branch %(branch_base)s."""
+    def __init__(self, branch, path):
+        BzrNewError.__init__(self)
+        self.branch = branch
+        self.branch_base = branch.base
+        self.path = path
+
+
+class UnsupportedFormatError(BzrError):
+    """Specified path is a bzr branch that we cannot read."""
     def __str__(self):
-        return 'not a branch: %s' % self.args[0]
-
-
-class NotVersionedError(BzrError):
-    """Specified object is not versioned."""
+        return 'unsupported branch format: %s' % self.args[0]
+
+
+class NotVersionedError(BzrNewError):
+    """%(path)s is not versioned"""
+    def __init__(self, path):
+        BzrNewError.__init__(self)
+        self.path = path
 
 
 class BadFileKindError(BzrError):
     """Specified file is of a kind that cannot be added.
 
     (For example a symlink or device file.)"""
-    pass
 
 
 class ForbiddenFileError(BzrError):
     """Cannot operate on a file because it is a control file."""
-    pass
 
 
 class LockError(Exception):
-    """All exceptions from the lock/unlock functions should be from
-    this exception class.  They will be translated as necessary. The
-    original exception is available as e.original_error
-    """
-    def __init__(self, e=None):
-        self.original_error = e
-        if e:
-            Exception.__init__(self, e)
-        else:
-            Exception.__init__(self)
-
-
-class PointlessCommit(Exception):
-    """Commit failed because nothing was changed."""
-
+    """Lock error"""
+    # All exceptions from the lock/unlock functions should be from
+    # this exception class.  They will be translated as necessary. The
+    # original exception is available as e.original_error
+
+
+class CommitNotPossible(LockError):
+    """A commit was attempted but we do not have a write lock open."""
+
+
+class AlreadyCommitted(LockError):
+    """A rollback was requested, but is not able to be accomplished."""
+
+
+class ReadOnlyError(LockError):
+    """A write attempt was made in a read only transaction."""
+
+
+class PointlessCommit(BzrNewError):
+    """No changes to commit"""
+
+class StrictCommitFailed(Exception):
+    """Commit refused because there are unknowns in the tree."""
 
 class NoSuchRevision(BzrError):
     def __init__(self, branch, revision):
@@ -116,7 +278,7 @@
 
 class DivergedBranches(BzrError):
     def __init__(self, branch1, branch2):
-        BzrError.__init__(self, "These branches have diverged.")
+        BzrError.__init__(self, "These branches have diverged.  Try merge.")
         self.branch1 = branch1
         self.branch2 = branch2
 
@@ -148,15 +310,6 @@
         self.not_ancestor_id = not_ancestor_id
 
 
-class NotAncestor(BzrError):
-    def __init__(self, rev_id, not_ancestor_id):
-        self.rev_id = rev_id
-        self.not_ancestor_id = not_ancestor_id
-        msg = "Revision %s is not an ancestor of %s" % (not_ancestor_id, 
-                                                        rev_id)
-        BzrError.__init__(self, msg)
-
-
 class InstallFailed(BzrError):
     def __init__(self, revisions):
         msg = "Could not install revisions:\n%s" % " ,".join(revisions)
@@ -185,7 +338,50 @@
         BzrError.__init__(self, "Stores for branch %s are not listable" % br)
 
 
-from bzrlib.weave import WeaveError
+class WeaveError(BzrNewError):
+    """Error in processing weave: %(message)s"""
+    def __init__(self, message=None):
+        BzrNewError.__init__(self)
+        self.message = message
+
+
+class WeaveRevisionAlreadyPresent(WeaveError):
+    """Revision {%(revision_id)s} already present in %(weave)s"""
+    def __init__(self, revision_id, weave):
+        WeaveError.__init__(self)
+        self.revision_id = revision_id
+        self.weave = weave
+
+
+class WeaveRevisionNotPresent(WeaveError):
+    """Revision {%(revision_id)s} not present in %(weave)s"""
+    def __init__(self, revision_id, weave):
+        WeaveError.__init__(self)
+        self.revision_id = revision_id
+        self.weave = weave
+
+
+class WeaveFormatError(WeaveError):
+    """Weave invariant violated: %(what)s"""
+    def __init__(self, what):
+        WeaveError.__init__(self)
+        self.what = what
+
+
+class WeaveParentMismatch(WeaveError):
+    """Parents are mismatched between two revisions."""
+    
+
+class WeaveInvalidChecksum(WeaveError):
+    """Text did not match it's checksum: %(message)s"""
+
+
+class NoSuchExportFormat(BzrNewError):
+    """Export format %(format)r not supported"""
+    def __init__(self, format):
+        BzrNewError.__init__(self)
+        self.format = format
+
 
 class TransportError(BzrError):
     """All errors thrown by Transport implementations should derive
@@ -205,40 +401,15 @@
     """
     pass
 
-class NonRelativePath(TransportError):
-    """An absolute path was supplied, that could not be decoded into
-    a relative path.
-    """
-    pass
-
-class NoSuchFile(TransportError, IOError):
-    """A get() was issued for a file that doesn't exist."""
-
-    # XXX: Is multiple inheritance for exceptions really needed?
-
-    def __str__(self):
-        return 'no such file: ' + self.msg
-
-    def __init__(self, msg=None, orig_error=None):
-        import errno
-        TransportError.__init__(self, msg=msg, orig_error=orig_error)
-        IOError.__init__(self, errno.ENOENT, self.msg)
-
-class FileExists(TransportError, OSError):
-    """An operation was attempted, which would overwrite an entry,
-    but overwritting is not supported.
-
-    mkdir() can throw this, but put() just overwites existing files.
-    """
-    # XXX: Is multiple inheritance for exceptions really needed?
-    def __init__(self, msg=None, orig_error=None):
-        import errno
-        TransportError.__init__(self, msg=msg, orig_error=orig_error)
-        OSError.__init__(self, errno.EEXIST, self.msg)
-
-class PermissionDenied(TransportError):
-    """An operation cannot succeed because of a lack of permissions."""
-    pass
+
+class ConnectionError(TransportError):
+    """A connection problem prevents file retrieval.
+    This does not indicate whether the file exists or not; it indicates that a
+    precondition for requesting the file was not met.
+    """
+    def __init__(self, msg=None, orig_error=None):
+        TransportError.__init__(self, msg=msg, orig_error=orig_error)
+
 
 class ConnectionReset(TransportError):
     """The connection has been closed."""
@@ -247,3 +418,65 @@
 class ConflictsInTree(BzrError):
     def __init__(self):
         BzrError.__init__(self, "Working tree has conflicts.")
+
+class ParseConfigError(BzrError):
+    def __init__(self, errors, filename):
+        if filename is None:
+            filename = ""
+        message = "Error(s) parsing config file %s:\n%s" % \
+            (filename, ('\n'.join(e.message for e in errors)))
+        BzrError.__init__(self, message)
+
+class SigningFailed(BzrError):
+    def __init__(self, command_line):
+        BzrError.__init__(self, "Failed to gpg sign data with command '%s'"
+                               % command_line)
+
+class WorkingTreeNotRevision(BzrError):
+    def __init__(self, tree):
+        BzrError.__init__(self, "The working tree for %s has changed since"
+                          " last commit, but weave merge requires that it be"
+                          " unchanged." % tree.basedir)
+
+class CantReprocessAndShowBase(BzrNewError):
+    """Can't reprocess and show base.
+Reprocessing obscures relationship of conflicting lines to base."""
+
+class GraphCycleError(BzrNewError):
+    """Cycle in graph %(graph)r"""
+    def __init__(self, graph):
+        BzrNewError.__init__(self)
+        self.graph = graph
+
+
+class NotConflicted(BzrNewError):
+    """File %(filename)s is not conflicted."""
+
+    def __init__(self, filename):
+        BzrNewError.__init__(self)
+        self.filename = filename
+
+
+class MustUseDecorated(Exception):
+    """A decorating function has requested its original command be used.
+    
+    This should never escape bzr, so does not need to be printable.
+    """
+
+class MissingText(BzrNewError):
+    """Branch %(base)s is missing revision %(text_revision)s of %(file_id)s"""
+
+    def __init__(self, branch, text_revision, file_id):
+        BzrNewError.__init__(self)
+        self.branch = branch
+        self.base = branch.base
+        self.text_revision = text_revision
+        self.file_id = file_id
+
+
+class BzrBadParameter(BzrNewError):
+    """Parameter %(param)s is neither unicode nor utf8."""
+    
+    def __init__(self, param):
+        BzrNewError.__init__(self)
+        self.param = param

=== added directory 'bzrlib/export'
=== added file 'bzrlib/export/__init__.py'
--- bzrlib/export/__init__.py	1970-01-01 00:00:00 +0000
+++ bzrlib/export/__init__.py	2005-11-15 00:09:39 +0000
@@ -0,0 +1,132 @@
+# Copyright (C) 2005 Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Export functionality, which can take a Tree and create a different representation.
+
+Such as non-controlled directories, tarfiles, zipfiles, etc.
+"""
+
+from bzrlib.trace import mutter
+import os
+import bzrlib.errors as errors
+
+# Maps format name => export function
+_exporters = {}
+# Maps filename extensions => export format name
+_exporter_extensions = {}
+
+def register_exporter(format, extensions, func, override=False):
+    """Register an exporter.
+
+    :param format: This is the name of the format, such as 'tgz' or 'zip'
+    :param extensions: Extensions which should be used in the case that a 
+                       format was not explicitly specified.
+    :type extensions: List
+    :param func: The function. It will be called with (tree, dest, root)
+    :param override: Whether to override an object which already exists.
+                     Frequently plugins will want to provide functionality
+                     until it shows up in mainline, so the default is False.
+    """
+    global _exporters, _exporter_extensions
+
+    if not _exporters.has_key(format) or override:
+        _exporters[format] = func
+
+    for ext in extensions:
+        if not _exporter_extensions.has_key(ext) or override:
+            _exporter_extensions[ext] = format
+
+
+def register_lazy_exporter(scheme, extensions, module, funcname):
+    """Register lazy-loaded exporter function.
+
+    When requesting a specific type of export, load the respective path.
+    """
+    def _loader(tree, dest, root):
+        mod = __import__(module, globals(), locals(), [funcname])
+        func = getattr(mod, funcname)
+        return func(tree, dest, root)
+    register_exporter(scheme, extensions, _loader)
+
+
+def export(tree, dest, format=None, root=None):
+    """Export the given Tree to the specific destination.
+
+    :param tree: A Tree (such as RevisionTree) to export
+    :param dest: The destination where the files,etc should be put
+    :param format: The format (dir, zip, etc), if None, it will check the
+                   extension on dest, looking for a match
+    :param root: The root location inside the format.
+                 It is common practise to have zipfiles and tarballs 
+                 extract into a subdirectory, rather than into the
+                 current working directory.
+                 If root is None, the default root will be
+                 selected as the destination without its
+                 extension.
+    """
+    global _exporters, _exporter_extensions
+
+    if format is None:
+        for ext in _exporter_extensions:
+            if dest.endswith(ext):
+                format = _exporter_extensions[ext]
+                break
+
+    # Most of the exporters will just have to call
+    # this function anyway, so why not do it for them
+    if root is None:
+        root = get_root_name(dest)
+
+    if not _exporters.has_key(format):
+        raise errors.NoSuchExportFormat(format)
+    return _exporters[format](tree, dest, root)
+
+
+def get_root_name(dest):
+    """Get just the root name for an export.
+
+    >>> get_root_name('mytar.tar')
+    'mytar'
+    >>> get_root_name('mytar.tar.bz2')
+    'mytar'
+    >>> get_root_name('tar.tar.tar.tgz')
+    'tar.tar.tar'
+    >>> get_root_name('bzr-0.0.5.tar.gz')
+    'bzr-0.0.5'
+    >>> get_root_name('bzr-0.0.5.zip')
+    'bzr-0.0.5'
+    >>> get_root_name('bzr-0.0.5')
+    'bzr-0.0.5'
+    >>> get_root_name('a/long/path/mytar.tgz')
+    'mytar'
+    >>> get_root_name('../parent/../dir/other.tbz2')
+    'other'
+    """
+    global _exporter_extensions
+    dest = os.path.basename(dest)
+    for ext in _exporter_extensions:
+        if dest.endswith(ext):
+            return dest[:-len(ext)]
+    return dest
+
+
+register_lazy_exporter(None, [], 'bzrlib.export.dir_exporter', 'dir_exporter')
+register_lazy_exporter('dir', [], 'bzrlib.export.dir_exporter', 'dir_exporter')
+register_lazy_exporter('tar', ['.tar'], 'bzrlib.export.tar_exporter', 'tar_exporter')
+register_lazy_exporter('tgz', ['.tar.gz', '.tgz'], 'bzrlib.export.tar_exporter', 'tgz_exporter')
+register_lazy_exporter('tbz2', ['.tar.bz2', '.tbz2'], 'bzrlib.export.tar_exporter', 'tbz_exporter')
+register_lazy_exporter('zip', ['.zip'], 'bzrlib.export.zip_exporter', 'zip_exporter')
+

=== added file 'bzrlib/export/dir_exporter.py'
--- bzrlib/export/dir_exporter.py	1970-01-01 00:00:00 +0000
+++ bzrlib/export/dir_exporter.py	2005-12-29 03:26:31 +0000
@@ -0,0 +1,46 @@
+# Copyright (C) 2005 Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Export a Tree to a non-versioned directory.
+"""
+
+import os
+from bzrlib.trace import mutter
+
+def dir_exporter(tree, dest, root):
+    """Export this tree to a new directory.
+
+    `dest` should not exist, and will be created holding the
+    contents of this tree.
+
+    TODO: To handle subdirectories we need to create the
+           directories first.
+
+    :note: If the export fails, the destination directory will be
+           left in a half-assed state.
+    """
+    import os
+    os.mkdir(dest)
+    mutter('export version %r', tree)
+    inv = tree.inventory
+    for dp, ie in inv.iter_entries():
+        # .bzrignore has no meaning outside of a working tree
+        # so do not export it
+        if dp == ".bzrignore":
+            continue
+        
+        ie.put_on_disk(dest, dp, tree)
+

=== added file 'bzrlib/export/tar_exporter.py'
--- bzrlib/export/tar_exporter.py	1970-01-01 00:00:00 +0000
+++ bzrlib/export/tar_exporter.py	2005-12-29 03:26:31 +0000
@@ -0,0 +1,60 @@
+# Copyright (C) 2005 Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Export a Tree to a non-versioned directory.
+"""
+
+import os
+from bzrlib.trace import mutter
+import tarfile
+
+
+def tar_exporter(tree, dest, root, compression=None):
+    """Export this tree to a new tar file.
+
+    `dest` will be created holding the contents of this tree; if it
+    already exists, it will be clobbered, like with "tar -c".
+    """
+    from time import time
+    now = time()
+    compression = str(compression or '')
+    if root is None:
+        root = get_root_name(dest)
+    try:
+        ball = tarfile.open(dest, 'w:' + compression)
+    except tarfile.CompressionError, e:
+        raise BzrError(str(e))
+    mutter('export version %r', tree)
+    inv = tree.inventory
+    for dp, ie in inv.iter_entries():
+        # .bzrignore has no meaning outside of a working tree
+        # so do not export it
+        if dp == ".bzrignore":
+            continue
+        
+        mutter("  export {%s} kind %s to %s", ie.file_id, ie.kind, dest)
+        item, fileobj = ie.get_tar_item(root, dp, now, tree)
+        ball.addfile(item, fileobj)
+    ball.close()
+
+
+def tgz_exporter(tree, dest, root):
+    tar_exporter(tree, dest, root, compression='gz')
+
+
+def tbz_exporter(tree, dest, root):
+    tar_exporter(tree, dest, root, compression='bz2')
+

=== added file 'bzrlib/export/zip_exporter.py'
--- bzrlib/export/zip_exporter.py	1970-01-01 00:00:00 +0000
+++ bzrlib/export/zip_exporter.py	2006-01-02 21:20:36 +0000
@@ -0,0 +1,76 @@
+# Copyright (C) 2005 Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Export a Tree to a non-versioned directory.
+"""
+
+import os
+from bzrlib.trace import mutter
+import zipfile
+
+def zip_exporter(tree, dest, root):
+    """ Export this tree to a new zip file.
+
+    `dest` will be created holding the contents of this tree; if it
+    already exists, it will be overwritten".
+    """
+    import time
+
+    now = time.localtime()[:6]
+    mutter('export version %r', tree)
+
+    compression = zipfile.ZIP_DEFLATED
+    zipf = zipfile.ZipFile(dest, "w", compression)
+
+    inv = tree.inventory
+
+    try:
+        for dp, ie in inv.iter_entries():
+            # .bzrignore has no meaning outside of a working tree
+            # so do not export it
+            if dp == ".bzrignore":
+                continue
+
+            file_id = ie.file_id
+            mutter("  export {%s} kind %s to %s", file_id, ie.kind, dest)
+
+            if ie.kind == "file": 
+                zinfo = zipfile.ZipInfo(
+                            filename=str(os.path.join(root, dp)),
+                            date_time=now)
+                zinfo.compress_type = compression
+                zipf.writestr(zinfo, tree.get_file_text(file_id))
+            elif ie.kind == "directory":
+                zinfo = zipfile.ZipInfo(
+                            filename=str(os.path.join(root, dp)+os.sep),
+                            date_time=now)
+                zinfo.compress_type = compression
+                zipf.writestr(zinfo,'')
+            elif ie.kind == "symlink":
+                zinfo = zipfile.ZipInfo(
+                            filename=str(os.path.join(root, dp+".lnk")),
+                            date_time=now)
+                zinfo.compress_type = compression
+                zipf.writestr(zinfo, ie.symlink_target)
+
+        zipf.close()
+
+    except UnicodeEncodeError:
+        zipf.close()
+        os.remove(dest)
+        from bzrlib.errors import BzrError
+        raise BzrError("Can't export non-ascii filenames to zip")
+

=== modified file 'bzrlib/externalcommand.py'
--- bzrlib/externalcommand.py	2005-09-26 08:56:15 +0000
+++ bzrlib/externalcommand.py	2005-12-01 23:52:16 +0000
@@ -23,6 +23,7 @@
 import sys
 
 from bzrlib.commands import Command
+from bzrlib.osutils import pathjoin
 
 
 class ExternalCommand(Command):
@@ -34,6 +35,11 @@
         bzrpath = os.environ.get('BZRPATH', '')
 
         for dir in bzrpath.split(os.pathsep):
+            ## Empty directories are not real paths
+            if not dir:
+                continue
+            # This needs to be os.path.join() or windows cannot
+            # find the batch file that you are wanting to execute
             path = os.path.join(dir, cmd)
             if os.path.isfile(path):
                 return ExternalCommand(path)
@@ -44,20 +50,17 @@
     def __init__(self, path):
         self.path = path
 
-
     def name(self):
-        return self.path.split(os.sep)[-1]
-
+        return os.path.basename(self.path)
 
     def run(self, *args, **kwargs):
         raise NotImplementedError('should not be called on %r' % self)
 
-
     def run_argv(self, argv):
         return os.spawnv(os.P_WAIT, self.path, [self.path] + argv)
 
-
     def help(self):
         m = 'external command from %s\n\n' % self.path
         pipe = os.popen('%s --help' % self.path)
         return m + pipe.read()
+

=== modified file 'bzrlib/fetch.py'
--- bzrlib/fetch.py	2005-10-06 06:31:33 +0000
+++ bzrlib/fetch.py	2005-12-05 03:24:19 +0000
@@ -14,16 +14,19 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+from copy import copy
 import os
 from cStringIO import StringIO
 
-import bzrlib.errors
+import bzrlib
+import bzrlib.errors as errors
+from bzrlib.errors import (InstallFailed, NoSuchRevision, WeaveError,
+                           MissingText)
 from bzrlib.trace import mutter, note, warning
 from bzrlib.branch import Branch
 from bzrlib.progress import ProgressBar
 from bzrlib.xml5 import serializer_v5
 from bzrlib.osutils import sha_string, split_lines
-from bzrlib.errors import InstallFailed, NoSuchRevision, WeaveError
 
 """Copying of history from one branch to another.
 
@@ -98,16 +101,24 @@
         self.count_total = 0
         self.count_weaves = 0
         self.copied_file_ids = set()
+        self.file_ids_names = {}
         if pb is None:
             self.pb = bzrlib.ui.ui_factory.progress_bar()
         else:
             self.pb = pb
+        self.from_branch.lock_read()
         try:
-            self.last_revision = self._find_last_revision(last_revision)
-        except NoSuchRevision, e:
-            mutter('failed getting last revision: %s', e)
-            raise InstallFailed([last_revision])
+            self._fetch_revisions(last_revision)
+        finally:
+            self.from_branch.unlock()
+            self.pb.clear()
+
+    def _fetch_revisions(self, last_revision):
+        self.last_revision = self._find_last_revision(last_revision)
         mutter('fetch up to rev {%s}', self.last_revision)
+        if (self.last_revision is not None and 
+            self.to_branch.has_revision(self.last_revision)):
+            return
         try:
             revs_to_fetch = self._compare_ancestries()
         except WeaveError:
@@ -115,7 +126,6 @@
         self._copy_revisions(revs_to_fetch)
         self.new_ancestry = revs_to_fetch
 
-
     def _find_last_revision(self, last_revision):
         """Find the limiting source revision.
 
@@ -123,13 +133,12 @@
 
         Returns the revision_id, or returns None if there's no history
         in the source branch."""
+        if last_revision:
+            return last_revision
         self.pb.update('get source history')
         from_history = self.from_branch.revision_history()
         self.pb.update('get destination history')
-        if last_revision:
-            self.from_branch.get_revision(last_revision)
-            return last_revision
-        elif from_history:
+        if from_history:
             return from_history[-1]
         else:
             return None                 # no history in the source branch
@@ -167,7 +176,7 @@
                 continue
             if self.to_branch.has_revision(rev_id):
                 continue
-            self.pb.update('fetch revision', i, self.count_total)
+            self.pb.update('copy revision', i, self.count_total)
             self._copy_one_revision(rev_id)
             self.count_copied += 1
 
@@ -186,17 +195,18 @@
                len(rev.parent_ids))
         self._copy_new_texts(rev_id, inv)
         parents = rev.parent_ids
+        new_parents = copy(parents)
         for parent in parents:
             if not self.to_branch.has_revision(parent):
-                parents.pop(parents.index(parent))
-        self._copy_inventory(rev_id, inv_xml, parents)
+                new_parents.pop(new_parents.index(parent))
+        self._copy_inventory(rev_id, inv_xml, new_parents)
         self.to_branch.revision_store.add(StringIO(rev_xml), rev_id)
         mutter('copied revision %s', rev_id)
 
-
     def _copy_inventory(self, rev_id, inv_xml, parent_ids):
         self.to_control.add_text('inventory', rev_id,
-                                split_lines(inv_xml), parent_ids)
+                                split_lines(inv_xml), parent_ids,
+                                self.to_branch.get_transaction())
 
     def _copy_new_texts(self, rev_id, inv):
         """Copy any new texts occuring in this revision."""
@@ -204,25 +214,40 @@
         # in memory until everything's done?  But this way is nicer
         # if it's interrupted.
         for path, ie in inv.iter_entries():
-            if ie.revision != rev_id:
-                continue
-            mutter('%s {%s} is changed in this revision',
-                   path, ie.file_id)
-            self._copy_one_weave(rev_id, ie.file_id)
-
-
-    def _copy_one_weave(self, rev_id, file_id):
-        """Copy one file weave."""
+            self._copy_one_weave(rev_id, ie.file_id, ie.revision)
+
+    def _copy_one_weave(self, rev_id, file_id, text_revision):
+        """Copy one file weave, esuring the result contains text_revision."""
+        # check if the revision is already there
+        if file_id in self.file_ids_names.keys( ) and \
+            text_revision in self.file_ids_names[file_id]:
+                return        
+        to_weave = self.to_weaves.get_weave_or_empty(file_id,
+            self.to_branch.get_transaction())
+        if not file_id in self.file_ids_names.keys( ):
+            self.file_ids_names[file_id] = to_weave.names( )
+        if text_revision in to_weave:
+            return
+        from_weave = self.from_weaves.get_weave(file_id,
+            self.from_branch.get_transaction())
+        if text_revision not in from_weave:
+            raise MissingText(self.from_branch, text_revision, file_id)
         mutter('copy file {%s} modified in {%s}', file_id, rev_id)
-        if file_id in self.copied_file_ids:
-            mutter('file {%s} already copied', file_id)
-            return
-        from_weave = self.from_weaves.get_weave(file_id)
-        to_weave = self.to_weaves.get_weave_or_empty(file_id)
-        to_weave.join(from_weave)
-        self.to_weaves.put_weave(file_id, to_weave)
+
+        if to_weave.numversions() > 0:
+            # destination has contents, must merge
+            try:
+                to_weave.join(from_weave)
+            except errors.WeaveParentMismatch:
+                to_weave.reweave(from_weave)
+        else:
+            # destination is empty, just replace it
+            to_weave = from_weave.copy( )
+        self.to_weaves.put_weave(file_id, to_weave,
+            self.to_branch.get_transaction())
         self.count_weaves += 1
         self.copied_file_ids.add(file_id)
+        self.file_ids_names[file_id] = to_weave.names()
         mutter('copied file {%s}', file_id)
 
 

=== added file 'bzrlib/gpg.py'
--- bzrlib/gpg.py	1970-01-01 00:00:00 +0000
+++ bzrlib/gpg.py	2005-10-17 21:57:32 +0000
@@ -0,0 +1,79 @@
+# Copyright (C) 2005 by Canonical Ltd
+#   Authors: Robert Collins <robert.collins@canonical.com>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""GPG signing and checking logic."""
+
+import errno
+import subprocess
+
+import bzrlib.errors as errors
+
+class DisabledGPGStrategy(object):
+    """A GPG Strategy that makes everything fail."""
+
+    def __init__(self, ignored):
+        """Real strategies take a configuration."""
+
+    def sign(self, content):
+        raise errors.SigningFailed('Signing is disabled.')
+
+
+class LoopbackGPGStrategy(object):
+    """A GPG Strategy that acts like 'cat' - data is just passed through."""
+
+    def __init__(self, ignored):
+        """Real strategies take a configuration."""
+
+    def sign(self, content):
+        return content
+
+
+class GPGStrategy(object):
+    """GPG Signing and checking facilities."""
+        
+    def _command_line(self):
+        return [self._config.gpg_signing_command(), '--clearsign']
+
+    def __init__(self, config):
+        self._config = config
+
+    def sign(self, content):
+        try:
+            process = subprocess.Popen(self._command_line(),
+                                       stdin=subprocess.PIPE,
+                                       stdout=subprocess.PIPE)
+            try:
+                result = process.communicate(content)[0]
+                if process.returncode is None:
+                    process.wait()
+                if process.returncode != 0:
+                    raise errors.SigningFailed(self._command_line())
+                return result
+            except OSError, e:
+                if e.errno == errno.EPIPE:
+                    raise errors.SigningFailed(self._command_line())
+                else:
+                    raise
+        except ValueError:
+            # bad subprocess parameters, should never happen.
+            raise
+        except OSError, e:
+            if e.errno == errno.ENOENT:
+                # gpg is not installed
+                raise errors.SigningFailed(self._command_line())
+            else:
+                raise

=== modified file 'bzrlib/hashcache.py'
--- bzrlib/hashcache.py	2005-09-30 02:54:51 +0000
+++ bzrlib/hashcache.py	2006-01-11 19:58:59 +0000
@@ -32,11 +32,14 @@
 import os, stat, time
 import sha
 
-from bzrlib.osutils import sha_file
+from bzrlib.osutils import sha_file, pathjoin
 from bzrlib.trace import mutter, warning
 from bzrlib.atomicfile import AtomicFile
-
-
+from bzrlib.errors import BzrError
+
+
+FP_MTIME_COLUMN = 1
+FP_CTIME_COLUMN = 2
 FP_MODE_COLUMN = 5
 
 def _fingerprint(abspath):
@@ -101,12 +104,10 @@
         self.update_count = 0
         self._cache = {}
 
-
     def cache_file_name(self):
-        return os.sep.join([self.basedir, '.bzr', 'stat-cache'])
-
-
-
+        # FIXME: duplicate path logic here, this should be 
+        # something like 'branch.controlfile'.
+        return pathjoin(self.basedir, '.bzr', 'stat-cache')
 
     def clear(self):
         """Discard all cached information.
@@ -127,7 +128,7 @@
         prep.sort()
         
         for inum, path, cache_entry in prep:
-            abspath = os.sep.join([self.basedir, path])
+            abspath = pathjoin(self.basedir, path)
             fp = _fingerprint(abspath)
             self.stat_count += 1
             
@@ -143,7 +144,7 @@
     def get_sha1(self, path):
         """Return the sha1 of a file.
         """
-        abspath = os.sep.join([self.basedir, path])
+        abspath = pathjoin(self.basedir, path)
         self.stat_count += 1
         file_fp = _fingerprint(abspath)
         
@@ -171,16 +172,23 @@
         if stat.S_ISREG(mode):
             digest = sha_file(file(abspath, 'rb', buffering=65000))
         elif stat.S_ISLNK(mode):
-            link_target = os.readlink(abspath)
             digest = sha.new(os.readlink(abspath)).hexdigest()
         else:
             raise BzrError("file %r: unknown file stat mode: %o"%(abspath,mode))
 
         now = int(time.time())
-        if file_fp[1] >= now or file_fp[2] >= now:
+        if file_fp[FP_MTIME_COLUMN] >= now or file_fp[FP_CTIME_COLUMN] >= now:
             # changed too recently; can't be cached.  we can
             # return the result and it could possibly be cached
             # next time.
+            #
+            # the point is that we only want to cache when we are sure that any
+            # subsequent modifications of the file can be detected.  If a
+            # modification neither changes the inode, the device, the size, nor
+            # the mode, then we can only distinguish it by time; therefore we
+            # need to let sufficient time elapse before we may cache this entry
+            # again.  If we didn't do this, then, for example, a very quick 1
+            # byte replacement in the file might go undetected.
             self.danger_count += 1 
             if cache_fp:
                 self.removed_count += 1
@@ -212,8 +220,6 @@
         finally:
             if not outf.closed:
                 outf.abort()
-        
-
 
     def read(self):
         """Reinstate cache from file.
@@ -228,7 +234,7 @@
         try:
             inf = file(fn, 'rb', buffering=65000)
         except IOError, e:
-            mutter("failed to open %s: %s" % (fn, e))
+            mutter("failed to open %s: %s", fn, e)
             # better write it now so it is valid
             self.needs_write = True
             return
@@ -236,8 +242,8 @@
 
         hdr = inf.readline()
         if hdr != CACHE_HEADER:
-            mutter('cache header marker not found at top of %s; discarding cache'
-                   % fn)
+            mutter('cache header marker not found at top of %s;'
+                   ' discarding cache', fn)
             self.needs_write = True
             return
 

=== modified file 'bzrlib/help.py'
--- bzrlib/help.py	2005-09-16 14:57:59 +0000
+++ bzrlib/help.py	2005-11-18 08:10:07 +0000
@@ -20,6 +20,7 @@
 # executable files with reasonable names.
 
 # TODO: `help commands --all` should show hidden commands
+import textwrap
 
 global_help = \
 """Bazaar-NG -- a free distributed version-control tool
@@ -112,27 +113,30 @@
     outfile.write(doc)
     if doc[-1] != '\n':
         outfile.write('\n')
-    
-    help_on_options(cmd_object.takes_options, outfile=None)
-
-
-def help_on_options(options, outfile=None):
-    from bzrlib.commands import SHORT_OPTIONS
-    
+    help_on_command_options(cmd_object, outfile)
+
+
+def help_on_command_options(cmd, outfile=None):
+    from bzrlib.option import Option
+    options = cmd.options()
     if not options:
         return
-    
     if outfile == None:
         outfile = sys.stdout
-
     outfile.write('\noptions:\n')
-    for on in options:
-        l = '    --' + on
-        for shortname, longname in SHORT_OPTIONS.items():
-            if longname == on:
-                l += ', -' + shortname
-                break
-        outfile.write(l + '\n')
+    for option_name, option in sorted(options.items()):
+        l = '    --' + option_name
+        if option.type is not None:
+            l += ' ' + option.argname.upper()
+        short_name = option.short_name()
+        if short_name:
+            assert len(short_name) == 1
+            l += ', -' + short_name
+        l += (30 - len(l)) * ' ' + option.help
+        # TODO: split help over multiple lines with correct indenting and 
+        # wrapping
+        wrapped = textwrap.fill(l, initial_indent='', subsequent_indent=30*' ')
+        outfile.write(wrapped + '\n')
 
 
 def help_commands(outfile=None):

=== added file 'bzrlib/identitymap.py'
--- bzrlib/identitymap.py	1970-01-01 00:00:00 +0000
+++ bzrlib/identitymap.py	2005-10-11 02:41:25 +0000
@@ -0,0 +1,96 @@
+# Copyright (C) 2005 by Canonical Ltd
+#   Authors: Robert Collins <robert.collins@canonical.com>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""This module provides an IdentityMap."""
+
+
+import bzrlib.errors as errors
+
+
+class IdentityMap(object):
+    """An in memory map from object id to instance.
+    
+    An IdentityMap maps from keys to single instances of objects in memory.
+    We have explicit calls on the map for the root of each inheritance tree
+    that is store in the map. Look for find_CLASS and add_CLASS methods.
+    """
+
+    def add_revision_history(self, revision_history):
+        """Add a revision_history object to the map.
+
+        There can only be one!
+        """
+        if self._revision_history is not None:
+            raise errors.BzrError("A revision history (%s) is already "
+                                  "identity map" % self._revision_history)
+        self._revision_history = revision_history
+
+    def add_weave(self, id, weave):
+        """Add weave to the map with a given id."""
+        if self._weave_key(id) in self._map:
+            raise errors.BzrError('weave %s already in the identity map' % id)
+        self._map[self._weave_key(id)] = weave
+        self._reverse_map[weave] = self._weave_key(id)
+
+    def find_revision_history(self):
+        return self._revision_history
+
+    def find_weave(self, id):
+        """Return the weave for 'id', or None if it is not present."""
+        return self._map.get(self._weave_key(id), None)
+
+    def __init__(self):
+        super(IdentityMap, self).__init__()
+        self._map = {}
+        self._reverse_map = {}
+        self._revision_history = None
+
+    def remove_object(self, an_object):
+        """Remove object from map."""
+        if isinstance(an_object, list):
+            if self._revision_history is an_object:
+                self._revision_history = None
+            else:
+                raise KeyError('%r not in identity map' % an_object)
+        else:
+            self._map.pop(self._reverse_map[an_object])
+            self._reverse_map.pop(an_object)
+        
+
+    def _weave_key(self, id):
+        """Return the key for a weaves id."""
+        return "weave-" + id
+
+        
+class NullIdentityMap(object):
+    """A pretend in memory map from object id to instance.
+    
+    A NullIdentityMap is an Identity map that does not store anything in it.
+    """
+
+    def add_weave(self, id, weave):
+        """See IdentityMap.add_weave."""
+
+    def add_revision_history(self, revision_history):
+        """See IdentityMap.add_revision_history."""
+
+    def find_weave(self, id):
+        """See IdentityMap.find_weave."""
+        return None
+
+    def find_revision_history(self):
+        """See IdentityMap.find_revision_history."""

=== modified file 'bzrlib/intset.py'
--- bzrlib/intset.py	2005-09-15 13:32:36 +0000
+++ bzrlib/intset.py	2005-12-19 17:48:25 +0000
@@ -208,3 +208,22 @@
         if not self._val & m:
             raise KeyError(to_remove)
         self._val ^= m
+
+    def set_remove(self, to_remove):
+        """Remove all values that exist in to_remove.
+
+        >>> a = IntSet(range(10))
+        >>> b = IntSet([2,3,4,7,12])
+        >>> a.set_remove(b)
+        >>> list(a)
+        [0, 1, 5, 6, 8, 9]
+        >>> a.set_remove([1,2,5])
+        >>> list(a)
+        [0, 6, 8, 9]
+        """
+        if not isinstance(to_remove, IntSet):
+            self.set_remove(IntSet(to_remove))
+            return
+        intersect = self._val & to_remove._val
+        self._val ^= intersect
+

=== modified file 'bzrlib/inventory.py'
--- bzrlib/inventory.py	2005-10-06 02:51:02 +0000
+++ bzrlib/inventory.py	2005-12-19 20:31:01 +0000
@@ -1,19 +1,23 @@
 # (C) 2005 Canonical Ltd
-
+#
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
-
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-
+#
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+# FIXME: This refactoring of the workingtree code doesn't seem to keep 
+# the WorkingTree's copy of the inventory in sync with the branch.  The
+# branch modifies its working inventory when it does a commit to make
+# missing files permanently removed.
 
 # TODO: Maybe also keep the full path of the entry, and the children?
 # But those depend on its position within a particular inventory, and
@@ -31,12 +35,11 @@
 import types
 
 import bzrlib
-from bzrlib.errors import BzrError, BzrCheckError
-
 from bzrlib.osutils import (pumpfile, quotefn, splitpath, joinpath,
-                            appendpath, sha_strings)
+                            pathjoin, sha_strings)
 from bzrlib.trace import mutter
-from bzrlib.errors import NotVersionedError
+from bzrlib.errors import (NotVersionedError, InvalidEntryName,
+                           BzrError, BzrCheckError)
 
 
 class InventoryEntry(object):
@@ -76,11 +79,12 @@
     InventoryDirectory('123', 'src', parent_id='TREE_ROOT')
     >>> i.add(InventoryFile('2323', 'hello.c', parent_id='123'))
     InventoryFile('2323', 'hello.c', parent_id='123')
-    >>> for j in i.iter_entries():
-    ...   print j
+    >>> shouldbe = {0: 'src', 1: pathjoin('src','hello.c')}
+    >>> for ix, j in enumerate(i.iter_entries()):
+    ...   print (j[0] == shouldbe[ix], j[1])
     ... 
-    ('src', InventoryDirectory('123', 'src', parent_id='TREE_ROOT'))
-    ('src/hello.c', InventoryFile('2323', 'hello.c', parent_id='123'))
+    (True, InventoryDirectory('123', 'src', parent_id='TREE_ROOT'))
+    (True, InventoryFile('2323', 'hello.c', parent_id='123'))
     >>> i.add(InventoryFile('2323', 'bye.c', '123'))
     Traceback (most recent call last):
     ...
@@ -98,7 +102,7 @@
     >>> i['2326']
     InventoryFile('2326', 'wibble.c', parent_id='2325')
     >>> for path, entry in i.iter_entries():
-    ...     print path.replace('\\\\', '/')     # for win32 os.sep
+    ...     print path
     ...     assert i.path2id(path)
     ... 
     src
@@ -106,7 +110,7 @@
     src/hello.c
     src/wibble
     src/wibble/wibble.c
-    >>> i.id2path('2326').replace('\\\\', '/')
+    >>> i.id2path('2326')
     'src/wibble/wibble.c'
     """
     
@@ -114,8 +118,9 @@
                  'text_id', 'parent_id', 'children', 'executable', 
                  'revision']
 
-    def _add_text_to_weave(self, new_lines, parents, weave_store):
-        weave_store.add_text(self.file_id, self.revision, new_lines, parents)
+    def _add_text_to_weave(self, new_lines, parents, weave_store, transaction):
+        weave_store.add_text(self.file_id, self.revision, new_lines, parents,
+                             transaction)
 
     def detect_changes(self, old_entry):
         """Return a (text_modified, meta_modified) from this to old_entry.
@@ -165,6 +170,14 @@
                 ie = inv[self.file_id]
                 assert ie.file_id == self.file_id
                 if ie.revision in heads:
+                    # fixup logic, there was a bug in revision updates.
+                    # with x bit support.
+                    try:
+                        if heads[ie.revision].executable != ie.executable:
+                            heads[ie.revision].executable = False
+                            ie.executable = False
+                    except AttributeError:
+                        pass
                     assert heads[ie.revision] == ie
                 else:
                     # may want to add it.
@@ -189,7 +202,7 @@
 
     def get_tar_item(self, root, dp, now, tree):
         """Get a tarfile item and a file stream for its content."""
-        item = tarfile.TarInfo(os.path.join(root, dp))
+        item = tarfile.TarInfo(pathjoin(root, dp))
         # TODO: would be cool to actually set it to the timestamp of the
         # revision it was last changed
         item.mtime = now
@@ -220,12 +233,11 @@
         '123'
         >>> e = InventoryFile('123', 'src/hello.c', ROOT_ID)
         Traceback (most recent call last):
-        BzrCheckError: InventoryEntry name 'src/hello.c' is invalid
+        InvalidEntryName: Invalid entry name: src/hello.c
         """
         assert isinstance(name, basestring), name
         if '/' in name or '\\' in name:
-            raise BzrCheckError('InventoryEntry name %r is invalid' % name)
-        
+            raise InvalidEntryName(name=name)
         self.executable = False
         self.revision = None
         self.text_sha1 = None
@@ -255,9 +267,10 @@
         
         This is a template method - implement _put_on_disk in subclasses.
         """
-        fullpath = appendpath(dest, dp)
+        fullpath = pathjoin(dest, dp)
         self._put_on_disk(fullpath, tree)
-        mutter("  export {%s} kind %s to %s" % (self.file_id, self.kind, fullpath))
+        mutter("  export {%s} kind %s to %s", self.file_id,
+                self.kind, fullpath)
 
     def _put_on_disk(self, fullpath, tree):
         """Put this entry onto disk at fullpath, from tree tree."""
@@ -310,7 +323,7 @@
                    self.parent_id))
 
     def snapshot(self, revision, path, previous_entries,
-                 work_tree, weave_store):
+                 work_tree, weave_store, transaction):
         """Make a snapshot of this entry which may or may not have changed.
         
         This means that all its fields are populated, that it has its
@@ -326,25 +339,26 @@
                 self.revision = parent_ie.revision
                 return "unchanged"
         return self.snapshot_revision(revision, previous_entries, 
-                                      work_tree, weave_store)
+                                      work_tree, weave_store, transaction)
 
     def snapshot_revision(self, revision, previous_entries, work_tree,
-                          weave_store):
+                          weave_store, transaction):
         """Record this revision unconditionally."""
         mutter('new revision for {%s}', self.file_id)
         self.revision = revision
         change = self._get_snapshot_change(previous_entries)
-        self._snapshot_text(previous_entries, work_tree, weave_store)
+        self._snapshot_text(previous_entries, work_tree, weave_store,
+                            transaction)
         return change
 
-    def _snapshot_text(self, file_parents, work_tree, weave_store): 
+    def _snapshot_text(self, file_parents, work_tree, weave_store, transaction): 
         """Record the 'text' of this entry, whatever form that takes.
         
         This default implementation simply adds an empty text.
         """
         mutter('storing file {%s} in revision {%s}',
                self.file_id, self.revision)
-        self._add_text_to_weave([], file_parents, weave_store)
+        self._add_text_to_weave([], file_parents, weave_store, transaction)
 
     def __eq__(self, other):
         if not isinstance(other, InventoryEntry):
@@ -388,6 +402,10 @@
         Note that this should be modified to be a noop on virtual trees
         as all entries created there are prepopulated.
         """
+        # TODO: Rather than running this manually, we should check the 
+        # working sha1 and other expensive properties when they're
+        # first requested, or preload them if they're already known
+        pass            # nothing to do by default
 
 
 class RootEntry(InventoryEntry):
@@ -400,7 +418,7 @@
         self.children = {}
         self.kind = 'root_directory'
         self.parent_id = None
-        self.name = ''
+        self.name = u''
 
     def __eq__(self, other):
         if not isinstance(other, RootEntry):
@@ -464,13 +482,26 @@
             else:
                 checker.repeated_text_cnt += 1
                 return
+
+        if self.file_id not in checker.checked_weaves:
+            mutter('check weave {%s}', self.file_id)
+            w = tree.get_weave(self.file_id)
+            # Not passing a progress bar, because it creates a new
+            # progress, which overwrites the current progress,
+            # and doesn't look nice
+            w.check()
+            checker.checked_weaves[self.file_id] = True
+        else:
+            w = tree.get_weave_prelude(self.file_id)
+
         mutter('check version {%s} of {%s}', rev_id, self.file_id)
-        file_lines = tree.get_file_lines(self.file_id)
         checker.checked_text_cnt += 1 
-        if self.text_size != sum(map(len, file_lines)):
-            raise BzrCheckError('text {%s} wrong size' % self.text_id)
-        if self.text_sha1 != sha_strings(file_lines):
-            raise BzrCheckError('text {%s} wrong sha1' % self.text_id)
+        # We can't check the length, because Weave doesn't store that
+        # information, and the whole point of looking at the weave's
+        # sha1sum is that we don't have to extract the text.
+        if self.text_sha1 != w.get_sha1(self.revision):
+            raise BzrCheckError('text {%s} version {%s} wrong sha1' 
+                                % (self.file_id, self.revision))
         checker.checked_texts[t] = self.text_sha1
 
     def copy(self):
@@ -539,7 +570,7 @@
         self.text_sha1 = work_tree.get_file_sha1(self.file_id)
         self.executable = work_tree.is_executable(self.file_id)
 
-    def _snapshot_text(self, file_parents, work_tree, weave_store): 
+    def _snapshot_text(self, file_parents, work_tree, weave_store, transaction):
         """See InventoryEntry._snapshot_text."""
         mutter('storing file {%s} in revision {%s}',
                self.file_id, self.revision)
@@ -551,10 +582,11 @@
             previous_ie = file_parents.values()[0]
             weave_store.add_identical_text(
                 self.file_id, previous_ie.revision, 
-                self.revision, file_parents)
+                self.revision, file_parents, transaction)
         else:
             new_lines = work_tree.get_file(self.file_id).readlines()
-            self._add_text_to_weave(new_lines, file_parents, weave_store)
+            self._add_text_to_weave(new_lines, file_parents, weave_store,
+                                    transaction)
             self.text_sha1 = sha_strings(new_lines)
             self.text_size = sum(map(len, new_lines))
 
@@ -568,6 +600,8 @@
             # FIXME: 20050930 probe for the text size when getting sha1
             # in _read_tree_state
             self.text_size = previous_ie.text_size
+        if self.executable != previous_ie.executable:
+            compatible = False
         return compatible
 
 
@@ -627,7 +661,7 @@
 
     def _put_in_tar(self, item, tree):
         """See InventoryEntry._put_in_tar."""
-        iterm.type = tarfile.SYMTYPE
+        item.type = tarfile.SYMTYPE
         fileobj = None
         item.size = 0
         item.mode = 0755
@@ -743,7 +777,7 @@
             yield name, ie
             if ie.kind == 'directory':
                 for cn, cie in self.iter_entries(from_dir=ie.file_id):
-                    yield os.path.join(name, cn), cie
+                    yield pathjoin(name, cn), cie
 
 
     def entries(self):
@@ -756,12 +790,12 @@
             kids = dir_ie.children.items()
             kids.sort()
             for name, ie in kids:
-                child_path = os.path.join(dir_path, name)
+                child_path = pathjoin(dir_path, name)
                 accum.append((child_path, ie))
                 if ie.kind == 'directory':
                     descend(ie, child_path)
 
-        descend(self.root, '')
+        descend(self.root, u'')
         return accum
 
 
@@ -776,9 +810,9 @@
             kids.sort()
 
             for name, child_ie in kids:
-                child_path = os.path.join(parent_path, name)
+                child_path = pathjoin(parent_path, name)
                 descend(child_ie, child_path)
-        descend(self.root, '')
+        descend(self.root, u'')
         return accum
         
 
@@ -843,7 +877,7 @@
 
         if parent.children.has_key(entry.name):
             raise BzrError("%s is already versioned" %
-                    appendpath(self.id2path(parent.file_id), entry.name))
+                    pathjoin(self.id2path(parent.file_id), entry.name))
 
         self._byid[entry.file_id] = entry
         parent.children[entry.name] = entry
@@ -856,7 +890,7 @@
         The immediate parent must already be versioned.
 
         Returns the new entry object."""
-        from bzrlib.branch import gen_file_id
+        from bzrlib.workingtree import gen_file_id
         
         parts = bzrlib.osutils.splitpath(relpath)
         if len(parts) == 0:
@@ -868,8 +902,7 @@
         parent_path = parts[:-1]
         parent_id = self.path2id(parent_path)
         if parent_id == None:
-            raise NotVersionedError(parent_path)
-
+            raise NotVersionedError(path=parent_path)
         if kind == 'directory':
             ie = InventoryDirectory(file_id, parts[-1], parent_id)
         elif kind == 'file':
@@ -967,12 +1000,15 @@
         >>> i = Inventory()
         >>> e = i.add(InventoryDirectory('src-id', 'src', ROOT_ID))
         >>> e = i.add(InventoryFile('foo-id', 'foo.c', parent_id='src-id'))
-        >>> print i.id2path('foo-id').replace(os.sep, '/')
+        >>> print i.id2path('foo-id')
         src/foo.c
         """
         # get all names, skipping root
         p = [self._byid[fid].name for fid in self.get_idpath(file_id)[1:]]
-        return os.sep.join(p)
+        if p:
+            return pathjoin(*p)
+        else:
+            return ''
             
 
 

=== modified file 'bzrlib/lock.py'
--- bzrlib/lock.py	2005-09-23 12:49:00 +0000
+++ bzrlib/lock.py	2005-11-10 03:03:53 +0000
@@ -52,7 +52,7 @@
                 raise
 
             # maybe this is an old branch (before may 2005)
-            mutter("trying to create missing branch lock %r" % filename)
+            mutter("trying to create missing branch lock %r", filename)
             
             self.f = open(filename, 'wb')
             return self.f
@@ -112,9 +112,9 @@
         import win32con, win32file, pywintypes
 
 
-        #LOCK_SH = 0 # the default
-        #LOCK_EX = win32con.LOCKFILE_EXCLUSIVE_LOCK
-        #LOCK_NB = win32con.LOCKFILE_FAIL_IMMEDIATELY
+        LOCK_SH = 0 # the default
+        LOCK_EX = win32con.LOCKFILE_EXCLUSIVE_LOCK
+        LOCK_NB = win32con.LOCKFILE_FAIL_IMMEDIATELY
 
         class _w32c_FileLock(_base_Lock):
             def _lock(self, filename, openmode, lockmode):
@@ -139,12 +139,13 @@
 
         class _w32c_ReadLock(_w32c_FileLock):
             def __init__(self, filename):
-                _w32c_FileLock._lock(self, filename, 'rb', 0)
+                _w32c_FileLock._lock(self, filename, 'rb',
+                                     LOCK_NB)
 
         class _w32c_WriteLock(_w32c_FileLock):
             def __init__(self, filename):
                 _w32c_FileLock._lock(self, filename, 'wb',
-                                     win32con.LOCKFILE_EXCLUSIVE_LOCK)
+                                     LOCK_EX + LOCK_NB)
 
 
 

=== modified file 'bzrlib/log.py'
--- bzrlib/log.py	2005-10-06 04:02:15 +0000
+++ bzrlib/log.py	2005-11-25 06:28:56 +0000
@@ -50,9 +50,13 @@
 """
 
 
+# TODO: option to show delta summaries for merged-in revisions
+
+import bzrlib.errors as errors
 from bzrlib.tree import EmptyTree
 from bzrlib.delta import compare_trees
 from bzrlib.trace import mutter
+import re
 
 
 def find_touching_revisions(branch, file_id):
@@ -114,10 +118,6 @@
     
     This is used to show summaries in verbose logs, and also for finding 
     revisions which touch a given file."""
-    # FIXME: The current version is very inefficient; it retrieves all revisions
-    # twice and reads the weave twice.  We ought to keep revisions in memory 
-    # in case they're used again, either in a general cache or perhaps 
-    # in this code.
     # XXX: What are we supposed to do when showing a summary for something 
     # other than a mainline revision.  The delta to it's first parent, or
     # (more useful) the delta to a nominated other revision.
@@ -154,6 +154,22 @@
     end_revision
         If not None, only show revisions <= end_revision
     """
+    branch.lock_read()
+    try:
+        _show_log(branch, lf, specific_fileid, verbose, direction,
+                  start_revision, end_revision, search)
+    finally:
+        branch.unlock()
+    
+def _show_log(branch,
+             lf,
+             specific_fileid=None,
+             verbose=False,
+             direction='reverse',
+             start_revision=None,
+             end_revision=None,
+             search=None):
+    """Worker function for show_log - see show_log."""
     from bzrlib.osutils import format_date
     from bzrlib.errors import BzrCheckError
     from bzrlib.textui import show_status
@@ -164,7 +180,7 @@
         warn("not a LogFormatter instance: %r" % lf)
 
     if specific_fileid:
-        mutter('get log for file_id %r' % specific_fileid)
+        mutter('get log for file_id %r', specific_fileid)
 
     if search is not None:
         import re
@@ -194,6 +210,7 @@
     else:
         raise ValueError('invalid direction %r' % direction)
 
+    revision_history = branch.revision_history()
     for revno, rev_id in cut_revs:
         if verbose or specific_fileid:
             delta = _get_revision_delta(branch, revno)
@@ -213,7 +230,25 @@
                 continue
 
         lf.show(revno, rev, delta)
-
+        if hasattr(lf, 'show_merge'):
+            if revno == 1:
+                excludes = set()
+            else:
+                # revno is 1 based, so -2 to get back 1 less.
+                excludes = set(branch.get_ancestry(revision_history[revno - 2]))
+            pending = list(rev.parent_ids)
+            while pending:
+                rev_id = pending.pop()
+                if rev_id in excludes:
+                    continue
+                # prevent showing merged revs twice if they multi-path.
+                excludes.add(rev_id)
+                try:
+                    rev = branch.get_revision(rev_id)
+                except errors.NoSuchRevision:
+                    continue
+                pending.extend(rev.parent_ids)
+                lf.show_merge(rev)
 
 
 def deltas_for_log_dummy(branch, which_revs):
@@ -309,75 +344,121 @@
     def show(self, revno, rev, delta):
         raise NotImplementedError('not implemented in abstract base')
 
+    def short_committer(self, rev):
+        return re.sub('<.*@.*>', '', rev.committer).strip(' ')
+    
     
 class LongLogFormatter(LogFormatter):
     def show(self, revno, rev, delta):
-        from osutils import format_date
-
+        return self._show_helper(revno=revno, rev=rev, delta=delta)
+
+    def show_merge(self, rev):
+        return self._show_helper(rev=rev, indent='    ', merged=True, delta=None)
+
+    def _show_helper(self, rev=None, revno=None, indent='', merged=False, delta=None):
+	"""Show a revision, either merged or not."""
+        from bzrlib.osutils import format_date
         to_file = self.to_file
-
-        print >>to_file,  '-' * 60
-        print >>to_file,  'revno:', revno
+        print >>to_file,  indent+'-' * 60
+        if revno is not None:
+            print >>to_file,  'revno:', revno
+        if merged:
+            print >>to_file,  indent+'merged:', rev.revision_id
+        elif self.show_ids:
+            print >>to_file,  indent+'revision-id:', rev.revision_id
         if self.show_ids:
-            print >>to_file,  'revision-id:', rev.revision_id
-
             for parent_id in rev.parent_ids:
-                print >>to_file, 'parent:', parent_id
-            
-        print >>to_file,  'committer:', rev.committer
-
+                print >>to_file, indent+'parent:', parent_id
+        print >>to_file,  indent+'committer:', rev.committer
+        try:
+            print >>to_file, indent+'branch nick: %s' % \
+                rev.properties['branch-nick']
+        except KeyError:
+            pass
         date_str = format_date(rev.timestamp,
                                rev.timezone or 0,
                                self.show_timezone)
-        print >>to_file,  'timestamp: %s' % date_str
+        print >>to_file,  indent+'timestamp: %s' % date_str
 
-        print >>to_file,  'message:'
+        print >>to_file,  indent+'message:'
         if not rev.message:
-            print >>to_file,  '  (no message)'
+            print >>to_file,  indent+'  (no message)'
         else:
-            for l in rev.message.split('\n'):
-                print >>to_file,  '  ' + l
-
+            message = rev.message.rstrip('\r\n')
+            for l in message.split('\n'):
+                print >>to_file,  indent+'  ' + l
         if delta != None:
             delta.show(to_file, self.show_ids)
 
 
-
 class ShortLogFormatter(LogFormatter):
     def show(self, revno, rev, delta):
         from bzrlib.osutils import format_date
 
         to_file = self.to_file
-
-        print >>to_file, "%5d %s\t%s" % (revno, rev.committer,
+        date_str = format_date(rev.timestamp, rev.timezone or 0,
+                            self.show_timezone)
+        print >>to_file, "%5d %s\t%s" % (revno, self.short_committer(rev),
                 format_date(rev.timestamp, rev.timezone or 0,
-                            self.show_timezone))
+                            self.show_timezone, date_fmt="%Y-%m-%d",
+                           show_offset=False))
         if self.show_ids:
             print >>to_file,  '      revision-id:', rev.revision_id
         if not rev.message:
             print >>to_file,  '      (no message)'
         else:
-            for l in rev.message.split('\n'):
+            message = rev.message.rstrip('\r\n')
+            for l in message.split('\n'):
                 print >>to_file,  '      ' + l
 
         # TODO: Why not show the modified files in a shorter form as
         # well? rewrap them single lines of appropriate length
         if delta != None:
             delta.show(to_file, self.show_ids)
-        print
-
-
+        print >>to_file, ''
+
+class LineLogFormatter(LogFormatter):
+    def truncate(self, str, max_len):
+        if len(str) <= max_len:
+            return str
+        return str[:max_len-3]+'...'
+
+    def date_string(self, rev):
+        from bzrlib.osutils import format_date
+        return format_date(rev.timestamp, rev.timezone or 0, 
+                           self.show_timezone, date_fmt="%Y-%m-%d",
+                           show_offset=False)
+
+    def message(self, rev):
+        if not rev.message:
+            return '(no message)'
+        else:
+            return rev.message
+
+    def show(self, revno, rev, delta):
+        print >> self.to_file, self.log_string(rev, 79) 
+
+    def log_string(self, rev, max_chars):
+        out = [self.truncate(self.short_committer(rev), 20)]
+        out.append(self.date_string(rev))
+        out.append(self.message(rev).replace('\n', ' '))
+        return self.truncate(" ".join(out).rstrip('\n'), max_chars)
+
+def line_log(rev, max_chars):
+    lf = LineLogFormatter(None)
+    return lf.log_string(rev, max_chars)
 
 FORMATTERS = {'long': LongLogFormatter,
               'short': ShortLogFormatter,
+              'line': LineLogFormatter,
               }
 
 
 def log_formatter(name, *args, **kwargs):
     """Construct a formatter from arguments.
 
-    name -- Name of the formatter to construct; currently 'long' and
-        'short' are supported.
+    name -- Name of the formatter to construct; currently 'long', 'short' and
+        'line' are supported.
     """
     from bzrlib.errors import BzrCommandError
     try:
@@ -389,3 +470,58 @@
     # deprecated; for compatability
     lf = LongLogFormatter(to_file=to_file, show_timezone=show_timezone)
     lf.show(revno, rev, delta)
+
+def show_changed_revisions(branch, old_rh, new_rh, to_file=None, log_format='long'):
+    """Show the change in revision history comparing the old revision history to the new one.
+
+    :param branch: The branch where the revisions exist
+    :param old_rh: The old revision history
+    :param new_rh: The new revision history
+    :param to_file: A file to write the results to. If None, stdout will be used
+    """
+    if to_file is None:
+        import sys
+        import codecs
+        import bzrlib
+        to_file = codecs.getwriter(bzrlib.user_encoding)(sys.stdout, errors='replace')
+    lf = log_formatter(log_format,
+                       show_ids=False,
+                       to_file=to_file,
+                       show_timezone='original')
+
+    # This is the first index which is different between
+    # old and new
+    base_idx = None
+    for i in xrange(max(len(new_rh),
+                        len(old_rh))):
+        if (len(new_rh) <= i
+            or len(old_rh) <= i
+            or new_rh[i] != old_rh[i]):
+            base_idx = i
+            break
+
+    if base_idx is None:
+        to_file.write('Nothing seems to have changed\n')
+        return
+    ## TODO: It might be nice to do something like show_log
+    ##       and show the merged entries. But since this is the
+    ##       removed revisions, it shouldn't be as important
+    if base_idx < len(old_rh):
+        to_file.write('*'*60)
+        to_file.write('\nRemoved Revisions:\n')
+        for i in range(base_idx, len(old_rh)):
+            rev = branch.get_revision(old_rh[i])
+            lf.show(i+1, rev, None)
+        to_file.write('*'*60)
+        to_file.write('\n\n')
+    if base_idx < len(new_rh):
+        to_file.write('Added Revisions:\n')
+        show_log(branch,
+                 lf,
+                 None,
+                 verbose=True,
+                 direction='forward',
+                 start_revision=base_idx+1,
+                 end_revision=len(new_rh),
+                 search=None)
+

=== added file 'bzrlib/lsprof.py'
--- bzrlib/lsprof.py	1970-01-01 00:00:00 +0000
+++ bzrlib/lsprof.py	2005-12-08 07:10:37 +0000
@@ -0,0 +1,108 @@
+# this is copied from the lsprof distro because somehow
+# it is not installed by distutils
+# I made one modification to profile so that it returns a pair
+# instead of just the Stats object
+
+import sys
+from _lsprof import Profiler, profiler_entry, profiler_subentry
+
+__all__ = ['profile', 'Stats']
+
+def profile(f, *args, **kwds):
+    """XXX docstring"""
+    p = Profiler()
+    p.enable(subcalls=True)
+    try:
+        ret = f(*args, **kwds)
+    finally:
+        p.disable()
+    return ret,Stats(p.getstats())
+
+
+class Stats(object):
+    """XXX docstring"""
+
+    def __init__(self, data):
+        self.data = data
+
+    def sort(self, crit="inlinetime"):
+        """XXX docstring"""
+        if crit not in profiler_entry.__dict__:
+            raise ValueError, "Can't sort by %s" % crit
+        self.data.sort(lambda b, a: cmp(getattr(a, crit),
+                                        getattr(b, crit)))
+        for e in self.data:
+            if e.calls:
+                e.calls.sort(lambda b, a: cmp(getattr(a, crit),
+                                              getattr(b, crit)))
+
+    def pprint(self, top=None, file=None):
+        """XXX docstring"""
+        if file is None:
+            file = sys.stdout
+        d = self.data
+        if top is not None:
+            d = d[:top]
+        cols = "% 12s %12s %11.4f %11.4f   %s\n"
+        hcols = "% 12s %12s %12s %12s %s\n"
+        cols2 = "+%12s %12s %11.4f %11.4f +  %s\n"
+        file.write(hcols % ("CallCount", "Recursive", "Total(ms)",
+                            "Inline(ms)", "module:lineno(function)"))
+        for e in d:
+            file.write(cols % (e.callcount, e.reccallcount, e.totaltime,
+                               e.inlinetime, label(e.code)))
+            if e.calls:
+                for se in e.calls:
+                    file.write(cols % ("+%s" % se.callcount, se.reccallcount,
+                                       se.totaltime, se.inlinetime,
+                                       "+%s" % label(se.code)))
+
+    def freeze(self):
+        """Replace all references to code objects with string
+        descriptions; this makes it possible to pickle the instance."""
+
+        # this code is probably rather ickier than it needs to be!
+        for i in range(len(self.data)):
+            e = self.data[i]
+            if not isinstance(e.code, str):
+                self.data[i] = type(e)((label(e.code),) + e[1:])
+                if e.calls:
+                    for j in range(len(e.calls)):
+                        se = e.calls[j]
+                        if not isinstance(se.code, str):
+                            e.calls[j] = type(se)((label(se.code),) + se[1:])
+
+_fn2mod = {}
+
+def label(code):
+    if isinstance(code, str):
+        return code
+    try:
+        mname = _fn2mod[code.co_filename]
+    except KeyError:
+        for k, v in sys.modules.iteritems():
+            if v is None:
+                continue
+            if not hasattr(v, '__file__'):
+                continue
+            if not isinstance(v.__file__, str):
+                continue
+            if v.__file__.startswith(code.co_filename):
+                mname = _fn2mod[code.co_filename] = k
+                break
+        else:
+            mname = _fn2mod[code.co_filename] = '<%s>'%code.co_filename
+    
+    return '%s:%d(%s)' % (mname, code.co_firstlineno, code.co_name)
+
+
+if __name__ == '__main__':
+    import os
+    sys.argv = sys.argv[1:]
+    if not sys.argv:
+        print >> sys.stderr, "usage: lsprof.py <script> <arguments...>"
+        sys.exit(2)
+    sys.path.insert(0, os.path.abspath(os.path.dirname(sys.argv[0])))
+    stats = profile(execfile, sys.argv[0], globals(), locals())
+    stats.sort()
+    stats.pprint()

=== removed file 'bzrlib/mdiff.py'
--- bzrlib/mdiff.py	2005-09-26 08:56:15 +0000
+++ bzrlib/mdiff.py	1970-01-01 00:00:00 +0000
@@ -1,166 +0,0 @@
-# (C) 2005 Matt Mackall
-# (C) 2005 Canonical Ltd
-
-# based on code by Matt Mackall, hacked by Martin Pool
-
-# mm's code works line-by-line; this just works on byte strings.
-# Possibly slower; possibly gives better results for code not
-# regularly separated by newlines and anyhow a bit simpler.
-
-
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-
-
-# TODO: maybe work on files not strings?
-
-# FIXME: doesn't work properly on files without trailing newlines
-
-import difflib
-import struct
-import sys
-import unittest
-from cStringIO import StringIO
-
-
-def linesplit(a):
-    """Split into two lists: content and line positions.
-
-    This returns (al, ap).
-
-    al[i] is the string content of line i of the file, including its
-    newline (if any).
-
-    ap[i] is the byte position in the file where that line starts.
-
-    ap[-1] is the byte position of the end of the file (i.e. the
-    length of the file.)
-
-    This transformation allows us to do a line-based diff and then map
-    back to byte positions.
-    """
-
-    al, ap = [], []
-    last = 0
-
-    n = a.find("\n") + 1
-    while n > 0:
-        ap.append(last)
-        al.append(a[last:n])
-        last = n
-        n = a.find("\n", n) + 1
-
-    if last < len(a):
-        al.append(a[last:])
-        ap.append(last)
-
-    # position at the end
-    ap.append(len(a))
-
-    return (al, ap)
-
-
-def diff(a, b):
-    # TODO: Use different splits, perhaps rsync-like, for binary files?
-    
-    (al, ap) = linesplit(a)
-    (bl, bp) = linesplit(b)
-
-    d = difflib.SequenceMatcher(None, al, bl)
-    
-    ## sys.stderr.write('  ~ real_quick_ratio: %.4f\n' % d.real_quick_ratio())
-    
-    for o, m, n, s, t in d.get_opcodes():
-        if o == 'equal': continue
-        # a[m:n] should be replaced by b[s:t]
-        if s == t:
-            yield ap[m], ap[n], ''
-        else:
-            yield ap[m], ap[n], ''.join(bl[s:t])
-
-
-def tobinary(ops):
-    b = StringIO()
-    for f in ops:
-        b.write(struct.pack(">III", f[0], f[1], len(f[2])))
-        b.write(f[2])
-    return b.getvalue()
-
-
-def bdiff(a, b):
-    return tobinary(diff(a, b))
-
-
-def patch(t, ops):
-    last = 0
-    b = StringIO()
-
-    for m, n, r in ops:
-        b.write(t[last:m])
-        if r:
-            b.write(r)
-        last = n
-        
-    b.write(t[last:])
-    return b.getvalue()
-
-
-def frombinary(b):
-    bin = StringIO(b)
-    while True:
-        p = bin.read(12)
-        if not p:
-            break
-
-        m, n, l = struct.unpack(">III", p)
-        
-        if l == 0:
-            r = ''
-        else:
-            r = bin.read(l)
-            if len(r) != l:
-                raise Exception("truncated patch data")
-            
-        yield m, n, r
-
-
-def bpatch(t, b):
-    return patch(t, frombinary(b))
-
-
-
-
-class TestDiffPatch(unittest.TestCase):
-    def doDiffPatch(self, old, new):
-        diff = bdiff(old, new)
-        result = bpatch(old, diff)
-        self.assertEquals(new, result)
-
-
-    def testSimpleDiff(self):
-        """Simply add a line at the end"""
-        self.doDiffPatch('a\nb\n', 'a\nb\nc\n')
-        
-
-        
-    def testTrailingLine(self):
-        """Test diff that adds an unterminated line.
-
-        (Old versions didn't do this properly.)"""
-        self.doDiffPatch('a\nb\nc\n',
-                         'a\nb\nc\nd')
-
-
-if __name__ == '__main__':
-    unittest.main()

=== modified file 'bzrlib/merge.py' (properties changed: -x to +x)
--- bzrlib/merge.py	2005-10-04 04:32:32 +0000
+++ bzrlib/merge.py	2005-12-22 23:25:30 +0000
@@ -16,29 +16,39 @@
 
 
 import os
-import tempfile
 import shutil
 import errno
 
 import bzrlib.osutils
 import bzrlib.revision
 from bzrlib.merge_core import merge_flex, ApplyMerge3, BackupBeforeChange
+from bzrlib.merge_core import WeaveMerge
 from bzrlib.changeset import generate_changeset, ExceptionConflictHandler
 from bzrlib.changeset import Inventory, Diff3Merge, ReplaceContents
 from bzrlib.branch import Branch
-from bzrlib.errors import BzrCommandError, UnrelatedBranches, NoCommonAncestor
-from bzrlib.errors import NoCommits
+from bzrlib.errors import (BzrCommandError,
+                           UnrelatedBranches,
+                           NoCommonAncestor,
+                           NoCommits,
+                           WorkingTreeNotRevision,
+                           NotBranchError,
+                           NotVersionedError,
+                           BzrError)
 from bzrlib.delta import compare_trees
-from bzrlib.trace import mutter, warning
+from bzrlib.trace import mutter, warning, note
 from bzrlib.fetch import greedy_fetch, fetch
-from bzrlib.revision import is_ancestor
-from bzrlib.osutils import rename
+from bzrlib.revision import is_ancestor, NULL_REVISION
+from bzrlib.osutils import rename, pathjoin
 from bzrlib.revision import common_ancestor, MultipleRevisionSources
 from bzrlib.errors import NoSuchRevision
 
+# TODO: Report back as changes are merged in
+
 # TODO: build_working_dir can be built on something simpler than merge()
 
 # FIXME: merge() parameters seem oriented towards the command line
+# NOTABUG: merge is a helper for commandline functions.  merge_inner is the
+#          the core functionality.
 
 # comments from abentley on irc: merge happens in two stages, each
 # of which generates a changeset object
@@ -81,7 +91,7 @@
         for line in lines:
             d_file.write(line)
 
-    def add_suffix(self, name, suffix, last_new_name=None):
+    def add_suffix(self, name, suffix, last_new_name=None, fix_inventory=True):
         """Rename a file to append a suffix.  If the new name exists, the
         suffix is added repeatedly until a non-existant name is found
 
@@ -94,11 +104,24 @@
         new_name = last_new_name+suffix
         try:
             rename(name, new_name)
-            return new_name
+            if fix_inventory is True:
+                try:
+                    relpath = self.this_tree.relpath(name)
+                except NotBranchError:
+                    relpath = None
+                if relpath is not None:
+                    file_id = self.this_tree.path2id(relpath)
+                    if file_id is not None:
+                        new_path = self.this_tree.relpath(new_name)
+                        rename(new_name, name)
+                        self.this_tree.rename_one(relpath, new_path)
+                        assert self.this_tree.id2path(file_id) == new_path
         except OSError, e:
             if e.errno != errno.EEXIST and e.errno != errno.ENOTEMPTY:
                 raise
-            return self.add_suffix(name, suffix, last_new_name=new_name)
+            return self.add_suffix(name, suffix, last_new_name=new_name, 
+                                   fix_inventory=fix_inventory)
+        return new_name
 
     def conflict(self, text):
         warning(text)
@@ -114,15 +137,25 @@
         :param base_path: Path to the file text for the BASE tree
         :param other_path: Path to the file text for the OTHER tree
         """
-        self.add_suffix(this_path, ".THIS")
+        self.add_suffix(this_path, ".THIS", fix_inventory=False)
         self.dump(base_lines, this_path+".BASE")
         self.dump(other_lines, this_path+".OTHER")
         rename(new_file, this_path)
         self.conflict("Diff3 conflict encountered in %s" % this_path)
 
+    def weave_merge_conflict(self, filename, weave, other_i, out_file):
+        """
+        Handle weave conflicts by producing a .THIS, and .OTHER.  The
+        main file will be a version with diff3-style conflicts.
+        """
+        self.add_suffix(filename, ".THIS", fix_inventory=False)
+        out_file.commit()
+        self.dump(weave.get_iter(other_i), filename+".OTHER")
+        self.conflict("Text conflict encountered in %s" % filename)
+
     def new_contents_conflict(self, filename, other_contents):
         """Conflicting contents for newly added file."""
-        self.copy(other_contents, filename + ".OTHER")
+        other_contents(filename + ".OTHER", self, False)
         self.conflict("Conflict in newly added file %s" % filename)
     
 
@@ -138,20 +171,19 @@
         return "skip"
 
     def rem_contents_conflict(self, filename, this_contents, base_contents):
-        base_contents(filename+".BASE", self, False)
-        this_contents(filename+".THIS", self, False)
+        base_contents(filename+".BASE", self)
+        this_contents(filename+".THIS", self)
         self.conflict("Other branch deleted locally modified file %s" %
                       filename)
         return ReplaceContents(this_contents, None)
 
     def abs_this_path(self, file_id):
         """Return the absolute path for a file_id in the this tree."""
-        relpath = self.this_tree.id2path(file_id)
-        return self.this_tree.tree.abspath(relpath)
+        return self.this_tree.id2abspath(file_id)
 
     def add_missing_parents(self, file_id, tree):
         """If some of the parents for file_id are missing, add them."""
-        entry = tree.tree.inventory[file_id]
+        entry = tree.inventory[file_id]
         if entry.parent_id not in self.this_tree:
             return self.create_all_missing(entry.parent_id, tree)
         else:
@@ -159,47 +191,55 @@
 
     def create_all_missing(self, file_id, tree):
         """Add contents for a file_id and all its parents to a tree."""
-        entry = tree.tree.inventory[file_id]
+        entry = tree.inventory[file_id]
         if entry.parent_id is not None and entry.parent_id not in self.this_tree:
             abspath = self.create_all_missing(entry.parent_id, tree)
         else:
             abspath = self.abs_this_path(entry.parent_id)
-        entry_path = os.path.join(abspath, entry.name)
+        entry_path = pathjoin(abspath, entry.name)
         if not os.path.isdir(entry_path):
             self.create(file_id, entry_path, tree)
         return entry_path
 
-    def create(self, file_id, path, tree, reverse=False):
+    def create(self, file_id, path, tree):
         """Uses tree data to create a filesystem object for the file_id"""
-        from merge_core import get_id_contents
-        get_id_contents(file_id, tree)(path, self, reverse)
+        from changeset import get_contents
+        get_contents(tree, file_id)(path, self)
 
     def missing_for_merge(self, file_id, other_path):
         """The file_id doesn't exist in THIS, but does in OTHER and BASE"""
         self.conflict("Other branch modified locally deleted file %s" %
                       other_path)
         parent_dir = self.add_missing_parents(file_id, self.other_tree)
-        stem = os.path.join(parent_dir, os.path.basename(other_path))
+        stem = pathjoin(parent_dir, os.path.basename(other_path))
         self.create(file_id, stem+".OTHER", self.other_tree)
         self.create(file_id, stem+".BASE", self.base_tree)
 
+    def threeway_contents_conflict(filename, this_contents, base_contents,
+                                   other_contents):
+        self.conflict("Three-way conflict merging %s" % filename)
+
     def finalize(self):
-        if not self.ignore_zero:
-            print "%d conflicts encountered.\n" % self.conflicts
+        if self.conflicts == 0:
+            if not self.ignore_zero:
+                note("All changes applied successfully.")
+        else:
+            note("%d conflicts encountered." % self.conflicts)
             
-def get_tree(treespec, temp_root, label, local_branch=None):
+def get_tree(treespec, local_branch=None):
     location, revno = treespec
-    branch = Branch.open_containing(location)
+    branch = Branch.open_containing(location)[0]
     if revno is None:
         revision = None
     elif revno == -1:
         revision = branch.last_revision()
     else:
         revision = branch.get_rev_id(revno)
-    return branch, get_revid_tree(branch, revision, temp_root, label,
-                                  local_branch)
+        if revision is None:
+            revision = NULL_REVISION
+    return branch, get_revid_tree(branch, revision, local_branch)
 
-def get_revid_tree(branch, revision, temp_root, label, local_branch):
+def get_revid_tree(branch, revision, local_branch):
     if revision is None:
         base_tree = branch.working_tree()
     else:
@@ -208,85 +248,13 @@
             base_tree = local_branch.revision_tree(revision)
         else:
             base_tree = branch.revision_tree(revision)
-    temp_path = os.path.join(temp_root, label)
-    os.mkdir(temp_path)
-    return MergeTree(base_tree, temp_path)
+    return base_tree
 
 
 def file_exists(tree, file_id):
     return tree.has_filename(tree.id2path(file_id))
     
 
-class MergeTree(object):
-    def __init__(self, tree, tempdir):
-        object.__init__(self)
-        if hasattr(tree, "basedir"):
-            self.root = tree.basedir
-        else:
-            self.root = None
-        self.tree = tree
-        self.tempdir = tempdir
-        os.mkdir(os.path.join(self.tempdir, "texts"))
-        os.mkdir(os.path.join(self.tempdir, "symlinks"))
-        self.cached = {}
-
-    def __iter__(self):
-        return self.tree.__iter__()
-
-    def __contains__(self, file_id):
-        return file_id in self.tree
-
-    def get_file(self, file_id):
-        return self.tree.get_file(file_id)
-
-    def get_file_sha1(self, id):
-        return self.tree.get_file_sha1(id)
-
-    def is_executable(self, id):
-        return self.tree.is_executable(id)
-
-    def id2path(self, file_id):
-        return self.tree.id2path(file_id)
-
-    def has_id(self, file_id):
-        return self.tree.has_id(file_id)
-
-    def has_or_had_id(self, file_id):
-        if file_id == self.tree.inventory.root.file_id:
-            return True
-        return self.tree.inventory.has_id(file_id)
-
-    def has_or_had_id(self, file_id):
-        if file_id == self.tree.inventory.root.file_id:
-            return True
-        return self.tree.inventory.has_id(file_id)
-
-    def readonly_path(self, id):
-        if id not in self.tree:
-            return None
-        if self.root is not None:
-            return self.tree.abspath(self.tree.id2path(id))
-        else:
-            kind = self.tree.inventory[id].kind
-            if kind in ("directory", "root_directory"):
-                return self.tempdir
-            if not self.cached.has_key(id):
-                if kind == "file":
-                    path = os.path.join(self.tempdir, "texts", id)
-                    outfile = file(path, "wb")
-                    outfile.write(self.tree.get_file(id).read())
-                    assert(bzrlib.osutils.lexists(path))
-                    if self.tree.is_executable(id):
-                        os.chmod(path, 0755)
-                else:
-                    assert kind == "symlink"
-                    path = os.path.join(self.tempdir, "symlinks", id)
-                    target = self.tree.get_symlink_target(id)
-                    os.symlink(target, path)
-                self.cached[id] = path
-            return self.cached[id]
-
-
 def build_working_dir(to_dir):
     """Build a working directory in an empty directory.
 
@@ -297,20 +265,28 @@
     eventually be done by just building the tree directly calling into 
     lower-level code (e.g. constructing a changeset).
     """
-    merge((to_dir, -1), (to_dir, 0), this_dir=to_dir,
-          check_clean=False, ignore_zero=True)
+    # RBC 20051019 is this not just 'export' ?
+    # AB Well, export doesn't take care of inventory...
+    this_branch = Branch.open_containing(to_dir)[0]
+    transform_tree(this_branch.working_tree(), this_branch.basis_tree())
+
+
+def transform_tree(from_tree, to_tree, interesting_ids=None):
+    merge_inner(from_tree.branch, to_tree, from_tree, ignore_zero=True,
+                interesting_ids=interesting_ids)
 
 
 def merge(other_revision, base_revision,
           check_clean=True, ignore_zero=False,
           this_dir=None, backup_files=False, merge_type=ApplyMerge3,
-          file_list=None):
+          file_list=None, show_base=False, reprocess=False):
     """Merge changes into a tree.
 
     base_revision
-        tuple(path, revision) Base for three-way merge.
+        list(path, revno) Base for three-way merge.  
+        If [None, None] then a base will be automatically determined.
     other_revision
-        tuple(path, revision) Other revision for three-way merge.
+        list(path, revno) Other revision for three-way merge.
     this_dir
         Directory to merge changes into; '.' by default.
     check_clean
@@ -319,195 +295,336 @@
     ignore_zero - If true, suppress the "zero conflicts" message when 
         there are no conflicts; should be set when doing something we expect
         to complete perfectly.
+    file_list - If supplied, merge only changes to selected files.
 
     All available ancestors of other_revision and base_revision are
     automatically pulled into the branch.
+
+    The revno may be -1 to indicate the last revision on the branch, which is
+    the typical case.
+
+    This function is intended for use from the command line; programmatic
+    clients might prefer to call merge_inner(), which has less magic behavior.
     """
-    tempdir = tempfile.mkdtemp(prefix="bzr-")
-    try:
-        if this_dir is None:
-            this_dir = '.'
-        this_branch = Branch.open_containing(this_dir)
-        this_rev_id = this_branch.last_revision()
-        if this_rev_id is None:
+    if this_dir is None:
+        this_dir = u'.'
+    this_branch = Branch.open_containing(this_dir)[0]
+    if show_base and not merge_type is ApplyMerge3:
+        raise BzrCommandError("Show-base is not supported for this merge"
+                              " type. %s" % merge_type)
+    if reprocess and not merge_type is ApplyMerge3:
+        raise BzrCommandError("Reprocess is not supported for this merge"
+                              " type. %s" % merge_type)
+    if reprocess and show_base:
+        raise BzrCommandError("Cannot reprocess and show base.")
+    merger = Merger(this_branch)
+    merger.check_basis(check_clean)
+    merger.set_other(other_revision)
+    merger.set_base(base_revision)
+    if merger.base_rev_id == merger.other_rev_id:
+        note('Nothing to do.')
+        return 0
+    merger.backup_files = backup_files
+    merger.merge_type = merge_type 
+    merger.set_interesting_files(file_list)
+    merger.show_base = show_base 
+    merger.reprocess = reprocess
+    merger.conflict_handler = MergeConflictHandler(merger.this_tree, 
+                                                   merger.base_tree, 
+                                                   merger.other_tree,
+                                                   ignore_zero=ignore_zero)
+    conflicts = merger.do_merge()
+    merger.set_pending()
+    return conflicts
+
+def merge_inner(this_branch, other_tree, base_tree, ignore_zero=False,
+                backup_files=False, 
+                merge_type=ApplyMerge3, 
+                interesting_ids=None, 
+                show_base=False, 
+                reprocess=False, 
+                other_rev_id=None,
+                interesting_files=None):
+    """Primary interface for merging. 
+
+        typical use is probably 
+        'merge_inner(branch, branch.get_revision_tree(other_revision),
+                     branch.get_revision_tree(base_revision))'
+        """
+    merger = Merger(this_branch, other_tree, base_tree)
+    merger.backup_files = backup_files
+    merger.merge_type = merge_type
+    merger.interesting_ids = interesting_ids
+    if interesting_files:
+        assert not interesting_ids, ('Only supply interesting_ids'
+                                     ' or interesting_files')
+        merger._set_interesting_files(interesting_files)
+    merger.show_base = show_base 
+    merger.reprocess = reprocess
+    merger.conflict_handler = MergeConflictHandler(merger.this_tree, base_tree, 
+                                                   other_tree,
+                                                   ignore_zero=ignore_zero)
+    merger.other_rev_id = other_rev_id
+    merger.other_basis = other_rev_id
+    return merger.do_merge()
+
+
+class Merger(object):
+    def __init__(self, this_branch, other_tree=None, base_tree=None):
+        object.__init__(self)
+        self.this_branch = this_branch
+        self.this_basis = this_branch.last_revision()
+        self.this_rev_id = None
+        self.this_tree = this_branch.working_tree()
+        self.this_revision_tree = None
+        self.this_basis_tree = None
+        self.other_tree = other_tree
+        self.base_tree = base_tree
+        self.ignore_zero = False
+        self.backup_files = False
+        self.interesting_ids = None
+        self.show_base = False
+        self.reprocess = False
+        self.conflict_handler = MergeConflictHandler(self.this_tree, base_tree, 
+                                                     other_tree)
+
+    def revision_tree(self, revision_id):
+        return self.this_branch.revision_tree(revision_id)
+
+    def ensure_revision_trees(self):
+        if self.this_revision_tree is None:
+            self.this_basis_tree = self.this_branch.revision_tree(
+                self.this_basis)
+            if self.this_basis == self.this_rev_id:
+                self.this_revision_tree = self.this_basis_tree
+
+
+        if self.other_rev_id is None:
+            other_basis_tree = self.revision_tree(self.other_basis)
+            changes = compare_trees(self.other_tree, other_basis_tree)
+            if changes.has_changed():
+                raise WorkingTreeNotRevision(self.this_tree)
+            other_rev_id = other_basis
+            self.other_tree = other_basis_tree
+
+
+    def file_revisions(self, file_id):
+        self.ensure_revision_trees()
+        def get_id(tree, file_id):
+            revision_id = tree.inventory[file_id].revision
+            assert revision_id is not None
+            return revision_id
+        if self.this_rev_id is None:
+            if self.this_basis_tree.get_file_sha1(file_id) != \
+                self.this_tree.get_file_sha1(file_id):
+                raise WorkingTreeNotRevision(self.this_tree)
+
+        trees = (self.this_basis_tree, self.other_tree)
+        return [get_id(tree, file_id) for tree in trees]
+            
+
+    def merge_factory(self, file_id, base, other):
+        if self.merge_type.history_based:
+            if self.show_base is True:
+                raise BzrError("Cannot show base for hisory-based merges")
+            if self.reprocess is True:
+                raise BzrError("Cannot reprocess history-based merges")
+                
+            t_revid, o_revid = self.file_revisions(file_id)
+            weave = self.this_basis_tree.get_weave(file_id)
+            contents_change = self.merge_type(weave, t_revid, o_revid)
+        else:
+            if self.show_base is True or self.reprocess is True:
+                contents_change = self.merge_type(file_id, base, other, 
+                                                  show_base=self.show_base, 
+                                                  reprocess=self.reprocess)
+            else:
+                contents_change = self.merge_type(file_id, base, other)
+        if self.backup_files:
+            contents_change = BackupBeforeChange(contents_change)
+        return contents_change
+
+    def check_basis(self, check_clean):
+        if self.this_basis is None:
             raise BzrCommandError("This branch has no commits")
         if check_clean:
-            changes = compare_trees(this_branch.working_tree(), 
-                                    this_branch.basis_tree(), False)
-            if changes.has_changed():
+            self.compare_basis()
+            if self.this_basis != self.this_rev_id:
                 raise BzrCommandError("Working tree has uncommitted changes.")
-        other_branch, other_tree = get_tree(other_revision, tempdir, "other",
-                                            this_branch)
+
+    def compare_basis(self):
+        changes = compare_trees(self.this_branch.working_tree(), 
+                                self.this_branch.basis_tree(), False)
+        if not changes.has_changed():
+            self.this_rev_id = self.this_basis
+
+    def set_interesting_files(self, file_list):
+        try:
+            self._set_interesting_files(file_list)
+        except NotVersionedError, e:
+            raise BzrCommandError("%s is not a source file in any"
+                                      " tree." % e.path)
+
+    def _set_interesting_files(self, file_list):
+        """Set the list of interesting ids from a list of files."""
+        if file_list is None:
+            self.interesting_ids = None
+            return
+
+        interesting_ids = set()
+        for fname in file_list:
+            path = self.this_tree.relpath(fname)
+            found_id = False
+            for tree in (self.this_tree, self.base_tree, self.other_tree):
+                file_id = tree.inventory.path2id(path)
+                if file_id is not None:
+                    interesting_ids.add(file_id)
+                    found_id = True
+            if not found_id:
+                raise NotVersionedError(path=fname)
+        self.interesting_ids = interesting_ids
+
+    def set_pending(self):
+        if not self.base_is_ancestor:
+            return
+        if self.other_rev_id is None:
+            return
+        if self.other_rev_id in self.this_branch.get_ancestry(self.this_basis):
+            return
+        self.this_branch.working_tree().add_pending_merge(self.other_rev_id)
+
+    def set_other(self, other_revision):
+        other_branch, self.other_tree = get_tree(other_revision, 
+                                                 self.this_branch)
         if other_revision[1] == -1:
-            other_rev_id = other_branch.last_revision()
-            if other_rev_id is None:
+            self.other_rev_id = other_branch.last_revision()
+            if self.other_rev_id is None:
                 raise NoCommits(other_branch)
-            other_basis = other_rev_id
+            self.other_basis = self.other_rev_id
         elif other_revision[1] is not None:
-            other_rev_id = other_branch.get_rev_id(other_revision[1])
-            other_basis = other_rev_id
+            self.other_rev_id = other_branch.get_rev_id(other_revision[1])
+            self.other_basis = self.other_rev_id
         else:
-            other_rev_id = None
-            other_basis = other_branch.last_revision()
-            if other_basis is None:
+            self.other_rev_id = None
+            self.other_basis = other_branch.last_revision()
+            if self.other_basis is None:
                 raise NoCommits(other_branch)
+        fetch(from_branch=other_branch, to_branch=self.this_branch, 
+              last_revision=self.other_basis)
+
+    def set_base(self, base_revision):
+        mutter("doing merge() with no base_revision specified")
         if base_revision == [None, None]:
             try:
-                base_rev_id = common_ancestor(this_rev_id, other_basis, 
-                                              this_branch)
+                self.base_rev_id = common_ancestor(self.this_basis, 
+                                                   self.other_basis, 
+                                                   self.this_branch)
             except NoCommonAncestor:
                 raise UnrelatedBranches()
-            base_tree = get_revid_tree(this_branch, base_rev_id, tempdir, 
-                                       "base", None)
-            base_is_ancestor = True
+            self.base_tree = get_revid_tree(self.this_branch, self.base_rev_id,
+                                            None)
+            self.base_is_ancestor = True
         else:
-            base_branch, base_tree = get_tree(base_revision, tempdir, "base")
+            base_branch, self.base_tree = get_tree(base_revision)
             if base_revision[1] == -1:
-                base_rev_id = base_branch.last_revision()
+                self.base_rev_id = base_branch.last_revision()
             elif base_revision[1] is None:
-                base_rev_id = None
-            else:
-                base_rev_id = base_branch.get_rev_id(base_revision[1])
-            fetch(from_branch=base_branch, to_branch=this_branch)
-            base_is_ancestor = is_ancestor(this_rev_id, base_rev_id,
-                                           this_branch)
-        if file_list is None:
-            interesting_ids = None
-        else:
-            interesting_ids = set()
-            this_tree = this_branch.working_tree()
-            for fname in file_list:
-                path = this_branch.relpath(fname)
-                found_id = False
-                for tree in (this_tree, base_tree.tree, other_tree.tree):
-                    file_id = tree.inventory.path2id(path)
-                    if file_id is not None:
-                        interesting_ids.add(file_id)
-                        found_id = True
-                if not found_id:
-                    raise BzrCommandError("%s is not a source file in any"
-                                          " tree." % fname)
-        merge_inner(this_branch, other_tree, base_tree, tempdir, 
-                    ignore_zero=ignore_zero, backup_files=backup_files, 
-                    merge_type=merge_type, interesting_ids=interesting_ids)
-        if base_is_ancestor and other_rev_id is not None\
-            and other_rev_id not in this_branch.revision_history():
-            this_branch.add_pending_merge(other_rev_id)
-    finally:
-        shutil.rmtree(tempdir)
-
-
-def set_interesting(inventory_a, inventory_b, interesting_ids):
-    """Mark files whose ids are in interesting_ids as interesting
-    """
-    for inventory in (inventory_a, inventory_b):
-        for path, source_file in inventory.iteritems():
-             source_file.interesting = source_file.id in interesting_ids
-
-
-def generate_cset_optimized(tree_a, tree_b, interesting_ids=None):
-    """Generate a changeset.  If interesting_ids is supplied, only changes
-    to those files will be shown.  Metadata changes are stripped.
-    """ 
-    cset =  generate_changeset(tree_a, tree_b, interesting_ids)
-    for entry in cset.entries.itervalues():
-        entry.metadata_change = None
-    return cset
-
-
-def merge_inner(this_branch, other_tree, base_tree, tempdir, 
-                ignore_zero=False, merge_type=ApplyMerge3, backup_files=False,
-                interesting_ids=None):
-
-    def merge_factory(file_id, base, other):
-        contents_change = merge_type(file_id, base, other)
-        if backup_files:
-            contents_change = BackupBeforeChange(contents_change)
-        return contents_change
-
-    this_tree = get_tree((this_branch.base, None), tempdir, "this")[1]
-
-    def get_inventory(tree):
-        return tree.tree.inventory
-
-    inv_changes = merge_flex(this_tree, base_tree, other_tree,
-                             generate_cset_optimized, get_inventory,
-                             MergeConflictHandler(this_tree, base_tree,
-                             other_tree, ignore_zero=ignore_zero),
-                             merge_factory=merge_factory, 
-                             interesting_ids=interesting_ids)
-
-    adjust_ids = []
-    for id, path in inv_changes.iteritems():
-        if path is not None:
-            if path == '.':
-                path = ''
-            else:
-                assert path.startswith('.' + os.sep), "path is %s" % path
-            path = path[2:]
-        adjust_ids.append((path, id))
-    if len(adjust_ids) > 0:
-        this_branch.set_inventory(regen_inventory(this_branch, this_tree.root,
-                                                  adjust_ids))
-
-
-def regen_inventory(this_branch, root, new_entries):
-    old_entries = this_branch.read_working_inventory()
-    new_inventory = {}
-    by_path = {}
-    new_entries_map = {} 
-    for path, file_id in new_entries:
-        if path is None:
-            continue
-        new_entries_map[file_id] = path
-
-    def id2path(file_id):
-        path = new_entries_map.get(file_id)
-        if path is not None:
-            return path
-        entry = old_entries[file_id]
-        if entry.parent_id is None:
-            return entry.name
-        return os.path.join(id2path(entry.parent_id), entry.name)
-        
-    for file_id in old_entries:
-        entry = old_entries[file_id]
-        path = id2path(file_id)
-        new_inventory[file_id] = (path, file_id, entry.parent_id, entry.kind)
-        by_path[path] = file_id
-    
-    deletions = 0
-    insertions = 0
-    new_path_list = []
-    for path, file_id in new_entries:
-        if path is None:
-            del new_inventory[file_id]
-            deletions += 1
-        else:
-            new_path_list.append((path, file_id))
-            if file_id not in old_entries:
-                insertions += 1
-    # Ensure no file is added before its parent
-    new_path_list.sort()
-    for path, file_id in new_path_list:
-        if path == '':
-            parent = None
-        else:
-            parent = by_path[os.path.dirname(path)]
-        kind = bzrlib.osutils.file_kind(os.path.join(root, path))
-        new_inventory[file_id] = (path, file_id, parent, kind)
-        by_path[path] = file_id 
-
-    # Get a list in insertion order
-    new_inventory_list = new_inventory.values()
-    mutter ("""Inventory regeneration:
-old length: %i insertions: %i deletions: %i new_length: %i"""\
-        % (len(old_entries), insertions, deletions, len(new_inventory_list)))
-    assert len(new_inventory_list) == len(old_entries) + insertions - deletions
-    new_inventory_list.sort()
-    return new_inventory_list
+                self.base_rev_id = None
+            else:
+                self.base_rev_id = base_branch.get_rev_id(base_revision[1])
+            fetch(from_branch=base_branch, to_branch=self.this_branch)
+            self.base_is_ancestor = is_ancestor(self.this_basis, 
+                                                self.base_rev_id,
+                                                self.this_branch)
+
+    def do_merge(self):
+        def get_inventory(tree):
+            return tree.inventory
+        
+        inv_changes = merge_flex(self.this_tree, self.base_tree, 
+                                 self.other_tree,
+                                 generate_changeset, get_inventory,
+                                 self.conflict_handler,
+                                 merge_factory=self.merge_factory, 
+                                 interesting_ids=self.interesting_ids)
+
+        adjust_ids = []
+        for id, path in inv_changes.iteritems():
+            if path is not None:
+                if path == u'.':
+                    path = u''
+                else:
+                    assert path.startswith('.' + '/') or path.startswith('.' + '\\'), "path is %s" % path
+                path = path[2:]
+            adjust_ids.append((path, id))
+        if len(adjust_ids) > 0:
+            self.this_branch.working_tree().set_inventory(self.regen_inventory(adjust_ids))
+        conflicts = self.conflict_handler.conflicts
+        self.conflict_handler.finalize()
+        return conflicts
+
+    def regen_inventory(self, new_entries):
+        old_entries = self.this_branch.working_tree().read_working_inventory()
+        new_inventory = {}
+        by_path = {}
+        new_entries_map = {} 
+        for path, file_id in new_entries:
+            if path is None:
+                continue
+            new_entries_map[file_id] = path
+
+        def id2path(file_id):
+            path = new_entries_map.get(file_id)
+            if path is not None:
+                return path
+            entry = old_entries[file_id]
+            if entry.parent_id is None:
+                return entry.name
+            return pathjoin(id2path(entry.parent_id), entry.name)
+            
+        for file_id in old_entries:
+            entry = old_entries[file_id]
+            path = id2path(file_id)
+            new_inventory[file_id] = (path, file_id, entry.parent_id, 
+                                      entry.kind)
+            by_path[path] = file_id
+        
+        deletions = 0
+        insertions = 0
+        new_path_list = []
+        for path, file_id in new_entries:
+            if path is None:
+                del new_inventory[file_id]
+                deletions += 1
+            else:
+                new_path_list.append((path, file_id))
+                if file_id not in old_entries:
+                    insertions += 1
+        # Ensure no file is added before its parent
+        new_path_list.sort()
+        for path, file_id in new_path_list:
+            if path == '':
+                parent = None
+            else:
+                parent = by_path[os.path.dirname(path)]
+            abspath = pathjoin(self.this_tree.basedir, path)
+            kind = bzrlib.osutils.file_kind(abspath)
+            new_inventory[file_id] = (path, file_id, parent, kind)
+            by_path[path] = file_id 
+
+        # Get a list in insertion order
+        new_inventory_list = new_inventory.values()
+        mutter ("""Inventory regeneration:
+    old length: %i insertions: %i deletions: %i new_length: %i"""\
+            % (len(old_entries), insertions, deletions, 
+               len(new_inventory_list)))
+        assert len(new_inventory_list) == len(old_entries) + insertions\
+            - deletions
+        new_inventory_list.sort()
+        return new_inventory_list
 
 merge_types = {     "merge3": (ApplyMerge3, "Native diff3-style merge"), 
-                     "diff3": (Diff3Merge,  "Merge using external diff3")
+                     "diff3": (Diff3Merge,  "Merge using external diff3"),
+                     'weave': (WeaveMerge, "Weave-based merge")
               }
 

=== modified file 'bzrlib/merge3.py'
--- bzrlib/merge3.py	2005-09-04 02:59:56 +0000
+++ bzrlib/merge3.py	2005-11-29 07:39:19 +0000
@@ -19,6 +19,8 @@
 # s: "i hate that."
 
 
+from difflib import SequenceMatcher
+from bzrlib.errors import CantReprocessAndShowBase
 
 def intersect(ra, rb):
     """Given two ranges return the range where they intersect or None.
@@ -67,27 +69,32 @@
         self.base = base
         self.a = a
         self.b = b
-        from difflib import SequenceMatcher
-        self.a_ops = SequenceMatcher(None, base, a).get_opcodes()
-        self.b_ops = SequenceMatcher(None, base, b).get_opcodes()
 
 
 
     def merge_lines(self,
                     name_a=None,
                     name_b=None,
+                    name_base=None,
                     start_marker='<<<<<<<',
                     mid_marker='=======',
                     end_marker='>>>>>>>',
-                    show_base=False):
+                    base_marker=None,
+                    reprocess=False):
         """Return merge in cvs-like form.
         """
+        if base_marker and reprocess:
+            raise CantReprocessAndShowBase()
         if name_a:
             start_marker = start_marker + ' ' + name_a
         if name_b:
             end_marker = end_marker + ' ' + name_b
-            
-        for t in self.merge_regions():
+        if name_base and base_marker:
+            base_marker = base_marker + ' ' + name_base
+        merge_regions = self.merge_regions()
+        if reprocess is True:
+            merge_regions = self.reprocess_merge_regions(merge_regions)
+        for t in merge_regions:
             what = t[0]
             if what == 'unchanged':
                 for i in range(t[1], t[2]):
@@ -102,6 +109,10 @@
                 yield start_marker + '\n'
                 for i in range(t[3], t[4]):
                     yield self.a[i]
+                if base_marker is not None:
+                    yield base_marker + '\n'
+                    for i in range(t[1], t[2]):
+                        yield self.base[i]
                 yield mid_marker + '\n'
                 for i in range(t[5], t[6]):
                     yield self.b[i]
@@ -263,16 +274,52 @@
                 iz = zend
                 ia = aend
                 ib = bend
-        
-
-        
+    
+
+    def reprocess_merge_regions(self, merge_regions):
+        """Where there are conflict regions, remove the agreed lines.
+
+        Lines where both A and B have made the same changes are 
+        eliminated.
+        """
+        for region in merge_regions:
+            if region[0] != "conflict":
+                yield region
+                continue
+            type, iz, zmatch, ia, amatch, ib, bmatch = region
+            a_region = self.a[ia:amatch]
+            b_region = self.b[ib:bmatch]
+            matches = SequenceMatcher(None, a_region, 
+                                      b_region).get_matching_blocks()
+            next_a = ia
+            next_b = ib
+            for region_ia, region_ib, region_len in matches[:-1]:
+                region_ia += ia
+                region_ib += ib
+                reg = self.mismatch_region(next_a, region_ia, next_b,
+                                           region_ib)
+                if reg is not None:
+                    yield reg
+                yield 'same', region_ia, region_len+region_ia
+                next_a = region_ia + region_len
+                next_b = region_ib + region_len
+            reg = self.mismatch_region(next_a, amatch, next_b, bmatch)
+            if reg is not None:
+                yield reg
+
+
+    @staticmethod
+    def mismatch_region(next_a, region_ia,  next_b, region_ib):
+        if next_a < region_ia or next_b < region_ib:
+            return 'conflict', None, None, next_a, region_ia, next_b, region_ib
+            
+
     def find_sync_regions(self):
         """Return a list of sync regions, where both descendents match the base.
 
         Generates a list of (base1, base2, a1, a2, b1, b2).  There is
         always a zero-length sync region at the end of all the files.
         """
-        from difflib import SequenceMatcher
 
         ia = ib = 0
         amatches = SequenceMatcher(None, self.base, self.a).get_matching_blocks()
@@ -332,7 +379,6 @@
 
     def find_unconflicted(self):
         """Return a list of ranges in base that are not conflicted."""
-        from difflib import SequenceMatcher
 
         import re
 

=== modified file 'bzrlib/merge_core.py'
--- bzrlib/merge_core.py	2005-10-04 04:32:32 +0000
+++ bzrlib/merge_core.py	2005-12-22 23:32:43 +0000
@@ -2,17 +2,31 @@
 
 import changeset
 from changeset import Inventory, apply_changeset, invert_dict
-from bzrlib.osutils import backup_file, rename
+from bzrlib.osutils import backup_file, rename, pathjoin
 from bzrlib.merge3 import Merge3
 import bzrlib
+from bzrlib.atomicfile import AtomicFile
+from changeset import get_contents
+
 
 class ApplyMerge3:
     """Contents-change wrapper around merge3.Merge3"""
-    def __init__(self, file_id, base, other):
+
+    history_based = False
+
+    def __init__(self, file_id, base, other, show_base=False, reprocess=False):
         self.file_id = file_id
         self.base = base
         self.other = other
- 
+        self.show_base = show_base
+        self.reprocess = reprocess
+
+    def is_creation(self):
+        return False
+
+    def is_deletion(self):
+        return False
+
     def __eq__(self, other):
         if not isinstance(other, ApplyMerge3):
             return False
@@ -22,15 +36,10 @@
     def __ne__(self, other):
         return not (self == other)
 
-
-    def apply(self, filename, conflict_handler, reverse=False):
+    def apply(self, filename, conflict_handler):
         new_file = filename+".new" 
-        if not reverse:
-            base = self.base
-            other = self.other
-        else:
-            base = self.other
-            other = self.base
+        base = self.base
+        other = self.other
         def get_lines(tree):
             if self.file_id not in tree:
                 raise Exception("%s not in tree" % self.file_id)
@@ -43,8 +52,14 @@
         new_conflicts = False
         output_file = file(new_file, "wb")
         start_marker = "!START OF MERGE CONFLICT!" + "I HOPE THIS IS UNIQUE"
+        if self.show_base is True:
+            base_marker = '|' * 7
+        else:
+            base_marker = None
         for line in m3.merge_lines(name_a = "TREE", name_b = "MERGE-SOURCE", 
-                       start_marker=start_marker):
+                       name_base = "BASE-REVISION",
+                       start_marker=start_marker, base_marker=base_marker,
+                       reprocess = self.reprocess):
             if line.startswith(start_marker):
                 new_conflicts = True
                 output_file.write(line.replace(start_marker, '<' * 7))
@@ -60,11 +75,62 @@
                                             other_lines)
 
 
+class WeaveMerge:
+    """Contents-change wrapper around weave merge"""
+
+    history_based = True
+
+    def __init__(self, weave, this_revision_id, other_revision_id):
+        self.weave = weave
+        self.this_revision_id = this_revision_id
+        self.other_revision_id = other_revision_id
+
+    def is_creation(self):
+        return False
+
+    def is_deletion(self):
+        return False
+
+    def __eq__(self, other):
+        if not isinstance(other, WeaveMerge):
+            return False
+        return self.weave == other.weave and\
+            self.this_revision_id == other.this_revision_id and\
+            self.other_revision_id == other.other_revision_id
+
+    def __ne__(self, other):
+        return not (self == other)
+
+    def apply(self, filename, conflict_handler):
+        this_i = self.weave.lookup(self.this_revision_id)
+        other_i = self.weave.lookup(self.other_revision_id)
+        plan = self.weave.plan_merge(this_i, other_i)
+        lines = self.weave.weave_merge(plan)
+        conflicts = False
+        out_file = AtomicFile(filename, mode='wb')
+        for line in lines:
+            if line == '<<<<<<<\n':
+                conflicts = True
+            out_file.write(line)
+        if conflicts:
+            conflict_handler.weave_merge_conflict(filename, self.weave,
+                                                  other_i, out_file)
+        else:
+            out_file.commit()
+
+
 class BackupBeforeChange:
     """Contents-change wrapper to back up file first"""
+
     def __init__(self, contents_change):
         self.contents_change = contents_change
- 
+
+    def is_creation(self):
+        return self.contents_change.is_creation()
+
+    def is_deletion(self):
+        return self.contents_change.is_deletion()
+
     def __eq__(self, other):
         if not isinstance(other, BackupBeforeChange):
             return False
@@ -73,9 +139,9 @@
     def __ne__(self, other):
         return not (self == other)
 
-    def apply(self, filename, conflict_handler, reverse=False):
+    def apply(self, filename, conflict_handler):
         backup_file(filename)
-        self.contents_change.apply(filename, conflict_handler, reverse)
+        self.contents_change.apply(filename, conflict_handler)
 
 
 def invert_invent(inventory):
@@ -95,22 +161,14 @@
     cset = changeset_function(base, other, interesting_ids)
     new_cset = make_merge_changeset(cset, this, base, other, 
                                     conflict_handler, merge_factory)
-    result = apply_changeset(new_cset, invert_invent(this.tree.inventory),
-                             this.root, conflict_handler, False)
-    conflict_handler.finalize()
+    result = apply_changeset(new_cset, invert_invent(this.inventory),
+                             this.basedir, conflict_handler)
     return result
-
     
 
 def make_merge_changeset(cset, this, base, other, 
                          conflict_handler, merge_factory):
     new_cset = changeset.Changeset()
-    def get_this_contents(id):
-        path = this.readonly_path(id)
-        if os.path.isdir(path):
-            return changeset.dir_create
-        else:
-            return changeset.FileCreate(file(path, "rb").read())
 
     for entry in cset.entries.itervalues():
         if entry.is_boring():
@@ -127,6 +185,7 @@
 
     return new_cset
 
+
 class ThreeWayConflict(Exception):
     def __init__(self, this, base, other):
         self.this = this
@@ -135,6 +194,7 @@
         msg = "Conflict merging %s %s and %s" % (this, base, other)
         Exception.__init__(self, msg)
 
+
 def threeway_select(this, base, other):
     """Returns a value selected by the three-way algorithm.
     Raises ThreewayConflict if the algorithm yields a conflict"""
@@ -154,7 +214,7 @@
         assert hasattr(tree, "__contains__"), "%s" % tree
         if not tree.has_or_had_id(file_id):
             return (None, None, "")
-        entry = tree.tree.inventory[file_id]
+        entry = tree.inventory[file_id]
         my_dir = tree.id2path(entry.parent_id)
         if my_dir is None:
             my_dir = ""
@@ -162,8 +222,8 @@
     this_name, this_parent, this_dir = entry_data(entry.id, this)
     base_name, base_parent, base_dir = entry_data(entry.id, base)
     other_name, other_parent, other_dir = entry_data(entry.id, other)
-    mutter("Dirs: this, base, other %r %r %r" % (this_dir, base_dir, other_dir))
-    mutter("Names: this, base, other %r %r %r" % (this_name, base_name, other_name))
+    mutter("Dirs: this, base, other %r %r %r", this_dir, base_dir, other_dir)
+    mutter("Names: this, base, other %r %r %r", this_name, base_name, other_name)
     old_name = this_name
     try:
         new_name = threeway_select(this_name, base_name, other_name)
@@ -185,7 +245,7 @@
             parent_dir = {this_parent: this_dir, other_parent: other_dir, 
                           base_parent: base_dir}
             directory = parent_dir[parent]
-            return os.path.join(directory, name)
+            return pathjoin(directory, name)
         else:
             assert parent is None
             return None
@@ -199,28 +259,15 @@
     return new_entry
 
 
-def get_contents(entry, tree):
-    return get_id_contents(entry.id, tree)
-
-def get_id_contents(file_id, tree):
-    """Get a contents change element suitable for use with ReplaceContents
-    """
-    tree_entry = tree.tree.inventory[file_id]
-    if tree_entry.kind == "file":
-        return changeset.FileCreate(tree.get_file(file_id).read())
-    elif tree_entry.kind == "symlink":
-        return changeset.SymlinkCreate(tree.get_symlink_target(file_id))
-    else:
-        assert tree_entry.kind in ("root_directory", "directory")
-        return changeset.dir_create
-
-
 def make_merged_contents(entry, this, base, other, conflict_handler,
                          merge_factory):
     contents = entry.contents_change
     if contents is None:
         return None
-    this_path = this.readonly_path(entry.id)
+    if entry.id in this:
+        this_path = this.id2abspath(entry.id)
+    else:
+        this_path = None
     def make_merge():
         if this_path is None:
             return conflict_handler.missing_for_merge(entry.id, 
@@ -228,69 +275,88 @@
         return merge_factory(entry.id, base, other)
 
     if isinstance(contents, changeset.ReplaceContents):
-        if contents.old_contents is None and contents.new_contents is None:
+        base_contents = contents.old_contents
+        other_contents = contents.new_contents
+        if base_contents is None and other_contents is None:
             return None
-        if contents.new_contents is None:
-            this_contents = get_contents(entry, this)
+        if other_contents is None:
+            this_contents = get_contents(this, entry.id)
             if this_path is not None and bzrlib.osutils.lexists(this_path):
-                if this_contents != contents.old_contents:
+                if this_contents != base_contents:
                     return conflict_handler.rem_contents_conflict(this_path, 
-                        this_contents, contents.old_contents)
+                        this_contents, base_contents)
                 return contents
             else:
                 return None
-        elif contents.old_contents is None:
+        elif base_contents is None:
             if this_path is None or not bzrlib.osutils.lexists(this_path):
                 return contents
             else:
-                this_contents = get_contents(entry, this)
-                if this_contents == contents.new_contents:
+                this_contents = get_contents(this, entry.id)
+                if this_contents == other_contents:
                     return None
                 else:
-                    other_path = other.readonly_path(entry.id)    
                     conflict_handler.new_contents_conflict(this_path, 
-                                                           other_path)
-        elif isinstance(contents.old_contents, changeset.FileCreate) and \
-            isinstance(contents.new_contents, changeset.FileCreate):
+                        other_contents)
+        elif isinstance(base_contents, changeset.TreeFileCreate) and \
+            isinstance(other_contents, changeset.TreeFileCreate):
             return make_merge()
         else:
-            raise Exception("Unhandled merge scenario")
+            this_contents = get_contents(this, entry.id)
+            if this_contents == base_contents:
+                return contents
+            elif this_contents == other_contents:
+                return None
+            elif base_contents == other_contents:
+                return None
+            else:
+                conflict_handler.threeway_contents_conflict(this_path,
+                                                            this_contents,
+                                                            base_contents,
+                                                            other_contents)
+                
 
 def make_merged_metadata(entry, base, other):
     metadata = entry.metadata_change
     if metadata is None:
         return None
-    if isinstance(metadata, changeset.ChangeUnixPermissions):
-        if metadata.new_mode is None:
-            return None
-        elif metadata.old_mode is None:
-            return metadata
-        else:
-            base_path = base.readonly_path(entry.id)
-            other_path = other.readonly_path(entry.id)    
-            return PermissionsMerge(base_path, other_path)
+    assert isinstance(metadata, changeset.ChangeExecFlag)
+    if metadata.new_exec_flag is None:
+        return None
+    elif metadata.old_exec_flag is None:
+        return metadata
+    else:
+        return ExecFlagMerge(base, other, entry.id)
     
 
-class PermissionsMerge(object):
-    def __init__(self, base_path, other_path):
-        self.base_path = base_path
-        self.other_path = other_path
-
-    def apply(self, filename, conflict_handler, reverse=False):
-        if not reverse:
-            base = self.base_path
-            other = self.other_path
-        else:
-            base = self.other_path
-            other = self.base_path
-        base_stat = os.stat(base).st_mode
-        other_stat = os.stat(other).st_mode
-        this_stat = os.stat(filename).st_mode
-        if base_stat &0777 == other_stat &0777:
-            return
-        elif this_stat &0777 == other_stat &0777:
-            return
-        elif this_stat &0777 == base_stat &0777:
-            os.chmod(filename, other_stat)
-        else:
-            conflict_handler.permission_conflict(filename, base, other)
+class ExecFlagMerge(object):
+    def __init__(self, base_tree, other_tree, file_id):
+        self.base_tree = base_tree
+        self.other_tree = other_tree
+        self.file_id = file_id
+
+    def apply(self, filename, conflict_handler):
+        base = self.base_tree
+        other = self.other_tree
+        base_exec_flag = base.is_executable(self.file_id)
+        other_exec_flag = other.is_executable(self.file_id)
+        this_mode = os.stat(filename).st_mode
+        this_exec_flag = bool(this_mode & 0111)
+        if (base_exec_flag != other_exec_flag and
+            this_exec_flag != other_exec_flag):
+            assert this_exec_flag == base_exec_flag
+            current_mode = os.stat(filename).st_mode
+            if other_exec_flag:
+                umask = os.umask(0)
+                os.umask(umask)
+                to_mode = current_mode | (0100 & ~umask)
+                # Enable x-bit for others only if they can read it.
+                if current_mode & 0004:
+                    to_mode |= 0001 & ~umask
+                if current_mode & 0040:
+                    to_mode |= 0010 & ~umask
+            else:
+                to_mode = current_mode & ~0111
+            os.chmod(filename, to_mode)
+
+

=== modified file 'bzrlib/missing.py'
--- bzrlib/missing.py	2005-09-29 12:49:59 +0000
+++ bzrlib/missing.py	2005-12-11 22:29:51 +0000
@@ -1,77 +1,102 @@
 """\
 A plugin for displaying what revisions are in 'other' but not in local.
 """
-
-def show_missing(br_local, br_remote, verbose=False, quiet=False):
-    """Show the revisions which exist in br_remote, that 
-    do not exist in br_local.
-    """
-    from bzrlib.log import show_one_log
-    import sys
-    local_history = br_local.revision_history()
-    remote_history = br_remote.revision_history()
-    if local_history == remote_history:
-        if not quiet:
-            print 'Trees are identical.'
-        return 0
-    if local_history[:len(remote_history)] == remote_history:
-        # Local is not missing anything from remote, so consider it
-        # up-to-date
-        if not quiet:
-            print 'Local tree has all of remote revisions (remote is missing local)'
-        return 0
-    if quiet:
-        return 1
-
-    # Check for divergence
-    common_idx = min(len(local_history), len(remote_history)) - 1
-    if common_idx >= 0 and local_history[common_idx] != remote_history[common_idx]:
-        print 'Trees have diverged'
-
-    local_rev_set = set(local_history)
-
-    # Find the last common revision between the two trees
-    revno = 0
-    for revno, (local_rev, remote_rev) in enumerate(zip(local_history, remote_history)):
-        if local_rev != remote_rev:
-            break
-
-    missing_remote = []
-    for rno, rev_id in enumerate(remote_history[revno:]):
-        # This assumes that you can have a revision in the
-        # local history, which does not have the same ancestry
-        # as the remote ancestry.
-        # This may or may not be possible.
-        # In the future this should also checked for merged revisions.
-        if rev_id not in local_rev_set:
-            missing_remote.append((rno+revno+1, rev_id))
-
-    print 'Missing %d revisions' %  len(missing_remote)
-    print
-
-    if verbose:
-        from bzrlib.diff import compare_trees
-        from bzrlib.tree import EmptyTree
-        show_ids = True
-        last_tree = EmptyTree
-        last_rev_id = None
-    else:
-        show_ids = False
-    for revno, rev_id in missing_remote:
-        rev = br_remote.get_revision(rev_id)
+from bzrlib.ui import ui_factory
+def iter_log_data(revisions, revision_source, verbose):
+    from bzrlib.diff import compare_trees
+    from bzrlib.tree import EmptyTree
+    last_tree = EmptyTree
+    last_rev_id = None
+    for revno, rev_id in revisions:
+        rev = revision_source.get_revision(rev_id)
         if verbose:
-            parent_rev_id = rev.parents[0].revision_id
+            parent_rev_id = rev.parent_ids[0]
             if last_rev_id == parent_rev_id:
                 parent_tree = last_tree
             else:
-                parent_tree = br_remote.revision_tree(parent_rev_id)
-            revision_tree = br_remote.revision_tree(rev_id)
+                parent_tree = revision_source.revision_tree(parent_rev_id)
+            revision_tree = revision_source.revision_tree(rev_id)
             last_rev_id = rev_id
             last_tree = revision_tree
             delta = compare_trees(revision_tree, parent_tree)
         else:
             delta = None
-
-        show_one_log(revno, rev, delta, verbose, sys.stdout, 'original')
-    return 1
-
+        yield revno, rev, delta
+
+
+def find_unmerged(local_branch, remote_branch):
+    progress = ui_factory.progress_bar()
+    local_branch.lock_read()
+    try:
+        remote_branch.lock_read()
+        try:
+            local_rev_history, local_rev_history_map = \
+                _get_history(local_branch, progress, "local", 0)
+            remote_rev_history, remote_rev_history_map = \
+                _get_history(remote_branch, progress, "remote", 1)
+            result = _shortcut(local_rev_history, remote_rev_history)
+            if result is not None:
+                local_extra, remote_extra = result
+                local_extra = sorted_revisions(local_extra, 
+                                               local_rev_history_map)
+                remote_extra = sorted_revisions(remote_extra, 
+                                                remote_rev_history_map)
+                return local_extra, remote_extra
+
+            local_ancestry = _get_ancestry(local_branch, progress, "local",
+                                           2, local_rev_history)
+            remote_ancestry = _get_ancestry(remote_branch, progress, "remote",
+                                            3, remote_rev_history)
+            progress.update('pondering', 4, 5)
+            extras = local_ancestry.symmetric_difference(remote_ancestry) 
+            local_extra = extras.intersection(set(local_rev_history))
+            remote_extra = extras.intersection(set(remote_rev_history))
+            local_extra = sorted_revisions(local_extra, local_rev_history_map)
+            remote_extra = sorted_revisions(remote_extra, 
+                                            remote_rev_history_map)
+                    
+        finally:
+            remote_branch.unlock()
+    finally:
+        local_branch.unlock()
+        progress.clear()
+    return (local_extra, remote_extra)
+
+def _shortcut(local_rev_history, remote_rev_history):
+    local_history = set(local_rev_history)
+    remote_history = set(remote_rev_history)
+    if len(local_rev_history) == 0:
+        return set(), remote_history
+    elif len(remote_rev_history) == 0:
+        return local_history, set()
+    elif local_rev_history[-1] in remote_history:
+        return set(), _after(remote_rev_history, local_rev_history)
+    elif remote_rev_history[-1] in local_history:
+        return _after(local_rev_history, remote_rev_history), set()
+    else:
+        return None
+
+def _after(larger_history, smaller_history):
+    return set(larger_history[larger_history.index(smaller_history[-1])+1:])
+
+def _get_history(branch, progress, label, step):
+    progress.update('%s history' % label, step, 5)
+    rev_history = branch.revision_history()
+    rev_history_map = dict(
+        [(rev, rev_history.index(rev) + 1)
+         for rev in rev_history])
+    return rev_history, rev_history_map
+
+def _get_ancestry(branch, progress, label, step, rev_history):
+    progress.update('%s ancestry' % label, step, 5)
+    if len(rev_history) > 0:
+        ancestry = set(branch.get_ancestry(rev_history[-1]))
+    else:
+        ancestry = set()
+    return ancestry
+    
+
+def sorted_revisions(revisions, history_map):
+    revisions = [(history_map[r],r) for r in revisions]
+    revisions.sort()
+    return revisions

=== modified file 'bzrlib/msgeditor.py'
--- bzrlib/msgeditor.py	2005-10-02 13:14:12 +0000
+++ bzrlib/msgeditor.py	2005-12-02 05:25:53 +0000
@@ -21,18 +21,18 @@
 
 import os
 from subprocess import call
+
+import bzrlib.config as config
 from bzrlib.errors import BzrError
 
 def _get_editor():
     """Return a sequence of possible editor binaries for the current platform"""
-    from bzrlib.osutils import _read_config_value
-    
     try:
         yield os.environ["BZR_EDITOR"]
     except KeyError:
         pass
 
-    e = _read_config_value("editor")
+    e = config.GlobalConfig().get_editor()
     if e is not None:
         yield e
         
@@ -79,7 +79,7 @@
         ignoreline = "-- This line and the following will be ignored --"
         
     try:
-        tmp_fileno, msgfilename = tempfile.mkstemp()
+        tmp_fileno, msgfilename = tempfile.mkstemp(prefix='bzr_log.', dir=u'.')
         msgfile = os.close(tmp_fileno)
         if infotext is not None and infotext != "":
             hasinfo = True
@@ -127,3 +127,21 @@
         try: os.unlink(msgfilename)
         except IOError: pass
 
+
+def make_commit_message_template(working_tree, specific_files):
+    """Prepare a template file for a commit into a branch.
+
+    Returns a unicode string containing the template.
+    """
+    # TODO: Should probably be given the WorkingTree not the branch
+    #
+    # TODO: make provision for this to be overridden or modified by a hook
+    #
+    # TODO: Rather than running the status command, should prepare a draft of
+    # the revision to be committed, then pause and ask the user to
+    # confirm/write a message.
+    from StringIO import StringIO       # must be unicode-safe
+    from bzrlib.status import show_status
+    status_tmp = StringIO()
+    show_status(working_tree.branch, specific_files=specific_files, to_file=status_tmp)
+    return status_tmp.getvalue()

=== added file 'bzrlib/option.py'
--- bzrlib/option.py	1970-01-01 00:00:00 +0000
+++ bzrlib/option.py	2005-12-12 07:06:34 +0000
@@ -0,0 +1,221 @@
+# Copyright (C) 2004, 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+import re
+
+import bzrlib.commands
+from bzrlib.trace import warning, mutter
+from bzrlib.revisionspec import RevisionSpec
+from bzrlib.errors import BzrCommandError
+
+
+def _parse_revision_str(revstr):
+    """This handles a revision string -> revno.
+
+    This always returns a list.  The list will have one element for
+    each revision specifier supplied.
+
+    >>> _parse_revision_str('234')
+    [<RevisionSpec_int 234>]
+    >>> _parse_revision_str('234..567')
+    [<RevisionSpec_int 234>, <RevisionSpec_int 567>]
+    >>> _parse_revision_str('..')
+    [<RevisionSpec None>, <RevisionSpec None>]
+    >>> _parse_revision_str('..234')
+    [<RevisionSpec None>, <RevisionSpec_int 234>]
+    >>> _parse_revision_str('234..')
+    [<RevisionSpec_int 234>, <RevisionSpec None>]
+    >>> _parse_revision_str('234..456..789') # Maybe this should be an error
+    [<RevisionSpec_int 234>, <RevisionSpec_int 456>, <RevisionSpec_int 789>]
+    >>> _parse_revision_str('234....789') #Error ?
+    [<RevisionSpec_int 234>, <RevisionSpec None>, <RevisionSpec_int 789>]
+    >>> _parse_revision_str('revid:test@other.com-234234')
+    [<RevisionSpec_revid revid:test@other.com-234234>]
+    >>> _parse_revision_str('revid:test@other.com-234234..revid:test@other.com-234235')
+    [<RevisionSpec_revid revid:test@other.com-234234>, <RevisionSpec_revid revid:test@other.com-234235>]
+    >>> _parse_revision_str('revid:test@other.com-234234..23')
+    [<RevisionSpec_revid revid:test@other.com-234234>, <RevisionSpec_int 23>]
+    >>> _parse_revision_str('date:2005-04-12')
+    [<RevisionSpec_date date:2005-04-12>]
+    >>> _parse_revision_str('date:2005-04-12 12:24:33')
+    [<RevisionSpec_date date:2005-04-12 12:24:33>]
+    >>> _parse_revision_str('date:2005-04-12T12:24:33')
+    [<RevisionSpec_date date:2005-04-12T12:24:33>]
+    >>> _parse_revision_str('date:2005-04-12,12:24:33')
+    [<RevisionSpec_date date:2005-04-12,12:24:33>]
+    >>> _parse_revision_str('-5..23')
+    [<RevisionSpec_int -5>, <RevisionSpec_int 23>]
+    >>> _parse_revision_str('-5')
+    [<RevisionSpec_int -5>]
+    >>> _parse_revision_str('123a')
+    Traceback (most recent call last):
+      ...
+    BzrError: No namespace registered for string: '123a'
+    >>> _parse_revision_str('abc')
+    Traceback (most recent call last):
+      ...
+    BzrError: No namespace registered for string: 'abc'
+    >>> _parse_revision_str('branch:../branch2')
+    [<RevisionSpec_branch branch:../branch2>]
+    """
+    # TODO: Maybe move this into revisionspec.py
+    old_format_re = re.compile('\d*:\d*')
+    m = old_format_re.match(revstr)
+    revs = []
+    if m:
+        warning('Colon separator for revision numbers is deprecated.'
+                ' Use .. instead')
+        for rev in revstr.split(':'):
+            if rev:
+                revs.append(RevisionSpec(int(rev)))
+            else:
+                revs.append(RevisionSpec(None))
+    else:
+        next_prefix = None
+        for x in revstr.split('..'):
+            if not x:
+                revs.append(RevisionSpec(None))
+            elif x[-1] == ':':
+                # looks like a namespace:.. has happened
+                next_prefix = x + '..'
+            else:
+                if next_prefix is not None:
+                    x = next_prefix + x
+                revs.append(RevisionSpec(x))
+                next_prefix = None
+        if next_prefix is not None:
+            revs.append(RevisionSpec(next_prefix))
+    return revs
+
+
+def _parse_merge_type(typestring):
+    return get_merge_type(typestring)
+
+def get_merge_type(typestring):
+    """Attempt to find the merge class/factory associated with a string."""
+    from merge import merge_types
+    try:
+        return merge_types[typestring][0]
+    except KeyError:
+        templ = '%s%%7s: %%s' % (' '*12)
+        lines = [templ % (f[0], f[1][1]) for f in merge_types.iteritems()]
+        type_list = '\n'.join(lines)
+        msg = "No known merge type %s. Supported types are:\n%s" %\
+            (typestring, type_list)
+        raise BzrCommandError(msg)
+
+class Option(object):
+    """Description of a command line option"""
+    # TODO: Some way to show in help a description of the option argument
+
+    OPTIONS = {}
+    SHORT_OPTIONS = {}
+
+    def __init__(self, name, help='', type=None, argname=None):
+        """Make a new command option.
+
+        name -- regular name of the command, used in the double-dash
+            form and also as the parameter to the command's run() 
+            method.
+
+        help -- help message displayed in command help
+
+        type -- function called to parse the option argument, or 
+            None (default) if this option doesn't take an argument.
+
+        argname -- name of option argument, if any
+        """
+        # TODO: perhaps a subclass that automatically does 
+        # --option, --no-option for reversable booleans
+        self.name = name
+        self.help = help
+        self.type = type
+        if type is None:
+            assert argname is None
+        elif argname is None:
+            argname = 'ARG'
+        self.argname = argname
+
+    def short_name(self):
+        """Return the single character option for this command, if any.
+
+        Short options are globally registered.
+        """
+        for short, option in Option.SHORT_OPTIONS.iteritems():
+            if option is self:
+                return short
+
+
+def _global_option(name, **kwargs):
+    """Register o as a global option."""
+    Option.OPTIONS[name] = Option(name, **kwargs)
+
+_global_option('all')
+_global_option('overwrite', help='Ignore differences between branches and '
+               'overwrite unconditionally')
+_global_option('basis', type=str)
+_global_option('diff-options', type=str)
+_global_option('help',
+               help='show help message')
+_global_option('file', type=unicode)
+_global_option('force')
+_global_option('format', type=unicode)
+_global_option('forward')
+_global_option('message', type=unicode)
+_global_option('no-recurse')
+_global_option('profile',
+               help='show performance profiling information')
+_global_option('revision', type=_parse_revision_str)
+_global_option('short')
+_global_option('show-ids', 
+               help='show internal object ids')
+_global_option('timezone', 
+               type=str,
+               help='display timezone as local, original, or utc')
+_global_option('verbose',
+               help='display more information')
+_global_option('version')
+_global_option('email')
+_global_option('update')
+_global_option('long', help='Use detailed log format')
+_global_option('short', help='Use moderately short log format')
+_global_option('line', help='Use log format with one line per revision')
+_global_option('root', type=str)
+_global_option('no-backup')
+_global_option('merge-type', type=_parse_merge_type)
+_global_option('pattern', type=str)
+_global_option('quiet')
+_global_option('remember', help='Remember the specified location as a'
+               ' default.')
+_global_option('reprocess', help='Reprocess to reduce spurious conflicts')
+_global_option('kind', type=str)
+_global_option('dry-run',
+               help="show what would be done, but don't actually do anything")
+
+
+def _global_short(short_name, long_name):
+    assert short_name not in Option.SHORT_OPTIONS
+    Option.SHORT_OPTIONS[short_name] = Option.OPTIONS[long_name]
+    
+
+Option.SHORT_OPTIONS['F'] = Option.OPTIONS['file']
+Option.SHORT_OPTIONS['h'] = Option.OPTIONS['help']
+Option.SHORT_OPTIONS['m'] = Option.OPTIONS['message']
+Option.SHORT_OPTIONS['r'] = Option.OPTIONS['revision']
+Option.SHORT_OPTIONS['v'] = Option.OPTIONS['verbose']
+Option.SHORT_OPTIONS['l'] = Option.OPTIONS['long']
+Option.SHORT_OPTIONS['q'] = Option.OPTIONS['quiet']

=== modified file 'bzrlib/osutils.py'
--- bzrlib/osutils.py	2005-10-03 05:16:47 +0000
+++ bzrlib/osutils.py	2006-01-05 22:30:59 +0000
@@ -24,12 +24,18 @@
 import os
 import re
 import sha
+import string
 import sys
 import time
 import types
+import tempfile
 
 import bzrlib
-from bzrlib.errors import BzrError
+from bzrlib.errors import (BzrError,
+                           BzrBadParameter,
+                           NoSuchFile,
+                           PathNotChild,
+                           )
 from bzrlib.trace import mutter
 
 
@@ -96,6 +102,8 @@
         raise BzrError('invalid file kind %r' % kind)
 
 def lexists(f):
+    if hasattr(os.path, 'lexists'):
+        return os.path.lexists(f)
     try:
         if hasattr(os, 'lstat'):
             os.lstat(f)
@@ -108,17 +116,119 @@
         else:
             raise BzrError("lstat/stat of (%r): %r" % (f, e))
 
+def fancy_rename(old, new, rename_func, unlink_func):
+    """A fancy rename, when you don't have atomic rename.
+    
+    :param old: The old path, to rename from
+    :param new: The new path, to rename to
+    :param rename_func: The potentially non-atomic rename function
+    :param unlink_func: A way to delete the target file if the full rename succeeds
+    """
+
+    # sftp rename doesn't allow overwriting, so play tricks:
+    import random
+    base = os.path.basename(new)
+    dirname = os.path.dirname(new)
+    tmp_name = u'tmp.%s.%.9f.%d.%d' % (base, time.time(), os.getpid(), random.randint(0, 0x7FFFFFFF))
+    tmp_name = pathjoin(dirname, tmp_name)
+
+    # Rename the file out of the way, but keep track if it didn't exist
+    # We don't want to grab just any exception
+    # something like EACCES should prevent us from continuing
+    # The downside is that the rename_func has to throw an exception
+    # with an errno = ENOENT, or NoSuchFile
+    file_existed = False
+    try:
+        rename_func(new, tmp_name)
+    except (NoSuchFile,), e:
+        pass
+    except IOError, e:
+        # RBC 20060103 abstraction leakage: the paramiko SFTP clients rename
+        # function raises an IOError with errno == None when a rename fails.
+        # This then gets caught here.
+        if e.errno not in (None, errno.ENOENT, errno.ENOTDIR):
+            raise
+    except Exception, e:
+        if (not hasattr(e, 'errno') 
+            or e.errno not in (errno.ENOENT, errno.ENOTDIR)):
+            raise
+    else:
+        file_existed = True
+
+    success = False
+    try:
+        # This may throw an exception, in which case success will
+        # not be set.
+        rename_func(old, new)
+        success = True
+    finally:
+        if file_existed:
+            # If the file used to exist, rename it back into place
+            # otherwise just delete it from the tmp location
+            if success:
+                unlink_func(tmp_name)
+            else:
+                rename_func(tmp_name, new)
+
+# Default is to just use the python builtins
+abspath = os.path.abspath
+realpath = os.path.realpath
+pathjoin = os.path.join
+normpath = os.path.normpath
+getcwd = os.getcwdu
+mkdtemp = tempfile.mkdtemp
+rename = os.rename
+dirname = os.path.dirname
+basename = os.path.basename
+
+if os.name == "posix":
+    # In Python 2.4.2 and older, os.path.abspath and os.path.realpath
+    # choke on a Unicode string containing a relative path if
+    # os.getcwd() returns a non-sys.getdefaultencoding()-encoded
+    # string.
+    _fs_enc = sys.getfilesystemencoding()
+    def abspath(path):
+        return os.path.abspath(path.encode(_fs_enc)).decode(_fs_enc)
+
+    def realpath(path):
+        return os.path.realpath(path.encode(_fs_enc)).decode(_fs_enc)
+
+if sys.platform == 'win32':
+    # We need to use the Unicode-aware os.path.abspath and
+    # os.path.realpath on Windows systems.
+    def abspath(path):
+        return os.path.abspath(path).replace('\\', '/')
+
+    def realpath(path):
+        return os.path.realpath(path).replace('\\', '/')
+
+    def pathjoin(*args):
+        return os.path.join(*args).replace('\\', '/')
+
+    def normpath(path):
+        return os.path.normpath(path).replace('\\', '/')
+
+    def getcwd():
+        return os.getcwdu().replace('\\', '/')
+
+    def mkdtemp(*args, **kwargs):
+        return tempfile.mkdtemp(*args, **kwargs).replace('\\', '/')
+
+    def rename(old, new):
+        fancy_rename(old, new, rename_func=os.rename, unlink_func=os.unlink)
+
+
 def normalizepath(f):
     if hasattr(os.path, 'realpath'):
-        F = os.path.realpath
+        F = realpath
     else:
-        F = os.path.abspath
+        F = abspath
     [p,e] = os.path.split(f)
     if e == "" or e == "." or e == "..":
         return F(f)
     else:
-        return os.path.join(F(p), e)
-    
+        return pathjoin(F(p), e)
+
 
 def backup_file(fn):
     """Copy a file to a backup.
@@ -131,6 +241,10 @@
         return
     bfn = fn + '~'
 
+    if has_symlinks() and os.path.islink(fn):
+        target = os.readlink(fn)
+        os.symlink(target, bfn)
+        return
     inf = file(fn, 'rb')
     try:
         content = inf.read()
@@ -143,12 +257,6 @@
     finally:
         outf.close()
 
-if os.name == 'nt':
-    import shutil
-    rename = shutil.move
-else:
-    rename = os.rename
-
 
 def isdir(f):
     """True if f is an accessible directory."""
@@ -175,18 +283,18 @@
 def is_inside(dir, fname):
     """True if fname is inside dir.
     
-    The parameters should typically be passed to os.path.normpath first, so
+    The parameters should typically be passed to osutils.normpath first, so
     that . and .. and repeated slashes are eliminated, and the separators
     are canonical for the platform.
     
     The empty string as a dir name is taken as top-of-tree and matches 
     everything.
     
-    >>> is_inside('src', os.path.join('src', 'foo.c'))
+    >>> is_inside('src', pathjoin('src', 'foo.c'))
     True
     >>> is_inside('src', 'srccontrol')
     False
-    >>> is_inside('src', os.path.join('src', 'a', 'a', 'a', 'foo.c'))
+    >>> is_inside('src', pathjoin('src', 'a', 'a', 'a', 'foo.c'))
     True
     >>> is_inside('foo.c', 'foo.c')
     True
@@ -203,8 +311,8 @@
     if dir == '':
         return True
 
-    if dir[-1] != os.sep:
-        dir += os.sep
+    if dir[-1] != '/':
+        dir += '/'
 
     return fname.startswith(dir)
 
@@ -220,7 +328,12 @@
 
 def pumpfile(fromfile, tofile):
     """Copy contents of one file to another."""
-    tofile.write(fromfile.read())
+    BUFSIZE = 32768
+    while True:
+        b = fromfile.read(BUFSIZE)
+        if not b:
+            break
+        tofile.write(b)
 
 
 def sha_file(f):
@@ -259,130 +372,6 @@
             'sha1': s.hexdigest()}
 
 
-def config_dir():
-    """Return per-user configuration directory.
-
-    By default this is ~/.bzr.conf/
-    
-    TODO: Global option --config-dir to override this.
-    """
-    return os.path.join(os.path.expanduser("~"), ".bzr.conf")
-
-
-def _auto_user_id():
-    """Calculate automatic user identification.
-
-    Returns (realname, email).
-
-    Only used when none is set in the environment or the id file.
-
-    This previously used the FQDN as the default domain, but that can
-    be very slow on machines where DNS is broken.  So now we simply
-    use the hostname.
-    """
-    import socket
-
-    # XXX: Any good way to get real user name on win32?
-
-    try:
-        import pwd
-        uid = os.getuid()
-        w = pwd.getpwuid(uid)
-        gecos = w.pw_gecos.decode(bzrlib.user_encoding)
-        username = w.pw_name.decode(bzrlib.user_encoding)
-        comma = gecos.find(',')
-        if comma == -1:
-            realname = gecos
-        else:
-            realname = gecos[:comma]
-        if not realname:
-            realname = username
-
-    except ImportError:
-        import getpass
-        realname = username = getpass.getuser().decode(bzrlib.user_encoding)
-
-    return realname, (username + '@' + socket.gethostname())
-
-
-def _get_user_id(branch):
-    """Return the full user id from a file or environment variable.
-
-    e.g. "John Hacker <jhacker@foo.org>"
-
-    branch
-        A branch to use for a per-branch configuration, or None.
-
-    The following are searched in order:
-
-    1. $BZREMAIL
-    2. .bzr/email for this branch.
-    3. ~/.bzr.conf/email
-    4. $EMAIL
-    """
-    v = os.environ.get('BZREMAIL')
-    if v:
-        return v.decode(bzrlib.user_encoding)
-
-    if branch:
-        try:
-            return (branch.controlfile("email", "r") 
-                    .read()
-                    .decode(bzrlib.user_encoding)
-                    .rstrip("\r\n"))
-        except IOError, e:
-            if e.errno != errno.ENOENT:
-                raise
-        except BzrError, e:
-            pass
-    
-    try:
-        return (open(os.path.join(config_dir(), "email"))
-                .read()
-                .decode(bzrlib.user_encoding)
-                .rstrip("\r\n"))
-    except IOError, e:
-        if e.errno != errno.ENOENT:
-            raise e
-
-    v = os.environ.get('EMAIL')
-    if v:
-        return v.decode(bzrlib.user_encoding)
-    else:    
-        return None
-
-
-def username(branch):
-    """Return email-style username.
-
-    Something similar to 'Martin Pool <mbp@sourcefrog.net>'
-
-    TODO: Check it's reasonably well-formed.
-    """
-    v = _get_user_id(branch)
-    if v:
-        return v
-    
-    name, email = _auto_user_id()
-    if name:
-        return '%s <%s>' % (name, email)
-    else:
-        return email
-
-
-def user_email(branch):
-    """Return just the email component of a username."""
-    e = _get_user_id(branch)
-    if e:
-        m = re.search(r'[\w+.-]+@[\w+.-]+', e)
-        if not m:
-            raise BzrError("%r doesn't seem to contain "
-                           "a reasonable email address" % e)
-        return m.group(0)
-
-    return _auto_user_id()[1]
-
-
 def compare_files(a, b):
     """Returns true if equal in contents"""
     BUFSIZE = 4096
@@ -407,7 +396,8 @@
         return -time.timezone
 
     
-def format_date(t, offset=0, timezone='original'):
+def format_date(t, offset=0, timezone='original', date_fmt=None, 
+                show_offset=True):
     ## TODO: Perhaps a global option to use either universal or local time?
     ## Or perhaps just let people set $TZ?
     assert isinstance(t, float)
@@ -425,9 +415,13 @@
     else:
         raise BzrError("unsupported timezone format %r" % timezone,
                        ['options are "utc", "original", "local"'])
-
-    return (time.strftime("%a %Y-%m-%d %H:%M:%S", tt)
-            + ' %+03d%02d' % (offset / 3600, (offset / 60) % 60))
+    if date_fmt is None:
+        date_fmt = "%a %Y-%m-%d %H:%M:%S"
+    if show_offset:
+        offset_str = ' %+03d%02d' % (offset / 3600, (offset / 60) % 60)
+    else:
+        offset_str = ''
+    return (time.strftime(date_fmt, tt) +  offset_str)
 
 
 def compact_date(when):
@@ -501,28 +495,16 @@
     for f in p:
         if (f == '..') or (f == None) or (f == ''):
             raise BzrError("sorry, %r not allowed in path" % f)
-    return os.path.join(*p)
+    return pathjoin(*p)
 
 
 def appendpath(p1, p2):
     if p1 == '':
         return p2
     else:
-        return os.path.join(p1, p2)
+        return pathjoin(p1, p2)
     
 
-def _read_config_value(name):
-    """Read a config value from the file ~/.bzr.conf/<name>
-    Return None if the file does not exist"""
-    try:
-        f = file(os.path.join(config_dir(), name), "r")
-        return f.read().decode(bzrlib.user_encoding).rstrip("\r\n")
-    except IOError, e:
-        if e.errno == errno.ENOENT:
-            return None
-        raise
-
-
 def split_lines(s):
     """Split s into lines, but without removing the newline characters."""
     return StringIO(s).readlines()
@@ -550,3 +532,81 @@
         return True
     else:
         return False
+        
+
+def contains_whitespace(s):
+    """True if there are any whitespace characters in s."""
+    for ch in string.whitespace:
+        if ch in s:
+            return True
+    else:
+        return False
+
+
+def contains_linebreaks(s):
+    """True if there is any vertical whitespace in s."""
+    for ch in '\f\n\r':
+        if ch in s:
+            return True
+    else:
+        return False
+
+
+def relpath(base, path):
+    """Return path relative to base, or raise exception.
+
+    The path may be either an absolute path or a path relative to the
+    current working directory.
+
+    os.path.commonprefix (python2.4) has a bad bug that it works just
+    on string prefixes, assuming that '/u' is a prefix of '/u2'.  This
+    avoids that problem."""
+    rp = abspath(path)
+
+    s = []
+    head = rp
+    while len(head) >= len(base):
+        if head == base:
+            break
+        head, tail = os.path.split(head)
+        if tail:
+            s.insert(0, tail)
+    else:
+        # XXX This should raise a NotChildPath exception, as its not tied
+        # to branch anymore.
+        raise PathNotChild(rp, base)
+
+    if s:
+        return pathjoin(*s)
+    else:
+        return ''
+
+
+def safe_unicode(unicode_or_utf8_string):
+    """Coerce unicode_or_utf8_string into unicode.
+
+    If it is unicode, it is returned.
+    Otherwise it is decoded from utf-8. If a decoding error
+    occurs, it is wrapped as a If the decoding fails, the exception is wrapped 
+    as a BzrBadParameter exception.
+    """
+    if isinstance(unicode_or_utf8_string, unicode):
+        return unicode_or_utf8_string
+    try:
+        return unicode_or_utf8_string.decode('utf8')
+    except UnicodeDecodeError:
+        raise BzrBadParameter(unicode_or_utf8_string)
+
+
+def terminal_width():
+    """Return estimated terminal width."""
+
+    # TODO: Do something smart on Windows?
+
+    # TODO: Is there anything that gets a better update when the window
+    # is resized while the program is running? We could use the Python termcap
+    # library.
+    try:
+        return int(os.environ['COLUMNS'])
+    except (IndexError, KeyError, ValueError):
+        return 80

=== modified file 'bzrlib/plugin.py'
--- bzrlib/plugin.py	2005-09-15 21:35:53 +0000
+++ bzrlib/plugin.py	2005-12-16 03:43:14 +0000
@@ -15,55 +15,91 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 
-# This module implements plug-in support.
-# Any python module in $BZR_PLUGIN_PATH will be imported upon initialization
-# of bzrlib (and then forgotten about).  In the plugin's main body, it should
-# update any bzrlib registries it wants to extend; for example, to add new
-# commands, import bzrlib.commands and add your new command to the
-# plugin_cmds variable.
-
-
+"""bzr python plugin support
+
+Any python module in $BZR_PLUGIN_PATH will be imported upon initialization of
+bzrlib. The module will be imported as 'bzrlib.plugins.$BASENAME(PLUGIN)'.
+In the plugin's main body, it should update any bzrlib registries it wants to
+extend; for example, to add new commands, import bzrlib.commands and add your
+new command to the plugin_cmds variable.
+"""
+
+# TODO: Refactor this to make it more testable.  The main problem at the
+# moment is that loading plugins affects the global process state -- for bzr
+# in general use it's a reasonable assumption that all plugins are loaded at
+# startup and then stay loaded, but this is less good for testing.
+# 
+# Several specific issues:
+#  - plugins can't be unloaded and will continue to effect later tests
+#  - load_plugins does nothing if called a second time
+#  - plugin hooks can't be removed
+#
+# Our options are either to remove these restrictions, or work around them by
+# loading the plugins into a different space than the one running the tests.
+# That could be either a separate Python interpreter or perhaps a new
+# namespace inside this interpreter.
+
+import imp
 import os
-from bzrlib.osutils import config_dir
-DEFAULT_PLUGIN_PATH = os.path.join(config_dir(), 'plugins')
-
-all_plugins = []
+import sys
+import types
+
+import bzrlib
+from bzrlib.config import config_dir
+from bzrlib.trace import log_error, mutter, log_exception, warning, \
+        log_exception_quietly
+from bzrlib.errors import BzrError
+from bzrlib import plugins
+from bzrlib.osutils import pathjoin
+
+DEFAULT_PLUGIN_PATH = pathjoin(config_dir(), 'plugins')
+
 _loaded = False
 
 
+def all_plugins():
+    """Return a dictionary of the plugins."""
+    result = {}
+    for name, plugin in bzrlib.plugins.__dict__.items():
+        if isinstance(plugin, types.ModuleType):
+            result[name] = plugin
+    return result
+
+
 def load_plugins():
-    """
-    Find all python plugins and load them.
-
-    Loading a plugin means importing it into the python interpreter.
-    The plugin is expected to make calls to register commands when
-    it's loaded (or perhaps access other hooks in future.)
-
-    A list of plugs is stored in bzrlib.plugin.all_plugins for future
-    reference.
+    """Load bzrlib plugins.
 
     The environment variable BZR_PLUGIN_PATH is considered a delimited
     set of paths to look through. Each entry is searched for *.py
     files (and whatever other extensions are used in the platform,
     such as *.pyd).
+
+    load_from_dirs() provides the underlying mechanism and is called with
+    the default directory list to provide the normal behaviour.
     """
-
-    global all_plugins, _loaded
+    global _loaded
     if _loaded:
         # People can make sure plugins are loaded, they just won't be twice
         return
         #raise BzrError("plugins already initialized")
     _loaded = True
 
-    import sys, os, imp
-    
-    from bzrlib.trace import log_error, mutter, log_exception
-    from bzrlib.errors import BzrError
-    from bzrlib import plugins
-
-    dirs = os.environ.get('BZR_PLUGIN_PATH', DEFAULT_PLUGIN_PATH).split(":")
+    dirs = os.environ.get('BZR_PLUGIN_PATH', DEFAULT_PLUGIN_PATH).split(os.pathsep)
     dirs.insert(0, os.path.dirname(plugins.__file__))
 
+    load_from_dirs(dirs)
+
+
+def load_from_dirs(dirs):
+    """Load bzrlib plugins found in each dir in dirs.
+
+    Loading a plugin means importing it into the python interpreter.
+    The plugin is expected to make calls to register commands when
+    it's loaded (or perhaps access other hooks in future.)
+
+    Plugins are loaded into bzrlib.plugins.NAME, and can be found there
+    for future reference.
+    """
     # The problem with imp.get_suffixes() is that it doesn't include
     # .pyo which is technically valid
     # It also means that "testmodule.so" will show up as both test and testmodule
@@ -73,21 +109,19 @@
     suffixes.append(('.pyo', 'rb', imp.PY_COMPILED))
     package_entries = ['__init__.py', '__init__.pyc', '__init__.pyo']
     for d in dirs:
-        # going through them one by one allows different plugins with the same
-        # filename in different directories in the path
-        mutter('looking for plugins in %s' % d)
         if not d:
             continue
+        mutter('looking for plugins in %s', d)
         plugin_names = set()
         if not os.path.isdir(d):
             continue
         for f in os.listdir(d):
-            path = os.path.join(d, f)
+            path = pathjoin(d, f)
             if os.path.isdir(path):
                 for entry in package_entries:
                     # This directory should be a package, and thus added to
                     # the list
-                    if os.path.isfile(os.path.join(path, entry)):
+                    if os.path.isfile(pathjoin(path, entry)):
                         break
                 else: # This directory is not a package
                     continue
@@ -100,25 +134,30 @@
                         break
                 else:
                     continue
-            mutter('add plugin name %s' % f)
-            plugin_names.add(f)
+            if getattr(bzrlib.plugins, f, None):
+                mutter('Plugin name %s already loaded', f)
+            else:
+                mutter('add plugin name %s', f)
+                plugin_names.add(f)
 
         plugin_names = list(plugin_names)
         plugin_names.sort()
         for name in plugin_names:
             try:
                 plugin_info = imp.find_module(name, [d])
-                mutter('load plugin %r' % (plugin_info,))
+                mutter('load plugin %r', plugin_info)
                 try:
-                    plugin = imp.load_module('bzrlib.plugin.' + name,
+                    plugin = imp.load_module('bzrlib.plugins.' + name,
                                              *plugin_info)
-                    all_plugins.append(plugin)
+                    setattr(bzrlib.plugins, name, plugin)
                 finally:
                     if plugin_info[0] is not None:
                         plugin_info[0].close()
 
                 mutter('loaded succesfully')
-            except:
-                log_error('Unable to load plugin %r from %r' % (name, d))
-                log_exception()
-
+            except KeyboardInterrupt:
+                raise
+            except Exception, e:
+                ## import pdb; pdb.set_trace()
+                warning('Unable to load plugin %r from %r' % (name, d))
+                log_exception_quietly()

=== modified file 'bzrlib/progress.py'
--- bzrlib/progress.py	2005-09-16 09:56:24 +0000
+++ bzrlib/progress.py	2005-11-29 01:20:57 +0000
@@ -39,20 +39,7 @@
 import sys
 import time
 import os
-
-
-def _width():
-    """Return estimated terminal width.
-
-    TODO: Do something smart on Windows?
-
-    TODO: Is there anything that gets a better update when the window
-          is resized while the program is running?
-    """
-    try:
-        return int(os.environ['COLUMNS'])
-    except (IndexError, KeyError, ValueError):
-        return 80
+from collections import deque
 
 
 def _supports_progress(f):
@@ -161,11 +148,13 @@
 
 
     def __init__(self, **kwargs):
+        from bzrlib.osutils import terminal_width
         _BaseProgressBar.__init__(self, **kwargs)
         self.spin_pos = 0
-        self.width = _width()
+        self.width = terminal_width()
         self.start_time = None
         self.last_update = None
+        self.last_updates = deque()
     
 
     def throttle(self):
@@ -179,6 +168,7 @@
             if interval > 0 and interval < self.MIN_PAUSE:
                 return True
 
+        self.last_updates.append(now - self.last_update)
         self.last_update = now
         return False
         
@@ -206,7 +196,8 @@
             return 
         
         if self.show_eta and self.start_time and total_cnt:
-            eta = get_eta(self.start_time, current_cnt, total_cnt)
+            eta = get_eta(self.start_time, current_cnt, total_cnt,
+                    last_updates = self.last_updates)
             eta_str = " " + str_tdelta(eta)
         else:
             eta_str = ""
@@ -280,7 +271,7 @@
                              delt % 60)
 
 
-def get_eta(start_time, current, total, enough_samples=3):
+def get_eta(start_time, current, total, enough_samples=3, last_updates=None, n_recent=10):
     if start_time is None:
         return None
 
@@ -302,6 +293,17 @@
 
     assert total_duration >= elapsed
 
+    if last_updates and len(last_updates) >= n_recent:
+        while len(last_updates) > n_recent:
+            last_updates.popleft()
+        avg = sum(last_updates) / float(len(last_updates))
+        time_left = avg * (total - current)
+
+        old_time_left = total_duration - elapsed
+
+        # We could return the average, or some other value here
+        return (time_left + old_time_left) / 2
+
     return total_duration - elapsed
 
 

=== removed file 'bzrlib/revfile.py'
--- bzrlib/revfile.py	2005-09-26 08:56:15 +0000
+++ bzrlib/revfile.py	1970-01-01 00:00:00 +0000
@@ -1,582 +0,0 @@
-#! /usr/bin/env python
-
-# (C) 2005 Canonical Ltd
-
-# based on an idea by Matt Mackall
-# modified to squish into bzr by Martin Pool
-
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-
-
-"""Packed file revision storage.
-
-A Revfile holds the text history of a particular source file, such
-as Makefile.  It can represent a tree of text versions for that
-file, allowing for microbranches within a single repository.
-
-This is stored on disk as two files: an index file, and a data file.
-The index file is short and always read completely into memory; the
-data file is much longer and only the relevant bits of it,
-identified by the index file, need to be read.
-
-Each text version is identified by the SHA-1 of the full text of
-that version.  It also has a sequence number within the file.
-
-The index file has a short header and then a sequence of fixed-length
-records:
-
-* byte[20]    SHA-1 of text (as binary, not hex)
-* uint32      sequence number this is based on, or -1 for full text
-* uint32      flags: 1=zlib compressed
-* uint32      offset in text file of start
-* uint32      length of compressed delta in text file
-* uint32[3]   reserved
-
-total 48 bytes.
-
-The header is also 48 bytes for tidyness and easy calculation.
-
-Both the index and the text are only ever appended to; a consequence
-is that sequence numbers are stable references.  But not every
-repository in the world will assign the same sequence numbers,
-therefore the SHA-1 is the only universally unique reference.
-
-This is meant to scale to hold 100,000 revisions of a single file, by
-which time the index file will be ~4.8MB and a bit big to read
-sequentially.
-
-Some of the reserved fields could be used to implement a (semi?)
-balanced tree indexed by SHA1 so we can much more efficiently find the
-index associated with a particular hash.  For 100,000 revs we would be
-able to find it in about 17 random reads, which is not too bad.
-
-This performs pretty well except when trying to calculate deltas of
-really large files.  For that the main thing would be to plug in
-something faster than difflib, which is after all pure Python.
-Another approach is to just store the gzipped full text of big files,
-though perhaps that's too perverse?
-
-The iter method here will generally read through the whole index file
-in one go.  With readahead in the kernel and python/libc (typically
-128kB) this means that there should be no seeks and often only one
-read() call to get everything into memory.
-"""
- 
-
-# TODO: Something like pread() would make this slightly simpler and
-# perhaps more efficient.
-
-# TODO: Could also try to mmap things...  Might be faster for the
-# index in particular?
-
-# TODO: Some kind of faster lookup of SHAs?  The bad thing is that probably means
-# rewriting existing records, which is not so nice.
-
-# TODO: Something to check that regions identified in the index file
-# completely butt up and do not overlap.  Strictly it's not a problem
-# if there are gaps and that can happen if we're interrupted while
-# writing to the datafile.  Overlapping would be very bad though.
-
-# TODO: Shouldn't need to lock if we always write in append mode and
-# then ftell after writing to see where it went.  In any case we
-# assume the whole branch is protected by a lock.
-
-import os
-import sha
-import stat
-import struct
-import sys
-import zlib
-from binascii import hexlify, unhexlify
-
-import bzrlib.mdiff as mdiff
-
-
-_RECORDSIZE = 48
-
-_HEADER = "bzr revfile v1\n"
-_HEADER = _HEADER + ('\xff' * (_RECORDSIZE - len(_HEADER)))
-_NO_RECORD = 0xFFFFFFFFL
-
-# fields in the index record
-I_SHA = 0
-I_BASE = 1
-I_FLAGS = 2
-I_OFFSET = 3
-I_LEN = 4
-
-FL_GZIP = 1
-
-# maximum number of patches in a row before recording a whole text.
-CHAIN_LIMIT = 10
-
-
-class RevfileError(Exception):
-    pass
-
-class LimitHitException(Exception):
-    pass
-
-class Revfile(object):
-    def __init__(self, basename, mode):
-        # TODO: Lock file  while open
-
-        # TODO: advise of random access
-
-        self.basename = basename
-
-        if mode not in ['r', 'w']:
-            raise RevfileError("invalid open mode %r" % mode)
-        self.mode = mode
-        
-        idxname = basename + '.irev'
-        dataname = basename + '.drev'
-
-        idx_exists = os.path.exists(idxname)
-        data_exists = os.path.exists(dataname)
-
-        if idx_exists != data_exists:
-            raise RevfileError("half-assed revfile")
-        
-        if not idx_exists:
-            if mode == 'r':
-                raise RevfileError("Revfile %r does not exist" % basename)
-            
-            self.idxfile = open(idxname, 'w+b')
-            self.datafile = open(dataname, 'w+b')
-            
-            self.idxfile.write(_HEADER)
-            self.idxfile.flush()
-        else:
-            if mode == 'r':
-                diskmode = 'rb'
-            else:
-                diskmode = 'r+b'
-                
-            self.idxfile = open(idxname, diskmode)
-            self.datafile = open(dataname, diskmode)
-            
-            h = self.idxfile.read(_RECORDSIZE)
-            if h != _HEADER:
-                raise RevfileError("bad header %r in index of %r"
-                                   % (h, self.basename))
-
-
-    def _check_index(self, idx):
-        if idx < 0 or idx > len(self):
-            raise RevfileError("invalid index %r" % idx)
-
-    def _check_write(self):
-        if self.mode != 'w':
-            raise RevfileError("%r is open readonly" % self.basename)
-
-
-    def find_sha(self, s):
-        assert isinstance(s, str)
-        assert len(s) == 20
-        
-        for idx, idxrec in enumerate(self):
-            if idxrec[I_SHA] == s:
-                return idx
-        else:
-            return _NO_RECORD
-
-
-
-    def _add_compressed(self, text_sha, data, base, compress):
-        # well, maybe compress
-        flags = 0
-        if compress:
-            data_len = len(data)
-            if data_len > 50:
-                # don't do compression if it's too small; it's unlikely to win
-                # enough to be worthwhile
-                compr_data = zlib.compress(data)
-                compr_len = len(compr_data)
-                if compr_len < data_len:
-                    data = compr_data
-                    flags = FL_GZIP
-                    ##print '- compressed %d -> %d, %.1f%%' \
-                    ##      % (data_len, compr_len, float(compr_len)/float(data_len) * 100.0)
-        return self._add_raw(text_sha, data, base, flags)
-        
-
-
-    def _add_raw(self, text_sha, data, base, flags):
-        """Add pre-processed data, can be either full text or delta.
-
-        This does the compression if that makes sense."""
-        idx = len(self)
-        self.datafile.seek(0, 2)        # to end
-        self.idxfile.seek(0, 2)
-        assert self.idxfile.tell() == _RECORDSIZE * (idx + 1)
-        data_offset = self.datafile.tell()
-
-        assert isinstance(data, str) # not unicode or anything weird
-
-        self.datafile.write(data)
-        self.datafile.flush()
-
-        assert isinstance(text_sha, str)
-        entry = text_sha
-        entry += struct.pack(">IIII12x", base, flags, data_offset, len(data))
-        assert len(entry) == _RECORDSIZE
-
-        self.idxfile.write(entry)
-        self.idxfile.flush()
-
-        return idx
-        
-
-
-    def _add_full_text(self, text, text_sha, compress):
-        """Add a full text to the file.
-
-        This is not compressed against any reference version.
-
-        Returns the index for that text."""
-        return self._add_compressed(text_sha, text, _NO_RECORD, compress)
-
-
-    # NOT USED
-    def _choose_base(self, seed, base):
-        while seed & 3 == 3:
-            if base == _NO_RECORD:
-                return _NO_RECORD
-            idxrec = self[base]
-            if idxrec[I_BASE] == _NO_RECORD:
-                return base
-
-            base = idxrec[I_BASE]
-            seed >>= 2
-                
-        return base        # relative to this full text
-        
-
-
-    def _add_delta(self, text, text_sha, base, compress):
-        """Add a text stored relative to a previous text."""
-        self._check_index(base)
-
-        try:
-            base_text = self.get(base, CHAIN_LIMIT)
-        except LimitHitException:
-            return self._add_full_text(text, text_sha, compress)
-        
-        data = mdiff.bdiff(base_text, text)
-
-
-        if True: # paranoid early check for bad diff
-            result = mdiff.bpatch(base_text, data)
-            assert result == text
-            
-        
-        # If the delta is larger than the text, we might as well just
-        # store the text.  (OK, the delta might be more compressible,
-        # but the overhead of applying it probably still makes it
-        # bad, and I don't want to compress both of them to find out.)
-        if len(data) >= len(text):
-            return self._add_full_text(text, text_sha, compress)
-        else:
-            return self._add_compressed(text_sha, data, base, compress)
-
-
-    def add(self, text, base=None, compress=True):
-        """Add a new text to the revfile.
-
-        If the text is already present them its existing id is
-        returned and the file is not changed.
-
-        If compress is true then gzip compression will be used if it
-        reduces the size.
-
-        If a base index is specified, that text *may* be used for
-        delta compression of the new text.  Delta compression will
-        only be used if it would be a size win and if the existing
-        base is not at too long of a delta chain already.
-        """
-        if base == None:
-            base = _NO_RECORD
-        
-        self._check_write()
-        
-        text_sha = sha.new(text).digest()
-
-        idx = self.find_sha(text_sha)
-        if idx != _NO_RECORD:
-            # TODO: Optional paranoid mode where we read out that record and make sure
-            # it's the same, in case someone ever breaks SHA-1.
-            return idx                  # already present
-        
-        # base = self._choose_base(ord(text_sha[0]), base)
-
-        if base == _NO_RECORD:
-            return self._add_full_text(text, text_sha, compress)
-        else:
-            return self._add_delta(text, text_sha, base, compress)
-
-
-
-    def get(self, idx, recursion_limit=None):
-        """Retrieve text of a previous revision.
-
-        If recursion_limit is an integer then walk back at most that
-        many revisions and then raise LimitHitException, indicating
-        that we ought to record a new file text instead of another
-        delta.  Don't use this when trying to get out an existing
-        revision."""
-        
-        idxrec = self[idx]
-        base = idxrec[I_BASE]
-        if base == _NO_RECORD:
-            text = self._get_full_text(idx, idxrec)
-        else:
-            text = self._get_patched(idx, idxrec, recursion_limit)
-
-        if sha.new(text).digest() != idxrec[I_SHA]:
-            raise RevfileError("corrupt SHA-1 digest on record %d in %s"
-                               % (idx, self.basename))
-
-        return text
-
-
-
-    def _get_raw(self, idx, idxrec):
-        flags = idxrec[I_FLAGS]
-        if flags & ~FL_GZIP:
-            raise RevfileError("unsupported index flags %#x on index %d"
-                               % (flags, idx))
-        
-        l = idxrec[I_LEN]
-        if l == 0:
-            return ''
-
-        self.datafile.seek(idxrec[I_OFFSET])
-
-        data = self.datafile.read(l)
-        if len(data) != l:
-            raise RevfileError("short read %d of %d "
-                               "getting text for record %d in %r"
-                               % (len(data), l, idx, self.basename))
-
-        if flags & FL_GZIP:
-            data = zlib.decompress(data)
-
-        return data
-        
-
-    def _get_full_text(self, idx, idxrec):
-        assert idxrec[I_BASE] == _NO_RECORD
-
-        text = self._get_raw(idx, idxrec)
-
-        return text
-
-
-    def _get_patched(self, idx, idxrec, recursion_limit):
-        base = idxrec[I_BASE]
-        assert base >= 0
-        assert base < idx    # no loops!
-
-        if recursion_limit == None:
-            sub_limit = None
-        else:
-            sub_limit = recursion_limit - 1
-            if sub_limit < 0:
-                raise LimitHitException()
-            
-        base_text = self.get(base, sub_limit)
-        patch = self._get_raw(idx, idxrec)
-
-        text = mdiff.bpatch(base_text, patch)
-
-        return text
-
-
-
-    def __len__(self):
-        """Return number of revisions."""
-        l = os.fstat(self.idxfile.fileno())[stat.ST_SIZE]
-        if l % _RECORDSIZE:
-            raise RevfileError("bad length %d on index of %r" % (l, self.basename))
-        if l < _RECORDSIZE:
-            raise RevfileError("no header present in index of %r" % (self.basename))
-        return int(l / _RECORDSIZE) - 1
-
-
-    def __getitem__(self, idx):
-        """Index by sequence id returns the index field"""
-        ## TODO: Can avoid seek if we just moved there...
-        self._seek_index(idx)
-        idxrec = self._read_next_index()
-        if idxrec == None:
-            raise IndexError("no index %d" % idx)
-        else:
-            return idxrec
-
-
-    def _seek_index(self, idx):
-        if idx < 0:
-            raise RevfileError("invalid index %r" % idx)
-        self.idxfile.seek((idx + 1) * _RECORDSIZE)
-
-
-
-    def __iter__(self):
-        """Read back all index records.
-
-        Do not seek the index file while this is underway!"""
-        ## sys.stderr.write(" ** iter called ** \n")
-        self._seek_index(0)
-        while True:
-            idxrec = self._read_next_index()
-            if not idxrec:
-                break
-            yield idxrec
-        
-
-    def _read_next_index(self):
-        rec = self.idxfile.read(_RECORDSIZE)
-        if not rec:
-            return None
-        elif len(rec) != _RECORDSIZE:
-            raise RevfileError("short read of %d bytes getting index %d from %r"
-                               % (len(rec), idx, self.basename))
-        
-        return struct.unpack(">20sIIII12x", rec)
-
-        
-    def dump(self, f=sys.stdout):
-        f.write('%-8s %-40s %-8s %-8s %-8s %-8s\n' 
-                % tuple('idx sha1 base flags offset len'.split()))
-        f.write('-------- ---------------------------------------- ')
-        f.write('-------- -------- -------- --------\n')
-
-        for i, rec in enumerate(self):
-            f.write("#%-7d %40s " % (i, hexlify(rec[0])))
-            if rec[1] == _NO_RECORD:
-                f.write("(none)   ")
-            else:
-                f.write("#%-7d " % rec[1])
-                
-            f.write("%8x %8d %8d\n" % (rec[2], rec[3], rec[4]))
-
-
-    def total_text_size(self):
-        """Return the sum of sizes of all file texts.
-
-        This is how much space they would occupy if they were stored without
-        delta and gzip compression.
-
-        As a side effect this completely validates the Revfile, checking that all
-        texts can be reproduced with the correct SHA-1."""
-        t = 0L
-        for idx in range(len(self)):
-            t += len(self.get(idx))
-        return t
-
-
-    def check(self, pb=None):
-        """Extract every version and check its hash."""
-        total = len(self)
-        for i in range(total):
-            if pb:
-                pb.update("check revision", i, total)
-            # the get method implicitly checks the SHA-1
-            self.get(i)
-        if pb:
-            pb.clear()
-        
-
-
-def main(argv):
-    try:
-        cmd = argv[1]
-        filename = argv[2]
-    except IndexError:
-        sys.stderr.write("usage: revfile dump REVFILE\n"
-                         "       revfile add REVFILE < INPUT\n"
-                         "       revfile add-delta REVFILE BASE < INPUT\n"
-                         "       revfile add-series REVFILE BASE FILE...\n"
-                         "       revfile get REVFILE IDX\n"
-                         "       revfile find-sha REVFILE HEX\n"
-                         "       revfile total-text-size REVFILE\n"
-                         "       revfile last REVFILE\n")
-        return 1
-
-    if filename.endswith('.drev') or filename.endswith('.irev'):
-        filename = filename[:-5]
-
-    def rw():
-        return Revfile(filename, 'w')
-
-    def ro():
-        return Revfile(filename, 'r')
-
-    if cmd == 'add':
-        print rw().add(sys.stdin.read())
-    elif cmd == 'add-delta':
-        print rw().add(sys.stdin.read(), int(argv[3]))
-    elif cmd == 'add-series':
-        r = rw()
-        rev = int(argv[3])
-        for fn in argv[4:]:
-            print rev
-            rev = r.add(file(fn).read(), rev)
-    elif cmd == 'dump':
-        ro().dump()
-    elif cmd == 'get':
-        try:
-            idx = int(argv[3])
-        except IndexError:
-            sys.stderr.write("usage: revfile get FILE IDX\n")
-            return 1
-
-        r = ro()
-
-        if idx < 0 or idx >= len(r):
-            sys.stderr.write("invalid index %r\n" % idx)
-            return 1
-
-        sys.stdout.write(r.get(idx))
-    elif cmd == 'find-sha':
-        try:
-            s = unhexlify(argv[3])
-        except IndexError:
-            sys.stderr.write("usage: revfile find-sha FILE HEX\n")
-            return 1
-
-        idx = ro().find_sha(s)
-        if idx == _NO_RECORD:
-            sys.stderr.write("no such record\n")
-            return 1
-        else:
-            print idx
-    elif cmd == 'total-text-size':
-        print ro().total_text_size()
-    elif cmd == 'last':
-        print len(ro())-1
-    elif cmd == 'check':
-        import bzrlib.progress
-        pb = bzrlib.progress.ProgressBar()
-        ro().check(pb)
-    else:
-        sys.stderr.write("unknown command %r\n" % cmd)
-        return 1
-    
-
-if __name__ == '__main__':
-    import sys
-    sys.exit(main(sys.argv) or 0)

=== modified file 'bzrlib/revision.py'
--- bzrlib/revision.py	2005-10-02 22:47:02 +0000
+++ bzrlib/revision.py	2005-11-21 20:10:56 +0000
@@ -14,9 +14,12 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+# TODO: Some kind of command-line display of revision properties: 
+# perhaps show them in log -v and allow them as options to the commit command.
 
 import bzrlib.errors
 from bzrlib.graph import node_distances, select_farthest, all_descendants
+from bzrlib.osutils import contains_whitespace
 
 NULL_REVISION="null:"
 
@@ -31,13 +34,20 @@
 
     parent_ids
         List of parent revision_ids
+
+    properties
+        Dictionary of revision properties.  These are attached to the
+        revision as extra metadata.  The name must be a single 
+        word; the value can be an arbitrary string.
     """
     
-    def __init__(self, revision_id, **args):
+    def __init__(self, revision_id, properties=None, **args):
         self.revision_id = revision_id
-        self.__dict__.update(args)
+        self.properties = properties or {}
+        self._check_properties()
         self.parent_ids = []
         self.parent_sha1s = []
+        self.__dict__.update(args)
 
     def __repr__(self):
         return "<Revision id %s>" % self.revision_id
@@ -52,23 +62,21 @@
                 and self.timestamp == other.timestamp
                 and self.message == other.message
                 and self.timezone == other.timezone
-                and self.committer == other.committer)
+                and self.committer == other.committer
+                and self.properties == other.properties)
 
     def __ne__(self, other):
         return not self.__eq__(other)
 
-        
-REVISION_ID_RE = None
-
-def validate_revision_id(rid):
-    """Check rid is syntactically valid for a revision id."""
-    global REVISION_ID_RE
-    if not REVISION_ID_RE:
-        import re
-        REVISION_ID_RE = re.compile('[\w:.-]+@[\w%.-]+--?[\w]+--?[0-9a-f]+\Z')
-
-    if not REVISION_ID_RE.match(rid):
-        raise ValueError("malformed revision-id %r" % rid)
+    def _check_properties(self):
+        """Verify that all revision properties are OK.
+        """
+        for name, value in self.properties.iteritems():
+            if not isinstance(name, basestring) or contains_whitespace(name):
+                raise ValueError("invalid property name %r" % name)
+            if not isinstance(value, basestring):
+                raise ValueError("invalid property value %r for %r" % 
+                                 (name, value))
 
 
 def is_ancestor(revision_id, candidate_id, branch):
@@ -143,16 +151,16 @@
         if b_ancestors.has_key(revision):
             a_intersection.append((a_distance, a_order, revision))
             b_intersection.append((b_ancestors[revision][1], a_order, revision))
-    mutter("a intersection: %r" % a_intersection)
-    mutter("b intersection: %r" % b_intersection)
+    mutter("a intersection: %r", a_intersection)
+    mutter("b intersection: %r", b_intersection)
 
     a_closest = __get_closest(a_intersection)
     if len(a_closest) == 0:
         return None
     b_closest = __get_closest(b_intersection)
     assert len(b_closest) != 0
-    mutter ("a_closest %r" % a_closest)
-    mutter ("b_closest %r" % b_closest)
+    mutter ("a_closest %r", a_closest)
+    mutter ("b_closest %r", b_closest)
     if a_closest[0] in b_closest:
         return a_closest[0]
     elif b_closest[0] in a_closest:
@@ -168,6 +176,8 @@
     TODO: Produce graphs with the NULL revision as root, so that we can find
     a common even when trees are not branches don't represent a single line
     of descent.
+    RBC: 20051024: note that when we have two partial histories, this may not
+         be possible. But if we are willing to pretend :)... sure.
     """
     ancestors = {}
     descendants = {}
@@ -200,6 +210,29 @@
             if parents is not None:
                 ancestors[line] = set(parents)
         lines = new_lines
+    if root is None:
+        # The history for revision becomes inaccessible without
+        # actually hitting a no-parents revision. This then
+        # makes these asserts below trigger. So, if root is None
+        # determine the actual root by walking the accessible tree
+        # and then stash NULL_REVISION at the end.
+        root = NULL_REVISION
+        descendants[root] = {}
+        # for every revision, check we can access at least
+        # one parent, if we cant, add NULL_REVISION and
+        # a link
+        for rev in ancestors:
+            if len(ancestors[rev]) == 0:
+                raise RuntimeError('unreachable code ?!')
+            ok = False
+            for parent in ancestors[rev]:
+                if parent in ancestors:
+                    ok = True
+            if ok:
+                continue
+            descendants[root][rev] = 1
+            ancestors[rev].add(root)
+        ancestors[root] = set()
     assert root not in descendants[root]
     assert root not in ancestors[root]
     return root, ancestors, descendants

=== modified file 'bzrlib/revisionspec.py'
--- bzrlib/revisionspec.py	2005-09-29 20:34:25 +0000
+++ bzrlib/revisionspec.py	2005-12-30 20:54:47 +0000
@@ -57,7 +57,7 @@
         # TODO: otherwise, it should depend on how I was built -
         # if it's in_history(branch), then check revision_history(),
         # if it's in_store(branch), do the check below
-        return self.rev_id in self.branch.revision_store
+        return self.branch.revision_store.has_id(self.rev_id)
 
     def __len__(self):
         return 2
@@ -151,6 +151,16 @@
         revs = branch.revision_history()
         return self._match_on_and_check(branch, revs)
 
+        # FIXME: in_history is somewhat broken,
+        # it will return non-history revisions in many
+        # circumstances. The expected facility is that
+        # in_history only returns revision-history revs,
+        # in_store returns any rev. RBC 20051010
+    # aliases for now, when we fix the core logic, then they
+    # will do what you expect.
+    in_store = in_history
+    in_branch = in_store
+        
     def __repr__(self):
         # this is mostly for helping with testing
         return '<%s %s%s>' % (self.__class__.__name__,
@@ -194,7 +204,7 @@
         try:
             return RevisionInfo(branch, revs.index(self.spec) + 1, self.spec)
         except ValueError:
-            return RevisionInfo(branch, None)
+            return RevisionInfo(branch, None, self.spec)
 
 SPEC_TYPES.append(RevisionSpec_revid)
 
@@ -303,7 +313,7 @@
     def _match_on(self, branch, revs):
         from branch import Branch
         from revision import common_ancestor, MultipleRevisionSources
-        other_branch = Branch.open_containing(self.spec)
+        other_branch = Branch.open_containing(self.spec)[0]
         revision_a = branch.last_revision()
         revision_b = other_branch.last_revision()
         for r, b in ((revision_a, branch), (revision_b, other_branch)):
@@ -318,3 +328,27 @@
         return RevisionInfo(branch, revno, rev_id)
         
 SPEC_TYPES.append(RevisionSpec_ancestor)
+
+class RevisionSpec_branch(RevisionSpec):
+    """A branch: revision specifier.
+
+    This takes the path to a branch and returns its tip revision id.
+    """
+    prefix = 'branch:'
+
+    def _match_on(self, branch, revs):
+        from branch import Branch
+        from fetch import greedy_fetch
+        other_branch = Branch.open_containing(self.spec)[0]
+        revision_b = other_branch.last_revision()
+        if revision_b is None:
+            raise NoCommits(other_branch)
+        # pull in the remote revisions so we can diff
+        greedy_fetch(branch, other_branch, revision=revision_b)
+        try:
+            revno = branch.revision_id_to_revno(revision_b)
+        except NoSuchRevision:
+            revno = None
+        return RevisionInfo(branch, revno, revision_b)
+        
+SPEC_TYPES.append(RevisionSpec_branch)

=== added file 'bzrlib/rio.py'
--- bzrlib/rio.py	1970-01-01 00:00:00 +0000
+++ bzrlib/rio.py	2005-11-28 08:35:56 +0000
@@ -0,0 +1,224 @@
+# Copyright (C) 2005 by Canonical Ltd
+#
+# Distributed under the GNU General Public Licence v2
+#
+# \subsection{\emph{rio} - simple text metaformat}
+# 
+# \emph{r} stands for `restricted', `reproducible', or `rfc822-like'.
+# 
+# The stored data consists of a series of \emph{stanzas}, each of which contains
+# \emph{fields} identified by an ascii name, with Unicode or string contents.
+# The field tag is constrained to alphanumeric characters.  
+# There may be more than one field in a stanza with the same name.
+# 
+# The format itself does not deal with character encoding issues, though
+# the result will normally be written in Unicode.
+# 
+# The format is intended to be simple enough that there is exactly one character
+# stream representation of an object and vice versa, and that this relation
+# will continue to hold for future versions of bzr.
+
+# In comments, $\min(1,10)$
+
+min(1,10)
+ 
+import re
+
+# XXX: some redundancy is allowing to write stanzas in isolation as well as
+# through a writer object.  
+
+class RioWriter(object):
+    def __init__(self, to_file):
+        self._soft_nl = False
+        self._to_file = to_file
+
+    def write_stanza(self, stanza):
+        if self._soft_nl:
+            print >>self._to_file
+        stanza.write(self._to_file)
+        self._soft_nl = True
+
+
+class RioReader(object):
+    """Read stanzas from a file as a sequence
+    
+    to_file can be anything that can be enumerated as a sequence of 
+    lines (with newlines.)
+    """
+    def __init__(self, from_file):
+        self._from_file = from_file
+
+    def __iter__(self):
+        while True:
+            s = read_stanza(self._from_file)
+            if s is None:
+                break
+            else:
+                yield s
+
+def read_stanzas(from_file):
+    while True:
+        s = read_stanza(from_file)
+        if s is None:
+            break
+        else:
+            yield s
+
+class Stanza(object):
+    """One stanza for rio.
+
+    Each stanza contains a set of named fields.  
+    
+    Names must be non-empty ascii alphanumeric plus _.  Names can be repeated
+    within a stanza.  Names are case-sensitive.  The ordering of fields is
+    preserved.
+
+    Each field value must be either an int or a string.
+    """
+
+    __slots__ = ['items']
+
+    def __init__(self, **kwargs):
+        """Construct a new Stanza.
+
+        The keyword arguments, if any, are added in sorted order to the stanza.
+        """
+        self.items = []
+        if kwargs:
+            for tag, value in sorted(kwargs.items()):
+                self.add(tag, value)
+
+    def add(self, tag, value):
+        """Append a name and value to the stanza."""
+        assert valid_tag(tag), \
+            ("invalid tag %r" % tag)
+        if isinstance(value, (str, unicode)):
+            pass
+        ## elif isinstance(value, (int, long)):
+        ##    value = str(value)           # XXX: python2.4 without L-suffix
+        else:
+            raise ValueError("invalid value %r" % value)
+        self.items.append((tag, value))
+        
+    def __contains__(self, find_tag):
+        """True if there is any field in this stanza with the given tag."""
+        for tag, value in self.items:
+            if tag == find_tag:
+                return True
+        return False
+
+    def __len__(self):
+        """Return number of pairs in the stanza."""
+        return len(self.items)
+
+    def __eq__(self, other):
+        if not isinstance(other, Stanza):
+            return False
+        return self.items == other.items
+
+    def __ne__(self, other):
+        return not self.__eq__(other)
+
+    def __repr__(self):
+        return "Stanza(%r)" % self.items
+
+    def iter_pairs(self):
+        """Return iterator of tag, value pairs."""
+        return iter(self.items)
+
+    def to_lines(self):
+        """Generate sequence of lines for external version of this file."""
+        if not self.items:
+            # max() complains if sequence is empty
+            return []
+        result = []
+        for tag, value in self.items:
+            assert isinstance(value, (str, unicode))
+            if value == '':
+                result.append(tag + ': \n')
+            elif '\n' in value:
+                # don't want splitlines behaviour on empty lines
+                val_lines = value.split('\n')
+                result.append(tag + ': ' + val_lines[0] + '\n')
+                for line in val_lines[1:]:
+                    result.append('\t' + line + '\n')
+            else:
+                result.append(tag + ': ' + value + '\n')
+        return result
+
+    def to_string(self):
+        """Return stanza as a single string"""
+        return ''.join(self.to_lines())
+
+    def write(self, to_file):
+        """Write stanza to a file"""
+        to_file.writelines(self.to_lines())
+
+    def get(self, tag):
+        """Return the value for a field wih given tag.
+
+        If there is more than one value, only the first is returned.  If the
+        tag is not present, KeyError is raised.
+        """
+        for t, v in self.items:
+            if t == tag:
+                return v
+        else:
+            raise KeyError(tag)
+
+    __getitem__ = get
+
+    def get_all(self, tag):
+        r = []
+        for t, v in self.items:
+            if t == tag:
+                r.append(v)
+        return r
+         
+_tag_re = re.compile(r'^[-a-zA-Z0-9_]+$')
+def valid_tag(tag):
+    return bool(_tag_re.match(tag))
+
+
+def read_stanza(line_iter):
+    """Return new Stanza read from list of lines or a file
+    
+    Returns one Stanza that was read, or returns None at end of file.  If a
+    blank line follows the stanza, it is consumed.  It's not an error for
+    there to be no blank at end of file.  If there is a blank file at the
+    start of the input this is really an empty stanza and that is returned. 
+
+    Only the stanza lines and the trailing blank (if any) are consumed
+    from the line_iter.
+    """
+    items = []
+    stanza = Stanza()
+    tag = None
+    accum_value = None
+    for line in line_iter:
+        if line == None or line == '':
+            break       # end of file
+        if line == '\n':
+            break       # end of stanza
+        assert line[-1] == '\n'
+        real_l = line
+        if line[0] == '\t': # continues previous value
+            if tag is None:
+                raise ValueError('invalid continuation line %r' % real_l)
+            accum_value += '\n' + line[1:-1]
+        else: # new tag:value line
+            if tag is not None:
+                stanza.add(tag, accum_value)
+            try:
+                colon_index = line.index(': ')
+            except ValueError:
+                raise ValueError('tag/value separator not found in line %r' % real_l)
+            tag = line[:colon_index]
+            assert valid_tag(tag), \
+                    "invalid rio tag %r" % tag
+            accum_value = line[colon_index+2:-1]
+    if tag is not None: # add last tag-value
+        stanza.add(tag, accum_value)
+        return stanza
+    else:     # didn't see any content
+        return None    

=== modified file 'bzrlib/shellcomplete.py'
--- bzrlib/shellcomplete.py	2005-09-09 17:12:26 +0000
+++ bzrlib/shellcomplete.py	2005-11-18 08:10:07 +0000
@@ -28,17 +28,14 @@
         outfile.write(aname + '\n')
 
 
-def shellcomplete_on_option(options, outfile = None):
-    import commands
-    
+def shellcomplete_on_option(options, outfile=None):
+    from bzrlib.option import Option
     if not options:
         return
-    
     if outfile == None:
         outfile = sys.stdout
-
     for on in options:
-        for shortname, longname in commands.SHORT_OPTIONS.items():
+        for shortname, longname in Option.SHORT_OPTIONS.items():
             if longname == on:
                 l = '"(--' + on + ' -' + shortname + ')"{--' + on + ',-' + shortname + '}'
                 break

=== modified file 'bzrlib/status.py'
--- bzrlib/status.py	2005-10-06 13:08:14 +0000
+++ bzrlib/status.py	2005-12-01 05:07:54 +0000
@@ -17,6 +17,11 @@
 import sys
 from bzrlib.osutils import is_inside_any
 from bzrlib.delta import compare_trees
+from bzrlib.log import line_log
+from bzrlib.errors import NoSuchRevision
+
+# TODO: when showing single-line logs, truncate to the width of the terminal
+# if known, but only if really going to the terminal (not into a file)
 
 
 def show_status(branch, show_unchanged=False,
@@ -25,7 +30,14 @@
                 to_file=None,
                 show_pending=True,
                 revision=None):
-    """Display status for non-ignored working files.
+    """Display summary of changes.
+
+    By default this compares the working tree to a previous revision. 
+    If the revision argument is given, summarizes changes between the 
+    working tree and another, or between two revisions.
+
+    The result is written out as Unicode and to_file should be able 
+    to encode that.
 
     show_unchanged
         If set, includes unchanged files.
@@ -81,16 +93,40 @@
                    show_unchanged=show_unchanged)
 
         if new_is_working_tree:
-            conflicts = new.iter_conflicts()
-            unknowns = new.unknowns()
-            list_paths('unknown', unknowns, specific_files, to_file)
-            list_paths('conflicts', conflicts, specific_files, to_file)
-            if show_pending and len(branch.pending_merges()) > 0:
-                print >>to_file, 'pending merges:'
-                for merge in branch.pending_merges():
-                    print >> to_file, ' ', merge
+            list_paths('unknown', new.unknowns(), specific_files, to_file)
+            list_paths('conflicts', new.iter_conflicts(), specific_files, to_file)
+        if new_is_working_tree and show_pending:
+            show_pending_merges(new, to_file)
     finally:
         branch.unlock()
+
+def show_pending_merges(new, to_file):
+    """Write out a display of pending merges in a working tree."""
+    pending = new.pending_merges()
+    branch = new.branch
+    if len(pending) == 0:
+        return
+    print >>to_file, 'pending merges:'
+    last_revision = branch.last_revision()
+    if last_revision is not None:
+        ignore = set(branch.get_ancestry(last_revision))
+    else:
+        ignore = set()
+    for merge in new.pending_merges():
+        ignore.add(merge)
+        try:
+            m_revision = branch.get_revision(merge)
+            print >> to_file, ' ', line_log(m_revision, 77)
+            inner_merges = branch.get_ancestry(merge)
+            inner_merges.reverse()
+            for mmerge in inner_merges:
+                if mmerge in ignore:
+                    continue
+                mm_revision = branch.get_revision(mmerge)
+                print >> to_file, '   ', line_log(mm_revision, 75)
+                ignore.add(mmerge)
+        except NoSuchRevision:
+            print >> to_file, ' ', merge 
         
 def list_paths(header, paths, specific_files, to_file):
     done_header = False

=== modified file 'bzrlib/store/__init__.py'
--- bzrlib/store/__init__.py	2005-10-04 04:37:29 +0000
+++ bzrlib/store/__init__.py	2005-12-15 21:39:34 +0000
@@ -24,11 +24,16 @@
 unique ID.
 """
 
+import os
 from cStringIO import StringIO
+import urllib
+from zlib import adler32
 
+import bzrlib
+import bzrlib.errors as errors
 from bzrlib.errors import BzrError, UnlistableStore, TransportNotPossible
 from bzrlib.trace import mutter
-import bzrlib.transport
+import bzrlib.transport as transport
 from bzrlib.transport.local import LocalTransport
 
 ######################################################################
@@ -49,13 +54,20 @@
     def __len__(self):
         raise NotImplementedError('Children should define their length')
 
+    def get(self, fileid, suffix=None):
+        """Returns a file reading from a particular entry.
+        
+        If suffix is present, retrieve the named suffix for fileid.
+        """
+        raise NotImplementedError
+
     def __getitem__(self, fileid):
-        """Returns a file reading from a particular entry."""
+        """DEPRECATED. Please use .get(fileid) instead."""
         raise NotImplementedError
 
-    def __contains__(self, fileid):
-        """"""
-        raise NotImplementedError
+    #def __contains__(self, fileid):
+    #    """Deprecated, please use has_id"""
+    #    raise NotImplementedError
 
     def __iter__(self):
         raise NotImplementedError
@@ -64,48 +76,17 @@
         """Add a file object f to the store accessible from the given fileid"""
         raise NotImplementedError('Children of Store must define their method of adding entries.')
 
-    def add_multi(self, entries):
-        """Add a series of file-like or string objects to the store with the given
-        identities.
+    def has_id(self, fileid, suffix=None):
+        """Return True or false for the presence of fileid in the store.
         
-        :param entries: A list of tuples of file,id pairs [(file1, id1), (file2, id2), ...]
-                        This could also be a generator yielding (file,id) pairs.
-        """
-        for f, fileid in entries:
-            self.add(f, fileid)
-
-    def has(self, fileids):
-        """Return True/False for each entry in fileids.
-
-        :param fileids: A List or generator yielding file ids.
-        :return: A generator or list returning True/False for each entry.
-        """
-        for fileid in fileids:
-            if fileid in self:
-                yield True
-            else:
-                yield False
+        suffix, if present, is a per file suffix, i.e. for digital signature 
+        data."""
+        raise NotImplementedError
 
     def listable(self):
         """Return True if this store is able to be listed."""
         return hasattr(self, "__iter__")
 
-    def get(self, fileids, permit_failure=False, pb=None):
-        """Return a set of files, one for each requested entry.
-        
-        :param permit_failure: If true, return None for entries which do not 
-                               exist.
-        :return: A list or generator of file-like objects, one for each id.
-        """
-        for fileid in fileids:
-            try:
-                yield self[fileid]
-            except KeyError:
-                if permit_failure:
-                    yield None
-                else:
-                    raise
-
     def copy_multi(self, other, ids, pb=None, permit_failure=False):
         """Copy texts for ids from other into self.
 
@@ -122,79 +103,174 @@
         """
         if pb is None:
             pb = bzrlib.ui.ui_factory.progress_bar()
-
-        # XXX: Is there any reason why we couldn't make this accept a generator
-        # and build a list as it finds things to copy?
-        ids = list(ids) # Make sure we don't have a generator, since we iterate 2 times
         pb.update('preparing to copy')
-        to_copy = []
-        for file_id, has in zip(ids, self.has(ids)):
-            if not has:
-                to_copy.append(file_id)
-        return self._do_copy(other, to_copy, pb, permit_failure=permit_failure)
-
-    def _do_copy(self, other, to_copy, pb, permit_failure=False):
-        """This is the standard copying mechanism, just get them one at
-        a time from remote, and store them locally.
-
-        :param other: Another Store object
-        :param to_copy: A list of entry ids to copy
-        :param pb: A ProgressBar object to display completion status.
-        :param permit_failure: Allow missing entries to be ignored
-        :return: (n_copied, [failed])
-            The number of entries copied, and a list of failed entries.
-        """
-        # This should be updated to use add_multi() rather than
-        # the current methods of buffering requests.
-        # One question, is it faster to queue up 1-10 and then copy 1-10
-        # then queue up 11-20, copy 11-20
-        # or to queue up 1-10, copy 1, queue 11, copy 2, etc?
-        # sort of pipeline versus batch.
-
-        # We can't use self._transport.copy_to because we don't know
-        # whether the local tree is in the same format as other
         failed = set()
-        def buffer_requests():
-            count = 0
-            buffered_requests = []
-            for fileid in to_copy:
-                try:
-                    f = other[fileid]
-                except KeyError:
-                    if permit_failure:
-                        failed.add(fileid)
-                        continue
-                    else:
-                        raise
-
-                buffered_requests.append((f, fileid))
-                if len(buffered_requests) > self._max_buffered_requests:
-                    yield buffered_requests.pop(0)
-                    count += 1
-                    pb.update('copy', count, len(to_copy))
-
-            for req in buffered_requests:
-                yield req
-                count += 1
-                pb.update('copy', count, len(to_copy))
-
-            assert count == len(to_copy)
-
-        self.add_multi(buffer_requests())
-
+        count = 0
+        ids = list(ids) # get the list for showing a length.
+        for fileid in ids:
+            count += 1
+            if self.has_id(fileid):
+                continue
+            try:
+                self._copy_one(fileid, None, other, pb)
+                for suffix in self._suffixes:
+                    try:
+                        self._copy_one(fileid, suffix, other, pb)
+                    except KeyError:
+                        pass
+                pb.update('copy', count, len(ids))
+            except KeyError:
+                if permit_failure:
+                    failed.add(fileid)
+                else:
+                    raise
+        assert count == len(ids)
         pb.clear()
-        return len(to_copy), failed
+        return count, failed
+
+    def _copy_one(self, fileid, suffix, other, pb):
+        """Most generic copy-one object routine.
+        
+        Subclasses can override this to provide an optimised
+        copy between their own instances. Such overriden routines
+        should call this if they have no optimised facility for a 
+        specific 'other'.
+        """
+        mutter('Store._copy_one: %r', fileid)
+        f = other.get(fileid, suffix)
+        self.add(f, fileid, suffix)
 
 
 class TransportStore(Store):
     """A TransportStore is a Store superclass for Stores that use Transports."""
 
-    _max_buffered_requests = 10
-
-    def __init__(self, transport):
-        assert isinstance(transport, bzrlib.transport.Transport)
+    def add(self, f, fileid, suffix=None):
+        """Add contents of a file into the store.
+
+        f -- A file-like object, or string
+        """
+        mutter("add store entry %r", fileid)
+        
+        names = self._id_to_names(fileid, suffix)
+        if self._transport.has_any(names):
+            raise BzrError("store %r already contains id %r" 
+                           % (self._transport.base, fileid))
+
+        # Most of the time, just adding the file will work
+        # if we find a time where it fails, (because the dir
+        # doesn't exist), then create the dir, and try again
+        self._add(names[0], f)
+
+
+    def _add(self, relpath, f):
+        """Actually add the file to the given location.
+        This should be overridden by children.
+        """
+        raise NotImplementedError('children need to implement this function.')
+
+    def _check_fileid(self, fileid):
+        if not isinstance(fileid, basestring):
+            raise TypeError('Fileids should be a string type: %s %r' % (type(fileid), fileid))
+        if '\\' in fileid or '/' in fileid:
+            raise ValueError("invalid store id %r" % fileid)
+
+    def _id_to_names(self, fileid, suffix):
+        """Return the names in the expected order"""
+        if suffix is not None:
+            fn = self._relpath(fileid, [suffix])
+        else:
+            fn = self._relpath(fileid)
+
+        fn_gz = fn + '.gz'
+        if self._compressed:
+            return fn_gz, fn
+        else:
+            return fn, fn_gz
+
+    def has_id(self, fileid, suffix=None):
+        """See Store.has_id."""
+        return self._transport.has_any(self._id_to_names(fileid, suffix))
+
+    def _get_name(self, fileid, suffix=None):
+        """A special check, which returns the name of an existing file.
+        
+        This is similar in spirit to 'has_id', but it is designed
+        to return information about which file the store has.
+        """
+        for name in self._id_to_names(fileid, suffix=suffix):
+            if self._transport.has(name):
+                return name
+        return None
+
+    def _get(self, filename):
+        """Return an vanilla file stream for clients to read from.
+
+        This is the body of a template method on 'get', and should be 
+        implemented by subclasses.
+        """
+        raise NotImplementedError
+
+    def get(self, fileid, suffix=None):
+        """See Store.get()."""
+        names = self._id_to_names(fileid, suffix)
+        for name in names:
+            try:
+                return self._get(name)
+            except errors.NoSuchFile:
+                pass
+        raise KeyError(fileid)
+
+    def __init__(self, a_transport, prefixed=False, compressed=False,
+                 dir_mode=None, file_mode=None):
+        assert isinstance(a_transport, transport.Transport)
         super(TransportStore, self).__init__()
-        self._transport = transport
+        self._transport = a_transport
+        self._prefixed = prefixed
+        self._compressed = compressed
+        self._suffixes = set()
+
+        # It is okay for these to be None, it just means they
+        # will just use the filesystem defaults
+        self._dir_mode = dir_mode
+        self._file_mode = file_mode
+
+    def _iter_files_recursive(self):
+        """Iterate through the files in the transport."""
+        for quoted_relpath in self._transport.iter_files_recursive():
+            yield urllib.unquote(quoted_relpath)
+
+    def __iter__(self):
+        for relpath in self._iter_files_recursive():
+            # worst case is one of each suffix.
+            name = os.path.basename(relpath)
+            if name.endswith('.gz'):
+                name = name[:-3]
+            skip = False
+            for count in range(len(self._suffixes)):
+                for suffix in self._suffixes:
+                    if name.endswith('.' + suffix):
+                        skip = True
+            if not skip:
+                yield name
+
+    def __len__(self):
+        return len(list(self.__iter__()))
+
+    def _relpath(self, fileid, suffixes=None):
+        self._check_fileid(fileid)
+        if suffixes:
+            for suffix in suffixes:
+                if not suffix in self._suffixes:
+                    raise ValueError("Unregistered suffix %r" % suffix)
+                self._check_fileid(suffix)
+        else:
+            suffixes = []
+        if self._prefixed:
+            path = [hash_prefix(fileid) + fileid]
+        else:
+            path = [fileid]
+        path.extend(suffixes)
+        return transport.urlescape(u'.'.join(path))
 
     def __repr__(self):
         if self._transport is None:
@@ -208,102 +284,40 @@
         """Return True if this store is able to be listed."""
         return self._transport.listable()
 
-
-class ImmutableMemoryStore(Store):
-    """A memory only store."""
-
-    def __contains__(self, fileid):
-        return self._contents.has_key(fileid)
-
-    def __init__(self):
-        super(ImmutableMemoryStore, self).__init__()
-        self._contents = {}
-
-    def add(self, stream, fileid, compressed=True):
-        if self._contents.has_key(fileid):
-            raise StoreError("fileid %s already in the store" % fileid)
-        self._contents[fileid] = stream.read()
-
-    def __getitem__(self, fileid):
-        """Returns a file reading from a particular entry."""
-        if not self._contents.has_key(fileid):
-            raise IndexError
-        return StringIO(self._contents[fileid])
-
-    def _item_size(self, fileid):
-        return len(self._contents[fileid])
-
-    def __iter__(self):
-        return iter(self._contents.keys())
+    def register_suffix(self, suffix):
+        """Register a suffix as being expected in this store."""
+        self._check_fileid(suffix)
+        if suffix == 'gz':
+            raise ValueError('You cannot register the "gz" suffix.')
+        self._suffixes.add(suffix)
 
     def total_size(self):
-        result = 0
+        """Return (count, bytes)
+
+        This is the (compressed) size stored on disk, not the size of
+        the content."""
+        total = 0
         count = 0
-        for fileid in self:
+        for relpath in self._transport.iter_files_recursive():
             count += 1
-            result += self._item_size(fileid)
-        return count, result
+            total += self._transport.stat(relpath).st_size
+                
+        return count, total
+
+
+def ImmutableMemoryStore():
+    return bzrlib.store.text.TextStore(transport.memory.MemoryTransport())
         
 
-class CachedStore(Store):
-    """A store that caches data locally, to avoid repeated downloads.
-    The precacache method should be used to avoid server round-trips for
-    every piece of data.
-    """
-
-    def __init__(self, store, cache_dir):
-        super(CachedStore, self).__init__()
-        self.source_store = store
-        # This clones the source store type with a locally bound
-        # transport. FIXME: it assumes a constructor is == cloning.
-        # clonable store - it might be nicer to actually have a clone()
-        # or something. RBC 20051003
-        self.cache_store = store.__class__(LocalTransport(cache_dir))
-
-    def __getitem__(self, id):
-        mutter("Cache add %s" % id)
-        if id not in self.cache_store:
-            self.cache_store.add(self.source_store[id], id)
-        return self.cache_store[id]
-
-    def __contains__(self, fileid):
-        if fileid in self.cache_store:
-            return True
-        if fileid in self.source_store:
-            # We could copy at this time
-            return True
-        return False
-
-    def get(self, fileids, permit_failure=False, pb=None):
-        fileids = list(fileids)
-        hasids = self.cache_store.has(fileids)
-        needs = set()
-        for has, fileid in zip(hasids, fileids):
-            if not has:
-                needs.add(fileid)
-        if needs:
-            self.cache_store.copy_multi(self.source_store, needs,
-                    permit_failure=permit_failure)
-        return self.cache_store.get(fileids,
-                permit_failure=permit_failure, pb=pb)
-
-    def prefetch(self, ids):
-        """Copy a series of ids into the cache, before they are used.
-        For remote stores that support pipelining or async downloads, this can
-        increase speed considerably.
-
-        Failures while prefetching are ignored.
-        """
-        mutter("Prefetch of ids %s" % ",".join(ids))
-        self.cache_store.copy_multi(self.source_store, ids, 
-                                    permit_failure=True)
-
-
 def copy_all(store_from, store_to):
     """Copy all ids from one store to another."""
     # TODO: Optional progress indicator
     if not store_from.listable():
         raise UnlistableStore(store_from)
     ids = [f for f in store_from]
+    mutter('copy_all ids: %r', ids)
     store_to.copy_multi(store_from, ids)
 
+def hash_prefix(fileid):
+    return "%02x/" % (adler32(fileid) & 0xff)
+

=== removed file 'bzrlib/store/compressed_text.py'
--- bzrlib/store/compressed_text.py	2005-10-02 21:51:29 +0000
+++ bzrlib/store/compressed_text.py	1970-01-01 00:00:00 +0000
@@ -1,261 +0,0 @@
-# Copyright (C) 2005 by Canonical Development Ltd
-
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-
-"""
-An implementation the primary storage type CompressedTextStore.
-
-This store keeps compressed versions of the full text. It does not
-do any sort of delta compression.
-"""
-
-import os, tempfile, gzip
-
-import bzrlib.store
-from bzrlib.trace import mutter
-from bzrlib.errors import BzrError
-
-from StringIO import StringIO
-from stat import ST_SIZE
-
-class CompressedTextStore(bzrlib.store.TransportStore):
-    """Store that holds files indexed by unique names.
-
-    Files can be added, but not modified once they are in.  Typically
-    the hash is used as the name, or something else known to be unique,
-    such as a UUID.
-
-    Files are stored gzip compressed, with no delta compression.
-
-    >>> st = ScratchCompressedTextStore()
-
-    >>> st.add(StringIO('hello'), 'aa')
-    >>> 'aa' in st
-    True
-    >>> 'foo' in st
-    False
-
-    You are not allowed to add an id that is already present.
-
-    Entries can be retrieved as files, which may then be read.
-
-    >>> st.add(StringIO('goodbye'), '123123')
-    >>> st['123123'].read()
-    'goodbye'
-    """
-
-    def __init__(self, transport):
-        super(CompressedTextStore, self).__init__(transport)
-
-    def _check_fileid(self, fileid):
-        if '\\' in fileid or '/' in fileid:
-            raise ValueError("invalid store id %r" % fileid)
-
-    def _relpath(self, fileid):
-        self._check_fileid(fileid)
-        return fileid + '.gz'
-
-    def add(self, f, fileid):
-        """Add contents of a file into the store.
-
-        f -- An open file, or file-like object."""
-        # TODO: implement an add_multi which can do some of it's
-        #       own piplelining, and possible take advantage of
-        #       transport.put_multi(). The problem is that
-        #       entries potentially need to be compressed as they
-        #       are received, which implies translation, which
-        #       means it isn't as straightforward as we would like.
-        from cStringIO import StringIO
-        from bzrlib.osutils import pumpfile
-        
-        mutter("add store entry %r" % (fileid))
-        if isinstance(f, basestring):
-            f = StringIO(f)
-            
-        fn = self._relpath(fileid)
-        if self._transport.has(fn):
-            raise BzrError("store %r already contains id %r" % (self._transport.base, fileid))
-
-
-        sio = StringIO()
-        gf = gzip.GzipFile(mode='wb', fileobj=sio)
-        # if pumpfile handles files that don't fit in ram,
-        # so will this function
-        if isinstance(f, basestring):
-            gf.write(f)
-        else:
-            pumpfile(f, gf)
-        gf.close()
-        sio.seek(0)
-        self._transport.put(fn, sio)
-
-    def _do_copy(self, other, to_copy, pb, permit_failure=False):
-        if isinstance(other, CompressedTextStore):
-            return self._copy_multi_text(other, to_copy, pb,
-                    permit_failure=permit_failure)
-        return super(CompressedTextStore, self)._do_copy(other, to_copy,
-                pb, permit_failure=permit_failure)
-
-    def _copy_multi_text(self, other, to_copy, pb,
-            permit_failure=False):
-        # Because of _transport, we can no longer assume
-        # that they are on the same filesystem, we can, however
-        # assume that we only need to copy the exact bytes,
-        # we don't need to process the files.
-
-        failed = set()
-        if permit_failure:
-            new_to_copy = set()
-            for fileid, has in zip(to_copy, other.has(to_copy)):
-                if has:
-                    new_to_copy.add(fileid)
-                else:
-                    failed.add(fileid)
-            to_copy = new_to_copy
-            #mutter('_copy_multi_text copying %s, failed %s' % (to_copy, failed))
-
-        paths = [self._relpath(fileid) for fileid in to_copy]
-        count = other._transport.copy_to(paths, self._transport, pb=pb)
-        assert count == len(to_copy)
-        return count, failed
-
-    def __contains__(self, fileid):
-        """"""
-        fn = self._relpath(fileid)
-        return self._transport.has(fn)
-
-    def has(self, fileids, pb=None):
-        """Return True/False for each entry in fileids.
-
-        :param fileids: A List or generator yielding file ids.
-        :return: A generator or list returning True/False for each entry.
-        """
-        relpaths = (self._relpath(fid) for fid in fileids)
-        return self._transport.has_multi(relpaths, pb=pb)
-
-    def get(self, fileids, permit_failure=False, pb=None):
-        """Return a set of files, one for each requested entry.
-        
-        TODO: Write some tests to make sure that permit_failure is
-              handled correctly.
-
-        TODO: What should the exception be for a missing file?
-              KeyError, or NoSuchFile?
-        """
-
-        # This next code gets a bit hairy because it can allow
-        # to not request a file which doesn't seem to exist.
-        # Also, the same fileid may be requested twice, so we
-        # can't just build up a map.
-        rel_paths = [self._relpath(fid) for fid in fileids]
-        is_requested = []
-
-        #mutter('CompressedTextStore.get(permit_failure=%s)' % permit_failure)
-        if permit_failure:
-            existing_paths = []
-            for path, has in zip(rel_paths,
-                    self._transport.has_multi(rel_paths)):
-                if has:
-                    existing_paths.append(path)
-                    is_requested.append(True)
-                else:
-                    is_requested.append(False)
-            #mutter('Retrieving %s out of %s' % (existing_paths, rel_paths))
-        else:
-            #mutter('Retrieving all %s' % (rel_paths, ))
-            existing_paths = rel_paths
-            is_requested = [True for x in rel_paths]
-
-        count = 0
-        for f in self._transport.get_multi(existing_paths, pb=pb):
-            assert count < len(is_requested)
-            while not is_requested[count]:
-                yield None
-                count += 1
-            if hasattr(f, 'tell'):
-                yield gzip.GzipFile(mode='rb', fileobj=f)
-            else:
-                from cStringIO import StringIO
-                sio = StringIO(f.read())
-                yield gzip.GzipFile(mode='rb', fileobj=sio)
-            count += 1
-
-        while count < len(is_requested):
-            yield None
-            count += 1
-
-    def __iter__(self):
-        # TODO: case-insensitive?
-        for f in self._transport.list_dir('.'):
-            if f[-3:] == '.gz':
-                yield f[:-3]
-            else:
-                yield f
-
-    def __len__(self):
-        return len([f for f in self._transport.list_dir('.')])
-
-
-    def __getitem__(self, fileid):
-        """Returns a file reading from a particular entry."""
-        fn = self._relpath(fileid)
-        # This will throw if the file doesn't exist.
-        try:
-            f = self._transport.get(fn)
-        except:
-            raise KeyError('This store (%s) does not contain %s' % (self, fileid))
-
-        # gzip.GzipFile.read() requires a tell() function
-        # but some transports return objects that cannot seek
-        # so buffer them in a StringIO instead
-        if hasattr(f, 'tell'):
-            return gzip.GzipFile(mode='rb', fileobj=f)
-        else:
-            from cStringIO import StringIO
-            sio = StringIO(f.read())
-            return gzip.GzipFile(mode='rb', fileobj=sio)
-            
-
-    def total_size(self):
-        """Return (count, bytes)
-
-        This is the (compressed) size stored on disk, not the size of
-        the content."""
-        total = 0
-        count = 0
-        relpaths = [self._relpath(fid) for fid in self]
-        for st in self._transport.stat_multi(relpaths):
-            count += 1
-            total += st[ST_SIZE]
-                
-        return count, total
-
-
-class ScratchCompressedTextStore(CompressedTextStore):
-    """Self-destructing test subclass of CompressedTextStore.
-
-    The Store only exists for the lifetime of the Python object.
-    Obviously you should not put anything precious in it.
-    """
-    def __init__(self):
-        from transport import transport
-        t = transport(tempfile.mkdtemp())
-        super(ScratchCompressedTextStore, self).__init__(t)
-
-    def __del__(self):
-        self._transport.delete_multi(self._transport.list_dir('.'))
-        os.rmdir(self._transport.base)
-        mutter("%r destroyed" % self)
-

=== modified file 'bzrlib/store/text.py'
--- bzrlib/store/text.py	2005-10-02 21:51:29 +0000
+++ bzrlib/store/text.py	2005-12-15 21:11:09 +0000
@@ -21,14 +21,14 @@
 do any sort of delta compression.
 """
 
-import os, tempfile
-
+import os
 import bzrlib.store
+from bzrlib.store import hash_prefix
 from bzrlib.trace import mutter
-from bzrlib.errors import BzrError
+from bzrlib.errors import BzrError, NoSuchFile, FileExists
 
+import gzip
 from cStringIO import StringIO
-from stat import ST_SIZE
 
 
 class TextStore(bzrlib.store.TransportStore):
@@ -41,169 +41,88 @@
     Files are stored uncompressed, with no delta compression.
     """
 
-    def __init__(self, transport):
-        super(TextStore, self).__init__(transport)
-
-    def _check_fileid(self, fileid):
-        if not isinstance(fileid, basestring):
-            raise TypeError('Fileids should be a string type: %s %r' % (type(fileid), fileid))
-        if '\\' in fileid or '/' in fileid:
-            raise ValueError("invalid store id %r" % fileid)
-
-    def _relpath(self, fileid):
-        self._check_fileid(fileid)
-        return fileid
-
-    def add(self, f, fileid):
-        """Add contents of a file into the store.
-
-        f -- A file-like object, or string
-        """
-        mutter("add store entry %r" % (fileid))
-            
-        fn = self._relpath(fileid)
-        if self._transport.has(fn):
-            raise BzrError("store %r already contains id %r" % (self._transport.base, fileid))
-
-        self._transport.put(fn, f)
-
-    def _do_copy(self, other, to_copy, pb, permit_failure=False):
-        if isinstance(other, TextStore):
-            return self._copy_multi_text(other, to_copy, pb,
-                    permit_failure=permit_failure)
-        return super(TextStore, self)._do_copy(other, to_copy,
-                pb, permit_failure=permit_failure)
-
-    def _copy_multi_text(self, other, to_copy, pb,
-            permit_failure=False):
-        # Because of _transport, we can no longer assume
-        # that they are on the same filesystem, we can, however
-        # assume that we only need to copy the exact bytes,
-        # we don't need to process the files.
-
-        failed = set()
-        if permit_failure:
-            new_to_copy = set()
-            for fileid, has in zip(to_copy, other.has(to_copy)):
-                if has:
-                    new_to_copy.add(fileid)
-                else:
-                    failed.add(fileid)
-            to_copy = new_to_copy
-            #mutter('_copy_multi_text copying %s, failed %s' % (to_copy, failed))
-
-        paths = [self._relpath(fileid) for fileid in to_copy]
-        count = other._transport.copy_to(paths, self._transport, pb=pb)
-        assert count == len(to_copy)
-        return count, failed
-
-    def __contains__(self, fileid):
-        """"""
-        fn = self._relpath(fileid)
-        return self._transport.has(fn)
-
-    def has(self, fileids, pb=None):
-        """Return True/False for each entry in fileids.
-
-        :param fileids: A List or generator yielding file ids.
-        :return: A generator or list returning True/False for each entry.
-        """
-        relpaths = (self._relpath(fid) for fid in fileids)
-        return self._transport.has_multi(relpaths, pb=pb)
-
-    def get(self, fileids, permit_failure=False, pb=None):
-        """Return a set of files, one for each requested entry.
+    def _add_compressed(self, fn, f):
+        from cStringIO import StringIO
+        from bzrlib.osutils import pumpfile
         
-        TODO: Write some tests to make sure that permit_failure is
-              handled correctly.
-
-        TODO: What should the exception be for a missing file?
-              KeyError, or NoSuchFile?
-        """
-
-        # This next code gets a bit hairy because it can allow
-        # to not request a file which doesn't seem to exist.
-        # Also, the same fileid may be requested twice, so we
-        # can't just build up a map.
-        rel_paths = [self._relpath(fid) for fid in fileids]
-        is_requested = []
-
-        if permit_failure:
-            existing_paths = []
-            for path, has in zip(rel_paths,
-                    self._transport.has_multi(rel_paths)):
-                if has:
-                    existing_paths.append(path)
-                    is_requested.append(True)
-                else:
-                    is_requested.append(False)
-            #mutter('Retrieving %s out of %s' % (existing_paths, rel_paths))
-        else:
-            #mutter('Retrieving all %s' % (rel_paths, ))
-            existing_paths = rel_paths
-            is_requested = [True for x in rel_paths]
-
-        count = 0
-        for f in self._transport.get_multi(existing_paths, pb=pb):
-            assert count < len(is_requested)
-            while not is_requested[count]:
-                yield None
-                count += 1
-            yield f
-            count += 1
-
-        while count < len(is_requested):
-            yield None
-            count += 1
-
-    def __iter__(self):
-        # TODO: case-insensitive?
-        for f in self._transport.list_dir('.'):
-            yield f
-
-    def __len__(self):
-        return len([f for f in self._transport.list_dir('.')])
-
-
-    def __getitem__(self, fileid):
+        if isinstance(f, basestring):
+            f = StringIO(f)
+            
+        sio = StringIO()
+        gf = gzip.GzipFile(mode='wb', fileobj=sio)
+        # if pumpfile handles files that don't fit in ram,
+        # so will this function
+        pumpfile(f, gf)
+        gf.close()
+        sio.seek(0)
+        self._try_put(fn, sio)
+
+    def _add(self, fn, f):
+        if self._compressed:
+            self._add_compressed(fn, f)
+        else:
+            self._try_put(fn, f)
+
+    def _try_put(self, fn, f):
+        try:
+            self._transport.put(fn, f, mode=self._file_mode)
+        except NoSuchFile:
+            if not self._prefixed:
+                raise
+            try:
+                self._transport.mkdir(os.path.dirname(fn), mode=self._dir_mode)
+            except FileExists:
+                pass
+            self._transport.put(fn, f, mode=self._file_mode)
+
+    def _get(self, fn):
+        if fn.endswith('.gz'):
+            return self._get_compressed(fn)
+        else:
+            return self._transport.get(fn)
+
+    def _copy_one(self, fileid, suffix, other, pb):
+        # TODO: Once the copy_to interface is improved to allow a source
+        #       and destination targets, then we can always do the copy
+        #       as long as other is a TextStore
+        if not (isinstance(other, TextStore)
+            and other._prefixed == self._prefixed):
+            return super(TextStore, self)._copy_one(fileid, suffix, other, pb)
+
+        mutter('_copy_one: %r, %r', fileid, suffix)
+        path = other._get_name(fileid, suffix)
+        if path is None:
+            raise KeyError(fileid + '-' + str(suffix))
+
+        try:
+            result = other._transport.copy_to([path], self._transport, pb=pb,
+                                              mode=self._file_mode)
+        except NoSuchFile:
+            if not self._prefixed:
+                raise
+            try:
+                self._transport.mkdir(hash_prefix(fileid)[:-1], mode=self._dir_mode)
+            except FileExists:
+                pass
+            result = other._transport.copy_to([path], self._transport, pb=pb,
+                                              mode=self._file_mode)
+
+        if result != 1:
+            raise BzrError('Unable to copy file: %r' % (path,))
+
+    def _get_compressed(self, filename):
         """Returns a file reading from a particular entry."""
-        fn = self._relpath(fileid)
-        # This will throw if the file doesn't exist.
-        try:
-            f = self._transport.get(fn)
-        except:
-            raise KeyError('This store (%s) does not contain %s' % (self, fileid))
-
-        return f
-            
-
-    def total_size(self):
-        """Return (count, bytes)
-
-        This is the (compressed) size stored on disk, not the size of
-        the content."""
-        total = 0
-        count = 0
-        relpaths = [self._relpath(fid) for fid in self]
-        for st in self._transport.stat_multi(relpaths):
-            count += 1
-            total += st[ST_SIZE]
-                
-        return count, total
-
-
-class ScratchTextStore(TextStore):
-    """Self-destructing test subclass of TextStore.
-
-    The Store only exists for the lifetime of the Python object.
-    Obviously you should not put anything precious in it.
-    """
-    def __init__(self):
-        from transport import transport
-        super(ScratchTextStore, self).__init__(transport(tempfile.mkdtemp()))
-
-    def __del__(self):
-        self._transport.delete_multi(self._transport.list_dir('.'))
-        os.rmdir(self._transport.base)
-        mutter("%r destroyed" % self)
-
+        f = self._transport.get(filename)
+        # gzip.GzipFile.read() requires a tell() function
+        # but some transports return objects that cannot seek
+        # so buffer them in a StringIO instead
+        if hasattr(f, 'tell'):
+            return gzip.GzipFile(mode='rb', fileobj=f)
+        else:
+            from cStringIO import StringIO
+            sio = StringIO(f.read())
+            return gzip.GzipFile(mode='rb', fileobj=sio)
+
+
+def ScratchTextStore():
+    return TextStore(ScratchTransport())

=== modified file 'bzrlib/store/weave.py'
--- bzrlib/store/weave.py	2005-10-05 06:44:02 +0000
+++ bzrlib/store/weave.py	2005-12-19 20:31:01 +0000
@@ -20,104 +20,147 @@
 # files whose id differs only in case.  That should probably be forbidden.
 
 
+import errno
+import os
 from cStringIO import StringIO
-import os
-import errno
+import urllib
 
 from bzrlib.weavefile import read_weave, write_weave_v5
 from bzrlib.weave import Weave
-from bzrlib.store import Store
+from bzrlib.store import TransportStore, hash_prefix
 from bzrlib.atomicfile import AtomicFile
-from bzrlib.errors import NoSuchFile
+from bzrlib.errors import NoSuchFile, FileExists
 from bzrlib.trace import mutter
 
 
-class WeaveStore(Store):
+class WeaveStore(TransportStore):
     """Collection of several weave files in a directory.
 
     This has some shortcuts for reading and writing them.
     """
     FILE_SUFFIX = '.weave'
 
-    def __init__(self, transport):
-        self._transport = transport
-        self._cache = {}
-        self.enable_cache = False
-
+    def __init__(self, transport, prefixed=False, precious=False,
+                 dir_mode=None, file_mode=None):
+        super(WeaveStore, self).__init__(transport,
+                dir_mode=dir_mode, file_mode=file_mode,
+                prefixed=prefixed, compressed=False)
+        self._precious = precious
 
     def filename(self, file_id):
         """Return the path relative to the transport root."""
-        return file_id + WeaveStore.FILE_SUFFIX
+        if self._prefixed:
+            return hash_prefix(file_id) + urllib.quote(file_id) + WeaveStore.FILE_SUFFIX
+        else:
+            return urllib.quote(file_id) + WeaveStore.FILE_SUFFIX
 
     def __iter__(self):
         l = len(WeaveStore.FILE_SUFFIX)
-        for f in self._transport.list_dir('.'):
-            if f.endswith(WeaveStore.FILE_SUFFIX):
-                f = f[:-l]
-                yield f
+        for relpath in self._iter_files_recursive():
+            if relpath.endswith(WeaveStore.FILE_SUFFIX):
+                yield os.path.basename(relpath[:-l])
 
-    def __contains__(self, fileid):
-        """"""
+    def has_id(self, fileid):
         return self._transport.has(self.filename(fileid))
 
     def _get(self, file_id):
         return self._transport.get(self.filename(file_id))
 
     def _put(self, file_id, f):
-        return self._transport.put(self.filename(file_id), f)
+        if self._prefixed:
+            try:
+                self._transport.mkdir(hash_prefix(file_id), mode=self._dir_mode)
+            except FileExists:
+                pass
+        return self._transport.put(self.filename(file_id), f, mode=self._file_mode)
 
-    def get_weave(self, file_id):
-        if self.enable_cache:
-            if file_id in self._cache:
-                mutter("cache hit in %s for %s", self, file_id)
-                return self._cache[file_id]
+    def get_weave(self, file_id, transaction):
+        weave = transaction.map.find_weave(file_id)
+        if weave:
+            mutter("cache hit in %s for %s", self, file_id)
+            return weave
         w = read_weave(self._get(file_id))
-        if self.enable_cache:
-            self._cache[file_id] = w
-        return w
-
-
-    def get_lines(self, file_id, rev_id):
+        transaction.map.add_weave(file_id, w)
+        transaction.register_clean(w, precious=self._precious)
+        # TODO: jam 20051219 This should check if there is a prelude
+        #       which is already cached, and if so, should remove it
+        #       But transaction doesn't seem to have a 'remove'
+        #       One workaround would be to re-add the object with
+        #       the PRELUDE marker.
+        return w
+
+    def get_weave_prelude(self, file_id, transaction):
+        weave_id = file_id
+        weave = transaction.map.find_weave(weave_id)
+        if weave:
+            mutter("cache hit in %s for %s", self, weave_id)
+            return weave
+        # We want transactions to also cache preludes if that
+        # is all that we are loading. So we need a unique
+        # identifier, so that someone who wants the whole text
+        # won't get just the prelude
+        weave_id = 'PRELUDE-' + file_id
+        weave = transaction.map.find_weave(weave_id)
+        if weave:
+            mutter("cache hit in %s for %s", self, weave_id)
+            return weave
+        w = read_weave(self._get(file_id), prelude=True)
+        transaction.map.add_weave(weave_id, w)
+        transaction.register_clean(w, precious=self._precious)
+        return w
+
+    def get_lines(self, file_id, rev_id, transaction):
         """Return text from a particular version of a weave.
 
         Returned as a list of lines."""
-        w = self.get_weave(file_id)
+        w = self.get_weave(file_id, transaction)
         return w.get(w.lookup(rev_id))
     
+    def get_weave_prelude_or_empty(self, file_id, transaction):
+        """cheap version that reads the prelude but not the lines
+        """
+        try:
+            return self.get_weave_prelude(file_id, transaction)
+        except NoSuchFile:
+            # We can cache here, because we know that there
+            # is no complete object, since we got NoSuchFile
+            weave = Weave(weave_name=file_id)
+            transaction.map.add_weave(file_id, weave)
+            transaction.register_clean(weave, precious=self._precious)
+            return weave
 
-    def get_weave_or_empty(self, file_id):
+    def get_weave_or_empty(self, file_id, transaction):
         """Return a weave, or an empty one if it doesn't exist.""" 
         try:
-            inf = self._get(file_id)
+            return self.get_weave(file_id, transaction)
         except NoSuchFile:
-            return Weave(weave_name=file_id)
-        else:
-            return read_weave(inf)
-    
+            weave = Weave(weave_name=file_id)
+            transaction.map.add_weave(file_id, weave)
+            transaction.register_clean(weave, precious=self._precious)
+            return weave
 
-    def put_weave(self, file_id, weave):
+    def put_weave(self, file_id, weave, transaction):
         """Write back a modified weave"""
-        if self.enable_cache:
-            self._cache[file_id] = weave
-
+        transaction.register_dirty(weave)
+        # TODO FOR WRITE TRANSACTIONS: this should be done in a callback
+        # from the transaction, when it decides to save.
         sio = StringIO()
         write_weave_v5(weave, sio)
         sio.seek(0)
-
         self._put(file_id, sio)
 
-
-    def add_text(self, file_id, rev_id, new_lines, parents):
-        w = self.get_weave_or_empty(file_id)
+    def add_text(self, file_id, rev_id, new_lines, parents, transaction):
+        w = self.get_weave_or_empty(file_id, transaction)
         parent_idxs = map(w.lookup, parents)
         w.add(rev_id, parent_idxs, new_lines)
-        self.put_weave(file_id, w)
+        self.put_weave(file_id, w, transaction)
         
-    def add_identical_text(self, file_id, old_rev_id, new_rev_id, parents):
-        w = self.get_weave_or_empty(file_id)
+    def add_identical_text(self, file_id, old_rev_id, new_rev_id, parents,
+                           transaction):
+        w = self.get_weave_or_empty(file_id, transaction)
         parent_idxs = map(w.lookup, parents)
         w.add_identical(old_rev_id, new_rev_id, parent_idxs)
-        self.put_weave(file_id, w)
+        self.put_weave(file_id, w, transaction)
      
     def copy_multi(self, from_store, file_ids):
         assert isinstance(from_store, WeaveStore)

=== added file 'bzrlib/symbol_versioning.py'
--- bzrlib/symbol_versioning.py	1970-01-01 00:00:00 +0000
+++ bzrlib/symbol_versioning.py	2006-01-05 21:48:56 +0000
@@ -0,0 +1,114 @@
+# Copyright (C) 2006 by Canonical Ltd
+#   Authors: Robert Collins <robert.collins@canonical.com>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Symbol versioning
+
+The methods here allow for api symbol versioning.
+"""
+
+__all__ = ['warn', 'set_warning_method', 'zero_seven']
+
+from warnings import warn
+
+
+zero_seven = "%s was deprecated in version 0.7."
+
+
+def set_warning_method(method):
+    """Set the warning method to be used by this module.
+
+    It should take a message and a warning category as warnings.warn does.
+    """
+    global warn
+    warn = method
+
+
+# TODO - maybe this would be easier to use as one 'smart' method that
+# guess if it is a method or a class or an attribute ? If so, we can
+# add that on top of the primitives, once we have all three written
+# - RBC 20050105
+
+def deprecated_function(deprecation_version):
+    """Decorate a function so that use of it will trigger a warning."""
+
+    def function_decorator(callable):
+        """This is the function python calls to perform the decoration."""
+        
+        def decorated_function(*args, **kwargs):
+            """This is the decorated function."""
+            symbol = "%s.%s" % (callable.__module__, 
+                                callable.__name__
+                                )
+            warn(deprecation_version % symbol, DeprecationWarning)
+            return callable(*args, **kwargs)
+        _populate_decorated(callable, deprecation_version, "function",
+                            decorated_function)
+        return decorated_function
+    return function_decorator
+
+
+def deprecated_method(deprecation_version):
+    """Decorate a method so that use of it will trigger a warning.
+    
+    To deprecate an entire class, decorate __init__.
+    """
+
+    def method_decorator(callable):
+        """This is the function python calls to perform the decoration."""
+        
+        def decorated_method(self, *args, **kwargs):
+            """This is the decorated method."""
+            symbol = "%s.%s.%s" % (self.__class__.__module__, 
+                                   self.__class__.__name__,
+                                   callable.__name__
+                                   )
+            warn(deprecation_version % symbol, DeprecationWarning)
+            return callable(self, *args, **kwargs)
+        _populate_decorated(callable, deprecation_version, "method",
+                            decorated_method)
+        return decorated_method
+    return method_decorator
+
+
+def _decorate_docstring(callable, deprecation_version, label,
+                        decorated_callable):
+    docstring_lines = callable.__doc__.split('\n')
+    if len(docstring_lines) == 0:
+        decorated_callable.__doc__ = deprecation_version % ("This " + label)
+    elif len(docstring_lines) == 1:
+        decorated_callable.__doc__ = (callable.__doc__ 
+                                    + "\n"
+                                    + "\n"
+                                    + deprecation_version % ("This " + label)
+                                    + "\n")
+    else:
+        spaces = len(docstring_lines[-1])
+        new_doc = callable.__doc__
+        new_doc += "\n" + " " * spaces
+        new_doc += deprecation_version % ("This " + label)
+        new_doc += "\n" + " " * spaces
+        decorated_callable.__doc__ = new_doc
+
+
+def _populate_decorated(callable, deprecation_version, label,
+                        decorated_callable):
+    """Populate attributes like __name__ and __doc__ on the decorated callable.
+    """
+    _decorate_docstring(callable, deprecation_version, label,
+                        decorated_callable)
+    decorated_callable.__module__ = callable.__module__
+    decorated_callable.__name__ = callable.__name__

=== added file 'bzrlib/testament.py'
--- bzrlib/testament.py	1970-01-01 00:00:00 +0000
+++ bzrlib/testament.py	2005-10-19 09:34:39 +0000
@@ -0,0 +1,187 @@
+# Copyright (C) 2005 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Testament - a summary of a revision for signing.
+
+A testament can be defined as "something that serves as tangible 
+proof or evidence."  In bzr we use them to allow people to certify
+particular revisions as authentic.  
+
+The goal is that if two revisions are semantically equal, then they will
+have a byte-for-byte equal testament.  We can define different versions of
+"semantically equal" by using different testament classes; e.g. one that
+includes or ignores file-ids.
+
+We sign a testament rather than the revision XML itself for several reasons.
+The most important is that the form in which the revision is stored
+internally is designed for that purpose, and contains information which need
+not be attested to by the signer.  For example the inventory contains the
+last-changed revision for a file, but this is not necessarily something the
+user cares to sign.
+
+Having unnecessary fields signed makes the signatures brittle when the same
+revision is stored in different branches or when the format is upgraded.
+
+Handling upgrades is another motivation for using testaments separate from
+the stored revision.  We would like to be able to compare a signature
+generated from an old-format tree to newer tree, or vice versa.  This could
+be done by comparing the revisions but that makes it unclear about exactly
+what is being compared or not.
+
+Different signing keys might indicate different levels of trust; we can in
+the future extend this to allow signatures indicating not just that a
+particular version is authentic but that it has other properties.
+
+The signature can be applied to either the full testament or to just a
+hash of it.
+
+Testament format 1
+~~~~~~~~~~~~~~~~~~
+
+* timestamps are given as integers to avoid rounding errors
+* parents given in lexicographical order
+* indented-text form similar to log; intended to be human readable
+* paths are given with forward slashes
+* files are named using paths for ease of comparison/debugging
+* the testament uses unix line-endings (\n)
+"""
+
+# XXX: At the moment, clients trust that the graph described in a weave
+# is accurate, but that's not covered by the testament.  Perhaps the best
+# fix is when verifying a revision to make sure that every file mentioned 
+# in the revision has compatible ancestry links.
+
+# TODO: perhaps write timestamp in a more readable form
+
+# TODO: Perhaps these should just be different formats in which inventories/
+# revisions can be serialized.
+
+from copy import copy
+from cStringIO import StringIO
+import string
+from sha import sha
+
+from bzrlib.osutils import contains_whitespace, contains_linebreaks
+
+class Testament(object):
+    """Reduced summary of a revision.
+
+    Testaments can be 
+
+      - produced from a revision
+      - written to a stream
+      - loaded from a stream
+      - compared to a revision
+    """
+
+    @classmethod
+    def from_revision(cls, branch, revision_id):
+        """Produce a new testament from a historical revision"""
+        rev = branch.get_revision(revision_id)
+        inventory = branch.get_inventory(revision_id)
+        return cls(rev, inventory)
+
+    def __init__(self, rev, inventory):
+        """Create a new testament for rev using inventory."""
+        self.revision_id = str(rev.revision_id)
+        self.committer = rev.committer
+        self.timezone = rev.timezone or 0
+        self.timestamp = rev.timestamp
+        self.message = rev.message
+        self.parent_ids = rev.parent_ids[:]
+        self.inventory = inventory
+        self.revprops = copy(rev.properties)
+        assert not contains_whitespace(self.revision_id)
+        assert not contains_linebreaks(self.committer)
+
+    def as_text_lines(self):
+        """Yield text form as a sequence of lines.
+
+        The result is returned in utf-8, because it should be signed or
+        hashed in that encoding.
+        """
+        r = []
+        def a(s):
+            r.append(s)
+        a('bazaar-ng testament version 1\n')
+        a('revision-id: %s\n' % self.revision_id)
+        a('committer: %s\n' % self.committer)
+        a('timestamp: %d\n' % self.timestamp)
+        a('timezone: %d\n' % self.timezone)
+        # inventory length contains the root, which is not shown here
+        a('parents:\n')
+        for parent_id in sorted(self.parent_ids):
+            assert not contains_whitespace(parent_id)
+            a('  %s\n' % parent_id)
+        a('message:\n')
+        for l in self.message.splitlines():
+            a('  %s\n' % l)
+        a('inventory:\n')
+        for path, ie in self.inventory.iter_entries():
+            a(self._entry_to_line(path, ie))
+        r.extend(self._revprops_to_lines())
+        if __debug__:
+            for l in r:
+                assert isinstance(l, basestring), \
+                    '%r of type %s is not a plain string' % (l, type(l))
+        return r
+
+    def _escape_path(self, path):
+        assert not contains_linebreaks(path)
+        return unicode(path.replace('\\', '/').replace(' ', '\ ')).encode('utf-8')
+
+    def _entry_to_line(self, path, ie):
+        """Turn an inventory entry into a testament line"""
+        l = '  ' + str(ie.kind)
+        l += ' ' + self._escape_path(path)
+        assert not contains_whitespace(ie.file_id)
+        l += ' ' + unicode(ie.file_id).encode('utf-8')
+        if ie.kind == 'file':
+            # TODO: avoid switching on kind
+            assert ie.text_sha1
+            l += ' ' + ie.text_sha1
+        elif ie.kind == 'symlink':
+            assert ie.symlink_target
+            l += ' ' + self._escape_path(ie.symlink_target)
+        l += '\n'
+        return l
+
+    def as_text(self):
+        return ''.join(self.as_text_lines())
+
+    def as_short_text(self):
+        """Return short digest-based testament."""
+        s = sha()
+        map(s.update, self.as_text_lines())
+        return ('bazaar-ng testament short form 1\n'
+                'revision-id: %s\n'
+                'sha1: %s\n'
+                % (self.revision_id, s.hexdigest()))
+
+    def _revprops_to_lines(self):
+        """Pack up revision properties."""
+        if not self.revprops:
+            return []
+        r = ['properties:\n']
+        for name, value in sorted(self.revprops.items()):
+            assert isinstance(name, str)
+            assert not contains_whitespace(name)
+            r.append('  %s:\n' % name)
+            for line in value.splitlines():
+                if not isinstance(line, str):
+                    line = line.encode('utf-8')
+                r.append('    %s\n' % line)
+        return r

=== renamed directory 'bzrlib/selftest' => 'bzrlib/tests'
=== modified file 'bzrlib/tests/HTTPTestUtil.py'
--- bzrlib/selftest/HTTPTestUtil.py	2005-09-29 20:34:25 +0000
+++ bzrlib/tests/HTTPTestUtil.py	2006-01-12 01:02:32 +0000
@@ -14,132 +14,31 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-import BaseHTTPServer, SimpleHTTPServer, socket, errno, time
-from bzrlib.selftest import TestCaseInTempDir
-
-
-class WebserverNotAvailable(Exception):
-    pass
-
-class BadWebserverPath(ValueError):
-    def __str__(self):
-        return 'path %s is not in %s' % self.args
-
-class TestingHTTPRequestHandler(SimpleHTTPServer.SimpleHTTPRequestHandler):
-    def log_message(self, format, *args):
-        self.server.test_case.log("webserver - %s - - [%s] %s" %
-                                  (self.address_string(),
-                                   self.log_date_time_string(),
-                                   format%args))
-
-    def handle_one_request(self):
-        """Handle a single HTTP request.
-
-        You normally don't need to override this method; see the class
-        __doc__ string for information on how to handle specific HTTP
-        commands such as GET and POST.
-
-        """
-        for i in xrange(1,11): # Don't try more than 10 times
-            try:
-                self.raw_requestline = self.rfile.readline()
-            except socket.error, e:
-                if e.args[0] == errno.EAGAIN:
-                    self.log_message('EAGAIN (%d) while reading from raw_requestline' % i)
-                    time.sleep(0.01)
-                    continue
-                raise
-            else:
-                break
-        if not self.raw_requestline:
-            self.close_connection = 1
-            return
-        if not self.parse_request(): # An error code has been sent, just exit
-            return
-        mname = 'do_' + self.command
-        if not hasattr(self, mname):
-            self.send_error(501, "Unsupported method (%r)" % self.command)
-            return
-        method = getattr(self, mname)
-        method()
-
-class TestingHTTPServer(BaseHTTPServer.HTTPServer):
-    def __init__(self, server_address, RequestHandlerClass, test_case):
-        BaseHTTPServer.HTTPServer.__init__(self, server_address,
-                                                RequestHandlerClass)
-        self.test_case = test_case
+import os
+
+
+from bzrlib.tests import TestCaseInTempDir
+from bzrlib.transport.http import HttpServer
+from bzrlib.osutils import relpath
 
 
 class TestCaseWithWebserver(TestCaseInTempDir):
-    """Derived class that starts a localhost-only webserver
+    """Derived class that starts a localhost-only webserver.
     (in addition to what TestCaseInTempDir does).
 
-    This is useful for testing RemoteBranch.
+    This is useful for testing things with a web server.
     """
 
-    _HTTP_PORTS = range(13000, 0x8000)
-
-    def _http_start(self):
-        import SimpleHTTPServer, BaseHTTPServer, socket, errno
-        httpd = None
-        for port in self._HTTP_PORTS:
-            try:
-                httpd = TestingHTTPServer(('localhost', port),
-                                          TestingHTTPRequestHandler,
-                                          self)
-            except socket.error, e:
-                if e.args[0] == errno.EADDRINUSE:
-                    continue
-                print >>sys.stderr, "Cannot run webserver :-("
-                raise
-            else:
-                break
-
-        if httpd is None:
-            raise WebserverNotAvailable("Cannot run webserver :-( "
-                                        "no free ports in range %s..%s" %
-                                        (_HTTP_PORTS[0], _HTTP_PORTS[-1]))
-
-        self._http_base_url = 'http://localhost:%s/' % port
-        self._http_starting.release()
-        httpd.socket.settimeout(0.1)
-
-        while self._http_running:
-            try:
-                httpd.handle_request()
-            except socket.timeout:
-                pass
-
     def get_remote_url(self, path):
-        import os
 
-        path_parts = path.split(os.path.sep)
         if os.path.isabs(path):
-            if path_parts[:len(self._local_path_parts)] != \
-                   self._local_path_parts:
-                raise BadWebserverPath(path, self.test_dir)
-            remote_path = '/'.join(path_parts[len(self._local_path_parts):])
+            remote_path = relpath(self.test_dir, path)
         else:
-            remote_path = '/'.join(path_parts)
-
-        self._http_starting.acquire()
-        self._http_starting.release()
-        return self._http_base_url + remote_path
+            remote_path = path
+        return self.server.get_url() + remote_path
 
     def setUp(self):
-        TestCaseInTempDir.setUp(self)
-        import threading, os
-        self._local_path_parts = self.test_dir.split(os.path.sep)
-        self._http_starting = threading.Lock()
-        self._http_starting.acquire()
-        self._http_running = True
-        self._http_base_url = None
-        self._http_thread = threading.Thread(target=self._http_start)
-        self._http_thread.setDaemon(True)
-        self._http_thread.start()
-
-    def tearDown(self):
-        self._http_running = False
-        self._http_thread.join()
-        TestCaseInTempDir.tearDown(self)
-
+        super(TestCaseWithWebserver, self).setUp()
+        self.server = HttpServer()
+        self.server.setUp()
+        self.addCleanup(self.server.tearDown)

=== modified file 'bzrlib/tests/__init__.py'
--- bzrlib/selftest/__init__.py	2005-10-06 22:15:52 +0000
+++ bzrlib/tests/__init__.py	2006-01-12 06:37:23 +0000
@@ -15,29 +15,56 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 
+# TODO: Perhaps there should be an API to find out if bzr running under the
+# test suite -- some plugins might want to avoid making intrusive changes if
+# this is the case.  However, we want behaviour under to test to diverge as
+# little as possible, so this should be used rarely if it's added at all.
+# (Suggestion from j-a-meinel, 2005-11-24)
+
 from cStringIO import StringIO
+import difflib
+import errno
 import logging
-import unittest
-import tempfile
 import os
+import re
+import shutil
+import stat
 import sys
-import errno
-import subprocess
-import shutil
-import re
+import tempfile
+import unittest
+import time
+import codecs
 
+import bzrlib.branch
 import bzrlib.commands
+from bzrlib.errors import BzrError
+import bzrlib.inventory
+import bzrlib.merge3
+import bzrlib.osutils
+import bzrlib.osutils as osutils
+import bzrlib.plugin
+import bzrlib.store
 import bzrlib.trace
-import bzrlib.fetch
-from bzrlib.selftest import TestUtil
-from bzrlib.selftest.TestUtil import TestLoader, TestSuite
-
+from bzrlib.transport import urlescape
+from bzrlib.trace import mutter
+from bzrlib.tests.TestUtil import TestLoader, TestSuite
+from bzrlib.tests.treeshape import build_tree_contents
 
 MODULES_TO_TEST = []
-MODULES_TO_DOCTEST = []
-
-from logging import debug, warning, error
-
+MODULES_TO_DOCTEST = [
+                      bzrlib.branch,
+                      bzrlib.commands,
+                      bzrlib.errors,
+                      bzrlib.inventory,
+                      bzrlib.merge3,
+                      bzrlib.osutils,
+                      bzrlib.store,
+                      ]
+def packages_to_test():
+    import bzrlib.tests.blackbox
+    return [
+            bzrlib.tests.blackbox
+            ]
 
 
 class EarlyStoppingTestResultAdapter(object):
@@ -64,53 +91,100 @@
 
 
 class _MyResult(unittest._TextTestResult):
-    """
-    Custom TestResult.
-
-    No special behaviour for now.
-    """
+    """Custom TestResult.
+
+    Shows output in a different format, including displaying runtime for tests.
+    """
+
+    def _elapsedTime(self):
+        return "%5dms" % (1000 * (time.time() - self._start_time))
 
     def startTest(self, test):
         unittest.TestResult.startTest(self, test)
-        # TODO: Maybe show test.shortDescription somewhere?
-        what = test.shortDescription() or test.id()        
+        # In a short description, the important words are in
+        # the beginning, but in an id, the important words are
+        # at the end
+        SHOW_DESCRIPTIONS = False
         if self.showAll:
-            self.stream.write('%-70.70s' % what)
+            width = osutils.terminal_width()
+            name_width = width - 15
+            what = None
+            if SHOW_DESCRIPTIONS:
+                what = test.shortDescription()
+                if what:
+                    if len(what) > name_width:
+                        what = what[:name_width-3] + '...'
+            if what is None:
+                what = test.id()
+                if what.startswith('bzrlib.tests.'):
+                    what = what[13:]
+                if len(what) > name_width:
+                    what = '...' + what[3-name_width:]
+            what = what.ljust(name_width)
+            self.stream.write(what)
         self.stream.flush()
+        self._start_time = time.time()
 
     def addError(self, test, err):
-        super(_MyResult, self).addError(test, err)
+        if isinstance(err[1], TestSkipped):
+            return self.addSkipped(test, err)    
+        unittest.TestResult.addError(self, test, err)
+        if self.showAll:
+            self.stream.writeln("ERROR %s" % self._elapsedTime())
+        elif self.dots:
+            self.stream.write('E')
         self.stream.flush()
 
     def addFailure(self, test, err):
-        super(_MyResult, self).addFailure(test, err)
+        unittest.TestResult.addFailure(self, test, err)
+        if self.showAll:
+            self.stream.writeln(" FAIL %s" % self._elapsedTime())
+        elif self.dots:
+            self.stream.write('F')
         self.stream.flush()
 
     def addSuccess(self, test):
         if self.showAll:
-            self.stream.writeln('OK')
+            self.stream.writeln('   OK %s' % self._elapsedTime())
         elif self.dots:
             self.stream.write('~')
         self.stream.flush()
         unittest.TestResult.addSuccess(self, test)
 
+    def addSkipped(self, test, skip_excinfo):
+        if self.showAll:
+            print >>self.stream, ' SKIP %s' % self._elapsedTime()
+            print >>self.stream, '     %s' % skip_excinfo[1]
+        elif self.dots:
+            self.stream.write('S')
+        self.stream.flush()
+        # seems best to treat this as success from point-of-view of unittest
+        # -- it actually does nothing so it barely matters :)
+        unittest.TestResult.addSuccess(self, test)
+
     def printErrorList(self, flavour, errors):
         for test, err in errors:
             self.stream.writeln(self.separator1)
-            self.stream.writeln("%s: %s" % (flavour,self.getDescription(test)))
+            self.stream.writeln("%s: %s" % (flavour, self.getDescription(test)))
             if hasattr(test, '_get_log'):
-                self.stream.writeln()
-                self.stream.writeln('log from this test:')
+                print >>self.stream
+                print >>self.stream, \
+                        ('vvvv[log from %s]' % test.id()).ljust(78,'-')
                 print >>self.stream, test._get_log()
+                print >>self.stream, \
+                        ('^^^^[log from %s]' % test.id()).ljust(78,'-')
             self.stream.writeln(self.separator2)
             self.stream.writeln("%s" % err)
 
 
 class TextTestRunner(unittest.TextTestRunner):
+    stop_on_failure = False
 
     def _makeResult(self):
         result = _MyResult(self.stream, self.descriptions, self.verbosity)
-        return EarlyStoppingTestResultAdapter(result)
+        if self.stop_on_failure:
+            result = EarlyStoppingTestResultAdapter(result)
+        return result
 
 
 def iter_suite_tests(suite):
@@ -142,55 +216,193 @@
 
     Error and debug log messages are redirected from their usual
     location into a temporary file, the contents of which can be
-    retrieved by _get_log().
+    retrieved by _get_log().  We use a real OS file, not an in-memory object,
+    so that it can also capture file IO.  When the test completes this file
+    is read into memory and removed from disk.
        
     There are also convenience functions to invoke bzr's command-line
-    routine, and to build and check bzr trees."""
+    routine, and to build and check bzr trees.
+   
+    In addition to the usual method of overriding tearDown(), this class also
+    allows subclasses to register functions into the _cleanups list, which is
+    run in order as the object is torn down.  It's less likely this will be
+    accidentally overlooked.
+    """
 
     BZRPATH = 'bzr'
+    _log_file_name = None
+    _log_contents = ''
 
     def setUp(self):
         unittest.TestCase.setUp(self)
+        self._cleanups = []
+        self._cleanEnvironment()
         bzrlib.trace.disable_default_logging()
-        self._enable_file_logging()
-
-
-    def _enable_file_logging(self):
+        self._startLogFile()
+
+    def _ndiff_strings(self, a, b):
+        """Return ndiff between two strings containing lines.
+        
+        A trailing newline is added if missing to make the strings
+        print properly."""
+        if b and b[-1] != '\n':
+            b += '\n'
+        if a and a[-1] != '\n':
+            a += '\n'
+        difflines = difflib.ndiff(a.splitlines(True),
+                                  b.splitlines(True),
+                                  linejunk=lambda x: False,
+                                  charjunk=lambda x: False)
+        return ''.join(difflines)
+
+    def assertEqualDiff(self, a, b):
+        """Assert two texts are equal, if not raise an exception.
+        
+        This is intended for use with multi-line strings where it can 
+        be hard to find the differences by eye.
+        """
+        # TODO: perhaps override assertEquals to call this for strings?
+        if a == b:
+            return
+        raise AssertionError("texts not equal:\n" + 
+                             self._ndiff_strings(a, b))      
+        
+    def assertStartsWith(self, s, prefix):
+        if not s.startswith(prefix):
+            raise AssertionError('string %r does not start with %r' % (s, prefix))
+
+    def assertEndsWith(self, s, suffix):
+        if not s.endswith(prefix):
+            raise AssertionError('string %r does not end with %r' % (s, suffix))
+
+    def assertContainsRe(self, haystack, needle_re):
+        """Assert that a contains something matching a regular expression."""
+        if not re.search(needle_re, haystack):
+            raise AssertionError('pattern "%s" not found in "%s"'
+                    % (needle_re, haystack))
+
+    def AssertSubset(self, sublist, superlist):
+        """Assert that every entry in sublist is present in superlist."""
+        missing = []
+        for entry in sublist:
+            if entry not in superlist:
+                missing.append(entry)
+        if len(missing) > 0:
+            raise AssertionError("value(s) %r not present in container %r" % 
+                                 (missing, superlist))
+
+    def assertTransportMode(self, transport, path, mode):
+        """Fail if a path does not have mode mode.
+        
+        If modes are not supported on this platform, the test is skipped.
+        """
+        if sys.platform == 'win32':
+            return
+        path_stat = transport.stat(path)
+        actual_mode = stat.S_IMODE(path_stat.st_mode)
+        self.assertEqual(mode, actual_mode,
+            'mode of %r incorrect (%o != %o)' % (path, mode, actual_mode))
+
+    def _startLogFile(self):
+        """Send bzr and test log messages to a temporary file.
+
+        The file is removed as the test is torn down.
+        """
         fileno, name = tempfile.mkstemp(suffix='.log', prefix='testbzr')
-
-        self._log_file = os.fdopen(fileno, 'w+')
-
-        hdlr = logging.StreamHandler(self._log_file)
-        hdlr.setLevel(logging.DEBUG)
-        hdlr.setFormatter(logging.Formatter('%(levelname)8s  %(message)s'))
-        logging.getLogger('').addHandler(hdlr)
-        logging.getLogger('').setLevel(logging.DEBUG)
-        self._log_hdlr = hdlr
-        debug('opened log file %s', name)
-        
+        encoder, decoder, stream_reader, stream_writer = codecs.lookup('UTF-8')
+        self._log_file = stream_writer(os.fdopen(fileno, 'w+'))
+        bzrlib.trace.enable_test_log(self._log_file)
         self._log_file_name = name
-
-    def tearDown(self):
-        logging.getLogger('').removeHandler(self._log_hdlr)
-        bzrlib.trace.enable_default_logging()
-        logging.debug('%s teardown', self.id())
+        self.addCleanup(self._finishLogFile)
+
+    def _finishLogFile(self):
+        """Finished with the log file.
+
+        Read contents into memory, close, and delete.
+        """
+        bzrlib.trace.disable_test_log()
+        self._log_file.seek(0)
+        self._log_contents = self._log_file.read()
         self._log_file.close()
+        os.remove(self._log_file_name)
+        self._log_file = self._log_file_name = None
+
+    def addCleanup(self, callable):
+        """Arrange to run a callable when this case is torn down.
+
+        Callables are run in the reverse of the order they are registered, 
+        ie last-in first-out.
+        """
+        if callable in self._cleanups:
+            raise ValueError("cleanup function %r already registered on %s" 
+                    % (callable, self))
+        self._cleanups.append(callable)
+
+    def _cleanEnvironment(self):
+        new_env = {
+            'HOME': os.getcwd(),
+            'APPDATA': os.getcwd(),
+            'BZREMAIL': None,
+            'EMAIL': None,
+        }
+        self.__old_env = {}
+        self.addCleanup(self._restoreEnvironment)
+        for name, value in new_env.iteritems():
+            self._captureVar(name, value)
+
+
+    def _captureVar(self, name, newvalue):
+        """Set an environment variable, preparing it to be reset when finished."""
+        self.__old_env[name] = os.environ.get(name, None)
+        if newvalue is None:
+            if name in os.environ:
+                del os.environ[name]
+        else:
+            os.environ[name] = newvalue
+
+    @staticmethod
+    def _restoreVar(name, value):
+        if value is None:
+            if name in os.environ:
+                del os.environ[name]
+        else:
+            os.environ[name] = value
+
+    def _restoreEnvironment(self):
+        for name, value in self.__old_env.iteritems():
+            self._restoreVar(name, value)
+
+    def tearDown(self):
+        self._runCleanups()
         unittest.TestCase.tearDown(self)
 
+    def _runCleanups(self):
+        """Run registered cleanup functions. 
+
+        This should only be called from TestCase.tearDown.
+        """
+        # TODO: Perhaps this should keep running cleanups even if 
+        # one of them fails?
+        for cleanup_fn in reversed(self._cleanups):
+            cleanup_fn()
+
     def log(self, *args):
-        logging.debug(*args)
+        mutter(*args)
 
     def _get_log(self):
         """Return as a string the log for this test"""
-        return open(self._log_file_name).read()
-
-
-    def capture(self, cmd):
+        if self._log_file_name:
+            return open(self._log_file_name).read()
+        else:
+            return self._log_contents
+        # TODO: Delete the log after it's been read in
+
+    def capture(self, cmd, retcode=0):
         """Shortcut that splits cmd into words, runs, and returns stdout"""
-        return self.run_bzr_captured(cmd.split())[0]
+        return self.run_bzr_captured(cmd.split(), retcode=retcode)[0]
 
     def run_bzr_captured(self, argv, retcode=0):
-        """Invoke bzr and return (result, stdout, stderr).
+        """Invoke bzr and return (stdout, stderr).
 
         Useful for code that wants to check the contents of the
         output, the way error messages are presented, etc.
@@ -212,6 +424,7 @@
         stdout = StringIO()
         stderr = StringIO()
         self.log('run bzr: %s', ' '.join(argv))
+        # FIXME: don't call into logging here
         handler = logging.StreamHandler(stderr)
         handler.setFormatter(bzrlib.trace.QuietFormatter())
         handler.setLevel(logging.INFO)
@@ -332,7 +545,7 @@
             return
         i = 0
         while True:
-            root = 'test%04d.tmp' % i
+            root = u'test%04d.tmp' % i
             try:
                 os.mkdir(root)
             except OSError, e:
@@ -342,82 +555,88 @@
                 else:
                     raise
             # successfully created
-            TestCaseInTempDir.TEST_ROOT = os.path.abspath(root)
+            TestCaseInTempDir.TEST_ROOT = osutils.abspath(root)
             break
         # make a fake bzr directory there to prevent any tests propagating
         # up onto the source directory's real branch
-        os.mkdir(os.path.join(TestCaseInTempDir.TEST_ROOT, '.bzr'))
+        os.mkdir(osutils.pathjoin(TestCaseInTempDir.TEST_ROOT, '.bzr'))
 
     def setUp(self):
         super(TestCaseInTempDir, self).setUp()
         self._make_test_root()
-        self._currentdir = os.getcwdu()
-        short_id = self.id().replace('bzrlib.selftest.', '') \
+        _currentdir = os.getcwdu()
+        short_id = self.id().replace('bzrlib.tests.', '') \
                    .replace('__main__.', '')
-        self.test_dir = os.path.join(self.TEST_ROOT, short_id)
+        self.test_dir = osutils.pathjoin(self.TEST_ROOT, short_id)
         os.mkdir(self.test_dir)
         os.chdir(self.test_dir)
+        os.environ['HOME'] = self.test_dir
+        os.environ['APPDATA'] = self.test_dir
+        def _leaveDirectory():
+            os.chdir(_currentdir)
+        self.addCleanup(_leaveDirectory)
         
-    def tearDown(self):
-        os.chdir(self._currentdir)
-        super(TestCaseInTempDir, self).tearDown()
-
-    def build_tree(self, shape):
+    def build_tree(self, shape, line_endings='native', transport=None):
         """Build a test tree according to a pattern.
 
         shape is a sequence of file specifications.  If the final
         character is '/', a directory is created.
 
         This doesn't add anything to a branch.
+        :param line_endings: Either 'binary' or 'native'
+                             in binary mode, exact contents are written
+                             in native mode, the line endings match the
+                             default platform endings.
+
+        :param transport: A transport to write to, for building trees on 
+                          VFS's. If the transport is readonly or None,
+                          "." is opened automatically.
         """
         # XXX: It's OK to just create them using forward slashes on windows?
+        if transport is None or transport.is_readonly():
+            transport = bzrlib.transport.get_transport(".")
         for name in shape:
-            assert isinstance(name, basestring)
+            self.assert_(isinstance(name, basestring))
             if name[-1] == '/':
-                os.mkdir(name[:-1])
+                transport.mkdir(urlescape(name[:-1]))
             else:
-                f = file(name, 'wt')
-                print >>f, "contents of", name
-                f.close()
+                if line_endings == 'binary':
+                    end = '\n'
+                elif line_endings == 'native':
+                    end = os.linesep
+                else:
+                    raise BzrError('Invalid line ending request %r' % (line_endings,))
+                content = "contents of %s%s" % (name, end)
+                transport.put(urlescape(name), StringIO(content))
+
+    def build_tree_contents(self, shape):
+        build_tree_contents(shape)
 
     def failUnlessExists(self, path):
         """Fail unless path, which may be abs or relative, exists."""
-        self.failUnless(os.path.exists(path))
+        self.failUnless(osutils.lexists(path))
+
+    def failIfExists(self, path):
+        """Fail if path, which may be abs or relative, exists."""
+        self.failIf(osutils.lexists(path))
         
-
-class MetaTestLog(TestCase):
-    def test_logging(self):
-        """Test logs are captured when a test fails."""
-        logging.info('an info message')
-        warning('something looks dodgy...')
-        logging.debug('hello, test is running')
-        ##assert 0
+    def assertFileEqual(self, content, path):
+        """Fail if path does not contain 'content'."""
+        self.failUnless(osutils.lexists(path))
+        self.assertEqualDiff(content, open(path, 'r').read())
 
 
 def filter_suite_by_re(suite, pattern):
-    result = TestUtil.TestSuite()
+    result = TestSuite()
     filter_re = re.compile(pattern)
     for test in iter_suite_tests(suite):
-        if filter_re.match(test.id()):
+        if filter_re.search(test.id()):
             result.addTest(test)
     return result
 
 
-def filter_suite_by_names(suite, wanted_names):
-    """Return a new suite containing only selected tests.
-    
-    Names are considered to match if any name is a substring of the 
-    fully-qualified test id (i.e. the class ."""
-    result = TestSuite()
-    for test in iter_suite_tests(suite):
-        this_id = test.id()
-        for p in wanted_names:
-            if this_id.find(p) != -1:
-                result.addTest(test)
-    return result
-
-
-def run_suite(suite, name='test', verbose=False, pattern=".*", testnames=None):
+def run_suite(suite, name='test', verbose=False, pattern=".*",
+              stop_on_failure=False, keep_output=False):
     TestCaseInTempDir._TEST_NAME = name
     if verbose:
         verbosity = 2
@@ -426,15 +645,14 @@
     runner = TextTestRunner(stream=sys.stdout,
                             descriptions=0,
                             verbosity=verbosity)
-    if testnames:
-        suite = filter_suite_by_names(suite, testnames)
+    runner.stop_on_failure=stop_on_failure
     if pattern != '.*':
         suite = filter_suite_by_re(suite, pattern)
     result = runner.run(suite)
     # This is still a little bogus, 
     # but only a little. Folk not using our testrunner will
     # have to delete their temp directories themselves.
-    if result.wasSuccessful():
+    if result.wasSuccessful() or not keep_output:
         if TestCaseInTempDir.TEST_ROOT is not None:
             shutil.rmtree(TestCaseInTempDir.TEST_ROOT) 
     else:
@@ -442,71 +660,120 @@
     return result.wasSuccessful()
 
 
-def selftest(verbose=False, pattern=".*", testnames=None):
+def selftest(verbose=False, pattern=".*", stop_on_failure=True,
+             keep_output=False):
     """Run the whole test suite under the enhanced runner"""
     return run_suite(test_suite(), 'testbzr', verbose=verbose, pattern=pattern,
-                     testnames=testnames)
+                     stop_on_failure=stop_on_failure, keep_output=keep_output)
 
 
 def test_suite():
     """Build and return TestSuite for the whole program."""
-    import bzrlib.store, bzrlib.inventory, bzrlib.branch
-    import bzrlib.osutils, bzrlib.merge3, bzrlib.plugin
     from doctest import DocTestSuite
 
-    global MODULES_TO_TEST, MODULES_TO_DOCTEST
+    global MODULES_TO_DOCTEST
 
-    testmod_names = \
-                  ['bzrlib.selftest.MetaTestLog',
-                   'bzrlib.selftest.testinv',
-                   'bzrlib.selftest.test_ancestry',
-                   'bzrlib.selftest.test_commit',
-                   'bzrlib.selftest.test_commit_merge',
-                   'bzrlib.selftest.versioning',
-                   'bzrlib.selftest.testmerge3',
-                   'bzrlib.selftest.testmerge',
-                   'bzrlib.selftest.testhashcache',
-                   'bzrlib.selftest.teststatus',
-                   'bzrlib.selftest.testlog',
-                   'bzrlib.selftest.testrevisionnamespaces',
-                   'bzrlib.selftest.testbranch',
-                   'bzrlib.selftest.testrevision',
-                   'bzrlib.selftest.test_revision_info',
-                   'bzrlib.selftest.test_merge_core',
-                   'bzrlib.selftest.test_smart_add',
-                   'bzrlib.selftest.test_bad_files',
-                   'bzrlib.selftest.testdiff',
-                   'bzrlib.selftest.test_parent',
-                   'bzrlib.selftest.test_xml',
-                   'bzrlib.selftest.test_weave',
-                   'bzrlib.selftest.testfetch',
-                   'bzrlib.selftest.whitebox',
-                   'bzrlib.selftest.teststore',
-                   'bzrlib.selftest.blackbox',
-                   'bzrlib.selftest.testsampler',
-                   'bzrlib.selftest.testtransport',
-                   'bzrlib.selftest.testgraph',
-                   'bzrlib.selftest.testworkingtree',
-                   'bzrlib.selftest.test_upgrade',
-                   'bzrlib.selftest.test_conflicts',
+    testmod_names = [ \
+                   'bzrlib.tests.test_ancestry',
+                   'bzrlib.tests.test_annotate',
+                   'bzrlib.tests.test_api',
+                   'bzrlib.tests.test_bad_files',
+                   'bzrlib.tests.test_basis_inventory',
+                   'bzrlib.tests.test_branch',
+                   'bzrlib.tests.test_command',
+                   'bzrlib.tests.test_commit',
+                   'bzrlib.tests.test_commit_merge',
+                   'bzrlib.tests.test_config',
+                   'bzrlib.tests.test_conflicts',
+                   'bzrlib.tests.test_diff',
+                   'bzrlib.tests.test_fetch',
+                   'bzrlib.tests.test_gpg',
+                   'bzrlib.tests.test_graph',
+                   'bzrlib.tests.test_hashcache',
+                   'bzrlib.tests.test_http',
+                   'bzrlib.tests.test_identitymap',
+                   'bzrlib.tests.test_inv',
+                   'bzrlib.tests.test_log',
+                   'bzrlib.tests.test_merge',
+                   'bzrlib.tests.test_merge3',
+                   'bzrlib.tests.test_merge_core',
+                   'bzrlib.tests.test_missing',
+                   'bzrlib.tests.test_msgeditor',
+                   'bzrlib.tests.test_nonascii',
+                   'bzrlib.tests.test_options',
+                   'bzrlib.tests.test_osutils',
+                   'bzrlib.tests.test_parent',
+                   'bzrlib.tests.test_permissions',
+                   'bzrlib.tests.test_plugins',
+                   'bzrlib.tests.test_remove',
+                   'bzrlib.tests.test_revision',
+                   'bzrlib.tests.test_revisionnamespaces',
+                   'bzrlib.tests.test_revprops',
+                   'bzrlib.tests.test_reweave',
+                   'bzrlib.tests.test_rio',
+                   'bzrlib.tests.test_sampler',
+                   'bzrlib.tests.test_selftest',
+                   'bzrlib.tests.test_setup',
+                   'bzrlib.tests.test_sftp_transport',
+                   'bzrlib.tests.test_smart_add',
+                   'bzrlib.tests.test_source',
+                   'bzrlib.tests.test_status',
+                   'bzrlib.tests.test_store',
+                   'bzrlib.tests.test_symbol_versioning',
+                   'bzrlib.tests.test_testament',
+                   'bzrlib.tests.test_trace',
+                   'bzrlib.tests.test_transactions',
+                   'bzrlib.tests.test_transport',
+                   'bzrlib.tests.test_tsort',
+                   'bzrlib.tests.test_ui',
+                   'bzrlib.tests.test_uncommit',
+                   'bzrlib.tests.test_upgrade',
+                   'bzrlib.tests.test_weave',
+                   'bzrlib.tests.test_whitebox',
+                   'bzrlib.tests.test_workingtree',
+                   'bzrlib.tests.test_xml',
                    ]
-
-    for m in (bzrlib.store, bzrlib.inventory, bzrlib.branch,
-              bzrlib.osutils, bzrlib.commands, bzrlib.merge3):
-        if m not in MODULES_TO_DOCTEST:
-            MODULES_TO_DOCTEST.append(m)
-
-    TestCase.BZRPATH = os.path.join(os.path.realpath(os.path.dirname(bzrlib.__path__[0])), 'bzr')
-    print '%-30s %s' % ('bzr binary', TestCase.BZRPATH)
+    test_transport_implementations = [
+        'bzrlib.tests.test_transport_implementations']
+
+    TestCase.BZRPATH = osutils.pathjoin(
+            osutils.realpath(osutils.dirname(bzrlib.__path__[0])), 'bzr')
+    print '%10s: %s' % ('bzr', osutils.realpath(sys.argv[0]))
+    print '%10s: %s' % ('bzrlib', bzrlib.__path__[0])
     print
     suite = TestSuite()
-    suite.addTest(TestLoader().loadTestsFromNames(testmod_names))
+    # python2.4's TestLoader.loadTestsFromNames gives very poor 
+    # errors if it fails to load a named module - no indication of what's
+    # actually wrong, just "no such module".  We should probably override that
+    # class, but for the moment just load them ourselves. (mbp 20051202)
+    loader = TestLoader()
+    from bzrlib.transport import TransportTestProviderAdapter
+    adapter = TransportTestProviderAdapter()
+    for mod_name in test_transport_implementations:
+        mod = _load_module_by_name(mod_name)
+        for test in iter_suite_tests(loader.loadTestsFromModule(mod)):
+            suite.addTests(adapter.adapt(test))
+    for mod_name in testmod_names:
+        mod = _load_module_by_name(mod_name)
+        suite.addTest(loader.loadTestsFromModule(mod))
+    for package in packages_to_test():
+        suite.addTest(package.test_suite())
     for m in MODULES_TO_TEST:
-         suite.addTest(TestLoader().loadTestsFromModule(m))
+        suite.addTest(loader.loadTestsFromModule(m))
     for m in (MODULES_TO_DOCTEST):
         suite.addTest(DocTestSuite(m))
-    for p in bzrlib.plugin.all_plugins:
-        if hasattr(p, 'test_suite'):
-            suite.addTest(p.test_suite())
+    for name, plugin in bzrlib.plugin.all_plugins().items():
+        if hasattr(plugin, 'test_suite'):
+            suite.addTest(plugin.test_suite())
     return suite
 
+
+def _load_module_by_name(mod_name):
+    parts = mod_name.split('.')
+    module = __import__(mod_name)
+    del parts[0]
+    # for historical reasons python returns the top-level module even though
+    # it loads the submodule; we need to walk down to get the one we want.
+    while parts:
+        module = getattr(module, parts.pop(0))
+    return module

=== added directory 'bzrlib/tests/blackbox'
=== added file 'bzrlib/tests/blackbox/__init__.py'
--- bzrlib/tests/blackbox/__init__.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/__init__.py	2006-01-03 15:33:37 +0000
@@ -0,0 +1,51 @@
+# Copyright (C) 2005 by Canonical Ltd
+# -*- coding: utf-8 -*-
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+"""Black-box tests for bzr.
+
+These check that it behaves properly when it's invoked through the regular
+command-line interface. This doesn't actually run a new interpreter but 
+rather starts again from the run_bzr function.
+"""
+
+from bzrlib.tests import TestLoader
+from bzrlib.tests import TestCaseInTempDir, BzrTestBase
+
+def test_suite():
+    testmod_names = [
+                     'bzrlib.tests.blackbox.test_cat',
+                     'bzrlib.tests.blackbox.test_export',
+                     'bzrlib.tests.blackbox.test_missing',
+                     'bzrlib.tests.blackbox.test_pull',
+                     'bzrlib.tests.blackbox.test_revno',
+                     'bzrlib.tests.blackbox.test_revision_info',
+                     'bzrlib.tests.blackbox.test_too_much',
+                     'bzrlib.tests.blackbox.test_versioning',
+                     ]
+    return TestLoader().loadTestsFromNames(testmod_names)
+
+
+class ExternalBase(TestCaseInTempDir):
+
+    def runbzr(self, args, retcode=0, backtick=False):
+        if isinstance(args, basestring):
+            args = args.split()
+        if backtick:
+            return self.run_bzr_captured(args, retcode=retcode)[0]
+        else:
+            return self.run_bzr_captured(args, retcode=retcode)

=== added file 'bzrlib/tests/blackbox/test_cat.py'
--- bzrlib/tests/blackbox/test_cat.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_cat.py	2005-12-01 16:42:05 +0000
@@ -0,0 +1,54 @@
+# Copyright (C) 2005 by Canonical Ltd
+# -*- coding: utf-8 -*-
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+"""Black-box tests for bzr cat.
+"""
+
+import os
+
+from bzrlib.branch import Branch
+from bzrlib.tests import TestCaseInTempDir
+
+class TestCat(TestCaseInTempDir):
+
+    def test_cat(self):
+
+        def bzr(*args, **kwargs):
+            return self.run_bzr(*args, **kwargs)[0]
+
+        bzr('init')
+        open('a', 'wb').write('foo\n')
+        bzr('add', 'a')
+
+        # 'bzr cat' without an option should cat the last revision
+        bzr('cat', 'a', retcode=3)
+
+        bzr('commit', '-m', '1')
+        open('a', 'wb').write('baz\n')
+
+        self.assertEquals(bzr('cat', 'a'), 'foo\n')
+
+        bzr('commit', '-m', '2')
+        self.assertEquals(bzr('cat', 'a'), 'baz\n')
+        self.assertEquals(bzr('cat', 'a', '-r', '1'), 'foo\n')
+        self.assertEquals(bzr('cat', 'a', '-r', '-1'), 'baz\n')
+
+        rev_id = bzr('revision-history').strip().split('\n')[-1]
+
+        self.assertEquals(bzr('cat', 'a', '-r', 'revid:%s' % rev_id), 'baz\n')
+

=== added file 'bzrlib/tests/blackbox/test_export.py'
--- bzrlib/tests/blackbox/test_export.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_export.py	2005-12-30 03:25:20 +0000
@@ -0,0 +1,74 @@
+# -*- coding: utf-8 -*-
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+"""Black-box tests for bzr export.
+"""
+
+import os
+
+from bzrlib.branch import Branch
+from bzrlib.tests.blackbox import ExternalBase
+import tarfile, zipfile
+
+class TestExport(ExternalBase):
+
+    def test_tar_export(self):
+
+        os.mkdir('tar')
+        os.chdir('tar')
+
+        self.runbzr('init')
+        open('a', 'wb').write('foo\n')
+        self.runbzr('add a')
+        self.runbzr('ignore something')
+        self.runbzr('commit -m 1')
+        self.runbzr('export test.tar.gz')
+        ball = tarfile.open('test.tar.gz')
+        for m in ball.getnames():
+            #print m
+            self.failIf(os.path.basename(m) == '.bzrignore', 'tar export contains .bzrignore')
+
+    def test_zip_export(self):
+
+        os.mkdir('zip')
+        os.chdir('zip')
+
+        self.runbzr('init')
+        open('a', 'wb').write('foo\n')
+        self.runbzr('add a')
+        self.runbzr('ignore something')
+        self.runbzr('commit -m 1')
+        self.runbzr('export test.zip')
+        ball = zipfile.ZipFile('test.zip')
+        for m in ball.namelist():
+            #print m
+            self.failIf(os.path.basename(m) == '.bzrignore', 'zip export contains .bzrignore')
+
+    def test_dir_export(self):
+
+        os.mkdir('dir')
+        os.chdir('dir')
+
+        self.runbzr('init')
+        open('a', 'wb').write('foo\n')
+        self.runbzr('add a')
+        self.runbzr('ignore something')
+        self.runbzr('commit -m 1')
+        self.runbzr('export direxport')
+
+        files = sorted(os.listdir('direxport'))
+        self.assertEqual(['a'], files, 'dir contains .bzrignore')

=== added file 'bzrlib/tests/blackbox/test_missing.py'
--- bzrlib/tests/blackbox/test_missing.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_missing.py	2005-12-11 23:26:56 +0000
@@ -0,0 +1,82 @@
+"""Black-box tests for bzr missing.
+"""
+
+import os
+
+from bzrlib.branch import Branch
+from bzrlib.tests import TestCaseInTempDir
+
+class TestMissing(TestCaseInTempDir):
+    def test_missing(self):
+        missing = "You are missing 1 revision(s):"
+
+        # create a source branch
+        os.mkdir('a')
+        os.chdir('a')
+        self.capture('init')
+        open('a', 'wb').write('initial\n')
+        self.capture('add a')
+        self.capture('commit -m inital')
+
+        # clone and add a differing revision
+        self.capture('branch . ../b')
+        os.chdir('../b')
+        open('a', 'ab').write('more\n')
+        self.capture('commit -m more')
+
+        # compare a against b
+        os.chdir('../a')
+        lines = self.capture('missing ../b', retcode=1).splitlines()
+        # we're missing the extra revision here
+        self.assertEqual(missing, lines[0])
+        self.assertEqual(8, len(lines))
+
+        # get extra revision from b
+        self.capture('merge ../b')
+        self.capture('commit -m merge')
+
+        # compare again, but now we have the 'merge' commit extra
+        lines = self.capture('missing ../b', retcode=1).splitlines()
+        self.assertEqual("You have 1 extra revision(s):", lines[0])
+        self.assertEqual(8, len(lines))
+        lines2 = self.capture('missing ../b --mine-only', retcode=1)
+        lines2 = lines2.splitlines()
+        self.assertEqual(lines, lines2)
+        lines3 = self.capture('missing ../b --theirs-only', retcode=1)
+        lines3 = lines3.splitlines()
+        self.assertEqual(0, len(lines3))
+
+        # relative to a, missing the 'merge' commit 
+        os.chdir('../b')
+        lines = self.capture('missing ../a', retcode=1).splitlines()
+        self.assertEqual(missing, lines[0])
+        self.assertEqual(8, len(lines))
+        lines2 = self.capture('missing ../a --theirs-only', retcode=1)
+        lines2 = lines2.splitlines()
+        self.assertEqual(lines, lines2)
+        lines3 = self.capture('missing ../a --mine-only', retcode=1)
+        lines3 = lines3.splitlines()
+        self.assertEqual(0, len(lines3))
+        lines4 = self.capture('missing ../a --short', retcode=1)
+        lines4 = lines4.splitlines()
+        self.assertEqual(4, len(lines4))
+        lines5 = self.capture('missing ../a --line', retcode=1)
+        lines5 = lines5.splitlines()
+        self.assertEqual(2, len(lines5))
+        lines6 = self.capture('missing ../a --reverse', retcode=1)
+        lines6 = lines6.splitlines()
+        self.assertEqual(lines6, lines)
+        lines7 = self.capture('missing ../a --show-ids', retcode=1)
+        lines7 = lines7.splitlines()
+        self.assertEqual(11, len(lines7))
+        lines8 = self.capture('missing ../a --verbose', retcode=1)
+        lines8 = lines8.splitlines()
+        self.assertEqual("modified:", lines8[-2])
+        self.assertEqual("  a", lines8[-1])
+
+        
+        # after a pull we're back on track
+        self.capture('pull')
+        self.assertEqual("Branches are up to date.\n", 
+                         self.capture('missing ../a'))
+

=== added file 'bzrlib/tests/blackbox/test_pull.py'
--- bzrlib/tests/blackbox/test_pull.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_pull.py	2005-12-01 21:49:24 +0000
@@ -0,0 +1,198 @@
+# Copyright (C) 2005 by Canonical Ltd
+# -*- coding: utf-8 -*-
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+"""Black-box tests for bzr pull.
+"""
+
+import os
+import sys
+
+from bzrlib.branch import Branch
+from bzrlib.tests.blackbox import ExternalBase
+
+class TestPull(ExternalBase):
+
+    def example_branch(test):
+        test.runbzr('init')
+        file('hello', 'wt').write('foo')
+        test.runbzr('add hello')
+        test.runbzr('commit -m setup hello')
+        file('goodbye', 'wt').write('baz')
+        test.runbzr('add goodbye')
+        test.runbzr('commit -m setup goodbye')
+
+    def test_pull(self):
+        """Pull changes from one branch to another."""
+        os.mkdir('a')
+        os.chdir('a')
+
+        self.example_branch()
+        self.runbzr('pull', retcode=3)
+        self.runbzr('missing', retcode=3)
+        self.runbzr('missing .')
+        self.runbzr('missing')
+        if sys.platform not in ('win32', 'cygwin'):
+            # This is equivalent to doing "bzr pull ."
+            # Which means that bzr creates 2 branches grabbing
+            # the same location, and tries to pull.
+            # However, 2 branches mean 2 locks on the same file
+            # which ultimately implies a deadlock.
+            # (non windows platforms allow multiple locks on the
+            # same file by the same calling process)
+            self.runbzr('pull')
+        self.runbzr('pull /', retcode=3)
+        if sys.platform not in ('win32', 'cygwin'):
+            self.runbzr('pull')
+
+        os.chdir('..')
+        self.runbzr('branch a b')
+        os.chdir('b')
+        self.runbzr('pull')
+        os.mkdir('subdir')
+        self.runbzr('add subdir')
+        self.runbzr('commit -m blah --unchanged')
+        os.chdir('../a')
+        a = Branch.open('.')
+        b = Branch.open('../b')
+        self.assertEquals(a.revision_history(), b.revision_history()[:-1])
+        self.runbzr('pull ../b')
+        self.assertEquals(a.revision_history(), b.revision_history())
+        self.runbzr('commit -m blah2 --unchanged')
+        os.chdir('../b')
+        self.runbzr('commit -m blah3 --unchanged')
+        # no overwrite
+        self.runbzr('pull ../a', retcode=3)
+        os.chdir('..')
+        self.runbzr('branch b overwriteme')
+        os.chdir('overwriteme')
+        self.runbzr('pull --overwrite ../a')
+        overwritten = Branch.open('.')
+        self.assertEqual(overwritten.revision_history(),
+                         a.revision_history())
+        os.chdir('../a')
+        self.runbzr('merge ../b')
+        self.runbzr('commit -m blah4 --unchanged')
+        os.chdir('../b/subdir')
+        self.runbzr('pull ../../a')
+        self.assertEquals(a.revision_history()[-1], b.revision_history()[-1])
+        self.runbzr('commit -m blah5 --unchanged')
+        self.runbzr('commit -m blah6 --unchanged')
+        os.chdir('..')
+        self.runbzr('pull ../a')
+        os.chdir('../a')
+        self.runbzr('commit -m blah7 --unchanged')
+        self.runbzr('merge ../b')
+        self.runbzr('commit -m blah8 --unchanged')
+        self.runbzr('pull ../b')
+        self.runbzr('pull ../b')
+
+    def test_overwrite_uptodate(self):
+        # Make sure pull --overwrite overwrites
+        # even if the target branch has merged
+        # everything already.
+        bzr = self.run_bzr
+
+        def get_rh(expected_len):
+            rh = self.capture('revision-history')
+            # Make sure we don't have trailing empty revisions
+            rh = rh.strip().split('\n')
+            self.assertEqual(len(rh), expected_len)
+            return rh
+
+        os.mkdir('a')
+        os.chdir('a')
+        bzr('init')
+        open('foo', 'wb').write('original\n')
+        bzr('add', 'foo')
+        bzr('commit', '-m', 'initial commit')
+
+        os.chdir('..')
+        bzr('branch', 'a', 'b')
+
+        os.chdir('a')
+        open('foo', 'wb').write('changed\n')
+        bzr('commit', '-m', 'later change')
+
+        open('foo', 'wb').write('another\n')
+        bzr('commit', '-m', 'a third change')
+
+        rev_history_a = get_rh(3)
+
+        os.chdir('../b')
+        bzr('merge', '../a')
+        bzr('commit', '-m', 'merge')
+
+        rev_history_b = get_rh(2)
+
+        bzr('pull', '--overwrite', '../a')
+        rev_history_b = get_rh(3)
+
+        self.assertEqual(rev_history_b, rev_history_a)
+
+    def test_overwrite_children(self):
+        # Make sure pull --overwrite sets the revision-history
+        # to be identical to the pull source, even if we have convergence
+        bzr = self.run_bzr
+
+        def get_rh(expected_len):
+            rh = self.capture('revision-history')
+            # Make sure we don't have trailing empty revisions
+            rh = rh.strip().split('\n')
+            self.assertEqual(len(rh), expected_len)
+            return rh
+
+        os.mkdir('a')
+        os.chdir('a')
+        bzr('init')
+        open('foo', 'wb').write('original\n')
+        bzr('add', 'foo')
+        bzr('commit', '-m', 'initial commit')
+
+        os.chdir('..')
+        bzr('branch', 'a', 'b')
+
+        os.chdir('a')
+        open('foo', 'wb').write('changed\n')
+        bzr('commit', '-m', 'later change')
+
+        open('foo', 'wb').write('another\n')
+        bzr('commit', '-m', 'a third change')
+
+        rev_history_a = get_rh(3)
+
+        os.chdir('../b')
+        bzr('merge', '../a')
+        bzr('commit', '-m', 'merge')
+
+        rev_history_b = get_rh(2)
+
+        os.chdir('../a')
+        open('foo', 'wb').write('a fourth change\n')
+        bzr('commit', '-m', 'a fourth change')
+
+        rev_history_a = get_rh(4)
+
+        # With convergence, we could just pull over the
+        # new change, but with --overwrite, we want to switch our history
+        os.chdir('../b')
+        bzr('pull', '--overwrite', '../a')
+        rev_history_b = get_rh(4)
+
+        self.assertEqual(rev_history_b, rev_history_a)
+
+

=== renamed file 'bzrlib/selftest/test_revision_info.py' => 'bzrlib/tests/blackbox/test_revision_info.py'
--- bzrlib/selftest/test_revision_info.py	2005-09-19 06:05:19 +0000
+++ bzrlib/tests/blackbox/test_revision_info.py	2005-12-29 02:36:07 +0000
@@ -15,7 +15,7 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 import os
-from bzrlib.selftest import TestCaseInTempDir
+from bzrlib.tests import TestCaseInTempDir
 from bzrlib.errors import BzrCommandError, NoSuchRevision
 from bzrlib.branch import Branch
 from bzrlib.revisionspec import RevisionSpec
@@ -28,7 +28,7 @@
         The output is supplied first, so that you can supply a variable
         number of arguments to bzr.
         """
-        self.assertEquals(self.run_bzr_captured(args, retcode=1)[1], output)
+        self.assertContainsRe(self.run_bzr_captured(args, retcode=3)[1], output)
 
     def check_output(self, output, *args):
         """Verify that the expected output matches what bzr says.
@@ -42,14 +42,15 @@
         """Test that 'bzr revision-info' reports the correct thing.
         """
 
-        b = Branch.initialize('.')
+        b = Branch.initialize(u'.')
 
-        b.commit('Commit one', rev_id='a@r-0-1')
-        b.commit('Commit two', rev_id='a@r-0-2')
-        b.commit('Commit three', rev_id='a@r-0-3')
+        b.working_tree().commit('Commit one', rev_id='a@r-0-1')
+        b.working_tree().commit('Commit two', rev_id='a@r-0-2')
+        b.working_tree().commit('Commit three', rev_id='a@r-0-3')
 
         # Make sure revision-info without any arguments throws an exception
-        self.check_error('bzr: ERROR: You must supply a revision identifier\n',
+        self.check_error('bzr: ERROR: '
+                         'You must supply a revision identifier\n',
                          'revision-info')
 
         values = {
@@ -84,16 +85,16 @@
     def test_cat_revision(self):
         """Test bzr cat-revision.
         """
-        b = Branch.initialize('.')
+        b = Branch.initialize(u'.')
 
-        b.commit('Commit one', rev_id='a@r-0-1')
-        b.commit('Commit two', rev_id='a@r-0-2')
-        b.commit('Commit three', rev_id='a@r-0-3')
+        b.working_tree().commit('Commit one', rev_id='a@r-0-1')
+        b.working_tree().commit('Commit two', rev_id='a@r-0-2')
+        b.working_tree().commit('Commit three', rev_id='a@r-0-3')
 
         revs = {
-            1:b.get_revision_xml_file('a@r-0-1').read(),
-            2:b.get_revision_xml_file('a@r-0-2').read(),
-            3:b.get_revision_xml_file('a@r-0-3').read()
+            1:b.get_revision_xml('a@r-0-1'),
+            2:b.get_revision_xml('a@r-0-2'),
+            3:b.get_revision_xml('a@r-0-3')
         }
 
         self.check_output(revs[1], 'cat-revision', 'a@r-0-1')

=== added file 'bzrlib/tests/blackbox/test_revno.py'
--- bzrlib/tests/blackbox/test_revno.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/blackbox/test_revno.py	2005-12-04 21:45:39 +0000
@@ -0,0 +1,53 @@
+# Copyright (C) 2005 by Canonical Ltd
+# -*- coding: utf-8 -*-
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+"""Black-box tests for bzr revno.
+"""
+
+import os
+
+from bzrlib.branch import Branch
+from bzrlib.tests import TestCaseInTempDir
+
+class TestRevno(TestCaseInTempDir):
+
+    def test_revno(self):
+
+        def bzr(*args, **kwargs):
+            return self.run_bzr(*args, **kwargs)[0]
+
+        os.mkdir('a')
+        os.chdir('a')
+        bzr('init')
+        self.assertEquals(int(bzr('revno')), 0)
+
+        open('foo', 'wb').write('foo\n')
+        bzr('add', 'foo')
+        bzr('commit', '-m', 'foo')
+        self.assertEquals(int(bzr('revno')), 1)
+
+        os.mkdir('baz')
+        bzr('add', 'baz')
+        bzr('commit', '-m', 'baz')
+        self.assertEquals(int(bzr('revno')), 2)
+
+        os.chdir('..')
+        self.assertEquals(int(bzr('revno', 'a')), 2)
+        self.assertEquals(int(bzr('revno', 'a/baz')), 2)
+
+

=== renamed file 'bzrlib/selftest/blackbox.py' => 'bzrlib/tests/blackbox/test_too_much.py'
--- bzrlib/selftest/blackbox.py	2005-10-06 13:08:14 +0000
+++ bzrlib/tests/blackbox/test_too_much.py	2006-01-03 02:07:06 +0000
@@ -15,6 +15,10 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+# Mr. Smoketoomuch: I'm sorry?
+# Mr. Bounder: You'd better cut down a little then.
+# Mr. Smoketoomuch: Oh, I see! Smoke too much so I'd better cut down a little
+#                   then!
 
 """Black-box tests for bzr.
 
@@ -24,30 +28,25 @@
 """
 
 
+# XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
+# Note: Please don't add new tests here, it's too big and bulky.  Instead add
+# them into small suites in bzrlib.tests.blackbox.test_FOO for the particular
+# UI command/aspect that is being tested.
+
+
 from cStringIO import StringIO
 import os
+import re
 import shutil
 import sys
-import os
 
 from bzrlib.branch import Branch
+from bzrlib.clone import copy_branch
 from bzrlib.errors import BzrCommandError
-from bzrlib.osutils import has_symlinks
-from bzrlib.selftest import TestCaseInTempDir, BzrTestBase
-from bzrlib.selftest.HTTPTestUtil import TestCaseWithWebserver
-
-
-class ExternalBase(TestCaseInTempDir):
-
-    def runbzr(self, args, retcode=0, backtick=False):
-        if isinstance(args, basestring):
-            args = args.split()
-
-        if backtick:
-            return self.run_bzr_captured(args, retcode=retcode)[0]
-        else:
-            return self.run_bzr_captured(args, retcode=retcode)
-
+from bzrlib.osutils import has_symlinks, pathjoin
+from bzrlib.tests.HTTPTestUtil import TestCaseWithWebserver
+from bzrlib.tests.test_sftp_transport import TestCaseWithSFTPServer
+from bzrlib.tests.blackbox import ExternalBase
 
 class TestCommands(ExternalBase):
 
@@ -61,6 +60,20 @@
     def test_init_branch(self):
         self.runbzr(['init'])
 
+        # Can it handle subdirectories as well?
+        self.runbzr('init subdir1')
+        self.assert_(os.path.exists('subdir1'))
+        self.assert_(os.path.exists('subdir1/.bzr'))
+
+        self.runbzr('init subdir2/nothere', retcode=3)
+        
+        os.mkdir('subdir2')
+        self.runbzr('init subdir2')
+        self.runbzr('init subdir2', retcode=3)
+
+        self.runbzr('init subdir2/subsubdir1')
+        self.assert_(os.path.exists('subdir2/subsubdir1/.bzr'))
+
     def test_whoami(self):
         # this should always identify something, if only "john@localhost"
         self.runbzr("whoami")
@@ -92,55 +105,91 @@
         if bzr_email is not None:
             os.environ['BZREMAIL'] = bzr_email
 
+    def test_nick_command(self):
+        """bzr nick for viewing, setting nicknames"""
+        os.mkdir('me.dev')
+        os.chdir('me.dev')
+        self.runbzr('init')
+        nick = self.runbzr("nick",backtick=True)
+        self.assertEqual(nick, 'me.dev\n')
+        nick = self.runbzr("nick moo")
+        nick = self.runbzr("nick",backtick=True)
+        self.assertEqual(nick, 'moo\n')
+
+
     def test_invalid_commands(self):
-        self.runbzr("pants", retcode=1)
-        self.runbzr("--pants off", retcode=1)
-        self.runbzr("diff --message foo", retcode=1)
+        self.runbzr("pants", retcode=3)
+        self.runbzr("--pants off", retcode=3)
+        self.runbzr("diff --message foo", retcode=3)
 
     def test_empty_commit(self):
         self.runbzr("init")
         self.build_tree(['hello.txt'])
-        self.runbzr("commit -m empty", retcode=1)
+        self.runbzr("commit -m empty", retcode=3)
         self.runbzr("add hello.txt")
-        self.runbzr("commit -m added")
+        self.runbzr("commit -m added")       
 
     def test_empty_commit_message(self):
         self.runbzr("init")
         file('foo.c', 'wt').write('int main() {}')
         self.runbzr(['add', 'foo.c'])
-        self.runbzr(["commit", "-m", ""] , retcode=1) 
+        self.runbzr(["commit", "-m", ""] , retcode=3) 
+
+    def test_remove_deleted(self):
+        self.runbzr("init")
+        self.build_tree(['a'])
+        self.runbzr(['add', 'a'])
+        self.runbzr(['commit', '-m', 'added a'])
+        os.unlink('a')
+        self.runbzr(['remove', 'a'])
+
+    def test_other_branch_commit(self):
+        # this branch is to ensure consistent behaviour, whether we're run
+        # inside a branch, or not.
+        os.mkdir('empty_branch')
+        os.chdir('empty_branch')
+        self.runbzr('init')
+        os.mkdir('branch')
+        os.chdir('branch')
+        self.runbzr('init')
+        file('foo.c', 'wt').write('int main() {}')
+        file('bar.c', 'wt').write('int main() {}')
+        os.chdir('..')
+        self.runbzr(['add', 'branch/foo.c'])
+        self.runbzr(['add', 'branch'])
+        # can't commit files in different trees; sane error
+        self.runbzr('commit -m newstuff branch/foo.c .', retcode=3)
+        self.runbzr('commit -m newstuff branch/foo.c')
+        self.runbzr('commit -m newstuff branch')
+        self.runbzr('commit -m newstuff branch', retcode=3)
 
     def test_ignore_patterns(self):
         from bzrlib.branch import Branch
-        
-        b = Branch.initialize('.')
-        self.assertEquals(list(b.unknowns()), [])
+        Branch.initialize('.')
+        self.assertEquals(self.capture('unknowns'), '')
 
         file('foo.tmp', 'wt').write('tmp files are ignored')
-        self.assertEquals(list(b.unknowns()), [])
-        assert self.capture('unknowns') == ''
+        self.assertEquals(self.capture('unknowns'), '')
 
         file('foo.c', 'wt').write('int main() {}')
-        self.assertEquals(list(b.unknowns()), ['foo.c'])
-        assert self.capture('unknowns') == 'foo.c\n'
+        self.assertEquals(self.capture('unknowns'), 'foo.c\n')
 
         self.runbzr(['add', 'foo.c'])
-        assert self.capture('unknowns') == ''
+        self.assertEquals(self.capture('unknowns'), '')
 
         # 'ignore' works when creating the .bzignore file
         file('foo.blah', 'wt').write('blah')
-        self.assertEquals(list(b.unknowns()), ['foo.blah'])
+        self.assertEquals(self.capture('unknowns'), 'foo.blah\n')
         self.runbzr('ignore *.blah')
-        self.assertEquals(list(b.unknowns()), [])
-        assert file('.bzrignore', 'rU').read() == '*.blah\n'
+        self.assertEquals(self.capture('unknowns'), '')
+        self.assertEquals(file('.bzrignore', 'rU').read(), '*.blah\n')
 
         # 'ignore' works when then .bzrignore file already exists
         file('garh', 'wt').write('garh')
-        self.assertEquals(list(b.unknowns()), ['garh'])
-        assert self.capture('unknowns') == 'garh\n'
+        self.assertEquals(self.capture('unknowns'), 'garh\n')
         self.runbzr('ignore garh')
-        self.assertEquals(list(b.unknowns()), [])
-        assert file('.bzrignore', 'rU').read() == '*.blah\ngarh\n'
+        self.assertEquals(self.capture('unknowns'), '')
+        self.assertEquals(file('.bzrignore', 'rU').read(), '*.blah\ngarh\n')
 
     def test_revert(self):
         self.runbzr('init')
@@ -167,11 +216,20 @@
         os.rmdir('revertdir')
         self.runbzr('revert')
 
-        os.symlink('/unlikely/to/exist', 'symlink')
-        self.runbzr('add symlink')
-        self.runbzr('commit -m f')
-        os.unlink('symlink')
-        self.runbzr('revert')
+        if has_symlinks():
+            os.symlink('/unlikely/to/exist', 'symlink')
+            self.runbzr('add symlink')
+            self.runbzr('commit -m f')
+            os.unlink('symlink')
+            self.runbzr('revert')
+            self.failUnlessExists('symlink')
+            os.unlink('symlink')
+            os.symlink('a-different-path', 'symlink')
+            self.runbzr('revert')
+            self.assertEqual('/unlikely/to/exist',
+                             os.readlink('symlink'))
+        else:
+            self.log("skipping revert symlink tests")
         
         file('hello', 'wt').write('xyz')
         self.runbzr('commit -m xyz hello')
@@ -183,6 +241,21 @@
         self.runbzr('revert')
         os.chdir('..')
 
+    def test_status(self):
+        self.runbzr("init")
+        self.build_tree(['hello.txt'])
+        result = self.runbzr("status")
+        self.assert_("unknown:\n  hello.txt\n" in result, result)
+        self.runbzr("add hello.txt")
+        result = self.runbzr("status")
+        self.assert_("added:\n  hello.txt\n" in result, result)
+        self.runbzr("commit -m added")
+        result = self.runbzr("status -r 0..1")
+        self.assert_("added:\n  hello.txt\n" in result, result)
+        self.build_tree(['world.txt'])
+        result = self.runbzr("status -r 0")
+        self.assert_("added:\n  hello.txt\n" \
+                     "unknown:\n  world.txt\n" in result, result)
 
     def test_mv_modes(self):
         """Test two modes of operation for mv"""
@@ -227,43 +300,95 @@
         self.runbzr('export ../latest')
         self.assertEqual(file('../latest/goodbye', 'rt').read(), 'baz')
         self.runbzr('export ../first -r 1')
-        assert not os.path.exists('../first/goodbye')
+        self.assert_(not os.path.exists('../first/goodbye'))
         self.assertEqual(file('../first/hello', 'rt').read(), 'foo')
         self.runbzr('export ../first.gz -r 1')
         self.assertEqual(file('../first.gz/hello', 'rt').read(), 'foo')
         self.runbzr('export ../first.bz2 -r 1')
         self.assertEqual(file('../first.bz2/hello', 'rt').read(), 'foo')
+
+        from tarfile import TarFile
         self.runbzr('export ../first.tar -r 1')
-        assert os.path.isfile('../first.tar')
-        from tarfile import TarFile
+        self.assert_(os.path.isfile('../first.tar'))
         tf = TarFile('../first.tar')
-        assert 'first/hello' in tf.getnames(), tf.getnames()
+        self.assert_('first/hello' in tf.getnames(), tf.getnames())
         self.assertEqual(tf.extractfile('first/hello').read(), 'foo')
         self.runbzr('export ../first.tar.gz -r 1')
-        assert os.path.isfile('../first.tar.gz')
+        self.assert_(os.path.isfile('../first.tar.gz'))
         self.runbzr('export ../first.tbz2 -r 1')
-        assert os.path.isfile('../first.tbz2')
+        self.assert_(os.path.isfile('../first.tbz2'))
         self.runbzr('export ../first.tar.bz2 -r 1')
-        assert os.path.isfile('../first.tar.bz2')
+        self.assert_(os.path.isfile('../first.tar.bz2'))
         self.runbzr('export ../first.tar.tbz2 -r 1')
-        assert os.path.isfile('../first.tar.tbz2')
+        self.assert_(os.path.isfile('../first.tar.tbz2'))
+
         from bz2 import BZ2File
         tf = TarFile('../first.tar.tbz2', 
                      fileobj=BZ2File('../first.tar.tbz2', 'r'))
-        assert 'first.tar/hello' in tf.getnames(), tf.getnames()
+        self.assert_('first.tar/hello' in tf.getnames(), tf.getnames())
         self.assertEqual(tf.extractfile('first.tar/hello').read(), 'foo')
         self.runbzr('export ../first2.tar -r 1 --root pizza')
         tf = TarFile('../first2.tar')
-        assert 'pizza/hello' in tf.getnames(), tf.getnames()
+        self.assert_('pizza/hello' in tf.getnames(), tf.getnames())
+
+        from zipfile import ZipFile
+        self.runbzr('export ../first.zip -r 1')
+        self.failUnlessExists('../first.zip')
+        zf = ZipFile('../first.zip')
+        self.assert_('first/hello' in zf.namelist(), zf.namelist())
+        self.assertEqual(zf.read('first/hello'), 'foo')
+
+        self.runbzr('export ../first2.zip -r 1 --root pizza')
+        zf = ZipFile('../first2.zip')
+        self.assert_('pizza/hello' in zf.namelist(), zf.namelist())
+        
+        self.runbzr('export ../first-zip --format=zip -r 1')
+        zf = ZipFile('../first-zip')
+        self.assert_('first-zip/hello' in zf.namelist(), zf.namelist())
 
     def test_diff(self):
         self.example_branch()
         file('hello', 'wt').write('hello world!')
         self.runbzr('commit -m fixing hello')
-        output = self.runbzr('diff -r 2..3', backtick=1)
+        output = self.runbzr('diff -r 2..3', backtick=1, retcode=1)
         self.assert_('\n+hello world!' in output)
-        output = self.runbzr('diff -r last:3..last:1', backtick=1)
+        output = self.runbzr('diff -r last:3..last:1', backtick=1, retcode=1)
         self.assert_('\n+baz' in output)
+        file('moo', 'wb').write('moo')
+        self.runbzr('add moo')
+        os.unlink('moo')
+        self.runbzr('diff')
+
+    def test_diff_branches(self):
+        self.build_tree(['branch1/', 'branch1/file', 'branch2/'], line_endings='binary')
+        branch = Branch.initialize('branch1')
+        branch.working_tree().add(['file'])
+        branch.working_tree().commit('add file')
+        copy_branch(branch, 'branch2')
+        print >> open('branch2/file', 'wb'), 'new content'
+        branch2 = Branch.open('branch2')
+        branch2.working_tree().commit('update file')
+        # should open branch1 and diff against branch2, 
+        output = self.run_bzr_captured(['diff', '-r', 'branch:branch2', 
+                                        'branch1'],
+                                       retcode=1)
+        self.assertEquals(("=== modified file 'file'\n"
+                           "--- file\t\n"
+                           "+++ file\t\n"
+                           "@@ -1,1 +1,1 @@\n"
+                           "-new content\n"
+                           "+contents of branch1/file\n"
+                           "\n", ''), output)
+        output = self.run_bzr_captured(['diff', 'branch2', 'branch1'],
+                                       retcode=1)
+        self.assertEqualDiff(("=== modified file 'file'\n"
+                              "--- file\t\n"
+                              "+++ file\t\n"
+                              "@@ -1,1 +1,1 @@\n"
+                              "-new content\n"
+                              "+contents of branch1/file\n"
+                              "\n", ''), output)
+
 
     def test_branch(self):
         """Branch from one branch to another."""
@@ -272,6 +397,7 @@
         self.example_branch()
         os.chdir('..')
         self.runbzr('branch a b')
+        self.assertFileEqual('b\n', 'b/.bzr/branch-name')
         self.runbzr('branch a c -r 1')
         os.chdir('b')
         self.runbzr('commit -m foo --unchanged')
@@ -298,144 +424,230 @@
         os.chdir('../a')
         file('hello', 'wt').write('quuux')
         # We can't merge when there are in-tree changes
-        self.runbzr('merge ../b', retcode=1)
+        self.runbzr('merge ../b', retcode=3)
         self.runbzr(['commit', '-m', "Like an epidemic of u's"])
-        self.runbzr('merge ../b')
+        self.runbzr('merge ../b -r last:1..last:1 --merge-type blooof',
+                    retcode=3)
+        self.runbzr('merge ../b -r last:1..last:1 --merge-type merge3')
+        self.runbzr('revert --no-backup')
+        self.runbzr('merge ../b -r last:1..last:1 --merge-type weave')
+        self.runbzr('revert --no-backup')
+        self.runbzr('merge ../b -r last:1..last:1 --reprocess')
+        self.runbzr('revert --no-backup')
+        self.runbzr('merge ../b -r last:1')
         self.check_file_contents('goodbye', 'quux')
         # Merging a branch pulls its revision into the tree
         a = Branch.open('.')
         b = Branch.open('../b')
         a.get_revision_xml(b.last_revision())
-        self.log('pending merges: %s', a.pending_merges())
-        #        assert a.pending_merges() == [b.last_revision()], "Assertion %s %s" \
-        #        % (a.pending_merges(), b.last_patch())
-
-    def test_merge_with_missing_file(self):
-        """Merge handles missing file conflicts"""
-        os.mkdir('a')
-        os.chdir('a')
-        os.mkdir('sub')
-        print >> file('sub/a.txt', 'wb'), "hello"
-        print >> file('b.txt', 'wb'), "hello"
-        print >> file('sub/c.txt', 'wb'), "hello"
-        self.runbzr('init')
-        self.runbzr('add')
-        self.runbzr(('commit', '-m', 'added a'))
-        self.runbzr('branch . ../b')
-        print >> file('sub/a.txt', 'ab'), "there"
-        print >> file('b.txt', 'ab'), "there"
-        print >> file('sub/c.txt', 'ab'), "there"
-        self.runbzr(('commit', '-m', 'Added there'))
-        os.unlink('sub/a.txt')
-        os.unlink('sub/c.txt')
-        os.rmdir('sub')
-        os.unlink('b.txt')
-        self.runbzr(('commit', '-m', 'Removed a.txt'))
-        os.chdir('../b')
-        print >> file('sub/a.txt', 'ab'), "something"
-        print >> file('b.txt', 'ab'), "something"
-        print >> file('sub/c.txt', 'ab'), "something"
-        self.runbzr(('commit', '-m', 'Modified a.txt'))
-        self.runbzr('merge ../a/')
-        assert os.path.exists('sub/a.txt.THIS')
-        assert os.path.exists('sub/a.txt.BASE')
-        os.chdir('../a')
-        self.runbzr('merge ../b/')
-        assert os.path.exists('sub/a.txt.OTHER')
-        assert os.path.exists('sub/a.txt.BASE')
-
-    def test_merge_with_missing_file(self):
-        """Merge handles missing file conflicts"""
-        os.mkdir('a')
-        os.chdir('a')
-        os.mkdir('sub')
-        print >> file('sub/a.txt', 'wb'), "hello"
-        print >> file('b.txt', 'wb'), "hello"
-        print >> file('sub/c.txt', 'wb'), "hello"
-        self.runbzr('init')
-        self.runbzr('add')
-        self.runbzr(('commit', '-m', 'added a'))
-        self.runbzr('branch . ../b')
-        print >> file('sub/a.txt', 'ab'), "there"
-        print >> file('b.txt', 'ab'), "there"
-        print >> file('sub/c.txt', 'ab'), "there"
-        self.runbzr(('commit', '-m', 'Added there'))
-        os.unlink('sub/a.txt')
-        os.unlink('sub/c.txt')
-        os.rmdir('sub')
-        os.unlink('b.txt')
-        self.runbzr(('commit', '-m', 'Removed a.txt'))
-        os.chdir('../b')
-        print >> file('sub/a.txt', 'ab'), "something"
-        print >> file('b.txt', 'ab'), "something"
-        print >> file('sub/c.txt', 'ab'), "something"
-        self.runbzr(('commit', '-m', 'Modified a.txt'))
-        self.runbzr('merge ../a/')
-        assert os.path.exists('sub/a.txt.THIS')
-        assert os.path.exists('sub/a.txt.BASE')
-        os.chdir('../a')
-        self.runbzr('merge ../b/')
-        assert os.path.exists('sub/a.txt.OTHER')
-        assert os.path.exists('sub/a.txt.BASE')
-
-    def test_pull(self):
-        """Pull changes from one branch to another."""
-        os.mkdir('a')
-        os.chdir('a')
-
-        self.example_branch()
-        self.runbzr('pull', retcode=1)
-        self.runbzr('missing', retcode=1)
-        self.runbzr('missing .')
-        self.runbzr('missing')
-        self.runbzr('pull')
-        self.runbzr('pull /', retcode=1)
-        self.runbzr('pull')
-
-        os.chdir('..')
-        self.runbzr('branch a b')
-        os.chdir('b')
-        self.runbzr('pull')
+        self.log('pending merges: %s', a.working_tree().pending_merges())
+        self.assertEquals(a.working_tree().pending_merges(),
+                          [b.last_revision()])
+        self.runbzr('commit -m merged')
+        self.runbzr('merge ../b -r last:1')
+        self.assertEqual(Branch.open('.').working_tree().pending_merges(), [])
+
+    def test_merge_with_missing_file(self):
+        """Merge handles missing file conflicts"""
+        os.mkdir('a')
+        os.chdir('a')
+        os.mkdir('sub')
+        print >> file('sub/a.txt', 'wb'), "hello"
+        print >> file('b.txt', 'wb'), "hello"
+        print >> file('sub/c.txt', 'wb'), "hello"
+        self.runbzr('init')
+        self.runbzr('add')
+        self.runbzr(('commit', '-m', 'added a'))
+        self.runbzr('branch . ../b')
+        print >> file('sub/a.txt', 'ab'), "there"
+        print >> file('b.txt', 'ab'), "there"
+        print >> file('sub/c.txt', 'ab'), "there"
+        self.runbzr(('commit', '-m', 'Added there'))
+        os.unlink('sub/a.txt')
+        os.unlink('sub/c.txt')
+        os.rmdir('sub')
+        os.unlink('b.txt')
+        self.runbzr(('commit', '-m', 'Removed a.txt'))
+        os.chdir('../b')
+        print >> file('sub/a.txt', 'ab'), "something"
+        print >> file('b.txt', 'ab'), "something"
+        print >> file('sub/c.txt', 'ab'), "something"
+        self.runbzr(('commit', '-m', 'Modified a.txt'))
+        self.runbzr('merge ../a/', retcode=1)
+        self.assert_(os.path.exists('sub/a.txt.THIS'))
+        self.assert_(os.path.exists('sub/a.txt.BASE'))
+        os.chdir('../a')
+        self.runbzr('merge ../b/', retcode=1)
+        self.assert_(os.path.exists('sub/a.txt.OTHER'))
+        self.assert_(os.path.exists('sub/a.txt.BASE'))
+
+    def test_inventory(self):
+        bzr = self.runbzr
+        def output_equals(value, *args):
+            out = self.runbzr(['inventory'] + list(args), backtick=True)
+            self.assertEquals(out, value)
+
+        bzr('init')
+        open('a', 'wb').write('hello\n')
+        os.mkdir('b')
+
+        bzr('add a b')
+        bzr('commit -m add')
+
+        output_equals('a\n', '--kind', 'file')
+        output_equals('b\n', '--kind', 'directory')        
+
+    def test_ls(self):
+        """Test the abilities of 'bzr ls'"""
+        bzr = self.runbzr
+        def bzrout(*args, **kwargs):
+            kwargs['backtick'] = True
+            return self.runbzr(*args, **kwargs)
+
+        def ls_equals(value, *args):
+            out = self.runbzr(['ls'] + list(args), backtick=True)
+            self.assertEquals(out, value)
+
+        bzr('init')
+        open('a', 'wb').write('hello\n')
+
+        # Can't supply both
+        bzr('ls --verbose --null', retcode=3)
+
+        ls_equals('a\n')
+        ls_equals('?        a\n', '--verbose')
+        ls_equals('a\n', '--unknown')
+        ls_equals('', '--ignored')
+        ls_equals('', '--versioned')
+        ls_equals('a\n', '--unknown', '--ignored', '--versioned')
+        ls_equals('', '--ignored', '--versioned')
+        ls_equals('a\0', '--null')
+
+        bzr('add a')
+        ls_equals('V        a\n', '--verbose')
+        bzr('commit -m add')
+        
         os.mkdir('subdir')
-        self.runbzr('add subdir')
-        self.runbzr('commit -m blah --unchanged')
+        ls_equals('V        a\n'
+                  '?        subdir/\n'
+                  , '--verbose')
+        open('subdir/b', 'wb').write('b\n')
+        bzr('add')
+        ls_equals('V        a\n'
+                  'V        subdir/\n'
+                  'V        subdir/b\n'
+                  , '--verbose')
+        bzr('commit -m subdir')
+
+        ls_equals('a\n'
+                  'subdir\n'
+                  , '--non-recursive')
+
+        ls_equals('V        a\n'
+                  'V        subdir/\n'
+                  , '--verbose', '--non-recursive')
+
+        # Check what happens in a sub-directory
+        os.chdir('subdir')
+        ls_equals('b\n')
+        ls_equals('b\0'
+                  , '--null')
+        ls_equals('a\n'
+                  'subdir\n'
+                  'subdir/b\n'
+                  , '--from-root')
+        ls_equals('a\0'
+                  'subdir\0'
+                  'subdir/b\0'
+                  , '--from-root', '--null')
+        ls_equals('a\n'
+                  'subdir\n'
+                  , '--from-root', '--non-recursive')
+
+        os.chdir('..')
+
+        # Check what happens when we supply a specific revision
+        ls_equals('a\n', '--revision', '1')
+        ls_equals('V        a\n'
+                  , '--verbose', '--revision', '1')
+
+        os.chdir('subdir')
+        ls_equals('', '--revision', '1')
+
+        # Now try to do ignored files.
+        os.chdir('..')
+        open('blah.py', 'wb').write('unknown\n')
+        open('blah.pyo', 'wb').write('ignored\n')
+        ls_equals('a\n'
+                  'blah.py\n'
+                  'blah.pyo\n'
+                  'subdir\n'
+                  'subdir/b\n')
+        ls_equals('V        a\n'
+                  '?        blah.py\n'
+                  'I        blah.pyo\n'
+                  'V        subdir/\n'
+                  'V        subdir/b\n'
+                  , '--verbose')
+        ls_equals('blah.pyo\n'
+                  , '--ignored')
+        ls_equals('blah.py\n'
+                  , '--unknown')
+        ls_equals('a\n'
+                  'subdir\n'
+                  'subdir/b\n'
+                  , '--versioned')
+
+    def test_pull_verbose(self):
+        """Pull changes from one branch to another and watch the output."""
+
+        os.mkdir('a')
+        os.chdir('a')
+
+        bzr = self.runbzr
+        self.example_branch()
+
+        os.chdir('..')
+        bzr('branch a b')
+        os.chdir('b')
+        open('b', 'wb').write('else\n')
+        bzr('add b')
+        bzr(['commit', '-m', 'added b'])
+
         os.chdir('../a')
-        a = Branch.open('.')
-        b = Branch.open('../b')
-        assert a.revision_history() == b.revision_history()[:-1]
-        self.runbzr('pull ../b')
-        assert a.revision_history() == b.revision_history()
-        self.runbzr('commit -m blah2 --unchanged')
+        out = bzr('pull --verbose ../b', backtick=True)
+        self.failIfEqual(out.find('Added Revisions:'), -1)
+        self.failIfEqual(out.find('message:\n  added b'), -1)
+        self.failIfEqual(out.find('added b'), -1)
+
+        # Check that --overwrite --verbose prints out the removed entries
+        bzr('commit -m foo --unchanged')
         os.chdir('../b')
-        self.runbzr('commit -m blah3 --unchanged')
-        self.runbzr('pull ../a', retcode=1)
-        print "DECIDE IF PULL CAN CONVERGE, blackbox.py"
-        return
-        os.chdir('../a')
-        self.runbzr('merge ../b')
-        self.runbzr('commit -m blah4 --unchanged')
-        os.chdir('../b/subdir')
-        self.runbzr('pull ../../a')
-        assert a.revision_history()[-1] == b.revision_history()[-1]
-        self.runbzr('commit -m blah5 --unchanged')
-        self.runbzr('commit -m blah6 --unchanged')
-        os.chdir('..')
-        self.runbzr('pull ../a')
-        os.chdir('../a')
-        self.runbzr('commit -m blah7 --unchanged')
-        self.runbzr('merge ../b')
-        self.runbzr('commit -m blah8 --unchanged')
-        self.runbzr('pull ../b')
-        self.runbzr('pull ../b')
-
+        bzr('commit -m baz --unchanged')
+        bzr('pull ../a', retcode=3)
+        out = bzr('pull --overwrite --verbose ../a', backtick=1)
+
+        remove_loc = out.find('Removed Revisions:')
+        self.failIfEqual(remove_loc, -1)
+        added_loc = out.find('Added Revisions:')
+        self.failIfEqual(added_loc, -1)
+
+        removed_message = out.find('message:\n  baz')
+        self.failIfEqual(removed_message, -1)
+        self.failUnless(remove_loc < removed_message < added_loc)
+
+        added_message = out.find('message:\n  foo')
+        self.failIfEqual(added_message, -1)
+        self.failUnless(added_loc < added_message)
+        
     def test_locations(self):
         """Using and remembering different locations"""
         os.mkdir('a')
         os.chdir('a')
         self.runbzr('init')
         self.runbzr('commit -m unchanged --unchanged')
-        self.runbzr('pull', retcode=1)
-        self.runbzr('merge', retcode=1)
+        self.runbzr('pull', retcode=3)
+        self.runbzr('merge', retcode=3)
         self.runbzr('branch . ../b')
         os.chdir('../b')
         self.runbzr('pull')
@@ -452,31 +664,117 @@
         os.chdir('../b')
         self.runbzr('pull')
         os.chdir('../d')
-        self.runbzr('pull', retcode=1)
+        self.runbzr('pull', retcode=3)
         self.runbzr('pull ../a --remember')
         self.runbzr('pull')
         
     def test_add_reports(self):
         """add command prints the names of added files."""
         b = Branch.initialize('.')
+        self.build_tree(['top.txt', 'dir/', 'dir/sub.txt', 'CVS'])
+        out = self.run_bzr_captured(['add'], retcode=0)[0]
+        # the ordering is not defined at the moment
+        results = sorted(out.rstrip('\n').split('\n'))
+        self.assertEquals(['If you wish to add some of these files, please'\
+                           ' add them by name.',
+                           'added dir',
+                           'added dir/sub.txt',
+                           'added top.txt',
+                           'ignored 1 file(s) matching "CVS"'],
+                          results)
+        out = self.run_bzr_captured(['add', '-v'], retcode=0)[0]
+        results = sorted(out.rstrip('\n').split('\n'))
+        self.assertEquals(['If you wish to add some of these files, please'\
+                           ' add them by name.',
+                           'ignored CVS matching "CVS"'],
+                          results)
+
+    def test_add_quiet_is(self):
+        """add -q does not print the names of added files."""
+        b = Branch.initialize('.')
         self.build_tree(['top.txt', 'dir/', 'dir/sub.txt'])
-        out = self.run_bzr_captured(['add'], retcode = 0)[0]
+        out = self.run_bzr_captured(['add', '-q'], retcode=0)[0]
         # the ordering is not defined at the moment
         results = sorted(out.rstrip('\n').split('\n'))
-        self.assertEquals(['added dir',
-                           'added dir'+os.sep+'sub.txt',
-                           'added top.txt',],
-                          results)
+        self.assertEquals([''], results)
+
+    def test_add_in_unversioned(self):
+        """Try to add a file in an unversioned directory.
+
+        "bzr add" should add the parent(s) as necessary.
+        """
+        from bzrlib.branch import Branch
+        Branch.initialize('.')
+        self.build_tree(['inertiatic/', 'inertiatic/esp'])
+        self.assertEquals(self.capture('unknowns'), 'inertiatic\n')
+        self.run_bzr('add', 'inertiatic/esp')
+        self.assertEquals(self.capture('unknowns'), '')
+
+        # Multiple unversioned parents
+        self.build_tree(['veil/', 'veil/cerpin/', 'veil/cerpin/taxt'])
+        self.assertEquals(self.capture('unknowns'), 'veil\n')
+        self.run_bzr('add', 'veil/cerpin/taxt')
+        self.assertEquals(self.capture('unknowns'), '')
+
+        # Check whacky paths work
+        self.build_tree(['cicatriz/', 'cicatriz/esp'])
+        self.assertEquals(self.capture('unknowns'), 'cicatriz\n')
+        self.run_bzr('add', 'inertiatic/../cicatriz/esp')
+        self.assertEquals(self.capture('unknowns'), '')
+
+    def test_add_in_versioned(self):
+        """Try to add a file in a versioned directory.
+
+        "bzr add" should do this happily.
+        """
+        from bzrlib.branch import Branch
+        Branch.initialize('.')
+        self.build_tree(['inertiatic/', 'inertiatic/esp'])
+        self.assertEquals(self.capture('unknowns'), 'inertiatic\n')
+        self.run_bzr('add', '--no-recurse', 'inertiatic')
+        self.assertEquals(self.capture('unknowns'), 'inertiatic/esp\n')
+        self.run_bzr('add', 'inertiatic/esp')
+        self.assertEquals(self.capture('unknowns'), '')
+
+    def test_subdir_add(self):
+        """Add in subdirectory should add only things from there down"""
+        from bzrlib.branch import Branch
+        
+        eq = self.assertEqual
+        ass = self.assert_
+        chdir = os.chdir
+        
+        b = Branch.initialize('.')
+        t = b.working_tree()
+        self.build_tree(['src/', 'README'])
+        
+        eq(sorted(t.unknowns()),
+           ['README', 'src'])
+        
+        self.run_bzr('add', 'src')
+        
+        self.build_tree(['src/foo.c'])
+        
+        chdir('src')
+        self.run_bzr('add')
+        
+        self.assertEquals(self.capture('unknowns'), 'README\n')
+        eq(len(t.read_working_inventory()), 3)
+                
+        chdir('..')
+        self.run_bzr('add')
+        self.assertEquals(self.capture('unknowns'), '')
+        self.run_bzr('check')
 
     def test_unknown_command(self):
         """Handling of unknown command."""
         out, err = self.run_bzr_captured(['fluffy-badger'],
-                                         retcode=1)
+                                         retcode=3)
         self.assertEquals(out, '')
         err.index('unknown command')
 
-    def test_conflicts(self):
-        """Handling of merge conflicts"""
+    def create_conflicts(self):
+        """Create a conflicted tree"""
         os.mkdir('base')
         os.chdir('base')
         file('hello', 'wb').write("hi world")
@@ -496,20 +794,200 @@
         file('question', 'wb').write("What do you get when you multiply six"
                                    "times nine?")
         self.runbzr('commit -m this')
-        self.runbzr('merge ../other')
+
+    def test_remerge(self):
+        """Remerge command works as expected"""
+        self.create_conflicts()
+        self.runbzr('merge ../other --show-base', retcode=1)
+        conflict_text = file('hello').read()
+        assert '|||||||' in conflict_text
+        assert 'hi world' in conflict_text
+        self.runbzr('remerge', retcode=1)
+        conflict_text = file('hello').read()
+        assert '|||||||' not in conflict_text
+        assert 'hi world' not in conflict_text
+        os.unlink('hello.OTHER')
+        self.runbzr('remerge hello --merge-type weave', retcode=1)
+        assert os.path.exists('hello.OTHER')
+        file_id = self.runbzr('file-id hello')
+        file_id = self.runbzr('file-id hello.THIS', retcode=3)
+        self.runbzr('remerge --merge-type weave', retcode=1)
+        assert os.path.exists('hello.OTHER')
+        assert not os.path.exists('hello.BASE')
+        assert '|||||||' not in conflict_text
+        assert 'hi world' not in conflict_text
+        self.runbzr('remerge . --merge-type weave --show-base', retcode=3)
+        self.runbzr('remerge . --merge-type weave --reprocess', retcode=3)
+        self.runbzr('remerge . --show-base --reprocess', retcode=3)
+        self.runbzr('remerge hello --show-base', retcode=1)
+        self.runbzr('remerge hello --reprocess', retcode=1)
+        self.runbzr('resolve --all')
+        self.runbzr('commit -m done',)
+        self.runbzr('remerge', retcode=3)
+
+
+    def test_conflicts(self):
+        """Handling of merge conflicts"""
+        self.create_conflicts()
+        self.runbzr('merge ../other --show-base', retcode=1)
+        conflict_text = file('hello').read()
+        self.assert_('<<<<<<<' in conflict_text)
+        self.assert_('>>>>>>>' in conflict_text)
+        self.assert_('=======' in conflict_text)
+        self.assert_('|||||||' in conflict_text)
+        self.assert_('hi world' in conflict_text)
+        self.runbzr('revert')
+        self.runbzr('resolve --all')
+        self.runbzr('merge ../other', retcode=1)
+        conflict_text = file('hello').read()
+        self.assert_('|||||||' not in conflict_text)
+        self.assert_('hi world' not in conflict_text)
         result = self.runbzr('conflicts', backtick=1)
         self.assertEquals(result, "hello\nquestion\n")
         result = self.runbzr('status', backtick=1)
-        assert "conflicts:\n  hello\n  question\n" in result, result
+        self.assert_("conflicts:\n  hello\n  question\n" in result, result)
         self.runbzr('resolve hello')
         result = self.runbzr('conflicts', backtick=1)
         self.assertEquals(result, "question\n")
-        self.runbzr('commit -m conflicts', retcode=1)
+        self.runbzr('commit -m conflicts', retcode=3)
         self.runbzr('resolve --all')
         result = self.runbzr('conflicts', backtick=1)
         self.runbzr('commit -m conflicts')
         self.assertEquals(result, "")
 
+    def test_resign(self):
+        """Test re signing of data."""
+        import bzrlib.gpg
+        oldstrategy = bzrlib.gpg.GPGStrategy
+        branch = Branch.initialize('.')
+        branch.working_tree().commit("base", allow_pointless=True, rev_id='A')
+        try:
+            # monkey patch gpg signing mechanism
+            from bzrlib.testament import Testament
+            bzrlib.gpg.GPGStrategy = bzrlib.gpg.LoopbackGPGStrategy
+            self.runbzr('re-sign -r revid:A')
+            self.assertEqual(Testament.from_revision(branch,'A').as_short_text(),
+                             branch.revision_store.get('A', 'sig').read())
+        finally:
+            bzrlib.gpg.GPGStrategy = oldstrategy
+            
+    def test_resign_range(self):
+        import bzrlib.gpg
+        oldstrategy = bzrlib.gpg.GPGStrategy
+        branch = Branch.initialize('.')
+        branch.working_tree().commit("base", allow_pointless=True, rev_id='A')
+        branch.working_tree().commit("base", allow_pointless=True, rev_id='B')
+        branch.working_tree().commit("base", allow_pointless=True, rev_id='C')
+        try:
+            # monkey patch gpg signing mechanism
+            from bzrlib.testament import Testament
+            bzrlib.gpg.GPGStrategy = bzrlib.gpg.LoopbackGPGStrategy
+            self.runbzr('re-sign -r 1..')
+            self.assertEqual(Testament.from_revision(branch,'A').as_short_text(),
+                             branch.revision_store.get('A', 'sig').read())
+            self.assertEqual(Testament.from_revision(branch,'B').as_short_text(),
+                             branch.revision_store.get('B', 'sig').read())
+            self.assertEqual(Testament.from_revision(branch,'C').as_short_text(),
+                             branch.revision_store.get('C', 'sig').read())
+        finally:
+            bzrlib.gpg.GPGStrategy = oldstrategy
+
+    def test_push(self):
+        # create a source branch
+        os.mkdir('my-branch')
+        os.chdir('my-branch')
+        self.example_branch()
+
+        # with no push target, fail
+        self.runbzr('push', retcode=3)
+        # with an explicit target work
+        self.runbzr('push ../output-branch')
+        # with an implicit target work
+        self.runbzr('push')
+        # nothing missing
+        self.runbzr('missing ../output-branch')
+        # advance this branch
+        self.runbzr('commit --unchanged -m unchanged')
+
+        os.chdir('../output-branch')
+        # There is no longer a difference as long as we have
+        # access to the working tree
+        self.runbzr('diff')
+
+        # But we should be missing a revision
+        self.runbzr('missing ../my-branch', retcode=1)
+
+        # diverge the branches
+        self.runbzr('commit --unchanged -m unchanged')
+        os.chdir('../my-branch')
+        # cannot push now
+        self.runbzr('push', retcode=3)
+        # and there are difference
+        self.runbzr('missing ../output-branch', retcode=1)
+        self.runbzr('missing --verbose ../output-branch', retcode=1)
+        # but we can force a push
+        self.runbzr('push --overwrite')
+        # nothing missing
+        self.runbzr('missing ../output-branch')
+        
+        # pushing to a new dir with no parent should fail
+        self.runbzr('push ../missing/new-branch', retcode=3)
+        # unless we provide --create-prefix
+        self.runbzr('push --create-prefix ../missing/new-branch')
+        # nothing missing
+        self.runbzr('missing ../missing/new-branch')
+
+    def test_external_command(self):
+        """test that external commands can be run by setting the path"""
+        cmd_name = 'test-command'
+        output = 'Hello from test-command'
+        if sys.platform == 'win32':
+            cmd_name += '.bat'
+            output += '\r\n'
+        else:
+            output += '\n'
+
+        oldpath = os.environ.get('BZRPATH', None)
+
+        bzr = self.capture
+
+        try:
+            if os.environ.has_key('BZRPATH'):
+                del os.environ['BZRPATH']
+
+            f = file(cmd_name, 'wb')
+            if sys.platform == 'win32':
+                f.write('@echo off\n')
+            else:
+                f.write('#!/bin/sh\n')
+            f.write('echo Hello from test-command')
+            f.close()
+            os.chmod(cmd_name, 0755)
+
+            # It should not find the command in the local 
+            # directory by default, since it is not in my path
+            bzr(cmd_name, retcode=3)
+
+            # Now put it into my path
+            os.environ['BZRPATH'] = '.'
+
+            bzr(cmd_name)
+            # The test suite does not capture stdout for external commands
+            # this is because you have to have a real file object
+            # to pass to Popen(stdout=FOO), and StringIO is not one of those.
+            # (just replacing sys.stdout does not change a spawned objects stdout)
+            #self.assertEquals(bzr(cmd_name), output)
+
+            # Make sure empty path elements are ignored
+            os.environ['BZRPATH'] = os.pathsep
+
+            bzr(cmd_name, retcode=3)
+
+        finally:
+            if oldpath:
+                os.environ['BZRPATH'] = oldpath
+
+
 def listdir_sorted(dir):
     L = os.listdir(dir)
     L.sort()
@@ -533,7 +1011,7 @@
         runbzr('init')
 
         self.assertEquals(capture('root').rstrip(),
-                          os.path.join(self.test_dir, 'branch1'))
+                          pathjoin(self.test_dir, 'branch1'))
 
         progress("status of new file")
 
@@ -544,140 +1022,141 @@
         self.assertEquals(capture('unknowns'), 'test.txt\n')
 
         out = capture("status")
-        assert out == 'unknown:\n  test.txt\n'
+        self.assertEquals(out, 'unknown:\n  test.txt\n')
 
         out = capture("status --all")
-        assert out == "unknown:\n  test.txt\n"
+        self.assertEquals(out, "unknown:\n  test.txt\n")
 
         out = capture("status test.txt --all")
-        assert out == "unknown:\n  test.txt\n"
+        self.assertEquals(out, "unknown:\n  test.txt\n")
 
         f = file('test2.txt', 'wt')
         f.write('goodbye cruel world...\n')
         f.close()
 
         out = capture("status test.txt")
-        assert out == "unknown:\n  test.txt\n"
+        self.assertEquals(out, "unknown:\n  test.txt\n")
 
         out = capture("status")
-        assert out == ("unknown:\n"
-                       "  test.txt\n"
-                       "  test2.txt\n")
+        self.assertEquals(out, ("unknown:\n" "  test.txt\n" "  test2.txt\n"))
 
         os.unlink('test2.txt')
 
         progress("command aliases")
         out = capture("st --all")
-        assert out == ("unknown:\n"
-                       "  test.txt\n")
+        self.assertEquals(out, ("unknown:\n" "  test.txt\n"))
 
         out = capture("stat")
-        assert out == ("unknown:\n"
-                       "  test.txt\n")
+        self.assertEquals(out, ("unknown:\n" "  test.txt\n"))
 
         progress("command help")
         runbzr("help st")
         runbzr("help")
         runbzr("help commands")
-        runbzr("help slartibartfast", 1)
+        runbzr("help slartibartfast", 3)
 
         out = capture("help ci")
         out.index('aliases: ')
 
         progress("can't rename unversioned file")
-        runbzr("rename test.txt new-test.txt", 1)
+        runbzr("rename test.txt new-test.txt", 3)
 
         progress("adding a file")
 
         runbzr("add test.txt")
-        assert capture("unknowns") == ''
-        assert capture("status --all") == ("added:\n"
-                                                "  test.txt\n")
+        self.assertEquals(capture("unknowns"), '')
+        self.assertEquals(capture("status --all"), ("added:\n" "  test.txt\n"))
 
         progress("rename newly-added file")
         runbzr("rename test.txt hello.txt")
-        assert os.path.exists("hello.txt")
-        assert not os.path.exists("test.txt")
+        self.assert_(os.path.exists("hello.txt"))
+        self.assert_(not os.path.exists("test.txt"))
 
-        assert capture("revno") == '0\n'
+        self.assertEquals(capture("revno"), '0\n')
 
         progress("add first revision")
         runbzr(['commit', '-m', 'add first revision'])
 
         progress("more complex renames")
         os.mkdir("sub1")
-        runbzr("rename hello.txt sub1", 1)
-        runbzr("rename hello.txt sub1/hello.txt", 1)
-        runbzr("move hello.txt sub1", 1)
+        runbzr("rename hello.txt sub1", 3)
+        runbzr("rename hello.txt sub1/hello.txt", 3)
+        runbzr("move hello.txt sub1", 3)
 
         runbzr("add sub1")
         runbzr("rename sub1 sub2")
         runbzr("move hello.txt sub2")
-        assert capture("relpath sub2/hello.txt") == os.path.join("sub2", "hello.txt\n")
+        self.assertEqual(capture("relpath sub2/hello.txt"),
+                         pathjoin("sub2", "hello.txt\n"))
 
-        assert exists("sub2")
-        assert exists("sub2/hello.txt")
-        assert not exists("sub1")
-        assert not exists("hello.txt")
+        self.assert_(exists("sub2"))
+        self.assert_(exists("sub2/hello.txt"))
+        self.assert_(not exists("sub1"))
+        self.assert_(not exists("hello.txt"))
 
         runbzr(['commit', '-m', 'commit with some things moved to subdirs'])
 
         mkdir("sub1")
         runbzr('add sub1')
         runbzr('move sub2/hello.txt sub1')
-        assert not exists('sub2/hello.txt')
-        assert exists('sub1/hello.txt')
+        self.assert_(not exists('sub2/hello.txt'))
+        self.assert_(exists('sub1/hello.txt'))
         runbzr('move sub2 sub1')
-        assert not exists('sub2')
-        assert exists('sub1/sub2')
+        self.assert_(not exists('sub2'))
+        self.assert_(exists('sub1/sub2'))
 
         runbzr(['commit', '-m', 'rename nested subdirectories'])
 
         chdir('sub1/sub2')
         self.assertEquals(capture('root')[:-1],
-                          os.path.join(self.test_dir, 'branch1'))
+                          pathjoin(self.test_dir, 'branch1'))
         runbzr('move ../hello.txt .')
-        assert exists('./hello.txt')
+        self.assert_(exists('./hello.txt'))
         self.assertEquals(capture('relpath hello.txt'),
-                          os.path.join('sub1', 'sub2', 'hello.txt') + '\n')
-        assert capture('relpath ../../sub1/sub2/hello.txt') == os.path.join('sub1', 'sub2', 'hello.txt\n')
+                          pathjoin('sub1', 'sub2', 'hello.txt') + '\n')
+        self.assertEquals(capture('relpath ../../sub1/sub2/hello.txt'), pathjoin('sub1', 'sub2', 'hello.txt\n'))
         runbzr(['commit', '-m', 'move to parent directory'])
         chdir('..')
-        assert capture('relpath sub2/hello.txt') == os.path.join('sub1', 'sub2', 'hello.txt\n')
+        self.assertEquals(capture('relpath sub2/hello.txt'), pathjoin('sub1', 'sub2', 'hello.txt\n'))
 
         runbzr('move sub2/hello.txt .')
-        assert exists('hello.txt')
+        self.assert_(exists('hello.txt'))
 
         f = file('hello.txt', 'wt')
         f.write('some nice new content\n')
         f.close()
 
         f = file('msg.tmp', 'wt')
-        f.write('this is my new commit\n')
+        f.write('this is my new commit\nand it has multiple lines, for fun')
         f.close()
 
         runbzr('commit -F msg.tmp')
 
-        assert capture('revno') == '5\n'
+        self.assertEquals(capture('revno'), '5\n')
         runbzr('export -r 5 export-5.tmp')
         runbzr('export export.tmp')
 
         runbzr('log')
         runbzr('log -v')
         runbzr('log -v --forward')
-        runbzr('log -m', retcode=1)
+        runbzr('log -m', retcode=3)
         log_out = capture('log -m commit')
-        assert "this is my new commit" in log_out
-        assert "rename nested" not in log_out
-        assert 'revision-id' not in log_out
-        assert 'revision-id' in capture('log --show-ids -m commit')
+        self.assert_("this is my new commit\n  and" in log_out)
+        self.assert_("rename nested" not in log_out)
+        self.assert_('revision-id' not in log_out)
+        self.assert_('revision-id' in capture('log --show-ids -m commit'))
+
+        log_out = capture('log --line')
+        for line in log_out.splitlines():
+            self.assert_(len(line) <= 79, len(line))
+        self.assert_("this is my new commit and" in log_out)
 
 
         progress("file with spaces in name")
         mkdir('sub directory')
         file('sub directory/file with spaces ', 'wt').write('see how this works\n')
         runbzr('add .')
-        runbzr('diff')
+        runbzr('diff', retcode=1)
         runbzr('commit -m add-spaces')
         runbzr('check')
 
@@ -693,103 +1172,150 @@
             runbzr('init')
             os.symlink("NOWHERE1", "link1")
             runbzr('add link1')
-            assert self.capture('unknowns') == ''
+            self.assertEquals(self.capture('unknowns'), '')
             runbzr(['commit', '-m', '1: added symlink link1'])
     
             mkdir('d1')
             runbzr('add d1')
-            assert self.capture('unknowns') == ''
+            self.assertEquals(self.capture('unknowns'), '')
             os.symlink("NOWHERE2", "d1/link2")
-            assert self.capture('unknowns') == 'd1/link2\n'
+            self.assertEquals(self.capture('unknowns'), 'd1/link2\n')
             # is d1/link2 found when adding d1
             runbzr('add d1')
-            assert self.capture('unknowns') == ''
+            self.assertEquals(self.capture('unknowns'), '')
             os.symlink("NOWHERE3", "d1/link3")
-            assert self.capture('unknowns') == 'd1/link3\n'
+            self.assertEquals(self.capture('unknowns'), 'd1/link3\n')
             runbzr(['commit', '-m', '2: added dir, symlink'])
     
             runbzr('rename d1 d2')
             runbzr('move d2/link2 .')
             runbzr('move link1 d2')
-            assert os.readlink("./link2") == "NOWHERE2"
-            assert os.readlink("d2/link1") == "NOWHERE1"
+            self.assertEquals(os.readlink("./link2"), "NOWHERE2")
+            self.assertEquals(os.readlink("d2/link1"), "NOWHERE1")
             runbzr('add d2/link3')
-            runbzr('diff')
+            runbzr('diff', retcode=1)
             runbzr(['commit', '-m', '3: rename of dir, move symlinks, add link3'])
     
             os.unlink("link2")
             os.symlink("TARGET 2", "link2")
             os.unlink("d2/link1")
             os.symlink("TARGET 1", "d2/link1")
-            runbzr('diff')
-            assert self.capture("relpath d2/link1") == "d2/link1\n"
+            runbzr('diff', retcode=1)
+            self.assertEquals(self.capture("relpath d2/link1"), "d2/link1\n")
             runbzr(['commit', '-m', '4: retarget of two links'])
     
             runbzr('remove d2/link1')
-            assert self.capture('unknowns') == 'd2/link1\n'
+            self.assertEquals(self.capture('unknowns'), 'd2/link1\n')
             runbzr(['commit', '-m', '5: remove d2/link1'])
+            # try with the rm alias
+            runbzr('add d2/link1')
+            runbzr(['commit', '-m', '6: add d2/link1'])
+            runbzr('rm d2/link1')
+            self.assertEquals(self.capture('unknowns'), 'd2/link1\n')
+            runbzr(['commit', '-m', '7: remove d2/link1'])
     
             os.mkdir("d1")
             runbzr('add d1')
             runbzr('rename d2/link3 d1/link3new')
-            assert self.capture('unknowns') == 'd2/link1\n'
-            runbzr(['commit', '-m', '6: remove d2/link1, move/rename link3'])
+            self.assertEquals(self.capture('unknowns'), 'd2/link1\n')
+            runbzr(['commit', '-m', '8: remove d2/link1, move/rename link3'])
             
             runbzr(['check'])
             
             runbzr(['export', '-r', '1', 'exp1.tmp'])
             chdir("exp1.tmp")
-            assert listdir_sorted(".") == [ "link1" ]
-            assert os.readlink("link1") == "NOWHERE1"
+            self.assertEquals(listdir_sorted("."), [ "link1" ])
+            self.assertEquals(os.readlink("link1"), "NOWHERE1")
             chdir("..")
             
             runbzr(['export', '-r', '2', 'exp2.tmp'])
             chdir("exp2.tmp")
-            assert listdir_sorted(".") == [ "d1", "link1" ]
+            self.assertEquals(listdir_sorted("."), [ "d1", "link1" ])
             chdir("..")
             
             runbzr(['export', '-r', '3', 'exp3.tmp'])
             chdir("exp3.tmp")
-            assert listdir_sorted(".") == [ "d2", "link2" ]
-            assert listdir_sorted("d2") == [ "link1", "link3" ]
-            assert os.readlink("d2/link1") == "NOWHERE1"
-            assert os.readlink("link2")    == "NOWHERE2"
+            self.assertEquals(listdir_sorted("."), [ "d2", "link2" ])
+            self.assertEquals(listdir_sorted("d2"), [ "link1", "link3" ])
+            self.assertEquals(os.readlink("d2/link1"), "NOWHERE1")
+            self.assertEquals(os.readlink("link2")   , "NOWHERE2")
             chdir("..")
             
             runbzr(['export', '-r', '4', 'exp4.tmp'])
             chdir("exp4.tmp")
-            assert listdir_sorted(".") == [ "d2", "link2" ]
-            assert os.readlink("d2/link1") == "TARGET 1"
-            assert os.readlink("link2")    == "TARGET 2"
-            assert listdir_sorted("d2") == [ "link1", "link3" ]
+            self.assertEquals(listdir_sorted("."), [ "d2", "link2" ])
+            self.assertEquals(os.readlink("d2/link1"), "TARGET 1")
+            self.assertEquals(os.readlink("link2")   , "TARGET 2")
+            self.assertEquals(listdir_sorted("d2"), [ "link1", "link3" ])
             chdir("..")
             
             runbzr(['export', '-r', '5', 'exp5.tmp'])
             chdir("exp5.tmp")
-            assert listdir_sorted(".") == [ "d2", "link2" ]
-            assert os.path.islink("link2")
-            assert listdir_sorted("d2")== [ "link3" ]
+            self.assertEquals(listdir_sorted("."), [ "d2", "link2" ])
+            self.assert_(os.path.islink("link2"))
+            self.assert_(listdir_sorted("d2")== [ "link3" ])
             chdir("..")
             
-            runbzr(['export', '-r', '6', 'exp6.tmp'])
+            runbzr(['export', '-r', '8', 'exp6.tmp'])
             chdir("exp6.tmp")
-            assert listdir_sorted(".") == [ "d1", "d2", "link2" ]
-            assert listdir_sorted("d1") == [ "link3new" ]
-            assert listdir_sorted("d2") == []
-            assert os.readlink("d1/link3new") == "NOWHERE3"
+            self.assertEqual(listdir_sorted("."), [ "d1", "d2", "link2"])
+            self.assertEquals(listdir_sorted("d1"), [ "link3new" ])
+            self.assertEquals(listdir_sorted("d2"), [])
+            self.assertEquals(os.readlink("d1/link3new"), "NOWHERE3")
             chdir("..")
         else:
             progress("skipping symlink tests")
 
 
-class HttpTests(TestCaseWithWebserver):
+class RemoteTests(object):
     """Test bzr ui commands against remote branches."""
 
     def test_branch(self):
         os.mkdir('from')
         branch = Branch.initialize('from')
-        branch.commit('empty commit for nonsense', allow_pointless=True)
+        branch.working_tree().commit('empty commit for nonsense', allow_pointless=True)
         url = self.get_remote_url('from')
         self.run_bzr('branch', url, 'to')
         branch = Branch.open('to')
         self.assertEqual(1, len(branch.revision_history()))
+
+    def test_log(self):
+        self.build_tree(['branch/', 'branch/file'])
+        branch = Branch.initialize('branch')
+        branch.working_tree().add(['file'])
+        branch.working_tree().commit('add file', rev_id='A')
+        url = self.get_remote_url('branch/file')
+        output = self.capture('log %s' % url)
+        self.assertEqual(8, len(output.split('\n')))
+        
+    def test_check(self):
+        self.build_tree(['branch/', 'branch/file'])
+        branch = Branch.initialize('branch')
+        branch.working_tree().add(['file'])
+        branch.working_tree().commit('add file', rev_id='A')
+        url = self.get_remote_url('branch/')
+        self.run_bzr('check', url)
+    
+    
+class HTTPTests(TestCaseWithWebserver, RemoteTests):
+    """Test various commands against a HTTP server."""
+    
+    
+class SFTPTestsAbsolute(TestCaseWithSFTPServer, RemoteTests):
+    """Test various commands against a SFTP server using abs paths."""
+
+    
+class SFTPTestsAbsoluteSibling(TestCaseWithSFTPServer, RemoteTests):
+    """Test various commands against a SFTP server using abs paths."""
+
+    def setUp(self):
+        super(SFTPTestsAbsoluteSibling, self).setUp()
+        self._override_home = '/dev/noone/runs/tests/here'
+
+    
+class SFTPTestsRelative(TestCaseWithSFTPServer, RemoteTests):
+    """Test various commands against a SFTP server using homedir rel paths."""
+
+    def setUp(self):
+        super(SFTPTestsRelative, self).setUp()
+        self._get_remote_is_absolute = False

=== renamed file 'bzrlib/selftest/versioning.py' => 'bzrlib/tests/blackbox/test_versioning.py'
--- bzrlib/selftest/versioning.py	2005-09-29 20:34:25 +0000
+++ bzrlib/tests/blackbox/test_versioning.py	2005-12-01 17:11:25 +0000
@@ -22,29 +22,26 @@
 
 import os
 
-from bzrlib.selftest import BzrTestBase, TestCaseInTempDir
+from bzrlib.tests import BzrTestBase, TestCaseInTempDir
 from bzrlib.branch import Branch
-
-import logging
-logger = logging.getLogger('bzr.test.versioning')
-debug = logger.debug
+from bzrlib.trace import mutter
+from bzrlib.osutils import pathjoin
 
 
 class TestVersioning(TestCaseInTempDir):
     
     def test_mkdir(self): 
         """Basic 'bzr mkdir' operation"""
-        from bzrlib.commands import run_bzr
 
-        run_bzr(['init'])
-        run_bzr(['mkdir', 'foo'])
+        self.run_bzr('init')
+        self.run_bzr('mkdir', 'foo')
         self.assert_(os.path.isdir('foo'))
 
-        self.assertRaises(OSError, run_bzr, ['mkdir', 'foo'])
+        self.run_bzr('mkdir', 'foo', retcode=3)
 
         from bzrlib.diff import compare_trees
         from bzrlib.branch import Branch
-        b = Branch.open('.')
+        b = Branch.open(u'.')
         
         delta = compare_trees(b.basis_tree(), b.working_tree())
 
@@ -54,120 +51,102 @@
         self.assertEquals(delta.added[0][0], 'foo')
         self.failIf(delta.modified)
 
-    def test_branch_add_in_unversioned(self):
+    def test_mkdir_in_subdir(self):
+        """'bzr mkdir' operation in subdirectory"""
+
+        self.run_bzr('init')
+        self.run_bzr('mkdir', 'dir')
+        self.assert_(os.path.isdir('dir'))
+
+        os.chdir('dir')
+        self.log('Run mkdir in subdir')
+        self.run_bzr('mkdir', 'subdir')
+        self.assert_(os.path.isdir('subdir'))
+        os.chdir('..')
+
+        from bzrlib.diff import compare_trees
+        from bzrlib.branch import Branch
+        b = Branch.open(u'.')
+        
+        delta = compare_trees(b.basis_tree(), b.working_tree())
+
+        self.log('delta.added = %r' % delta.added)
+
+        self.assertEquals(len(delta.added), 2)
+        self.assertEquals(delta.added[0][0], 'dir')
+        self.assertEquals(delta.added[1][0], pathjoin('dir','subdir'))
+        self.failIf(delta.modified)
+
+    def test_mkdir_w_nested_trees(self):
+        """'bzr mkdir' with nested trees"""
+
+        self.run_bzr('init')
+        os.mkdir('a')
+        os.chdir('a')
+        self.run_bzr('init')
+        os.mkdir('b')
+        os.chdir('b')
+        self.run_bzr('init')
+        os.chdir('../..')
+
+        self.run_bzr('mkdir', 'dir', 'a/dir', 'a/b/dir')
+        self.failUnless(os.path.isdir('dir'))
+        self.failUnless(os.path.isdir('a/dir'))
+        self.failUnless(os.path.isdir('a/b/dir'))
+
+        from bzrlib.diff import compare_trees
+        b = Branch.open(u'.')
+        b_a = Branch.open('a')
+        b_b = Branch.open('a/b')
+        
+        delta = compare_trees(b.basis_tree(), b.working_tree())
+        self.assertEquals(len(delta.added), 1)
+        self.assertEquals(delta.added[0][0], 'dir')
+        self.failIf(delta.modified)
+
+        delta = compare_trees(b_a.basis_tree(), b_a.working_tree())
+        self.assertEquals(len(delta.added), 1)
+        self.assertEquals(delta.added[0][0], 'dir')
+        self.failIf(delta.modified)
+
+        delta = compare_trees(b_b.basis_tree(), b_b.working_tree())
+        self.assertEquals(len(delta.added), 1)
+        self.assertEquals(delta.added[0][0], 'dir')
+        self.failIf(delta.modified)
+
+    def test_working_tree_add_in_unversioned(self):
         """Try to add a file in an unversioned directory.
 
-        "bzr add" adds the parent as necessary, but simple branch add
+        "bzr add" adds the parent as necessary, but simple working tree add
         doesn't do that.
         """
         from bzrlib.branch import Branch
         from bzrlib.errors import NotVersionedError
+        from bzrlib.workingtree import WorkingTree
 
-        b = Branch.initialize('.')
+        b = Branch.initialize(u'.')
 
         self.build_tree(['foo/',
                          'foo/hello'])
 
         self.assertRaises(NotVersionedError,
-                          b.add,
+                          WorkingTree(b.base, b).add,
                           'foo/hello')
         
         self.check_branch()
 
-
-    def test_add_in_unversioned(self):
-        """Try to add a file in an unversioned directory.
-
-        "bzr add" should add the parent(s) as necessary.
-        """
-        from bzrlib.branch import Branch
-        from bzrlib.commands import run_bzr
-        eq = self.assertEqual
-
-        b = Branch.initialize('.')
-
-        self.build_tree(['inertiatic/', 'inertiatic/esp'])
-        eq(list(b.unknowns()), ['inertiatic'])
-        run_bzr(['add', 'inertiatic/esp'])
-        eq(list(b.unknowns()), [])
-
-        # Multiple unversioned parents
-        self.build_tree(['veil/', 'veil/cerpin/', 'veil/cerpin/taxt'])
-        eq(list(b.unknowns()), ['veil'])
-        run_bzr(['add', 'veil/cerpin/taxt'])
-        eq(list(b.unknowns()), [])
-
-        # Check whacky paths work
-        self.build_tree(['cicatriz/', 'cicatriz/esp'])
-        eq(list(b.unknowns()), ['cicatriz'])
-        run_bzr(['add', 'inertiatic/../cicatriz/esp'])
-        eq(list(b.unknowns()), [])
-
-    def test_add_in_versioned(self):
-        """Try to add a file in a versioned directory.
-
-        "bzr add" should do this happily.
-        """
-        from bzrlib.branch import Branch
-        from bzrlib.commands import run_bzr
-        eq = self.assertEqual
-
-        b = Branch.initialize('.')
-
-        self.build_tree(['inertiatic/', 'inertiatic/esp'])
-        eq(list(b.unknowns()), ['inertiatic'])
-        run_bzr(['add', '--no-recurse', 'inertiatic'])
-        eq(list(b.unknowns()), ['inertiatic'+os.sep+'esp'])
-        run_bzr(['add', 'inertiatic/esp'])
-        eq(list(b.unknowns()), [])
-
-    def test_subdir_add(self):
-        """Add in subdirectory should add only things from there down"""
-        
-        from bzrlib.branch import Branch
-        from bzrlib.commands import run_bzr
-        
-        eq = self.assertEqual
-        ass = self.assert_
-        chdir = os.chdir
-        
-        b = Branch.initialize('.')
-        self.build_tree(['src/', 'README'])
-        
-        eq(sorted(b.unknowns()),
-           ['README', 'src'])
-        
-        eq(run_bzr(['add', 'src']), 0)
-        
-        self.build_tree(['src/foo.c'])
-        
-        chdir('src')
-        eq(run_bzr(['add']), 0)
-        
-        eq(sorted(b.unknowns()), 
-           ['README'])
-        eq(len(b.inventory), 3)
-                
-        chdir('..')
-        eq(run_bzr(['add']), 0)
-        eq(list(b.unknowns()), [])
-
-        self.check_branch()
-
-
     def check_branch(self):
         """After all the above changes, run the check and upgrade commands.
 
         The upgrade should be a no-op."""
-        b = Branch.open('.')
-        debug('branch has %d revisions', b.revno())
+        b = Branch.open(u'.')
+        mutter('branch has %d revisions', b.revno())
         
-        debug('check branch...')
+        mutter('check branch...')
         from bzrlib.check import check
-        check(b)
-        
-
-        
+        check(b, False)
+
+
 class SubdirCommit(TestCaseInTempDir):
 
     def test_subdir_commit(self):
@@ -178,7 +157,7 @@
         self.build_tree(['a/', 'b/'])
         
         run_bzr('init')
-        b = Branch.open('.')
+        b = Branch.open(u'.')
         
         for fn in ('a/one', 'b/two', 'top'):
             file(fn, 'w').write('old contents')
@@ -189,7 +168,7 @@
         for fn in ('a/one', 'b/two', 'top'):
             file(fn, 'w').write('new contents')
             
-        debug('start selective subdir commit')
+        mutter('start selective subdir commit')
         run_bzr('commit', 'a', '-m', 'commit a only')
         
         old = b.revision_tree(b.get_rev_id(1))

=== added file 'bzrlib/tests/stub_sftp.py'
--- bzrlib/tests/stub_sftp.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/stub_sftp.py	2006-01-03 02:07:06 +0000
@@ -0,0 +1,186 @@
+# Copyright (C) 2005 Robey Pointer <robey@lag.net>, Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""
+A stub SFTP server for loopback SFTP testing.
+Adapted from the one in paramiko's unit tests.
+"""
+
+import os
+from paramiko import ServerInterface, SFTPServerInterface, SFTPServer, SFTPAttributes, \
+    SFTPHandle, SFTP_OK, AUTH_SUCCESSFUL, OPEN_SUCCEEDED
+from bzrlib.osutils import pathjoin
+from bzrlib.trace import mutter
+
+
+class StubServer (ServerInterface):
+    def __init__(self, test_case):
+        ServerInterface.__init__(self)
+        self._test_case = test_case
+
+    def check_auth_password(self, username, password):
+        # all are allowed
+        self._test_case.log('sftpserver - authorizing: %s' % (username,))
+        return AUTH_SUCCESSFUL
+
+    def check_channel_request(self, kind, chanid):
+        self._test_case.log('sftpserver - channel request: %s, %s' % (kind, chanid))
+        return OPEN_SUCCEEDED
+
+
+class StubSFTPHandle (SFTPHandle):
+    def stat(self):
+        try:
+            return SFTPAttributes.from_stat(os.fstat(self.readfile.fileno()))
+        except OSError, e:
+            return SFTPServer.convert_errno(e.errno)
+
+    def chattr(self, attr):
+        # python doesn't have equivalents to fchown or fchmod, so we have to
+        # use the stored filename
+        mutter('Changing permissions on %s to %s', self.filename, attr)
+        try:
+            SFTPServer.set_file_attr(self.filename, attr)
+        except OSError, e:
+            return SFTPServer.convert_errno(e.errno)
+
+
+class StubSFTPServer (SFTPServerInterface):
+    def __init__(self, server, root, home=None):
+        SFTPServerInterface.__init__(self, server)
+        self.root = root
+        if home is None:
+            self.home = self.root
+        else:
+            self.home = home[len(self.root):]
+        if (len(self.home) > 0) and (self.home[0] == '/'):
+            self.home = self.home[1:]
+
+    def _realpath(self, path):
+        return self.root + self.canonicalize(path)
+
+    def canonicalize(self, path):
+        if os.path.isabs(path):
+            return os.path.normpath(path)
+        else:
+            return os.path.normpath('/' + os.path.join(self.home, path))
+
+    def chattr(self, path, attr):
+        try:
+            SFTPServer.set_file_attr(path, attr)
+        except OSError, e:
+            return SFTPServer.convert_errno(e.errno)
+        return SFTP_OK
+
+    def list_folder(self, path):
+        path = self._realpath(path)
+        try:
+            out = [ ]
+            flist = os.listdir(path)
+            for fname in flist:
+                attr = SFTPAttributes.from_stat(os.stat(pathjoin(path, fname)))
+                attr.filename = fname
+                out.append(attr)
+            return out
+        except OSError, e:
+            return SFTPServer.convert_errno(e.errno)
+
+    def stat(self, path):
+        path = self._realpath(path)
+        try:
+            return SFTPAttributes.from_stat(os.stat(path))
+        except OSError, e:
+            return SFTPServer.convert_errno(e.errno)
+
+    def lstat(self, path):
+        path = self._realpath(path)
+        try:
+            return SFTPAttributes.from_stat(os.lstat(path))
+        except OSError, e:
+            return SFTPServer.convert_errno(e.errno)
+
+    def open(self, path, flags, attr):
+        path = self._realpath(path)
+        try:
+            if hasattr(os, 'O_BINARY'):
+                flags |= os.O_BINARY
+            if (attr is not None) and hasattr(attr, 'st_mode'):
+                fd = os.open(path, flags, attr.st_mode)
+            else:
+                fd = os.open(path, flags)
+        except OSError, e:
+            return SFTPServer.convert_errno(e.errno)
+        if (flags & os.O_CREAT) and (attr is not None):
+            attr._flags &= ~attr.FLAG_PERMISSIONS
+            SFTPServer.set_file_attr(path, attr)
+        if flags & os.O_WRONLY:
+            fstr = 'wb'
+        elif flags & os.O_RDWR:
+            fstr = 'rb+'
+        else:
+            # O_RDONLY (== 0)
+            fstr = 'rb'
+        try:
+            f = os.fdopen(fd, fstr)
+        except OSError, e:
+            return SFTPServer.convert_errno(e.errno)
+        fobj = StubSFTPHandle()
+        fobj.filename = path
+        fobj.readfile = f
+        fobj.writefile = f
+        return fobj
+
+    def remove(self, path):
+        path = self._realpath(path)
+        try:
+            os.remove(path)
+        except OSError, e:
+            return SFTPServer.convert_errno(e.errno)
+        return SFTP_OK
+
+    def rename(self, oldpath, newpath):
+        oldpath = self._realpath(oldpath)
+        newpath = self._realpath(newpath)
+        try:
+            os.rename(oldpath, newpath)
+        except OSError, e:
+            return SFTPServer.convert_errno(e.errno)
+        return SFTP_OK
+
+    def mkdir(self, path, attr):
+        path = self._realpath(path)
+        try:
+            if attr is not None and hasattr(attr, 'st_mode'):
+                os.mkdir(path, attr.st_mode)
+            else:
+                os.mkdir(path)
+            if attr is not None:
+                attr._flags &= ~attr.FLAG_PERMISSIONS
+                SFTPServer.set_file_attr(path, attr)
+        except OSError, e:
+            return SFTPServer.convert_errno(e.errno)
+        return SFTP_OK
+
+    def rmdir(self, path):
+        path = self._realpath(path)
+        try:
+            os.rmdir(path)
+        except OSError, e:
+            return SFTPServer.convert_errno(e.errno)
+        return SFTP_OK
+
+    # removed: chattr, symlink, readlink
+    # (nothing in bzr's sftp transport uses those)

=== modified file 'bzrlib/tests/test_ancestry.py'
--- bzrlib/selftest/test_ancestry.py	2005-10-06 06:31:33 +0000
+++ bzrlib/tests/test_ancestry.py	2005-11-30 00:25:50 +0000
@@ -19,7 +19,7 @@
 
 from cStringIO import StringIO
 
-from bzrlib.selftest import TestCase, TestCaseInTempDir
+from bzrlib.tests import TestCase, TestCaseInTempDir
 from bzrlib.branch import Branch
 from bzrlib.revision import is_ancestor
 
@@ -28,15 +28,16 @@
 
     def test_straightline_ancestry(self):
         """Test ancestry file when just committing."""
-        b = Branch.initialize('.')
-
-        b.commit(message='one',
-                 allow_pointless=True,
-                 rev_id='tester@foo--1')
-
-        b.commit(message='two',
-                 allow_pointless=True,
-                 rev_id='tester@foo--2')
+        b = Branch.initialize(u'.')
+        wt = b.working_tree()
+
+        wt.commit(message='one',
+                  allow_pointless=True,
+                  rev_id='tester@foo--1')
+
+        wt.commit(message='two',
+                  allow_pointless=True,
+                  rev_id='tester@foo--2')
 
         ancs = b.get_ancestry('tester@foo--2')
         self.assertEqual([None, 'tester@foo--1', 'tester@foo--2'], ancs)
@@ -44,12 +45,13 @@
                          b.get_ancestry('tester@foo--1'))
 
     def test_none_is_always_an_ancestor(self):
-        b = Branch.initialize('.')
+        b = Branch.initialize(u'.')
         # note this is tested before any commits are done.
         self.assertEqual(True, is_ancestor(None, None, b))
-        b.commit(message='one',
-                 allow_pointless=True,
-                 rev_id='tester@foo--1')
+        wt = b.working_tree()
+        wt.commit(message='one',
+                  allow_pointless=True,
+                  rev_id='tester@foo--1')
         self.assertEqual(True, is_ancestor(None, None, b))
         self.assertEqual(True, is_ancestor('tester@foo--1', None, b))
         self.assertEqual(False, is_ancestor(None, 'tester@foo--1', b))

=== added file 'bzrlib/tests/test_annotate.py'
--- bzrlib/tests/test_annotate.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_annotate.py	2005-11-30 00:25:50 +0000
@@ -0,0 +1,69 @@
+# Copyright (C) 2005 by Canonical Ltd
+# -*- coding: utf-8 -*-
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+"""Black-box tests for bzr.
+
+These check that it behaves properly when it's invoked through the regular
+command-line interface. This doesn't actually run a new interpreter but 
+rather starts again from the run_bzr function.
+"""
+
+
+from cStringIO import StringIO
+import os
+import shutil
+import sys
+import os
+
+from bzrlib.branch import Branch
+from bzrlib.clone import copy_branch
+from bzrlib.errors import BzrCommandError
+from bzrlib.osutils import has_symlinks
+from bzrlib.tests import TestCaseInTempDir, BzrTestBase
+from bzrlib.annotate import annotate_file
+
+
+class TestAnnotate(TestCaseInTempDir):
+    def setUp(self):
+        super(TestAnnotate, self).setUp()
+        b = Branch.initialize(u'.')
+        self.build_tree_contents([('hello.txt', 'my helicopter\n'),
+                                  ('nomail.txt', 'nomail\n')])
+        b.working_tree().add(['hello.txt'])
+        b.working_tree().commit('add hello', 
+                                committer='test@user')
+        b.working_tree().add(['nomail.txt'])
+        b.working_tree().commit('add nomail', committer='no mail')
+
+    def test_help_annotate(self):
+        """Annotate command exists"""
+        out, err = self.run_bzr_captured(['--no-plugins', 'annotate', '--help'])
+
+    def test_annotate_cmd(self):
+        out, err = self.run_bzr_captured(['annotate', 'hello.txt'])
+        self.assertEquals(err, '')
+        self.assertEqualDiff(out, '''\
+    1 test@us | my helicopter
+''')
+
+    def test_no_mail(self):
+        out, err = self.run_bzr_captured(['annotate', 'nomail.txt'])
+        self.assertEquals(err, '')
+        self.assertEqualDiff(out, '''\
+    2 no mail | nomail
+''')

=== added file 'bzrlib/tests/test_api.py'
--- bzrlib/tests/test_api.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_api.py	2005-11-23 15:44:24 +0000
@@ -0,0 +1,37 @@
+# Copyright (C) 2005 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for library API infrastructure
+
+This is specifically for things controlling the interface, such as versioning.  
+Tests for particular parts of the library interface should be in specific
+relevant test modules.
+"""
+
+import os
+import sys
+
+import bzrlib
+from bzrlib.tests import TestCase
+
+class APITests(TestCase):
+
+    def test_library_version(self):
+        """Library API version is exposed"""
+        self.assert_(isinstance(bzrlib.__version__, str))
+        self.assert_(isinstance(bzrlib.version_string, str))
+        self.assert_(isinstance(bzrlib.version_info, tuple))
+        self.assertEqual(len(bzrlib.version_info), 5)

=== modified file 'bzrlib/tests/test_bad_files.py'
--- bzrlib/selftest/test_bad_files.py	2005-09-18 16:09:41 +0000
+++ bzrlib/tests/test_bad_files.py	2005-11-30 00:25:50 +0000
@@ -18,7 +18,7 @@
 """Tests being able to ignore mad filetypes.
 """
 
-from bzrlib.selftest import TestCaseInTempDir
+from bzrlib.tests import TestCaseInTempDir
 from bzrlib.errors import BadFileKindError
 import os
 
@@ -40,16 +40,16 @@
         from bzrlib.add import smart_add
         from bzrlib.branch import Branch
 
-        b = Branch.initialize('.')
+        b = Branch.initialize(u'.')
 
         self.build_tree(['one', 'two', 'three'])
-        smart_add('.')
+        smart_add(u'.')
         commit(b, "Commit one", rev_id="a@u-0-0")
         self.build_tree(['four'])
-        smart_add('.')
+        smart_add(u'.')
         commit(b, "Commit two", rev_id="a@u-0-1")
         self.build_tree(['five'])
-        smart_add('.')
+        smart_add(u'.')
         commit(b, "Commit three", rev_id="a@u-0-2")
 
         # We should now have a few files, lets try to
@@ -73,7 +73,7 @@
         
         # Make sure smart_add can handle having a bogus
         # file in the way
-        smart_add('.')
+        smart_add(u'.')
         verify_status(self, b,
                           ['added:\n',
                            '  six\n',
@@ -89,5 +89,5 @@
 
         # We should raise an error if we are adding a bogus file
         # Is there a way to test the actual error that should be raised?
-        self.run_bzr('add', 'a-fifo', retcode=1)
+        self.run_bzr('add', 'a-fifo', retcode=3)
 

=== added file 'bzrlib/tests/test_basis_inventory.py'
--- bzrlib/tests/test_basis_inventory.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_basis_inventory.py	2006-01-02 22:37:32 +0000
@@ -0,0 +1,55 @@
+# Copyright (C) 2004, 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+import os
+
+from bzrlib.tests import TestCaseInTempDir
+from bzrlib.branch import Branch
+from bzrlib.xml5 import serializer_v5
+
+
+class TestBasisInventory(TestCaseInTempDir):
+
+    def test_create(self):
+        # Make sure the basis file is created by a commit
+        b = Branch.initialize(u'.')
+        t = b.working_tree()
+        open('a', 'wb').write('a\n')
+        t.add('a')
+        t.commit('a', rev_id='r1')
+
+        self.failUnlessExists('.bzr/basis-inventory.r1')
+
+        basis_inv_txt = t.read_basis_inventory('r1')
+        basis_inv = serializer_v5.read_inventory_from_string(basis_inv_txt)
+        #self.assertEquals('r1', basis_inv.revision_id)
+        
+        store_inv = b.get_inventory('r1')
+        self.assertEquals(store_inv._byid, basis_inv._byid)
+
+        open('b', 'wb').write('b\n')
+        t.add('b')
+        t.commit('b', rev_id='r2')
+
+        self.failIfExists('.bzr/basis-inventory.r1')
+        self.failUnlessExists('.bzr/basis-inventory.r2')
+
+        basis_inv_txt = t.read_basis_inventory('r2')
+        basis_inv = serializer_v5.read_inventory_from_string(basis_inv_txt)
+        store_inv = b.get_inventory('r2')
+
+        self.assertEquals(store_inv._byid, basis_inv._byid)
+

=== renamed file 'bzrlib/selftest/testbranch.py' => 'bzrlib/tests/test_branch.py'
--- bzrlib/selftest/testbranch.py	2005-10-02 23:48:11 +0000
+++ bzrlib/tests/test_branch.py	2005-12-01 19:57:43 +0000
@@ -15,19 +15,30 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 import os
-from bzrlib.branch import Branch
+import sys
+
+from bzrlib.branch import Branch, needs_read_lock, needs_write_lock
 from bzrlib.clone import copy_branch
 from bzrlib.commit import commit
-from bzrlib.errors import NoSuchRevision, UnlistableBranch
-from bzrlib.selftest import TestCaseInTempDir
+import bzrlib.errors as errors
+from bzrlib.errors import NoSuchRevision, UnlistableBranch, NotBranchError
+import bzrlib.gpg
+from bzrlib.osutils import getcwd
+from bzrlib.tests import TestCase, TestCaseInTempDir
+from bzrlib.tests.HTTPTestUtil import TestCaseWithWebserver
 from bzrlib.trace import mutter
+import bzrlib.transactions as transactions
+from bzrlib.revision import NULL_REVISION
 
+# TODO: Make a branch using basis branch, and check that it 
+# doesn't request any files that could have been avoided, by 
+# hooking into the Transport.
 
 class TestBranch(TestCaseInTempDir):
 
     def test_append_revisions(self):
         """Test appending more than one revision"""
-        br = Branch.initialize(".")
+        br = Branch.initialize(u".")
         br.append_revision("rev1")
         self.assertEquals(br.revision_history(), ["rev1",])
         br.append_revision("rev2", "rev3")
@@ -40,9 +51,9 @@
         os.mkdir('b2')
         b1 = Branch.initialize('b1')
         b2 = Branch.initialize('b2')
-        file(os.sep.join(['b1', 'foo']), 'w').write('hello')
-        b1.add(['foo'], ['foo-id'])
-        b1.commit('lala!', rev_id='revision-1', allow_pointless=False)
+        file('b1/foo', 'w').write('hello')
+        b1.working_tree().add(['foo'], ['foo-id'])
+        b1.working_tree().commit('lala!', rev_id='revision-1', allow_pointless=False)
 
         mutter('start fetch')
         f = Fetcher(from_branch=b1, to_branch=b2)
@@ -54,19 +65,40 @@
         tree = b2.revision_tree('revision-1')
         eq(tree.get_file_text('foo-id'), 'hello')
 
-    def test_push_stores(self):
-        """Copy the stores from one branch to another"""
+    def test_revision_tree(self):
+        b1 = Branch.initialize(u'.')
+        b1.working_tree().commit('lala!', rev_id='revision-1', allow_pointless=True)
+        tree = b1.revision_tree('revision-1')
+        tree = b1.revision_tree(None)
+        self.assertEqual(len(tree.list_files()), 0)
+        tree = b1.revision_tree(NULL_REVISION)
+        self.assertEqual(len(tree.list_files()), 0)
+
+    def get_unbalanced_branch_pair(self):
+        """Return two branches, a and b, with one file in a."""
         os.mkdir('a')
         br_a = Branch.initialize("a")
         file('a/b', 'wb').write('b')
-        br_a.add('b')
-        commit(br_a, "silly commit")
-
+        br_a.working_tree().add('b')
+        commit(br_a, "silly commit", rev_id='A')
         os.mkdir('b')
         br_b = Branch.initialize("b")
+        return br_a, br_b
+
+    def get_balanced_branch_pair(self):
+        """Returns br_a, br_b as with one commit in a, and b has a's stores."""
+        br_a, br_b = self.get_unbalanced_branch_pair()
+        br_a.push_stores(br_b)
+        return br_a, br_b
+
+    def test_push_stores(self):
+        """Copy the stores from one branch to another"""
+        br_a, br_b = self.get_unbalanced_branch_pair()
+        # ensure the revision is missing.
         self.assertRaises(NoSuchRevision, br_b.get_revision, 
                           br_a.revision_history()[0])
         br_a.push_stores(br_b)
+        # check that b now has all the data from a's first commit.
         rev = br_b.get_revision(br_a.revision_history()[0])
         tree = br_b.revision_tree(br_a.revision_history()[0])
         for file_id in tree:
@@ -76,41 +108,41 @@
 
     def test_copy_branch(self):
         """Copy the stores from one branch to another"""
-        br_a, br_b = self.test_push_stores()
+        br_a, br_b = self.get_balanced_branch_pair()
         commit(br_b, "silly commit")
         os.mkdir('c')
         br_c = copy_branch(br_a, 'c', basis_branch=br_b)
         self.assertEqual(br_a.revision_history(), br_c.revision_history())
-        ## # basis branches currently disabled for weave format
-        ## self.assertFalse(br_b.last_revision() in br_c.revision_history())
-        ## br_c.get_revision(br_b.last_revision())
 
     def test_copy_partial(self):
         """Copy only part of the history of a branch."""
         self.build_tree(['a/', 'a/one'])
         br_a = Branch.initialize('a')
-        br_a.add(['one'])
-        br_a.commit('commit one', rev_id='u@d-1')
+        br_a.working_tree().add(['one'])
+        br_a.working_tree().commit('commit one', rev_id='u@d-1')
         self.build_tree(['a/two'])
-        br_a.add(['two'])
-        br_a.commit('commit two', rev_id='u@d-2')
+        br_a.working_tree().add(['two'])
+        br_a.working_tree().commit('commit two', rev_id='u@d-2')
         br_b = copy_branch(br_a, 'b', revision='u@d-1')
         self.assertEqual(br_b.last_revision(), 'u@d-1')
         self.assertTrue(os.path.exists('b/one'))
         self.assertFalse(os.path.exists('b/two'))
         
-
     def test_record_initial_ghost_merge(self):
         """A pending merge with no revision present is still a merge."""
-        branch = Branch.initialize('.')
-        branch.add_pending_merge('non:existent@rev--ision--0--2')
-        branch.commit('pretend to merge nonexistent-revision', rev_id='first')
+        branch = Branch.initialize(u'.')
+        branch.working_tree().add_pending_merge('non:existent@rev--ision--0--2')
+        branch.working_tree().commit('pretend to merge nonexistent-revision', rev_id='first')
         rev = branch.get_revision(branch.last_revision())
         self.assertEqual(len(rev.parent_ids), 1)
         # parent_sha1s is not populated now, WTF. rbc 20051003
         self.assertEqual(len(rev.parent_sha1s), 0)
         self.assertEqual(rev.parent_ids[0], 'non:existent@rev--ision--0--2')
 
+    def test_bad_revision(self):
+        branch = Branch.initialize(u'.')
+        self.assertRaises(errors.InvalidRevisionId, branch.get_revision, None)
+
 # TODO 20051003 RBC:
 # compare the gpg-to-sign info for a commit with a ghost and 
 #     an identical tree without a ghost
@@ -118,18 +150,18 @@
         
     def test_pending_merges(self):
         """Tracking pending-merged revisions."""
-        b = Branch.initialize('.')
-
-        self.assertEquals(b.pending_merges(), [])
-        b.add_pending_merge('foo@azkhazan-123123-abcabc')
-        self.assertEquals(b.pending_merges(), ['foo@azkhazan-123123-abcabc'])
-        b.add_pending_merge('foo@azkhazan-123123-abcabc')
-        self.assertEquals(b.pending_merges(), ['foo@azkhazan-123123-abcabc'])
-        b.add_pending_merge('wibble@fofof--20050401--1928390812')
-        self.assertEquals(b.pending_merges(),
+        b = Branch.initialize(u'.')
+        wt = b.working_tree()
+        self.assertEquals(wt.pending_merges(), [])
+        wt.add_pending_merge('foo@azkhazan-123123-abcabc')
+        self.assertEquals(wt.pending_merges(), ['foo@azkhazan-123123-abcabc'])
+        wt.add_pending_merge('foo@azkhazan-123123-abcabc')
+        self.assertEquals(wt.pending_merges(), ['foo@azkhazan-123123-abcabc'])
+        wt.add_pending_merge('wibble@fofof--20050401--1928390812')
+        self.assertEquals(wt.pending_merges(),
                           ['foo@azkhazan-123123-abcabc',
                            'wibble@fofof--20050401--1928390812'])
-        b.commit("commit from base with two merges")
+        b.working_tree().commit("commit from base with two merges")
         rev = b.get_revision(b.revision_history()[0])
         self.assertEquals(len(rev.parent_ids), 2)
         self.assertEquals(rev.parent_ids[0],
@@ -137,15 +169,78 @@
         self.assertEquals(rev.parent_ids[1],
                            'wibble@fofof--20050401--1928390812')
         # list should be cleared when we do a commit
-        self.assertEquals(b.pending_merges(), [])
- 
-
+        self.assertEquals(wt.pending_merges(), [])
+
+    def test_sign_existing_revision(self):
+        branch = Branch.initialize(u'.')
+        branch.working_tree().commit("base", allow_pointless=True, rev_id='A')
+        from bzrlib.testament import Testament
+        branch.sign_revision('A', bzrlib.gpg.LoopbackGPGStrategy(None))
+        self.assertEqual(Testament.from_revision(branch, 'A').as_short_text(),
+                         branch.revision_store.get('A', 'sig').read())
+
+    def test_store_signature(self):
+        branch = Branch.initialize(u'.')
+        branch.store_revision_signature(bzrlib.gpg.LoopbackGPGStrategy(None),
+                                        'FOO', 'A')
+        self.assertEqual('FOO', branch.revision_store.get('A', 'sig').read())
+
+    def test__relcontrolfilename(self):
+        branch = Branch.initialize(u'.')
+        self.assertEqual('.bzr/%25', branch._rel_controlfilename('%'))
+        
+    def test__relcontrolfilename_empty(self):
+        branch = Branch.initialize(u'.')
+        self.assertEqual('.bzr', branch._rel_controlfilename(''))
+
+    def test_nicks(self):
+        """Branch nicknames"""
+        os.mkdir('bzr.dev')
+        branch = Branch.initialize('bzr.dev')
+        self.assertEqual(branch.nick, 'bzr.dev')
+        os.rename('bzr.dev', 'bzr.ab')
+        branch = Branch.open('bzr.ab')
+        self.assertEqual(branch.nick, 'bzr.ab')
+        branch.nick = "Aaron's branch"
+        branch.nick = "Aaron's branch"
+        self.failUnless(os.path.exists(branch.controlfilename("branch.conf")))
+        self.assertEqual(branch.nick, "Aaron's branch")
+        os.rename('bzr.ab', 'integration')
+        branch = Branch.open('integration')
+        self.assertEqual(branch.nick, "Aaron's branch")
+        branch.nick = u"\u1234"
+        self.assertEqual(branch.nick, u"\u1234")
+
+    def test_commit_nicks(self):
+        """Nicknames are committed to the revision"""
+        os.mkdir('bzr.dev')
+        branch = Branch.initialize('bzr.dev')
+        branch.nick = "My happy branch"
+        branch.working_tree().commit('My commit respect da nick.')
+        committed = branch.get_revision(branch.last_revision())
+        self.assertEqual(committed.properties["branch-nick"], 
+                         "My happy branch")
+
+
+class TestRemote(TestCaseWithWebserver):
+
+    def test_open_containing(self):
+        self.assertRaises(NotBranchError, Branch.open_containing,
+                          self.get_remote_url(''))
+        self.assertRaises(NotBranchError, Branch.open_containing,
+                          self.get_remote_url('g/p/q'))
+        b = Branch.initialize(u'.')
+        branch, relpath = Branch.open_containing(self.get_remote_url(''))
+        self.assertEqual('', relpath)
+        branch, relpath = Branch.open_containing(self.get_remote_url('g/p/q'))
+        self.assertEqual('g/p/q', relpath)
+        
 # TODO: rewrite this as a regular unittest, without relying on the displayed output        
 #         >>> from bzrlib.commit import commit
 #         >>> bzrlib.trace.silent = True
 #         >>> br1 = ScratchBranch(files=['foo', 'bar'])
-#         >>> br1.add('foo')
-#         >>> br1.add('bar')
+#         >>> br1.working_tree().add('foo')
+#         >>> br1.working_tree().add('bar')
 #         >>> commit(br1, "lala!", rev_id="REVISION-ID-1", verbose=False)
 #         >>> br2 = ScratchBranch()
 #         >>> br2.update_revisions(br1)
@@ -158,3 +253,147 @@
 #         Added 0 revisions.
 #         >>> br1.text_store.total_size() == br2.text_store.total_size()
 #         True
+
+class InstrumentedTransaction(object):
+
+    def finish(self):
+        self.calls.append('finish')
+
+    def __init__(self):
+        self.calls = []
+
+
+class TestDecorator(object):
+
+    def __init__(self):
+        self._calls = []
+
+    def lock_read(self):
+        self._calls.append('lr')
+
+    def lock_write(self):
+        self._calls.append('lw')
+
+    def unlock(self):
+        self._calls.append('ul')
+
+    @needs_read_lock
+    def do_with_read(self):
+        return 1
+
+    @needs_read_lock
+    def except_with_read(self):
+        raise RuntimeError
+
+    @needs_write_lock
+    def do_with_write(self):
+        return 2
+
+    @needs_write_lock
+    def except_with_write(self):
+        raise RuntimeError
+
+
+class TestDecorators(TestCase):
+
+    def test_needs_read_lock(self):
+        branch = TestDecorator()
+        self.assertEqual(1, branch.do_with_read())
+        self.assertEqual(['lr', 'ul'], branch._calls)
+
+    def test_excepts_in_read_lock(self):
+        branch = TestDecorator()
+        self.assertRaises(RuntimeError, branch.except_with_read)
+        self.assertEqual(['lr', 'ul'], branch._calls)
+
+    def test_needs_write_lock(self):
+        branch = TestDecorator()
+        self.assertEqual(2, branch.do_with_write())
+        self.assertEqual(['lw', 'ul'], branch._calls)
+
+    def test_excepts_in_write_lock(self):
+        branch = TestDecorator()
+        self.assertRaises(RuntimeError, branch.except_with_write)
+        self.assertEqual(['lw', 'ul'], branch._calls)
+
+
+class TestBranchTransaction(TestCaseInTempDir):
+
+    def setUp(self):
+        super(TestBranchTransaction, self).setUp()
+        self.branch = Branch.initialize(u'.')
+        
+    def test_default_get_transaction(self):
+        """branch.get_transaction on a new branch should give a PassThrough."""
+        self.failUnless(isinstance(self.branch.get_transaction(),
+                                   transactions.PassThroughTransaction))
+
+    def test__set_new_transaction(self):
+        self.branch._set_transaction(transactions.ReadOnlyTransaction())
+
+    def test__set_over_existing_transaction_raises(self):
+        self.branch._set_transaction(transactions.ReadOnlyTransaction())
+        self.assertRaises(errors.LockError,
+                          self.branch._set_transaction,
+                          transactions.ReadOnlyTransaction())
+
+    def test_finish_no_transaction_raises(self):
+        self.assertRaises(errors.LockError, self.branch._finish_transaction)
+
+    def test_finish_readonly_transaction_works(self):
+        self.branch._set_transaction(transactions.ReadOnlyTransaction())
+        self.branch._finish_transaction()
+        self.assertEqual(None, self.branch._transaction)
+
+    def test_unlock_calls_finish(self):
+        self.branch.lock_read()
+        transaction = InstrumentedTransaction()
+        self.branch._transaction = transaction
+        self.branch.unlock()
+        self.assertEqual(['finish'], transaction.calls)
+
+    def test_lock_read_acquires_ro_transaction(self):
+        self.branch.lock_read()
+        self.failUnless(isinstance(self.branch.get_transaction(),
+                                   transactions.ReadOnlyTransaction))
+        self.branch.unlock()
+        
+    def test_lock_write_acquires_passthrough_transaction(self):
+        self.branch.lock_write()
+        # cannot use get_transaction as its magic
+        self.failUnless(isinstance(self.branch._transaction,
+                                   transactions.PassThroughTransaction))
+        self.branch.unlock()
+
+
+class TestBranchPushLocations(TestCaseInTempDir):
+
+    def setUp(self):
+        super(TestBranchPushLocations, self).setUp()
+        self.branch = Branch.initialize(u'.')
+        
+    def test_get_push_location_unset(self):
+        self.assertEqual(None, self.branch.get_push_location())
+
+    def test_get_push_location_exact(self):
+        from bzrlib.config import (branches_config_filename,
+                                   ensure_config_dir_exists)
+        ensure_config_dir_exists()
+        fn = branches_config_filename()
+        print >> open(fn, 'wt'), ("[%s]\n"
+                                  "push_location=foo" %
+                                  getcwd())
+        self.assertEqual("foo", self.branch.get_push_location())
+
+    def test_set_push_location(self):
+        from bzrlib.config import (branches_config_filename,
+                                   ensure_config_dir_exists)
+        ensure_config_dir_exists()
+        fn = branches_config_filename()
+        self.branch.set_push_location('foo')
+        self.assertFileEqual("[%s]\n"
+                             "push_location = foo" % getcwd(),
+                             fn)
+
+    # TODO RBC 20051029 test getting a push location from a branch in a 
+    # recursive section - that is, it appends the branch name.

=== added file 'bzrlib/tests/test_command.py'
--- bzrlib/tests/test_command.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_command.py	2005-11-23 15:44:24 +0000
@@ -0,0 +1,35 @@
+# Copyright (C) 2004, 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+from bzrlib.tests import TestCase
+from bzrlib.commands import display_command
+import errno
+
+class TestCommands(TestCase):
+    def test_display_command(self):
+        """EPIPE message is selectively suppressed"""
+        def pipe_thrower():
+            raise IOError(errno.EPIPE, "Bogus pipe error")
+        self.assertRaises(IOError, pipe_thrower)
+        @display_command
+        def non_thrower():
+            pipe_thrower()
+        non_thrower()
+        @display_command
+        def other_thrower():
+            raise IOError(errno.ESPIPE, "Bogus pipe error")
+        self.assertRaises(IOError, other_thrower)
+

=== modified file 'bzrlib/tests/test_commit.py'
--- bzrlib/selftest/test_commit.py	2005-10-06 07:03:33 +0000
+++ bzrlib/tests/test_commit.py	2005-12-01 17:36:29 +0000
@@ -17,26 +17,44 @@
 
 import os
 
-from bzrlib.selftest import TestCaseInTempDir
+import bzrlib
+from bzrlib.tests import TestCaseInTempDir
 from bzrlib.branch import Branch
+from bzrlib.workingtree import WorkingTree
 from bzrlib.commit import Commit
-from bzrlib.errors import PointlessCommit, BzrError
+from bzrlib.config import BranchConfig
+from bzrlib.errors import PointlessCommit, BzrError, SigningFailed
 
 
 # TODO: Test commit with some added, and added-but-missing files
 
+class MustSignConfig(BranchConfig):
+
+    def signature_needed(self):
+        return True
+
+    def gpg_signing_command(self):
+        return ['cat', '-']
+
+
+class BranchWithHooks(BranchConfig):
+
+    def post_commit(self):
+        return "bzrlib.ahook bzrlib.ahook"
+
+
 class TestCommit(TestCaseInTempDir):
 
     def test_simple_commit(self):
         """Commit and check two versions of a single file."""
-        b = Branch.initialize('.')
+        b = Branch.initialize(u'.')
         file('hello', 'w').write('hello world')
-        b.add('hello')
-        b.commit(message='add hello')
+        b.working_tree().add('hello')
+        b.working_tree().commit(message='add hello')
         file_id = b.working_tree().path2id('hello')
 
         file('hello', 'w').write('version 2')
-        b.commit(message='commit 2')
+        b.working_tree().commit(message='commit 2')
 
         eq = self.assertEquals
         eq(b.revno(), 2)
@@ -51,66 +69,62 @@
         tree2 = b.revision_tree(rh[1])
         eq(tree2.get_file_text(file_id), 'version 2')
 
-
     def test_delete_commit(self):
         """Test a commit with a deleted file"""
-        b = Branch.initialize('.')
+        b = Branch.initialize(u'.')
         file('hello', 'w').write('hello world')
-        b.add(['hello'], ['hello-id'])
-        b.commit(message='add hello')
+        b.working_tree().add(['hello'], ['hello-id'])
+        b.working_tree().commit(message='add hello')
 
         os.remove('hello')
-        b.commit('removed hello', rev_id='rev2')
+        b.working_tree().commit('removed hello', rev_id='rev2')
 
         tree = b.revision_tree('rev2')
         self.assertFalse(tree.has_id('hello-id'))
 
-
     def test_pointless_commit(self):
         """Commit refuses unless there are changes or it's forced."""
-        b = Branch.initialize('.')
+        b = Branch.initialize(u'.')
         file('hello', 'w').write('hello')
-        b.add(['hello'])
-        b.commit(message='add hello')
+        b.working_tree().add(['hello'])
+        b.working_tree().commit(message='add hello')
         self.assertEquals(b.revno(), 1)
         self.assertRaises(PointlessCommit,
-                          b.commit,
+                          b.working_tree().commit,
                           message='fails',
                           allow_pointless=False)
         self.assertEquals(b.revno(), 1)
         
-
-
     def test_commit_empty(self):
         """Commiting an empty tree works."""
-        b = Branch.initialize('.')
-        b.commit(message='empty tree', allow_pointless=True)
+        b = Branch.initialize(u'.')
+        b.working_tree().commit(message='empty tree', allow_pointless=True)
         self.assertRaises(PointlessCommit,
-                          b.commit,
+                          b.working_tree().commit,
                           message='empty tree',
                           allow_pointless=False)
-        b.commit(message='empty tree', allow_pointless=True)
+        b.working_tree().commit(message='empty tree', allow_pointless=True)
         self.assertEquals(b.revno(), 2)
 
 
     def test_selective_delete(self):
         """Selective commit in tree with deletions"""
-        b = Branch.initialize('.')
+        b = Branch.initialize(u'.')
         file('hello', 'w').write('hello')
         file('buongia', 'w').write('buongia')
-        b.add(['hello', 'buongia'],
+        b.working_tree().add(['hello', 'buongia'],
               ['hello-id', 'buongia-id'])
-        b.commit(message='add files',
+        b.working_tree().commit(message='add files',
                  rev_id='test@rev-1')
         
         os.remove('hello')
         file('buongia', 'w').write('new text')
-        b.commit(message='update text',
+        b.working_tree().commit(message='update text',
                  specific_files=['buongia'],
                  allow_pointless=False,
                  rev_id='test@rev-2')
 
-        b.commit(message='remove hello',
+        b.working_tree().commit(message='remove hello',
                  specific_files=['hello'],
                  allow_pointless=False,
                  rev_id='test@rev-3')
@@ -130,13 +144,14 @@
 
     def test_commit_rename(self):
         """Test commit of a revision where a file is renamed."""
-        b = Branch.initialize('.')
-        self.build_tree(['hello'])
-        b.add(['hello'], ['hello-id'])
-        b.commit(message='one', rev_id='test@rev-1', allow_pointless=False)
+        b = Branch.initialize(u'.')
+        tree = WorkingTree(u'.', b)
+        self.build_tree(['hello'], line_endings='binary')
+        tree.add(['hello'], ['hello-id'])
+        tree.commit(message='one', rev_id='test@rev-1', allow_pointless=False)
 
-        b.rename_one('hello', 'fruity')
-        b.commit(message='renamed', rev_id='test@rev-2', allow_pointless=False)
+        tree.rename_one('hello', 'fruity')
+        tree.commit(message='renamed', rev_id='test@rev-2', allow_pointless=False)
 
         eq = self.assertEquals
         tree1 = b.revision_tree('test@rev-1')
@@ -154,64 +169,60 @@
         ie = tree2.inventory['hello-id']
         eq(ie.revision, 'test@rev-2')
 
-
     def test_reused_rev_id(self):
         """Test that a revision id cannot be reused in a branch"""
-        b = Branch.initialize('.')
-        b.commit('initial', rev_id='test@rev-1', allow_pointless=True)
+        b = Branch.initialize(u'.')
+        b.working_tree().commit('initial', rev_id='test@rev-1', allow_pointless=True)
         self.assertRaises(Exception,
-                          b.commit,
+                          b.working_tree().commit,
                           message='reused id',
                           rev_id='test@rev-1',
                           allow_pointless=True)
-                          
-
 
     def test_commit_move(self):
         """Test commit of revisions with moved files and directories"""
         eq = self.assertEquals
-        b = Branch.initialize('.')
+        b = Branch.initialize(u'.')
         r1 = 'test@rev-1'
         self.build_tree(['hello', 'a/', 'b/'])
-        b.add(['hello', 'a', 'b'], ['hello-id', 'a-id', 'b-id'])
-        b.commit('initial', rev_id=r1, allow_pointless=False)
-
-        b.move(['hello'], 'a')
+        b.working_tree().add(['hello', 'a', 'b'], ['hello-id', 'a-id', 'b-id'])
+        b.working_tree().commit('initial', rev_id=r1, allow_pointless=False)
+        b.working_tree().move(['hello'], 'a')
         r2 = 'test@rev-2'
-        b.commit('two', rev_id=r2, allow_pointless=False)
-        self.check_inventory_shape(b.inventory,
+        b.working_tree().commit('two', rev_id=r2, allow_pointless=False)
+        self.check_inventory_shape(b.working_tree().read_working_inventory(),
                                    ['a', 'a/hello', 'b'])
 
-        b.move(['b'], 'a')
+        b.working_tree().move(['b'], 'a')
         r3 = 'test@rev-3'
-        b.commit('three', rev_id=r3, allow_pointless=False)
-        self.check_inventory_shape(b.inventory,
+        b.working_tree().commit('three', rev_id=r3, allow_pointless=False)
+        self.check_inventory_shape(b.working_tree().read_working_inventory(),
                                    ['a', 'a/hello', 'a/b'])
         self.check_inventory_shape(b.get_revision_inventory(r3),
                                    ['a', 'a/hello', 'a/b'])
 
-        b.move([os.sep.join(['a', 'hello'])],
-               os.sep.join(['a', 'b']))
+        b.working_tree().move(['a/hello'], 'a/b')
         r4 = 'test@rev-4'
-        b.commit('four', rev_id=r4, allow_pointless=False)
-        self.check_inventory_shape(b.inventory,
+        b.working_tree().commit('four', rev_id=r4, allow_pointless=False)
+        self.check_inventory_shape(b.working_tree().read_working_inventory(),
                                    ['a', 'a/b/hello', 'a/b'])
 
         inv = b.get_revision_inventory(r4)
         eq(inv['hello-id'].revision, r4)
         eq(inv['a-id'].revision, r1)
         eq(inv['b-id'].revision, r3)
-
         
     def test_removed_commit(self):
-        """Test a commit with a removed file"""
-        b = Branch.initialize('.')
+        """Commit with a removed file"""
+        b = Branch.initialize(u'.')
+        wt = b.working_tree()
         file('hello', 'w').write('hello world')
-        b.add(['hello'], ['hello-id'])
-        b.commit(message='add hello')
+        b.working_tree().add(['hello'], ['hello-id'])
+        b.working_tree().commit(message='add hello')
 
-        b.remove('hello')
-        b.commit('removed hello', rev_id='rev2')
+        wt = b.working_tree()  # FIXME: kludge for aliasing of working inventory
+        wt.remove('hello')
+        b.working_tree().commit('removed hello', rev_id='rev2')
 
         tree = b.revision_tree('rev2')
         self.assertFalse(tree.has_id('hello-id'))
@@ -219,15 +230,15 @@
 
     def test_committed_ancestry(self):
         """Test commit appends revisions to ancestry."""
-        b = Branch.initialize('.')
+        b = Branch.initialize(u'.')
         rev_ids = []
         for i in range(4):
             file('hello', 'w').write((str(i) * 4) + '\n')
             if i == 0:
-                b.add(['hello'], ['hello-id'])
+                b.working_tree().add(['hello'], ['hello-id'])
             rev_id = 'test@rev-%d' % (i+1)
             rev_ids.append(rev_id)
-            b.commit(message='rev %d' % (i+1),
+            b.working_tree().commit(message='rev %d' % (i+1),
                      rev_id=rev_id)
         eq = self.assertEquals
         eq(b.revision_history(), rev_ids)
@@ -236,13 +247,107 @@
             eq(anc, [None] + rev_ids[:i+1])
 
     def test_commit_new_subdir_child_selective(self):
-        b = Branch.initialize('.')
+        b = Branch.initialize(u'.')
         self.build_tree(['dir/', 'dir/file1', 'dir/file2'])
-        b.add(['dir', 'dir/file1', 'dir/file2'],
+        b.working_tree().add(['dir', 'dir/file1', 'dir/file2'],
               ['dirid', 'file1id', 'file2id'])
-        b.commit('dir/file1', specific_files=['dir/file1'], rev_id='1')
+        b.working_tree().commit('dir/file1', specific_files=['dir/file1'], rev_id='1')
         inv = b.get_inventory('1')
         self.assertEqual('1', inv['dirid'].revision)
         self.assertEqual('1', inv['file1id'].revision)
         # FIXME: This should raise a KeyError I think, rbc20051006
         self.assertRaises(BzrError, inv.__getitem__, 'file2id')
+
+    def test_strict_commit(self):
+        """Try and commit with unknown files and strict = True, should fail."""
+        from bzrlib.errors import StrictCommitFailed
+        b = Branch.initialize(u'.')
+        file('hello', 'w').write('hello world')
+        b.working_tree().add('hello')
+        file('goodbye', 'w').write('goodbye cruel world!')
+        self.assertRaises(StrictCommitFailed, b.working_tree().commit,
+            message='add hello but not goodbye', strict=True)
+
+    def test_strict_commit_without_unknowns(self):
+        """Try and commit with no unknown files and strict = True,
+        should work."""
+        from bzrlib.errors import StrictCommitFailed
+        b = Branch.initialize(u'.')
+        file('hello', 'w').write('hello world')
+        b.working_tree().add('hello')
+        b.working_tree().commit(message='add hello', strict=True)
+
+    def test_nonstrict_commit(self):
+        """Try and commit with unknown files and strict = False, should work."""
+        b = Branch.initialize(u'.')
+        file('hello', 'w').write('hello world')
+        b.working_tree().add('hello')
+        file('goodbye', 'w').write('goodbye cruel world!')
+        b.working_tree().commit(message='add hello but not goodbye', strict=False)
+
+    def test_nonstrict_commit_without_unknowns(self):
+        """Try and commit with no unknown files and strict = False,
+        should work."""
+        b = Branch.initialize(u'.')
+        file('hello', 'w').write('hello world')
+        b.working_tree().add('hello')
+        b.working_tree().commit(message='add hello', strict=False)
+
+    def test_signed_commit(self):
+        import bzrlib.gpg
+        import bzrlib.commit as commit
+        oldstrategy = bzrlib.gpg.GPGStrategy
+        branch = Branch.initialize(u'.')
+        branch.working_tree().commit("base", allow_pointless=True, rev_id='A')
+        self.failIf(branch.revision_store.has_id('A', 'sig'))
+        try:
+            from bzrlib.testament import Testament
+            # monkey patch gpg signing mechanism
+            bzrlib.gpg.GPGStrategy = bzrlib.gpg.LoopbackGPGStrategy
+            commit.Commit(config=MustSignConfig(branch)).commit(branch, "base",
+                                                      allow_pointless=True,
+                                                      rev_id='B')
+            self.assertEqual(Testament.from_revision(branch,'B').as_short_text(),
+                             branch.revision_store.get('B', 'sig').read())
+        finally:
+            bzrlib.gpg.GPGStrategy = oldstrategy
+
+    def test_commit_failed_signature(self):
+        import bzrlib.gpg
+        import bzrlib.commit as commit
+        oldstrategy = bzrlib.gpg.GPGStrategy
+        branch = Branch.initialize(u'.')
+        branch.working_tree().commit("base", allow_pointless=True, rev_id='A')
+        self.failIf(branch.revision_store.has_id('A', 'sig'))
+        try:
+            from bzrlib.testament import Testament
+            # monkey patch gpg signing mechanism
+            bzrlib.gpg.GPGStrategy = bzrlib.gpg.DisabledGPGStrategy
+            config = MustSignConfig(branch)
+            self.assertRaises(SigningFailed,
+                              commit.Commit(config=config).commit,
+                              branch, "base",
+                              allow_pointless=True,
+                              rev_id='B')
+            branch = Branch.open(u'.')
+            self.assertEqual(branch.revision_history(), ['A'])
+            self.failIf(branch.revision_store.has_id('B'))
+        finally:
+            bzrlib.gpg.GPGStrategy = oldstrategy
+
+    def test_commit_invokes_hooks(self):
+        import bzrlib.commit as commit
+        branch = Branch.initialize(u'.')
+        calls = []
+        def called(branch, rev_id):
+            calls.append('called')
+        bzrlib.ahook = called
+        try:
+            config = BranchWithHooks(branch)
+            commit.Commit(config=config).commit(
+                            branch, "base",
+                            allow_pointless=True,
+                            rev_id='A')
+            self.assertEqual(['called', 'called'], calls)
+        finally:
+            del bzrlib.ahook

=== modified file 'bzrlib/tests/test_commit_merge.py'
--- bzrlib/selftest/test_commit_merge.py	2005-09-30 14:27:41 +0000
+++ bzrlib/tests/test_commit_merge.py	2005-11-25 06:28:56 +0000
@@ -18,11 +18,11 @@
 import os
 import shutil
 
-from bzrlib.selftest import TestCaseInTempDir
+from bzrlib.tests import TestCaseInTempDir
 from bzrlib.branch import Branch
 from bzrlib.commit import commit
 from bzrlib.errors import PointlessCommit, BzrError, PointlessCommit
-from bzrlib.selftest.testrevision import make_branches
+from bzrlib.tests.test_revision import make_branches
 from bzrlib.fetch import fetch
 from bzrlib.check import check
 
@@ -44,13 +44,15 @@
         commit(bx, 'commit one', rev_id='x@u-0-1', allow_pointless=True)
         commit(by, 'commit two', rev_id='y@u-0-1', allow_pointless=True)
 
-        fetch(from_branch=bx, to_branch=by)
+        fetcher = fetch(from_branch=bx, to_branch=by)
+        self.assertEqual(1, fetcher.count_copied)
+        self.assertEqual([], fetcher.failed_revisions)
         # just having the history there does nothing
         self.assertRaises(PointlessCommit,
                           commit,
                           by, 'no changes yet', rev_id='y@u-0-2',
                           allow_pointless=False)
-        by.add_pending_merge('x@u-0-1')
+        by.working_tree().add_pending_merge('x@u-0-1')
         commit(by, 'merge from x', rev_id='y@u-0-2', allow_pointless=False)
 
         self.assertEquals(by.revno(), 2)
@@ -60,16 +62,14 @@
         self.assertEquals(rev.parent_ids,
                           ['y@u-0-1', 'x@u-0-1'])
 
-
-
     def test_merge_new_file(self):
         """Commit merge of two trees with no overlapping files."""
         self.build_tree(['x/', 'x/ecks', 'y/', 'y/why'])
 
         bx = Branch.initialize('x')
         by = Branch.initialize('y')
-        bx.add(['ecks'], ['ecks-id'])
-        by.add(['why'], ['why-id'])
+        bx.working_tree().add(['ecks'], ['ecks-id'])
+        by.working_tree().add(['why'], ['why-id'])
 
         commit(bx, 'commit one', rev_id='x@u-0-1', allow_pointless=True)
         commit(by, 'commit two', rev_id='y@u-0-1', allow_pointless=True)
@@ -77,8 +77,8 @@
         fetch(from_branch=bx, to_branch=by)
         # we haven't merged the texts, but let's fake it
         shutil.copyfile('x/ecks', 'y/ecks')
-        by.add(['ecks'], ['ecks-id'])
-        by.add_pending_merge('x@u-0-1')
+        by.working_tree().add(['ecks'], ['ecks-id'])
+        by.working_tree().add_pending_merge('x@u-0-1')
 
         # partial commit of merges is currently not allowed, because
         # it would give different merge graphs for each file which
@@ -94,5 +94,5 @@
         self.assertEquals(inv['ecks-id'].revision, 'x@u-0-1')
         self.assertEquals(inv['why-id'].revision, 'y@u-0-1')
 
-        check(bx)
-        check(by)
+        check(bx, False)
+        check(by, False)

=== added file 'bzrlib/tests/test_config.py'
--- bzrlib/tests/test_config.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_config.py	2005-12-30 22:03:42 +0000
@@ -0,0 +1,610 @@
+# Copyright (C) 2005 by Canonical Ltd
+#   Authors: Robert Collins <robert.collins@canonical.com>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for finding and reading the bzr config file[s]."""
+# import system imports here
+from bzrlib.util.configobj.configobj import ConfigObj, ConfigObjError
+from cStringIO import StringIO
+import os
+import sys
+
+#import bzrlib specific imports here
+import bzrlib.config as config
+import bzrlib.errors as errors
+from bzrlib.tests import TestCase, TestCaseInTempDir
+
+
+sample_config_text = ("[DEFAULT]\n"
+                      "email=Robert Collins <robertc@example.com>\n"
+                      "editor=vim\n"
+                      "gpg_signing_command=gnome-gpg\n"
+                      "user_global_option=something\n")
+
+
+sample_always_signatures = ("[DEFAULT]\n"
+                            "check_signatures=require\n")
+
+
+sample_ignore_signatures = ("[DEFAULT]\n"
+                            "check_signatures=ignore\n")
+
+
+sample_maybe_signatures = ("[DEFAULT]\n"
+                            "check_signatures=check-available\n")
+
+
+sample_branches_text = ("[http://www.example.com]\n"
+                        "# Top level policy\n"
+                        "email=Robert Collins <robertc@example.org>\n"
+                        "[http://www.example.com/useglobal]\n"
+                        "# different project, forces global lookup\n"
+                        "recurse=false\n"
+                        "[/b/]\n"
+                        "check_signatures=require\n"
+                        "# test trailing / matching with no children\n"
+                        "[/a/]\n"
+                        "check_signatures=check-available\n"
+                        "gpg_signing_command=false\n"
+                        "user_local_option=local\n"
+                        "# test trailing / matching\n"
+                        "[/a/*]\n"
+                        "#subdirs will match but not the parent\n"
+                        "recurse=False\n"
+                        "[/a/c]\n"
+                        "check_signatures=ignore\n"
+                        "post_commit=bzrlib.tests.test_config.post_commit\n"
+                        "#testing explicit beats globs\n")
+
+
+class InstrumentedConfigObj(object):
+    """A config obj look-enough-alike to record calls made to it."""
+
+    def __contains__(self, thing):
+        self._calls.append(('__contains__', thing))
+        return False
+
+    def __getitem__(self, key):
+        self._calls.append(('__getitem__', key))
+        return self
+
+    def __init__(self, input):
+        self._calls = [('__init__', input)]
+
+    def __setitem__(self, key, value):
+        self._calls.append(('__setitem__', key, value))
+
+    def write(self):
+        self._calls.append(('write',))
+
+
+class FakeBranch(object):
+
+    def __init__(self):
+        self.base = "http://example.com/branches/demo"
+        self.email = 'Robert Collins <robertc@example.net>\n'
+
+    def controlfile(self, filename, mode):
+        if filename != 'email':
+            raise NotImplementedError
+        if self.email is not None:
+            return StringIO(self.email)
+        raise errors.NoSuchFile(filename)
+
+
+class InstrumentedConfig(config.Config):
+    """An instrumented config that supplies stubs for template methods."""
+    
+    def __init__(self):
+        super(InstrumentedConfig, self).__init__()
+        self._calls = []
+        self._signatures = config.CHECK_NEVER
+
+    def _get_user_id(self):
+        self._calls.append('_get_user_id')
+        return "Robert Collins <robert.collins@example.org>"
+
+    def _get_signature_checking(self):
+        self._calls.append('_get_signature_checking')
+        return self._signatures
+
+
+class TestConfig(TestCase):
+
+    def test_constructs(self):
+        config.Config()
+ 
+    def test_no_default_editor(self):
+        self.assertRaises(NotImplementedError, config.Config().get_editor)
+
+    def test_user_email(self):
+        my_config = InstrumentedConfig()
+        self.assertEqual('robert.collins@example.org', my_config.user_email())
+        self.assertEqual(['_get_user_id'], my_config._calls)
+
+    def test_username(self):
+        my_config = InstrumentedConfig()
+        self.assertEqual('Robert Collins <robert.collins@example.org>',
+                         my_config.username())
+        self.assertEqual(['_get_user_id'], my_config._calls)
+
+    def test_signatures_default(self):
+        my_config = config.Config()
+        self.assertEqual(config.CHECK_IF_POSSIBLE,
+                         my_config.signature_checking())
+
+    def test_signatures_template_method(self):
+        my_config = InstrumentedConfig()
+        self.assertEqual(config.CHECK_NEVER, my_config.signature_checking())
+        self.assertEqual(['_get_signature_checking'], my_config._calls)
+
+    def test_signatures_template_method_none(self):
+        my_config = InstrumentedConfig()
+        my_config._signatures = None
+        self.assertEqual(config.CHECK_IF_POSSIBLE,
+                         my_config.signature_checking())
+        self.assertEqual(['_get_signature_checking'], my_config._calls)
+
+    def test_gpg_signing_command_default(self):
+        my_config = config.Config()
+        self.assertEqual('gpg', my_config.gpg_signing_command())
+
+    def test_get_user_option_default(self):
+        my_config = config.Config()
+        self.assertEqual(None, my_config.get_user_option('no_option'))
+
+    def test_post_commit_default(self):
+        my_config = config.Config()
+        self.assertEqual(None, my_config.post_commit())
+
+
+class TestConfigPath(TestCase):
+
+    def setUp(self):
+        super(TestConfigPath, self).setUp()
+        self.old_home = os.environ.get('HOME', None)
+        self.old_appdata = os.environ.get('APPDATA', None)
+        os.environ['HOME'] = '/home/bogus'
+        os.environ['APPDATA'] = \
+            r'C:\Documents and Settings\bogus\Application Data'
+
+    def tearDown(self):
+        if self.old_home is None:
+            del os.environ['HOME']
+        else:
+            os.environ['HOME'] = self.old_home
+        if self.old_appdata is None:
+            del os.environ['APPDATA']
+        else:
+            os.environ['APPDATA'] = self.old_appdata
+        super(TestConfigPath, self).tearDown()
+    
+    def test_config_dir(self):
+        if sys.platform == 'win32':
+            self.assertEqual(config.config_dir(), 
+                'C:/Documents and Settings/bogus/Application Data/bazaar/2.0')
+        else:
+            self.assertEqual(config.config_dir(), '/home/bogus/.bazaar')
+
+    def test_config_filename(self):
+        if sys.platform == 'win32':
+            self.assertEqual(config.config_filename(), 
+                'C:/Documents and Settings/bogus/Application Data/bazaar/2.0/bazaar.conf')
+        else:
+            self.assertEqual(config.config_filename(),
+                             '/home/bogus/.bazaar/bazaar.conf')
+
+    def test_branches_config_filename(self):
+        if sys.platform == 'win32':
+            self.assertEqual(config.branches_config_filename(), 
+                'C:/Documents and Settings/bogus/Application Data/bazaar/2.0/branches.conf')
+        else:
+            self.assertEqual(config.branches_config_filename(),
+                             '/home/bogus/.bazaar/branches.conf')
+
+class TestIniConfig(TestCase):
+
+    def test_contructs(self):
+        my_config = config.IniBasedConfig("nothing")
+
+    def test_from_fp(self):
+        config_file = StringIO(sample_config_text)
+        my_config = config.IniBasedConfig(None)
+        self.failUnless(
+            isinstance(my_config._get_parser(file=config_file),
+                        ConfigObj))
+
+    def test_cached(self):
+        config_file = StringIO(sample_config_text)
+        my_config = config.IniBasedConfig(None)
+        parser = my_config._get_parser(file=config_file)
+        self.failUnless(my_config._get_parser() is parser)
+
+
+class TestGetConfig(TestCase):
+
+    def test_constructs(self):
+        my_config = config.GlobalConfig()
+
+    def test_calls_read_filenames(self):
+        # replace the class that is constructured, to check its parameters
+        oldparserclass = config.ConfigObj
+        config.ConfigObj = InstrumentedConfigObj
+        my_config = config.GlobalConfig()
+        try:
+            parser = my_config._get_parser()
+        finally:
+            config.ConfigObj = oldparserclass
+        self.failUnless(isinstance(parser, InstrumentedConfigObj))
+        self.assertEqual(parser._calls, [('__init__', config.config_filename())])
+
+
+class TestBranchConfig(TestCaseInTempDir):
+
+    def test_constructs(self):
+        branch = FakeBranch()
+        my_config = config.BranchConfig(branch)
+        self.assertRaises(TypeError, config.BranchConfig)
+
+    def test_get_location_config(self):
+        branch = FakeBranch()
+        my_config = config.BranchConfig(branch)
+        location_config = my_config._get_location_config()
+        self.assertEqual(branch.base, location_config.location)
+        self.failUnless(location_config is my_config._get_location_config())
+
+
+class TestGlobalConfigItems(TestCase):
+
+    def test_user_id(self):
+        config_file = StringIO(sample_config_text)
+        my_config = config.GlobalConfig()
+        my_config._parser = my_config._get_parser(file=config_file)
+        self.assertEqual("Robert Collins <robertc@example.com>",
+                         my_config._get_user_id())
+
+    def test_absent_user_id(self):
+        config_file = StringIO("")
+        my_config = config.GlobalConfig()
+        my_config._parser = my_config._get_parser(file=config_file)
+        self.assertEqual(None, my_config._get_user_id())
+
+    def test_configured_editor(self):
+        config_file = StringIO(sample_config_text)
+        my_config = config.GlobalConfig()
+        my_config._parser = my_config._get_parser(file=config_file)
+        self.assertEqual("vim", my_config.get_editor())
+
+    def test_signatures_always(self):
+        config_file = StringIO(sample_always_signatures)
+        my_config = config.GlobalConfig()
+        my_config._parser = my_config._get_parser(file=config_file)
+        self.assertEqual(config.CHECK_ALWAYS,
+                         my_config.signature_checking())
+        self.assertEqual(True, my_config.signature_needed())
+
+    def test_signatures_if_possible(self):
+        config_file = StringIO(sample_maybe_signatures)
+        my_config = config.GlobalConfig()
+        my_config._parser = my_config._get_parser(file=config_file)
+        self.assertEqual(config.CHECK_IF_POSSIBLE,
+                         my_config.signature_checking())
+        self.assertEqual(False, my_config.signature_needed())
+
+    def test_signatures_ignore(self):
+        config_file = StringIO(sample_ignore_signatures)
+        my_config = config.GlobalConfig()
+        my_config._parser = my_config._get_parser(file=config_file)
+        self.assertEqual(config.CHECK_NEVER,
+                         my_config.signature_checking())
+        self.assertEqual(False, my_config.signature_needed())
+
+    def _get_sample_config(self):
+        config_file = StringIO(sample_config_text)
+        my_config = config.GlobalConfig()
+        my_config._parser = my_config._get_parser(file=config_file)
+        return my_config
+
+    def test_gpg_signing_command(self):
+        my_config = self._get_sample_config()
+        self.assertEqual("gnome-gpg", my_config.gpg_signing_command())
+        self.assertEqual(False, my_config.signature_needed())
+
+    def _get_empty_config(self):
+        config_file = StringIO("")
+        my_config = config.GlobalConfig()
+        my_config._parser = my_config._get_parser(file=config_file)
+        return my_config
+
+    def test_gpg_signing_command_unset(self):
+        my_config = self._get_empty_config()
+        self.assertEqual("gpg", my_config.gpg_signing_command())
+
+    def test_get_user_option_default(self):
+        my_config = self._get_empty_config()
+        self.assertEqual(None, my_config.get_user_option('no_option'))
+
+    def test_get_user_option_global(self):
+        my_config = self._get_sample_config()
+        self.assertEqual("something",
+                         my_config.get_user_option('user_global_option'))
+        
+    def test_post_commit_default(self):
+        my_config = self._get_sample_config()
+        self.assertEqual(None, my_config.post_commit())
+
+
+class TestLocationConfig(TestCase):
+
+    def test_constructs(self):
+        my_config = config.LocationConfig('http://example.com')
+        self.assertRaises(TypeError, config.LocationConfig)
+
+    def test_branch_calls_read_filenames(self):
+        # This is testing the correct file names are provided.
+        # TODO: consolidate with the test for GlobalConfigs filename checks.
+        #
+        # replace the class that is constructured, to check its parameters
+        oldparserclass = config.ConfigObj
+        config.ConfigObj = InstrumentedConfigObj
+        my_config = config.LocationConfig('http://www.example.com')
+        try:
+            parser = my_config._get_parser()
+        finally:
+            config.ConfigObj = oldparserclass
+        self.failUnless(isinstance(parser, InstrumentedConfigObj))
+        self.assertEqual(parser._calls,
+                         [('__init__', config.branches_config_filename())])
+
+    def test_get_global_config(self):
+        my_config = config.LocationConfig('http://example.com')
+        global_config = my_config._get_global_config()
+        self.failUnless(isinstance(global_config, config.GlobalConfig))
+        self.failUnless(global_config is my_config._get_global_config())
+
+    def test__get_section_no_match(self):
+        self.get_location_config('/')
+        self.assertEqual(None, self.my_config._get_section())
+        
+    def test__get_section_exact(self):
+        self.get_location_config('http://www.example.com')
+        self.assertEqual('http://www.example.com',
+                         self.my_config._get_section())
+   
+    def test__get_section_suffix_does_not(self):
+        self.get_location_config('http://www.example.com-com')
+        self.assertEqual(None, self.my_config._get_section())
+
+    def test__get_section_subdir_recursive(self):
+        self.get_location_config('http://www.example.com/com')
+        self.assertEqual('http://www.example.com',
+                         self.my_config._get_section())
+
+    def test__get_section_subdir_matches(self):
+        self.get_location_config('http://www.example.com/useglobal')
+        self.assertEqual('http://www.example.com/useglobal',
+                         self.my_config._get_section())
+
+    def test__get_section_subdir_nonrecursive(self):
+        self.get_location_config(
+            'http://www.example.com/useglobal/childbranch')
+        self.assertEqual('http://www.example.com',
+                         self.my_config._get_section())
+
+    def test__get_section_subdir_trailing_slash(self):
+        self.get_location_config('/b')
+        self.assertEqual('/b/', self.my_config._get_section())
+
+    def test__get_section_subdir_child(self):
+        self.get_location_config('/a/foo')
+        self.assertEqual('/a/*', self.my_config._get_section())
+
+    def test__get_section_subdir_child_child(self):
+        self.get_location_config('/a/foo/bar')
+        self.assertEqual('/a/', self.my_config._get_section())
+
+    def test__get_section_trailing_slash_with_children(self):
+        self.get_location_config('/a/')
+        self.assertEqual('/a/', self.my_config._get_section())
+
+    def test__get_section_explicit_over_glob(self):
+        self.get_location_config('/a/c')
+        self.assertEqual('/a/c', self.my_config._get_section())
+
+    def get_location_config(self, location, global_config=None):
+        if global_config is None:
+            global_file = StringIO(sample_config_text)
+        else:
+            global_file = StringIO(global_config)
+        branches_file = StringIO(sample_branches_text)
+        self.my_config = config.LocationConfig(location)
+        self.my_config._get_parser(branches_file)
+        self.my_config._get_global_config()._get_parser(global_file)
+
+    def test_location_without_username(self):
+        self.get_location_config('http://www.example.com/useglobal')
+        self.assertEqual('Robert Collins <robertc@example.com>',
+                         self.my_config.username())
+
+    def test_location_not_listed(self):
+        self.get_location_config('/home/robertc/sources')
+        self.assertEqual('Robert Collins <robertc@example.com>',
+                         self.my_config.username())
+
+    def test_overriding_location(self):
+        self.get_location_config('http://www.example.com/foo')
+        self.assertEqual('Robert Collins <robertc@example.org>',
+                         self.my_config.username())
+
+    def test_signatures_not_set(self):
+        self.get_location_config('http://www.example.com',
+                                 global_config=sample_ignore_signatures)
+        self.assertEqual(config.CHECK_NEVER,
+                         self.my_config.signature_checking())
+
+    def test_signatures_never(self):
+        self.get_location_config('/a/c')
+        self.assertEqual(config.CHECK_NEVER,
+                         self.my_config.signature_checking())
+        
+    def test_signatures_when_available(self):
+        self.get_location_config('/a/', global_config=sample_ignore_signatures)
+        self.assertEqual(config.CHECK_IF_POSSIBLE,
+                         self.my_config.signature_checking())
+        
+    def test_signatures_always(self):
+        self.get_location_config('/b')
+        self.assertEqual(config.CHECK_ALWAYS,
+                         self.my_config.signature_checking())
+        
+    def test_gpg_signing_command(self):
+        self.get_location_config('/b')
+        self.assertEqual("gnome-gpg", self.my_config.gpg_signing_command())
+
+    def test_gpg_signing_command_missing(self):
+        self.get_location_config('/a')
+        self.assertEqual("false", self.my_config.gpg_signing_command())
+
+    def test_get_user_option_global(self):
+        self.get_location_config('/a')
+        self.assertEqual('something',
+                         self.my_config.get_user_option('user_global_option'))
+
+    def test_get_user_option_local(self):
+        self.get_location_config('/a')
+        self.assertEqual('local',
+                         self.my_config.get_user_option('user_local_option'))
+        
+    def test_post_commit_default(self):
+        self.get_location_config('/a/c')
+        self.assertEqual('bzrlib.tests.test_config.post_commit',
+                         self.my_config.post_commit())
+
+
+class TestLocationConfig(TestCaseInTempDir):
+
+    def get_location_config(self, location, global_config=None):
+        if global_config is None:
+            global_file = StringIO(sample_config_text)
+        else:
+            global_file = StringIO(global_config)
+        branches_file = StringIO(sample_branches_text)
+        self.my_config = config.LocationConfig(location)
+        self.my_config._get_parser(branches_file)
+        self.my_config._get_global_config()._get_parser(global_file)
+
+    def test_set_user_setting_sets_and_saves(self):
+        self.get_location_config('/a/c')
+        record = InstrumentedConfigObj("foo")
+        self.my_config._parser = record
+
+        real_mkdir = os.mkdir
+        self.created = False
+        def checked_mkdir(path, mode=0777):
+            self.log('making directory: %s', path)
+            real_mkdir(path, mode)
+            self.created = True
+
+        os.mkdir = checked_mkdir
+        try:
+            self.my_config.set_user_option('foo', 'bar')
+        finally:
+            os.mkdir = real_mkdir
+
+        self.failUnless(self.created, 'Failed to create ~/.bazaar')
+        self.assertEqual([('__contains__', '/a/c'),
+                          ('__contains__', '/a/c/'),
+                          ('__setitem__', '/a/c', {}),
+                          ('__getitem__', '/a/c'),
+                          ('__setitem__', 'foo', 'bar'),
+                          ('write',)],
+                         record._calls[1:])
+
+
+class TestBranchConfigItems(TestCase):
+
+    def test_user_id(self):
+        branch = FakeBranch()
+        my_config = config.BranchConfig(branch)
+        self.assertEqual("Robert Collins <robertc@example.net>",
+                         my_config._get_user_id())
+        branch.email = "John"
+        self.assertEqual("John", my_config._get_user_id())
+
+    def test_not_set_in_branch(self):
+        branch = FakeBranch()
+        my_config = config.BranchConfig(branch)
+        branch.email = None
+        config_file = StringIO(sample_config_text)
+        (my_config._get_location_config().
+            _get_global_config()._get_parser(config_file))
+        self.assertEqual("Robert Collins <robertc@example.com>",
+                         my_config._get_user_id())
+        branch.email = "John"
+        self.assertEqual("John", my_config._get_user_id())
+
+    def test_BZREMAIL_OVERRIDES(self):
+        os.environ['BZREMAIL'] = "Robert Collins <robertc@example.org>"
+        branch = FakeBranch()
+        my_config = config.BranchConfig(branch)
+        self.assertEqual("Robert Collins <robertc@example.org>",
+                         my_config.username())
+    
+    def test_signatures_forced(self):
+        branch = FakeBranch()
+        my_config = config.BranchConfig(branch)
+        config_file = StringIO(sample_always_signatures)
+        (my_config._get_location_config().
+            _get_global_config()._get_parser(config_file))
+        self.assertEqual(config.CHECK_ALWAYS, my_config.signature_checking())
+
+    def test_gpg_signing_command(self):
+        branch = FakeBranch()
+        my_config = config.BranchConfig(branch)
+        config_file = StringIO(sample_config_text)
+        (my_config._get_location_config().
+            _get_global_config()._get_parser(config_file))
+        self.assertEqual('gnome-gpg', my_config.gpg_signing_command())
+
+    def test_get_user_option_global(self):
+        branch = FakeBranch()
+        my_config = config.BranchConfig(branch)
+        config_file = StringIO(sample_config_text)
+        (my_config._get_location_config().
+            _get_global_config()._get_parser(config_file))
+        self.assertEqual('something',
+                         my_config.get_user_option('user_global_option'))
+
+    def test_post_commit_default(self):
+        branch = FakeBranch()
+        branch.base='/a/c'
+        my_config = config.BranchConfig(branch)
+        config_file = StringIO(sample_config_text)
+        (my_config._get_location_config().
+            _get_global_config()._get_parser(config_file))
+        branch_file = StringIO(sample_branches_text)
+        my_config._get_location_config()._get_parser(branch_file)
+        self.assertEqual('bzrlib.tests.test_config.post_commit',
+                         my_config.post_commit())
+
+
+class TestMailAddressExtraction(TestCase):
+
+    def test_extract_email_address(self):
+        self.assertEqual('jane@test.com',
+                         config.extract_email_address('Jane <jane@test.com>'))
+        self.assertRaises(errors.BzrError,
+                          config.extract_email_address, 'Jane Tester')

=== modified file 'bzrlib/tests/test_conflicts.py'
--- bzrlib/selftest/test_conflicts.py	2005-10-06 03:15:08 +0000
+++ bzrlib/tests/test_conflicts.py	2005-11-30 00:25:50 +0000
@@ -17,10 +17,11 @@
 
 import os
 
-from bzrlib.selftest import TestCaseInTempDir
+from bzrlib.tests import TestCaseInTempDir
 from bzrlib.branch import Branch
 from bzrlib.commit import Commit
-
+from bzrlib.conflicts import restore
+from bzrlib.errors import NotConflicted
 
 # TODO: Test commit with some added, and added-but-missing files
 
@@ -28,17 +29,23 @@
 
     def test_conflicts(self):
         """Conflicts are detected properly"""
-        b = Branch.initialize('.')
-        file('hello', 'w').write('hello world')
-        file('hello.BASE', 'w').write('hello world')
+        b = Branch.initialize(u'.')
+        file('hello', 'w').write('hello world4')
+        file('hello.THIS', 'w').write('hello world2')
+        file('hello.BASE', 'w').write('hello world1')
+        file('hello.OTHER', 'w').write('hello world3')
         file('hello.sploo.BASE', 'w').write('yellow world')
+        file('hello.sploo.OTHER', 'w').write('yellow world2')
         tree = b.working_tree()
-        self.assertEqual(len(list(tree.list_files())), 3)
+        self.assertEqual(len(list(tree.list_files())), 6)
         conflicts = list(tree.iter_conflicts())
         self.assertEqual(len(conflicts), 2)
-        assert 'hello' in conflicts
-        assert 'hello.sploo' in conflicts
-        os.unlink('hello.BASE')
-        os.unlink('hello.sploo.BASE')
+        self.assert_('hello' in conflicts)
+        self.assert_('hello.sploo' in conflicts)
+        restore('hello')
+        restore('hello.sploo')
         self.assertEqual(len(list(tree.iter_conflicts())), 0)
-
+        self.assertFileEqual('hello world2', 'hello')
+        assert not os.path.lexists('hello.sploo')
+        self.assertRaises(NotConflicted, restore, 'hello')
+        self.assertRaises(NotConflicted, restore, 'hello.sploo')

=== renamed file 'bzrlib/selftest/testdiff.py' => 'bzrlib/tests/test_diff.py'
--- bzrlib/selftest/testdiff.py	2005-08-25 12:46:42 +0000
+++ bzrlib/tests/test_diff.py	2005-11-23 15:44:24 +0000
@@ -1,4 +1,4 @@
-from bzrlib.selftest import TestCase
+from bzrlib.tests import TestCase
 from bzrlib.diff import internal_diff
 from cStringIO import StringIO
 def udiff_lines(old, new):
@@ -7,42 +7,43 @@
     output.seek(0, 0)
     return output.readlines()
 
-def check_patch(lines):
-    assert len(lines) > 1, \
-        "Not enough lines for a file header for patch:\n%s" % "".join(lines)
-    assert lines[0].startswith ('---'), \
-        'No orig line for patch:\n%s' % "".join(lines)
-    assert lines[1].startswith ('+++'), \
-        'No mod line for patch:\n%s' % "".join(lines)
-    assert len(lines) > 2, \
-        "No hunks for patch:\n%s" % "".join(lines)
-    assert lines[2].startswith('@@'),\
-        "No hunk header for patch:\n%s" % "".join(lines)
-    assert '@@' in lines[2][2:], \
-        "Unterminated hunk header for patch:\n%s" % "".join(lines)
-
 class TestDiff(TestCase):
     def test_add_nl(self):
         """diff generates a valid diff for patches that add a newline"""
         lines = udiff_lines(['boo'], ['boo\n'])
-        check_patch(lines)
-        assert lines[4] == '\\ No newline at end of file\n', \
-            "expected no-nl, got %r" % lines[4]
+        self.check_patch(lines)
+        self.assertEquals(lines[4], '\\ No newline at end of file\n')
+            ## "expected no-nl, got %r" % lines[4]
 
     def test_add_nl_2(self):
         """diff generates a valid diff for patches that change last line and
         add a newline.
         """
         lines = udiff_lines(['boo'], ['goo\n'])
-        check_patch(lines)
-        assert lines[4] == '\\ No newline at end of file\n', \
-            "expected no-nl, got %r" % lines[4]
+        self.check_patch(lines)
+        self.assertEquals(lines[4], '\\ No newline at end of file\n')
+            ## "expected no-nl, got %r" % lines[4]
 
     def test_remove_nl(self):
         """diff generates a valid diff for patches that change last line and
         add a newline.
         """
         lines = udiff_lines(['boo\n'], ['boo'])
-        check_patch(lines)
-        assert lines[5] == '\\ No newline at end of file\n', \
-            "expected no-nl, got %r" % lines[5]
+        self.check_patch(lines)
+        self.assertEquals(lines[5], '\\ No newline at end of file\n')
+            ## "expected no-nl, got %r" % lines[5]
+
+    def check_patch(self, lines):
+        self.assert_(len(lines) > 1)
+            ## "Not enough lines for a file header for patch:\n%s" % "".join(lines)
+        self.assert_(lines[0].startswith ('---'))
+            ## 'No orig line for patch:\n%s' % "".join(lines)
+        self.assert_(lines[1].startswith ('+++'))
+            ## 'No mod line for patch:\n%s' % "".join(lines)
+        self.assert_(len(lines) > 2)
+            ## "No hunks for patch:\n%s" % "".join(lines)
+        self.assert_(lines[2].startswith('@@'))
+            ## "No hunk header for patch:\n%s" % "".join(lines)
+        self.assert_('@@' in lines[2][2:])
+            ## "Unterminated hunk header for patch:\n%s" % "".join(lines)
+

=== renamed file 'bzrlib/selftest/testfetch.py' => 'bzrlib/tests/test_fetch.py'
--- bzrlib/selftest/testfetch.py	2005-10-06 06:31:33 +0000
+++ bzrlib/tests/test_fetch.py	2006-01-10 05:50:28 +0000
@@ -18,18 +18,20 @@
 import sys
 
 import bzrlib.errors
-from bzrlib.selftest.testrevision import make_branches
+from bzrlib.tests.test_revision import make_branches
 from bzrlib.trace import mutter
 from bzrlib.branch import Branch
 from bzrlib.fetch import greedy_fetch
+from bzrlib.merge import merge
+from bzrlib.clone import copy_branch
 
-from bzrlib.selftest import TestCaseInTempDir
-from bzrlib.selftest.HTTPTestUtil import TestCaseWithWebserver
+from bzrlib.tests import TestCaseInTempDir
+from bzrlib.tests.HTTPTestUtil import TestCaseWithWebserver
 
 
 def has_revision(branch, revision_id):
     try:
-        branch.get_revision_xml_file(revision_id)
+        branch.get_revision_xml(revision_id)
         return True
     except bzrlib.errors.NoSuchRevision:
         return False
@@ -40,20 +42,20 @@
         os.mkdir(name)
         return Branch.initialize(name)
             
-    assert not has_revision(br_b, br_a.revision_history()[3])
-    assert has_revision(br_b, br_a.revision_history()[2])
-    assert len(br_b.revision_history()) == 7
-    assert greedy_fetch(br_b, br_a, br_a.revision_history()[2])[0] == 0
+    self.assertFalse(has_revision(br_b, br_a.revision_history()[3]))
+    self.assert_(has_revision(br_b, br_a.revision_history()[2]))
+    self.assertEquals(len(br_b.revision_history()), 7)
+    self.assertEquals(greedy_fetch(br_b, br_a, br_a.revision_history()[2])[0], 0)
 
     # greedy_fetch is not supposed to alter the revision history
-    assert len(br_b.revision_history()) == 7
-    assert not has_revision(br_b, br_a.revision_history()[3])
+    self.assertEquals(len(br_b.revision_history()), 7)
+    self.assertFalse(has_revision(br_b, br_a.revision_history()[3]))
 
-    assert len(br_b.revision_history()) == 7
-    assert greedy_fetch(br_b, br_a, br_a.revision_history()[3])[0] == 1
-    assert has_revision(br_b, br_a.revision_history()[3])
-    assert not has_revision(br_a, br_b.revision_history()[6])
-    assert has_revision(br_a, br_b.revision_history()[5])
+    self.assertEquals(len(br_b.revision_history()), 7)
+    self.assertEquals(greedy_fetch(br_b, br_a, br_a.revision_history()[3])[0], 1)
+    self.assert_(has_revision(br_b, br_a.revision_history()[3]))
+    self.assertFalse(has_revision(br_a, br_b.revision_history()[6]))
+    self.assert_(has_revision(br_a, br_b.revision_history()[5]))
 
     # When a non-branch ancestor is missing, it should be unlisted...
     # as its not reference from the inventory weave.
@@ -63,28 +65,28 @@
     self.assertEqual(failures, [])
 
     self.assertEqual(greedy_fetch(writable_a, br_b)[0], 1)
-    assert has_revision(br_a, br_b.revision_history()[3])
-    assert has_revision(br_a, br_b.revision_history()[4])
+    self.assert_(has_revision(br_a, br_b.revision_history()[3]))
+    self.assert_(has_revision(br_a, br_b.revision_history()[4]))
         
     br_b2 = new_branch('br_b2')
-    assert greedy_fetch(br_b2, br_b)[0] == 7
-    assert has_revision(br_b2, br_b.revision_history()[4])
-    assert has_revision(br_b2, br_a.revision_history()[2])
-    assert not has_revision(br_b2, br_a.revision_history()[3])
+    self.assertEquals(greedy_fetch(br_b2, br_b)[0], 7)
+    self.assert_(has_revision(br_b2, br_b.revision_history()[4]))
+    self.assert_(has_revision(br_b2, br_a.revision_history()[2]))
+    self.assertFalse(has_revision(br_b2, br_a.revision_history()[3]))
 
     br_a2 = new_branch('br_a2')
-    assert greedy_fetch(br_a2, br_a)[0] == 9
-    assert has_revision(br_a2, br_b.revision_history()[4])
-    assert has_revision(br_a2, br_a.revision_history()[3])
-    assert has_revision(br_a2, br_a.revision_history()[2])
+    self.assertEquals(greedy_fetch(br_a2, br_a)[0], 9)
+    self.assert_(has_revision(br_a2, br_b.revision_history()[4]))
+    self.assert_(has_revision(br_a2, br_a.revision_history()[3]))
+    self.assert_(has_revision(br_a2, br_a.revision_history()[2]))
 
     br_a3 = new_branch('br_a3')
-    assert greedy_fetch(br_a3, br_a2)[0] == 0
+    self.assertEquals(greedy_fetch(br_a3, br_a2)[0], 0)
     for revno in range(4):
-        assert not has_revision(br_a3, br_a.revision_history()[revno])
+        self.assertFalse(has_revision(br_a3, br_a.revision_history()[revno]))
     self.assertEqual(greedy_fetch(br_a3, br_a2, br_a.revision_history()[2])[0], 3)
     fetched = greedy_fetch(br_a3, br_a2, br_a.revision_history()[3])[0]
-    assert fetched == 3, "fetched %d instead of 3" % fetched
+    self.assertEquals(fetched, 3, "fetched %d instead of 3" % fetched)
     # InstallFailed should be raised if the branch is missing the revision
     # that was requested.
     self.assertRaises(bzrlib.errors.InstallFailed, greedy_fetch, br_a3,
@@ -96,43 +98,126 @@
                       br_a2)
 
 
+    #TODO: test that fetch correctly does reweaving when needed. RBC 20051008
+
 class TestFetch(TestCaseInTempDir):
 
     def test_fetch(self):
         #highest indices a: 5, b: 7
-        br_a, br_b = make_branches()
+        br_a, br_b = make_branches(self)
         fetch_steps(self, br_a, br_b, br_a)
 
 
+class TestMergeFetch(TestCaseInTempDir):
+
+    def test_merge_fetches_unrelated(self):
+        """Merge brings across history from unrelated source"""
+        os.mkdir('br1')
+        br1 = Branch.initialize('br1')
+        br1.working_tree().commit(message='rev 1-1', rev_id='1-1')
+        br1.working_tree().commit(message='rev 1-2', rev_id='1-2')
+        os.mkdir('br2')
+        br2 = Branch.initialize('br2')
+        br2.working_tree().commit(message='rev 2-1', rev_id='2-1')
+        merge(other_revision=['br1', -1], base_revision=['br1', 0],
+              this_dir='br2')
+        self._check_revs_present(br2)
+
+    def test_merge_fetches(self):
+        """Merge brings across history from source"""
+        os.mkdir('br1')
+        br1 = Branch.initialize('br1')
+        br1.working_tree().commit(message='rev 1-1', rev_id='1-1')
+        copy_branch(br1, 'br2')
+        br2 = Branch.open('br2')
+        br1.working_tree().commit(message='rev 1-2', rev_id='1-2')
+        br2.working_tree().commit(message='rev 2-1', rev_id='2-1')
+        merge(other_revision=['br1', -1], base_revision=[None, None], 
+              this_dir='br2')
+        self._check_revs_present(br2)
+
+    def _check_revs_present(self, br2):
+        for rev_id in '1-1', '1-2', '2-1':
+            self.assertTrue(br2.has_revision(rev_id))
+            rev = br2.get_revision(rev_id)
+            self.assertEqual(rev.revision_id, rev_id)
+            self.assertTrue(br2.get_inventory(rev_id))
+
+
+
+class TestMergeFileHistory(TestCaseInTempDir):
+    def setUp(self):
+        TestCaseInTempDir.setUp(self)
+        os.mkdir('br1')
+        br1 = Branch.initialize('br1')
+        self.build_tree_contents([('br1/file', 'original contents\n')])
+        br1.working_tree().add(['file'], ['this-file-id'])
+        br1.working_tree().commit(message='rev 1-1', rev_id='1-1')
+        copy_branch(br1, 'br2')
+        br2 = Branch.open('br2')
+        self.build_tree_contents([('br1/file', 'original from 1\n')])
+        br1.working_tree().commit(message='rev 1-2', rev_id='1-2')
+        self.build_tree_contents([('br1/file', 'agreement\n')])
+        br1.working_tree().commit(message='rev 1-3', rev_id='1-3')
+        self.build_tree_contents([('br2/file', 'contents in 2\n')])
+        br2.working_tree().commit(message='rev 2-1', rev_id='2-1')
+        self.build_tree_contents([('br2/file', 'agreement\n')])
+        br2.working_tree().commit(message='rev 2-2', rev_id='2-2')
+
+    def test_merge_fetches_file_history(self):
+        """Merge brings across file histories"""
+        br2 = Branch.open('br2')
+        merge(other_revision=['br1', -1], base_revision=[None, None], 
+              this_dir='br2')
+        for rev_id, text in [('1-2', 'original from 1\n'),
+                             ('1-3', 'agreement\n'),
+                             ('2-1', 'contents in 2\n'),
+                             ('2-2', 'agreement\n')]:
+            self.assertEqualDiff(br2.revision_tree(rev_id).get_file_text('this-file-id'),
+                                 text)
+
+
+
+
 class TestHttpFetch(TestCaseWithWebserver):
 
-    def setUp(self):
-        super(TestHttpFetch, self).setUp()
-        self.weblogs = []
-
     def test_fetch(self):
         #highest indices a: 5, b: 7
-        br_a, br_b = make_branches()
+        br_a, br_b = make_branches(self)
         br_rem_a = Branch.open(self.get_remote_url(br_a._transport.base))
         fetch_steps(self, br_rem_a, br_b, br_a)
 
-    def log(self, *args):
-        """Capture web server log messages for introspection."""
-        super(TestHttpFetch, self).log(*args)
-        if args[0].startswith("webserver"):
-            self.weblogs.append(args[0])
-
     def test_weaves_are_retrieved_once(self):
         self.build_tree(("source/", "source/file", "target/"))
         branch = Branch.initialize("source")
-        branch.add(["file"], ["id"])
-        branch.commit("added file")
+        branch.working_tree().add(["file"], ["id"])
+        branch.working_tree().commit("added file")
         print >>open("source/file", 'w'), "blah"
-        branch.commit("changed file")
+        branch.working_tree().commit("changed file")
         target = Branch.initialize("target/")
         source = Branch.open(self.get_remote_url("source/"))
-        source.weave_store.enable_cache = False
         self.assertEqual(greedy_fetch(target, source), (2, []))
-        weave_suffix = 'weaves/id.weave HTTP/1.1" 200 -'
-        self.assertEqual(1,
-            len([log for log in self.weblogs if log.endswith(weave_suffix)]))
+        # this is the path to the literal file. As format changes 
+        # occur it needs to be updated. FIXME: ask the store for the
+        # path.
+        weave_suffix = 'weaves/ce/id.weave HTTP/1.1" 200 -'
+        self.assertEqual(1,
+            len([log for log in self.server.logs if log.endswith(weave_suffix)]))
+        inventory_weave_suffix = 'inventory.weave HTTP/1.1" 200 -'
+        self.assertEqual(1,
+            len([log for log in self.server.logs if log.endswith(
+                inventory_weave_suffix)]))
+        # this r-h check test will prevent regressions, but it currently already 
+        # passes, before the patch to cache-rh is applied :[
+        revision_history_suffix = 'revision-history HTTP/1.1" 200 -'
+        self.assertEqual(1,
+            len([log for log in self.server.logs if log.endswith(
+                revision_history_suffix)]))
+        # FIXME naughty poking in there.
+        self.server.logs = []
+        # check there is nothing more to fetch
+        source = Branch.open(self.get_remote_url("source/"))
+        self.assertEqual(greedy_fetch(target, source), (0, []))
+        self.failUnless(self.server.logs[0].endswith('branch-format HTTP/1.1" 200 -'))
+        self.failUnless(self.server.logs[1].endswith('revision-history HTTP/1.1" 200 -'))
+        self.assertEqual(2, len(self.server.logs))

=== added file 'bzrlib/tests/test_gpg.py'
--- bzrlib/tests/test_gpg.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_gpg.py	2005-12-16 02:25:56 +0000
@@ -0,0 +1,72 @@
+# Copyright (C) 2005 by Canonical Ltd
+#   Authors: Robert Collins <robert.collins@canonical.com>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for signing and verifying blobs of data via gpg."""
+
+# import system imports here
+import os
+import sys
+
+import bzrlib.errors as errors
+import bzrlib.gpg as gpg
+from bzrlib.tests import TestCase, TestCaseInTempDir
+
+class FakeConfig(object):
+
+    def gpg_signing_command(self):
+        return "false"
+        
+
+class TestCommandLine(TestCase):
+
+    def test_signing_command_line(self):
+        my_gpg = gpg.GPGStrategy(FakeConfig())
+        self.assertEqual(['false',  '--clearsign'],
+                         my_gpg._command_line())
+
+    def test_checks_return_code(self):
+        # This test needs a unix like platform - one with 'false' to run.
+        # if you have one, please make this work :)
+        my_gpg = gpg.GPGStrategy(FakeConfig())
+        self.assertRaises(errors.SigningFailed, my_gpg.sign, 'content')
+
+    def test_returns_output(self):
+        # This test needs a 'cat' command or similar to work.
+        my_gpg = gpg.GPGStrategy(FakeConfig())
+        content = "some content\nwith newlines\n"
+
+        if sys.platform == 'win32':
+            # Windows doesn't come with cat, and we don't require it
+            # so lets try using python instead.
+            # But stupid windows and line-ending conversions. 
+            # It is too much work to make sys.stdout be in binary mode.
+            # http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/65443
+            my_gpg._command_line = lambda:[sys.executable, '-c',
+                    'import sys; sys.stdout.write(sys.stdin.read())']
+            new_content = content.replace('\n', '\r\n')
+
+            self.assertEqual(new_content, my_gpg.sign(content))
+        else:
+            my_gpg._command_line = lambda:['cat', '-']
+            self.assertEqual(content, my_gpg.sign(content))
+
+
+class TestDisabled(TestCase):
+
+    def test_sign(self):
+        self.assertRaises(errors.SigningFailed,
+                          gpg.DisabledGPGStrategy(None).sign, 'content')

=== renamed file 'bzrlib/selftest/testgraph.py' => 'bzrlib/tests/test_graph.py'
--- bzrlib/selftest/testgraph.py	2005-09-21 15:33:23 +0000
+++ bzrlib/tests/test_graph.py	2005-11-23 15:44:24 +0000
@@ -1,4 +1,4 @@
-from bzrlib.selftest import TestCase
+from bzrlib.tests import TestCase
 from bzrlib.graph import node_distances, nodes_by_distance
 
 class TestBase(TestCase):
@@ -33,10 +33,10 @@
         distances = node_distances(self.graph, descendants, 'A')
         nodes = nodes_by_distance(distances)
         self.assertEqual(nodes[0], 'D')
-        assert nodes[1] in ('N', 'C')
-        assert nodes[2] in ('N', 'C')
-        assert nodes[3] in ('B', 'M')
-        assert nodes[4] in ('B', 'M')
+        self.assert_(nodes[1] in ('N', 'C'))
+        self.assert_(nodes[2] in ('N', 'C'))
+        self.assert_(nodes[3] in ('B', 'M'))
+        self.assert_(nodes[4] in ('B', 'M'))
 
         #Ensure we don't shortcut through B when there's only a difference of
         # 1 in distance

=== renamed file 'bzrlib/selftest/testhashcache.py' => 'bzrlib/tests/test_hashcache.py'
--- bzrlib/selftest/testhashcache.py	2005-09-27 07:24:40 +0000
+++ bzrlib/tests/test_hashcache.py	2005-12-26 05:28:38 +0000
@@ -15,8 +15,9 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 import os
+import sys
 import time
-from bzrlib.selftest import TestCaseInTempDir
+from bzrlib.tests import TestCaseInTempDir
 
 
 
@@ -28,7 +29,7 @@
 def pause():
     if False:
         return
-    if os.name == 'nt':
+    if sys.platform in ('win32', 'cygwin'):
         time.sleep(3)
         return
     # allow it to stabilize
@@ -46,7 +47,7 @@
 
         # make a dummy bzr directory just to hold the cache
         os.mkdir('.bzr')
-        hc = HashCache('.')
+        hc = HashCache(u'.')
 
         file('foo', 'wb').write('hello')
         os.mkdir('subdir')
@@ -60,21 +61,21 @@
         # check we hit without re-reading
         self.assertEquals(hc.get_sha1('foo'),
                           'aaf4c61ddcc5e8a2dabede0f3b482cd9aea9434d')
-        ##self.assertEquals(hc.miss_count, 1)
-        ##self.assertEquals(hc.hit_count, 1)
+        self.assertEquals(hc.miss_count, 1)
+        self.assertEquals(hc.hit_count, 1)
 
         # check again without re-reading
         self.assertEquals(hc.get_sha1('foo'),
                           'aaf4c61ddcc5e8a2dabede0f3b482cd9aea9434d')
-        ##self.assertEquals(hc.miss_count, 1)
-        ##self.assertEquals(hc.hit_count, 2)
+        self.assertEquals(hc.miss_count, 1)
+        self.assertEquals(hc.hit_count, 2)
 
         # write new file and make sure it is seen
         file('foo', 'wb').write('goodbye')
         pause()
         self.assertEquals(hc.get_sha1('foo'),
                           '3c8ec4874488f6090a157b014ce3397ca8e06d4f')
-        ##self.assertEquals(hc.miss_count, 2)
+        self.assertEquals(hc.miss_count, 2)
 
         # quickly write new file of same size and make sure it is seen
         # this may rely on detection of timestamps that are too close
@@ -100,20 +101,20 @@
 
         # should now be safe to cache it if we reread them
         self.assertEquals(hc.get_sha1('foo'), sha1('g00dbye'))
-        ##self.assertEquals(len(hc._cache), 1)
+        self.assertEquals(len(hc._cache), 1)
         self.assertEquals(hc.get_sha1('foo2'), sha1('new content'))
-        ##self.assertEquals(len(hc._cache), 2)
+        self.assertEquals(len(hc._cache), 2)
 
         # write out, read back in and check that we don't need to
         # re-read any files
         hc.write()
         del hc
 
-        hc = HashCache('.')
+        hc = HashCache(u'.')
         hc.read()
 
-        ##self.assertEquals(len(hc._cache), 2)
+        self.assertEquals(len(hc._cache), 2)
         self.assertEquals(hc.get_sha1('foo'), sha1('g00dbye'))
-        ##self.assertEquals(hc.hit_count, 1)
-        ##self.assertEquals(hc.miss_count, 0)
+        self.assertEquals(hc.hit_count, 1)
+        self.assertEquals(hc.miss_count, 0)
         self.assertEquals(hc.get_sha1('foo2'), sha1('new content'))

=== added file 'bzrlib/tests/test_http.py'
--- bzrlib/tests/test_http.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_http.py	2005-11-30 15:15:17 +0000
@@ -0,0 +1,53 @@
+# (C) 2005 Canonical
+
+from bzrlib.tests import TestCase
+from bzrlib.transport.http import HttpTransport, extract_auth
+
+class FakeManager (object):
+    def __init__(self):
+        self.credentials = []
+        
+    def add_password(self, realm, host, username, password):
+        self.credentials.append([realm, host, username, password])
+
+        
+class TestHttpUrls(TestCase):
+    def test_url_parsing(self):
+        f = FakeManager()
+        url = extract_auth('http://example.com', f)
+        self.assertEquals('http://example.com', url)
+        self.assertEquals(0, len(f.credentials))
+        url = extract_auth('http://user:pass@www.bazaar-ng.org/bzr/bzr.dev', f)
+        self.assertEquals('http://www.bazaar-ng.org/bzr/bzr.dev', url)
+        self.assertEquals(1, len(f.credentials))
+        self.assertEquals([None, 'www.bazaar-ng.org', 'user', 'pass'], f.credentials[0])
+        
+    def test_abs_url(self):
+        """Construction of absolute http URLs"""
+        t = HttpTransport('http://bazaar-ng.org/bzr/bzr.dev/')
+        eq = self.assertEqualDiff
+        eq(t.abspath('.'),
+           'http://bazaar-ng.org/bzr/bzr.dev')
+        eq(t.abspath('foo/bar'), 
+           'http://bazaar-ng.org/bzr/bzr.dev/foo/bar')
+        eq(t.abspath('.bzr'),
+           'http://bazaar-ng.org/bzr/bzr.dev/.bzr')
+        eq(t.abspath('.bzr/1//2/./3'),
+           'http://bazaar-ng.org/bzr/bzr.dev/.bzr/1/2/3')
+
+    def test_invalid_http_urls(self):
+        """Trap invalid construction of urls"""
+        t = HttpTransport('http://bazaar-ng.org/bzr/bzr.dev/')
+        self.assertRaises(ValueError,
+            t.abspath,
+            '.bzr/')
+        self.assertRaises(ValueError,
+            t.abspath,
+            '/.bzr')
+
+    def test_http_root_urls(self):
+        """Construction of URLs from server root"""
+        t = HttpTransport('http://bzr.ozlabs.org/')
+        eq = self.assertEqualDiff
+        eq(t.abspath('.bzr/tree-version'),
+           'http://bzr.ozlabs.org/.bzr/tree-version')

=== added file 'bzrlib/tests/test_identitymap.py'
--- bzrlib/tests/test_identitymap.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_identitymap.py	2005-11-23 15:44:24 +0000
@@ -0,0 +1,111 @@
+# Copyright (C) 2005 by Canonical Ltd
+#   Authors: Robert Collins <robert.collins@canonical.com>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for the IdentityMap class."""
+
+# import system imports here
+
+#import bzrlib specific imports here
+import bzrlib.errors as errors
+from bzrlib.tests import TestCase
+import bzrlib.identitymap as identitymap
+
+
+class TestIdentityMap(TestCase):
+
+    def test_symbols(self):
+        from bzrlib.identitymap import IdentityMap
+
+    def test_construct(self):
+        identitymap.IdentityMap()
+
+    def test_add_weave(self):
+        map = identitymap.IdentityMap()
+        weave = "foo"
+        map.add_weave("id", weave)
+        self.assertEqual(weave, map.find_weave("id"))
+
+    def test_double_add_weave(self):
+        map = identitymap.IdentityMap()
+        weave = "foo"
+        map.add_weave("id", weave)
+        self.assertRaises(errors.BzrError, map.add_weave, "id", weave)
+        self.assertEqual(weave, map.find_weave("id"))
+ 
+    def test_remove_object(self):
+        map = identitymap.IdentityMap()
+        weave = "foo"
+        map.add_weave("id", weave)
+        map.remove_object(weave)
+        map.add_weave("id", weave)
+        rev_history = [1]
+        map.add_revision_history(rev_history)
+        map.remove_object(rev_history)
+
+    def test_add_revision_history(self):
+        map = identitymap.IdentityMap()
+        rev_history = [1,2,3]
+        map.add_revision_history(rev_history)
+        self.assertEqual(rev_history, map.find_revision_history())
+
+    def test_double_add_revision_history(self):
+        map = identitymap.IdentityMap()
+        revision_history = [1]
+        map.add_revision_history(revision_history)
+        self.assertRaises(errors.BzrError,
+                          map.add_revision_history,
+                          revision_history)
+        self.assertEqual(revision_history, map.find_revision_history())
+
+ 
+class TestNullIdentityMap(TestCase):
+
+    def test_symbols(self):
+        from bzrlib.identitymap import NullIdentityMap
+
+    def test_construct(self):
+        identitymap.NullIdentityMap()
+
+    def test_add_weave(self):
+        map = identitymap.NullIdentityMap()
+        weave = "foo"
+        map.add_weave("id", weave)
+        self.assertEqual(None, map.find_weave("id"))
+
+    def test_double_add_weave(self):
+        map = identitymap.NullIdentityMap()
+        weave = "foo"
+        map.add_weave("id", weave)
+        map.add_weave("id", weave)
+        self.assertEqual(None, map.find_weave("id"))
+        
+    def test_null_identity_map_has_no_remove(self):
+        map = identitymap.NullIdentityMap()
+        self.assertEqual(None, getattr(map, 'remove_object', None))
+
+    def test_add_revision_history(self):
+        map = identitymap.NullIdentityMap()
+        rev_history = [1,2,3]
+        map.add_revision_history(rev_history)
+        self.assertEqual(None, map.find_revision_history())
+
+    def test_double_add_revision_history(self):
+        map = identitymap.NullIdentityMap()
+        revision_history = [1]
+        map.add_revision_history(revision_history)
+        map.add_revision_history(revision_history)
+        self.assertEqual(None, map.find_revision_history())

=== renamed file 'bzrlib/selftest/testinv.py' => 'bzrlib/tests/test_inv.py'
--- bzrlib/selftest/testinv.py	2005-10-06 02:51:02 +0000
+++ bzrlib/tests/test_inv.py	2005-12-08 18:32:43 +0000
@@ -23,8 +23,8 @@
 from bzrlib.diff import internal_diff
 from bzrlib.inventory import Inventory, ROOT_ID
 import bzrlib.inventory as inventory
-from bzrlib.osutils import has_symlinks, rename
-from bzrlib.selftest import TestCase, TestCaseInTempDir
+from bzrlib.osutils import has_symlinks, rename, pathjoin
+from bzrlib.tests import TestCase, TestCaseInTempDir
 
 
 class TestInventory(TestCase):
@@ -32,7 +32,7 @@
     def test_is_within(self):
         from bzrlib.osutils import is_inside_any
 
-        SRC_FOO_C = os.path.join('src', 'foo.c')
+        SRC_FOO_C = pathjoin('src', 'foo.c')
         for dirs, fn in [(['src', 'doc'], SRC_FOO_C),
                          (['src'], SRC_FOO_C),
                          (['src'], 'src'),
@@ -137,13 +137,14 @@
 
     def setUp(self):
         super(TestEntryDiffing, self).setUp()
-        self.branch = Branch.initialize('.')
+        self.branch = Branch.initialize(u'.')
+        self.wt = self.branch.working_tree()
         print >> open('file', 'wb'), 'foo'
-        self.branch.add(['file'], ['fileid'])
+        self.branch.working_tree().add(['file'], ['fileid'])
         if has_symlinks():
             os.symlink('target1', 'symlink')
-            self.branch.add(['symlink'], ['linkid'])
-        self.branch.commit('message_1', rev_id = '1')
+            self.branch.working_tree().add(['symlink'], ['linkid'])
+        self.wt.commit('message_1', rev_id = '1')
         print >> open('file', 'wb'), 'bar'
         if has_symlinks():
             os.unlink('symlink')
@@ -152,7 +153,7 @@
         self.inv_1 = self.branch.get_inventory('1')
         self.file_1 = self.inv_1['fileid']
         self.tree_2 = self.branch.working_tree()
-        self.inv_2 = self.branch.inventory
+        self.inv_2 = self.tree_2.read_working_inventory()
         self.file_2 = self.inv_2['fileid']
         if has_symlinks():
             self.link_1 = self.inv_1['linkid']
@@ -164,8 +165,8 @@
                           "old_label", self.tree_1,
                           "/dev/null", None, None,
                           output)
-        self.assertEqual(output.getvalue(), "--- old_label\n"
-                                            "+++ /dev/null\n"
+        self.assertEqual(output.getvalue(), "--- old_label\t\n"
+                                            "+++ /dev/null\t\n"
                                             "@@ -1,1 +0,0 @@\n"
                                             "-foo\n"
                                             "\n")
@@ -176,8 +177,8 @@
                           "new_label", self.tree_1,
                           "/dev/null", None, None,
                           output, reverse=True)
-        self.assertEqual(output.getvalue(), "--- /dev/null\n"
-                                            "+++ new_label\n"
+        self.assertEqual(output.getvalue(), "--- /dev/null\t\n"
+                                            "+++ new_label\t\n"
                                             "@@ -0,0 +1,1 @@\n"
                                             "+foo\n"
                                             "\n")
@@ -188,14 +189,16 @@
                           "/dev/null", self.tree_1, 
                           "new_label", self.file_2, self.tree_2,
                           output)
-        self.assertEqual(output.getvalue(), "--- /dev/null\n"
-                                            "+++ new_label\n"
+        self.assertEqual(output.getvalue(), "--- /dev/null\t\n"
+                                            "+++ new_label\t\n"
                                             "@@ -1,1 +1,1 @@\n"
                                             "-foo\n"
                                             "+bar\n"
                                             "\n")
         
     def test_link_diff_deleted(self):
+        if not has_symlinks():
+            return
         output = StringIO()
         self.link_1.diff(internal_diff, 
                           "old_label", self.tree_1,
@@ -205,6 +208,8 @@
                          "=== target was 'target1'\n")
 
     def test_link_diff_added(self):
+        if not has_symlinks():
+            return
         output = StringIO()
         self.link_1.diff(internal_diff, 
                           "new_label", self.tree_1,
@@ -214,6 +219,8 @@
                          "=== target is 'target1'\n")
 
     def test_link_diff_changed(self):
+        if not has_symlinks():
+            return
         output = StringIO()
         self.link_1.diff(internal_diff, 
                           "/dev/null", self.tree_1, 
@@ -233,12 +240,14 @@
         # to change, and then test merge patterns
         # with fake parent entries.
         super(TestSnapshot, self).setUp()
-        self.branch = Branch.initialize('.')
-        self.build_tree(['subdir/', 'subdir/file'])
-        self.branch.add(['subdir', 'subdir/file'], ['dirid', 'fileid'])
+        self.branch = Branch.initialize(u'.')
+        self.build_tree(['subdir/', 'subdir/file'], line_endings='binary')
+        self.branch.working_tree().add(['subdir', 'subdir/file'],
+                                       ['dirid', 'fileid'])
         if has_symlinks():
             pass
-        self.branch.commit('message_1', rev_id = '1')
+        self.wt = self.branch.working_tree()
+        self.wt.commit('message_1', rev_id = '1')
         self.tree_1 = self.branch.revision_tree('1')
         self.inv_1 = self.branch.get_inventory('1')
         self.file_1 = self.inv_1['fileid']
@@ -249,24 +258,28 @@
         # This tests that a simple commit with no parents makes a new
         # revision value in the inventory entry
         self.file_active.snapshot('2', 'subdir/file', {}, self.work_tree, 
-                                  self.branch.weave_store)
+                                  self.branch.weave_store,
+                                  self.branch.get_transaction())
         # expected outcome - file_1 has a revision id of '2', and we can get
         # its text of 'file contents' out of the weave.
         self.assertEqual(self.file_1.revision, '1')
         self.assertEqual(self.file_active.revision, '2')
         # this should be a separate test probably, but lets check it once..
-        lines = self.branch.weave_store.get_lines('fileid','2')
+        lines = self.branch.weave_store.get_lines('fileid','2',
+            self.branch.get_transaction())
         self.assertEqual(lines, ['contents of subdir/file\n'])
 
     def test_snapshot_unchanged(self):
         #This tests that a simple commit does not make a new entry for
         # an unchanged inventory entry
         self.file_active.snapshot('2', 'subdir/file', {'1':self.file_1},
-                                  self.work_tree, self.branch.weave_store)
+                                  self.work_tree, self.branch.weave_store,
+                                  self.branch.get_transaction())
         self.assertEqual(self.file_1.revision, '1')
         self.assertEqual(self.file_active.revision, '1')
         self.assertRaises(errors.WeaveError,
-                          self.branch.weave_store.get_lines, 'fileid', '2')
+                          self.branch.weave_store.get_lines, 'fileid', '2',
+                          self.branch.get_transaction())
 
     def test_snapshot_merge_identical_different_revid(self):
         # This tests that a commit with two identical parents, one of which has
@@ -280,10 +293,12 @@
         self.assertEqual(self.file_1, other_ie)
         other_ie.revision = 'other'
         self.assertNotEqual(self.file_1, other_ie)
-        self.branch.weave_store.add_identical_text('fileid', '1', 'other', ['1'])
+        self.branch.weave_store.add_identical_text('fileid', '1', 'other', ['1'],
+            self.branch.get_transaction())
         self.file_active.snapshot('2', 'subdir/file', 
                                   {'1':self.file_1, 'other':other_ie},
-                                  self.work_tree, self.branch.weave_store)
+                                  self.work_tree, self.branch.weave_store,
+                                  self.branch.get_transaction())
         self.assertEqual(self.file_active.revision, '2')
 
     def test_snapshot_changed(self):
@@ -293,7 +308,8 @@
         rename('subdir/file', 'subdir/newname')
         self.file_active.snapshot('2', 'subdir/newname', {'1':self.file_1}, 
                                   self.work_tree, 
-                                  self.branch.weave_store)
+                                  self.branch.weave_store,
+                                  self.branch.get_transaction())
         # expected outcome - file_1 has a revision id of '2'
         self.assertEqual(self.file_active.revision, '2')
 
@@ -311,21 +327,23 @@
         #   a descendent of the other. (B, D)
         super(TestPreviousHeads, self).setUp()
         self.build_tree(['file'])
-        self.branch = Branch.initialize('.')
-        self.branch.commit('new branch', allow_pointless=True, rev_id='A')
+        self.branch = Branch.initialize(u'.')
+        self.wt = self.branch.working_tree()
+        self.wt.commit('new branch', allow_pointless=True, rev_id='A')
         self.inv_A = self.branch.get_inventory('A')
-        self.branch.add(['file'], ['fileid'])
-        self.branch.commit('add file', rev_id='B')
+        self.branch.working_tree().add(['file'], ['fileid'])
+        self.wt.commit('add file', rev_id='B')
         self.inv_B = self.branch.get_inventory('B')
         self.branch.put_controlfile('revision-history', 'A\n')
         self.assertEqual(self.branch.revision_history(), ['A'])
-        self.branch.commit('another add of file', rev_id='C')
+        self.wt.commit('another add of file', rev_id='C')
         self.inv_C = self.branch.get_inventory('C')
-        self.branch.add_pending_merge('B')
-        self.branch.commit('merge in B', rev_id='D')
+        self.wt.add_pending_merge('B')
+        self.wt.commit('merge in B', rev_id='D')
         self.inv_D = self.branch.get_inventory('D')
-        self.file_active = self.branch.working_tree().inventory['fileid']
-        self.weave = self.branch.weave_store.get_weave('fileid')
+        self.file_active = self.wt.inventory['fileid']
+        self.weave = self.branch.weave_store.get_weave('fileid',
+            self.branch.get_transaction())
         
     def get_previous_heads(self, inventories):
         return self.file_active.find_previous_heads(inventories, self.weave)
@@ -356,3 +374,119 @@
                          self.get_previous_heads([self.inv_D, self.inv_B]))
 
     # TODO: test two inventories with the same file revision 
+
+
+class TestExecutable(TestCaseInTempDir):
+
+    def test_stays_executable(self):
+        basic_inv = """<inventory format="5">
+<file file_id="a-20051208024829-849e76f7968d7a86" name="a" executable="yes" />
+<file file_id="b-20051208024829-849e76f7968d7a86" name="b" />
+</inventory>
+"""
+        os.mkdir('b1')
+        b = Branch.initialize('b1')
+        open('b1/a', 'wb').write('a test\n')
+        open('b1/b', 'wb').write('b test\n')
+        os.chmod('b1/a', 0755)
+        os.chmod('b1/b', 0644)
+        # Manually writing the inventory, to ensure that
+        # the executable="yes" entry is set for 'a' and not for 'b'
+        open('b1/.bzr/inventory', 'wb').write(basic_inv)
+
+        a_id = "a-20051208024829-849e76f7968d7a86"
+        b_id = "b-20051208024829-849e76f7968d7a86"
+        t = b.working_tree()
+        self.assertEqual(['a', 'b'], [cn for cn,ie in t.inventory.iter_entries()])
+
+        self.failUnless(t.is_executable(a_id), "'a' lost the execute bit")
+        self.failIf(t.is_executable(b_id), "'b' gained an execute bit")
+
+        t.commit('adding a,b', rev_id='r1')
+
+        rev_tree = b.revision_tree('r1')
+        self.failUnless(rev_tree.is_executable(a_id), "'a' lost the execute bit")
+        self.failIf(rev_tree.is_executable(b_id), "'b' gained an execute bit")
+
+        self.failUnless(rev_tree.inventory[a_id].executable)
+        self.failIf(rev_tree.inventory[b_id].executable)
+
+        # Make sure the entries are gone
+        os.remove('b1/a')
+        os.remove('b1/b')
+        self.failIf(t.has_id(a_id))
+        self.failIf(t.has_filename('a'))
+        self.failIf(t.has_id(b_id))
+        self.failIf(t.has_filename('b'))
+
+        # Make sure that revert is able to bring them back,
+        # and sets 'a' back to being executable
+
+        t.revert(['b1/a', 'b1/b'], rev_tree, backups=False)
+        self.assertEqual(['a', 'b'], [cn for cn,ie in t.inventory.iter_entries()])
+
+        self.failUnless(t.is_executable(a_id), "'a' lost the execute bit")
+        self.failIf(t.is_executable(b_id), "'b' gained an execute bit")
+
+        # Now remove them again, and make sure that after a
+        # commit, they are still marked correctly
+        os.remove('b1/a')
+        os.remove('b1/b')
+        t.commit('removed', rev_id='r2')
+
+        self.assertEqual([], [cn for cn,ie in t.inventory.iter_entries()])
+        self.failIf(t.has_id(a_id))
+        self.failIf(t.has_filename('a'))
+        self.failIf(t.has_id(b_id))
+        self.failIf(t.has_filename('b'))
+
+        # Now revert back to the previous commit
+        t.revert([], rev_tree, backups=False)
+        # TODO: FIXME: For some reason, after revert, the tree does not 
+        # regenerate its working inventory, so we have to manually delete
+        # the working tree, and create a new one
+        # This seems to happen any time you do a merge operation on the
+        # working tree
+        del t
+        t = b.working_tree()
+
+        self.assertEqual(['a', 'b'], [cn for cn,ie in t.inventory.iter_entries()])
+
+        self.failUnless(t.is_executable(a_id), "'a' lost the execute bit")
+        self.failIf(t.is_executable(b_id), "'b' gained an execute bit")
+
+        # Now make sure that 'bzr branch' also preserves the
+        # executable bit
+        # TODO: Maybe this should be a blackbox test
+        from bzrlib.clone import copy_branch
+        copy_branch(b, 'b2', revision='r1')
+        b2 = Branch.open('b2')
+        self.assertEquals('r1', b2.last_revision())
+        t2 = b2.working_tree()
+
+        self.assertEqual(['a', 'b'], [cn for cn,ie in t2.inventory.iter_entries()])
+        self.failUnless(t2.is_executable(a_id), "'a' lost the execute bit")
+        self.failIf(t2.is_executable(b_id), "'b' gained an execute bit")
+
+        # Make sure pull will delete the files
+        t2.pull(b)
+        self.assertEquals('r2', b2.last_revision())
+        # FIXME: Same thing here, t2 needs to be recreated
+        del t2
+        t2 = b2.working_tree()
+        self.assertEqual([], [cn for cn,ie in t2.inventory.iter_entries()])
+
+        # Now commit the changes on the first branch
+        # so that the second branch can pull the changes
+        # and make sure that the executable bit has been copied
+        t.commit('resurrected', rev_id='r3')
+
+        t2.pull(b)
+        # FIXME: And here
+        del t2
+        t2 = b2.working_tree()
+        self.assertEquals('r3', b2.last_revision())
+        self.assertEqual(['a', 'b'], [cn for cn,ie in t2.inventory.iter_entries()])
+
+        self.failUnless(t2.is_executable(a_id), "'a' lost the execute bit")
+        self.failIf(t2.is_executable(b_id), "'b' gained an execute bit")

=== renamed file 'bzrlib/selftest/testlog.py' => 'bzrlib/tests/test_log.py'
--- bzrlib/selftest/testlog.py	2005-10-02 21:51:29 +0000
+++ bzrlib/tests/test_log.py	2005-12-01 17:36:29 +0000
@@ -17,8 +17,8 @@
 import os
 from cStringIO import StringIO
 
-from bzrlib.selftest import BzrTestBase, TestCaseInTempDir
-from bzrlib.log import LogFormatter, show_log, LongLogFormatter
+from bzrlib.tests import BzrTestBase, TestCaseInTempDir
+from bzrlib.log import LogFormatter, show_log, LongLogFormatter, ShortLogFormatter
 from bzrlib.branch import Branch
 from bzrlib.errors import InvalidRevisionNumber
 
@@ -57,18 +57,18 @@
             expected = kw.get(n, [])
 
             # tests are written with unix paths; fix them up for windows
-            if os.sep != '/':
-                expected = [x.replace('/', os.sep) for x in expected]
+            #if os.sep != '/':
+            #    expected = [x.replace('/', os.sep) for x in expected]
 
             # strip out only the path components
             got = [x[0] for x in getattr(delta, n)]
             self.assertEquals(expected, got)
 
     def test_cur_revno(self):
-        b = Branch('.', init=True)
+        b = Branch(u'.', init=True)
 
         lf = LogCatcher()
-        b.commit('empty commit')
+        b.working_tree().commit('empty commit')
         show_log(b, lf, verbose=True, start_revision=1, end_revision=1)
         self.assertRaises(InvalidRevisionNumber, show_log, b, lf,
                           start_revision=2, end_revision=1) 
@@ -84,10 +84,10 @@
                           start_revision=1, end_revision=-1) 
 
     def test_cur_revno(self):
-        b = Branch.initialize('.')
+        b = Branch.initialize(u'.')
 
         lf = LogCatcher()
-        b.commit('empty commit')
+        b.working_tree().commit('empty commit')
         show_log(b, lf, verbose=True, start_revision=1, end_revision=1)
         self.assertRaises(InvalidRevisionNumber, show_log, b, lf,
                           start_revision=2, end_revision=1) 
@@ -105,15 +105,14 @@
     def test_simple_log(self):
         eq = self.assertEquals
         
-        b = Branch.initialize('.')
+        b = Branch.initialize(u'.')
 
         lf = LogCatcher()
         show_log(b, lf)
         # no entries yet
         eq(lf.logs, [])
 
-
-        b.commit('empty commit')
+        b.working_tree().commit('empty commit')
         lf = LogCatcher()
         show_log(b, lf, verbose=True)
         eq(len(lf.logs), 1)
@@ -123,10 +122,9 @@
         self.log('log delta: %r' % d)
         self.checkDelta(d)
 
-
         self.build_tree(['hello'])
-        b.add('hello')
-        b.commit('add one file')
+        b.working_tree().add('hello')
+        b.working_tree().commit('add one file')
 
         lf = StringIO()
         # log using regular thing
@@ -153,10 +151,125 @@
         
         # commit a log message with control characters
         msg = "All 8-bit chars: " +  ''.join([unichr(x) for x in range(256)])
-        b.commit(msg)
+        self.log("original commit message: %r", msg)
+        b.working_tree().commit(msg)
         lf = LogCatcher()
         show_log(b, lf, verbose=True)
         committed_msg = lf.logs[0].rev.message
         self.log("escaped commit message: %r", committed_msg)
         self.assert_(msg != committed_msg)
         self.assert_(len(committed_msg) > len(msg))
+
+        # Check that log message with only XML-valid characters isn't
+        # escaped.  As ElementTree apparently does some kind of
+        # newline conversion, neither LF (\x0A) nor CR (\x0D) are
+        # included in the test commit message, even though they are
+        # valid XML 1.0 characters.
+        msg = "\x09" + ''.join([unichr(x) for x in range(0x20, 256)])
+        self.log("original commit message: %r", msg)
+        b.working_tree().commit(msg)
+        lf = LogCatcher()
+        show_log(b, lf, verbose=True)
+        committed_msg = lf.logs[0].rev.message
+        self.log("escaped commit message: %r", committed_msg)
+        self.assert_(msg == committed_msg)
+
+    def test_trailing_newlines(self):
+        b = Branch.initialize(u'.')
+        b.nick='test'
+        wt = b.working_tree()
+        open('a', 'wb').write('hello moto\n')
+        wt.add('a')
+        wt.commit('simple log message', rev_id='a1'
+                , timestamp=1132586655.459960938, timezone=-6*3600
+                , committer='Joe Foo <joe@foo.com>')
+        open('b', 'wb').write('goodbye\n')
+        wt.add('b')
+        wt.commit('multiline\nlog\nmessage\n', rev_id='a2'
+                , timestamp=1132586842.411175966, timezone=-6*3600
+                , committer='Joe Foo <joe@foo.com>')
+
+        open('c', 'wb').write('just another manic monday\n')
+        wt.add('c')
+        wt.commit('single line with trailing newline\n', rev_id='a3'
+                , timestamp=1132587176.835228920, timezone=-6*3600
+                , committer = 'Joe Foo <joe@foo.com>')
+
+        sio = StringIO()
+        lf = ShortLogFormatter(to_file=sio)
+        show_log(b, lf)
+        self.assertEquals(sio.getvalue(), """\
+    3 Joe Foo\t2005-11-21
+      single line with trailing newline
+
+    2 Joe Foo\t2005-11-21
+      multiline
+      log
+      message
+
+    1 Joe Foo\t2005-11-21
+      simple log message
+
+""")
+
+        sio = StringIO()
+        lf = LongLogFormatter(to_file=sio)
+        show_log(b, lf)
+        self.assertEquals(sio.getvalue(), """\
+------------------------------------------------------------
+revno: 3
+committer: Joe Foo <joe@foo.com>
+branch nick: test
+timestamp: Mon 2005-11-21 09:32:56 -0600
+message:
+  single line with trailing newline
+------------------------------------------------------------
+revno: 2
+committer: Joe Foo <joe@foo.com>
+branch nick: test
+timestamp: Mon 2005-11-21 09:27:22 -0600
+message:
+  multiline
+  log
+  message
+------------------------------------------------------------
+revno: 1
+committer: Joe Foo <joe@foo.com>
+branch nick: test
+timestamp: Mon 2005-11-21 09:24:15 -0600
+message:
+  simple log message
+""")
+        
+    def test_verbose_log(self):
+        """Verbose log includes changed files
+        
+        bug #4676
+        """
+        b = Branch.initialize(u'.')
+        self.build_tree(['a'])
+        wt = b.working_tree()
+        wt.add('a')
+        # XXX: why does a longer nick show up?
+        b.nick = 'test_verbose_log'
+        wt.commit(message='add a', 
+                  timestamp=1132711707, 
+                  timezone=36000,
+                  committer='Lorem Ipsum <test@example.com>')
+        logfile = file('out.tmp', 'w+')
+        formatter = LongLogFormatter(to_file=logfile)
+        show_log(b, formatter, verbose=True)
+        logfile.flush()
+        logfile.seek(0)
+        log_contents = logfile.read()
+        self.assertEqualDiff(log_contents, '''\
+------------------------------------------------------------
+revno: 1
+committer: Lorem Ipsum <test@example.com>
+branch nick: test_verbose_log
+timestamp: Wed 2005-11-23 12:08:27 +1000
+message:
+  add a
+added:
+  a
+''')

=== renamed file 'bzrlib/selftest/testmerge.py' => 'bzrlib/tests/test_merge.py'
--- bzrlib/selftest/testmerge.py	2005-09-29 20:34:25 +0000
+++ bzrlib/tests/test_merge.py	2005-12-01 17:11:25 +0000
@@ -2,21 +2,22 @@
 
 from bzrlib.branch import Branch
 from bzrlib.commit import commit
-from bzrlib.selftest import TestCaseInTempDir
-from bzrlib.merge import merge
-from bzrlib.errors import UnrelatedBranches, NoCommits
+from bzrlib.tests import TestCaseInTempDir
+from bzrlib.merge import merge, transform_tree
+from bzrlib.errors import UnrelatedBranches, NoCommits, BzrCommandError
 from bzrlib.revision import common_ancestor
 from bzrlib.fetch import fetch
+from bzrlib.osutils import pathjoin
 
 
 class TestMerge(TestCaseInTempDir):
     """Test appending more than one revision"""
     def test_pending(self):
-        br = Branch.initialize(".")
+        br = Branch.initialize(u".")
         commit(br, "lala!")
-        self.assertEquals(len(br.pending_merges()), 0)
-        merge(['.', -1], [None, None])
-        self.assertEquals(len(br.pending_merges()), 0)
+        self.assertEquals(len(br.working_tree().pending_merges()), 0)
+        merge([u'.', -1], [None, None])
+        self.assertEquals(len(br.working_tree().pending_merges()), 0)
 
     def test_nocommits(self):
         self.test_pending()
@@ -36,12 +37,14 @@
     def test_pending_with_null(self):
         """When base is forced to revno 0, pending_merges is set"""
         br2 = self.test_unrelated()
-        br1 = Branch.open('.')
+        br1 = Branch.open(u'.')
         fetch(from_branch=br2, to_branch=br1)
         # merge all of branch 2 into branch 1 even though they 
         # are not related.
-        merge(['branch2', -1], ['branch2', 0])
-        self.assertEquals(len(br1.pending_merges()), 1)
+        self.assertRaises(BzrCommandError, merge, ['branch2', -1], 
+                          ['branch2', 0], reprocess=True, show_base=True)
+        merge(['branch2', -1], ['branch2', 0], reprocess=True)
+        self.assertEquals(len(br1.working_tree().pending_merges()), 1)
         return (br1, br2)
 
     def test_two_roots(self):
@@ -50,3 +53,29 @@
         commit(br1, "blah")
         last = br1.last_revision()
         self.assertEquals(common_ancestor(last, last, br1), last)
+
+    def test_create_rename(self):
+        """Rename an inventory entry while creating the file"""
+        b = Branch.initialize(u'.')
+        file('name1', 'wb').write('Hello')
+        tree = b.working_tree()
+        tree.add('name1')
+        tree.commit(message="hello")
+        tree.rename_one('name1', 'name2')
+        os.unlink('name2')
+        transform_tree(tree, b.basis_tree())
+
+    def test_layered_rename(self):
+        """Rename both child and parent at same time"""
+        b = Branch.initialize(u'.')
+        tree = b.working_tree()
+        os.mkdir('dirname1')
+        tree.add('dirname1')
+        filename = pathjoin('dirname1', 'name1')
+        file(filename, 'wb').write('Hello')
+        tree.add(filename)
+        tree.commit(message="hello")
+        filename2 = pathjoin('dirname1', 'name2')
+        tree.rename_one(filename, filename2)
+        tree.rename_one('dirname1', 'dirname2')
+        transform_tree(tree, b.basis_tree())

=== renamed file 'bzrlib/selftest/testmerge3.py' => 'bzrlib/tests/test_merge3.py'
--- bzrlib/selftest/testmerge3.py	2005-09-04 02:59:56 +0000
+++ bzrlib/tests/test_merge3.py	2005-11-23 15:44:24 +0000
@@ -15,8 +15,9 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 
-from bzrlib.selftest import TestCaseInTempDir, TestCase
+from bzrlib.tests import TestCaseInTempDir, TestCase
 from bzrlib.merge3 import Merge3
+from bzrlib.errors import CantReprocessAndShowBase
 
 def split_lines(t):
     from cStringIO import StringIO
@@ -300,3 +301,26 @@
         self.log('merge result:')
         self.log(''.join(ml))
         self.assertEquals(ml, MERGED_RESULT)
+
+    def test_minimal_conflicts(self):
+        """Reprocessing"""
+        base_text = ("a\n" * 20).splitlines(True)
+        this_text = ("a\n"*10+"b\n" * 10).splitlines(True)
+        other_text = ("a\n"*10+"c\n"+"b\n" * 8 + "c\n").splitlines(True)
+        m3 = Merge3(base_text, other_text, this_text)
+        m_lines = m3.merge_lines('OTHER', 'THIS', reprocess=True)
+        merged_text = "".join(list(m_lines))
+        optimal_text = "a\n" * 10 + "<<<<<<< OTHER\nc\n=======\n>>>>>>> THIS"\
+            + "\n" + 8* "b\n" + "<<<<<<< OTHER\nc\n=======\nb\nb\n>>>>>>>"\
+            + " THIS\n"
+        self.assertEqualDiff(merged_text, optimal_text)
+
+    def test_reprocess_and_base(self):
+        """Reprocessing and showing base breaks correctly"""
+        base_text = ("a\n" * 20).splitlines(True)
+        this_text = ("a\n"*10+"b\n" * 10).splitlines(True)
+        other_text = ("a\n"*10+"c\n"+"b\n" * 8 + "c\n").splitlines(True)
+        m3 = Merge3(base_text, other_text, this_text)
+        m_lines = m3.merge_lines('OTHER', 'THIS', reprocess=True, 
+                                 base_marker='|||||||')
+        self.assertRaises(CantReprocessAndShowBase, list, m_lines)

=== modified file 'bzrlib/tests/test_merge_core.py'
--- bzrlib/selftest/test_merge_core.py	2005-10-03 15:19:25 +0000
+++ bzrlib/tests/test_merge_core.py	2005-12-22 23:32:43 +0000
@@ -1,18 +1,23 @@
 import os
 import shutil
-import tempfile
-import unittest
+import stat
+import sys
 
-from bzrlib.selftest import TestCaseInTempDir, TestCase
+from bzrlib.tests import TestCaseInTempDir, TestCase
 from bzrlib.branch import ScratchBranch, Branch
-from bzrlib.errors import NotBranchError, NotVersionedError
+from bzrlib.errors import (NotBranchError, NotVersionedError,
+                           WorkingTreeNotRevision, BzrCommandError)
 from bzrlib.inventory import RootEntry
 import bzrlib.inventory as inventory
-from bzrlib.osutils import file_kind, rename
+from bzrlib.osutils import file_kind, rename, sha_file, pathjoin, mkdtemp
 from bzrlib import changeset
 from bzrlib.merge_core import (ApplyMerge3, make_merge_changeset,
-                                BackupBeforeChange, PermissionsMerge)
-from bzrlib.changeset import Inventory, apply_changeset, invert_dict
+                               BackupBeforeChange, ExecFlagMerge, WeaveMerge)
+from bzrlib.changeset import Inventory, apply_changeset, invert_dict, \
+    get_contents, ReplaceContents, ChangeExecFlag
+from bzrlib.clone import copy_branch
+from bzrlib.merge import merge
+from bzrlib.workingtree import WorkingTree
 
 
 class FalseTree(object):
@@ -27,14 +32,14 @@
         return entry
         
     def make_inventory_entry(self, file_id):
-        path = self._realtree.inventory.get(file_id)
+        path = self._realtree.inventory_dict.get(file_id)
         if path is None:
             return None
         if path == "":
             return RootEntry(file_id)
         dir, name = os.path.split(path)
         kind = file_kind(self._realtree.abs_path(path))
-        for parent_id, path in self._realtree.inventory.iteritems():
+        for parent_id, path in self._realtree.inventory_dict.iteritems():
             if path == dir:
                 break
         if path != dir:
@@ -53,11 +58,11 @@
     def __init__(self, dir):
         self.dir = dir;
         os.mkdir(dir)
-        self.inventory = {'0': ""}
-        self.tree = FalseTree(self)
+        self.inventory_dict = {'0': ""}
+        self.inventory = FalseTree(self)
     
     def child_path(self, parent, name):
-        return os.path.join(self.inventory[parent], name)
+        return pathjoin(self.inventory_dict[parent], name)
 
     def add_file(self, id, parent, name, contents, mode):
         path = self.child_path(parent, name)
@@ -65,11 +70,18 @@
         assert not os.path.exists(full_path)
         file(full_path, "wb").write(contents)
         os.chmod(self.abs_path(path), mode)
-        self.inventory[id] = path
+        self.inventory_dict[id] = path
+
+    def add_symlink(self, id, parent, name, target):
+        path = self.child_path(parent, name)
+        full_path = self.abs_path(path)
+        assert not os.path.exists(full_path)
+        os.symlink(target, full_path)
+        self.inventory_dict[id] = path
 
     def remove_file(self, id):
         os.unlink(self.full_path(id))
-        del self.inventory[id]
+        del self.inventory_dict[id]
 
     def add_dir(self, id, parent, name, mode):
         path = self.child_path(parent, name)
@@ -77,14 +89,14 @@
         assert not os.path.exists(full_path)
         os.mkdir(self.abs_path(path))
         os.chmod(self.abs_path(path), mode)
-        self.inventory[id] = path
+        self.inventory_dict[id] = path
 
     def abs_path(self, path):
-        return os.path.join(self.dir, path)
+        return pathjoin(self.dir, path)
 
     def full_path(self, id):
         try:
-            tree_path = self.inventory[id]
+            tree_path = self.inventory_dict[id]
         except KeyError:
             return None
         return self.abs_path(tree_path)
@@ -93,39 +105,56 @@
         return self.full_path(id)
 
     def __contains__(self, file_id):
-        return file_id in self.inventory
+        return file_id in self.inventory_dict
 
     def has_or_had_id(self, file_id):
         return file_id in self
 
     def get_file(self, file_id):
-        path = self.readonly_path(file_id)
+        path = self.full_path(file_id)
         return file(path, "rb")
 
     def id2path(self, file_id):
-        return self.inventory[file_id]
+        return self.inventory_dict[file_id]
+
+    def id2abspath(self, id):
+        return self.full_path(id)
 
     def change_path(self, id, path):
-        old_path = os.path.join(self.dir, self.inventory[id])
+        old_path = pathjoin(self.dir, self.inventory_dict[id])
         rename(old_path, self.abs_path(path))
-        self.inventory[id] = path
+        self.inventory_dict[id] = path
+
+    def is_executable(self, file_id):
+        mode = os.lstat(self.full_path(file_id)).st_mode
+        return bool(stat.S_ISREG(mode) and stat.S_IEXEC & mode)
+
+    def kind(self, file_id):
+        return file_kind(self.full_path(file_id))
+
+    def get_symlink_target(self, file_id):
+        return os.readlink(self.full_path(file_id))
+
+    def get_file_sha1(self, file_id):
+        return sha_file(file(self.full_path(file_id), "rb"))
 
 
 class MergeBuilder(object):
     def __init__(self):
-        self.dir = tempfile.mkdtemp(prefix="BaZing")
-        self.base = MergeTree(os.path.join(self.dir, "base"))
-        self.this = MergeTree(os.path.join(self.dir, "this"))
-        self.other = MergeTree(os.path.join(self.dir, "other"))
+        self.dir = mkdtemp(prefix="BaZing")
+        self.base = MergeTree(pathjoin(self.dir, "base"))
+        self.this = MergeTree(pathjoin(self.dir, "this"))
+        self.other = MergeTree(pathjoin(self.dir, "other"))
         
         self.cset = changeset.Changeset()
         self.cset.add_entry(changeset.ChangesetEntry("0", 
                                                      changeset.NULL_ID, "./."))
+
     def get_cset_path(self, parent, name):
         if name is None:
             assert (parent is None)
             return None
-        return os.path.join(self.cset.entries[parent].path, name)
+        return pathjoin(self.cset.entries[parent].path, name)
 
     def add_file(self, id, parent, name, contents, mode):
         self.base.add_file(id, parent, name, contents, mode)
@@ -134,6 +163,13 @@
         path = self.get_cset_path(parent, name)
         self.cset.add_entry(changeset.ChangesetEntry(id, parent, path))
 
+    def add_symlink(self, id, parent, name, contents):
+        self.base.add_symlink(id, parent, name, contents)
+        self.this.add_symlink(id, parent, name, contents)
+        self.other.add_symlink(id, parent, name, contents)
+        path = self.get_cset_path(parent, name)
+        self.cset.add_entry(changeset.ChangesetEntry(id, parent, path))
+
     def remove_file(self, id, base=False, this=False, other=False):
         for option, tree in ((base, self.base), (this, self.this), 
                              (other, self.other)):
@@ -145,7 +181,7 @@
                     change = changeset.ReplaceContents(None, None)
                     self.cset.entries[id].contents_change = change
                     def create_file(tree):
-                        return changeset.FileCreate(tree.get_file(id).read())
+                        return changeset.TreeFileCreate(tree, id)
                     if not other:
                         change.new_contents = create_file(self.other)
                     if not base:
@@ -159,7 +195,6 @@
                 if change.old_contents is None and change.new_contents is None:
                     change = None
 
-
     def add_dir(self, id, parent, name, mode):
         path = self.get_cset_path(parent, name)
         self.base.add_dir(id, parent, name, mode)
@@ -167,7 +202,6 @@
         self.this.add_dir(id, parent, name, mode)
         self.other.add_dir(id, parent, name, mode)
 
-
     def change_name(self, id, base=None, this=None, other=None):
         if base is not None:
             self.change_name_tree(id, self.base, base)
@@ -208,8 +242,23 @@
         if base is not None or other is not None:
             old_contents = file(self.base.full_path(id)).read()
             new_contents = file(self.other.full_path(id)).read()
-            contents = changeset.ReplaceFileContents(old_contents, 
-                                                     new_contents)
+            contents = changeset.ReplaceFileContents(self.base, self.other, id)
+            self.cset.entries[id].contents_change = contents
+
+    def change_target(self, id, base=None, this=None, other=None):
+        if base is not None:
+            self.change_target_tree(id, self.base, base)
+
+        if this is not None:
+            self.change_target_tree(id, self.this, this)
+
+        if other is not None:
+            self.change_target_tree(id, self.other, other)
+
+        if base is not None or other is not None:
+            old_contents = get_contents(self.base, id)
+            new_contents = get_contents(self.other, id)
+            contents = ReplaceContents(old_contents, new_contents)
             self.cset.entries[id].contents_change = contents
 
     def change_perms(self, id, base=None, this=None, other=None):
@@ -223,11 +272,10 @@
             self.change_perms_tree(id, self.other, other)
 
         if base is not None or other is not None:
-            old_perms = os.stat(self.base.full_path(id)).st_mode &077
-            new_perms = os.stat(self.other.full_path(id)).st_mode &077
-            contents = changeset.ChangeUnixPermissions(old_perms, 
-                                                       new_perms)
-            self.cset.entries[id].metadata_change = contents
+            old_exec = self.base.is_executable(id)
+            new_exec = self.other.is_executable(id)
+            metadata = changeset.ChangeExecFlag(old_exec, new_exec)
+            self.cset.entries[id].metadata_change = metadata
 
     def change_name_tree(self, id, tree, name):
         new_path = tree.child_path(self.cset.entries[id].parent, name)
@@ -243,6 +291,11 @@
         file(path, "w").write(contents)
         os.chmod(path, mode)
 
+    def change_target_tree(self, id, tree, target):
+        path = tree.full_path(id)
+        os.unlink(path)
+        os.symlink(target, path)
+
     def change_perms_tree(self, id, tree, mode):
         os.chmod(tree.full_path(id), mode)
 
@@ -275,7 +328,7 @@
                 if parent is None:
                     return orig_inventory[file_id]
                 dirname = new_path(parent)
-                return os.path.join(dirname, orig_inventory[file_id])
+                return pathjoin(dirname, os.path.basename(orig_inventory[file_id]))
 
         new_inventory = {}
         for file_id in orig_inventory.iterkeys():
@@ -290,18 +343,19 @@
             new_inventory[file_id] = path
         return new_inventory
 
-    def apply_changeset(self, cset, conflict_handler=None, reverse=False):
+    def apply_changeset(self, cset, conflict_handler=None):
         inventory_change = changeset.apply_changeset(cset,
-                                                     self.this.inventory,
+                                                     self.this.inventory_dict,
                                                      self.this.dir,
-                                                     conflict_handler, reverse)
-        self.this.inventory =  self.apply_inv_change(inventory_change, 
-                                                     self.this.inventory)
+                                                     conflict_handler)
+        self.this.inventory_dict =  self.apply_inv_change(inventory_change, 
+                                                     self.this.inventory_dict)
 
     def cleanup(self):
         shutil.rmtree(self.dir)
 
-class MergeTest(unittest.TestCase):
+
+class MergeTest(TestCase):
     def test_change_name(self):
         """Test renames"""
         builder = MergeBuilder()
@@ -312,18 +366,18 @@
         builder.add_file("3", "0", "name5", "hello3", 0755)
         builder.change_name("3", this="name6")
         cset = builder.merge_changeset(ApplyMerge3)
-        assert(cset.entries["2"].is_boring())
-        assert(cset.entries["1"].name == "name1")
-        assert(cset.entries["1"].new_name == "name2")
-        assert(cset.entries["3"].is_boring())
+        self.failUnless(cset.entries["2"].is_boring())
+        self.assertEqual(cset.entries["1"].name, "name1")
+        self.assertEqual(cset.entries["1"].new_name, "name2")
+        self.failUnless(cset.entries["3"].is_boring())
         for tree in (builder.this, builder.other, builder.base):
-            assert(tree.dir != builder.dir and 
-                   tree.dir.startswith(builder.dir))
-            for path in tree.inventory.itervalues():
+            self.assertNotEqual(tree.dir, builder.dir)
+            self.assertStartsWith(tree.dir, builder.dir)
+            for path in tree.inventory_dict.itervalues():
                 fullpath = tree.abs_path(path)
-                assert(fullpath.startswith(tree.dir))
-                assert(not path.startswith(tree.dir))
-                assert os.path.exists(fullpath)
+                self.assertStartsWith(fullpath, tree.dir)
+                self.failIf(path.startswith(tree.dir))
+                self.failUnless(os.path.lexists(fullpath))
         builder.apply_changeset(cset)
         builder.cleanup()
         builder = MergeBuilder()
@@ -342,16 +396,16 @@
         builder.add_file("4", "1", "file2", "hello2", 0644)
         builder.add_file("5", "1", "file3", "hello3", 0644)
         builder.change_parent("3", other="2")
-        assert(Inventory(builder.other.inventory).get_parent("3") == "2")
+        self.assert_(Inventory(builder.other.inventory_dict).get_parent("3") == "2")
         builder.change_parent("4", this="2")
-        assert(Inventory(builder.this.inventory).get_parent("4") == "2")
+        self.assert_(Inventory(builder.this.inventory_dict).get_parent("4") == "2")
         builder.change_parent("5", base="2")
-        assert(Inventory(builder.base.inventory).get_parent("5") == "2")
+        self.assert_(Inventory(builder.base.inventory_dict).get_parent("5") == "2")
         cset = builder.merge_changeset(ApplyMerge3)
         for id in ("1", "2", "4", "5"):
-            assert(cset.entries[id].is_boring())
-        assert(cset.entries["3"].parent == "1")
-        assert(cset.entries["3"].new_parent == "2")
+            self.assert_(cset.entries[id].is_boring())
+        self.assert_(cset.entries["3"].parent == "1")
+        self.assert_(cset.entries["3"].new_parent == "2")
         builder.apply_changeset(cset)
         builder.cleanup()
 
@@ -380,9 +434,9 @@
         builder = self.contents_test_success(backup_merge)
         def backup_exists(file_id):
             return os.path.exists(builder.this.full_path(file_id)+"~")
-        assert backup_exists("1")
-        assert backup_exists("2")
-        assert not backup_exists("3")
+        self.assert_(backup_exists("1"))
+        self.assert_(backup_exists("2"))
+        self.assert_(not backup_exists("3"))
         builder.cleanup()
 
     def do_contents_test(self, merge_factory):
@@ -401,23 +455,24 @@
         builder.add_file("3", "0", "name5", "text3", 0744)
         builder.add_file("4", "0", "name6", "text4", 0744)
         builder.remove_file("4", base=True)
-        assert not builder.cset.entries["4"].is_boring()
+        self.assert_(not builder.cset.entries["4"].is_boring())
         builder.change_contents("3", this="text6")
         cset = builder.merge_changeset(merge_factory)
-        assert(cset.entries["1"].contents_change is not None)
+        self.assert_(cset.entries["1"].contents_change is not None)
         if isclass(merge_factory):
-            assert(isinstance(cset.entries["1"].contents_change,
-                          merge_factory))
-            assert(isinstance(cset.entries["2"].contents_change,
-                          merge_factory))
-        assert(cset.entries["3"].is_boring())
-        assert(cset.entries["4"].is_boring())
+            self.assert_(isinstance(cset.entries["1"].contents_change,
+                          merge_factory))
+            self.assert_(isinstance(cset.entries["2"].contents_change,
+                          merge_factory))
+        self.assert_(cset.entries["3"].is_boring())
+        self.assert_(cset.entries["4"].is_boring())
         builder.apply_changeset(cset)
-        assert(file(builder.this.full_path("1"), "rb").read() == "text4" )
-        assert(file(builder.this.full_path("2"), "rb").read() == "text2" )
-        assert(os.stat(builder.this.full_path("1")).st_mode &0777 == 0755)
-        assert(os.stat(builder.this.full_path("2")).st_mode &0777 == 0655)
-        assert(os.stat(builder.this.full_path("3")).st_mode &0777 == 0744)
+        self.assert_(file(builder.this.full_path("1"), "rb").read() == "text4" )
+        self.assert_(file(builder.this.full_path("2"), "rb").read() == "text2" )
+        if sys.platform != "win32":
+            self.assert_(os.stat(builder.this.full_path("1")).st_mode &0777 == 0755)
+            self.assert_(os.stat(builder.this.full_path("2")).st_mode &0777 == 0655)
+            self.assert_(os.stat(builder.this.full_path("3")).st_mode &0777 == 0744)
         return builder
 
     def contents_test_conflicts(self, merge_factory):
@@ -429,62 +484,67 @@
                           cset)
         builder.cleanup()
 
-        builder = MergeBuilder()
-        builder.add_file("1", "0", "name1", "text1", 0755)
-        builder.change_contents("1", other="text4", base="text3")
-        builder.remove_file("1", base=True)
-        self.assertRaises(changeset.NewContentsConflict,
-                          builder.merge_changeset, merge_factory)
-        builder.cleanup()
+    def test_symlink_conflicts(self):
+        if sys.platform != "win32":
+            builder = MergeBuilder()
+            builder.add_symlink("2", "0", "name2", "target1")
+            builder.change_target("2", other="target4", base="text3")
+            self.assertRaises(changeset.ThreewayContentsConflict,
+                              builder.merge_changeset, ApplyMerge3)
+            builder.cleanup()
 
-        builder = MergeBuilder()
-        builder.add_file("1", "0", "name1", "text1", 0755)
-        builder.change_contents("1", other="text4", base="text3")
-        builder.remove_file("1", this=True)
-        self.assertRaises(changeset.MissingForMerge, builder.merge_changeset, 
-                          merge_factory)
-        builder.cleanup()
+    def test_symlink_merge(self):
+        if sys.platform != "win32":
+            builder = MergeBuilder()
+            builder.add_symlink("1", "0", "name1", "target1")
+            builder.add_symlink("2", "0", "name2", "target1")
+            builder.add_symlink("3", "0", "name3", "target1")
+            builder.change_target("1", this="target2")
+            builder.change_target("2", base="target2")
+            builder.change_target("3", other="target2")
+            self.assertNotEqual(builder.cset.entries['2'].contents_change,
+                                builder.cset.entries['3'].contents_change)
+            cset = builder.merge_changeset(ApplyMerge3)
+            builder.apply_changeset(cset)
+            self.assertEqual(builder.this.get_symlink_target("1"), "target2")
+            self.assertEqual(builder.this.get_symlink_target("2"), "target1")
+            self.assertEqual(builder.this.get_symlink_target("3"), "target2")
+            builder.cleanup()
 
     def test_perms_merge(self):
         builder = MergeBuilder()
         builder.add_file("1", "0", "name1", "text1", 0755)
-        builder.change_perms("1", other=0655)
+        builder.change_perms("1", other=0644)
         builder.add_file("2", "0", "name2", "text2", 0755)
-        builder.change_perms("2", base=0655)
+        builder.change_perms("2", base=0644)
         builder.add_file("3", "0", "name3", "text3", 0755)
-        builder.change_perms("3", this=0655)
+        builder.change_perms("3", this=0644)
         cset = builder.merge_changeset(ApplyMerge3)
-        assert(cset.entries["1"].metadata_change is not None)
-        assert(isinstance(cset.entries["1"].metadata_change,
-                          PermissionsMerge))
-        assert(isinstance(cset.entries["2"].metadata_change,
-                          PermissionsMerge))
-        assert(cset.entries["3"].is_boring())
+        self.assert_(cset.entries["1"].metadata_change is not None)
+        self.assert_(isinstance(cset.entries["1"].metadata_change, ExecFlagMerge))
+        self.assert_(isinstance(cset.entries["2"].metadata_change, ExecFlagMerge))
+        self.assert_(cset.entries["3"].is_boring())
         builder.apply_changeset(cset)
-        assert(os.stat(builder.this.full_path("1")).st_mode &0777 == 0655)
-        assert(os.stat(builder.this.full_path("2")).st_mode &0777 == 0755)
-        assert(os.stat(builder.this.full_path("3")).st_mode &0777 == 0655)
+        if sys.platform != "win32":
+            self.assert_(os.lstat(builder.this.full_path("1")).st_mode &0100 == 0000)
+            self.assert_(os.lstat(builder.this.full_path("2")).st_mode &0100 == 0100)
+            self.assert_(os.lstat(builder.this.full_path("3")).st_mode &0100 == 0000)
         builder.cleanup();
-        builder = MergeBuilder()
-        builder.add_file("1", "0", "name1", "text1", 0755)
-        builder.change_perms("1", other=0655, base=0555)
-        cset = builder.merge_changeset(ApplyMerge3)
-        self.assertRaises(changeset.MergePermissionConflict, 
-                     builder.apply_changeset, cset)
-        builder.cleanup()
+
 
 class FunctionalMergeTest(TestCaseInTempDir):
 
     def test_trivial_star_merge(self):
         """Test that merges in a star shape Just Work.""" 
-        from bzrlib.add import smart_add_branch, add_reporter_null
+        from bzrlib.add import smart_add_tree
         from bzrlib.clone import copy_branch
         from bzrlib.merge import merge
         # John starts a branch
         self.build_tree(("original/", "original/file1", "original/file2"))
         branch = Branch.initialize("original")
-        smart_add_branch(branch, ["original"], True, add_reporter_null)
-        branch.commit("start branch.", verbose=False)
+        tree = WorkingTree('original', branch)
+        smart_add_tree(tree, ["original"])
+        tree.commit("start branch.", verbose=False)
         # Mary branches it.
         self.build_tree(("mary/",))
         copy_branch(branch, "mary")
@@ -492,17 +552,287 @@
         file = open("original/file1", "wt")
         file.write("John\n")
         file.close()
-        branch.commit("change file1")
+        branch.working_tree().commit("change file1")
         # Mary does too
         mary_branch = Branch.open("mary")
         file = open("mary/file2", "wt")
         file.write("Mary\n")
         file.close()
-        mary_branch.commit("change file2")
+        mary_branch.working_tree().commit("change file2")
         # john should be able to merge with no conflicts.
         merge_type = ApplyMerge3
         base = [None, None]
         other = ("mary", -1)
-        merge(other, base, check_clean=True, merge_type=merge_type, this_dir="original")
+        self.assertRaises(BzrCommandError, merge, other, base, check_clean=True,
+                          merge_type=WeaveMerge, this_dir="original",
+                          show_base=True)
+        merge(other, base, check_clean=True, merge_type=merge_type,
+              this_dir="original")
         self.assertEqual("John\n", open("original/file1", "rt").read())
         self.assertEqual("Mary\n", open("original/file2", "rt").read())
+ 
+    def test_conflicts(self):
+        os.mkdir('a')
+        a = Branch.initialize('a')
+        file('a/file', 'wb').write('contents\n')
+        a.working_tree().add('file')
+        a.working_tree().commit('base revision', allow_pointless=False)
+        b = copy_branch(a, 'b')
+        file('a/file', 'wb').write('other contents\n')
+        a.working_tree().commit('other revision', allow_pointless=False)
+        file('b/file', 'wb').write('this contents contents\n')
+        b.working_tree().commit('this revision', allow_pointless=False)
+        self.assertEqual(merge(['a', -1], [None, None], this_dir='b'), 1)
+        self.assert_(os.path.lexists('b/file.THIS'))
+        self.assert_(os.path.lexists('b/file.BASE'))
+        self.assert_(os.path.lexists('b/file.OTHER'))
+        self.assertRaises(WorkingTreeNotRevision, merge, ['a', -1], 
+                          [None, None], this_dir='b', check_clean=False,
+                          merge_type=WeaveMerge)
+        merge(['b', -1], ['b', None], this_dir='b', check_clean=False)
+        os.unlink('b/file.THIS')
+        os.unlink('b/file.OTHER')
+        os.unlink('b/file.BASE')
+        self.assertEqual(merge(['a', -1], [None, None], this_dir='b', 
+                               check_clean=False, merge_type=WeaveMerge), 1)
+        self.assert_(os.path.lexists('b/file'))
+        self.assert_(os.path.lexists('b/file.THIS'))
+        self.assert_(not os.path.lexists('b/file.BASE'))
+        self.assert_(os.path.lexists('b/file.OTHER'))
+
+    def test_merge_unrelated(self):
+        """Sucessfully merges unrelated branches with no common names"""
+        os.mkdir('a')
+        a = Branch.initialize('a')
+        file('a/a_file', 'wb').write('contents\n')
+        a.working_tree().add('a_file')
+        a.working_tree().commit('a_revision', allow_pointless=False)
+        os.mkdir('b')
+        b = Branch.initialize('b')
+        file('b/b_file', 'wb').write('contents\n')
+        b.working_tree().add('b_file')
+        b.working_tree().commit('b_revision', allow_pointless=False)
+        merge(['b', -1], ['b', 0], this_dir='a')
+        self.assert_(os.path.lexists('a/b_file'))
+        self.assertEqual(a.working_tree().pending_merges(),
+                         [b.last_revision()]) 
+
+    def test_merge_unrelated_conflicting(self):
+        """Sucessfully merges unrelated branches with common names"""
+        os.mkdir('a')
+        a = Branch.initialize('a')
+        file('a/file', 'wb').write('contents\n')
+        a.working_tree().add('file')
+        a.working_tree().commit('a_revision', allow_pointless=False)
+        os.mkdir('b')
+        b = Branch.initialize('b')
+        file('b/file', 'wb').write('contents\n')
+        b.working_tree().add('file')
+        b.working_tree().commit('b_revision', allow_pointless=False)
+        merge(['b', -1], ['b', 0], this_dir='a')
+        self.assert_(os.path.lexists('a/file'))
+        self.assert_(os.path.lexists('a/file.moved'))
+        self.assertEqual(a.working_tree().pending_merges(), [b.last_revision()])
+
+    def test_merge_deleted_conflicts(self):
+        os.mkdir('a')
+        a = Branch.initialize('a')
+        file('a/file', 'wb').write('contents\n')
+        a.working_tree().add('file')
+        a.working_tree().commit('a_revision', allow_pointless=False)
+        del a
+        self.run_bzr('branch', 'a', 'b')
+        a = Branch.open('a')
+        os.remove('a/file')
+        a.working_tree().commit('removed file', allow_pointless=False)
+        file('b/file', 'wb').write('changed contents\n')
+        b = Branch.open('b')
+        b.working_tree().commit('changed file', allow_pointless=False)
+        merge(['a', -1], ['a', 1], this_dir='b')
+        self.failIf(os.path.lexists('b/file'))
+
+    def test_merge_metadata_vs_deletion(self):
+        """Conflict deletion vs metadata change"""
+        os.mkdir('a')
+        a = Branch.initialize('a')
+        file('a/file', 'wb').write('contents\n')
+        a_wt = a.working_tree()
+        a_wt.add('file')
+        a_wt.commit('r0')
+        copy_branch(a, 'b')
+        b = Branch.open('b')
+        b_wt = b.working_tree()
+        os.chmod('b/file', 0755)
+        os.remove('a/file')
+        a_wt.commit('removed a')
+        self.assertEqual(a.revno(), 2)
+        self.assertFalse(os.path.exists('a/file'))
+        b_wt.commit('exec a')
+        merge(['b', -1], ['b', 0], this_dir='a')
+        self.assert_(os.path.exists('a/file'))
+
+    def test_merge_swapping_renames(self):
+        os.mkdir('a')
+        a = Branch.initialize('a')
+        file('a/un','wb').write('UN')
+        file('a/deux','wb').write('DEUX')
+        a_wt = a.working_tree()
+        a_wt.add('un')
+        a_wt.add('deux')
+        a_wt.commit('r0')
+        copy_branch(a,'b')
+        b = Branch.open('b')
+        b_wt = b.working_tree()
+        b_wt.rename_one('un','tmp')
+        b_wt.rename_one('deux','un')
+        b_wt.rename_one('tmp','deux')
+        b_wt.commit('r1')
+        merge(['b', -1],['b', 1],this_dir='a')
+        self.assert_(os.path.exists('a/un'))
+        self.assert_(os.path.exists('a/deux'))
+        self.assertFalse(os.path.exists('a/tmp'))
+        self.assertEqual(file('a/un').read(),'DEUX')
+        self.assertEqual(file('a/deux').read(),'UN')
+
+    def test_merge_delete_and_add_same(self):
+        os.mkdir('a')
+        a = Branch.initialize('a')
+        file('a/file', 'wb').write('THIS')
+        a_wt = a.working_tree()
+        a_wt.add('file')
+        a_wt.commit('r0')
+        copy_branch(a, 'b')
+        b = Branch.open('b')
+        b_wt = b.working_tree()
+        os.remove('b/file')
+        b_wt.commit('r1')
+        file('b/file', 'wb').write('THAT')
+        b_wt.add('file')
+        b_wt.commit('r2')
+        merge(['b', -1],['b', 1],this_dir='a')
+        self.assert_(os.path.exists('a/file'))
+        self.assertEqual(file('a/file').read(),'THAT')
+
+    def test_merge_rename_before_create(self):
+        """rename before create
+        
+        This case requires that you must not do creates
+        before move-into-place:
+
+        $ touch foo
+        $ bzr add foo
+        $ bzr commit
+        $ bzr mv foo bar
+        $ touch foo
+        $ bzr add foo
+        $ bzr commit
+        """
+        os.mkdir('a')
+        a = Branch.initialize('a')
+        file('a/foo', 'wb').write('A/FOO')
+        a_wt = a.working_tree()
+        a_wt.add('foo')
+        a_wt.commit('added foo')
+        copy_branch(a, 'b')
+        b = Branch.open('b')
+        b_wt = b.working_tree()
+        b_wt.rename_one('foo', 'bar')
+        file('b/foo', 'wb').write('B/FOO')
+        b_wt.add('foo')
+        b_wt.commit('moved foo to bar, added new foo')
+        merge(['b', -1],['b', 1],this_dir='a')
+
+    def test_merge_create_before_rename(self):
+        """create before rename, target parents before children
+
+        This case requires that you must not do move-into-place
+        before creates, and that you must not do children after
+        parents:
+
+        $ touch foo
+        $ bzr add foo
+        $ bzr commit
+        $ bzr mkdir bar
+        $ bzr add bar
+        $ bzr mv foo bar/foo
+        $ bzr commit
+        """
+        os.mkdir('a')
+        a = Branch.initialize('a')
+        file('a/foo', 'wb').write('A/FOO')
+        a_wt = a.working_tree()
+        a_wt.add('foo')
+        a_wt.commit('added foo')
+        copy_branch(a, 'b')
+        b = Branch.open('b')
+        b_wt = b.working_tree()
+        os.mkdir('b/bar')
+        b_wt.add('bar')
+        b_wt.rename_one('foo', 'bar/foo')
+        b_wt.commit('created bar dir, moved foo into bar')
+        merge(['b', -1],['b', 1],this_dir='a')
+
+    def test_merge_rename_to_temp_before_delete(self):
+        """rename to temp before delete, source children before parents
+
+        This case requires that you must not do deletes before
+        move-out-of-the-way, and that you must not do children
+        after parents:
+        
+        $ mkdir foo
+        $ touch foo/bar
+        $ bzr add foo/bar
+        $ bzr commit
+        $ bzr mv foo/bar bar
+        $ rmdir foo
+        $ bzr commit
+        """
+        os.mkdir('a')
+        a = Branch.initialize('a')
+        os.mkdir('a/foo')
+        file('a/foo/bar', 'wb').write('A/FOO/BAR')
+        a_wt = a.working_tree()
+        a_wt.add('foo')
+        a_wt.add('foo/bar')
+        a_wt.commit('added foo/bar')
+        copy_branch(a, 'b')
+        b = Branch.open('b')
+        b_wt = b.working_tree()
+        b_wt.rename_one('foo/bar', 'bar')
+        os.rmdir('b/foo')
+        b_wt.remove('foo')
+        b_wt.commit('moved foo/bar to bar, deleted foo')
+        merge(['b', -1],['b', 1],this_dir='a')
+
+    def test_merge_delete_before_rename_to_temp(self):
+        """delete before rename to temp
+
+        This case requires that you must not do
+        move-out-of-the-way before deletes:
+        
+        $ touch foo
+        $ touch bar
+        $ bzr add foo bar
+        $ bzr commit
+        $ rm foo
+        $ bzr rm foo
+        $ bzr mv bar foo
+        $ bzr commit
+        """
+        os.mkdir('a')
+        a = Branch.initialize('a')
+        file('a/foo', 'wb').write('A/FOO')
+        file('a/bar', 'wb').write('A/BAR')
+        a_wt = a.working_tree()
+        a_wt.add('foo')
+        a_wt.add('bar')
+        a_wt.commit('added foo and bar')
+        copy_branch(a, 'b')
+        b = Branch.open('b')
+        b_wt = b.working_tree()
+        os.unlink('b/foo')
+        b_wt.remove('foo')
+        b_wt.rename_one('bar', 'foo')
+        b_wt.commit('deleted foo, renamed bar to foo')
+        merge(['b', -1],['b', 1],this_dir='a')
+

=== added file 'bzrlib/tests/test_missing.py'
--- bzrlib/tests/test_missing.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_missing.py	2005-12-12 00:20:32 +0000
@@ -0,0 +1,49 @@
+# Copyright (C) 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+from bzrlib.missing import find_unmerged
+from bzrlib.branch import Branch
+from bzrlib.merge import merge
+from bzrlib.tests import TestCaseInTempDir
+import os
+
+class TestMissing(TestCaseInTempDir):
+    def test_find_unmerged(self):
+        os.mkdir('original')
+        os.mkdir('puller')
+        os.mkdir('merger')
+        original = Branch.initialize('original')
+        puller = Branch.initialize('puller')
+        merger = Branch.initialize('merger')
+        self.assertEqual(find_unmerged(original, puller), ([], []))
+        original.working_tree().commit('a', rev_id='a')
+        self.assertEqual(find_unmerged(original, puller), ([(1, u'a')], []))
+        puller.pull(original)
+        self.assertEqual(find_unmerged(original, puller), ([], []))
+        merger.pull(original)
+        original.working_tree().commit('b', rev_id='b')
+        original.working_tree().commit('c', rev_id='c')
+        self.assertEqual(find_unmerged(original, puller), ([(2, u'b'), 
+                                                            (3, u'c')], []))
+
+        puller.pull(original)
+        self.assertEqual(find_unmerged(original, puller), ([], []))
+        self.assertEqual(find_unmerged(original, merger), ([(2, u'b'), 
+                                                            (3, u'c')], []))
+        merge(['original', -1], [None, None], this_dir='merger')
+        self.assertEqual(find_unmerged(original, merger), ([(2, u'b'), 
+                                                            (3, u'c')], []))
+        merger.working_tree().commit('d', rev_id='d')
+        self.assertEqual(find_unmerged(original, merger), ([], [(2, 'd')]))

=== added file 'bzrlib/tests/test_msgeditor.py'
--- bzrlib/tests/test_msgeditor.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_msgeditor.py	2005-12-14 09:01:40 +0000
@@ -0,0 +1,50 @@
+# Copyright (C) 2005 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License version 2 as published by
+# the Free Software Foundation.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Test commit message editor.
+"""
+
+import os
+import sys
+
+from bzrlib.branch import Branch
+from bzrlib.msgeditor import make_commit_message_template
+from bzrlib.tests import TestCaseInTempDir, TestSkipped
+
+
+class MsgEditorTest(TestCaseInTempDir):
+
+    def make_uncommitted_tree(self):
+        """Build a branch with uncommitted unicode named changes in the cwd."""
+        b = Branch.initialize('.')
+        working_tree = b.working_tree()
+        filename = u'hell\u00d8'
+        try:
+            self.build_tree_contents([(filename, 'contents of hello')])
+        except UnicodeEncodeError:
+            raise TestSkipped("can't build unicode working tree in "
+                "filesystem encoding %s" % sys.getfilesystemencoding())
+        working_tree.add(filename)
+        return working_tree
+    
+    def test_commit_template(self):
+        """Test building a commit message template"""
+        working_tree = self.make_uncommitted_tree()
+        template = make_commit_message_template(working_tree, None)
+        self.assertEqualDiff(template,
+u"""\
+added:
+  hell\u00d8
+""")

=== added file 'bzrlib/tests/test_nonascii.py'
--- bzrlib/tests/test_nonascii.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_nonascii.py	2005-11-25 06:28:56 +0000
@@ -0,0 +1,38 @@
+# Copyright (C) 2005 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Test that various operations work in a non-ASCII environment."""
+
+import os
+
+from bzrlib.tests import TestCaseInTempDir
+from bzrlib.branch import Branch
+
+
+class NonAsciiTest(TestCaseInTempDir):
+
+    def test_add_in_nonascii_branch(self):
+        """Test adding in a non-ASCII branch."""
+        br_dir = u"\u1234"
+        try:
+            os.mkdir(br_dir)
+            os.chdir(br_dir)
+        except UnicodeEncodeError:
+            self.log("filesystem can't accomodate nonascii names")
+            return
+        br = Branch.initialize(u".")
+        file("a", "w").write("hello")
+        br.working_tree().add(["a"], ["a-id"])

=== added file 'bzrlib/tests/test_options.py'
--- bzrlib/tests/test_options.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_options.py	2005-11-23 15:44:24 +0000
@@ -0,0 +1,57 @@
+# (C) 2005 Canonical
+
+from bzrlib.tests import TestCase
+from bzrlib.commands import Command, parse_args
+from bzrlib.builtins import cmd_commit, cmd_log, cmd_status
+
+
+class OptionTests(TestCase):
+    """Command-line option tests"""
+
+    def test_parse_args(self):
+        """Option parser"""
+        eq = self.assertEquals
+        eq(parse_args(cmd_commit(), ['--help']),
+           ([], {'help': True}))
+        eq(parse_args(cmd_status(), ['--all']),
+           ([], {'all': True}))
+        eq(parse_args(cmd_commit(), ['--message=biter']),
+           ([], {'message': 'biter'}))
+        ## eq(parse_args(cmd_log(),  '-r 500'.split()),
+        ##   ([], {'revision': RevisionSpec_int(500)}))
+
+    def test_no_more_opts(self):
+        """Terminated options"""
+        self.assertEquals(parse_args(cmd_commit(), ['--', '-file-with-dashes']),
+                          (['-file-with-dashes'], {}))
+
+    def test_option_help(self):
+        """Options have help strings."""
+        out, err = self.run_bzr_captured(['commit', '--help'])
+        self.assertContainsRe(out, r'--file.*file containing commit message')
+        self.assertContainsRe(out, r'--help.*-h')
+
+    def test_option_help_global(self):
+        """Global options have help strings."""
+        out, err = self.run_bzr_captured(['help', 'status'])
+        self.assertContainsRe(out, r'--show-ids.*show internal object')
+
+    def test_option_arg_help(self):
+        """Help message shows option arguments."""
+        out, err = self.run_bzr_captured(['help', 'commit'])
+        self.assertEquals(err, '')
+        self.assertContainsRe(out, r'--file[ =]MSGFILE')
+
+    def test_unknown_short_opt(self):
+        out, err = self.run_bzr_captured(['help', '-r'], retcode=3)
+        self.assertContainsRe(err, r'unknown short option')
+
+
+#     >>> parse_args('log -r 500'.split())
+#     (['log'], {'revision': [<RevisionSpec_int 500>]})
+#     >>> parse_args('log -r500..600'.split())
+#     (['log'], {'revision': [<RevisionSpec_int 500>, <RevisionSpec_int 600>]})
+#     >>> parse_args('log -vr500..600'.split())
+#     (['log'], {'verbose': True, 'revision': [<RevisionSpec_int 500>, <RevisionSpec_int 600>]})
+#     >>> parse_args('log -rrevno:500..600'.split()) #the r takes an argument
+#     (['log'], {'revision': [<RevisionSpec_revno revno:500>, <RevisionSpec_int 600>]})

=== added file 'bzrlib/tests/test_osutils.py'
--- bzrlib/tests/test_osutils.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_osutils.py	2006-01-05 22:44:46 +0000
@@ -0,0 +1,81 @@
+# Copyright (C) 2005 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for the osutils wrapper.
+"""
+
+import os
+import sys
+
+import bzrlib
+from bzrlib.errors import BzrBadParameter
+import bzrlib.osutils as osutils
+from bzrlib.tests import TestCaseInTempDir, TestCase
+
+
+class TestOSUtils(TestCaseInTempDir):
+
+    def test_fancy_rename(self):
+        # This should work everywhere
+        def rename(a, b):
+            osutils.fancy_rename(a, b,
+                    rename_func=os.rename,
+                    unlink_func=os.unlink)
+
+        open('a', 'wb').write('something in a\n')
+        rename('a', 'b')
+        self.failIfExists('a')
+        self.failUnlessExists('b')
+        self.check_file_contents('b', 'something in a\n')
+
+        open('a', 'wb').write('new something in a\n')
+        rename('b', 'a')
+
+        self.check_file_contents('a', 'something in a\n')
+
+    def test_rename(self):
+        # Rename should be semi-atomic on all platforms
+        open('a', 'wb').write('something in a\n')
+        osutils.rename('a', 'b')
+        self.failIfExists('a')
+        self.failUnlessExists('b')
+        self.check_file_contents('b', 'something in a\n')
+
+        open('a', 'wb').write('new something in a\n')
+        osutils.rename('b', 'a')
+
+        self.check_file_contents('a', 'something in a\n')
+
+
+    # TODO: test fancy_rename using a MemoryTransport
+
+
+class TestSafeUnicode(TestCase):
+
+    def test_from_ascii_string(self):
+        self.assertEqual(u'foobar', osutils.safe_unicode('foobar'))
+
+    def test_from_unicode_string_ascii_contents(self):
+        self.assertEqual(u'bargam', osutils.safe_unicode(u'bargam'))
+
+    def test_from_unicode_string_unicode_contents(self):
+        self.assertEqual(u'bargam\xae', osutils.safe_unicode(u'bargam\xae'))
+
+    def test_from_utf8_string(self):
+        self.assertEqual(u'foo\xae', osutils.safe_unicode('foo\xc2\xae'))
+
+    def test_bad_utf8_string(self):
+        self.assertRaises(BzrBadParameter, osutils.safe_unicode, '\xbb\xbb')

=== modified file 'bzrlib/tests/test_parent.py'
--- bzrlib/selftest/test_parent.py	2005-09-29 20:34:25 +0000
+++ bzrlib/tests/test_parent.py	2006-01-03 02:07:06 +0000
@@ -16,9 +16,10 @@
 
 
 import os
-from bzrlib.selftest import TestCaseInTempDir
+from bzrlib.tests import TestCaseInTempDir
 from bzrlib.branch import Branch
 from bzrlib.clone import copy_branch
+from bzrlib.osutils import abspath, realpath
 
 
 """Tests for Branch parent URL"""
@@ -27,13 +28,13 @@
 class TestParent(TestCaseInTempDir):
     def test_no_default_parent(self):
         """Branches should have no parent by default"""
-        b = Branch.initialize('.')
+        b = Branch.initialize(u'.')
         self.assertEquals(b.get_parent(), None)
         
     
     def test_set_get_parent(self):
         """Set and then re-get the parent"""
-        b = Branch.initialize('.')
+        b = Branch.initialize(u'.')
         url = 'http://bazaar-ng.org/bzr/bzr.dev'
         b.set_parent(url)
         self.assertEquals(b.get_parent(), url)
@@ -45,14 +46,11 @@
         os.mkdir('from')
         branch_from = Branch.initialize('from')
         file('from/foo', 'wt').write('contents of foo')
-        branch_from.add('foo')
-        branch_from.commit('initial commit')
+        branch_from.working_tree().add('foo')
+        branch_from.working_tree().commit('initial commit')
         
         os.mkdir('to')
         copy_branch(branch_from, 'to', None)
 
         branch_to = Branch.open('to')
-        abspath = os.path.abspath('from')
-        self.assertEquals(branch_to.get_parent(), abspath)
-        
-
+        self.assertEquals(branch_to.get_parent(), branch_from.base)

=== added file 'bzrlib/tests/test_permissions.py'
--- bzrlib/tests/test_permissions.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_permissions.py	2006-01-12 01:02:32 +0000
@@ -0,0 +1,321 @@
+# Copyright (C) 2005 by Canonical Ltd
+# -*- coding: utf-8 -*-
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+"""Tests for bzr setting permissions.
+
+Files which are created underneath .bzr/ should inherit its permissions.
+So if the directory is group writable, the files and subdirs should be as well.
+
+In the future, when we have Repository/Branch/Checkout information, the
+permissions should be inherited individually, rather than all be the same.
+
+TODO: jam 20051215 There are no tests for ftp yet, because we have no ftp server
+TODO: jam 20051215 Currently the default behavior for 'bzr branch' is just 
+                   defined by the local umask. This isn't terrible, is it
+                   the truly desired behavior?
+"""
+
+import os
+import sys
+import stat
+
+from bzrlib.branch import Branch
+from bzrlib.tests import TestCaseInTempDir, TestSkipped
+from bzrlib.tests.test_sftp_transport import TestCaseWithSFTPServer
+from bzrlib.transport import get_transport
+
+
+def chmod_r(base, file_mode, dir_mode):
+    """Recursively chmod from a base directory"""
+    assert os.path.isdir(base)
+    os.chmod(base, dir_mode)
+    for root, dirs, files in os.walk(base):
+        for d in dirs:
+            p = os.path.join(root, d)
+            os.chmod(p, dir_mode)
+        for f in files:
+            p = os.path.join(root, f)
+            os.chmod(p, file_mode)
+
+
+def check_mode_r(test, base, file_mode, dir_mode, include_base=True):
+    """Check that all permissions match
+
+    :param test: The TestCase being run
+    :param base: The path to the root directory to check
+    :param file_mode: The mode for all files
+    :param dir_mode: The mode for all directories
+    :param include_base: If false, only check the subdirectories
+    """
+    assert os.path.isdir(base)
+    t = get_transport(".")
+    if include_base:
+        test.assertTransportMode(t, base, dir_mode)
+    for root, dirs, files in os.walk(base):
+        for d in dirs:
+            p = os.path.join(root, d)
+            test.assertTransportMode(t, p, dir_mode)
+        for f in files:
+            p = os.path.join(root, f)
+            test.assertTransportMode(t, p, file_mode)
+
+
+def assertEqualMode(test, mode, mode_test):
+    test.assertEqual(mode, mode_test,
+                     'mode mismatch %o != %o' % (mode, mode_test))
+
+
+class TestPermissions(TestCaseInTempDir):
+
+    def test_new_files(self):
+        if sys.platform == 'win32':
+            raise TestSkipped('chmod has no effect on win32')
+
+        b = Branch.initialize(u'.')
+        t = b.working_tree()
+        open('a', 'wb').write('foo\n')
+        t.add('a')
+        t.commit('foo')
+
+        # Delete them because we are modifying the filesystem underneath them
+        del b, t 
+        chmod_r('.bzr', 0644, 0755)
+        check_mode_r(self, '.bzr', 0644, 0755)
+
+        b = Branch.open('.')
+        t = b.working_tree()
+        assertEqualMode(self, 0755, b._dir_mode)
+        assertEqualMode(self, 0644, b._file_mode)
+
+        # Modifying a file shouldn't break the permissions
+        open('a', 'wb').write('foo2\n')
+        t.commit('foo2')
+        # The mode should be maintained after commit
+        check_mode_r(self, '.bzr', 0644, 0755)
+
+        # Adding a new file should maintain the permissions
+        open('b', 'wb').write('new b\n')
+        t.add('b')
+        t.commit('new b')
+        check_mode_r(self, '.bzr', 0644, 0755)
+
+        del b, t
+        # Recursively update the modes of all files
+        chmod_r('.bzr', 0664, 0775)
+        check_mode_r(self, '.bzr', 0664, 0775)
+        b = Branch.open('.')
+        t = b.working_tree()
+        assertEqualMode(self, 0775, b._dir_mode)
+        assertEqualMode(self, 0664, b._file_mode)
+
+        open('a', 'wb').write('foo3\n')
+        t.commit('foo3')
+        check_mode_r(self, '.bzr', 0664, 0775)
+
+        open('c', 'wb').write('new c\n')
+        t.add('c')
+        t.commit('new c')
+        check_mode_r(self, '.bzr', 0664, 0775)
+
+        # Test the group sticky bit
+        del b, t
+        # Recursively update the modes of all files
+        chmod_r('.bzr', 0664, 02775)
+        check_mode_r(self, '.bzr', 0664, 02775)
+        b = Branch.open('.')
+        t = b.working_tree()
+        assertEqualMode(self, 02775, b._dir_mode)
+        assertEqualMode(self, 0664, b._file_mode)
+
+        open('a', 'wb').write('foo4\n')
+        t.commit('foo4')
+        check_mode_r(self, '.bzr', 0664, 02775)
+
+        open('d', 'wb').write('new d\n')
+        t.add('d')
+        t.commit('new d')
+        check_mode_r(self, '.bzr', 0664, 02775)
+
+    def test_disable_set_mode(self):
+        # TODO: jam 20051215 Ultimately, this test should probably test that
+        #                    extra chmod calls aren't being made
+        import bzrlib.branch
+        try:
+            b = Branch.initialize(u'.')
+            self.assertNotEqual(None, b._dir_mode)
+            self.assertNotEqual(None, b._file_mode)
+
+            bzrlib.branch.BzrBranch._set_dir_mode = False
+            b = Branch.open(u'.')
+            self.assertEqual(None, b._dir_mode)
+            self.assertNotEqual(None, b._file_mode)
+
+            bzrlib.branch.BzrBranch._set_file_mode = False
+            b = Branch.open(u'.')
+            self.assertEqual(None, b._dir_mode)
+            self.assertEqual(None, b._file_mode)
+
+            bzrlib.branch.BzrBranch._set_dir_mode = True
+            b = Branch.open(u'.')
+            self.assertNotEqual(None, b._dir_mode)
+            self.assertEqual(None, b._file_mode)
+
+            bzrlib.branch.BzrBranch._set_file_mode = True
+            b = Branch.open(u'.')
+            self.assertNotEqual(None, b._dir_mode)
+            self.assertNotEqual(None, b._file_mode)
+        finally:
+            bzrlib.branch.BzrBranch._set_dir_mode = True
+            bzrlib.branch.BzrBranch._set_file_mode = True
+
+    def test_new_branch(self):
+        if sys.platform == 'win32':
+            raise TestSkipped('chmod has no effect on win32')
+
+        os.mkdir('a')
+        mode = stat.S_IMODE(os.stat('a').st_mode)
+        b = Branch.initialize('a')
+        assertEqualMode(self, mode, b._dir_mode)
+        assertEqualMode(self, mode & ~07111, b._file_mode)
+
+        os.mkdir('b')
+        os.chmod('b', 02777)
+        b = Branch.initialize('b')
+        assertEqualMode(self, 02777, b._dir_mode)
+        assertEqualMode(self, 00666, b._file_mode)
+        check_mode_r(self, 'b/.bzr', 00666, 02777)
+
+        os.mkdir('c')
+        os.chmod('c', 02750)
+        b = Branch.initialize('c')
+        assertEqualMode(self, 02750, b._dir_mode)
+        assertEqualMode(self, 00640, b._file_mode)
+        check_mode_r(self, 'c/.bzr', 00640, 02750)
+
+        os.mkdir('d')
+        os.chmod('d', 0700)
+        b = Branch.initialize('d')
+        assertEqualMode(self, 0700, b._dir_mode)
+        assertEqualMode(self, 0600, b._file_mode)
+        check_mode_r(self, 'd/.bzr', 00600, 0700)
+
+
+class TestSftpPermissions(TestCaseWithSFTPServer):
+
+    def test_new_files(self):
+        if sys.platform == 'win32':
+            raise TestSkipped('chmod has no effect on win32')
+        # Though it would be nice to test that SFTP to a server
+        # which does support chmod has the right effect
+
+        from bzrlib.transport.sftp import SFTPTransport
+
+        # We don't actually use it directly, we just want to
+        # keep the connection open, since StubSFTPServer only
+        # allows 1 connection
+        _transport = SFTPTransport(self._sftp_url)
+
+        os.mkdir('local')
+        b_local = Branch.initialize(u'local')
+        t_local = b_local.working_tree()
+        open('local/a', 'wb').write('foo\n')
+        t_local.add('a')
+        t_local.commit('foo')
+
+        # Delete them because we are modifying the filesystem underneath them
+        del b_local, t_local 
+        chmod_r('local/.bzr', 0644, 0755)
+        check_mode_r(self, 'local/.bzr', 0644, 0755)
+
+        b_local = Branch.open(u'local')
+        t_local = b_local.working_tree()
+        assertEqualMode(self, 0755, b_local._dir_mode)
+        assertEqualMode(self, 0644, b_local._file_mode)
+
+        os.mkdir('sftp')
+        sftp_url = self.get_remote_url('sftp')
+        b_sftp = Branch.initialize(sftp_url)
+
+        b_sftp.pull(b_local)
+        del b_sftp
+        chmod_r('sftp/.bzr', 0644, 0755)
+        check_mode_r(self, 'sftp/.bzr', 0644, 0755)
+
+        b_sftp = Branch.open(sftp_url)
+        assertEqualMode(self, 0755, b_sftp._dir_mode)
+        assertEqualMode(self, 0644, b_sftp._file_mode)
+
+        open('local/a', 'wb').write('foo2\n')
+        t_local.commit('foo2')
+        b_sftp.pull(b_local)
+        # The mode should be maintained after commit
+        check_mode_r(self, 'sftp/.bzr', 0644, 0755)
+
+        open('local/b', 'wb').write('new b\n')
+        t_local.add('b')
+        t_local.commit('new b')
+        b_sftp.pull(b_local)
+        check_mode_r(self, 'sftp/.bzr', 0644, 0755)
+
+        del b_sftp
+        # Recursively update the modes of all files
+        chmod_r('sftp/.bzr', 0664, 0775)
+        check_mode_r(self, 'sftp/.bzr', 0664, 0775)
+
+        b_sftp = Branch.open(sftp_url)
+        assertEqualMode(self, 0775, b_sftp._dir_mode)
+        assertEqualMode(self, 0664, b_sftp._file_mode)
+
+        open('local/a', 'wb').write('foo3\n')
+        t_local.commit('foo3')
+        b_sftp.pull(b_local)
+        check_mode_r(self, 'sftp/.bzr', 0664, 0775)
+
+        open('local/c', 'wb').write('new c\n')
+        t_local.add('c')
+        t_local.commit('new c')
+        b_sftp.pull(b_local)
+        check_mode_r(self, 'sftp/.bzr', 0664, 0775)
+
+    def test_sftp_server_modes(self):
+        if sys.platform == 'win32':
+            raise TestSkipped('chmod has no effect on win32')
+
+        umask = 0022
+        original_umask = os.umask(umask)
+
+        try:
+            from bzrlib.transport.sftp import SFTPTransport
+            t = SFTPTransport(self._sftp_url)
+            # Direct access should be masked by umask
+            t._sftp_open_exclusive('a', mode=0666).write('foo\n')
+            self.assertTransportMode(t, 'a', 0666 &~umask)
+
+            # but Transport overrides umask
+            t.put('b', 'txt', mode=0666)
+            self.assertTransportMode(t, 'b', 0666)
+
+            t._sftp.mkdir('c', mode=0777)
+            self.assertTransportMode(t, 'c', 0777 &~umask)
+
+            t.mkdir('d', mode=0777)
+            self.assertTransportMode(t, 'd', 0777)
+        finally:
+            os.umask(original_umask)
+
+

=== renamed file 'bzrlib/selftest/plugins.py' => 'bzrlib/tests/test_plugins.py'
--- bzrlib/selftest/plugins.py	2005-08-30 03:10:32 +0000
+++ bzrlib/tests/test_plugins.py	2005-12-16 03:43:14 +0000
@@ -17,56 +17,36 @@
 
 """Tests for plugins"""
 
-
-
-# **************************************************
-# NOT RUN YET
-# **************************************************
-
-
-
-
-
-
-
-
-from bzrlib.selftest import TestCaseInTempDir
-
+# XXX: There are no plugin tests at the moment because the plugin module
+# affects the global state of the process.  See bzrlib/plugins.py for more
+# comments.
+
+import os
+
+import bzrlib.plugin
+import bzrlib.plugins
+from bzrlib.tests import TestCaseInTempDir
+from bzrlib.osutils import pathjoin, abspath
 
 class PluginTest(TestCaseInTempDir):
     """Create an external plugin and test loading."""
-    def test_plugin_loading(self):
-        import os
-        
-        orig_help = self.backtick('bzr help commands') # No plugins yet
-        os.mkdir('plugin_test')
-        f = open(os.path.join('plugin_test', 'myplug.py'), 'wt')
-        f.write(PLUGIN_TEXT)
-        f.close()
-
-        newhelp = backtick('bzr help commands')
-        assert newhelp.startswith('You have been overridden\n')
-        # We added a line, but the rest should work
-        assert newhelp[25:] == help
-
-        assert backtick('bzr commit -m test') == "I'm sorry dave, you can't do that\n"
-
-        shutil.rmtree('plugin_test')
-
-
-
-
-#         PLUGIN_TEXT = \
-#         """import bzrlib, bzrlib.commands
-#         class cmd_myplug(bzrlib.commands.Command):
-#             '''Just a simple test plugin.'''
-#             aliases = ['mplg']
-#             def run(self):
-#                 print 'Hello from my plugin'
-#         """
-#         f.close()
-
-#         os.environ['BZRPLUGINPATH'] = os.path.abspath('plugin_test')
+#    def test_plugin_loading(self):
+#        orig_help = self.run_bzr_captured('bzr help commands')[0]
+#        os.mkdir('plugin_test')
+#        f = open(pathjoin('plugin_test', 'myplug.py'), 'wt')
+#        f.write(PLUGIN_TEXT)
+#        f.close()
+#        newhelp = self.run_bzr_captured('bzr help commands')[0]
+#        assert newhelp.startswith('You have been overridden\n')
+#        # We added a line, but the rest should work
+#        assert newhelp[25:] == help
+#
+#        assert backtick('bzr commit -m test') == "I'm sorry dave, you can't do that\n"
+#
+#        shutil.rmtree('plugin_test')
+#
+
+#         os.environ['BZRPLUGINPATH'] = abspath('plugin_test')
 #         help = backtick('bzr help commands')
 #         assert help.find('myplug') != -1
 #         assert help.find('Just a simple test plugin.') != -1
@@ -75,7 +55,7 @@
 #         assert backtick('bzr myplug') == 'Hello from my plugin\n'
 #         assert backtick('bzr mplg') == 'Hello from my plugin\n'
 
-#         f = open(os.path.join('plugin_test', 'override.py'), 'wb')
+#         f = open(pathjoin('plugin_test', 'override.py'), 'wb')
 #         f.write("""import bzrlib, bzrlib.commands
 #     class cmd_commit(bzrlib.commands.cmd_commit):
 #         '''Commit changes into a new revision.'''
@@ -89,3 +69,71 @@
 #             bzrlib.commands.cmd_help.run(self, *args, **kwargs)
 
 #         """
+
+PLUGIN_TEXT = """\
+import bzrlib.commands
+class cmd_myplug(bzrlib.commands.Command):
+    '''Just a simple test plugin.'''
+    aliases = ['mplg']
+    def run(self):
+        print 'Hello from my plugin'
+"""
+
+# TODO: Write a test for plugin decoration of commands.
+
+class TestOneNamedPluginOnly(TestCaseInTempDir):
+
+    activeattributes = {}
+
+    def test_plugins_with_the_same_name_are_not_loaded(self):
+        # This test tests that having two plugins in different
+        # directories does not result in both being loaded.
+        # get a file name we can use which is also a valid attribute
+        # for accessing in activeattributes. - we cannot give import parameters.
+        tempattribute = "0"
+        self.failIf(tempattribute in self.activeattributes)
+        # set a place for the plugins to record their loading, and at the same
+        # time validate that the location the plugins should record to is
+        # valid and correct.
+        bzrlib.tests.test_plugins.TestOneNamedPluginOnly.activeattributes \
+            [tempattribute] = []
+        self.failUnless(tempattribute in self.activeattributes)
+        # create two plugin directories
+        os.mkdir('first')
+        os.mkdir('second')
+        # write a plugin that will record when its loaded in the 
+        # tempattribute list.
+        template = ("from bzrlib.tests.test_plugins import TestOneNamedPluginOnly\n"
+                    "TestOneNamedPluginOnly.activeattributes[%r].append('%s')\n")
+        print >> file(os.path.join('first', 'plugin.py'), 'w'), template % (tempattribute, 'first')
+        print >> file(os.path.join('second', 'plugin.py'), 'w'), template % (tempattribute, 'second')
+        try:
+            bzrlib.plugin.load_from_dirs(['first', 'second'])
+            self.assertEqual(['first'], self.activeattributes[tempattribute])
+        finally:
+            # remove the plugin 'plugin'
+            del self.activeattributes[tempattribute]
+            if getattr(bzrlib.plugins, 'plugin', None):
+                del bzrlib.plugins.plugin
+        self.failIf(getattr(bzrlib.plugins, 'plugin', None))
+
+
+class TestAllPlugins(TestCaseInTempDir):
+
+    def test_plugin_appears_in_all_plugins(self):
+        # This test tests a new plugin appears in bzrlib.plugin.all_plugins().
+        # check the plugin is not loaded already
+        self.failIf(getattr(bzrlib.plugins, 'plugin', None))
+        # write a plugin that _cannot_ fail to load.
+        print >> file('plugin.py', 'w'), ""
+        try:
+            bzrlib.plugin.load_from_dirs(['.'])
+            self.failUnless('plugin' in bzrlib.plugin.all_plugins())
+            self.failUnless(getattr(bzrlib.plugins, 'plugin', None))
+            self.assertEqual(bzrlib.plugin.all_plugins()['plugin'],
+                             bzrlib.plugins.plugin)
+        finally:
+            # remove the plugin 'plugin'
+            if getattr(bzrlib.plugins, 'plugin', None):
+                del bzrlib.plugins.plugin
+        self.failIf(getattr(bzrlib.plugins, 'plugin', None))

=== added file 'bzrlib/tests/test_remove.py'
--- bzrlib/tests/test_remove.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_remove.py	2005-12-12 07:20:02 +0000
@@ -0,0 +1,12 @@
+from bzrlib.tests import TestCaseInTempDir
+from bzrlib.branch import Branch
+
+class TestRemove(TestCaseInTempDir):
+    def test_remove_verbose(self):
+        b = Branch.initialize(u'.')
+        self.build_tree(['hello'])
+        wt = b.working_tree() 
+        wt.add(['hello'])
+        wt.commit(message='add hello')
+        eq = self.assertEquals
+        wt.remove(['hello'], verbose=True)

=== renamed file 'bzrlib/selftest/testrevision.py' => 'bzrlib/tests/test_revision.py'
--- bzrlib/selftest/testrevision.py	2005-10-02 23:17:00 +0000
+++ bzrlib/tests/test_revision.py	2005-11-23 15:44:24 +0000
@@ -17,7 +17,7 @@
 
 import os
 
-from bzrlib.selftest import TestCaseInTempDir
+from bzrlib.tests import TestCaseInTempDir
 from bzrlib.branch import Branch
 from bzrlib.commit import commit
 from bzrlib.fetch import fetch
@@ -26,7 +26,8 @@
 from bzrlib.trace import mutter
 from bzrlib.errors import NoSuchRevision
 
-def make_branches():
+# XXX: Make this a method of a merge base case
+def make_branches(self):
     """Create two branches
 
     branch 1 has 6 commits, branch 2 has 3 commits
@@ -62,18 +63,18 @@
     revisions_2 = br2.revision_history()
     
     fetch(from_branch=br2, to_branch=br1)
-    br1.add_pending_merge(revisions_2[4])
-    assert revisions_2[4] == 'b@u-0-4'
+    br1.working_tree().add_pending_merge(revisions_2[4])
+    self.assertEquals(revisions_2[4], 'b@u-0-4')
     commit(br1, "Commit six", rev_id="a@u-0-3")
     commit(br1, "Commit seven", rev_id="a@u-0-4")
     commit(br2, "Commit eight", rev_id="b@u-0-5")
     
     fetch(from_branch=br2, to_branch=br1)
-    br1.add_pending_merge(br2.revision_history()[5])
+    br1.working_tree().add_pending_merge(br2.revision_history()[5])
     commit(br1, "Commit nine", rev_id="a@u-0-5")
     # DO NOT FETCH HERE - we WANT a GHOST.
     #fetch(from_branch=br1, to_branch=br2)
-    br2.add_pending_merge(br1.revision_history()[4])
+    br2.working_tree().add_pending_merge(br1.revision_history()[4])
     commit(br2, "Commit ten - ghost merge", rev_id="b@u-0-6")
     
     return br1, br2
@@ -82,7 +83,7 @@
 class TestIsAncestor(TestCaseInTempDir):
     def test_recorded_ancestry(self):
         """Test that commit records all ancestors"""
-        br1, br2 = make_branches()
+        br1, br2 = make_branches(self)
         d = [('a@u-0-0', ['a@u-0-0']),
              ('a@u-0-1', ['a@u-0-0', 'a@u-0-1']),
              ('a@u-0-2', ['a@u-0-0', 'a@u-0-1', 'a@u-0-2']),
@@ -118,22 +119,22 @@
     
     def test_is_ancestor(self):
         """Test checking whether a revision is an ancestor of another revision"""
-        br1, br2 = make_branches()
+        br1, br2 = make_branches(self)
         revisions = br1.revision_history()
         revisions_2 = br2.revision_history()
         sources = br1
 
-        assert is_ancestor(revisions[0], revisions[0], br1)
-        assert is_ancestor(revisions[1], revisions[0], sources)
-        assert not is_ancestor(revisions[0], revisions[1], sources)
-        assert is_ancestor(revisions_2[3], revisions[0], sources)
+        self.assert_(is_ancestor(revisions[0], revisions[0], br1))
+        self.assert_(is_ancestor(revisions[1], revisions[0], sources))
+        self.assert_(not is_ancestor(revisions[0], revisions[1], sources))
+        self.assert_(is_ancestor(revisions_2[3], revisions[0], sources))
         # disabled mbp 20050914, doesn't seem to happen anymore
         ## self.assertRaises(NoSuchRevision, is_ancestor, revisions_2[3],
         ##                  revisions[0], br1)        
-        assert is_ancestor(revisions[3], revisions_2[4], sources)
-        assert is_ancestor(revisions[3], revisions_2[4], br1)
-        assert is_ancestor(revisions[3], revisions_2[3], sources)
-        ## assert not is_ancestor(revisions[3], revisions_2[3], br1)
+        self.assert_(is_ancestor(revisions[3], revisions_2[4], sources))
+        self.assert_(is_ancestor(revisions[3], revisions_2[4], br1))
+        self.assert_(is_ancestor(revisions[3], revisions_2[3], sources))
+        ## self.assert_(not is_ancestor(revisions[3], revisions_2[3], br1))
 
 
 class TestIntermediateRevisions(TestCaseInTempDir):
@@ -141,19 +142,19 @@
     def setUp(self):
         from bzrlib.commit import commit
         TestCaseInTempDir.setUp(self)
-        self.br1, self.br2 = make_branches()
+        self.br1, self.br2 = make_branches(self)
 
-        self.br2.commit("Commit eleven", rev_id="b@u-0-7")
-        self.br2.commit("Commit twelve", rev_id="b@u-0-8")
-        self.br2.commit("Commit thirtteen", rev_id="b@u-0-9")
+        self.br2.working_tree().commit("Commit eleven", rev_id="b@u-0-7")
+        self.br2.working_tree().commit("Commit twelve", rev_id="b@u-0-8")
+        self.br2.working_tree().commit("Commit thirtteen", rev_id="b@u-0-9")
 
         fetch(from_branch=self.br2, to_branch=self.br1)
-        self.br1.add_pending_merge(self.br2.revision_history()[6])
-        self.br1.commit("Commit fourtten", rev_id="a@u-0-6")
+        self.br1.working_tree().add_pending_merge(self.br2.revision_history()[6])
+        self.br1.working_tree().commit("Commit fourtten", rev_id="a@u-0-6")
 
         fetch(from_branch=self.br1, to_branch=self.br2)
-        self.br2.add_pending_merge(self.br1.revision_history()[6])
-        self.br2.commit("Commit fifteen", rev_id="b@u-0-10")
+        self.br2.working_tree().add_pending_merge(self.br1.revision_history()[6])
+        self.br2.working_tree().commit("Commit fifteen", rev_id="b@u-0-10")
 
         from bzrlib.revision import MultipleRevisionSources
         self.sources = MultipleRevisionSources(self.br1, self.br2)
@@ -166,7 +167,7 @@
     def test_intervene(self):
         """Find intermediate revisions, without requiring history"""
         from bzrlib.errors import NotAncestor, NoSuchRevision
-        assert len(self.intervene('a@u-0-0', 'a@u-0-0')) == 0
+        self.assertEquals(len(self.intervene('a@u-0-0', 'a@u-0-0')), 0)
         self.assertEqual(self.intervene('a@u-0-0', 'a@u-0-1'), ['a@u-0-1'])
         self.assertEqual(self.intervene('a@u-0-0', 'a@u-0-2'), 
                          ['a@u-0-1', 'a@u-0-2'])
@@ -181,15 +182,13 @@
                                         self.br1.revision_history()), 
                          ['a@u-0-1', 'a@u-0-2', 'a@u-0-3', 'a@u-0-4', 
                           'a@u-0-5'])
-        print ("testrevision.py 191 - intervene appears to return b..6 even"
-               "though it is not reachable!")
-#        self.assertEqual(self.intervene('a@u-0-0', 'b@u-0-6', 
-#                         self.br1.revision_history()), 
-#                         ['a@u-0-1', 'a@u-0-2', 'a@u-0-3', 'a@u-0-4', 
-#                          'b@u-0-6'])
-#        self.assertEqual(self.intervene('a@u-0-0', 'b@u-0-5'), 
-#                         ['a@u-0-1', 'a@u-0-2', 'b@u-0-3', 'b@u-0-4', 
-#                          'b@u-0-5'])
+        self.assertEqual(self.intervene('a@u-0-0', 'b@u-0-6', 
+                         self.br1.revision_history()), 
+                         ['a@u-0-1', 'a@u-0-2', 'a@u-0-3', 'a@u-0-4', 
+                          'b@u-0-6'])
+        self.assertEqual(self.intervene('a@u-0-0', 'b@u-0-5'), 
+                         ['a@u-0-1', 'a@u-0-2', 'b@u-0-3', 'b@u-0-4', 
+                          'b@u-0-5'])
         self.assertEqual(self.intervene('b@u-0-3', 'b@u-0-6', 
                          self.br2.revision_history()), 
                          ['b@u-0-4', 'b@u-0-5', 'b@u-0-6'])
@@ -212,7 +211,7 @@
     def test_old_common_ancestor(self):
         """Pick a resonable merge base using the old functionality"""
         from bzrlib.revision import old_common_ancestor as common_ancestor
-        br1, br2 = make_branches()
+        br1, br2 = make_branches(self)
         revisions = br1.revision_history()
         revisions_2 = br2.revision_history()
         sources = br1
@@ -224,7 +223,7 @@
                                    revisions_2[3]:(4, 2),
                                    revisions[0]:(5, 3) }
         ancestors_list = find_present_ancestors(revisions[3], sources)
-        assert len(expected_ancestors_list) == len(ancestors_list)
+        self.assertEquals(len(expected_ancestors_list), len(ancestors_list))
         for key, value in expected_ancestors_list.iteritems():
             self.assertEqual(ancestors_list[key], value, 
                               "key %r, %r != %r" % (key, ancestors_list[key],
@@ -251,7 +250,7 @@
     def test_common_ancestor(self):
         """Pick a reasonable merge base"""
         from bzrlib.revision import common_ancestor
-        br1, br2 = make_branches()
+        br1, br2 = make_branches(self)
         revisions = br1.revision_history()
         revisions_2 = br2.revision_history()
         sources = MultipleRevisionSources(br1, br2)
@@ -262,7 +261,7 @@
                                    revisions_2[3]:(4, 2),
                                    revisions[0]:(5, 3) }
         ancestors_list = find_present_ancestors(revisions[3], sources)
-        assert len(expected_ancestors_list) == len(ancestors_list)
+        self.assertEquals(len(expected_ancestors_list), len(ancestors_list))
         for key, value in expected_ancestors_list.iteritems():
             self.assertEqual(ancestors_list[key], value, 
                               "key %r, %r != %r" % (key, ancestors_list[key],
@@ -288,13 +287,13 @@
         """combined_graph
         Ensure it's not order-sensitive
         """
-        br1, br2 = make_branches()
+        br1, br2 = make_branches(self)
         source = MultipleRevisionSources(br1, br2)
         combined_1 = combined_graph(br1.last_revision(), 
                                     br2.last_revision(), source)
         combined_2 = combined_graph(br2.last_revision(),
                                     br1.last_revision(), source)
-        assert combined_1[1] == combined_2[1]
-        assert combined_1[2] == combined_2[2]
-        assert combined_1[3] == combined_2[3]
-        assert combined_1 == combined_2
+        self.assertEquals(combined_1[1], combined_2[1])
+        self.assertEquals(combined_1[2], combined_2[2])
+        self.assertEquals(combined_1[3], combined_2[3])
+        self.assertEquals(combined_1, combined_2)

=== renamed file 'bzrlib/selftest/testrevisionnamespaces.py' => 'bzrlib/tests/test_revisionnamespaces.py'
--- bzrlib/selftest/testrevisionnamespaces.py	2005-09-29 20:34:25 +0000
+++ bzrlib/tests/test_revisionnamespaces.py	2005-12-30 20:54:47 +0000
@@ -16,10 +16,14 @@
 
 import os
 import time
-from bzrlib.selftest import TestCaseInTempDir
+
+from bzrlib.branch import Branch
+from bzrlib.tests import TestCaseInTempDir
 from bzrlib.errors import NoCommonAncestor, NoCommits
+from bzrlib.errors import NoSuchRevision
 from bzrlib.clone import copy_branch
 from bzrlib.merge import merge
+from bzrlib.revisionspec import RevisionSpec
 
 class TestRevisionNamespaces(TestCaseInTempDir):
 
@@ -27,15 +31,12 @@
         """Test revision specifiers.
 
         These identify revisions by date, etc."""
-        from bzrlib.errors import NoSuchRevision
-        from bzrlib.branch import Branch
-        from bzrlib.revisionspec import RevisionSpec
-
-        b = Branch.initialize('.')
-
-        b.commit('Commit one', rev_id='a@r-0-1', timestamp=time.time() - 60*60*24)
-        b.commit('Commit two', rev_id='a@r-0-2')
-        b.commit('Commit three', rev_id='a@r-0-3')
+
+        b = Branch.initialize(u'.')
+
+        b.working_tree().commit('Commit one', rev_id='a@r-0-1', timestamp=time.time() - 60*60*24)
+        b.working_tree().commit('Commit two', rev_id='a@r-0-2')
+        b.working_tree().commit('Commit three', rev_id='a@r-0-3')
 
         self.assertEquals(RevisionSpec(None).in_history(b), (0, None))
         self.assertEquals(RevisionSpec(1).in_history(b), (1, 'a@r-0-1'))
@@ -69,12 +70,31 @@
 
         os.mkdir('copy')
         b3 = copy_branch(b, 'copy')
-        b3.commit('Commit four', rev_id='b@r-0-4')
+        b3.working_tree().commit('Commit four', rev_id='b@r-0-4')
         self.assertEquals(RevisionSpec('ancestor:.').in_history(b3).rev_id,
                           'a@r-0-3')
         merge(['copy', -1], [None, None])
-        b.commit('Commit five', rev_id='a@r-0-4')
+        b.working_tree().commit('Commit five', rev_id='a@r-0-4')
         self.assertEquals(RevisionSpec('ancestor:copy').in_history(b).rev_id,
                           'b@r-0-4')
         self.assertEquals(RevisionSpec('ancestor:.').in_history(b3).rev_id,
                           'b@r-0-4')
+
+        # This should be in the revision store, but not in revision-history
+        self.assertEquals((None, 'b@r-0-4'),
+                RevisionSpec('revid:b@r-0-4').in_history(b))
+
+    def test_branch_namespace(self):
+        """Ensure that the branch namespace pulls in the requisite content."""
+        self.build_tree(['branch1/', 'branch1/file', 'branch2/'])
+        branch = Branch.initialize('branch1')
+        branch.working_tree().add(['file'])
+        branch.working_tree().commit('add file')
+        copy_branch(branch, 'branch2')
+        print >> open('branch2/file', 'w'), 'new content'
+        branch2 = Branch.open('branch2')
+        branch2.working_tree().commit('update file', rev_id='A')
+        spec = RevisionSpec('branch:./branch2/.bzr/../')
+        rev_info = spec.in_history(branch)
+        self.assertEqual(rev_info, (None, 'A'))
+

=== added file 'bzrlib/tests/test_revprops.py'
--- bzrlib/tests/test_revprops.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_revprops.py	2005-11-30 00:25:50 +0000
@@ -0,0 +1,37 @@
+# (C) 2005 Canonical
+
+"""Tests for revision properties."""
+
+from bzrlib.branch import Branch
+from bzrlib.tests import TestCaseInTempDir
+
+class TestRevProps(TestCaseInTempDir):
+    def test_simple_revprops(self):
+        """Simple revision properties"""
+        b = Branch.initialize(u'.')
+        b.nick = 'Nicholas'
+        props = dict(flavor='choc-mint', 
+                     condiment='orange\n  mint\n\tcandy')
+        b.working_tree().commit(message='initial null commit', 
+                 revprops=props,
+                 allow_pointless=True,
+                 rev_id='test@user-1')
+        rev = b.get_revision('test@user-1')
+        self.assertTrue('flavor' in rev.properties)
+        self.assertEquals(rev.properties['flavor'], 'choc-mint')
+        self.assertEquals(sorted(rev.properties.items()),
+                          [('branch-nick', 'Nicholas'), 
+                           ('condiment', 'orange\n  mint\n\tcandy'),
+                           ('flavor', 'choc-mint')])
+
+    def test_invalid_revprops(self):
+        """Invalid revision properties"""
+        b = Branch.initialize(u'.')
+        self.assertRaises(ValueError,
+                          b.working_tree().commit, 
+                          message='invalid',
+                          revprops={'what a silly property': 'fine'})
+        self.assertRaises(ValueError,
+                          b.working_tree().commit, 
+                          message='invalid',
+                          revprops=dict(number=13))

=== added file 'bzrlib/tests/test_reweave.py'
--- bzrlib/tests/test_reweave.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_reweave.py	2005-11-23 15:44:24 +0000
@@ -0,0 +1,152 @@
+# Copyright (C) 2005 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Test reweave code.
+
+Reweave takes two weaves containing a partial view of history and combines
+them into a single weave containing all the information.  This can include 
+
+ - versions recorded in only one file
+
+ - versions with different (but not contradictory) lists of parent 
+   revisions
+
+It is an error if either of these conditions occur:
+
+ - contradictory ancestry graphs, e.g.
+   - v1 is an ancestor of v2 in one weave, and vice versa in the other
+   - different text for any version 
+"""
+
+import os
+import sys
+
+from bzrlib.tests import TestCase
+from bzrlib.weave import Weave, reweave
+from bzrlib.errors import WeaveParentMismatch
+
+class TestReweave(TestCase):
+
+    def test_reweave_add_parents(self):
+        """Reweave inserting new parents
+        
+        The new version must have the right parent list and must identify
+        lines originating in another parent.
+        """
+        w1 = Weave('w1')
+        w2 = Weave('w2')
+        w1.add('v-1', [], ['line 1\n'])
+        w2.add('v-2', [], ['line 2\n'])
+        w1.add('v-3', ['v-1'], ['line 1\n'])
+        w2.add('v-3', ['v-2'], ['line 1\n'])
+        w3 = reweave(w1, w2)
+        self.assertEqual(sorted(w3.names()),
+                         'v-1 v-2 v-3'.split())
+        self.assertEqualDiff(w3.get_text('v-3'),
+                'line 1\n')
+        self.assertEqual(sorted(w3.parent_names('v-3')),
+                ['v-1', 'v-2'])
+        ann = list(w3.annotate('v-3'))
+        self.assertEqual(len(ann), 1)
+        self.assertEqual(w3.idx_to_name(ann[0][0]), 'v-1')
+        self.assertEqual(ann[0][1], 'line 1\n')
+        
+    def build_weave1(self):
+        weave1 = Weave()
+        self.lines1 = ['hello\n']
+        self.lines3 = ['hello\n', 'cruel\n', 'world\n']
+        weave1.add('v1', [], self.lines1)
+        weave1.add('v2', [0], ['hello\n', 'world\n'])
+        weave1.add('v3', [1], self.lines3)
+        return weave1
+        
+    def test_reweave_with_empty(self):
+        """Reweave adding empty weave"""
+        wb = Weave()
+        w1 = self.build_weave1()
+        wr = reweave(w1, wb)
+        eq = self.assertEquals
+        eq(sorted(wr.iter_names()), ['v1', 'v2', 'v3'])
+        eq(wr.get_lines('v3'), ['hello\n', 'cruel\n', 'world\n'])
+        self.assertEquals(wr, w1)
+
+    def test_join_with_ghosts_raises_parent_mismatch(self):
+        """Join weave traps parent mismatch"""
+        wa = self.build_weave1()
+        wb = Weave()
+        wb.add('x1', [], ['line from x1\n'])
+        wb.add('v1', [], ['hello\n'])
+        wb.add('v2', ['v1', 'x1'], ['hello\n', 'world\n'])
+        self.assertRaises(WeaveParentMismatch, wa.join, wb)
+
+    def test_reweave_with_ghosts(self):
+        """Join that inserts parents of an existing revision.
+
+        This can happen when merging from another branch who
+        knows about revisions the destination does not.  In 
+        this test the second weave knows of an additional parent of 
+        v2.  Any revisions which are in common still have to have the 
+        same text.
+        """
+        w1 = self.build_weave1()
+        wa = w1.copy()
+        wb = Weave()
+        wb.add('x1', [], ['line from x1\n'])
+        wb.add('v1', [], ['hello\n'])
+        wb.add('v2', ['v1', 'x1'], ['hello\n', 'world\n'])
+        wc = reweave(wa, wb)
+        eq = self.assertEquals
+        eq(sorted(wc.iter_names()), ['v1', 'v2', 'v3', 'x1',])
+        eq(wc.get_text('x1'), 'line from x1\n')
+        eq(wc.get_lines('v2'), ['hello\n', 'world\n'])
+        eq(wc.parent_names('v2'), ['v1', 'x1'])
+        w1.reweave(wb)
+        self.assertEquals(wc, w1)
+
+    def build_empty_weave(self, *pattern):
+        w = Weave()
+        for version, parents in pattern:
+            w.add(version, parents, [])
+        return w
+
+    def test_reweave_reorder(self):
+        """Reweave requiring reordering of versions.
+
+        Weaves must be stored such that parents come before children.  When
+        reweaving, we may add new parents to some children, but it is required
+        that there must be *some* valid order that can be found, otherwise the
+        ancestries are contradictory.  (For the specific case of inserting
+        ghost revisions there will be no disagreement, only partial knowledge
+        of the history.)
+
+        Note that the weaves are only partially ordered: when there are two
+        versions where neither is an ancestor of the other the order in which
+        they occur is unconstrained.  When we reweave those versions into
+        another weave, they may become more constrained and it may be
+        necessary to change their order.
+
+        One simple case of this is 
+
+        w1: (c[], a[], b[a])
+        w2: (b[], c[b], a[])
+        
+        We need to recognize that the final weave must show the ordering
+        a[], b[a], c[b].  The version that must be first in the result is 
+        not first in either of the input weaves.
+        """
+        w1 = self.build_empty_weave(('c', []), ('a', []), ('b', ['a']))
+        w2 = self.build_empty_weave(('b', []), ('c', ['b']), ('a', []))
+        w3 = reweave(w1, w2)

=== added file 'bzrlib/tests/test_rio.py'
--- bzrlib/tests/test_rio.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_rio.py	2005-11-28 08:35:56 +0000
@@ -0,0 +1,282 @@
+# Copyright (C) 2005 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for rio serialization
+
+A simple, reproducible structured IO format.
+
+rio itself works in Unicode strings.  It is typically encoded to UTF-8,
+but this depends on the transport.
+"""
+
+import os
+import sys
+from tempfile import TemporaryFile
+
+from bzrlib.tests import TestCaseInTempDir, TestCase
+from bzrlib.rio import RioWriter, Stanza, read_stanza, read_stanzas
+
+
+class TestRio(TestCase):
+
+    def test_stanza(self):
+        """Construct rio stanza in memory"""
+        s = Stanza(number='42', name="fred")
+        self.assertTrue('number' in s)
+        self.assertFalse('color' in s)
+        self.assertFalse('42' in s)
+        self.assertEquals(list(s.iter_pairs()),
+                [('name', 'fred'), ('number', '42')])
+        self.assertEquals(s.get('number'), '42')
+        self.assertEquals(s.get('name'), 'fred')
+
+    def test_value_checks(self):
+        """rio checks types on construction"""
+        # these aren't enforced at construction time
+        ## self.assertRaises(ValueError,
+        ##        Stanza, complex=42 + 3j)
+        ## self.assertRaises(ValueError, 
+        ##        Stanza, several=range(10))
+
+    def test_empty_value(self):
+        """Serialize stanza with empty field"""
+        s = Stanza(empty='')
+        self.assertEqualDiff(s.to_string(),
+                "empty: \n")
+
+    def test_to_lines(self):
+        """Write simple rio stanza to string"""
+        s = Stanza(number='42', name='fred')
+        self.assertEquals(list(s.to_lines()),
+                ['name: fred\n',
+                 'number: 42\n'])
+
+    def test_to_file(self):
+        """Write rio to file"""
+        tmpf = TemporaryFile()
+        s = Stanza(a_thing='something with "quotes like \\"this\\""', number='42', name='fred')
+        s.write(tmpf)
+        tmpf.seek(0)
+        self.assertEqualDiff(tmpf.read(), r'''
+a_thing: something with "quotes like \"this\""
+name: fred
+number: 42
+'''[1:])
+
+    def test_multiline_string(self):
+        tmpf = TemporaryFile()
+        s = Stanza(motto="war is peace\nfreedom is slavery\nignorance is strength")
+        s.write(tmpf)
+        tmpf.seek(0)
+        self.assertEqualDiff(tmpf.read(), '''\
+motto: war is peace
+\tfreedom is slavery
+\tignorance is strength
+''')
+        tmpf.seek(0)
+        s2 = read_stanza(tmpf)
+        self.assertEquals(s, s2)
+
+    def test_read_stanza(self):
+        """Load stanza from string"""
+        lines = """\
+revision: mbp@sourcefrog.net-123-abc
+timestamp: 1130653962
+timezone: 36000
+committer: Martin Pool <mbp@test.sourcefrog.net>
+""".splitlines(True)
+        s = read_stanza(lines)
+        self.assertTrue('revision' in s)
+        self.assertEqualDiff(s.get('revision'), 'mbp@sourcefrog.net-123-abc')
+        self.assertEquals(list(s.iter_pairs()),
+                [('revision', 'mbp@sourcefrog.net-123-abc'),
+                 ('timestamp', '1130653962'),
+                 ('timezone', '36000'),
+                 ('committer', "Martin Pool <mbp@test.sourcefrog.net>")])
+        self.assertEquals(len(s), 4)
+
+    def test_repeated_field(self):
+        """Repeated field in rio"""
+        s = Stanza()
+        for k, v in [('a', '10'), ('b', '20'), ('a', '100'), ('b', '200'), 
+                     ('a', '1000'), ('b', '2000')]:
+            s.add(k, v)
+        s2 = read_stanza(s.to_lines())
+        self.assertEquals(s, s2)
+        self.assertEquals(s.get_all('a'), map(str, [10, 100, 1000]))
+        self.assertEquals(s.get_all('b'), map(str, [20, 200, 2000]))
+
+    def test_backslash(self):
+        s = Stanza(q='\\')
+        t = s.to_string()
+        self.assertEqualDiff(t, 'q: \\\n')
+        s2 = read_stanza(s.to_lines())
+        self.assertEquals(s, s2)
+
+    def test_blank_line(self):
+        s = Stanza(none='', one='\n', two='\n\n')
+        self.assertEqualDiff(s.to_string(), """\
+none: 
+one: 
+\t
+two: 
+\t
+\t
+""")
+        s2 = read_stanza(s.to_lines())
+        self.assertEquals(s, s2)
+
+    def test_whitespace_value(self):
+        s = Stanza(space=' ', tabs='\t\t\t', combo='\n\t\t\n')
+        self.assertEqualDiff(s.to_string(), """\
+combo: 
+\t\t\t
+\t
+space:  
+tabs: \t\t\t
+""")
+        s2 = read_stanza(s.to_lines())
+        self.assertEquals(s, s2)
+
+    def test_quoted(self):
+        """rio quoted string cases"""
+        s = Stanza(q1='"hello"', 
+                   q2=' "for', 
+                   q3='\n\n"for"\n',
+                   q4='for\n"\nfor',
+                   q5='\n',
+                   q6='"', 
+                   q7='""',
+                   q8='\\',
+                   q9='\\"\\"',
+                   )
+        s2 = read_stanza(s.to_lines())
+        self.assertEquals(s, s2)
+
+    def test_read_empty(self):
+        """Detect end of rio file"""
+        s = read_stanza([])
+        self.assertEqual(s, None)
+        self.assertTrue(s is None)
+        
+    def test_read_iter(self):
+        """Read several stanzas from file"""
+        tmpf = TemporaryFile()
+        tmpf.write("""\
+version_header: 1
+
+name: foo
+val: 123
+
+name: bar
+val: 129319
+""")
+        tmpf.seek(0)
+        reader = read_stanzas(tmpf)
+        read_iter = iter(reader)
+        stuff = list(reader)
+        self.assertEqual(stuff, 
+                [ Stanza(version_header='1'),
+                  Stanza(name="foo", val='123'),
+                  Stanza(name="bar", val='129319'), ])
+
+    def test_read_several(self):
+        """Read several stanzas from file"""
+        tmpf = TemporaryFile()
+        tmpf.write("""\
+version_header: 1
+
+name: foo
+val: 123
+
+name: quoted
+address:   "Willowglen"
+\t  42 Wallaby Way
+\t  Sydney
+
+name: bar
+val: 129319
+""")
+        tmpf.seek(0)
+        s = read_stanza(tmpf)
+        self.assertEquals(s, Stanza(version_header='1'))
+        s = read_stanza(tmpf)
+        self.assertEquals(s, Stanza(name="foo", val='123'))
+        s = read_stanza(tmpf)
+        self.assertEqualDiff(s.get('name'), 'quoted')
+        self.assertEqualDiff(s.get('address'), '  "Willowglen"\n  42 Wallaby Way\n  Sydney')
+        s = read_stanza(tmpf)
+        self.assertEquals(s, Stanza(name="bar", val='129319'))
+        s = read_stanza(tmpf)
+        self.assertEquals(s, None)
+
+    def test_tricky_quoted(self):
+        tmpf = TemporaryFile()
+        tmpf.write('''\
+s: "one"
+
+s: 
+\t"one"
+\t
+
+s: "
+
+s: ""
+
+s: """
+
+s: 
+\t
+
+s: \\
+
+s: 
+\t\\
+\t\\\\
+\t
+
+s: word\\
+
+s: quote"
+
+s: backslashes\\\\\\
+
+s: both\\\"
+
+''')
+        tmpf.seek(0)
+        expected_vals = ['"one"',
+            '\n"one"\n',
+            '"',
+            '""',
+            '"""',
+            '\n',
+            '\\',
+            '\n\\\n\\\\\n',
+            'word\\',
+            'quote\"',
+            'backslashes\\\\\\',
+            'both\\\"',
+            ]
+        for expected in expected_vals:
+            stanza = read_stanza(tmpf)
+            self.assertEquals(len(stanza), 1)
+            self.assertEqualDiff(stanza.get('s'), expected)
+
+    def test_write_empty_stanza(self):
+        """Write empty stanza"""
+        l = list(Stanza().to_lines())
+        self.assertEquals(l, [])

=== renamed file 'bzrlib/selftest/testsampler.py' => 'bzrlib/tests/test_sampler.py'
--- bzrlib/selftest/testsampler.py	2005-10-06 11:40:12 +0000
+++ bzrlib/tests/test_sampler.py	2006-01-05 10:49:26 +0000
@@ -1,4 +1,4 @@
-# Copyright (C) 2005 by Canonical Ltd
+# Copyright (C) 2006 by Canonical Ltd
 #   Authors: Robert Collins <robert.collins@canonical.com>
 #
 # This program is free software; you can redistribute it and/or modify
@@ -20,7 +20,7 @@
 If you need more than one line, make the first line a good sentence on its
 own and add more explanation here, like this.
 
-Be sure to register your new test script in bzrlib/selftest/__init__.py -
+Be sure to register your new test script in bzrlib/tests/__init__.py -
 search for sampler in there.
 """
 
@@ -29,7 +29,7 @@
 import sys
 
 #import bzrlib specific imports here
-from bzrlib.selftest import TestCaseInTempDir
+from bzrlib.tests import TestCaseInTempDir
 
 
 # Now we need a test script:

=== added file 'bzrlib/tests/test_selftest.py'
--- bzrlib/tests/test_selftest.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_selftest.py	2006-01-12 01:02:32 +0000
@@ -0,0 +1,202 @@
+# Copyright (C) 2005 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License version 2 as published by
+# the Free Software Foundation.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for the test framework
+"""
+
+import os
+import sys
+import unittest
+
+from bzrlib.tests import (
+                          _load_module_by_name,
+                          TestCase,
+                          TestCaseInTempDir,
+                          TestSkipped,
+                          TextTestRunner,
+                          )
+
+
+class SelftestTests(TestCase):
+
+    def test_import_tests(self):
+        mod = _load_module_by_name('bzrlib.tests.test_selftest')
+        self.assertEqual(mod.SelftestTests, SelftestTests)
+
+    def test_import_test_failure(self):
+        self.assertRaises(ImportError,
+                          _load_module_by_name,
+                          'bzrlib.no-name-yet')
+
+
+class MetaTestLog(TestCase):
+
+    def test_logging(self):
+        """Test logs are captured when a test fails."""
+        self.log('a test message')
+        self._log_file.flush()
+        self.assertContainsRe(self._get_log(), 'a test message\n')
+
+
+class TestTreeShape(TestCaseInTempDir):
+
+    def test_unicode_paths(self):
+        filename = u'hell\u00d8'
+        try:
+            self.build_tree_contents([(filename, 'contents of hello')])
+        except UnicodeEncodeError:
+            raise TestSkipped("can't build unicode working tree in "
+                "filesystem encoding %s" % sys.getfilesystemencoding())
+        self.failUnlessExists(filename)
+
+
+class TestSkippedTest(TestCase):
+    """Try running a test which is skipped, make sure it's reported properly."""
+
+    def test_skipped_test(self):
+        # must be hidden in here so it's not run as a real test
+        def skipping_test():
+            raise TestSkipped('test intentionally skipped')
+        runner = TextTestRunner(stream=self._log_file)
+        test = unittest.FunctionTestCase(skipping_test)
+        result = runner.run(test)
+        self.assertTrue(result.wasSuccessful())
+
+
+class TestTransportProviderAdapter(TestCase):
+    """A group of tests that test the transport implementation adaption core.
+
+    This will be generalised in the future which is why it is in this 
+    test file even though it is specific to transport tests at the moment.
+    """
+
+    def test_get_transport_permutations(self):
+        # this checks that we the module get_test_permutations call
+        # is made by the adapter get_transport_test_permitations method.
+        class MockModule(object):
+            def get_test_permutations(self):
+                return sample_permutation
+        sample_permutation = [(1,2), (3,4)]
+        from bzrlib.transport import TransportTestProviderAdapter
+        adapter = TransportTestProviderAdapter()
+        self.assertEqual(sample_permutation,
+                         adapter.get_transport_test_permutations(MockModule()))
+
+    def test_adapter_checks_all_modules(self):
+        # this checks that the adapter returns as many permurtations as
+        # there are in all the registered# transport modules for there
+        # - we assume if this matches its probably doing the right thing
+        # especially in combination with the tests for setting the right
+        # classes below.
+        from bzrlib.transport import (TransportTestProviderAdapter,
+                                      _get_transport_modules
+                                      )
+        modules = _get_transport_modules()
+        permutation_count = 0
+        for module in modules:
+            permutation_count += len(reduce(getattr, 
+                (module + ".get_test_permutations").split('.')[1:],
+                 __import__(module))())
+        input_test = TestTransportProviderAdapter(
+            "test_adapter_sets_transport_class")
+        adapter = TransportTestProviderAdapter()
+        self.assertEqual(permutation_count,
+                         len(list(iter(adapter.adapt(input_test)))))
+
+    def test_adapter_sets_transport_class(self):
+        # when the adapter adapts a test it needs to 
+        # place one of the permutations from the transport
+        # providers in each test case copy. This checks
+        # that it does not just use the same one all the time.
+        # and that the id is set correctly so that debugging is
+        # easy.
+        from bzrlib.transport.local import (LocalTransport,
+                                            LocalRelpathServer,
+                                            LocalAbspathServer,
+                                            LocalURLServer
+                                            )
+        from bzrlib.transport.sftp import (SFTPTransport,
+                                           SFTPAbsoluteServer,
+                                           SFTPHomeDirServer,
+                                           SFTPSiblingAbsoluteServer,
+                                           )
+        from bzrlib.transport.http import (HttpTransport,
+                                           HttpServer
+                                           )
+        from bzrlib.transport.ftp import FtpTransport
+        from bzrlib.transport.memory import (MemoryTransport,
+                                             MemoryServer
+                                             )
+        from bzrlib.transport import TransportTestProviderAdapter
+        # FIXME. What we want is a factory for the things
+        # needed to test the implementation. I.e. for transport we want:
+        # the class that connections should get; a local server factory
+        # so we would want the following permutations:
+        # LocalTransport relpath-factory
+        # LocalTransport abspath-factory
+        # LocalTransport file://-factory
+        # SFTPTransport homedir-factory
+        # SFTPTransport abssolute-factory
+        # HTTPTransport http-factory
+        # HTTPTransport https-factory
+        # etc, but we are currently lacking in this, so print out that
+        # this should be fixed.
+        input_test = TestTransportProviderAdapter(
+            "test_adapter_sets_transport_class")
+        suite = TransportTestProviderAdapter().adapt(input_test)
+        test_iter = iter(suite)
+        http_test = test_iter.next()
+        local_relpath_test = test_iter.next()
+        local_abspath_test = test_iter.next()
+        local_urlpath_test = test_iter.next()
+        memory_test = test_iter.next()
+        sftp_abs_test = test_iter.next()
+        sftp_homedir_test = test_iter.next()
+        sftp_sibling_abs_test = test_iter.next()
+        # ftp_test = test_iter.next()
+        self.assertRaises(StopIteration, test_iter.next)
+        self.assertEqual(LocalTransport, local_relpath_test.transport_class)
+        self.assertEqual(LocalRelpathServer, local_relpath_test.transport_server)
+        
+        self.assertEqual(LocalTransport, local_abspath_test.transport_class)
+        self.assertEqual(LocalAbspathServer, local_abspath_test.transport_server)
+
+        self.assertEqual(LocalTransport, local_urlpath_test.transport_class)
+        self.assertEqual(LocalURLServer, local_urlpath_test.transport_server)
+
+        self.assertEqual(SFTPTransport, sftp_abs_test.transport_class)
+        self.assertEqual(SFTPAbsoluteServer, sftp_abs_test.transport_server)
+        self.assertEqual(SFTPTransport, sftp_homedir_test.transport_class)
+        self.assertEqual(SFTPHomeDirServer, sftp_homedir_test.transport_server)
+        self.assertEqual(SFTPTransport, sftp_sibling_abs_test.transport_class)
+        self.assertEqual(SFTPSiblingAbsoluteServer,
+                         sftp_sibling_abs_test.transport_server)
+
+        self.assertEqual(HttpTransport, http_test.transport_class)
+        self.assertEqual(HttpServer, http_test.transport_server)
+        # self.assertEqual(FtpTransport, ftp_test.transport_class)
+
+        self.assertEqual(MemoryTransport, memory_test.transport_class)
+        self.assertEqual(MemoryServer, memory_test.transport_server)
+        
+        # we could test all of them for .id, but two is probably sufficient.
+        self.assertEqual("bzrlib.tests.test_selftest."
+                         "TestTransportProviderAdapter."
+                         "test_adapter_sets_transport_class(MemoryServer)",
+                         memory_test.id())
+        self.assertEqual("bzrlib.tests.test_selftest."
+                         "TestTransportProviderAdapter."
+                         "test_adapter_sets_transport_class(LocalRelpathServer)",
+                         local_relpath_test.id())

=== added file 'bzrlib/tests/test_setup.py'
--- bzrlib/tests/test_setup.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_setup.py	2005-12-16 04:53:00 +0000
@@ -0,0 +1,26 @@
+""" test for setup.py build process """
+
+import os
+import sys
+import subprocess
+import shutil
+
+from bzrlib.tests import TestCase
+
+
+class TestSetup(TestCase):
+
+    def setUp(self):
+        pass
+
+    def test_build(self):
+        """ test cmd `python setup.py build` """
+        # run setup.py build as subproces and catch return code
+        p = subprocess.Popen([sys.executable, 'setup.py', 'build'],
+                stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+        s = p.communicate()
+        self.assertEqual(0, p.returncode, '`python setup.py build` fails')
+
+    def tearDown(self):
+        """ cleanup build directory """
+        shutil.rmtree(u'build')

=== added file 'bzrlib/tests/test_sftp_transport.py'
--- bzrlib/tests/test_sftp_transport.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_sftp_transport.py	2006-01-12 01:02:32 +0000
@@ -0,0 +1,259 @@
+# Copyright (C) 2005 Robey Pointer <robey@lag.net>, Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+import os
+import socket
+import threading
+
+from bzrlib.branch import Branch
+import bzrlib.errors as errors
+from bzrlib.osutils import pathjoin, lexists
+from bzrlib.tests import TestCaseInTempDir, TestCase, TestSkipped
+import bzrlib.transport
+
+try:
+    import paramiko
+    paramiko_loaded = True
+except ImportError:
+    paramiko_loaded = False
+
+
+class TestCaseWithSFTPServer(TestCaseInTempDir):
+    """A test case base class that provides a sftp server on localhost."""
+
+    def setUp(self):
+        if not paramiko_loaded:
+            raise TestSkipped('you must have paramiko to run this test')
+        super(TestCaseWithSFTPServer, self).setUp()
+        from bzrlib.transport.sftp import SFTPAbsoluteServer, SFTPHomeDirServer
+        if getattr(self, '_get_remote_is_absolute', None) is None:
+            self._get_remote_is_absolute = True
+        if self._get_remote_is_absolute:
+            self.server = SFTPAbsoluteServer()
+        else:
+            self.server = SFTPHomeDirServer()
+        self.server.setUp()
+        self.addCleanup(self.server.tearDown)
+        self._sftp_url = self.server.get_url()
+        self._root = self.test_dir
+        # Set to a string in setUp to give sftp server a new homedir.
+        self._override_home = None
+        self._is_setup = False
+        self.sftplogs = []
+
+    def get_remote_url(self, relpath_to_test_root):
+        # FIXME use urljoin ?
+        return self._sftp_url + '/' + relpath_to_test_root
+
+    def get_transport(self, path=None):
+        """Return a transport relative to self._test_root."""
+        from bzrlib.transport import get_transport
+        transport = get_transport(self._sftp_url)
+        if path is None:
+            return transport
+        else:
+            return transport.clone(path)
+
+
+class SFTPLockTests (TestCaseWithSFTPServer):
+
+    def test_sftp_locks(self):
+        from bzrlib.errors import LockError
+        t = self.get_transport()
+
+        l = t.lock_write('bogus')
+        self.failUnlessExists('bogus.write-lock')
+
+        # Don't wait for the lock, locking an already locked
+        # file should raise an assert
+        self.assertRaises(LockError, t.lock_write, 'bogus')
+
+        l.unlock()
+        self.failIf(lexists('bogus.write-lock'))
+
+        open('something.write-lock', 'wb').write('fake lock\n')
+        self.assertRaises(LockError, t.lock_write, 'something')
+        os.remove('something.write-lock')
+
+        l = t.lock_write('something')
+
+        l2 = t.lock_write('bogus')
+
+        l.unlock()
+        l2.unlock()
+
+    def test_multiple_connections(self):
+        t = self.get_transport()
+        self.assertEquals(self.server.logs, 
+                ['sftpserver - authorizing: foo'
+               , 'sftpserver - channel request: session, 1'])
+        self.server.logs = []
+        # The second request should reuse the first connection
+        # SingleListener only allows for a single connection,
+        # So the next line fails unless the connection is reused
+        t2 = self.get_transport()
+        self.assertEquals(self.server.logs, [])
+
+
+class SFTPTransportTestRelative(TestCaseWithSFTPServer):
+    """Test the SFTP transport with homedir based relative paths."""
+
+    def test__remote_path(self):
+        t = self.get_transport()
+        # try what is currently used:
+        # remote path = self._abspath(relpath)
+        self.assertEqual(self._root + '/relative', t._remote_path('relative'))
+        # we dont os.path.join because windows gives us the wrong path
+        root_segments = self._root.split('/')
+        root_parent = '/'.join(root_segments[:-1])
+        # .. should be honoured
+        self.assertEqual(root_parent + '/sibling', t._remote_path('../sibling'))
+        # /  should be illegal ?
+        ### FIXME decide and then test for all transports. RBC20051208
+
+
+class SFTPTransportTestRelative(TestCaseWithSFTPServer):
+    """Test the SFTP transport with homedir based relative paths."""
+
+    def setUp(self):
+        self._get_remote_is_absolute = False
+        super(SFTPTransportTestRelative, self).setUp()
+
+    def test__remote_path_relative_root(self):
+        # relative paths are preserved
+        t = self.get_transport('')
+        self.assertEqual('a', t._remote_path('a'))
+
+
+class FakeSFTPTransport (object):
+    _sftp = object()
+fake = FakeSFTPTransport()
+
+
+class SFTPNonServerTest(TestCase):
+    def setUp(self):
+        TestCase.setUp(self)
+        if not paramiko_loaded:
+            raise TestSkipped('you must have paramiko to run this test')
+
+    def test_parse_url(self):
+        from bzrlib.transport.sftp import SFTPTransport
+        s = SFTPTransport('sftp://simple.example.com/%2fhome/source', clone_from=fake)
+        self.assertEquals(s._host, 'simple.example.com')
+        self.assertEquals(s._port, None)
+        self.assertEquals(s._path, '/home/source')
+        self.failUnless(s._password is None)
+
+        self.assertEquals(s.base, 'sftp://simple.example.com/%2Fhome/source/')
+        
+        s = SFTPTransport('sftp://ro%62ey:h%40t@example.com:2222/relative', clone_from=fake)
+        self.assertEquals(s._host, 'example.com')
+        self.assertEquals(s._port, 2222)
+        self.assertEquals(s._username, 'robey')
+        self.assertEquals(s._password, 'h@t')
+        self.assertEquals(s._path, 'relative')
+
+        # Base should not keep track of the password
+        self.assertEquals(s.base, 'sftp://robey@example.com:2222/relative/')
+
+        # Double slash should be accepted instead of using %2F
+        s = SFTPTransport('sftp://user@example.com:22//absolute/path/', clone_from=fake)
+        self.assertEquals(s._host, 'example.com')
+        self.assertEquals(s._port, 22)
+        self.assertEquals(s._username, 'user')
+        self.assertEquals(s._password, None)
+        self.assertEquals(s._path, '/absolute/path/')
+
+        # Also, don't show the port if it is the default 22
+        self.assertEquals(s.base, 'sftp://user@example.com:22/%2Fabsolute/path/')
+
+    def test_relpath(self):
+        from bzrlib.transport.sftp import SFTPTransport
+        from bzrlib.errors import PathNotChild
+
+        s = SFTPTransport('sftp://user@host.com//abs/path', clone_from=fake)
+        self.assertEquals(s.relpath('sftp://user@host.com//abs/path/sub'), 'sub')
+        # Can't test this one, because we actually get an AssertionError
+        # TODO: Consider raising an exception rather than an assert
+        #self.assertRaises(PathNotChild, s.relpath, 'http://user@host.com//abs/path/sub')
+        self.assertRaises(PathNotChild, s.relpath, 'sftp://user2@host.com//abs/path/sub')
+        self.assertRaises(PathNotChild, s.relpath, 'sftp://user@otherhost.com//abs/path/sub')
+        self.assertRaises(PathNotChild, s.relpath, 'sftp://user@host.com:33//abs/path/sub')
+        self.assertRaises(PathNotChild, s.relpath, 'sftp://user@host.com/abs/path/sub')
+
+        # Make sure it works when we don't supply a username
+        s = SFTPTransport('sftp://host.com//abs/path', clone_from=fake)
+        self.assertEquals(s.relpath('sftp://host.com//abs/path/sub'), 'sub')
+
+        # Make sure it works when parts of the path will be url encoded
+        # TODO: These may be incorrect, we might need to urllib.urlencode() before
+        # we pass the paths into the SFTPTransport constructor
+        s = SFTPTransport('sftp://host.com/dev/,path', clone_from=fake)
+        self.assertEquals(s.relpath('sftp://host.com/dev/,path/sub'), 'sub')
+        s = SFTPTransport('sftp://host.com/dev/%path', clone_from=fake)
+        self.assertEquals(s.relpath('sftp://host.com/dev/%path/sub'), 'sub')
+
+    def test_parse_invalid_url(self):
+        from bzrlib.transport.sftp import SFTPTransport, TransportError
+        try:
+            s = SFTPTransport('sftp://lilypond.org:~janneke/public_html/bzr/gub',
+                              clone_from=fake)
+            self.fail('expected exception not raised')
+        except TransportError, e:
+            self.assertEquals(str(e), 
+                    '~janneke: invalid port number')
+
+
+class SFTPBranchTest(TestCaseWithSFTPServer):
+    """Test some stuff when accessing a bzr Branch over sftp"""
+
+    def test_lock_file(self):
+        """Make sure that a Branch accessed over sftp tries to lock itself."""
+        b = Branch.initialize(self._sftp_url)
+        self.failUnlessExists('.bzr/')
+        self.failUnlessExists('.bzr/branch-format')
+        self.failUnlessExists('.bzr/branch-lock')
+
+        self.failIf(lexists('.bzr/branch-lock.write-lock'))
+        b.lock_write()
+        self.failUnlessExists('.bzr/branch-lock.write-lock')
+        b.unlock()
+        self.failIf(lexists('.bzr/branch-lock.write-lock'))
+
+    def test_no_working_tree(self):
+        b = Branch.initialize(self._sftp_url)
+        self.assertRaises(errors.NoWorkingTree, b.working_tree)
+
+    def test_push_support(self):
+        self.build_tree(['a/', 'a/foo'])
+        b = Branch.initialize('a')
+        t = b.working_tree()
+        t.add('foo')
+        t.commit('foo', rev_id='a1')
+
+        os.mkdir('b')
+        b2 = Branch.initialize(self._sftp_url + '/b')
+        b2.pull(b)
+
+        self.assertEquals(b2.revision_history(), ['a1'])
+
+        open('a/foo', 'wt').write('something new in foo\n')
+        t.commit('new', rev_id='a2')
+        b2.pull(b)
+
+        self.assertEquals(b2.revision_history(), ['a1', 'a2'])
+
+

=== modified file 'bzrlib/tests/test_smart_add.py'
--- bzrlib/selftest/test_smart_add.py	2005-10-03 15:19:25 +0000
+++ bzrlib/tests/test_smart_add.py	2005-12-15 08:32:03 +0000
@@ -1,10 +1,12 @@
 import os
 import unittest
 
-from bzrlib.selftest import TestCaseInTempDir, TestCase
+from bzrlib.tests import TestCaseInTempDir, TestCase
 from bzrlib.branch import Branch
-from bzrlib.errors import NotBranchError, NotVersionedError
-from bzrlib.inventory import InventoryFile
+from bzrlib.errors import NotBranchError, NoSuchFile
+from bzrlib.inventory import InventoryFile, Inventory
+from bzrlib.workingtree import WorkingTree
+from bzrlib.add import smart_add
 
 class TestSmartAdd(TestCaseInTempDir):
 
@@ -13,21 +15,21 @@
         from bzrlib.add import smart_add
         paths = ("original/", "original/file1", "original/file2")
         self.build_tree(paths)
-        branch = Branch.initialize(".")
-        smart_add((".",), recurse=True)
+        branch = Branch.initialize(u".")
+        smart_add((u".",))
         for path in paths:
-            self.assertNotEqual(branch.inventory.path2id(path), None)
+            self.assertNotEqual(branch.working_tree().path2id(path), None)
 
     def test_add_dot_from_subdir(self):
         """Test adding . from a subdir of the tree.""" 
         from bzrlib.add import smart_add
         paths = ("original/", "original/file1", "original/file2")
         self.build_tree(paths)
-        branch = Branch.initialize(".")
+        branch = Branch.initialize(u".")
         os.chdir("original")
-        smart_add((".",), recurse=True)
+        smart_add((u".",))
         for path in paths:
-            self.assertNotEqual(branch.inventory.path2id(path), None)
+            self.assertNotEqual(branch.working_tree().path2id(path), None)
 
     def test_add_tree_from_above_tree(self):
         """Test adding a tree from above the tree.""" 
@@ -39,12 +41,11 @@
         branch = Branch.initialize("branch")
         smart_add(("branch",))
         for path in paths:
-            self.assertNotEqual(branch.inventory.path2id(path), None)
+            self.assertNotEqual(branch.working_tree().path2id(path), None)
 
     def test_add_above_tree_preserves_tree(self):
         """Test nested trees are not affect by an add above them."""
-        from bzrlib.add import smart_add, add_reporter_null
-        
+        from bzrlib.add import smart_add
         paths = ("original/", "original/file1", "original/file2")
         child_paths = ("path",)
         full_child_paths = ("original/child", "original/child/path")
@@ -52,109 +53,160 @@
                        "original/child/", "original/child/path")
         
         self.build_tree(build_paths)
-        branch = Branch.initialize(".")
+        branch = Branch.initialize(u".")
         child_branch = Branch.initialize("original/child")
-        smart_add((".",), True, add_reporter_null)
+        smart_add((u".",))
         for path in paths:
-            self.assertNotEqual((path, branch.inventory.path2id(path)),
+            self.assertNotEqual((path, branch.working_tree().path2id(path)),
                                 (path, None))
         for path in full_child_paths:
-            self.assertEqual((path, branch.inventory.path2id(path)),
+            self.assertEqual((path, branch.working_tree().path2id(path)),
                              (path, None))
         for path in child_paths:
-            self.assertEqual(child_branch.inventory.path2id(path), None)
+            self.assertEqual(child_branch.working_tree().path2id(path), None)
 
     def test_add_paths(self):
         """Test smart-adding a list of paths."""
         from bzrlib.add import smart_add
         paths = ("file1", "file2")
         self.build_tree(paths)
-        branch = Branch.initialize(".")
+        branch = Branch.initialize(u".")
         smart_add(paths)
         for path in paths:
-            self.assertNotEqual(branch.inventory.path2id(path), None)
-            
+            self.assertNotEqual(branch.working_tree().path2id(path), None)
+
+    def test_add_dry_run(self):
+        """Test a dry run add, make sure nothing is added."""
+        from bzrlib.commands import run_bzr
+        eq = self.assertEqual
+        b = Branch.initialize(u'.')
+        t = b.working_tree()
+        self.build_tree(['inertiatic/', 'inertiatic/esp'])
+        eq(list(t.unknowns()), ['inertiatic'])
+        self.capture('add --dry-run .')
+        eq(list(t.unknowns()), ['inertiatic'])
+
+    def test_add_non_existant(self):
+        """Test smart-adding a file that does not exist."""
+        from bzrlib.add import smart_add
+        branch = Branch.initialize(u".")
+        self.assertRaises(NoSuchFile, smart_add, 'non-existant-file')
+
+    def test_returns(self):
+        """Correctly returns added/ignored files"""
+        from bzrlib.commands import run_bzr
+        b = Branch.initialize(u'.')
+        t = b.working_tree()
+        self.build_tree(['inertiatic/', 'inertiatic/esp', 'inertiatic/CVS', 
+                        'inertiatic/foo.pyc'])
+        added, ignored = smart_add(u'.')
+        self.AssertSubset(('inertiatic', 'inertiatic/esp'), added)
+        self.AssertSubset(('CVS', '*.py[oc]'), ignored)
+        self.AssertSubset(('inertiatic/CVS',), ignored['CVS'])
+        self.AssertSubset(('inertiatic/foo.pyc',), ignored['*.py[oc]'])
+
+
 class TestSmartAddBranch(TestCaseInTempDir):
     """Test smart adds with a specified branch."""
 
     def test_add_dot_from_root(self):
         """Test adding . from the root of the tree.""" 
-        from bzrlib.add import smart_add_branch
+        from bzrlib.add import smart_add_tree
         paths = ("original/", "original/file1", "original/file2")
         self.build_tree(paths)
-        branch = Branch.initialize(".")
-        smart_add_branch(branch, (".",))
+        Branch.initialize(u".")
+        tree = WorkingTree()
+        smart_add_tree(tree, (u".",))
         for path in paths:
-            self.assertNotEqual(branch.inventory.path2id(path), None)
+            self.assertNotEqual(tree.path2id(path), None)
 
     def test_add_dot_from_subdir(self):
         """Test adding . from a subdir of the tree.""" 
-        from bzrlib.add import smart_add_branch
+        from bzrlib.add import smart_add_tree
         paths = ("original/", "original/file1", "original/file2")
         self.build_tree(paths)
-        branch = Branch.initialize(".")
+        Branch.initialize(u".")
+        tree = WorkingTree()
         os.chdir("original")
-        smart_add_branch(branch, (".",))
+        smart_add_tree(tree, (u".",))
         for path in paths:
-            self.assertNotEqual(branch.inventory.path2id(path), None)
+            self.assertNotEqual(tree.path2id(path), None)
 
     def test_add_tree_from_above_tree(self):
         """Test adding a tree from above the tree.""" 
-        from bzrlib.add import smart_add_branch
+        from bzrlib.add import smart_add_tree
         paths = ("original/", "original/file1", "original/file2")
         branch_paths = ("branch/", "branch/original/", "branch/original/file1",
                         "branch/original/file2")
         self.build_tree(branch_paths)
-        branch = Branch.initialize("branch")
-        smart_add_branch(branch, ("branch",))
+        Branch.initialize("branch")
+        tree = WorkingTree("branch")
+        smart_add_tree(tree, ("branch",))
         for path in paths:
-            self.assertNotEqual(branch.inventory.path2id(path), None)
+            self.assertNotEqual(tree.path2id(path), None)
 
     def test_add_above_tree_preserves_tree(self):
         """Test nested trees are not affect by an add above them."""
-        from bzrlib.add import smart_add_branch
+        from bzrlib.add import smart_add_tree
         paths = ("original/", "original/file1", "original/file2")
         child_paths = ("path")
         full_child_paths = ("original/child", "original/child/path")
         build_paths = ("original/", "original/file1", "original/file2", 
                        "original/child/", "original/child/path")
         self.build_tree(build_paths)
-        branch = Branch.initialize(".")
+        Branch.initialize(u".")
+        tree = WorkingTree()
         child_branch = Branch.initialize("original/child")
-        smart_add_branch(branch, (".",))
+        smart_add_tree(tree, (u".",))
         for path in paths:
-            self.assertNotEqual((path, branch.inventory.path2id(path)),
+            self.assertNotEqual((path, tree.path2id(path)),
                                 (path, None))
         for path in full_child_paths:
-            self.assertEqual((path, branch.inventory.path2id(path)),
+            self.assertEqual((path, tree.path2id(path)),
                              (path, None))
         for path in child_paths:
-            self.assertEqual(child_branch.inventory.path2id(path), None)
+            self.assertEqual(child_branch.working_tree().path2id(path), None)
 
     def test_add_paths(self):
         """Test smart-adding a list of paths."""
-        from bzrlib.add import smart_add_branch
+        from bzrlib.add import smart_add_tree
         paths = ("file1", "file2")
         self.build_tree(paths)
-        branch = Branch.initialize(".")
-        smart_add_branch(branch, paths)
+        Branch.initialize(u".")
+        tree = WorkingTree()
+        smart_add_tree(tree, paths)
         for path in paths:
-            self.assertNotEqual(branch.inventory.path2id(path), None)
-
-class TestAddCallbacks(TestCaseInTempDir):
-
-    def setUp(self):
-        super(TestAddCallbacks, self).setUp()
+            self.assertNotEqual(tree.path2id(path), None)
+
+
+class TestAddActions(TestCaseInTempDir):
+
+    def test_null(self):
+        from bzrlib.add import add_action_null
+        self.run_action(add_action_null, "", False)
+
+    def test_add(self):
         self.entry = InventoryFile("id", "name", None)
-
-    def test_null_callback(self):
-        from bzrlib.add import add_reporter_null
-        add_reporter_null('path', 'file', self.entry)
-
-    def test_print_callback(self):
-        from bzrlib.add import add_reporter_print
+        from bzrlib.add import add_action_add
+        self.run_action(add_action_add, "", True)
+
+    def test_add_and_print(self):
+        from bzrlib.add import add_action_add_and_print
+        self.run_action(add_action_add_and_print, "added path\n", True)
+
+    def test_print(self):
+        from bzrlib.add import add_action_print
+        self.run_action(add_action_print, "added path\n", False)
+
+    def run_action(self, action, output, should_add):
         from StringIO import StringIO
+        inv = Inventory()
         stdout = StringIO()
-        self.apply_redirected(None, stdout, None, add_reporter_print,
-                              'path', 'file', self.entry)
-        self.assertEqual(stdout.getvalue(), "added path\n")
+
+        self.apply_redirected(None, stdout, None, action, inv, 'path', 'file')
+        self.assertEqual(stdout.getvalue(), output)
+
+        if should_add:
+            self.assertNotEqual(inv.path2id('path'), None)
+        else:
+            self.assertEqual(inv.path2id('path'), None)

=== added file 'bzrlib/tests/test_source.py'
--- bzrlib/tests/test_source.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_source.py	2005-12-12 22:34:21 +0000
@@ -0,0 +1,77 @@
+# Copyright (C) 2005 by Canonical Ltd
+#   Authors: Robert Collins <robert.collins@canonical.com>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""These tests are tests about the source code of bzrlib itself.
+
+They are useful for testing code quality, checking coverage metric etc.
+"""
+
+# import system imports here
+import os
+import sys
+
+#import bzrlib specific imports here
+from bzrlib.tests import TestCase
+import bzrlib.branch
+
+
+class TestApiUsage(TestCase):
+
+    def find_occurences(self, rule, filename):
+        """Find the number of occurences of rule in a file."""
+        occurences = 0
+        source = file(filename, 'r')
+        for line in source:
+            if line.find(rule) > -1:
+                occurences += 1
+        return occurences
+
+    def source_file_name(self, package):
+        """Return the path of the .py file for package."""
+        path = package.__file__
+        if path[-1] in 'co':
+            return path[:-1]
+        else:
+            return path
+
+    def test_branch_working_tree(self):
+        """Test that the number of uses of working_tree in branch is stable."""
+        occurences = self.find_occurences('self.working_tree()',
+                                          self.source_file_name(bzrlib.branch))
+        # do not even think of increasing this number. If you think you need to
+        # increase it, then you almost certainly are doing something wrong as
+        # the relationship from working_tree to branch is one way.
+        # This number should be 0, but the basis_inventory merge was done
+        # before this test was written. Note that this is an exact equality
+        # so that when the number drops, it is not given a buffer but rather
+        # this test updated immediately.
+        self.assertEqual(2, occurences)
+
+    def test_branch_WorkingTree(self):
+        """Test that the number of uses of working_tree in branch is stable."""
+        occurences = self.find_occurences('WorkingTree',
+                                          self.source_file_name(bzrlib.branch))
+        # do not even think of increasing this number. If you think you need to
+        # increase it, then you almost certainly are doing something wrong as
+        # the relationship from working_tree to branch is one way.
+        # This number should be 4 (import NoWorkingTree and WorkingTree, 
+        # raise NoWorkingTree from working_tree(), and construct a working tree
+        # there) but a merge that regressed this was done before this test was
+        # written. Note that this is an exact equality so that when the number
+        # drops, it is not given a buffer but rather this test updated
+        # immediately.
+        self.assertEqual(6, occurences)

=== renamed file 'bzrlib/selftest/teststatus.py' => 'bzrlib/tests/test_status.py'
--- bzrlib/selftest/teststatus.py	2005-10-03 01:53:47 +0000
+++ bzrlib/tests/test_status.py	2005-12-02 05:25:53 +0000
@@ -21,18 +21,26 @@
 """
 
 
-from bzrlib.selftest import TestCaseInTempDir
+from cStringIO import StringIO
+from os import mkdir
+from tempfile import TemporaryFile
+import codecs
+
+from bzrlib.tests import TestCaseInTempDir
 from bzrlib.revisionspec import RevisionSpec
+from bzrlib.merge import merge
+from bzrlib.status import show_status
+from bzrlib.branch import Branch
+from bzrlib.clone import copy_branch
 
 class BranchStatus(TestCaseInTempDir):
     
     def test_branch_status(self): 
         """Test basic branch status"""
-        from cStringIO import StringIO
         from bzrlib.status import show_status
         from bzrlib.branch import Branch
         
-        b = Branch.initialize('.')
+        b = Branch.initialize(u'.')
 
         # status with nothing
         tof = StringIO()
@@ -41,7 +49,7 @@
 
         tof = StringIO()
         self.build_tree(['hello.c', 'bye.c'])
-        b.add_pending_merge('pending@pending-0-0')
+        b.working_tree().add_pending_merge('pending@pending-0-0')
         show_status(b, to_file=tof)
         tof.seek(0)
         self.assertEquals(tof.readlines(),
@@ -54,17 +62,14 @@
 
     def test_branch_status_revisions(self):
         """Tests branch status with revisions"""
-        from cStringIO import StringIO
-        from bzrlib.status import show_status
-        from bzrlib.branch import Branch
         
-        b = Branch.initialize('.')
+        b = Branch.initialize(u'.')
 
         tof = StringIO()
         self.build_tree(['hello.c', 'bye.c'])
-        b.add('hello.c')
-        b.add('bye.c')
-        b.commit('Test message')
+        b.working_tree().add('hello.c')
+        b.working_tree().add('bye.c')
+        b.working_tree().commit('Test message')
 
         tof = StringIO()
         revs =[]
@@ -79,8 +84,8 @@
                            '  hello.c\n'])
 
         self.build_tree(['more.c'])
-        b.add('more.c')
-        b.commit('Another test message')
+        b.working_tree().add('more.c')
+        b.working_tree().commit('Another test message')
         
         tof = StringIO()
         revs.append(RevisionSpec(1))
@@ -93,18 +98,47 @@
                            '  bye.c\n',
                            '  hello.c\n'])
 
+    def status_string(self, branch):
+        # use a real file rather than StringIO because it doesn't handle
+        # Unicode very well.
+        tof = codecs.getwriter('utf-8')(TemporaryFile())
+        show_status(branch, to_file=tof)
+        tof.seek(0)
+        return tof.read().decode('utf-8')
+
+    def test_pending(self):
+        """Pending merges display works, including Unicode"""
+        mkdir("./branch")
+        b = Branch.initialize('./branch')
+        b.working_tree().commit("Empty commit 1")
+        b_2 = copy_branch(b, './copy')
+        b.working_tree().commit(u"\N{TIBETAN DIGIT TWO} Empty commit 2")
+        merge(["./branch", -1], [None, None], this_dir = './copy')
+        message = self.status_string(b_2)
+        self.assert_(message.startswith("pending merges:\n"))
+        self.assert_(message.endswith("Empty commit 2\n")) 
+        b_2.working_tree().commit("merged")
+        # must be long to make sure we see elipsis at the end
+        b.working_tree().commit("Empty commit 3 " + 
+                                "blah blah blah blah " * 10)
+        merge(["./branch", -1], [None, None], this_dir = './copy')
+        message = self.status_string(b_2)
+        self.assert_(message.startswith("pending merges:\n"))
+        self.assert_("Empty commit 3" in message)
+        self.assert_(message.endswith("...\n")) 
+
     def test_branch_status_specific_files(self): 
         """Tests branch status with given specific files"""
         from cStringIO import StringIO
         from bzrlib.status import show_status
         from bzrlib.branch import Branch
         
-        b = Branch.initialize('.')
+        b = Branch.initialize(u'.')
 
         self.build_tree(['directory/','directory/hello.c', 'bye.c','test.c','dir2/'])
-        b.add('directory')
-        b.add('test.c')
-        b.commit('testing')
+        b.working_tree().add('directory')
+        b.working_tree().add('test.c')
+        b.working_tree().commit('testing')
         
         tof = StringIO()
         show_status(b, to_file=tof)

=== renamed file 'bzrlib/selftest/teststore.py' => 'bzrlib/tests/test_store.py'
--- bzrlib/selftest/teststore.py	2005-10-05 06:44:02 +0000
+++ bzrlib/tests/test_store.py	2005-12-18 04:08:46 +0000
@@ -14,134 +14,94 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-"""Test Store implementation
-"""
+"""Test Store implementations."""
+
 from cStringIO import StringIO
 import os
+import gzip
 
+from bzrlib.errors import BzrError, UnlistableStore, NoSuchFile
 from bzrlib.store import copy_all
 from bzrlib.transport.local import LocalTransport
-from bzrlib.transport import NoSuchFile
-from bzrlib.store.compressed_text import CompressedTextStore
 from bzrlib.store.text import TextStore
-from bzrlib.selftest import TestCase, TestCaseInTempDir
-from bzrlib.errors import BzrError, UnlistableStore
-import bzrlib.store
-
-
-def fill_store(store):
-    store.add(StringIO('hello'), 'a')
-    store.add(StringIO('other'), 'b')
-    store.add(StringIO('something'), 'c')
-    store.add(StringIO('goodbye'), '123123')
-
-def check_equals(tester, store, files, values, permit_failure=False):
-    files = store.get(files, permit_failure=permit_failure)
-    count = 0
-    for f, v in zip(files, values):
-        count += 1
-        if v is None:
-            tester.assert_(f is None)
-        else:
-            tester.assertEquals(f.read(), v)
-    tester.assertEquals(count, len(values))
-    # We need to check to make sure there are no more
-    # files to be returned, I'm using a cheezy way
-    # Convert to a list, and there shouldn't be any left
-    tester.assertEquals(len(list(files)), 0)
-
-def test_multiple_add(tester, store):
-    fill_store(store)
-    tester.assertRaises(BzrError, store.add, StringIO('goodbye'), '123123')
-
-def test_get(tester, store):
-    fill_store(store)
-
-    check_equals(tester, store, ['a'], ['hello'])
-    check_equals(tester, store, ['b', 'c'], ['other', 'something'])
-
-    # Make sure that requesting a non-existing file fails
-    tester.assertRaises(NoSuchFile, check_equals, tester, store,
-            ['d'], [None])
-    tester.assertRaises(NoSuchFile, check_equals, tester, store,
-            ['a', 'd'], ['hello', None])
-    tester.assertRaises(NoSuchFile, check_equals, tester, store,
-            ['d', 'a'], [None, 'hello'])
-    tester.assertRaises(NoSuchFile, check_equals, tester, store,
-            ['d', 'd', 'd'], [None, None, None])
-    tester.assertRaises(NoSuchFile, check_equals, tester, store,
-            ['a', 'd', 'b'], ['hello', None, 'other'])
-
-def test_ignore_get(tester, store):
-    fill_store(store)
-
-    files = store.get(['d'], permit_failure=True)
-    files = list(files)
-    tester.assertEquals(len(files), 1)
-    tester.assert_(files[0] is None)
-
-    check_equals(tester, store, ['a', 'd'], ['hello', None],
-            permit_failure=True)
-    check_equals(tester, store, ['d', 'a'], [None, 'hello'],
-            permit_failure=True)
-    check_equals(tester, store, ['d', 'd'], [None, None],
-            permit_failure=True)
-    check_equals(tester, store, ['a', 'd', 'b'], ['hello', None, 'other'],
-            permit_failure=True)
-    check_equals(tester, store, ['a', 'd', 'b'], ['hello', None, 'other'],
-            permit_failure=True)
-    check_equals(tester, store, ['b', 'd', 'c'], ['other', None, 'something'],
-            permit_failure=True)
-
-
-def get_compressed_store(path='.'):
-    t = LocalTransport(path)
-    return CompressedTextStore(t)
-
-
-def get_text_store(path='.'):
-    t = LocalTransport(path)
-    return TextStore(t)
-
-
-class TestCompressedTextStore(TestCaseInTempDir):
+from bzrlib.tests import TestCase, TestCaseInTempDir
+import bzrlib.store as store
+import bzrlib.transport as transport
+from bzrlib.transport.memory import MemoryTransport
+
+
+class TestStores(object):
+    """Mixin template class that provides some common tests for stores"""
+
+    def check_content(self, store, fileid, value):
+        f = store.get(fileid)
+        self.assertEqual(f.read(), value)
+
+    def fill_store(self, store):
+        store.add(StringIO('hello'), 'a')
+        store.add(StringIO('other'), 'b')
+        store.add(StringIO('something'), 'c')
+        store.add(StringIO('goodbye'), '123123')
+
+    def test_copy_all(self):
+        """Test copying"""
+        os.mkdir('a')
+        store_a = self.get_store('a')
+        store_a.add('foo', '1')
+        os.mkdir('b')
+        store_b = self.get_store('b')
+        copy_all(store_a, store_b)
+        self.assertEqual(store_a.get('1').read(), 'foo')
+        self.assertEqual(store_b.get('1').read(), 'foo')
+        # TODO: Switch the exception form UnlistableStore to
+        #       or make Stores throw UnlistableStore if their
+        #       Transport doesn't support listing
+        # store_c = RemoteStore('http://example.com/')
+        # self.assertRaises(UnlistableStore, copy_all, store_c, store_b)
+
+    def test_get(self):
+        store = self.get_store()
+        self.fill_store(store)
+    
+        self.check_content(store, 'a', 'hello')
+        self.check_content(store, 'b', 'other')
+        self.check_content(store, 'c', 'something')
+    
+        # Make sure that requesting a non-existing file fails
+        self.assertRaises(KeyError, self.check_content, store, 'd', None)
 
     def test_multiple_add(self):
         """Multiple add with same ID should raise a BzrError"""
-        store = get_compressed_store()
-        test_multiple_add(self, store)
-
-    def test_get(self):
-        store = get_compressed_store()
-        test_get(self, store)
-
-    def test_ignore_get(self):
-        store = get_compressed_store()
-        test_ignore_get(self, store)
+        store = self.get_store()
+        self.fill_store(store)
+        self.assertRaises(BzrError, store.add, StringIO('goodbye'), '123123')
+
+
+class TestCompressedTextStore(TestCaseInTempDir, TestStores):
+
+    def get_store(self, path=u'.'):
+        t = LocalTransport(path)
+        return TextStore(t, compressed=True)
 
     def test_total_size(self):
-        store = get_compressed_store('.')
+        store = self.get_store(u'.')
+        store.register_suffix('dsc')
         store.add(StringIO('goodbye'), '123123')
-        store.add(StringIO('goodbye2'), '123123.dsc')
+        store.add(StringIO('goodbye2'), '123123', 'dsc')
         # these get gzipped - content should be stable
         self.assertEqual(store.total_size(), (2, 55))
         
-    def test_copy_all(self):
-        """Test copying"""
-        os.mkdir('a')
-        store_a = get_text_store('a')
-        store_a.add('foo', '1')
-        os.mkdir('b')
-        store_b = get_text_store('b')
-        copy_all(store_a, store_b)
-        self.assertEqual(store_a['1'].read(), 'foo')
-        self.assertEqual(store_b['1'].read(), 'foo')
+    def test__relpath_suffixed(self):
+        my_store = TextStore(MockTransport(),
+                             prefixed=True, compressed=True)
+        my_store.register_suffix('dsc')
+        self.assertEqual('45/foo.dsc', my_store._relpath('foo', ['dsc']))
 
 
 class TestMemoryStore(TestCase):
     
     def get_store(self):
-        return bzrlib.store.ImmutableMemoryStore()
+        return store.ImmutableMemoryStore()
     
     def test_imports(self):
         from bzrlib.store import ImmutableMemoryStore
@@ -149,60 +109,291 @@
     def test_add_and_retrieve(self):
         store = self.get_store()
         store.add(StringIO('hello'), 'aa')
-        self.assertNotEqual(store['aa'], None)
-        self.assertEqual(store['aa'].read(), 'hello')
+        self.assertNotEqual(store.get('aa'), None)
+        self.assertEqual(store.get('aa').read(), 'hello')
         store.add(StringIO('hello world'), 'bb')
-        self.assertNotEqual(store['bb'], None)
-        self.assertEqual(store['bb'].read(), 'hello world')
+        self.assertNotEqual(store.get('bb'), None)
+        self.assertEqual(store.get('bb').read(), 'hello world')
 
     def test_missing_is_absent(self):
         store = self.get_store()
         self.failIf('aa' in store)
 
     def test_adding_fails_when_present(self):
-        store = self.get_store()
-        store.add(StringIO('hello'), 'aa')
-        self.assertRaises(bzrlib.store.StoreError,
-                          store.add, StringIO('hello'), 'aa')
-
-    def test_total_size(self):
-        store = self.get_store()
-        store.add(StringIO('goodbye'), '123123')
-        store.add(StringIO('goodbye2'), '123123.dsc')
-        self.assertEqual(store.total_size(), (2, 15))
-        # TODO: Switch the exception form UnlistableStore to
-        #       or make Stores throw UnlistableStore if their
-        #       Transport doesn't support listing
-        # store_c = RemoteStore('http://example.com/')
-        # self.assertRaises(UnlistableStore, copy_all, store_c, store_b)
-
-
-class TestTextStore(TestCaseInTempDir):
-    def test_multiple_add(self):
-        """Multiple add with same ID should raise a BzrError"""
-        store = get_text_store()
-        test_multiple_add(self, store)
-
-    def test_get(self):
-        store = get_text_store()
-        test_get(self, store)
-
-    def test_ignore_get(self):
-        store = get_text_store()
-        test_ignore_get(self, store)
-
-    def test_copy_all(self):
-        """Test copying"""
-        os.mkdir('a')
-        store_a = get_text_store('a')
-        store_a.add('foo', '1')
-        os.mkdir('b')
-        store_b = get_text_store('b')
-        copy_all(store_a, store_b)
-        self.assertEqual(store_a['1'].read(), 'foo')
-        self.assertEqual(store_b['1'].read(), 'foo')
-        # TODO: Switch the exception form UnlistableStore to
-        #       or make Stores throw UnlistableStore if their
-        #       Transport doesn't support listing
-        # store_c = RemoteStore('http://example.com/')
-        # self.assertRaises(UnlistableStore, copy_all, store_c, store_b)
+        my_store = self.get_store()
+        my_store.add(StringIO('hello'), 'aa')
+        self.assertRaises(BzrError,
+                          my_store.add, StringIO('hello'), 'aa')
+
+    def test_total_size(self):
+        store = self.get_store()
+        store.add(StringIO('goodbye'), '123123')
+        store.add(StringIO('goodbye2'), '123123.dsc')
+        self.assertEqual(store.total_size(), (2, 15))
+        # TODO: Switch the exception form UnlistableStore to
+        #       or make Stores throw UnlistableStore if their
+        #       Transport doesn't support listing
+        # store_c = RemoteStore('http://example.com/')
+        # self.assertRaises(UnlistableStore, copy_all, store_c, store_b)
+
+
+class TestTextStore(TestCaseInTempDir, TestStores):
+
+    def get_store(self, path=u'.'):
+        t = LocalTransport(path)
+        return TextStore(t, compressed=False)
+
+    def test_total_size(self):
+        store = self.get_store()
+        store.add(StringIO('goodbye'), '123123')
+        store.add(StringIO('goodbye2'), '123123.dsc')
+        self.assertEqual(store.total_size(), (2, 15))
+        # TODO: Switch the exception form UnlistableStore to
+        #       or make Stores throw UnlistableStore if their
+        #       Transport doesn't support listing
+        # store_c = RemoteStore('http://example.com/')
+        # self.assertRaises(UnlistableStore, copy_all, store_c, store_b)
+
+
+class TestMixedTextStore(TestCaseInTempDir, TestStores):
+
+    def get_store(self, path=u'.', compressed=True):
+        t = LocalTransport(path)
+        return TextStore(t, compressed=compressed)
+
+    def test_get_mixed(self):
+        cs = self.get_store(u'.', compressed=True)
+        s = self.get_store(u'.', compressed=False)
+        cs.add(StringIO('hello there'), 'a')
+
+        self.failUnlessExists('a.gz')
+        self.failIf(os.path.lexists('a'))
+
+        self.assertEquals(gzip.GzipFile('a.gz').read(), 'hello there')
+
+        self.assertEquals(cs.has_id('a'), True)
+        self.assertEquals(s.has_id('a'), True)
+        self.assertEquals(cs.get('a').read(), 'hello there')
+        self.assertEquals(s.get('a').read(), 'hello there')
+        
+        self.assertRaises(BzrError, s.add, StringIO('goodbye'), 'a')
+
+        s.add(StringIO('goodbye'), 'b')
+        self.failUnlessExists('b')
+        self.failIf(os.path.lexists('b.gz'))
+        self.assertEquals(open('b').read(), 'goodbye')
+
+        self.assertEquals(cs.has_id('b'), True)
+        self.assertEquals(s.has_id('b'), True)
+        self.assertEquals(cs.get('b').read(), 'goodbye')
+        self.assertEquals(s.get('b').read(), 'goodbye')
+        
+        self.assertRaises(BzrError, cs.add, StringIO('again'), 'b')
+
+class MockTransport(transport.Transport):
+    """A fake transport for testing with."""
+
+    def has(self, filename):
+        return False
+
+    def __init__(self, url=None):
+        if url is None:
+            url = "http://example.com"
+        super(MockTransport, self).__init__(url)
+
+    def mkdir(self, filename):
+        return
+
+
+class InstrumentedTransportStore(store.TransportStore):
+    """An instrumented TransportStore.
+
+    Here we replace template method worker methods with calls that record the
+    expected results.
+    """
+
+    def _add(self, filename, file):
+        self._calls.append(("_add", filename, file))
+
+    def __init__(self, transport, prefixed=False):
+        super(InstrumentedTransportStore, self).__init__(transport, prefixed)
+        self._calls = []
+
+
+class TestInstrumentedTransportStore(TestCase):
+
+    def test__add_records(self):
+        my_store = InstrumentedTransportStore(MockTransport())
+        my_store._add("filename", "file")
+        self.assertEqual([("_add", "filename", "file")], my_store._calls)
+
+
+class TestMockTransport(TestCase):
+
+    def test_isinstance(self):
+        self.failUnless(isinstance(MockTransport(), transport.Transport))
+
+    def test_has(self):
+        self.assertEqual(False, MockTransport().has('foo'))
+
+    def test_mkdir(self):
+        MockTransport().mkdir('45')
+
+
+class TestTransportStore(TestCase):
+    
+    def test__relpath_invalid(self):
+        my_store = store.TransportStore(MockTransport())
+        self.assertRaises(ValueError, my_store._relpath, '/foo')
+        self.assertRaises(ValueError, my_store._relpath, 'foo/')
+
+    def test_register_invalid_suffixes(self):
+        my_store = store.TransportStore(MockTransport())
+        self.assertRaises(ValueError, my_store.register_suffix, '/')
+        self.assertRaises(ValueError, my_store.register_suffix, '.gz/bar')
+
+    def test__relpath_unregister_suffixes(self):
+        my_store = store.TransportStore(MockTransport())
+        self.assertRaises(ValueError, my_store._relpath, 'foo', ['gz'])
+        self.assertRaises(ValueError, my_store._relpath, 'foo', ['dsc', 'gz'])
+
+    def test__relpath_simple(self):
+        my_store = store.TransportStore(MockTransport())
+        self.assertEqual("foo", my_store._relpath('foo'))
+
+    def test__relpath_prefixed(self):
+        my_store = store.TransportStore(MockTransport(), True)
+        self.assertEqual('45/foo', my_store._relpath('foo'))
+
+    def test__relpath_simple_suffixed(self):
+        my_store = store.TransportStore(MockTransport())
+        my_store.register_suffix('bar')
+        my_store.register_suffix('baz')
+        self.assertEqual('foo.baz', my_store._relpath('foo', ['baz']))
+        self.assertEqual('foo.bar.baz', my_store._relpath('foo', ['bar', 'baz']))
+
+    def test__relpath_prefixed_suffixed(self):
+        my_store = store.TransportStore(MockTransport(), True)
+        my_store.register_suffix('bar')
+        my_store.register_suffix('baz')
+        self.assertEqual('45/foo.baz', my_store._relpath('foo', ['baz']))
+        self.assertEqual('45/foo.bar.baz',
+                         my_store._relpath('foo', ['bar', 'baz']))
+
+    def test_add_simple(self):
+        stream = StringIO("content")
+        my_store = InstrumentedTransportStore(MockTransport())
+        my_store.add(stream, "foo")
+        self.assertEqual([("_add", "foo", stream)], my_store._calls)
+
+    def test_add_prefixed(self):
+        stream = StringIO("content")
+        my_store = InstrumentedTransportStore(MockTransport(), True)
+        my_store.add(stream, "foo")
+        self.assertEqual([("_add", "45/foo", stream)], my_store._calls)
+
+    def test_add_simple_suffixed(self):
+        stream = StringIO("content")
+        my_store = InstrumentedTransportStore(MockTransport())
+        my_store.register_suffix('dsc')
+        my_store.add(stream, "foo", 'dsc')
+        self.assertEqual([("_add", "foo.dsc", stream)], my_store._calls)
+        
+    def test_add_simple_suffixed(self):
+        stream = StringIO("content")
+        my_store = InstrumentedTransportStore(MockTransport(), True)
+        my_store.register_suffix('dsc')
+        my_store.add(stream, "foo", 'dsc')
+        self.assertEqual([("_add", "45/foo.dsc", stream)], my_store._calls)
+
+    def get_populated_store(self, prefixed=False,
+            store_class=TextStore, compressed=False):
+        my_store = store_class(MemoryTransport(), prefixed,
+                               compressed=compressed)
+        my_store.register_suffix('sig')
+        stream = StringIO("signature")
+        my_store.add(stream, "foo", 'sig')
+        stream = StringIO("content")
+        my_store.add(stream, "foo")
+        stream = StringIO("signature for missing base")
+        my_store.add(stream, "missing", 'sig')
+        return my_store
+        
+    def test_has_simple(self):
+        my_store = self.get_populated_store()
+        self.assertEqual(True, my_store.has_id('foo'))
+        my_store = self.get_populated_store(True)
+        self.assertEqual(True, my_store.has_id('foo'))
+
+    def test_has_suffixed(self):
+        my_store = self.get_populated_store()
+        self.assertEqual(True, my_store.has_id('foo', 'sig'))
+        my_store = self.get_populated_store(True)
+        self.assertEqual(True, my_store.has_id('foo', 'sig'))
+
+    def test_has_suffixed_no_base(self):
+        my_store = self.get_populated_store()
+        self.assertEqual(False, my_store.has_id('missing'))
+        my_store = self.get_populated_store(True)
+        self.assertEqual(False, my_store.has_id('missing'))
+
+    def test_get_simple(self):
+        my_store = self.get_populated_store()
+        self.assertEqual('content', my_store.get('foo').read())
+        my_store = self.get_populated_store(True)
+        self.assertEqual('content', my_store.get('foo').read())
+
+    def test_get_suffixed(self):
+        my_store = self.get_populated_store()
+        self.assertEqual('signature', my_store.get('foo', 'sig').read())
+        my_store = self.get_populated_store(True)
+        self.assertEqual('signature', my_store.get('foo', 'sig').read())
+
+    def test_get_suffixed_no_base(self):
+        my_store = self.get_populated_store()
+        self.assertEqual('signature for missing base',
+                         my_store.get('missing', 'sig').read())
+        my_store = self.get_populated_store(True)
+        self.assertEqual('signature for missing base',
+                         my_store.get('missing', 'sig').read())
+
+    def test___iter__no_suffix(self):
+        my_store = TextStore(MemoryTransport(),
+                             prefixed=False, compressed=False)
+        stream = StringIO("content")
+        my_store.add(stream, "foo")
+        self.assertEqual(set(['foo']),
+                         set(my_store.__iter__()))
+
+    def test___iter__(self):
+        self.assertEqual(set(['foo']),
+                         set(self.get_populated_store().__iter__()))
+        self.assertEqual(set(['foo']),
+                         set(self.get_populated_store(True).__iter__()))
+
+    def test___iter__compressed(self):
+        self.assertEqual(set(['foo']),
+                         set(self.get_populated_store(
+                             compressed=True).__iter__()))
+        self.assertEqual(set(['foo']),
+                         set(self.get_populated_store(
+                             True, compressed=True).__iter__()))
+
+    def test___len__(self):
+        self.assertEqual(1, len(self.get_populated_store()))
+
+    def test_copy_suffixes(self):
+        from_store = self.get_populated_store()
+        to_store = TextStore(MemoryTransport(),
+                             prefixed=True, compressed=True)
+        to_store.register_suffix('sig')
+        copy_all(from_store, to_store)
+        self.assertEqual(1, len(to_store))
+        self.assertEqual(set(['foo']), set(to_store.__iter__()))
+        self.assertEqual('content', to_store.get('foo').read())
+        self.assertEqual('signature', to_store.get('foo', 'sig').read())
+        self.assertRaises(KeyError, to_store.get, 'missing', 'sig')
+
+    def test_relpath_escaped(self):
+        my_store = store.TransportStore(MemoryTransport())
+        self.assertEqual('%25', my_store._relpath('%'))

=== added file 'bzrlib/tests/test_symbol_versioning.py'
--- bzrlib/tests/test_symbol_versioning.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_symbol_versioning.py	2006-01-05 21:48:56 +0000
@@ -0,0 +1,91 @@
+# Copyright (C) 2006 by Canonical Ltd
+#   Authors: Robert Collins <robert.collins@canonical.com>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Symbol versioning tests."""
+
+import bzrlib.symbol_versioning as symbol_versioning
+from bzrlib.tests import TestCase
+
+
+@symbol_versioning.deprecated_function(symbol_versioning.zero_seven)
+def deprecated_function():
+    """Deprecated function docstring."""
+    return 1
+
+
+class TestDeprecationWarnings(TestCase):
+
+    def capture_warning(self, message, category):
+        self._warnings.append((message, category))
+
+    def setUp(self):
+        super(TestDeprecationWarnings, self).setUp()
+        self._warnings = []
+    
+    @symbol_versioning.deprecated_method(symbol_versioning.zero_seven)
+    def deprecated_method(self):
+        """Deprecated method docstring.
+        
+        This might explain stuff.
+        """
+        return 1
+
+    def test_deprecated_method(self):
+        expected_warning = (
+            "bzrlib.tests.test_symbol_versioning."
+            "TestDeprecationWarnings.deprecated_method "
+            "was deprecated in version 0.7.", DeprecationWarning)
+        expected_docstring = ('Deprecated method docstring.\n'
+                              '        \n'
+                              '        This might explain stuff.\n'
+                              '        \n'
+                              '        This method was deprecated in version 0.7.\n'
+                              '        ')
+        self.check_deprecated_callable(expected_warning, expected_docstring,
+                                       "deprecated_method",
+                                       "bzrlib.tests.test_symbol_versioning",
+                                       self.deprecated_method)
+
+    def test_deprecated_function(self):
+        expected_warning = (
+            "bzrlib.tests.test_symbol_versioning.deprecated_function "
+            "was deprecated in version 0.7.", DeprecationWarning)
+        expected_docstring = ('Deprecated function docstring.\n'
+                              '\n'
+                              'This function was deprecated in version 0.7.\n'
+                              )
+        self.check_deprecated_callable(expected_warning, expected_docstring,
+                                       "deprecated_function",
+                                       "bzrlib.tests.test_symbol_versioning",
+                                       deprecated_function)
+
+    def check_deprecated_callable(self, expected_warning, expected_docstring,
+                                  expected_name, expected_module,
+                                  deprecated_callable):
+        old_warning_method = symbol_versioning.warn
+        try:
+            symbol_versioning.set_warning_method(self.capture_warning)
+            self.assertEqual(1, deprecated_callable())
+            self.assertEqual([expected_warning], self._warnings)
+            deprecated_callable()
+            self.assertEqual([expected_warning, expected_warning],
+                             self._warnings)
+            self.assertEqualDiff(expected_docstring, deprecated_callable.__doc__)
+            self.assertEqualDiff(expected_name, deprecated_callable.__name__)
+            self.assertEqualDiff(expected_module, deprecated_callable.__module__)
+        finally:
+            symbol_versioning.set_warning_method(old_warning_method)

=== added file 'bzrlib/tests/test_testament.py'
--- bzrlib/tests/test_testament.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_testament.py	2005-11-30 00:25:50 +0000
@@ -0,0 +1,221 @@
+# Copyright (C) 2005 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Test testaments for gpg signing."""
+
+# TODO: Testaments with x-bits
+
+import os
+from sha import sha
+import sys
+
+from bzrlib.tests import TestCaseInTempDir
+from bzrlib.branch import Branch
+from bzrlib.testament import Testament
+from bzrlib.trace import mutter
+from bzrlib.osutils import has_symlinks
+
+
+class TestamentTests(TestCaseInTempDir):
+
+    def setUp(self):
+        super(TestamentTests, self).setUp()
+        b = self.b = Branch.initialize(u'.')
+        b.nick = "test branch"
+        b.working_tree().commit(message='initial null commit',
+                 committer='test@user',
+                 timestamp=1129025423, # 'Tue Oct 11 20:10:23 2005'
+                 timezone=0,
+                 rev_id='test@user-1')
+        self.build_tree_contents([('hello', 'contents of hello file'),
+                             ('src/', ),
+                             ('src/foo.c', 'int main()\n{\n}\n')])
+        b.working_tree().add(['hello', 'src', 'src/foo.c'],
+                             ['hello-id', 'src-id', 'foo.c-id'])
+        b.working_tree().commit(message='add files and directories',
+                 timestamp=1129025483,
+                 timezone=36000,
+                 rev_id='test@user-2',
+                 committer='test@user')
+
+    def test_null_testament(self):
+        """Testament for a revision with no contents."""
+        t = Testament.from_revision(self.b, 'test@user-1')
+        ass = self.assertTrue
+        eq = self.assertEqual
+        ass(isinstance(t, Testament))
+        eq(t.revision_id, 'test@user-1')
+        eq(t.committer, 'test@user')
+        eq(t.timestamp, 1129025423)
+        eq(t.timezone, 0)
+
+    def test_testment_text_form(self):
+        """Conversion of testament to canonical text form."""
+        t = Testament.from_revision(self.b, 'test@user-1')
+        text_form = t.as_text()
+        self.log('testament text form:\n' + text_form)
+        self.assertEqual(text_form, REV_1_TESTAMENT)
+
+    def test_testament_with_contents(self):
+        """Testament containing a file and a directory."""
+        t = Testament.from_revision(self.b, 'test@user-2')
+        text_form = t.as_text()
+        self.log('testament text form:\n' + text_form)
+        self.assertEqualDiff(text_form, REV_2_TESTAMENT)
+        actual_short = t.as_short_text()
+        self.assertEqualDiff(actual_short, REV_2_SHORT)
+
+    def test_testament_command(self):
+        """Testament containing a file and a directory."""
+        out, err = self.run_bzr_captured(['testament', '--long'])
+        self.assertEqualDiff(err, '')
+        self.assertEqualDiff(out, REV_2_TESTAMENT)
+
+    def test_testament_command_2(self):
+        """Command getting short testament of previous version."""
+        out, err = self.run_bzr_captured(['testament', '-r1'])
+        self.assertEqualDiff(err, '')
+        self.assertEqualDiff(out, REV_1_SHORT)
+
+    def test_testament_symlinks(self):
+        """Testament containing symlink (where possible)"""
+        if not has_symlinks():
+            return
+        os.symlink('wibble/linktarget', 'link')
+        self.b.working_tree().add(['link'], ['link-id'])
+        self.b.working_tree().commit(message='add symlink',
+                 timestamp=1129025493,
+                 timezone=36000,
+                 rev_id='test@user-3',
+                 committer='test@user')
+        t = Testament.from_revision(self.b, 'test@user-3')
+        self.assertEqualDiff(t.as_text(), REV_3_TESTAMENT)
+
+    def test_testament_revprops(self):
+        """Testament to revision with extra properties"""
+        props = dict(flavor='sour cherry\ncream cheese',
+                     size='medium')
+        self.b.working_tree().commit(message='revision with properties',
+                      timestamp=1129025493,
+                      timezone=36000,
+                      rev_id='test@user-3',
+                      committer='test@user',
+                      revprops=props)
+        t = Testament.from_revision(self.b, 'test@user-3')
+        self.assertEqualDiff(t.as_text(), REV_PROPS_TESTAMENT)
+
+    def test___init__(self):
+        revision = self.b.get_revision('test@user-2')
+        inventory = self.b.get_inventory('test@user-2')
+        testament_1 = Testament(revision, inventory).as_short_text()
+        testament_2 = Testament.from_revision(self.b, 
+                                              'test@user-2').as_short_text()
+        self.assertEqual(testament_1, testament_2)
+                    
+
+REV_1_TESTAMENT = """\
+bazaar-ng testament version 1
+revision-id: test@user-1
+committer: test@user
+timestamp: 1129025423
+timezone: 0
+parents:
+message:
+  initial null commit
+inventory:
+properties:
+  branch-nick:
+    test branch
+"""
+
+REV_1_SHORT = """\
+bazaar-ng testament short form 1
+revision-id: test@user-1
+sha1: %s
+""" % sha(REV_1_TESTAMENT).hexdigest()
+
+
+REV_2_TESTAMENT = """\
+bazaar-ng testament version 1
+revision-id: test@user-2
+committer: test@user
+timestamp: 1129025483
+timezone: 36000
+parents:
+  test@user-1
+message:
+  add files and directories
+inventory:
+  file hello hello-id 34dd0ac19a24bf80c4d33b5c8960196e8d8d1f73
+  directory src src-id
+  file src/foo.c foo.c-id a2a049c20f908ae31b231d98779eb63c66448f24
+properties:
+  branch-nick:
+    test branch
+"""
+
+
+REV_2_SHORT = """\
+bazaar-ng testament short form 1
+revision-id: test@user-2
+sha1: %s
+""" % sha(REV_2_TESTAMENT).hexdigest()
+
+
+REV_PROPS_TESTAMENT = """\
+bazaar-ng testament version 1
+revision-id: test@user-3
+committer: test@user
+timestamp: 1129025493
+timezone: 36000
+parents:
+  test@user-2
+message:
+  revision with properties
+inventory:
+  file hello hello-id 34dd0ac19a24bf80c4d33b5c8960196e8d8d1f73
+  directory src src-id
+  file src/foo.c foo.c-id a2a049c20f908ae31b231d98779eb63c66448f24
+properties:
+  branch-nick:
+    test branch
+  flavor:
+    sour cherry
+    cream cheese
+  size:
+    medium
+"""
+
+
+REV_3_TESTAMENT = """\
+bazaar-ng testament version 1
+revision-id: test@user-3
+committer: test@user
+timestamp: 1129025493
+timezone: 36000
+parents:
+  test@user-2
+message:
+  add symlink
+inventory:
+  file hello hello-id 34dd0ac19a24bf80c4d33b5c8960196e8d8d1f73
+  symlink link link-id wibble/linktarget
+  directory src src-id
+  file src/foo.c foo.c-id a2a049c20f908ae31b231d98779eb63c66448f24
+properties:
+  branch-nick:
+    test branch
+"""

=== added file 'bzrlib/tests/test_trace.py'
--- bzrlib/tests/test_trace.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_trace.py	2005-11-29 07:58:36 +0000
@@ -0,0 +1,68 @@
+# Copyright (C) 2005 by Canonical Ltd
+#   Authors: Robert Collins <robert.collins@canonical.com>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+# "weren't nothing promised to you.  do i look like i got a promise face?"
+
+"""Tests for trace library"""
+
+import os
+import sys
+
+from bzrlib.tests import TestCaseInTempDir, TestCase
+from bzrlib.trace import format_exception_short, mutter
+from bzrlib.errors import NotBranchError, BzrError, BzrNewError
+
+class TestTrace(TestCase):
+    def test_format_sys_exception(self):
+        """Short formatting of exceptions"""
+        try:
+            raise NotImplementedError, "time travel"
+        except NotImplementedError:
+            pass
+        error_lines = format_exception_short(sys.exc_info()).splitlines()
+        self.assertEqualDiff(error_lines[0], 
+                'exceptions.NotImplementedError: time travel')
+        self.assertContainsRe(error_lines[1], 
+                r'^  at .*trace\.py line \d+$')  
+        self.assertContainsRe(error_lines[2], 
+                r'^  in test_format_sys_exception$')
+
+    def test_format_exception(self):
+        """Short formatting of exceptions"""
+        try:
+            raise NotBranchError, 'wibble'
+        except NotBranchError:
+            pass
+        msg = format_exception_short(sys.exc_info())
+        self.assertEqualDiff(msg, 'Not a branch: wibble')
+
+    def test_format_old_exception(self):
+        # format a class that doesn't descend from BzrNewError; 
+        # remove this test when everything is unified there
+        self.assertFalse(issubclass(BzrError, BzrNewError))
+        try:
+            raise BzrError('some old error')
+        except BzrError:
+            pass
+        msg = format_exception_short(sys.exc_info())
+        self.assertEqualDiff(msg, 'some old error')
+
+    def test_trace_unicode(self):
+        """Write Unicode to trace log"""
+        self.log(u'the unicode character for benzene is \N{BENZENE RING}')
+        self.assertContainsRe('the unicode character',
+                self._get_log())

=== added file 'bzrlib/tests/test_transactions.py'
--- bzrlib/tests/test_transactions.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_transactions.py	2005-11-23 15:44:24 +0000
@@ -0,0 +1,192 @@
+# Copyright (C) 2005 by Canonical Ltd
+#   Authors: Robert Collins <robert.collins@canonical.com>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for the behaviour of the Transaction concept in bzr."""
+
+# import system imports here
+import os
+import sys
+
+#import bzrlib specific imports here
+import bzrlib.errors as errors
+from bzrlib.tests import TestCase, TestCaseInTempDir
+import bzrlib.transactions as transactions
+
+
+class DummyWeave(object):
+    """A class that can be instantiated and compared."""
+
+    def __init__(self, message):
+        self._message = message
+
+    def __eq__(self, other):
+        if other is None:
+            return False
+        return self._message == other._message
+
+
+class TestSymbols(TestCase):
+
+    def test_public_symbols(self):
+        from bzrlib.transactions import ReadOnlyTransaction
+        from bzrlib.transactions import PassThroughTransaction
+
+
+class TestReadOnlyTransaction(TestCase):
+
+    def setUp(self):
+        self.transaction = transactions.ReadOnlyTransaction()
+        super(TestReadOnlyTransaction, self).setUp()
+
+    def test_register_clean(self):
+        self.transaction.register_clean("anobject")
+
+    def test_register_dirty_raises(self):
+        self.assertRaises(errors.ReadOnlyError, 
+                          self.transaction.register_dirty,"anobject")
+    
+    def test_commit_raises(self):
+        self.assertRaises(errors.CommitNotPossible, self.transaction.commit)
+
+    def test_map(self):
+        self.assertNotEqual(None, getattr(self.transaction, "map", None))
+    
+    def test_add_and_get(self):
+        weave = "a weave"
+        self.transaction.map.add_weave("id", weave)
+        self.assertEqual(weave, self.transaction.map.find_weave("id"))
+
+    def test_rollback_returns(self):
+        self.transaction.rollback()
+
+    def test_finish_returns(self):
+        self.transaction.finish()
+
+    def test_zero_size_cache(self):
+        self.transaction.set_cache_size(0)
+        weave = DummyWeave('a weave')
+        self.transaction.map.add_weave("id", weave)
+        self.assertEqual(weave, self.transaction.map.find_weave("id"))
+        weave = None
+        # add an object, should fall right out if there are no references
+        self.transaction.register_clean(self.transaction.map.find_weave("id"))
+        self.assertEqual(None, self.transaction.map.find_weave("id"))
+        # but if we have a reference it should stick around
+        weave = DummyWeave("another weave")
+        self.transaction.map.add_weave("id", weave)
+        self.transaction.register_clean(self.transaction.map.find_weave("id"))
+        self.assertEqual(weave, self.transaction.map.find_weave("id"))
+        del weave
+        # its not a weakref system
+        self.assertEqual(DummyWeave("another weave"),
+                         self.transaction.map.find_weave("id"))
+        
+    def test_small_cache(self):
+        self.transaction.set_cache_size(1)
+        # add an object, should not fall right out if there are no references
+        #sys.getrefcounts(foo)
+        self.transaction.map.add_weave("id", DummyWeave("a weave"))
+        self.transaction.register_clean(self.transaction.map.find_weave("id"))
+        self.assertEqual(DummyWeave("a weave"),
+                         self.transaction.map.find_weave("id"))
+        self.transaction.map.add_weave("id2", DummyWeave("a weave also"))
+        self.transaction.register_clean(self.transaction.map.find_weave("id2"))
+        # currently a fifo
+        self.assertEqual(None, self.transaction.map.find_weave("id"))
+        self.assertEqual(DummyWeave("a weave also"),
+                         self.transaction.map.find_weave("id2"))
+
+    def test_small_cache_with_references(self):
+        # if we have a reference it should stick around
+        weave = "a weave"
+        weave2 = "another weave"
+        self.transaction.map.add_weave("id", weave)
+        self.transaction.map.add_weave("id2", weave2)
+        self.assertEqual(weave, self.transaction.map.find_weave("id"))
+        self.assertEqual(weave2, self.transaction.map.find_weave("id2"))
+        weave = None
+        # its not a weakref system
+        self.assertEqual("a weave", self.transaction.map.find_weave("id"))
+
+    def test_precious_with_zero_size_cache(self):
+        self.transaction.set_cache_size(0)
+        weave = DummyWeave('a weave')
+        self.transaction.map.add_weave("id", weave)
+        self.assertEqual(weave, self.transaction.map.find_weave("id"))
+        weave = None
+        # add an object, should not fall out even with no references.
+        self.transaction.register_clean(self.transaction.map.find_weave("id"),
+                                        precious=True)
+        self.assertEqual(DummyWeave('a weave'),
+                         self.transaction.map.find_weave("id"))
+
+    def test_precious_revision_history(self):
+        """Disabled test until revision-history is a real object."""
+        print "Disabled: test_precious_revision_history"
+        return
+        self.transaction.set_cache_size(0)
+        history = []
+        self.transaction.map.add_revision_history(history)
+        self.assertEqual(history, self.transaction.map.find_revision_history())
+        history = None
+        # add an object, should not fall out even with no references.
+        self.transaction.register_clean(
+            self.transaction.map.find_revision_history(), precious=True)
+        self.assertEqual([], self.transaction.map.find_revision_history())
+
+
+class TestPassThroughTransaction(TestCase):
+
+    def test_construct(self):
+        transactions.PassThroughTransaction()
+
+    def test_register_clean(self):
+        transaction = transactions.PassThroughTransaction()
+        transaction.register_clean("anobject")
+    
+    def test_register_dirty(self):
+        transaction = transactions.PassThroughTransaction()
+        transaction.register_dirty("anobject")
+    
+    def test_commit_nothing_returns(self):
+        transaction = transactions.PassThroughTransaction()
+        transaction.commit()
+
+    def test_map(self):
+        transaction = transactions.PassThroughTransaction()
+        self.assertNotEqual(None, getattr(transaction, "map", None))
+    
+    def test_add_and_get(self):
+        transaction = transactions.PassThroughTransaction()
+        weave = "a weave"
+        transaction.map.add_weave("id", weave)
+        self.assertEqual(None, transaction.map.find_weave("id"))
+        
+    def test_rollback_asserts(self):
+        transaction = transactions.PassThroughTransaction()
+        self.assertRaises(errors.AlreadyCommitted, transaction.rollback)
+
+    def test_finish_returns(self):
+        transaction = transactions.PassThroughTransaction()
+        transaction.finish()
+
+    def test_cache_is_ignored(self):
+        transaction = transactions.PassThroughTransaction()
+        transaction.set_cache_size(100)
+        weave = "a weave"
+        transaction.map.add_weave("id", weave)
+        self.assertEqual(None, transaction.map.find_weave("id"))

=== renamed file 'bzrlib/selftest/testtransport.py' => 'bzrlib/tests/test_transport.py'
--- bzrlib/selftest/testtransport.py	2005-09-19 03:26:14 +0000
+++ bzrlib/tests/test_transport.py	2006-01-10 05:45:16 +0000
@@ -16,500 +16,57 @@
 
 
 import os
+import sys
+import stat
 from cStringIO import StringIO
-from bzrlib.selftest import TestCaseInTempDir
-from bzrlib.selftest.HTTPTestUtil import TestCaseWithWebserver
-from bzrlib.errors import NoSuchFile, FileExists, TransportNotPossible
-
-def _append(fn, txt):
-    """Append the given text (file-like object) to the supplied filename."""
-    f = open(fn, 'ab')
-    f.write(txt)
-    f.flush()
-    f.close()
-    del f
-
-class TestTransportMixIn(object):
-    """Subclass this, and it will provide a series of tests for a Transport.
-    It assumes that the Transport object is connected to the 
-    current working directory.  So that whatever is done 
-    through the transport, should show up in the working 
-    directory, and vice-versa.
-
-    This also tests to make sure that the functions work with both
-    generators and lists (assuming iter(list) is effectively a generator)
-    """
-    readonly = False
-    def get_transport(self):
-        """Children should override this to return the Transport object.
-        """
-        raise NotImplementedError
-
-    def test_has(self):
-        t = self.get_transport()
-
-        files = ['a', 'b', 'e', 'g']
-        self.build_tree(files)
-        self.assertEqual(t.has('a'), True)
-        self.assertEqual(t.has('c'), False)
-        self.assertEqual(list(t.has_multi(['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h'])),
-                [True, True, False, False, True, False, True, False])
-        self.assertEqual(list(t.has_multi(iter(['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h']))),
-                [True, True, False, False, True, False, True, False])
-
-    def test_get(self):
-        t = self.get_transport()
-
-        files = ['a', 'b', 'e', 'g']
-        self.build_tree(files)
-        self.assertEqual(t.get('a').read(), open('a').read())
-        content_f = t.get_multi(files)
-        for path,f in zip(files, content_f):
-            self.assertEqual(open(path).read(), f.read())
-
-        content_f = t.get_multi(iter(files))
-        for path,f in zip(files, content_f):
-            self.assertEqual(open(path).read(), f.read())
-
-        self.assertRaises(NoSuchFile, t.get, 'c')
-        try:
-            files = list(t.get_multi(['a', 'b', 'c']))
-        except NoSuchFile:
-            pass
-        else:
-            self.fail('Failed to raise NoSuchFile for missing file in get_multi')
-        try:
-            files = list(t.get_multi(iter(['a', 'b', 'c', 'e'])))
-        except NoSuchFile:
-            pass
-        else:
-            self.fail('Failed to raise NoSuchFile for missing file in get_multi')
-
-    def test_put(self):
-        t = self.get_transport()
-
-        if self.readonly:
-            self.assertRaises(TransportNotPossible,
-                    t.put, 'a', 'some text for a\n')
-            open('a', 'wb').write('some text for a\n')
-        else:
-            t.put('a', 'some text for a\n')
-        self.assert_(os.path.exists('a'))
-        self.check_file_contents('a', 'some text for a\n')
-        self.assertEqual(t.get('a').read(), 'some text for a\n')
-        # Make sure 'has' is updated
-        self.assertEqual(list(t.has_multi(['a', 'b', 'c', 'd', 'e'])),
-                [True, False, False, False, False])
-        if self.readonly:
-            self.assertRaises(TransportNotPossible,
-                    t.put_multi,
-                    [('a', 'new\ncontents for\na\n'),
-                        ('d', 'contents\nfor d\n')])
-            open('a', 'wb').write('new\ncontents for\na\n')
-            open('d', 'wb').write('contents\nfor d\n')
-        else:
-            # Put also replaces contents
-            self.assertEqual(t.put_multi([('a', 'new\ncontents for\na\n'),
-                                          ('d', 'contents\nfor d\n')]),
-                             2)
-        self.assertEqual(list(t.has_multi(['a', 'b', 'c', 'd', 'e'])),
-                [True, False, False, True, False])
-        self.check_file_contents('a', 'new\ncontents for\na\n')
-        self.check_file_contents('d', 'contents\nfor d\n')
-
-        if self.readonly:
-            self.assertRaises(TransportNotPossible,
-                t.put_multi, iter([('a', 'diff\ncontents for\na\n'),
-                                  ('d', 'another contents\nfor d\n')]))
-            open('a', 'wb').write('diff\ncontents for\na\n')
-            open('d', 'wb').write('another contents\nfor d\n')
-        else:
-            self.assertEqual(
-                t.put_multi(iter([('a', 'diff\ncontents for\na\n'),
-                                  ('d', 'another contents\nfor d\n')]))
-                             , 2)
-        self.check_file_contents('a', 'diff\ncontents for\na\n')
-        self.check_file_contents('d', 'another contents\nfor d\n')
-
-        if self.readonly:
-            self.assertRaises(TransportNotPossible,
-                    t.put, 'path/doesnt/exist/c', 'contents')
-        else:
-            self.assertRaises(NoSuchFile,
-                    t.put, 'path/doesnt/exist/c', 'contents')
-
-    def test_put_file(self):
-        t = self.get_transport()
-
-        # Test that StringIO can be used as a file-like object with put
-        f1 = StringIO('this is a string\nand some more stuff\n')
-        if self.readonly:
-            open('f1', 'wb').write(f1.read())
-        else:
-            t.put('f1', f1)
-
-        del f1
-
-        self.check_file_contents('f1', 
-                'this is a string\nand some more stuff\n')
-
-        f2 = StringIO('here is some text\nand a bit more\n')
-        f3 = StringIO('some text for the\nthird file created\n')
-
-        if self.readonly:
-            open('f2', 'wb').write(f2.read())
-            open('f3', 'wb').write(f3.read())
-        else:
-            t.put_multi([('f2', f2), ('f3', f3)])
-
-        del f2, f3
-
-        self.check_file_contents('f2', 'here is some text\nand a bit more\n')
-        self.check_file_contents('f3', 'some text for the\nthird file created\n')
-
-        # Test that an actual file object can be used with put
-        f4 = open('f1', 'rb')
-        if self.readonly:
-            open('f4', 'wb').write(f4.read())
-        else:
-            t.put('f4', f4)
-
-        del f4
-
-        self.check_file_contents('f4', 
-                'this is a string\nand some more stuff\n')
-
-        f5 = open('f2', 'rb')
-        f6 = open('f3', 'rb')
-        if self.readonly:
-            open('f5', 'wb').write(f5.read())
-            open('f6', 'wb').write(f6.read())
-        else:
-            t.put_multi([('f5', f5), ('f6', f6)])
-
-        del f5, f6
-
-        self.check_file_contents('f5', 'here is some text\nand a bit more\n')
-        self.check_file_contents('f6', 'some text for the\nthird file created\n')
-
-
-
-    def test_mkdir(self):
-        t = self.get_transport()
-
-        # Test mkdir
-        os.mkdir('dir_a')
-        self.assertEqual(t.has('dir_a'), True)
-        self.assertEqual(t.has('dir_b'), False)
-
-        if self.readonly:
-            self.assertRaises(TransportNotPossible,
-                    t.mkdir, 'dir_b')
-            os.mkdir('dir_b')
-        else:
-            t.mkdir('dir_b')
-        self.assertEqual(t.has('dir_b'), True)
-        self.assert_(os.path.isdir('dir_b'))
-
-        if self.readonly:
-            self.assertRaises(TransportNotPossible,
-                    t.mkdir_multi, ['dir_c', 'dir_d'])
-            os.mkdir('dir_c')
-            os.mkdir('dir_d')
-        else:
-            t.mkdir_multi(['dir_c', 'dir_d'])
-
-        if self.readonly:
-            self.assertRaises(TransportNotPossible,
-                    t.mkdir_multi, iter(['dir_e', 'dir_f']))
-            os.mkdir('dir_e')
-            os.mkdir('dir_f')
-        else:
-            t.mkdir_multi(iter(['dir_e', 'dir_f']))
-        self.assertEqual(list(t.has_multi(
-            ['dir_a', 'dir_b', 'dir_c', 'dir_q',
-             'dir_d', 'dir_e', 'dir_f', 'dir_b'])),
-            [True, True, True, False,
-             True, True, True, True])
-        for d in ['dir_a', 'dir_b', 'dir_c', 'dir_d', 'dir_e', 'dir_f']:
-            self.assert_(os.path.isdir(d))
-
-        if not self.readonly:
-            self.assertRaises(NoSuchFile, t.mkdir, 'path/doesnt/exist')
-            self.assertRaises(FileExists, t.mkdir, 'dir_a') # Creating a directory again should fail
-
-        # Make sure the transport recognizes when a
-        # directory is created by other means
-        # Caching Transports will fail, because dir_e was already seen not
-        # to exist. So instead, we will search for a new directory
-        #os.mkdir('dir_e')
-        #if not self.readonly:
-        #    self.assertRaises(FileExists, t.mkdir, 'dir_e')
-
-        os.mkdir('dir_g')
-        if not self.readonly:
-            self.assertRaises(FileExists, t.mkdir, 'dir_g')
-
-        # Test get/put in sub-directories
-        if self.readonly:
-            open('dir_a/a', 'wb').write('contents of dir_a/a')
-            open('dir_b/b', 'wb').write('contents of dir_b/b')
-        else:
-            self.assertEqual(
-                t.put_multi([('dir_a/a', 'contents of dir_a/a'),
-                             ('dir_b/b', 'contents of dir_b/b')])
-                          , 2)
-        for f in ('dir_a/a', 'dir_b/b'):
-            self.assertEqual(t.get(f).read(), open(f).read())
-
-    def test_copy_to(self):
-        import tempfile
-        from bzrlib.transport.local import LocalTransport
-
-        t = self.get_transport()
-
-        files = ['a', 'b', 'c', 'd']
-        self.build_tree(files)
-
-        dtmp = tempfile.mkdtemp(dir='.', prefix='test-transport-')
-        dtmp_base = os.path.basename(dtmp)
-        local_t = LocalTransport(dtmp)
-
-        t.copy_to(files, local_t)
-        for f in files:
-            self.assertEquals(open(f).read(),
-                    open(os.path.join(dtmp_base, f)).read())
-
-        del dtmp, dtmp_base, local_t
-
-        dtmp = tempfile.mkdtemp(dir='.', prefix='test-transport-')
-        dtmp_base = os.path.basename(dtmp)
-        local_t = LocalTransport(dtmp)
-
-        files = ['a', 'b', 'c', 'd']
-        t.copy_to(iter(files), local_t)
-        for f in files:
-            self.assertEquals(open(f).read(),
-                    open(os.path.join(dtmp_base, f)).read())
-
-        del dtmp, dtmp_base, local_t
-
-    def test_append(self):
-        t = self.get_transport()
-
-        if self.readonly:
-            open('a', 'wb').write('diff\ncontents for\na\n')
-            open('b', 'wb').write('contents\nfor b\n')
-        else:
-            t.put_multi([
-                    ('a', 'diff\ncontents for\na\n'),
-                    ('b', 'contents\nfor b\n')
-                    ])
-
-        if self.readonly:
-            self.assertRaises(TransportNotPossible,
-                    t.append, 'a', 'add\nsome\nmore\ncontents\n')
-            _append('a', 'add\nsome\nmore\ncontents\n')
-        else:
-            t.append('a', 'add\nsome\nmore\ncontents\n')
-
-        self.check_file_contents('a', 
-            'diff\ncontents for\na\nadd\nsome\nmore\ncontents\n')
-
-        if self.readonly:
-            self.assertRaises(TransportNotPossible,
-                    t.append_multi,
-                        [('a', 'and\nthen\nsome\nmore\n'),
-                         ('b', 'some\nmore\nfor\nb\n')])
-            _append('a', 'and\nthen\nsome\nmore\n')
-            _append('b', 'some\nmore\nfor\nb\n')
-        else:
-            t.append_multi([('a', 'and\nthen\nsome\nmore\n'),
-                    ('b', 'some\nmore\nfor\nb\n')])
-        self.check_file_contents('a', 
-            'diff\ncontents for\na\n'
-            'add\nsome\nmore\ncontents\n'
-            'and\nthen\nsome\nmore\n')
-        self.check_file_contents('b', 
-                'contents\nfor b\n'
-                'some\nmore\nfor\nb\n')
-
-        if self.readonly:
-            _append('a', 'a little bit more\n')
-            _append('b', 'from an iterator\n')
-        else:
-            t.append_multi(iter([('a', 'a little bit more\n'),
-                    ('b', 'from an iterator\n')]))
-        self.check_file_contents('a', 
-            'diff\ncontents for\na\n'
-            'add\nsome\nmore\ncontents\n'
-            'and\nthen\nsome\nmore\n'
-            'a little bit more\n')
-        self.check_file_contents('b', 
-                'contents\nfor b\n'
-                'some\nmore\nfor\nb\n'
-                'from an iterator\n')
-
-    def test_append_file(self):
-        t = self.get_transport()
-
-        contents = [
-            ('f1', 'this is a string\nand some more stuff\n'),
-            ('f2', 'here is some text\nand a bit more\n'),
-            ('f3', 'some text for the\nthird file created\n'),
-            ('f4', 'this is a string\nand some more stuff\n'),
-            ('f5', 'here is some text\nand a bit more\n'),
-            ('f6', 'some text for the\nthird file created\n')
-        ]
-        
-        if self.readonly:
-            for f, val in contents:
-                open(f, 'wb').write(val)
-        else:
-            t.put_multi(contents)
-
-        a1 = StringIO('appending to\none\n')
-        if self.readonly:
-            _append('f1', a1.read())
-        else:
-            t.append('f1', a1)
-
-        del a1
-
-        self.check_file_contents('f1', 
-                'this is a string\nand some more stuff\n'
-                'appending to\none\n')
-
-        a2 = StringIO('adding more\ntext to two\n')
-        a3 = StringIO('some garbage\nto put in three\n')
-
-        if self.readonly:
-            _append('f2', a2.read())
-            _append('f3', a3.read())
-        else:
-            t.append_multi([('f2', a2), ('f3', a3)])
-
-        del a2, a3
-
-        self.check_file_contents('f2',
-                'here is some text\nand a bit more\n'
-                'adding more\ntext to two\n')
-        self.check_file_contents('f3', 
-                'some text for the\nthird file created\n'
-                'some garbage\nto put in three\n')
-
-        # Test that an actual file object can be used with put
-        a4 = open('f1', 'rb')
-        if self.readonly:
-            _append('f4', a4.read())
-        else:
-            t.append('f4', a4)
-
-        del a4
-
-        self.check_file_contents('f4', 
-                'this is a string\nand some more stuff\n'
-                'this is a string\nand some more stuff\n'
-                'appending to\none\n')
-
-        a5 = open('f2', 'rb')
-        a6 = open('f3', 'rb')
-        if self.readonly:
-            _append('f5', a5.read())
-            _append('f6', a6.read())
-        else:
-            t.append_multi([('f5', a5), ('f6', a6)])
-
-        del a5, a6
-
-        self.check_file_contents('f5',
-                'here is some text\nand a bit more\n'
-                'here is some text\nand a bit more\n'
-                'adding more\ntext to two\n')
-        self.check_file_contents('f6',
-                'some text for the\nthird file created\n'
-                'some text for the\nthird file created\n'
-                'some garbage\nto put in three\n')
-
-    def test_get_partial(self):
-        t = self.get_transport()
-
-        contents = [
-            ('f1', 
-                'here is some text\nand a bit more\n'
-                'adding more\ntext to two\n'),
-            ('f2',
-                'this is a string\nand some more stuff\n'
-                'appending to\none\n'),
-            ('f3',
-                'some text for the\nthird file created\n'
-                'some garbage\nto put in three\n')
-        ]
-        if self.readonly:
-            for f, val in contents:
-                open(f, 'wb').write(val)
-        else:
-            t.put_multi(contents)
-
-        self.assertRaises(NoSuchFile,
-                t.get_partial, 'a-missing-file', 20)
-        self.assertRaises(NoSuchFile,
-                t.get_partial, 'another-missing-file', 20, 30)
-        f = t.get_partial('f1', 33)
-        self.assertEqual(f.read(), 
-                'adding more\ntext to two\n')
-        f = t.get_partial('f1', 33, 10)
-        self.assertEqual(f.read(10), 
-                'adding mor')
-
-        del f
-
-        offsets = [('f2', 37), ('f3', 20, 10), ('f1', 10, 20)]
-        values = ['appending to\none\n',
-                  'ird file c',
-                  'me text\nand a bit mo'
-                 ]
-        contents_f = t.get_partial_multi(offsets)
-        count = 0
-        for f, val in zip(contents_f, values):
-            count += 1
-            self.assertEqual(val, f.read(len(val)))
-        # Make sure we saw all values, and no extra
-        self.assertEqual(len(values), count)
-        self.assertEqual(list(contents_f), [])
-
-        # Do the same thing with an iterator
-        offsets = iter([('f2', 34), ('f3', 18, 10), ('f1', 15, 15)])
-        values = ['ff\nappending to\none\n',
-                  'third file',
-                  'xt\nand a bit mo'
-                 ]
-        contents_f = t.get_partial_multi(offsets)
-        count = 0
-        for f, val in zip(contents_f, values):
-            count += 1
-            self.assertEqual(val, f.read(len(val)))
-        self.assertEqual(len(values), count)
-        self.assertEqual(list(contents_f), [])
-
-
-    def test_delete(self):
-        # TODO: Test Transport.delete
-        pass
-
-    def test_move(self):
-        # TODO: Test Transport.move
-        pass
-
-class LocalTransportTest(TestCaseInTempDir, TestTransportMixIn):
-    def get_transport(self):
-        from bzrlib.transport.local import LocalTransport
-        return LocalTransport('.')
-
-class HttpTransportTest(TestCaseWithWebserver, TestTransportMixIn):
-    readonly = True
-    def get_transport(self):
-        from bzrlib.transport.http import HttpTransport
-        url = self.get_remote_url('.')
-        return HttpTransport(url)
-
+
+from bzrlib.errors import (NoSuchFile, FileExists,
+                           TransportNotPossible, ConnectionError)
+from bzrlib.tests import TestCase
+from bzrlib.transport import (_get_protocol_handlers,
+                              _get_transport_modules,
+                              register_lazy_transport,
+                              _set_protocol_handlers,
+                              urlescape,
+                              )
+
+
+class TestTransport(TestCase):
+    """Test the non transport-concrete class functionality."""
+
+    def test_urlescape(self):
+        self.assertEqual('%25', urlescape('%'))
+
+    def test__get_set_protocol_handlers(self):
+        handlers = _get_protocol_handlers()
+        self.assertNotEqual({}, handlers)
+        try:
+            _set_protocol_handlers({})
+            self.assertEqual({}, _get_protocol_handlers())
+        finally:
+            _set_protocol_handlers(handlers)
+
+    def test_get_transport_modules(self):
+        handlers = _get_protocol_handlers()
+        class SampleHandler(object):
+            """I exist, isnt that enough?"""
+        try:
+            my_handlers = {}
+            _set_protocol_handlers(my_handlers)
+            register_lazy_transport('foo', 'bzrlib.tests.test_transport', 'TestTransport.SampleHandler')
+            register_lazy_transport('bar', 'bzrlib.tests.test_transport', 'TestTransport.SampleHandler')
+            self.assertEqual([SampleHandler.__module__],
+                             _get_transport_modules())
+        finally:
+            _set_protocol_handlers(handlers)
+            
+
+class MemoryTransportTest(TestCase):
+    """Memory transport specific tests."""
+
+    def test_parameters(self):
+        import bzrlib.transport.memory as memory
+        transport = memory.MemoryTransport()
+        self.assertEqual(True, transport.listable())
+        self.assertEqual(False, transport.should_cache())
+        self.assertEqual(False, transport.is_readonly())

=== added file 'bzrlib/tests/test_transport_implementations.py'
--- bzrlib/tests/test_transport_implementations.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_transport_implementations.py	2006-01-12 01:02:32 +0000
@@ -0,0 +1,732 @@
+# Copyright (C) 2004, 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for Transport implementations.
+
+Transport implementations tested here are supplied by
+TransportTestProviderAdapter.
+"""
+
+import os
+from cStringIO import StringIO
+import stat
+import sys
+
+from bzrlib.errors import (NoSuchFile, FileExists,
+                           TransportNotPossible, ConnectionError)
+from bzrlib.tests import TestCaseInTempDir, TestSkipped
+from bzrlib.transport import memory, urlescape
+import bzrlib.transport
+
+
+def _append(fn, txt):
+    """Append the given text (file-like object) to the supplied filename."""
+    f = open(fn, 'ab')
+    try:
+        f.write(txt.read())
+    finally:
+        f.close()
+
+
+class TestTransportImplementation(TestCaseInTempDir):
+    """Implementation verification for transports.
+    
+    To verify a transport we need a server factory, which is a callable
+    that accepts no parameters and returns an implementation of
+    bzrlib.transport.Server.
+    
+    That Server is then used to construct transport instances and test
+    the transport via loopback activity.
+
+    Currently this assumes that the Transport object is connected to the 
+    current working directory.  So that whatever is done 
+    through the transport, should show up in the working 
+    directory, and vice-versa. This is a bug, because its possible to have
+    URL schemes which provide access to something that may not be 
+    result in storage on the local disk, i.e. due to file system limits, or 
+    due to it being a database or some other non-filesystem tool.
+
+    This also tests to make sure that the functions work with both
+    generators and lists (assuming iter(list) is effectively a generator)
+    """
+    
+    def setUp(self):
+        super(TestTransportImplementation, self).setUp()
+        self._server = self.transport_server()
+        self._server.setUp()
+
+    def tearDown(self):
+        super(TestTransportImplementation, self).tearDown()
+        self._server.tearDown()
+        
+    def check_transport_contents(self, content, transport, relpath):
+        """Check that transport.get(relpath).read() == content."""
+        self.assertEqualDiff(content, transport.get(relpath).read())
+
+    def get_transport(self):
+        """Return a connected transport to the local directory."""
+        t = bzrlib.transport.get_transport(self._server.get_url())
+        self.failUnless(isinstance(t, self.transport_class), 
+                        "Got the wrong class from get_transport"
+                        "(%r, expected %r)" % (t.__class__, 
+                                               self.transport_class))
+        return t
+
+    def assertListRaises(self, excClass, func, *args, **kwargs):
+        """Fail unless excClass is raised when the iterator from func is used.
+        
+        Many transport functions can return generators this makes sure
+        to wrap them in a list() call to make sure the whole generator
+        is run, and that the proper exception is raised.
+        """
+        try:
+            list(func(*args, **kwargs))
+        except excClass:
+            return
+        else:
+            if hasattr(excClass,'__name__'): excName = excClass.__name__
+            else: excName = str(excClass)
+            raise self.failureException, "%s not raised" % excName
+
+    def test_has(self):
+        t = self.get_transport()
+
+        files = ['a', 'b', 'e', 'g', '%']
+        self.build_tree(files, transport=t)
+        self.assertEqual(True, t.has('a'))
+        self.assertEqual(False, t.has('c'))
+        self.assertEqual(True, t.has(urlescape('%')))
+        self.assertEqual(list(t.has_multi(['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h'])),
+                [True, True, False, False, True, False, True, False])
+        self.assertEqual(True, t.has_any(['a', 'b', 'c']))
+        self.assertEqual(False, t.has_any(['c', 'd', 'f', urlescape('%%')]))
+        self.assertEqual(list(t.has_multi(iter(['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h']))),
+                [True, True, False, False, True, False, True, False])
+        self.assertEqual(False, t.has_any(['c', 'c', 'c']))
+        self.assertEqual(True, t.has_any(['b', 'b', 'b']))
+
+    def test_get(self):
+        t = self.get_transport()
+
+        files = ['a', 'b', 'e', 'g']
+        contents = ['contents of a\n',
+                    'contents of b\n',
+                    'contents of e\n',
+                    'contents of g\n',
+                    ]
+        self.build_tree(files, transport=t)
+        self.check_transport_contents('contents of a\n', t, 'a')
+        content_f = t.get_multi(files)
+        for content, f in zip(contents, content_f):
+            self.assertEqual(content, f.read())
+
+        content_f = t.get_multi(iter(files))
+        for content, f in zip(contents, content_f):
+            self.assertEqual(content, f.read())
+
+        self.assertRaises(NoSuchFile, t.get, 'c')
+        self.assertListRaises(NoSuchFile, t.get_multi, ['a', 'b', 'c'])
+        self.assertListRaises(NoSuchFile, t.get_multi, iter(['a', 'b', 'c']))
+
+    def test_put(self):
+        t = self.get_transport()
+
+        if t.is_readonly():
+            self.assertRaises(TransportNotPossible,
+                    t.put, 'a', 'some text for a\n')
+            return
+
+        t.put('a', StringIO('some text for a\n'))
+        self.failUnless(t.has('a'))
+        self.check_transport_contents('some text for a\n', t, 'a')
+        # Make sure 'has' is updated
+        self.assertEqual(list(t.has_multi(['a', 'b', 'c', 'd', 'e'])),
+                [True, False, False, False, False])
+        # Put also replaces contents
+        self.assertEqual(t.put_multi([('a', StringIO('new\ncontents for\na\n')),
+                                      ('d', StringIO('contents\nfor d\n'))]),
+                         2)
+        self.assertEqual(list(t.has_multi(['a', 'b', 'c', 'd', 'e'])),
+                [True, False, False, True, False])
+        self.check_transport_contents('new\ncontents for\na\n', t, 'a')
+        self.check_transport_contents('contents\nfor d\n', t, 'd')
+
+        self.assertEqual(
+            t.put_multi(iter([('a', StringIO('diff\ncontents for\na\n')),
+                              ('d', StringIO('another contents\nfor d\n'))])),
+                        2)
+        self.check_transport_contents('diff\ncontents for\na\n', t, 'a')
+        self.check_transport_contents('another contents\nfor d\n', t, 'd')
+
+        self.assertRaises(NoSuchFile,
+                          t.put, 'path/doesnt/exist/c', 'contents')
+
+    def test_put_permissions(self):
+        t = self.get_transport()
+
+        if t.is_readonly():
+            return
+        t.put('mode644', StringIO('test text\n'), mode=0644)
+        self.assertTransportMode(t, 'mode644', 0644)
+        t.put('mode666', StringIO('test text\n'), mode=0666)
+        self.assertTransportMode(t, 'mode666', 0666)
+        t.put('mode600', StringIO('test text\n'), mode=0600)
+        self.assertTransportMode(t, 'mode600', 0600)
+        # Yes, you can put a file such that it becomes readonly
+        t.put('mode400', StringIO('test text\n'), mode=0400)
+        self.assertTransportMode(t, 'mode400', 0400)
+        t.put_multi([('mmode644', StringIO('text\n'))], mode=0644)
+        self.assertTransportMode(t, 'mmode644', 0644)
+        
+    def test_mkdir(self):
+        t = self.get_transport()
+
+        if t.is_readonly():
+            # cannot mkdir on readonly transports. We're not testing for 
+            # cache coherency because cache behaviour is not currently
+            # defined for the transport interface.
+            self.assertRaises(TransportNotPossible, t.mkdir, '.')
+            self.assertRaises(TransportNotPossible, t.mkdir, 'new_dir')
+            self.assertRaises(TransportNotPossible, t.mkdir_multi, ['new_dir'])
+            self.assertRaises(TransportNotPossible, t.mkdir, 'path/doesnt/exist')
+            return
+        # Test mkdir
+        t.mkdir('dir_a')
+        self.assertEqual(t.has('dir_a'), True)
+        self.assertEqual(t.has('dir_b'), False)
+
+        t.mkdir('dir_b')
+        self.assertEqual(t.has('dir_b'), True)
+
+        t.mkdir_multi(['dir_c', 'dir_d'])
+
+        t.mkdir_multi(iter(['dir_e', 'dir_f']))
+        self.assertEqual(list(t.has_multi(
+            ['dir_a', 'dir_b', 'dir_c', 'dir_q',
+             'dir_d', 'dir_e', 'dir_f', 'dir_b'])),
+            [True, True, True, False,
+             True, True, True, True])
+
+        # we were testing that a local mkdir followed by a transport
+        # mkdir failed thusly, but given that we * in one process * do not
+        # concurrently fiddle with disk dirs and then use transport to do 
+        # things, the win here seems marginal compared to the constraint on
+        # the interface. RBC 20051227
+        t.mkdir('dir_g')
+        self.assertRaises(FileExists, t.mkdir, 'dir_g')
+
+        # Test get/put in sub-directories
+        self.assertEqual(
+            t.put_multi([('dir_a/a', StringIO('contents of dir_a/a')),
+                         ('dir_b/b', StringIO('contents of dir_b/b'))])
+                        , 2)
+        self.check_transport_contents('contents of dir_a/a', t, 'dir_a/a')
+        self.check_transport_contents('contents of dir_b/b', t, 'dir_b/b')
+
+        # mkdir of a dir with an absent parent
+        self.assertRaises(NoSuchFile, t.mkdir, 'missing/dir')
+
+    def test_mkdir_permissions(self):
+        t = self.get_transport()
+        if t.is_readonly():
+            return
+        # Test mkdir with a mode
+        t.mkdir('dmode755', mode=0755)
+        self.assertTransportMode(t, 'dmode755', 0755)
+        t.mkdir('dmode555', mode=0555)
+        self.assertTransportMode(t, 'dmode555', 0555)
+        t.mkdir('dmode777', mode=0777)
+        self.assertTransportMode(t, 'dmode777', 0777)
+        t.mkdir('dmode700', mode=0700)
+        self.assertTransportMode(t, 'dmode700', 0700)
+        # TODO: jam 20051215 test mkdir_multi with a mode
+        t.mkdir_multi(['mdmode755'], mode=0755)
+        self.assertTransportMode(t, 'mdmode755', 0755)
+
+    def test_copy_to(self):
+        from bzrlib.transport.memory import MemoryTransport
+        t = self.get_transport()
+
+        files = ['a', 'b', 'c', 'd']
+        self.build_tree(files, transport=t)
+
+        temp_transport = MemoryTransport('memory:/')
+
+        t.copy_to(files, temp_transport)
+        for f in files:
+            self.check_transport_contents(temp_transport.get(f).read(),
+                                          t, f)
+
+        # Test that copying into a missing directory raises
+        # NoSuchFile
+        if t.is_readonly():
+            self.build_tree(['e/', 'e/f'])
+        else:
+            t.mkdir('e')
+            t.put('e/f', StringIO('contents of e'))
+        self.assertRaises(NoSuchFile, t.copy_to, ['e/f'], temp_transport)
+        temp_transport.mkdir('e')
+        t.copy_to(['e/f'], temp_transport)
+
+        del temp_transport
+        temp_transport = MemoryTransport('memory:/')
+
+        files = ['a', 'b', 'c', 'd']
+        t.copy_to(iter(files), temp_transport)
+        for f in files:
+            self.check_transport_contents(temp_transport.get(f).read(),
+                                          t, f)
+        del temp_transport
+
+        for mode in (0666, 0644, 0600, 0400):
+            temp_transport = MemoryTransport("memory:/")
+            t.copy_to(files, temp_transport, mode=mode)
+            for f in files:
+                self.assertTransportMode(temp_transport, f, mode)
+
+    def test_append(self):
+        t = self.get_transport()
+
+        if t.is_readonly():
+            open('a', 'wb').write('diff\ncontents for\na\n')
+            open('b', 'wb').write('contents\nfor b\n')
+        else:
+            t.put_multi([
+                    ('a', StringIO('diff\ncontents for\na\n')),
+                    ('b', StringIO('contents\nfor b\n'))
+                    ])
+
+        if t.is_readonly():
+            self.assertRaises(TransportNotPossible,
+                    t.append, 'a', 'add\nsome\nmore\ncontents\n')
+            _append('a', StringIO('add\nsome\nmore\ncontents\n'))
+        else:
+            t.append('a', StringIO('add\nsome\nmore\ncontents\n'))
+
+        self.check_transport_contents(
+            'diff\ncontents for\na\nadd\nsome\nmore\ncontents\n',
+            t, 'a')
+
+        if t.is_readonly():
+            self.assertRaises(TransportNotPossible,
+                    t.append_multi,
+                        [('a', 'and\nthen\nsome\nmore\n'),
+                         ('b', 'some\nmore\nfor\nb\n')])
+            _append('a', StringIO('and\nthen\nsome\nmore\n'))
+            _append('b', StringIO('some\nmore\nfor\nb\n'))
+        else:
+            t.append_multi([('a', StringIO('and\nthen\nsome\nmore\n')),
+                    ('b', StringIO('some\nmore\nfor\nb\n'))])
+        self.check_transport_contents(
+            'diff\ncontents for\na\n'
+            'add\nsome\nmore\ncontents\n'
+            'and\nthen\nsome\nmore\n',
+            t, 'a')
+        self.check_transport_contents(
+                'contents\nfor b\n'
+                'some\nmore\nfor\nb\n',
+                t, 'b')
+
+        if t.is_readonly():
+            _append('a', StringIO('a little bit more\n'))
+            _append('b', StringIO('from an iterator\n'))
+        else:
+            t.append_multi(iter([('a', StringIO('a little bit more\n')),
+                    ('b', StringIO('from an iterator\n'))]))
+        self.check_transport_contents(
+            'diff\ncontents for\na\n'
+            'add\nsome\nmore\ncontents\n'
+            'and\nthen\nsome\nmore\n'
+            'a little bit more\n',
+            t, 'a')
+        self.check_transport_contents(
+                'contents\nfor b\n'
+                'some\nmore\nfor\nb\n'
+                'from an iterator\n',
+                t, 'b')
+
+        if t.is_readonly():
+            _append('c', StringIO('some text\nfor a missing file\n'))
+            _append('a', StringIO('some text in a\n'))
+            _append('d', StringIO('missing file r\n'))
+        else:
+            t.append('c', StringIO('some text\nfor a missing file\n'))
+            t.append_multi([('a', StringIO('some text in a\n')),
+                            ('d', StringIO('missing file r\n'))])
+        self.check_transport_contents(
+            'diff\ncontents for\na\n'
+            'add\nsome\nmore\ncontents\n'
+            'and\nthen\nsome\nmore\n'
+            'a little bit more\n'
+            'some text in a\n',
+            t, 'a')
+        self.check_transport_contents('some text\nfor a missing file\n',
+                                      t, 'c')
+        self.check_transport_contents('missing file r\n', t, 'd')
+        
+        # a file with no parent should fail..
+        if not t.is_readonly():
+            self.assertRaises(NoSuchFile,
+                              t.append, 'missing/path', 
+                              StringIO('content'))
+
+    def test_append_file(self):
+        t = self.get_transport()
+
+        contents = [
+            ('f1', StringIO('this is a string\nand some more stuff\n')),
+            ('f2', StringIO('here is some text\nand a bit more\n')),
+            ('f3', StringIO('some text for the\nthird file created\n')),
+            ('f4', StringIO('this is a string\nand some more stuff\n')),
+            ('f5', StringIO('here is some text\nand a bit more\n')),
+            ('f6', StringIO('some text for the\nthird file created\n'))
+        ]
+        
+        if t.is_readonly():
+            for f, val in contents:
+                open(f, 'wb').write(val.read())
+        else:
+            t.put_multi(contents)
+
+        a1 = StringIO('appending to\none\n')
+        if t.is_readonly():
+            _append('f1', a1)
+        else:
+            t.append('f1', a1)
+
+        del a1
+
+        self.check_transport_contents(
+                'this is a string\nand some more stuff\n'
+                'appending to\none\n',
+                t, 'f1')
+
+        a2 = StringIO('adding more\ntext to two\n')
+        a3 = StringIO('some garbage\nto put in three\n')
+
+        if t.is_readonly():
+            _append('f2', a2)
+            _append('f3', a3)
+        else:
+            t.append_multi([('f2', a2), ('f3', a3)])
+
+        del a2, a3
+
+        self.check_transport_contents(
+                'here is some text\nand a bit more\n'
+                'adding more\ntext to two\n',
+                t, 'f2')
+        self.check_transport_contents( 
+                'some text for the\nthird file created\n'
+                'some garbage\nto put in three\n',
+                t, 'f3')
+
+        # Test that an actual file object can be used with put
+        a4 = t.get('f1')
+        if t.is_readonly():
+            _append('f4', a4)
+        else:
+            t.append('f4', a4)
+
+        del a4
+
+        self.check_transport_contents(
+                'this is a string\nand some more stuff\n'
+                'this is a string\nand some more stuff\n'
+                'appending to\none\n',
+                t, 'f4')
+
+        a5 = t.get('f2')
+        a6 = t.get('f3')
+        if t.is_readonly():
+            _append('f5', a5)
+            _append('f6', a6)
+        else:
+            t.append_multi([('f5', a5), ('f6', a6)])
+
+        del a5, a6
+
+        self.check_transport_contents(
+                'here is some text\nand a bit more\n'
+                'here is some text\nand a bit more\n'
+                'adding more\ntext to two\n',
+                t, 'f5')
+        self.check_transport_contents(
+                'some text for the\nthird file created\n'
+                'some text for the\nthird file created\n'
+                'some garbage\nto put in three\n',
+                t, 'f6')
+
+        a5 = t.get('f2')
+        a6 = t.get('f2')
+        a7 = t.get('f3')
+        if t.is_readonly():
+            _append('c', a5)
+            _append('a', a6)
+            _append('d', a7)
+        else:
+            t.append('c', a5)
+            t.append_multi([('a', a6), ('d', a7)])
+        del a5, a6, a7
+        self.check_transport_contents(t.get('f2').read(), t, 'c')
+        self.check_transport_contents(t.get('f3').read(), t, 'd')
+
+
+    def test_delete(self):
+        # TODO: Test Transport.delete
+        t = self.get_transport()
+
+        # Not much to do with a readonly transport
+        if t.is_readonly():
+            return
+
+        t.put('a', StringIO('a little bit of text\n'))
+        self.failUnless(t.has('a'))
+        t.delete('a')
+        self.failIf(t.has('a'))
+
+        self.assertRaises(NoSuchFile, t.delete, 'a')
+
+        t.put('a', StringIO('a text\n'))
+        t.put('b', StringIO('b text\n'))
+        t.put('c', StringIO('c text\n'))
+        self.assertEqual([True, True, True],
+                list(t.has_multi(['a', 'b', 'c'])))
+        t.delete_multi(['a', 'c'])
+        self.assertEqual([False, True, False],
+                list(t.has_multi(['a', 'b', 'c'])))
+        self.failIf(t.has('a'))
+        self.failUnless(t.has('b'))
+        self.failIf(t.has('c'))
+
+        self.assertRaises(NoSuchFile,
+                t.delete_multi, ['a', 'b', 'c'])
+
+        self.assertRaises(NoSuchFile,
+                t.delete_multi, iter(['a', 'b', 'c']))
+
+        t.put('a', StringIO('another a text\n'))
+        t.put('c', StringIO('another c text\n'))
+        t.delete_multi(iter(['a', 'b', 'c']))
+
+        # We should have deleted everything
+        # SftpServer creates control files in the
+        # working directory, so we can just do a
+        # plain "listdir".
+        # self.assertEqual([], os.listdir('.'))
+
+    def test_move(self):
+        t = self.get_transport()
+
+        if t.is_readonly():
+            return
+
+        # TODO: I would like to use os.listdir() to
+        # make sure there are no extra files, but SftpServer
+        # creates control files in the working directory
+        # perhaps all of this could be done in a subdirectory
+
+        t.put('a', StringIO('a first file\n'))
+        self.assertEquals([True, False], list(t.has_multi(['a', 'b'])))
+
+        t.move('a', 'b')
+        self.failUnless(t.has('b'))
+        self.failIf(t.has('a'))
+
+        self.check_transport_contents('a first file\n', t, 'b')
+        self.assertEquals([False, True], list(t.has_multi(['a', 'b'])))
+
+        # Overwrite a file
+        t.put('c', StringIO('c this file\n'))
+        t.move('c', 'b')
+        self.failIf(t.has('c'))
+        self.check_transport_contents('c this file\n', t, 'b')
+
+        # TODO: Try to write a test for atomicity
+        # TODO: Test moving into a non-existant subdirectory
+        # TODO: Test Transport.move_multi
+
+    def test_copy(self):
+        t = self.get_transport()
+
+        if t.is_readonly():
+            return
+
+        t.put('a', StringIO('a file\n'))
+        t.copy('a', 'b')
+        self.check_transport_contents('a file\n', t, 'b')
+
+        self.assertRaises(NoSuchFile, t.copy, 'c', 'd')
+        os.mkdir('c')
+        # What should the assert be if you try to copy a
+        # file over a directory?
+        #self.assertRaises(Something, t.copy, 'a', 'c')
+        t.put('d', StringIO('text in d\n'))
+        t.copy('d', 'b')
+        self.check_transport_contents('text in d\n', t, 'b')
+
+        # TODO: test copy_multi
+
+    def test_connection_error(self):
+        """ConnectionError is raised when connection is impossible"""
+        try:
+            url = self._server.get_bogus_url()
+        except NotImplementedError:
+            raise TestSkipped("Transport %s has no bogus URL support." %
+                              self._server.__class__)
+        t = bzrlib.transport.get_transport(url)
+        try:
+            t.get('.bzr/branch')
+        except (ConnectionError, NoSuchFile), e:
+            pass
+        except (Exception), e:
+            self.failIf(True, 'Wrong exception thrown: %s' % e)
+        else:
+            self.failIf(True, 'Did not get the expected exception.')
+
+    def test_stat(self):
+        # TODO: Test stat, just try once, and if it throws, stop testing
+        from stat import S_ISDIR, S_ISREG
+
+        t = self.get_transport()
+
+        try:
+            st = t.stat('.')
+        except TransportNotPossible, e:
+            # This transport cannot stat
+            return
+
+        paths = ['a', 'b/', 'b/c', 'b/d/', 'b/d/e']
+        sizes = [14, 0, 16, 0, 18] 
+        self.build_tree(paths, transport=t)
+
+        for path, size in zip(paths, sizes):
+            st = t.stat(path)
+            if path.endswith('/'):
+                self.failUnless(S_ISDIR(st.st_mode))
+                # directory sizes are meaningless
+            else:
+                self.failUnless(S_ISREG(st.st_mode))
+                self.assertEqual(size, st.st_size)
+
+        remote_stats = list(t.stat_multi(paths))
+        remote_iter_stats = list(t.stat_multi(iter(paths)))
+
+        self.assertRaises(NoSuchFile, t.stat, 'q')
+        self.assertRaises(NoSuchFile, t.stat, 'b/a')
+
+        self.assertListRaises(NoSuchFile, t.stat_multi, ['a', 'c', 'd'])
+        self.assertListRaises(NoSuchFile, t.stat_multi, iter(['a', 'c', 'd']))
+
+    def test_list_dir(self):
+        # TODO: Test list_dir, just try once, and if it throws, stop testing
+        t = self.get_transport()
+        
+        if not t.listable():
+            self.assertRaises(TransportNotPossible, t.list_dir, '.')
+            return
+
+        def sorted_list(d):
+            l = list(t.list_dir(d))
+            l.sort()
+            return l
+
+        # SftpServer creates control files in the working directory
+        # so lets move down a directory to avoid those.
+        t.mkdir('wd')
+        t = t.clone('wd')
+
+        self.assertEqual([], sorted_list(u'.'))
+        self.build_tree(['a', 'b', 'c/', 'c/d', 'c/e'], transport=t)
+
+        self.assertEqual([u'a', u'b', u'c'], sorted_list(u'.'))
+        self.assertEqual([u'd', u'e'], sorted_list(u'c'))
+
+        t.delete('c/d')
+        t.delete('b')
+        self.assertEqual([u'a', u'c'], sorted_list('.'))
+        self.assertEqual([u'e'], sorted_list(u'c'))
+
+        self.assertListRaises(NoSuchFile, t.list_dir, 'q')
+        self.assertListRaises(NoSuchFile, t.list_dir, 'c/f')
+        self.assertListRaises(NoSuchFile, t.list_dir, 'a')
+
+    def test_clone(self):
+        # TODO: Test that clone moves up and down the filesystem
+        t1 = self.get_transport()
+
+        self.build_tree(['a', 'b/', 'b/c'], transport=t1)
+
+        self.failUnless(t1.has('a'))
+        self.failUnless(t1.has('b/c'))
+        self.failIf(t1.has('c'))
+
+        t2 = t1.clone('b')
+        self.assertEqual(t1.base + 'b/', t2.base)
+
+        self.failUnless(t2.has('c'))
+        self.failIf(t2.has('a'))
+
+        t3 = t2.clone('..')
+        self.failUnless(t3.has('a'))
+        self.failIf(t3.has('c'))
+
+        self.failIf(t1.has('b/d'))
+        self.failIf(t2.has('d'))
+        self.failIf(t3.has('b/d'))
+
+        if t1.is_readonly():
+            open('b/d', 'wb').write('newfile\n')
+        else:
+            t2.put('d', StringIO('newfile\n'))
+
+        self.failUnless(t1.has('b/d'))
+        self.failUnless(t2.has('d'))
+        self.failUnless(t3.has('b/d'))
+
+    def test_relpath(self):
+        t = self.get_transport()
+        self.assertEqual('', t.relpath(t.base))
+        # base ends with /
+        self.assertEqual('', t.relpath(t.base[:-1]))
+        # subdirs which dont exist should still give relpaths.
+        self.assertEqual('foo', t.relpath(t.base + 'foo'))
+        # trailing slash should be the same.
+        self.assertEqual('foo', t.relpath(t.base + 'foo/'))
+
+    def test_abspath(self):
+        # smoke test for abspath. Corner cases for backends like unix fs's
+        # that have aliasing problems like symlinks should go in backend
+        # specific test cases.
+        transport = self.get_transport()
+        self.assertEqual(transport.base + 'relpath',
+                         transport.abspath('relpath'))
+
+    def test_iter_files_recursive(self):
+        transport = self.get_transport()
+        if not transport.listable():
+            self.assertRaises(TransportNotPossible, 
+                              transport.iter_files_recursive)
+            return
+        self.build_tree(['isolated/', 
+                         'isolated/dir/',
+                         'isolated/dir/foo',
+                         'isolated/dir/bar',
+                         'isolated/bar'],
+                        transport=transport)
+        transport = transport.clone('isolated')
+        paths = set(transport.iter_files_recursive())
+        self.assertEqual(set(['dir/foo', 'dir/bar', 'bar']), paths)

=== added file 'bzrlib/tests/test_tsort.py'
--- bzrlib/tests/test_tsort.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_tsort.py	2005-11-23 15:44:24 +0000
@@ -0,0 +1,77 @@
+# Copyright (C) 2005 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for topological sort.
+"""
+
+import os
+import sys
+
+from bzrlib.tests import TestCase
+from bzrlib.tsort import topo_sort
+from bzrlib.errors import GraphCycleError
+
+class TopoSortTests(TestCase):
+
+    def test_tsort_empty(self):
+        """TopoSort empty list"""
+        self.assertEquals(topo_sort([]), [])
+
+    def test_tsort_easy(self):
+        """TopoSort list with one node"""
+        self.assertEquals(topo_sort({0: []}.items()),
+                          [0])
+
+    def test_tsort_cycle(self):
+        """TopoSort traps graph with cycles"""
+        self.assertRaises(GraphCycleError,
+                          topo_sort,
+                          {0: [1], 
+                           1: [0]}.iteritems())
+
+    def test_tsort_cycle_2(self):
+        """TopoSort traps graph with longer cycle"""
+        self.assertRaises(GraphCycleError,
+                          topo_sort,
+                          {0: [1], 
+                           1: [2], 
+                           2: [0]}.iteritems())
+                 
+    def test_tsort_1(self):
+        """TopoSort simple nontrivial graph"""
+        self.assertEquals(topo_sort({0: [3], 
+                                     1: [4],
+                                     2: [1, 4],
+                                     3: [], 
+                                     4: [0, 3]}.iteritems()),
+                          [3, 0, 4, 1, 2])
+
+    def test_tsort_partial(self):
+        """Topological sort with partial ordering.
+
+        If the graph does not give an order between two nodes, they are 
+        returned in lexicographical order.
+        """
+        self.assertEquals(topo_sort([(0, []),
+                                    (1, [0]),
+                                    (2, [0]),
+                                    (3, [0]),
+                                    (4, [1, 2, 3]),
+                                    (5, [1, 2]),
+                                    (6, [1, 2]),
+                                    (7, [2, 3]),
+                                    (8, [0, 1, 4, 5, 6])]),
+                          [0, 1, 2, 3, 4, 5, 6, 7, 8, ])

=== added file 'bzrlib/tests/test_ui.py'
--- bzrlib/tests/test_ui.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_ui.py	2005-11-30 16:41:08 +0000
@@ -0,0 +1,52 @@
+# Copyright (C) 2005 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for the bzrlib ui
+"""
+
+import os
+import sys
+
+from bzrlib.tests import TestCase
+from bzrlib.ui import SilentUIFactory
+from bzrlib.ui.text import TextUIFactory
+
+class UITests(TestCase):
+
+    def test_silent_factory(self):
+
+        ui = SilentUIFactory()
+        pb = ui.progress_bar()
+        # TODO: Test that there is no output from SilentUIFactory
+
+        self.assertEquals(ui.get_password(), None)
+        self.assertEquals(ui.get_password(u'Hello There \u1234 %(user)s',
+                                          user=u'some\u1234')
+                         , None)
+
+    def test_text_factory(self):
+        ui = TextUIFactory()
+        pb = ui.progress_bar()
+        # TODO: Test the output from TextUIFactory, perhaps by overriding sys.stdout
+
+        # Unfortunately we can't actually test the ui.get_password() because 
+        # that would actually prompt the user for a password during the test suite
+        # This has been tested manually with both LANG=en_US.utf-8 and LANG=C
+        # print
+        # self.assertEquals(ui.get_password(u"%(user)s please type 'bogus'",
+        #                                   user=u'some\u1234')
+        #                  , 'bogus')
+

=== added file 'bzrlib/tests/test_uncommit.py'
--- bzrlib/tests/test_uncommit.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_uncommit.py	2005-11-23 15:44:24 +0000
@@ -0,0 +1,37 @@
+"""\
+Test the uncommit command.
+"""
+from bzrlib.tests import TestCaseInTempDir
+from bzrlib.errors import BzrError
+
+class TestUncommit(TestCaseInTempDir):
+    def test_uncommit(self):
+        """Test uncommit functionality."""
+        bzr = self.capture 
+
+        bzr('init')
+        self.build_tree(['a', 'b', 'c'])
+
+        bzr('add')
+        bzr('commit -m initial')
+
+        self.assertEquals(bzr('revno'), '1\n')
+
+        open('a', 'wb').write('new contents of a\n')
+        self.assertEquals(bzr('status'), 'modified:\n  a\n')
+        bzr('commit -m second')
+
+        self.assertEquals(bzr('status'), '')
+        self.assertEquals(bzr('revno'), '2\n')
+
+        txt = bzr('uncommit --dry-run --force')
+        self.failIfEqual(txt.find('Dry-run'), -1)
+
+        self.assertEquals(bzr('status'), '')
+        self.assertEquals(bzr('revno'), '2\n')
+
+        txt = bzr('uncommit --force')
+
+        self.assertEquals(bzr('revno'), '1\n')
+        self.assertEquals(bzr('status'), 'modified:\n  a\n')
+

=== modified file 'bzrlib/tests/test_upgrade.py'
--- bzrlib/selftest/test_upgrade.py	2005-10-06 11:26:51 +0000
+++ bzrlib/tests/test_upgrade.py	2005-11-30 00:25:50 +0000
@@ -23,28 +23,27 @@
 import os
 import sys
 
-from bzrlib.selftest import TestCase, TestCaseInTempDir
+from bzrlib.tests import TestCase, TestCaseInTempDir
 from bzrlib.branch import Branch
 from bzrlib.revision import is_ancestor
 from bzrlib.upgrade import upgrade
-from bzrlib.selftest.treeshape import build_tree_contents
 
 
 class TestUpgrade(TestCaseInTempDir):
     
     def test_build_tree(self):
         """Test tree-building test helper"""
-        build_tree_contents(_upgrade1_template)
+        self.build_tree_contents(_upgrade1_template)
         self.failUnlessExists('foo')
         self.failUnlessExists('.bzr/README')
 
     def test_upgrade_simple(self):
-        """Upgrade simple v0.0.4 format to v5"""
+        """Upgrade simple v0.0.4 format to v6"""
         eq = self.assertEquals
-        build_tree_contents(_upgrade1_template)
-        upgrade('.')
-        b = Branch.open('.')
-        eq(b._branch_format, 5)
+        self.build_tree_contents(_upgrade1_template)
+        upgrade(u'.')
+        b = Branch.open(u'.')
+        eq(b._branch_format, 6)
         rh = b.revision_history()
         eq(rh,
            ['mbp@sourcefrog.net-20051004035611-176b16534b086b3c',
@@ -66,19 +65,22 @@
         systems, where the importer knows about a revision but not 
         its contents."""
         eq = self.assertEquals
-        build_tree_contents(_ghost_template)
-        upgrade('.')
-        b = Branch.open('.')
+        self.build_tree_contents(_ghost_template)
+        upgrade(u'.')
+        b = Branch.open(u'.')
         revision_id = b.revision_history()[1]
         rev = b.get_revision(revision_id)
         eq(len(rev.parent_ids), 2)
         eq(rev.parent_ids[1], 'wibble@wobble-2')
 
     def test_upgrade_makes_dir_weaves(self):
-        build_tree_contents(_upgrade_dir_template)
+        self.build_tree_contents(_upgrade_dir_template)
         upgrade('.')
+        # this is the path to the literal file. As format changes 
+        # occur it needs to be updated. FIXME: ask the store for the
+        # path.
         self.failUnlessExists(
-            '.bzr/weaves/dir-20051005095101-da1441ea3fa6917a.weave')
+            '.bzr/weaves/de/dir-20051005095101-da1441ea3fa6917a.weave')
 
 
 _upgrade1_template = \

=== modified file 'bzrlib/tests/test_weave.py'
--- bzrlib/selftest/test_weave.py	2005-10-06 10:53:12 +0000
+++ bzrlib/tests/test_weave.py	2005-12-20 02:45:27 +0000
@@ -18,16 +18,17 @@
 
 
 # TODO: tests regarding version names
-
-
+# TODO: rbc 20050108 test that join does not leave an inconsistent weave 
+#       if it fails.
 
 """test suite for weave algorithm"""
 
 from pprint import pformat
 
-from bzrlib.weave import Weave, WeaveFormatError, WeaveError
+import bzrlib.errors as errors
+from bzrlib.weave import Weave, WeaveFormatError, WeaveError, reweave
 from bzrlib.weavefile import write_weave, read_weave
-from bzrlib.selftest import TestCase
+from bzrlib.tests import TestCase
 from bzrlib.osutils import sha_string
 
 
@@ -37,7 +38,6 @@
           "A second line"]
 
 
-
 class TestBase(TestCase):
     def check_read_write(self, k):
         """Check the weave k can be written & re-read."""
@@ -58,11 +58,16 @@
             self.log('         %r' % k._parents)
             self.log('         %r' % k2._parents)
             self.log('')
-
-            
             self.fail('read/write check failed')
-        
-        
+
+
+class WeaveContains(TestBase):
+    """Weave __contains__ operator"""
+    def runTest(self):
+        k = Weave()
+        self.assertFalse('foo' in k)
+        k.add('foo', [], TEXT_1)
+        self.assertTrue('foo' in k)
 
 
 class Easy(TestBase):
@@ -79,7 +84,6 @@
         self.assertEqual(idx, 0)
 
 
-
 class AnnotateOne(TestBase):
     def runTest(self):
         k = Weave()
@@ -102,7 +106,6 @@
         self.assertEqual(k.get(1), TEXT_1)
 
 
-
 class AddWithGivenSha(TestBase):
     def runTest(self):
         """Add with caller-supplied SHA-1"""
@@ -112,6 +115,17 @@
         k.add('text0', [], [t], sha1=sha_string(t))
 
 
+class GetSha1(TestBase):
+    def test_get_sha1(self):
+        k = Weave()
+        k.add('text0', [], 'text0')
+        self.assertEqual('34dc0e430c642a26c3dd1c2beb7a8b4f4445eb79',
+                         k.get_sha1('text0'))
+        self.assertRaises(errors.WeaveRevisionNotPresent,
+                          k.get_sha1, 0)
+        self.assertRaises(errors.WeaveRevisionNotPresent,
+                          k.get_sha1, 'text1')
+                        
 
 class InvalidAdd(TestBase):
     """Try to use invalid version number during add."""
@@ -134,7 +148,6 @@
         self.assertEqual(idx, idx2)
 
 
-
 class InvalidRepeatedAdd(TestBase):
     def runTest(self):
         k = Weave()
@@ -151,7 +164,6 @@
                           TEXT_0)
         
 
-
 class InsertLines(TestBase):
     """Store a revision that adds one line to the original.
 
@@ -208,7 +220,6 @@
                           (4, 'ccc')])
 
 
-
 class DeleteLines(TestBase):
     """Deletion of lines from existing text.
 
@@ -238,8 +249,6 @@
         for i in range(len(texts)):
             self.assertEqual(k.get(i+1),
                              texts[i])
-            
-
 
 
 class SuicideDelete(TestBase):
@@ -265,7 +274,6 @@
                           0)        
 
 
-
 class CannedDelete(TestBase):
     """Unpack canned weave with deleted lines."""
     def runTest(self):
@@ -282,6 +290,8 @@
                 'last line',
                 ('}', 0),
                 ]
+        k._sha1s = [sha_string('first lineline to be deletedlast line')
+                  , sha_string('first linelast line')]
 
         self.assertEqual(k.get(0),
                          ['first line',
@@ -295,7 +305,6 @@
                           ])
 
 
-
 class CannedReplacement(TestBase):
     """Unpack canned weave with deleted lines."""
     def runTest(self):
@@ -315,6 +324,8 @@
                 'last line',
                 ('}', 0),
                 ]
+        k._sha1s = [sha_string('first lineline to be deletedlast line')
+                  , sha_string('first linereplacement linelast line')]
 
         self.assertEqual(k.get(0),
                          ['first line',
@@ -329,7 +340,6 @@
                           ])
 
 
-
 class BadWeave(TestBase):
     """Test that we trap an insert which should not occur."""
     def runTest(self):
@@ -415,6 +425,12 @@
                 '}',
                 ('}', 0)]
 
+        k._sha1s = [sha_string('foo {}')
+                  , sha_string('foo {  added in version 1  also from v1}')
+                  , sha_string('foo {  added in v2}')
+                  , sha_string('foo {  added in version 1  added in v2  also from v1}')
+                  ]
+
         self.assertEqual(k.get(0),
                          ['foo {',
                           '}'])
@@ -438,7 +454,6 @@
                           '}'])
                          
 
-
 class DeleteLines2(TestBase):
     """Test recording revisions that delete lines.
 
@@ -466,7 +481,6 @@
                           (0, "fine")])
 
 
-
 class IncludeVersions(TestBase):
     """Check texts that are stored across multiple revisions.
 
@@ -488,6 +502,9 @@
                 "second line",
                 ('}', 1)]
 
+        k._sha1s = [sha_string('first line')
+                  , sha_string('first linesecond line')]
+
         self.assertEqual(k.get(1),
                          ["first line",
                           "second line"])
@@ -517,6 +534,10 @@
                 ('}', 2),                
                 ]
 
+        k._sha1s = [sha_string('first line')
+                  , sha_string('first linesecond line')
+                  , sha_string('first linealternative second line')]
+
         self.assertEqual(k.get(0),
                          ["first line"])
 
@@ -532,7 +553,6 @@
                          [0, 2])
 
 
-
 class ReplaceLine(TestBase):
     def runTest(self):
         k = Weave()
@@ -549,7 +569,6 @@
         self.assertEqual(k.get(1), text1)
 
 
-
 class Merge(TestBase):
     """Storage of versions that merge diverged parents"""
     def runTest(self):
@@ -606,7 +625,6 @@
                            [['bbb']]])
 
 
-
 class NonConflict(TestBase):
     """Two descendants insert compatible changes.
 
@@ -619,9 +637,6 @@
         k.add([0], ['111', 'aaa', 'ccc', 'bbb'])
         k.add([1], ['aaa', 'ccc', 'bbb', '222'])
 
-    
-    
-
 
 class AutoMerge(TestBase):
     def runTest(self):
@@ -645,7 +660,6 @@
                           'line from 1',
                           'bbb',
                           'line from 2', 'more from 2'])
-        
 
 
 class Khayyam(TestBase):
@@ -695,7 +709,6 @@
         self.check_read_write(k)
 
 
-
 class MergeCases(TestBase):
     def doMerge(self, base, a, b, mp):
         from cStringIO import StringIO
@@ -752,7 +765,8 @@
         self.doMerge(['aaa', 'bbb'],
                      ['aaa', 'xxx', 'yyy', 'bbb'],
                      ['aaa', 'xxx', 'bbb'],
-                     ['aaa', '<<<<', 'xxx', 'yyy', '====', 'xxx', '>>>>', 'bbb'])
+                     ['aaa', '<<<<<<<', 'xxx', 'yyy', '=======', 'xxx', 
+                      '>>>>>>>', 'bbb'])
 
         # really it ought to reduce this to 
         # ['aaa', 'xxx', 'yyy', 'bbb']
@@ -762,13 +776,15 @@
         self.doMerge(['aaa'],
                      ['xxx'],
                      ['yyy', 'zzz'],
-                     ['<<<<', 'xxx', '====', 'yyy', 'zzz', '>>>>'])
+                     ['<<<<<<<', 'xxx', '=======', 'yyy', 'zzz', 
+                      '>>>>>>>'])
 
     def testNonClashInsert(self):
         self.doMerge(['aaa'],
                      ['xxx', 'aaa'],
                      ['yyy', 'zzz'],
-                     ['<<<<', 'xxx', 'aaa', '====', 'yyy', 'zzz', '>>>>'])
+                     ['<<<<<<<', 'xxx', 'aaa', '=======', 'yyy', 'zzz', 
+                      '>>>>>>>'])
 
         self.doMerge(['aaa'],
                      ['aaa'],
@@ -790,7 +806,7 @@
         self.doMerge(['aaa', 'bbb', 'ccc'],
                      ['aaa', 'ddd', 'ccc'],
                      ['aaa', 'ccc'],
-                     ['<<<<', 'aaa', '====', '>>>>', 'ccc'])
+                     ['<<<<<<<<', 'aaa', '=======', '>>>>>>>', 'ccc'])
 
 
 class JoinWeavesTests(TestBase):
@@ -873,28 +889,105 @@
         eq(sorted(wa.iter_names()), ['v1', 'v2', 'v3', 'x1',])
         eq(wa.get_text('x1'), 'line from x1\n')
 
-    def test_join_with_ghosts(self):
-        """Join that inserts parents of an existing revision.
-
-        This can happen when merging from another branch who
-        knows about revisions the destination does not.  In 
-        this test the second weave knows of an additional parent of 
-        v2.  Any revisions which are in common still have to have the 
-        same text."""
-        return ###############################
-        wa = self.weave1.copy()
-        wb = Weave()
-        wb.add('x1', [], ['line from x1\n'])
-        wb.add('v1', [], ['hello\n'])
-        wb.add('v2', ['v1', 'x1'], ['hello\n', 'world\n'])
-        wa.join(wb)
-        eq = self.assertEquals
-        eq(sorted(wa.iter_names()), ['v1', 'v2', 'v3', 'x1',])
-        eq(wa.get_text('x1'), 'line from x1\n')
-
-
-if __name__ == '__main__':
-    import sys
-    import unittest
-    sys.exit(unittest.main())
-    
+
+class Corruption(TestCase):
+
+    def test_detection(self):
+        # Test weaves detect corruption.
+        #
+        # Weaves contain a checksum of their texts.
+        # When a text is extracted, this checksum should be
+        # verified.
+
+        w = Weave()
+        w.add('v1', [], ['hello\n'])
+        w.add('v2', ['v1'], ['hello\n', 'there\n'])
+
+        # We are going to invasively corrupt the text
+        # Make sure the internals of weave are the same
+        self.assertEqual([('{', 0)
+                        , 'hello\n'
+                        , ('}', None)
+                        , ('{', 1)
+                        , 'there\n'
+                        , ('}', None)
+                        ], w._weave)
+
+        self.assertEqual(['f572d396fae9206628714fb2ce00f72e94f2258f'
+                        , '90f265c6e75f1c8f9ab76dcf85528352c5f215ef'
+                        ], w._sha1s)
+        w.check()
+
+        # Corrupted
+        w._weave[4] = 'There\n'
+
+        self.assertEqual('hello\n', w.get_text('v1'))
+        self.assertRaises(errors.WeaveInvalidChecksum, w.get_text, 'v2')
+        self.assertRaises(errors.WeaveInvalidChecksum, w.get_lines, 'v2')
+        self.assertRaises(errors.WeaveInvalidChecksum, list, w.get_iter('v2'))
+        self.assertRaises(errors.WeaveInvalidChecksum, w.check)
+
+        # Corrected
+        w._weave[4] = 'there\n'
+        self.assertEqual('hello\nthere\n', w.get_text('v2'))
+
+        #Invalid checksum, first digit changed
+        w._sha1s[1] =  'f0f265c6e75f1c8f9ab76dcf85528352c5f215ef'
+
+        self.assertEqual('hello\n', w.get_text('v1'))
+        self.assertRaises(errors.WeaveInvalidChecksum, w.get_text, 'v2')
+        self.assertRaises(errors.WeaveInvalidChecksum, w.get_lines, 'v2')
+        self.assertRaises(errors.WeaveInvalidChecksum, list, w.get_iter('v2'))
+        self.assertRaises(errors.WeaveInvalidChecksum, w.check)
+
+    def test_written_detection(self):
+        # Test detection of weave file corruption.
+        #
+        # Make sure that we can detect if a weave file has
+        # been corrupted. This doesn't test all forms of corruption,
+        # but it at least helps verify the data you get, is what you want.
+        from cStringIO import StringIO
+
+        w = Weave()
+        w.add('v1', [], ['hello\n'])
+        w.add('v2', ['v1'], ['hello\n', 'there\n'])
+
+        tmpf = StringIO()
+        write_weave(w, tmpf)
+
+        # Because we are corrupting, we need to make sure we have the exact text
+        self.assertEquals('# bzr weave file v5\n'
+                          'i\n1 f572d396fae9206628714fb2ce00f72e94f2258f\nn v1\n\n'
+                          'i 0\n1 90f265c6e75f1c8f9ab76dcf85528352c5f215ef\nn v2\n\n'
+                          'w\n{ 0\n. hello\n}\n{ 1\n. there\n}\nW\n',
+                          tmpf.getvalue())
+
+        # Change a single letter
+        tmpf = StringIO('# bzr weave file v5\n'
+                        'i\n1 f572d396fae9206628714fb2ce00f72e94f2258f\nn v1\n\n'
+                        'i 0\n1 90f265c6e75f1c8f9ab76dcf85528352c5f215ef\nn v2\n\n'
+                        'w\n{ 0\n. hello\n}\n{ 1\n. There\n}\nW\n')
+
+        w = read_weave(tmpf)
+
+        self.assertEqual('hello\n', w.get_text('v1'))
+        self.assertRaises(errors.WeaveInvalidChecksum, w.get_text, 'v2')
+        self.assertRaises(errors.WeaveInvalidChecksum, w.get_lines, 'v2')
+        self.assertRaises(errors.WeaveInvalidChecksum, list, w.get_iter('v2'))
+        self.assertRaises(errors.WeaveInvalidChecksum, w.check)
+
+        # Change the sha checksum
+        tmpf = StringIO('# bzr weave file v5\n'
+                        'i\n1 f572d396fae9206628714fb2ce00f72e94f2258f\nn v1\n\n'
+                        'i 0\n1 f0f265c6e75f1c8f9ab76dcf85528352c5f215ef\nn v2\n\n'
+                        'w\n{ 0\n. hello\n}\n{ 1\n. there\n}\nW\n')
+
+        w = read_weave(tmpf)
+
+        self.assertEqual('hello\n', w.get_text('v1'))
+        self.assertRaises(errors.WeaveInvalidChecksum, w.get_text, 'v2')
+        self.assertRaises(errors.WeaveInvalidChecksum, w.get_lines, 'v2')
+        self.assertRaises(errors.WeaveInvalidChecksum, list, w.get_iter('v2'))
+        self.assertRaises(errors.WeaveInvalidChecksum, w.check)
+
+

=== renamed file 'bzrlib/selftest/whitebox.py' => 'bzrlib/tests/test_whitebox.py'
--- bzrlib/selftest/whitebox.py	2005-10-02 23:23:56 +0000
+++ bzrlib/tests/test_whitebox.py	2005-12-01 21:49:24 +0000
@@ -1,108 +1,52 @@
 import os
 import unittest
 
-from bzrlib.selftest import TestCaseInTempDir, TestCase
+from bzrlib.tests import TestCaseInTempDir, TestCase
 from bzrlib.branch import ScratchBranch, Branch
-from bzrlib.errors import NotBranchError, NotVersionedError
+from bzrlib.errors import PathNotChild
+from bzrlib.osutils import relpath, pathjoin, abspath, realpath
 
 
 class TestBranch(TestCaseInTempDir):
 
-    def test_unknowns(self):
-        b = Branch.initialize('.')
-
-        self.build_tree(['hello.txt',
-                         'hello.txt~'])
-
-        self.assertEquals(list(b.unknowns()),
-                          ['hello.txt'])
-
     def test_no_changes(self):
         from bzrlib.errors import PointlessCommit
         
-        b = Branch.initialize('.')
+        b = Branch.initialize(u'.')
 
         self.build_tree(['hello.txt'])
 
         self.assertRaises(PointlessCommit,
-                          b.commit,
+                          b.working_tree().commit,
                           'commit without adding',
                           allow_pointless=False)
 
-        b.commit('commit pointless tree',
+        b.working_tree().commit('commit pointless tree',
                  allow_pointless=True)
 
-        b.add('hello.txt')
+        b.working_tree().add('hello.txt')
         
-        b.commit('commit first added file',
+        b.working_tree().commit('commit first added file',
                  allow_pointless=False)
         
         self.assertRaises(PointlessCommit,
-                          b.commit,
+                          b.working_tree().commit,
                           'commit after adding file',
                           allow_pointless=False)
         
-        b.commit('commit pointless revision with one file',
+        b.working_tree().commit('commit pointless revision with one file',
                  allow_pointless=True)
 
-        
-
-
-class TestRevisionId(TestCase):
-    
-    def test_validate_revision_id(self):
-        from bzrlib.revision import validate_revision_id
-        validate_revision_id('mbp@sourcefrog.net-20050311061123-96a255005c7c9dbe')
-        self.assertRaises(ValueError,
-                          validate_revision_id,
-                          ' asdkjas')
-        self.assertRaises(ValueError,
-                          validate_revision_id,
-                          'mbp@sourcefrog.net-20050311061123-96a255005c7c9dbe\n')
-        self.assertRaises(ValueError,
-                          validate_revision_id,
-                          ' mbp@sourcefrog.net-20050311061123-96a255005c7c9dbe')
-        self.assertRaises(ValueError,
-                          validate_revision_id,
-                          'Martin Pool <mbp@sourcefrog.net>-20050311061123-96a255005c7c9dbe')
-
 
 class MoreTests(TestCaseInTempDir):
 
-    def test_revert(self):
-        """Test selected-file revert"""
-        b = Branch.initialize('.')
-
-        self.build_tree(['hello.txt'])
-        file('hello.txt', 'w').write('initial hello')
-
-        self.assertRaises(NotVersionedError,
-                          b.revert, ['hello.txt'])
-        
-        b.add(['hello.txt'])
-        b.commit('create initial hello.txt')
-
-        self.check_file_contents('hello.txt', 'initial hello')
-        file('hello.txt', 'w').write('new hello')
-        self.check_file_contents('hello.txt', 'new hello')
-
-        # revert file modified since last revision
-        b.revert(['hello.txt'])
-        self.check_file_contents('hello.txt', 'initial hello')
-        self.check_file_contents('hello.txt~', 'new hello')
-
-        # reverting again clobbers the backup
-        b.revert(['hello.txt'])
-        self.check_file_contents('hello.txt', 'initial hello')
-        self.check_file_contents('hello.txt~', 'initial hello')
-
     def test_rename_dirs(self):
         """Test renaming directories and the files within them."""
-        b = Branch.initialize('.')
+        b = Branch.initialize(u'.')
         self.build_tree(['dir/', 'dir/sub/', 'dir/sub/file'])
-        b.add(['dir', 'dir/sub', 'dir/sub/file'])
+        b.working_tree().add(['dir', 'dir/sub', 'dir/sub/file'])
 
-        b.commit('create initial state')
+        b.working_tree().commit('create initial state')
 
         # TODO: lift out to a test helper that checks the shape of
         # an inventory
@@ -116,52 +60,51 @@
         self.check_inventory_shape(inv,
                                    ['dir', 'dir/sub', 'dir/sub/file'])
 
-        b.rename_one('dir', 'newdir')
+        b.working_tree().rename_one('dir', 'newdir')
 
-        self.check_inventory_shape(b.inventory,
+        self.check_inventory_shape(b.working_tree().read_working_inventory(),
                                    ['newdir', 'newdir/sub', 'newdir/sub/file'])
 
-        b.rename_one('newdir/sub', 'newdir/newsub')
-        self.check_inventory_shape(b.inventory,
+        b.working_tree().rename_one('newdir/sub', 'newdir/newsub')
+        self.check_inventory_shape(b.working_tree().read_working_inventory(),
                                    ['newdir', 'newdir/newsub',
                                     'newdir/newsub/file'])
 
     def test_relpath(self):
         """test for branch path lookups
     
-        Branch.relpath and bzrlib.branch._relpath do a simple but subtle
+        bzrlib.osutils._relpath do a simple but subtle
         job: given a path (either relative to cwd or absolute), work out
         if it is inside a branch and return the path relative to the base.
         """
-        from bzrlib.branch import _relpath
         import tempfile, shutil
         
         savedir = os.getcwdu()
         dtmp = tempfile.mkdtemp()
         # On Mac OSX, /tmp actually expands to /private/tmp
-        dtmp = os.path.realpath(dtmp)
+        dtmp = realpath(dtmp)
 
         def rp(p):
-            return _relpath(dtmp, p)
+            return relpath(dtmp, p)
         
         try:
             # check paths inside dtmp while standing outside it
-            self.assertEqual(rp(os.path.join(dtmp, 'foo')), 'foo')
+            self.assertEqual(rp(pathjoin(dtmp, 'foo')), 'foo')
 
             # root = nothing
             self.assertEqual(rp(dtmp), '')
 
-            self.assertRaises(NotBranchError,
+            self.assertRaises(PathNotChild,
                               rp,
                               '/etc')
 
             # now some near-miss operations -- note that
             # os.path.commonprefix gets these wrong!
-            self.assertRaises(NotBranchError,
+            self.assertRaises(PathNotChild,
                               rp,
                               dtmp.rstrip('\\/') + '2')
 
-            self.assertRaises(NotBranchError,
+            self.assertRaises(PathNotChild,
                               rp,
                               dtmp.rstrip('\\/') + '2/foo')
 
@@ -169,16 +112,15 @@
             # directory, or nearby
             os.chdir(dtmp)
 
-            FOO_BAR_QUUX = os.path.join('foo', 'bar', 'quux')
-            self.assertEqual(rp('foo/bar/quux'), FOO_BAR_QUUX)
+            self.assertEqual(rp('foo/bar/quux'), 'foo/bar/quux')
 
             self.assertEqual(rp('foo'), 'foo')
 
             self.assertEqual(rp('./foo'), 'foo')
 
-            self.assertEqual(rp(os.path.abspath('foo')), 'foo')
+            self.assertEqual(rp(abspath('foo')), 'foo')
 
-            self.assertRaises(NotBranchError,
+            self.assertRaises(PathNotChild,
                               rp, '../foo')
 
         finally:

=== renamed file 'bzrlib/selftest/testworkingtree.py' => 'bzrlib/tests/test_workingtree.py'
--- bzrlib/selftest/testworkingtree.py	2005-10-04 02:43:06 +0000
+++ bzrlib/tests/test_workingtree.py	2006-01-03 02:07:06 +0000
@@ -17,9 +17,12 @@
 
 import os
 from bzrlib.branch import Branch
-from bzrlib.selftest import TestCaseInTempDir
+from bzrlib.errors import NotBranchError, NotVersionedError
+from bzrlib.tests import TestCaseInTempDir
 from bzrlib.trace import mutter
-from bzrlib.workingtree import TreeEntry, TreeDirectory, TreeFile, TreeLink
+from bzrlib.osutils import pathjoin, getcwd, has_symlinks
+from bzrlib.workingtree import (TreeEntry, TreeDirectory, TreeFile, TreeLink,
+                                WorkingTree)
 
 class TestTreeDirectory(TestCaseInTempDir):
 
@@ -48,12 +51,137 @@
 class TestWorkingTree(TestCaseInTempDir):
 
     def test_listfiles(self):
-        branch = Branch.initialize('.')
+        branch = Branch.initialize(u'.')
         os.mkdir('dir')
         print >> open('file', 'w'), "content"
-        os.symlink('target', 'symlink')
+        if has_symlinks():
+            os.symlink('target', 'symlink')
         tree = branch.working_tree()
         files = list(tree.list_files())
         self.assertEqual(files[0], ('dir', '?', 'directory', None, TreeDirectory()))
         self.assertEqual(files[1], ('file', '?', 'file', None, TreeFile()))
-        self.assertEqual(files[2], ('symlink', '?', 'symlink', None, TreeLink()))
+        if has_symlinks():
+            self.assertEqual(files[2], ('symlink', '?', 'symlink', None, TreeLink()))
+
+    def test_open_containing(self):
+        branch = Branch.initialize(u'.')
+        wt, relpath = WorkingTree.open_containing()
+        self.assertEqual('', relpath)
+        self.assertEqual(wt.basedir + '/', branch.base)
+        wt, relpath = WorkingTree.open_containing(u'.')
+        self.assertEqual('', relpath)
+        self.assertEqual(wt.basedir + '/', branch.base)
+        wt, relpath = WorkingTree.open_containing('./foo')
+        self.assertEqual('foo', relpath)
+        self.assertEqual(wt.basedir + '/', branch.base)
+        # paths that are urls are just plain wrong for working trees.
+        self.assertRaises(NotBranchError,
+                          WorkingTree.open_containing, 
+                          'file:///' + getcwd())
+
+    def test_construct_with_branch(self):
+        branch = Branch.initialize(u'.')
+        tree = WorkingTree(branch.base, branch)
+        self.assertEqual(branch, tree.branch)
+        self.assertEqual(branch.base, tree.basedir + '/')
+    
+    def test_construct_without_branch(self):
+        branch = Branch.initialize(u'.')
+        tree = WorkingTree(branch.base)
+        self.assertEqual(branch.base, tree.branch.base)
+        self.assertEqual(branch.base, tree.basedir + '/')
+
+    def test_basic_relpath(self):
+        # for comprehensive relpath tests, see whitebox.py.
+        branch = Branch.initialize(u'.')
+        tree = WorkingTree(branch.base)
+        self.assertEqual('child',
+                         tree.relpath(pathjoin(getcwd(), 'child')))
+
+    def test_lock_locks_branch(self):
+        branch = Branch.initialize(u'.')
+        tree = WorkingTree(branch.base)
+        tree.lock_read()
+        self.assertEqual(1, tree.branch._lock_count)
+        self.assertEqual('r', tree.branch._lock_mode)
+        tree.unlock()
+        self.assertEqual(None, tree.branch._lock_count)
+        tree.lock_write()
+        self.assertEqual(1, tree.branch._lock_count)
+        self.assertEqual('w', tree.branch._lock_mode)
+        tree.unlock()
+        self.assertEqual(None, tree.branch._lock_count)
+ 
+    def get_pullable_branches(self):
+        self.build_tree(['from/', 'from/file', 'to/'])
+        br_a = Branch.initialize('from')
+        tree = br_a.working_tree()
+        tree.add('file')
+        tree.commit('foo', rev_id='A')
+        br_b = Branch.initialize('to')
+        return br_a, br_b
+ 
+    def test_pull(self):
+        br_a, br_b = self.get_pullable_branches()
+        br_b.working_tree().pull(br_a)
+        self.failUnless(br_b.has_revision('A'))
+        self.assertEqual(['A'], br_b.revision_history())
+
+    def test_pull_overwrites(self):
+        br_a, br_b = self.get_pullable_branches()
+        br_b.working_tree().commit('foo', rev_id='B')
+        self.assertEqual(['B'], br_b.revision_history())
+        br_b.working_tree().pull(br_a, overwrite=True)
+        self.failUnless(br_b.has_revision('A'))
+        self.failUnless(br_b.has_revision('B'))
+        self.assertEqual(['A'], br_b.revision_history())
+
+    def test_revert(self):
+        """Test selected-file revert"""
+        b = Branch.initialize(u'.')
+
+        self.build_tree(['hello.txt'])
+        file('hello.txt', 'w').write('initial hello')
+
+        self.assertRaises(NotVersionedError,
+                          b.working_tree().revert, ['hello.txt'])
+        tree = WorkingTree(b.base, b)
+        tree.add(['hello.txt'])
+        tree.commit('create initial hello.txt')
+
+        self.check_file_contents('hello.txt', 'initial hello')
+        file('hello.txt', 'w').write('new hello')
+        self.check_file_contents('hello.txt', 'new hello')
+
+        # revert file modified since last revision
+        tree.revert(['hello.txt'])
+        self.check_file_contents('hello.txt', 'initial hello')
+        self.check_file_contents('hello.txt~', 'new hello')
+
+        # reverting again does not clobber the backup
+        tree.revert(['hello.txt'])
+        self.check_file_contents('hello.txt', 'initial hello')
+        self.check_file_contents('hello.txt~', 'new hello')
+
+    def test_unknowns(self):
+        b = Branch.initialize(u'.')
+        tree = WorkingTree(u'.', b)
+        self.build_tree(['hello.txt',
+                         'hello.txt~'])
+        self.assertEquals(list(tree.unknowns()),
+                          ['hello.txt'])
+
+    def test_hashcache(self):
+        from bzrlib.tests.test_hashcache import pause
+        b = Branch.initialize(u'.')
+        tree = WorkingTree(u'.', b)
+        self.build_tree(['hello.txt',
+                         'hello.txt~'])
+        tree.add('hello.txt')
+        pause()
+        sha = tree.get_file_sha1(tree.path2id('hello.txt'))
+        self.assertEqual(1, tree._hashcache.miss_count)
+        tree2 = WorkingTree(u'.', b)
+        sha2 = tree2.get_file_sha1(tree2.path2id('hello.txt'))
+        self.assertEqual(0, tree2._hashcache.miss_count)
+        self.assertEqual(1, tree2._hashcache.hit_count)

=== modified file 'bzrlib/tests/test_xml.py'
--- bzrlib/selftest/test_xml.py	2005-09-30 14:27:41 +0000
+++ bzrlib/tests/test_xml.py	2005-11-23 15:44:24 +0000
@@ -16,7 +16,7 @@
 
 from cStringIO import StringIO
 
-from bzrlib.selftest import TestCase
+from bzrlib.tests import TestCase
 from bzrlib.inventory import Inventory, InventoryEntry
 from bzrlib.xml4 import serializer_v4
 from bzrlib.xml5 import serializer_v5
@@ -126,10 +126,21 @@
         self.assertEqual(inv, inv2)
 
     def test_repack_revision_5(self):
+        """Round-trip revision to XML v5"""
         inp = StringIO(_revision_v5)
         rev = serializer_v5.read_revision(inp)
         outp = StringIO()
         serializer_v5.write_revision(rev, outp)
-        rev2 = serializer_v5.read_revision(StringIO(outp.getvalue()))
+        outfile_contents = outp.getvalue()
+        rev2 = serializer_v5.read_revision(StringIO(outfile_contents))
         self.assertEqual(rev, rev2)
 
+    def test_pack_revision_5(self):
+        """Pack revision to XML v5"""
+        # fixed 20051025, revisions should have final newline
+        rev = serializer_v5.read_revision_from_string(_revision_v5)
+        outp = StringIO()
+        serializer_v5.write_revision(rev, outp)
+        outfile_contents = outp.getvalue()
+        self.assertEqual(outfile_contents[-1], '\n')
+        self.assertEqualDiff(outfile_contents, serializer_v5.write_revision_to_string(rev))

=== modified file 'bzrlib/tests/treeshape.py'
--- bzrlib/selftest/treeshape.py	2005-10-04 10:10:48 +0000
+++ bzrlib/tests/treeshape.py	2005-12-01 17:25:16 +0000
@@ -31,6 +31,7 @@
 import stat
 
 from bzrlib.trace import warning
+from bzrlib.osutils import pathjoin
 
 def build_tree_contents(template):
     """Reconstitute some files from a text description.
@@ -63,8 +64,8 @@
         yield (dirpath + '/', )
         filenames.sort()
         for fn in filenames:
-            fullpath = os.path.join(dirpath, fn)
-            assert fullpath[-1] not in '@/'
+            fullpath = pathjoin(dirpath, fn)
+            self.assertFalse(fullpath[-1] in '@/')
             info = os.lstat(fullpath)
             if stat.S_ISLNK(info.st_mode):
                 yield (fullpath + '@', os.readlink(fullpath))

=== modified file 'bzrlib/trace.py'
--- bzrlib/trace.py	2005-10-06 07:28:41 +0000
+++ bzrlib/trace.py	2005-12-16 03:43:14 +0000
@@ -1,47 +1,58 @@
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-
-
-"""Messages and logging for bazaar-ng
-
-Messages are sent out through the Python logging library.
-
-They can be sent to two places: to stderr, and to ~/.bzr.log.
-
-~/.bzr.log gets all messages, and tracebacks of all uncaught
-exceptions.
-
-Normally stderr only gets messages of level INFO and higher, and gets
-only a summary of exceptions, not the traceback.
+# Copyright (C) 2005, Canonical Ltd
+
+"""Messages and logging for bazaar-ng.
+
+Messages are supplied by callers as a string-formatting template, plus values
+to be inserted into it.  The actual %-formatting is deferred to the log
+library so that it doesn't need to be done for messages that won't be emitted.
+
+Messages are classified by severity levels: critical, error, warning, info,
+and debug.
+
+They can be sent to two places: to stderr, and to ~/.bzr.log.  For purposes
+such as running the test suite, they can also be redirected away from both of
+those two places to another location.
+
+~/.bzr.log gets all messages, and full tracebacks for uncaught exceptions.
+This trace file is always in UTF-8, regardless of the user's default encoding,
+so that we can always rely on writing any message.
+
+Output to stderr depends on the mode chosen by the user.  By default, messages
+of info and above are sent out, which results in progress messages such as the
+list of files processed by add and commit.  In quiet mode, only warnings and
+above are shown.  In debug mode, stderr gets debug messages too.
+
+Errors that terminate an operation are generally passed back as exceptions;
+others may be just emitted as messages.
+
+Exceptions are reported in a brief form to stderr so as not to look scary.
+BzrErrors are required to be able to format themselves into a properly
+explanatory message.  This is not true for builtin excexceptions such as
+KeyError, which typically just str to "0".  They're printed in a different
+form.
 """
 
-
 # TODO: in debug mode, stderr should get full tracebacks and also
 # debug messages.  (Is this really needed?)
 
-# TODO: When running the test suites, we should add an additional
-# logger that sends messages into the test log file.
+# FIXME: Unfortunately it turns out that python's logging module
+# is quite expensive, even when the message is not printed by any handlers.
+# We should perhaps change back to just simply doing it here.
 
 
 import sys
 import os
 import logging
-import traceback
+
+import bzrlib
+from bzrlib.errors import BzrError, BzrNewError
 
 
 _file_handler = None
 _stderr_handler = None
+_stderr_quiet = False
+_trace_file = None
+_bzr_log_file = None
 
 
 class QuietFormatter(logging.Formatter):
@@ -58,44 +69,34 @@
             s = 'bzr: ' + record.levelname + ': '
         else:
             s = ''
-            
         s += record.getMessage()
-
-        ##import textwrap
-        ##s = textwrap.fill(s)
-            
         if record.exc_info:
-            # give just a summary of the exception, not the whole thing
-            exinfo = traceback.extract_tb(record.exc_info[2])
-            # the format of this string matches one of the REs
-            s += '\n'
-            s += ('  at %s line %d, in %s()\n' % exinfo[-1][:3])
-            s += '  see ~/.bzr.log for debug information'
-
+            s += '\n' + format_exception_short(record.exc_info)
         return s
         
-
-
-
-################
 # configure convenient aliases for output routines
 
 _bzr_logger = logging.getLogger('bzr')
-_bzr_logger.setLevel(logging.DEBUG) 
 
 info = note = _bzr_logger.info
 warning =   _bzr_logger.warning
 log_error = _bzr_logger.error
 error =     _bzr_logger.error
-mutter =    _bzr_logger.debug
-debug =     _bzr_logger.debug
-
-
-
-
-# we do the rollover using this code, rather than the default from python
-# logging, because we only want to rollover at program startup, not on each
-# message.  maybe that's not a good enough reason.
+
+
+def mutter(fmt, *args):
+    if _trace_file is None:
+        return
+    if hasattr(_trace_file, 'closed') and _trace_file.closed:
+        return
+    if len(args) > 0:
+        out = fmt % args
+    else:
+        out = fmt
+    out += '\n'
+    _trace_file.write(out)
+debug = mutter
+
 
 def _rollover_trace_maybe(trace_fname):
     import stat
@@ -104,92 +105,103 @@
         if size <= 4 << 20:
             return
         old_fname = trace_fname + '.old'
-
         from osutils import rename
         rename(trace_fname, old_fname)
-
     except OSError:
         return
 
 
-
 def open_tracefile(tracefilename='~/.bzr.log'):
     # Messages are always written to here, so that we have some
     # information if something goes wrong.  In a future version this
     # file will be removed on successful completion.
-    global _file_handler
+    global _file_handler, _bzr_log_file
     import stat, codecs
 
     trace_fname = os.path.join(os.path.expanduser(tracefilename))
     _rollover_trace_maybe(trace_fname)
-
-    # buffering=1 means line buffered
     try:
-        tf = codecs.open(trace_fname, 'at', 'utf8', buffering=1)
-
-        if os.fstat(tf.fileno())[stat.ST_SIZE] == 0:
+        LINE_BUFFERED = 1
+        tf = codecs.open(trace_fname, 'at', 'utf8', buffering=LINE_BUFFERED)
+        _bzr_log_file = tf
+        if tf.tell() == 0:
             tf.write("\nthis is a debug log for diagnosing/reporting problems in bzr\n")
             tf.write("you can delete or truncate this file, or include sections in\n")
             tf.write("bug reports to bazaar-ng@lists.canonical.com\n\n")
-        
         _file_handler = logging.StreamHandler(tf)
         fmt = r'[%(process)5d] %(asctime)s.%(msecs)03d %(levelname)s: %(message)s'
         datefmt = r'%a %H:%M:%S'
         _file_handler.setFormatter(logging.Formatter(fmt, datefmt))
         _file_handler.setLevel(logging.DEBUG)
         logging.getLogger('').addHandler(_file_handler)
-
     except IOError, e:
         warning("failed to open trace file: %s" % (e))
 
 
 def log_startup(argv):
-    import bzrlib
-
-    debug('bzr %s invoked on python %s (%s)',
+    debug('\n\nbzr %s invoked on python %s (%s)',
           bzrlib.__version__,
           '.'.join(map(str, sys.version_info)),
           sys.platform)
-
     debug('  arguments: %r', argv)
-    debug('  working dir: %s', os.getcwdu())
+    debug('  working dir: %r', os.getcwdu())
 
 
 def log_exception(msg=None):
-    """Log the last exception into the trace file.
+    """Log the last exception to stderr and the trace file.
 
     The exception string representation is used as the error
     summary, unless msg is given.
     """
-    if msg == None:
-        ei = sys.exc_info()
-        msg = str(ei[1])
-    if msg and (msg[-1] == '\n'):
-        msg = msg[:-1]
-    ## msg = "(%s) %s" % (str(type(ei[1])), msg)
-    _bzr_logger.exception(msg)
-
+    if msg:
+        error(msg)
+    else:
+        exc_str = format_exception_short(sys.exc_info())
+        error(exc_str)
+    log_exception_quietly()
+
+
+def log_exception_quietly():
+    """Log the last exception to the trace file only.
+
+    Used for exceptions that occur internally and that may be 
+    interesting to developers but not to users.  For example, 
+    errors loading plugins.
+    """
+    import traceback
+    debug(traceback.format_exc())
 
 
 def enable_default_logging():
     """Configure default logging to stderr and .bzr.log"""
-    global _stderr_handler, _file_handler
-
+    # FIXME: if this is run twice, things get confused
+    global _stderr_handler, _file_handler, _trace_file, _bzr_log_file
     _stderr_handler = logging.StreamHandler()
     _stderr_handler.setFormatter(QuietFormatter())
-
+    logging.getLogger('').addHandler(_stderr_handler)
+    _stderr_handler.setLevel(logging.INFO)
     if not _file_handler:
         open_tracefile()
-
-    if os.environ.get('BZR_DEBUG'):
-        level = logging.DEBUG
+    _trace_file = _bzr_log_file
+    if _file_handler:
+        _file_handler.setLevel(logging.DEBUG)
+    _bzr_logger.setLevel(logging.DEBUG) 
+
+
+
+def be_quiet(quiet=True):
+    global _stderr_handler, _stderr_quiet
+    
+    _stderr_quiet = quiet
+    if quiet:
+        _stderr_handler.setLevel(logging.WARNING)
     else:
-        level = logging.INFO
-
-    _stderr_handler.setLevel(logging.INFO)
-    _file_handler.setLevel(level)
-
-    logging.getLogger('').addHandler(_stderr_handler)
+        _stderr_handler.setLevel(logging.INFO)
+
+
+def is_quiet():
+    global _stderr_quiet
+    return _stderr_quiet
 
 
 def disable_default_logging():
@@ -203,3 +215,54 @@
     l.removeHandler(_stderr_handler)
     if _file_handler:
         l.removeHandler(_file_handler)
+    _trace_file = None
+
+
+def enable_test_log(to_file):
+    """Redirect logging to a temporary file for a test"""
+    disable_default_logging()
+    global _test_log_hdlr, _trace_file
+    hdlr = logging.StreamHandler(to_file)
+    hdlr.setLevel(logging.DEBUG)
+    hdlr.setFormatter(logging.Formatter('%(levelname)8s  %(message)s'))
+    _bzr_logger.addHandler(hdlr)
+    _bzr_logger.setLevel(logging.DEBUG)
+    _test_log_hdlr = hdlr
+    _trace_file = to_file
+
+
+def disable_test_log():
+    _bzr_logger.removeHandler(_test_log_hdlr)
+    _trace_file = None
+    enable_default_logging()
+
+
+def format_exception_short(exc_info):
+    """Make a short string form of an exception.
+
+    This is used for display to stderr.  It specially handles exception
+    classes without useful string methods.
+
+    The result has no trailing newline.
+
+    exc_info - typically an exception from sys.exc_info()
+    """
+    exc_type, exc_object, exc_tb = exc_info
+    try:
+        if exc_type is None:
+            return '(no exception)'
+        if isinstance(exc_object, (BzrError, BzrNewError)):
+            return str(exc_object)
+        else:
+            import traceback
+            tb = traceback.extract_tb(exc_tb)
+            msg = '%s: %s' % (exc_type, exc_object)
+            if msg[-1] == '\n':
+                msg = msg[:-1]
+            if tb:
+                msg += '\n  at %s line %d\n  in %s' % (tb[-1][:3])
+            return msg
+    except Exception, formatting_exc:
+        # XXX: is this really better than just letting it run up?
+        return '(error formatting exception of type %s: %s)' \
+                % (exc_type, formatting_exc)

=== added file 'bzrlib/transactions.py'
--- bzrlib/transactions.py	1970-01-01 00:00:00 +0000
+++ bzrlib/transactions.py	2005-10-11 01:05:24 +0000
@@ -0,0 +1,162 @@
+# Copyright (C) 2005 by Canonical Ltd
+#   Authors: Robert Collins <robert.collins@canonical.com>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""This module provides a transactional facility.
+
+Transactions provide hooks to allow data objects (i.e. inventory weaves or
+the revision-history file) to be placed in a registry and retrieved later
+during the same transaction.  This allows for repeated read isolation. At
+the end of a transaction, a callback is issued to each registered changed
+item informing it whether it should commit or not. We provide a two layer
+facility - domain objects are notified first, then data objects.
+
+Read only transactions raise an assert when objects are listed as dirty
+against them - preventing unintended writes. Once all the data storage is
+hooked into this facility, it might be nice to have a readonly transaction
+that just excepts on commit, for testing or simulating of things.
+
+Write transactions queue all changes in the transaction (which may in the 
+future involve writing them to uncommitted atomic files in preparation 
+for commit - i.e. on network connections where latency matters) and then
+notify each object of commit or rollback.
+
+Both read and write transactions *may* flush unchanged objects out of 
+memory, unless they are marked as 'preserve' which indicates that 
+repeated reads cannot be obtained if the object is ejected.
+"""
+
+import sys
+
+import bzrlib.errors as errors
+from bzrlib.identitymap import IdentityMap, NullIdentityMap
+from bzrlib.trace import mutter
+
+
+class ReadOnlyTransaction(object):
+    """A read only unit of work for data objects."""
+
+    def commit(self):
+        """ReadOnlyTransactions cannot commit."""
+        raise errors.CommitNotPossible('In a read only transaction')
+
+    def finish(self):
+        """Clean up this transaction
+
+        This will rollback on transactions that can if they have nto been
+        committed.
+        """
+
+    def __init__(self):
+        super(ReadOnlyTransaction, self).__init__()
+        self.map = IdentityMap()
+        self._clean_objects = set()
+        self._clean_queue = []
+        self._limit = -1
+        self._precious_objects = set()
+
+    def register_clean(self, an_object, precious=False):
+        """Register an_object as being clean.
+        
+        If the precious hint is True, the object will not
+        be ejected from the object identity map ever.
+        """
+        self._clean_objects.add(an_object)
+        self._clean_queue.append(an_object)
+        if precious:
+            self._precious_objects.add(an_object)
+        self._trim()
+
+    def register_dirty(self, an_object):
+        """Register an_object as being dirty."""
+        raise errors.ReadOnlyError(
+            "Cannot dirty objects in a read only transaction")
+
+    def rollback(self):
+        """Let people call this even though nothing has to happen."""
+
+    def set_cache_size(self, size):
+        """Set a new cache size."""
+        assert -1 <= size
+        self._limit = size
+        self._trim()
+
+    def _trim(self):
+        """Trim the cache back if needed."""
+        if self._limit < 0 or self._limit - len(self._clean_objects) > 0:
+            return
+        needed = len(self._clean_objects) - self._limit
+        offset = 0
+        while needed and offset < len(self._clean_objects):
+            # references we know of:
+            # temp passed to getrefcount in our frame
+            # temp in getrefcount's frame
+            # the map forward
+            # the map backwards
+            # _clean_objects
+            # _clean_queue
+            # 1 missing ?
+            if (sys.getrefcount(self._clean_queue[offset]) <= 7 and
+                not self._clean_queue[offset] in self._precious_objects):
+                removed = self._clean_queue[offset]
+                self._clean_objects.remove(removed)
+                del self._clean_queue[offset]
+                self.map.remove_object(removed)
+                mutter('removed object %r', removed)
+                needed -= 1
+            else:
+                offset += 1
+
+
+        
+class PassThroughTransaction(object):
+    """A pass through transaction
+    
+    - all actions are committed immediately.
+    - rollback is not supported.
+    - commit() is a no-op.
+    """
+
+    def commit(self):
+        """PassThroughTransactions have nothing to do."""
+
+    def finish(self):
+        """Clean up this transaction
+
+        This will rollback on transactions that can if they have nto been
+        committed.
+        """
+
+    def __init__(self):
+        super(PassThroughTransaction, self).__init__()
+        self.map = NullIdentityMap()
+
+    def register_clean(self, an_object, precious=False):
+        """Register an_object as being clean.
+        
+        Note that precious is only a hint, and PassThroughTransaction
+        ignores it.
+        """
+
+    def register_dirty(self, an_object):
+        """Register an_object as being dirty."""
+
+    def rollback(self):
+        """Cannot rollback a pass through transaction."""
+        raise errors.AlreadyCommitted
+
+    def set_cache_size(self, ignored):
+        """Do nothing, we are passing through."""

=== modified file 'bzrlib/transport/__init__.py'
--- bzrlib/transport/__init__.py	2005-10-04 04:37:29 +0000
+++ bzrlib/transport/__init__.py	2006-01-12 01:02:32 +0000
@@ -19,26 +19,59 @@
 as remote (such as http or sftp).
 """
 
+import errno
+from copy import deepcopy
+import sys
+from unittest import TestSuite
+
 from bzrlib.trace import mutter
-from bzrlib.errors import (BzrError, 
-    TransportError, TransportNotPossible, NonRelativePath,
-    NoSuchFile, FileExists, PermissionDenied,
-    ConnectionReset)
+import bzrlib.errors as errors
 
 _protocol_handlers = {
 }
 
 def register_transport(prefix, klass, override=True):
     global _protocol_handlers
-
+    # trace messages commented out because they're typically 
+    # run during import before trace is set up
     if _protocol_handlers.has_key(prefix):
         if override:
-            mutter('overriding transport: %s => %s' % (prefix, klass.__name__))
+            ## mutter('overriding transport: %s => %s' % (prefix, klass.__name__))
             _protocol_handlers[prefix] = klass
     else:
-        mutter('registering transport: %s => %s' % (prefix, klass.__name__))
+        ## mutter('registering transport: %s => %s' % (prefix, klass.__name__))
         _protocol_handlers[prefix] = klass
 
+
+def _get_protocol_handlers():
+    """Return a dictionary of prefix:transport-factories."""
+    return _protocol_handlers
+
+
+def _set_protocol_handlers(new_handlers):
+    """Replace the current protocol handlers dictionary.
+
+    WARNING this will remove all build in protocols. Use with care.
+    """
+    global _protocol_handlers
+    _protocol_handlers = new_handlers
+
+
+def _get_transport_modules():
+    """Return a list of the modules providing transports."""
+    modules = set()
+    for prefix, factory in _protocol_handlers.items():
+        if factory.__module__ == "bzrlib.transport":
+            # this is a lazy load transport, because no real ones
+            # are directlry in bzrlib.transport
+            modules.add(factory.module)
+        else:
+            modules.add(factory.__module__)
+    result = list(modules)
+    result.sort()
+    return result
+
+
 class Transport(object):
     """This class encapsulates methods for retrieving or putting a file
     from/to a storage location.
@@ -55,6 +88,27 @@
         super(Transport, self).__init__()
         self.base = base
 
+    def _translate_error(self, e, path, raise_generic=True):
+        """Translate an IOError or OSError into an appropriate bzr error.
+
+        This handles things like ENOENT, ENOTDIR, EEXIST, and EACCESS
+        """
+        if hasattr(e, 'errno'):
+            if e.errno in (errno.ENOENT, errno.ENOTDIR):
+                raise errors.NoSuchFile(path, extra=e)
+            # I would rather use errno.EFOO, but there doesn't seem to be
+            # any matching for 267
+            # This is the error when doing a listdir on a file:
+            # WindowsError: [Errno 267] The directory name is invalid
+            if sys.platform == 'win32' and e.errno in (errno.ESRCH, 267):
+                raise errors.NoSuchFile(path, extra=e)
+            if e.errno == errno.EEXIST:
+                raise errors.FileExists(path, extra=e)
+            if e.errno == errno.EACCES:
+                raise errors.PermissionDenied(path, extra=e)
+        if raise_generic:
+            raise errors.TransportError(orig_error=e)
+
     def clone(self, offset=None):
         """Return a new Transport object, cloned from the current location,
         using a subdirectory or parent directory. This allows connections 
@@ -121,16 +175,33 @@
     def abspath(self, relpath):
         """Return the full url to the given relative path.
         This can be supplied with a string or a list
+
+        XXX: Robert Collins 20051016 - is this really needed in the public
+             interface ?
         """
         raise NotImplementedError
 
     def relpath(self, abspath):
         """Return the local path portion from a given absolute path.
+
+        This default implementation is not suitable for filesystems with
+        aliasing, such as that given by symlinks, where a path may not 
+        start with our base, but still be a relpath once aliasing is 
+        resolved.
         """
-        raise NotImplementedError
+        # TODO: This might want to use bzrlib.osutils.relpath
+        #       but we have to watch out because of the prefix issues
+        if not (abspath == self.base[:-1] or abspath.startswith(self.base)):
+            raise errors.PathNotChild(abspath, self.base)
+        pl = len(self.base)
+        return abspath[pl:].strip('/')
 
     def has(self, relpath):
-        """Does the target location exist?"""
+        """Does the file relpath exist?
+        
+        Note that some transports MAY allow querying on directories, but this
+        is not part of the protocol.
+        """
         raise NotImplementedError
 
     def has_multi(self, relpaths, pb=None):
@@ -142,6 +213,24 @@
             yield self.has(relpath)
             count += 1
 
+    def has_any(self, relpaths):
+        """Return True if any of the paths exist."""
+        for relpath in relpaths:
+            if self.has(relpath):
+                return True
+        return False
+
+    def iter_files_recursive(self):
+        """Iter the relative paths of files in the transports sub-tree.
+        
+        As with other listing functions, only some transports implement this,.
+        you may check via is_listable to determine if it will.
+        """
+        raise errors.TransportNotPossible("This transport has not "
+                                          "implemented iter_files_recursive "
+                                          "(but must claim to be listable "
+                                          "to trigger this error).")
+
     def get(self, relpath):
         """Get the file at the given relative path.
 
@@ -166,63 +255,37 @@
             yield self.get(relpath)
             count += 1
 
-    def get_partial(self, relpath, start, length=None):
-        """Get just part of a file.
-
-        :param relpath: Path to the file, relative to base
-        :param start: The starting position to read from
-        :param length: The length to read. A length of None indicates
-                       read to the end of the file.
-        :return: A file-like object containing at least the specified bytes.
-                 Some implementations may return objects which can be read
-                 past this length, but this is not guaranteed.
-        """
-        raise NotImplementedError
-
-    def get_partial_multi(self, offsets, pb=None):
-        """Put a set of files or strings into the location.
-
-        Requesting multiple portions of the same file can be dangerous.
-
-        :param offsets: A list of tuples of relpath, start, length
-                         [(path1, start1, length1),
-                          (path2, start2, length2),
-                          (path3, start3), ...]
-                         length is optional, defaulting to None
-        :param pb:  An optional ProgressBar for indicating percent done.
-        :return: A generator of file-like objects.
-        """
-        total = self._get_total(offsets)
-        count = 0
-        for offset in offsets:
-            self._update_pb(pb, 'get_partial', count, total)
-            yield self.get_partial(*offset)
-            count += 1
-
-    def put(self, relpath, f):
+    def put(self, relpath, f, mode=None):
         """Copy the file-like or string object into the location.
 
         :param relpath: Location to put the contents, relative to base.
         :param f:       File-like or string object.
+        :param mode: The mode for the newly created file, 
+                     None means just use the default
         """
         raise NotImplementedError
 
-    def put_multi(self, files, pb=None):
-        """Put a set of files or strings into the location.
+    def put_multi(self, files, mode=None, pb=None):
+        """Put a set of files into the location.
 
         :param files: A list of tuples of relpath, file object [(path1, file1), (path2, file2),...]
         :param pb:  An optional ProgressBar for indicating percent done.
+        :param mode: The mode for the newly created files
         :return: The number of files copied.
         """
-        return self._iterate_over(files, self.put, pb, 'put', expand=True)
+        def put(path, f):
+            self.put(path, f, mode=mode)
+        return self._iterate_over(files, put, pb, 'put', expand=True)
 
-    def mkdir(self, relpath):
+    def mkdir(self, relpath, mode=None):
         """Create a directory at the given path."""
         raise NotImplementedError
 
-    def mkdir_multi(self, relpaths, pb=None):
+    def mkdir_multi(self, relpaths, mode=None, pb=None):
         """Create a group of directories"""
-        return self._iterate_over(relpaths, self.mkdir, pb, 'mkdir', expand=False)
+        def mkdir(path):
+            self.mkdir(path, mode=mode)
+        return self._iterate_over(relpaths, mkdir, pb, 'mkdir', expand=False)
 
     def append(self, relpath, f):
         """Append the text in the file-like or string object to 
@@ -240,8 +303,12 @@
         return self._iterate_over(files, self.append, pb, 'append', expand=True)
 
     def copy(self, rel_from, rel_to):
-        """Copy the item at rel_from to the location at rel_to"""
-        raise NotImplementedError
+        """Copy the item at rel_from to the location at rel_to.
+        
+        Override this for efficiency if a specific transport can do it 
+        faster than this default implementation.
+        """
+        self.put(rel_to, self.get(rel_from))
 
     def copy_multi(self, relpaths, pb=None):
         """Copy a bunch of entries.
@@ -252,21 +319,29 @@
         # implementors don't have to implement everything.
         return self._iterate_over(relpaths, self.copy, pb, 'copy', expand=True)
 
-    def copy_to(self, relpaths, other, pb=None):
+    def copy_to(self, relpaths, other, mode=None, pb=None):
         """Copy a set of entries from self into another Transport.
 
         :param relpaths: A list/generator of entries to be copied.
+        :param mode: This is the target mode for the newly created files
+        TODO: This interface needs to be updated so that the target location
+              can be different from the source location.
         """
         # The dummy implementation just does a simple get + put
         def copy_entry(path):
-            other.put(path, self.get(path))
+            other.put(path, self.get(path), mode=mode)
 
         return self._iterate_over(relpaths, copy_entry, pb, 'copy_to', expand=False)
 
 
     def move(self, rel_from, rel_to):
-        """Move the item at rel_from to the location at rel_to"""
-        raise NotImplementedError
+        """Move the item at rel_from to the location at rel_to.
+        
+        If a transport can directly implement this it is suggested that
+        it do so for efficiency.
+        """
+        self.copy(rel_from, rel_to)
+        self.delete(rel_from)
 
     def move_multi(self, relpaths, pb=None):
         """Move a bunch of entries.
@@ -300,6 +375,11 @@
         """Return the stat information for a file.
         WARNING: This may not be implementable for all protocols, so use
         sparingly.
+        NOTE: This returns an object with fields such as 'st_size'. It MAY
+        or MAY NOT return the literal result of an os.stat() call, so all
+        access should be via named fields.
+        ALSO NOTE: Stats of directories may not be supported on some 
+        transports.
         """
         raise NotImplementedError
 
@@ -324,8 +404,10 @@
         WARNING: many transports do not support this, so trying avoid using
         it if at all possible.
         """
-        raise TransportNotPossible("This transport has not "
-                                   "implemented list_dir.")
+        raise errors.TransportNotPossible("This transport has not "
+                                          "implemented list_dir "
+                                          "(but must claim to be listable "
+                                          "to trigger this error).")
 
     def lock_read(self, relpath):
         """Lock the given file for shared (read) access.
@@ -343,11 +425,17 @@
         """
         raise NotImplementedError
 
+    def is_readonly(self):
+        """Return true if this connection cannot be written to."""
+        return False
+
 
 def get_transport(base):
     global _protocol_handlers
     if base is None:
-        base = '.'
+        base = u'.'
+    else:
+        base = unicode(base)
     for proto, klass in _protocol_handlers.iteritems():
         if proto is not None and base.startswith(proto):
             return klass(base)
@@ -355,5 +443,105 @@
     # which has a lookup of None
     return _protocol_handlers[None](base)
 
-# Local transport should always be initialized
-import bzrlib.transport.local
+
+def register_lazy_transport(scheme, module, classname):
+    """Register lazy-loaded transport class.
+
+    When opening a URL with the given scheme, load the module and then
+    instantiate the particular class.  
+    """
+    def _loader(base):
+        mod = __import__(module, globals(), locals(), [classname])
+        klass = getattr(mod, classname)
+        return klass(base)
+    _loader.module = module
+    register_transport(scheme, _loader)
+
+
+def urlescape(relpath):
+    """Escape relpath to be a valid url."""
+    # TODO utf8 it first. utf8relpath = relpath.encode('utf8')
+    import urllib
+    return urllib.quote(relpath)
+
+
+class Server(object):
+    """A Transport Server.
+    
+    The Server interface provides a server for a given transport. We use
+    these servers as loopback testing tools. For any given transport the
+    Servers it provides must either allow writing, or serve the contents
+    of os.getcwdu() at the time setUp is called.
+    
+    Note that these are real servers - they must implement all the things
+    that we want bzr transports to take advantage of.
+    """
+
+    def setUp(self):
+        """Setup the server to service requests."""
+
+    def tearDown(self):
+        """Remove the server and cleanup any resources it owns."""
+
+    def get_url(self):
+        """Return a url for this server.
+        
+        If the transport does not represent a disk directory (i.e. it is 
+        a database like svn, or a memory only transport, it should return
+        a connection to a newly established resource for this Server.
+        Otherwise it should return a url that will provide access to the path
+        that was os.getcwdu() when setUp() was called.
+        
+        Subsequent calls will return the same resource.
+        """
+        raise NotImplementedError
+
+    def get_bogus_url(self):
+        """Return a url for this protocol, that will fail to connect."""
+        raise NotImplementedError
+
+
+class TransportTestProviderAdapter(object):
+    """A tool to generate a suite testing all transports for a single test.
+
+    This is done by copying the test once for each transport and injecting
+    the transport_class and transport_server classes into each copy. Each copy
+    is also given a new id() to make it easy to identify.
+    """
+
+    def adapt(self, test):
+        result = TestSuite()
+        for klass, server_factory in self._test_permutations():
+            new_test = deepcopy(test)
+            new_test.transport_class = klass
+            new_test.transport_server = server_factory
+            def make_new_test_id():
+                new_id = "%s(%s)" % (new_test.id(), server_factory.__name__)
+                return lambda: new_id
+            new_test.id = make_new_test_id()
+            result.addTest(new_test)
+        return result
+
+    def get_transport_test_permutations(self, module):
+        """Get the permutations module wants to have tested."""
+        return module.get_test_permutations()
+
+    def _test_permutations(self):
+        """Return a list of the klass, server_factory pairs to test."""
+        result = []
+        for module in _get_transport_modules():
+            result.extend(self.get_transport_test_permutations(reduce(getattr, 
+                (module).split('.')[1:],
+                 __import__(module))))
+        return result
+        
+
+# None is the default transport, for things with no url scheme
+register_lazy_transport(None, 'bzrlib.transport.local', 'LocalTransport')
+register_lazy_transport('file://', 'bzrlib.transport.local', 'LocalTransport')
+register_lazy_transport('sftp://', 'bzrlib.transport.sftp', 'SFTPTransport')
+register_lazy_transport('http://', 'bzrlib.transport.http', 'HttpTransport')
+register_lazy_transport('https://', 'bzrlib.transport.http', 'HttpTransport')
+register_lazy_transport('ftp://', 'bzrlib.transport.ftp', 'FtpTransport')
+register_lazy_transport('aftp://', 'bzrlib.transport.ftp', 'FtpTransport')
+register_lazy_transport('memory://', 'bzrlib.transport.memory', 'MemoryTransport')

=== added file 'bzrlib/transport/ftp.py'
--- bzrlib/transport/ftp.py	1970-01-01 00:00:00 +0000
+++ bzrlib/transport/ftp.py	2006-01-10 01:10:19 +0000
@@ -0,0 +1,312 @@
+# Copyright (C) 2005 Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+"""Implementation of Transport over ftp.
+
+Written by Daniel Silverstone <dsilvers@digital-scurf.org> with serious
+cargo-culting from the sftp transport and the http transport.
+
+It provides the ftp:// and aftp:// protocols where ftp:// is passive ftp
+and aftp:// is active ftp. Most people will want passive ftp for traversing
+NAT and other firewalls, so it's best to use it unless you explicitly want
+active, in which case aftp:// will be your friend.
+"""
+
+from cStringIO import StringIO
+import errno
+import ftplib
+import os
+import urllib
+import urlparse
+import stat
+from warnings import warn
+
+
+from bzrlib.transport import Transport
+from bzrlib.errors import (TransportNotPossible, TransportError,
+                           NoSuchFile, FileExists)
+from bzrlib.trace import mutter
+
+
+class FtpStatResult(object):
+    def __init__(self, f, relpath):
+        try:
+            self.st_size = f.size(relpath)
+            self.st_mode = stat.S_IFREG
+        except ftplib.error_perm:
+            pwd = f.pwd()
+            try:
+                f.cwd(relpath)
+                self.st_mode = stat.S_IFDIR
+            finally:
+                f.cwd(pwd)
+
+
+class FtpTransport(Transport):
+    """This is the transport agent for ftp:// access."""
+
+    def __init__(self, base, _provided_instance=None):
+        """Set the base path where files will be stored."""
+        assert base.startswith('ftp://') or base.startswith('aftp://')
+        super(FtpTransport, self).__init__(base)
+        self.is_active = base.startswith('aftp://')
+        if self.is_active:
+            base = base[1:]
+        (self._proto, self._host,
+            self._path, self._parameters,
+            self._query, self._fragment) = urlparse.urlparse(self.base)
+        self._FTP_instance = _provided_instance
+
+
+    def _get_FTP(self):
+        """Return the ftplib.FTP instance for this object."""
+        if self._FTP_instance is not None:
+            return self._FTP_instance
+        
+        try:
+            username = ''
+            password = ''
+            hostname = self._host
+            if '@' in hostname:
+                username, hostname = hostname.split("@", 1)
+            if ':' in username:
+                username, password = username.split(":", 1)
+
+            mutter("Constructing FTP instance")
+            self._FTP_instance = ftplib.FTP(hostname, username, password)
+            self._FTP_instance.set_pasv(not self.is_active)
+            return self._FTP_instance
+        except ftplib.error_perm, e:
+            raise TransportError(msg="Error setting up connection: %s"
+                                    % str(e), orig_error=e)
+
+    def should_cache(self):
+        """Return True if the data pulled across should be cached locally.
+        """
+        return True
+
+    def clone(self, offset=None):
+        """Return a new FtpTransport with root at self.base + offset.
+        """
+        mutter("FTP clone")
+        if offset is None:
+            return FtpTransport(self.base, self._FTP_instance)
+        else:
+            return FtpTransport(self.abspath(offset), self._FTP_instance)
+
+    def _abspath(self, relpath):
+        assert isinstance(relpath, basestring)
+        relpath = urllib.unquote(relpath)
+        if isinstance(relpath, basestring):
+            relpath_parts = relpath.split('/')
+        else:
+            # TODO: Don't call this with an array - no magic interfaces
+            relpath_parts = relpath[:]
+        if len(relpath_parts) > 1:
+            if relpath_parts[0] == '':
+                raise ValueError("path %r within branch %r seems to be absolute"
+                                 % (relpath, self._path))
+        basepath = self._path.split('/')
+        if len(basepath) > 0 and basepath[-1] == '':
+            basepath = basepath[:-1]
+        for p in relpath_parts:
+            if p == '..':
+                if len(basepath) == 0:
+                    # In most filesystems, a request for the parent
+                    # of root, just returns root.
+                    continue
+                basepath.pop()
+            elif p == '.' or p == '':
+                continue # No-op
+            else:
+                basepath.append(p)
+        # Possibly, we could use urlparse.urljoin() here, but
+        # I'm concerned about when it chooses to strip the last
+        # portion of the path, and when it doesn't.
+        return '/'.join(basepath)
+    
+    def abspath(self, relpath):
+        """Return the full url to the given relative path.
+        This can be supplied with a string or a list
+        """
+        path = self._abspath(relpath)
+        return urlparse.urlunparse((self._proto,
+                self._host, path, '', '', ''))
+
+    def has(self, relpath):
+        """Does the target location exist?
+
+        XXX: I assume we're never asked has(dirname) and thus I use
+        the FTP size command and assume that if it doesn't raise,
+        all is good.
+        """
+        try:
+            f = self._get_FTP()
+            s = f.size(self._abspath(relpath))
+            mutter("FTP has: %s" % self._abspath(relpath))
+            return True
+        except ftplib.error_perm:
+            mutter("FTP has not: %s" % self._abspath(relpath))
+            return False
+
+    def get(self, relpath, decode=False):
+        """Get the file at the given relative path.
+
+        :param relpath: The relative path to the file
+
+        We're meant to return a file-like object which bzr will
+        then read from. For now we do this via the magic of StringIO
+        """
+        try:
+            mutter("FTP get: %s" % self._abspath(relpath))
+            f = self._get_FTP()
+            ret = StringIO()
+            f.retrbinary('RETR '+self._abspath(relpath), ret.write, 8192)
+            ret.seek(0)
+            return ret
+        except ftplib.error_perm, e:
+            raise NoSuchFile(self.abspath(relpath), extra=extra)
+
+    def put(self, relpath, fp, mode=None):
+        """Copy the file-like or string object into the location.
+
+        :param relpath: Location to put the contents, relative to base.
+        :param f:       File-like or string object.
+        TODO: jam 20051215 This should be an atomic put, not overwritting files in place
+        TODO: jam 20051215 ftp as a protocol seems to support chmod, but ftplib does not
+        """
+        if not hasattr(fp, 'read'):
+            fp = StringIO(fp)
+        try:
+            mutter("FTP put: %s" % self._abspath(relpath))
+            f = self._get_FTP()
+            f.storbinary('STOR '+self._abspath(relpath), fp, 8192)
+        except ftplib.error_perm, e:
+            raise TransportError(orig_error=e)
+
+    def mkdir(self, relpath, mode=None):
+        """Create a directory at the given path."""
+        try:
+            mutter("FTP mkd: %s" % self._abspath(relpath))
+            f = self._get_FTP()
+            try:
+                f.mkd(self._abspath(relpath))
+            except ftplib.error_perm, e:
+                s = str(e)
+                if 'File exists' in s:
+                    raise FileExists(self.abspath(relpath), extra=s)
+                else:
+                    raise
+        except ftplib.error_perm, e:
+            raise TransportError(orig_error=e)
+
+    def append(self, relpath, f):
+        """Append the text in the file-like object into the final
+        location.
+        """
+        raise TransportNotPossible('ftp does not support append()')
+
+    def copy(self, rel_from, rel_to):
+        """Copy the item at rel_from to the location at rel_to"""
+        raise TransportNotPossible('ftp does not (yet) support copy()')
+
+    def move(self, rel_from, rel_to):
+        """Move the item at rel_from to the location at rel_to"""
+        try:
+            mutter("FTP mv: %s => %s" % (self._abspath(rel_from),
+                                         self._abspath(rel_to)))
+            f = self._get_FTP()
+            f.rename(self._abspath(rel_from), self._abspath(rel_to))
+        except ftplib.error_perm, e:
+            raise TransportError(orig_error=e)
+
+    def delete(self, relpath):
+        """Delete the item at relpath"""
+        try:
+            mutter("FTP rm: %s" % self._abspath(relpath))
+            f = self._get_FTP()
+            f.delete(self._abspath(relpath))
+        except ftplib.error_perm, e:
+            raise TransportError(orig_error=e)
+
+    def listable(self):
+        """See Transport.listable."""
+        return True
+
+    def list_dir(self, relpath):
+        """See Transport.list_dir."""
+        try:
+            mutter("FTP nlst: %s" % self._abspath(relpath))
+            f = self._get_FTP()
+            basepath = self._abspath(relpath)
+            # FTP.nlst returns paths prefixed by relpath, strip 'em
+            the_list = f.nlst(basepath)
+            stripped = [path[len(basepath)+1:] for path in the_list]
+            # Remove . and .. if present, and return
+            return [path for path in stripped if path not in (".", "..")]
+        except ftplib.error_perm, e:
+            raise TransportError(orig_error=e)
+
+    def iter_files_recursive(self):
+        """See Transport.iter_files_recursive.
+
+        This is cargo-culted from the SFTP transport"""
+        mutter("FTP iter_files_recursive")
+        queue = list(self.list_dir("."))
+        while queue:
+            relpath = urllib.quote(queue.pop(0))
+            st = self.stat(relpath)
+            if stat.S_ISDIR(st.st_mode):
+                for i, basename in enumerate(self.list_dir(relpath)):
+                    queue.insert(i, relpath+"/"+basename)
+            else:
+                yield relpath
+
+    def stat(self, relpath):
+        """Return the stat information for a file.
+        """
+        try:
+            mutter("FTP stat: %s" % self._abspath(relpath))
+            f = self._get_FTP()
+            return FtpStatResult(f, self._abspath(relpath))
+        except ftplib.error_perm, e:
+            raise TransportError(orig_error=e)
+
+    def lock_read(self, relpath):
+        """Lock the given file for shared (read) access.
+        :return: A lock object, which should be passed to Transport.unlock()
+        """
+        # The old RemoteBranch ignore lock for reading, so we will
+        # continue that tradition and return a bogus lock object.
+        class BogusLock(object):
+            def __init__(self, path):
+                self.path = path
+            def unlock(self):
+                pass
+        return BogusLock(relpath)
+
+    def lock_write(self, relpath):
+        """Lock the given file for exclusive (write) access.
+        WARNING: many transports do not support this, so trying avoid using it
+
+        :return: A lock object, which should be passed to Transport.unlock()
+        """
+        return self.lock_read(relpath)
+
+
+def get_test_permutations():
+    """Return the permutations to be used in testing."""
+    warn("There are no FTP transport provider tests yet.")
+    return []

=== modified file 'bzrlib/transport/http.py'
--- bzrlib/transport/http.py	2005-10-04 04:37:29 +0000
+++ bzrlib/transport/http.py	2006-01-10 05:50:28 +0000
@@ -16,31 +16,61 @@
 """Implementation of Transport over http.
 """
 
-from bzrlib.transport import Transport, register_transport
-from bzrlib.errors import (TransportNotPossible, NoSuchFile, 
-                           NonRelativePath, TransportError)
 import os, errno
 from cStringIO import StringIO
-import urllib2
+import urllib, urllib2
 import urlparse
+from warnings import warn
 
+from bzrlib.transport import Transport, Server
+from bzrlib.errors import (TransportNotPossible, NoSuchFile, 
+                           TransportError, ConnectionError)
 from bzrlib.errors import BzrError, BzrCheckError
 from bzrlib.branch import Branch
 from bzrlib.trace import mutter
 
-# velocitynet.com.au transparently proxies connections and thereby
-# breaks keep-alive -- sucks!
-
-
+
+def extract_auth(url, password_manager):
+    """
+    Extract auth parameters from am HTTP/HTTPS url and add them to the given
+    password manager.  Return the url, minus those auth parameters (which
+    confuse urllib2).
+    """
+    assert url.startswith('http://') or url.startswith('https://')
+    scheme, host = url.split('//', 1)
+    if '/' in host:
+        host, path = host.split('/', 1)
+        path = '/' + path
+    else:
+        path = ''
+    port = ''
+    if '@' in host:
+        auth, host = host.split('@', 1)
+        if ':' in auth:
+            username, password = auth.split(':', 1)
+        else:
+            username, password = auth, None
+        if ':' in host:
+            host, port = host.split(':', 1)
+            port = ':' + port
+        # FIXME: if password isn't given, should we ask for it?
+        if password is not None:
+            username = urllib.unquote(username)
+            password = urllib.unquote(password)
+            password_manager.add_password(None, host, username, password)
+    url = scheme + '//' + host + port + path
+    return url
+    
 def get_url(url):
     import urllib2
     mutter("get_url %s" % url)
-    url_f = urllib2.urlopen(url)
+    manager = urllib2.HTTPPasswordMgrWithDefaultRealm()
+    url = extract_auth(url, manager)
+    auth_handler = urllib2.HTTPBasicAuthHandler(manager)
+    opener = urllib2.build_opener(auth_handler)
+    url_f = opener.open(url)
     return url_f
 
-class HttpTransportError(TransportError):
-    pass
-
 class HttpTransport(Transport):
     """This is the transport agent for http:// access.
     
@@ -50,6 +80,8 @@
     def __init__(self, base):
         """Set the base path where files will be stored."""
         assert base.startswith('http://') or base.startswith('https://')
+        if base[-1] != '/':
+            base = base + '/'
         super(HttpTransport, self).__init__(base)
         # In the future we might actually connect to the remote host
         # rather than using get_url
@@ -77,24 +109,33 @@
         """Return the full url to the given relative path.
         This can be supplied with a string or a list
         """
+        assert isinstance(relpath, basestring)
         if isinstance(relpath, basestring):
-            relpath = [relpath]
+            relpath_parts = relpath.split('/')
+        else:
+            # TODO: Don't call this with an array - no magic interfaces
+            relpath_parts = relpath[:]
+        if len(relpath_parts) > 1:
+            if relpath_parts[0] == '':
+                raise ValueError("path %r within branch %r seems to be absolute"
+                                 % (relpath, self._path))
+            if relpath_parts[-1] == '':
+                raise ValueError("path %r within branch %r seems to be a directory"
+                                 % (relpath, self._path))
         basepath = self._path.split('/')
         if len(basepath) > 0 and basepath[-1] == '':
             basepath = basepath[:-1]
-
-        for p in relpath:
+        for p in relpath_parts:
             if p == '..':
-                if len(basepath) < 0:
+                if len(basepath) == 0:
                     # In most filesystems, a request for the parent
                     # of root, just returns root.
                     continue
                 basepath.pop()
-            elif p == '.':
+            elif p == '.' or p == '':
                 continue # No-op
             else:
                 basepath.append(p)
-
         # Possibly, we could use urlparse.urljoin() here, but
         # I'm concerned about when it chooses to strip the last
         # portion of the path, and when it doesn't.
@@ -102,13 +143,6 @@
         return urlparse.urlunparse((self._proto,
                 self._host, path, '', '', ''))
 
-    def relpath(self, abspath):
-        if not abspath.startswith(self.base):
-            raise NonRelativePath('path %r is not under base URL %r'
-                           % (abspath, self.base))
-        pl = len(self.base)
-        return abspath[pl:].lstrip('/')
-
     def has(self, relpath):
         """Does the target location exist?
 
@@ -120,57 +154,52 @@
         cleaner if we just do an http HEAD request, and parse
         the return code.
         """
+        path = relpath
         try:
-            f = get_url(self.abspath(relpath))
+            path = self.abspath(relpath)
+            f = get_url(path)
             # Without the read and then close()
             # we tend to have busy sockets.
             f.read()
             f.close()
             return True
-        except BzrError:
-            return False
-        except urllib2.URLError:
-            return False
+        except urllib2.URLError, e:
+            mutter('url error code: %s for has url: %r', e.code, path)
+            if e.code == 404:
+                return False
+            raise
         except IOError, e:
+            mutter('io error: %s %s for has url: %r', 
+                e.errno, errno.errorcode.get(e.errno), path)
             if e.errno == errno.ENOENT:
                 return False
-            raise HttpTransportError(orig_error=e)
+            raise TransportError(orig_error=e)
 
     def get(self, relpath, decode=False):
         """Get the file at the given relative path.
 
         :param relpath: The relative path to the file
         """
+        path = relpath
         try:
-            return get_url(self.abspath(relpath))
-        except (BzrError, urllib2.URLError, IOError), e:
-            raise NoSuchFile(orig_error=e)
-        except Exception,e:
-            raise HttpTransportError(orig_error=e)
-
-    def get_partial(self, relpath, start, length=None):
-        """Get just part of a file.
-
-        :param relpath: Path to the file, relative to base
-        :param start: The starting position to read from
-        :param length: The length to read. A length of None indicates
-                       read to the end of the file.
-        :return: A file-like object containing at least the specified bytes.
-                 Some implementations may return objects which can be read
-                 past this length, but this is not guaranteed.
-        """
-        # TODO: You can make specialized http requests for just
-        # a portion of the file. Figure out how to do that.
-        # For now, urllib2 returns files that cannot seek() so
-        # we just read bytes off the beginning, until we
-        # get to the point that we care about.
-        f = self.get(relpath)
-        # TODO: read in smaller chunks, in case things are
-        # buffered internally.
-        f.read(start)
-        return f
-
-    def put(self, relpath, f):
+            path = self.abspath(relpath)
+            return get_url(path)
+        except urllib2.HTTPError, e:
+            mutter('url error code: %s for has url: %r', e.code, path)
+            if e.code == 404:
+                raise NoSuchFile(path, extra=e)
+            raise
+        except (BzrError, IOError), e:
+            if hasattr(e, 'errno'):
+                mutter('io error: %s %s for has url: %r', 
+                    e.errno, errno.errorcode.get(e.errno), path)
+                if e.errno == errno.ENOENT:
+                    raise NoSuchFile(path, extra=e)
+            raise ConnectionError(msg = "Error retrieving %s: %s" 
+                             % (self.abspath(relpath), str(e)),
+                             orig_error=e)
+
+    def put(self, relpath, f, mode=None):
         """Copy the file-like or string object into the location.
 
         :param relpath: Location to put the contents, relative to base.
@@ -178,7 +207,7 @@
         """
         raise TransportNotPossible('http PUT not supported')
 
-    def mkdir(self, relpath):
+    def mkdir(self, relpath, mode=None):
         """Create a directory at the given path."""
         raise TransportNotPossible('http does not support mkdir()')
 
@@ -192,7 +221,7 @@
         """Copy the item at rel_from to the location at rel_to"""
         raise TransportNotPossible('http does not support copy()')
 
-    def copy_to(self, relpaths, other, pb=None):
+    def copy_to(self, relpaths, other, mode=None, pb=None):
         """Copy a set of entries from self into another Transport.
 
         :param relpaths: A list/generator of entries to be copied.
@@ -206,7 +235,7 @@
         if isinstance(other, HttpTransport):
             raise TransportNotPossible('http cannot be the target of copy_to()')
         else:
-            return super(HttpTransport, self).copy_to(relpaths, other, pb=pb)
+            return super(HttpTransport, self).copy_to(relpaths, other, mode=mode, pb=pb)
 
     def move(self, rel_from, rel_to):
         """Move the item at rel_from to the location at rel_to"""
@@ -216,6 +245,10 @@
         """Delete the item at relpath"""
         raise TransportNotPossible('http does not support delete()')
 
+    def is_readonly(self):
+        """See Transport.is_readonly."""
+        return True
+
     def listable(self):
         """See Transport.listable."""
         return False
@@ -246,5 +279,158 @@
         """
         raise TransportNotPossible('http does not support lock_write()')
 
-register_transport('http://', HttpTransport)
-register_transport('https://', HttpTransport)
+
+#---------------- test server facilities ----------------
+import BaseHTTPServer, SimpleHTTPServer, socket, time
+import threading
+
+
+class WebserverNotAvailable(Exception):
+    pass
+
+
+class BadWebserverPath(ValueError):
+    def __str__(self):
+        return 'path %s is not in %s' % self.args
+
+
+class TestingHTTPRequestHandler(SimpleHTTPServer.SimpleHTTPRequestHandler):
+
+    def log_message(self, format, *args):
+        self.server.test_case.log("webserver - %s - - [%s] %s",
+                                  self.address_string(),
+                                  self.log_date_time_string(),
+                                  format%args)
+
+    def handle_one_request(self):
+        """Handle a single HTTP request.
+
+        You normally don't need to override this method; see the class
+        __doc__ string for information on how to handle specific HTTP
+        commands such as GET and POST.
+
+        """
+        for i in xrange(1,11): # Don't try more than 10 times
+            try:
+                self.raw_requestline = self.rfile.readline()
+            except socket.error, e:
+                if e.args[0] in (errno.EAGAIN, errno.EWOULDBLOCK):
+                    # omitted for now because some tests look at the log of
+                    # the server and expect to see no errors.  see recent
+                    # email thread. -- mbp 20051021. 
+                    ## self.log_message('EAGAIN (%d) while reading from raw_requestline' % i)
+                    time.sleep(0.01)
+                    continue
+                raise
+            else:
+                break
+        if not self.raw_requestline:
+            self.close_connection = 1
+            return
+        if not self.parse_request(): # An error code has been sent, just exit
+            return
+        mname = 'do_' + self.command
+        if not hasattr(self, mname):
+            self.send_error(501, "Unsupported method (%r)" % self.command)
+            return
+        method = getattr(self, mname)
+        method()
+
+class TestingHTTPServer(BaseHTTPServer.HTTPServer):
+    def __init__(self, server_address, RequestHandlerClass, test_case):
+        BaseHTTPServer.HTTPServer.__init__(self, server_address,
+                                                RequestHandlerClass)
+        self.test_case = test_case
+
+
+class HttpServer(Server):
+    """A test server for http transports."""
+
+    _HTTP_PORTS = range(13000, 0x8000)
+
+    def _http_start(self):
+        httpd = None
+        for port in self._HTTP_PORTS:
+            try:
+                httpd = TestingHTTPServer(('localhost', port),
+                                          TestingHTTPRequestHandler,
+                                          self)
+            except socket.error, e:
+                if e.args[0] == errno.EADDRINUSE:
+                    continue
+                print >>sys.stderr, "Cannot run webserver :-("
+                raise
+            else:
+                break
+
+        if httpd is None:
+            raise WebserverNotAvailable("Cannot run webserver :-( "
+                                        "no free ports in range %s..%s" %
+                                        (_HTTP_PORTS[0], _HTTP_PORTS[-1]))
+
+        self._http_base_url = 'http://localhost:%s/' % port
+        self._http_starting.release()
+        httpd.socket.settimeout(0.1)
+
+        while self._http_running:
+            try:
+                httpd.handle_request()
+            except socket.timeout:
+                pass
+
+    def _get_remote_url(self, path):
+        path_parts = path.split(os.path.sep)
+        if os.path.isabs(path):
+            if path_parts[:len(self._local_path_parts)] != \
+                   self._local_path_parts:
+                raise BadWebserverPath(path, self.test_dir)
+            remote_path = '/'.join(path_parts[len(self._local_path_parts):])
+        else:
+            remote_path = '/'.join(path_parts)
+
+        self._http_starting.acquire()
+        self._http_starting.release()
+        return self._http_base_url + remote_path
+
+    def log(self, *args, **kwargs):
+        """Capture Server log output."""
+        self.logs.append(args[3])
+
+    def setUp(self):
+        """See bzrlib.transport.Server.setUp."""
+        self._home_dir = os.getcwdu()
+        self._local_path_parts = self._home_dir.split(os.path.sep)
+        self._http_starting = threading.Lock()
+        self._http_starting.acquire()
+        self._http_running = True
+        self._http_base_url = None
+        self._http_thread = threading.Thread(target=self._http_start)
+        self._http_thread.setDaemon(True)
+        self._http_thread.start()
+        self._http_proxy = os.environ.get("http_proxy")
+        if self._http_proxy is not None:
+            del os.environ["http_proxy"]
+        self.logs = []
+
+    def tearDown(self):
+        """See bzrlib.transport.Server.tearDown."""
+        self._http_running = False
+        self._http_thread.join()
+        if self._http_proxy is not None:
+            import os
+            os.environ["http_proxy"] = self._http_proxy
+
+    def get_url(self):
+        """See bzrlib.transport.Server.get_url."""
+        return self._get_remote_url(self._home_dir)
+        
+    def get_bogus_url(self):
+        """See bzrlib.transport.Server.get_bogus_url."""
+        return 'http://jasldkjsalkdjalksjdkljasd'
+
+
+def get_test_permutations():
+    """Return the permutations to be used in testing."""
+    warn("There are no HTTPS transport provider tests yet.")
+    return [(HttpTransport, HttpServer),
+            ]

=== modified file 'bzrlib/transport/local.py'
--- bzrlib/transport/local.py	2005-10-04 04:37:29 +0000
+++ bzrlib/transport/local.py	2006-01-10 01:10:19 +0000
@@ -13,15 +13,21 @@
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-"""Implementation of Transport for the local filesystem.
-"""
-
-from bzrlib.transport import Transport, register_transport, \
-    TransportError, NoSuchFile, FileExists
-import os, errno
-
-class LocalTransportError(TransportError):
-    pass
+
+"""Transport for the local filesystem.
+
+This is a fairly thin wrapper on regular file IO."""
+
+import os
+import shutil
+from stat import ST_MODE, S_ISDIR, ST_SIZE
+import tempfile
+import urllib
+
+from bzrlib.trace import mutter
+from bzrlib.transport import Transport, Server
+from bzrlib.osutils import abspath, realpath, normpath, pathjoin, rename
+
 
 class LocalTransport(Transport):
     """This is the transport agent for local filesystem access."""
@@ -32,8 +38,10 @@
             base = base[7:]
         # realpath is incompatible with symlinks. When we traverse
         # up we might be able to normpath stuff. RBC 20051003
-        super(LocalTransport, self).__init__(
-            os.path.normpath(os.path.abspath(base)))
+        base = normpath(abspath(base))
+        if base[-1] != '/':
+            base = base + '/'
+        super(LocalTransport, self).__init__(base)
 
     def should_cache(self):
         return False
@@ -49,18 +57,21 @@
             return LocalTransport(self.abspath(offset))
 
     def abspath(self, relpath):
-        """Return the full url to the given relative path.
+        """Return the full url to the given relative URL.
         This can be supplied with a string or a list
         """
-        if isinstance(relpath, basestring):
-            relpath = [relpath]
-        return os.path.join(self.base, *relpath)
+        assert isinstance(relpath, basestring), (type(relpath), relpath)
+        return pathjoin(self.base, urllib.unquote(relpath))
 
     def relpath(self, abspath):
         """Return the local path portion from a given absolute path.
         """
-        from bzrlib.branch import _relpath
-        return _relpath(self.base, abspath)
+        from bzrlib.osutils import relpath
+        if abspath is None:
+            abspath = u'.'
+        if abspath.endswith('/'):
+            abspath = abspath[:-1]
+        return relpath(self.base[:-1], abspath)
 
     def has(self, relpath):
         return os.access(self.abspath(relpath), os.F_OK)
@@ -73,36 +84,10 @@
         try:
             path = self.abspath(relpath)
             return open(path, 'rb')
-        except IOError,e:
-            if e.errno == errno.ENOENT:
-                raise NoSuchFile('File %r does not exist' % path, orig_error=e)
-            raise LocalTransportError(orig_error=e)
-
-    def get_partial(self, relpath, start, length=None):
-        """Get just part of a file.
-
-        :param relpath: Path to the file, relative to base
-        :param start: The starting position to read from
-        :param length: The length to read. A length of None indicates
-                       read to the end of the file.
-        :return: A file-like object containing at least the specified bytes.
-                 Some implementations may return objects which can be read
-                 past this length, but this is not guaranteed.
-        """
-        # LocalTransport.get_partial() doesn't care about the length
-        # argument, because it is using a local file, and thus just
-        # returns the file seek'ed to the appropriate location.
-        try:
-            path = self.abspath(relpath)
-            f = open(path, 'rb')
-            f.seek(start, 0)
-            return f
-        except IOError,e:
-            if e.errno == errno.ENOENT:
-                raise NoSuchFile('File %r does not exist' % path, orig_error=e)
-            raise LocalTransportError(orig_error=e)
-
-    def put(self, relpath, f):
+        except (IOError, OSError),e:
+            self._translate_error(e, path)
+
+    def put(self, relpath, f, mode=None):
         """Copy the file-like or string object into the location.
 
         :param relpath: Location to put the contents, relative to base.
@@ -110,35 +95,49 @@
         """
         from bzrlib.atomicfile import AtomicFile
 
+        path = relpath
         try:
             path = self.abspath(relpath)
-            fp = AtomicFile(path, 'wb')
-        except IOError, e:
-            if e.errno == errno.ENOENT:
-                raise NoSuchFile('File %r does not exist' % path, orig_error=e)
-            raise LocalTransportError(orig_error=e)
+            fp = AtomicFile(path, 'wb', new_mode=mode)
+        except (IOError, OSError),e:
+            self._translate_error(e, path)
         try:
             self._pump(f, fp)
             fp.commit()
         finally:
             fp.close()
 
-    def mkdir(self, relpath):
+    def iter_files_recursive(self):
+        """Iter the relative paths of files in the transports sub-tree."""
+        queue = list(self.list_dir(u'.'))
+        while queue:
+            relpath = urllib.quote(queue.pop(0))
+            st = self.stat(relpath)
+            if S_ISDIR(st[ST_MODE]):
+                for i, basename in enumerate(self.list_dir(relpath)):
+                    queue.insert(i, relpath+'/'+basename)
+            else:
+                yield relpath
+
+    def mkdir(self, relpath, mode=None):
         """Create a directory at the given path."""
+        path = relpath
         try:
-            os.mkdir(self.abspath(relpath))
-        except OSError,e:
-            if e.errno == errno.EEXIST:
-                raise FileExists(orig_error=e)
-            elif e.errno == errno.ENOENT:
-                raise NoSuchFile(orig_error=e)
-            raise LocalTransportError(orig_error=e)
+            path = self.abspath(relpath)
+            os.mkdir(path)
+            if mode is not None:
+                os.chmod(path, mode)
+        except (IOError, OSError),e:
+            self._translate_error(e, path)
 
     def append(self, relpath, f):
         """Append the text in the file-like object into the final
         location.
         """
-        fp = open(self.abspath(relpath), 'ab')
+        try:
+            fp = open(self.abspath(relpath), 'ab')
+        except (IOError, OSError),e:
+            self._translate_error(e, relpath)
         self._pump(f, fp)
 
     def copy(self, rel_from, rel_to):
@@ -148,8 +147,9 @@
         path_to = self.abspath(rel_to)
         try:
             shutil.copy(path_from, path_to)
-        except OSError,e:
-            raise LocalTransportError(orig_error=e)
+        except (IOError, OSError),e:
+            # TODO: What about path_to?
+            self._translate_error(e, path_from)
 
     def move(self, rel_from, rel_to):
         """Move the item at rel_from to the location at rel_to"""
@@ -157,18 +157,22 @@
         path_to = self.abspath(rel_to)
 
         try:
-            os.rename(path_from, path_to)
-        except OSError,e:
-            raise LocalTransportError(orig_error=e)
+            rename(path_from, path_to)
+        except (IOError, OSError),e:
+            # TODO: What about path_to?
+            self._translate_error(e, path_from)
 
     def delete(self, relpath):
         """Delete the item at relpath"""
+        path = relpath
         try:
-            os.remove(self.abspath(relpath))
-        except OSError,e:
-            raise LocalTransportError(orig_error=e)
+            path = self.abspath(relpath)
+            os.remove(path)
+        except (IOError, OSError),e:
+            # TODO: What about path_to?
+            self._translate_error(e, path)
 
-    def copy_to(self, relpaths, other, pb=None):
+    def copy_to(self, relpaths, other, mode=None, pb=None):
         """Copy a set of entries from self into another Transport.
 
         :param relpaths: A list/generator of entries to be copied.
@@ -183,11 +187,18 @@
             count = 0
             for path in relpaths:
                 self._update_pb(pb, 'copy-to', count, total)
-                shutil.copy(self.abspath(path), other.abspath(path))
+                try:
+                    mypath = self.abspath(path)
+                    otherpath = other.abspath(path)
+                    shutil.copy(mypath, otherpath)
+                    if mode is not None:
+                        os.chmod(otherpath, mode)
+                except (IOError, OSError),e:
+                    self._translate_error(e, path)
                 count += 1
             return count
         else:
-            return super(LocalTransport, self).copy_to(relpaths, other, pb=pb)
+            return super(LocalTransport, self).copy_to(relpaths, other, mode=mode, pb=pb)
 
     def listable(self):
         """See Transport.listable."""
@@ -198,18 +209,22 @@
         WARNING: many transports do not support this, so trying avoid using
         it if at all possible.
         """
+        path = relpath
         try:
-            return os.listdir(self.abspath(relpath))
-        except OSError,e:
-            raise LocalTransportError(orig_error=e)
+            path = self.abspath(relpath)
+            return os.listdir(path)
+        except (IOError, OSError),e:
+            self._translate_error(e, path)
 
     def stat(self, relpath):
         """Return the stat information for a file.
         """
+        path = relpath
         try:
-            return os.stat(self.abspath(relpath))
-        except OSError,e:
-            raise LocalTransportError(orig_error=e)
+            path = self.abspath(relpath)
+            return os.stat(path)
+        except (IOError, OSError),e:
+            self._translate_error(e, path)
 
     def lock_read(self, relpath):
         """Lock the given file for shared (read) access.
@@ -227,6 +242,52 @@
         from bzrlib.lock import WriteLock
         return WriteLock(self.abspath(relpath))
 
-# If nothing else matches, try the LocalTransport
-register_transport(None, LocalTransport)
-register_transport('file://', LocalTransport)
+
+class ScratchTransport(LocalTransport):
+    """A transport that works in a temporary dir and cleans up after itself.
+    
+    The dir only exists for the lifetime of the Python object.
+    Obviously you should not put anything precious in it.
+    """
+
+    def __init__(self, base=None):
+        if base is None:
+            base = tempfile.mkdtemp()
+        super(ScratchTransport, self).__init__(base)
+
+    def __del__(self):
+        shutil.rmtree(self.base, ignore_errors=True)
+        mutter("%r destroyed" % self)
+
+
+class LocalRelpathServer(Server):
+    """A pretend server for local transports, using relpaths."""
+
+    def get_url(self):
+        """See Transport.Server.get_url."""
+        return "."
+
+
+class LocalAbspathServer(Server):
+    """A pretend server for local transports, using absolute paths."""
+
+    def get_url(self):
+        """See Transport.Server.get_url."""
+        return os.path.abspath("")
+
+
+class LocalURLServer(Server):
+    """A pretend server for local transports, using file:// urls."""
+
+    def get_url(self):
+        """See Transport.Server.get_url."""
+        # FIXME: \ to / on windows
+        return "file://%s" % os.path.abspath("")
+
+
+def get_test_permutations():
+    """Return the permutations to be used in testing."""
+    return [(LocalTransport, LocalRelpathServer),
+            (LocalTransport, LocalAbspathServer),
+            (LocalTransport, LocalURLServer),
+            ]

=== added file 'bzrlib/transport/memory.py'
--- bzrlib/transport/memory.py	1970-01-01 00:00:00 +0000
+++ bzrlib/transport/memory.py	2006-01-10 05:25:43 +0000
@@ -0,0 +1,202 @@
+# Copyright (C) 2005 Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+"""Implementation of Transport that uses memory for its storage."""
+
+from copy import copy
+import os
+import errno
+from stat import *
+from cStringIO import StringIO
+
+from bzrlib.trace import mutter
+from bzrlib.errors import TransportError, NoSuchFile, FileExists
+from bzrlib.transport import Transport, register_transport, Server
+
+class MemoryStat(object):
+
+    def __init__(self, size, is_dir, perms):
+        self.st_size = size
+        if perms is None:
+            perms = 0644
+        if not is_dir:
+            self.st_mode = S_IFREG | perms
+        else:
+            self.st_mode = S_IFDIR | perms
+
+
+class MemoryTransport(Transport):
+    """This is the transport agent for local filesystem access."""
+
+    def __init__(self, url=""):
+        """Set the 'base' path where files will be stored."""
+        if url == "":
+            url = "memory:/"
+        if url[-1] != '/':
+            url = url + '/'
+        super(MemoryTransport, self).__init__(url)
+        self._cwd = url[url.find(':') + 1:]
+        self._dirs = {}
+        self._files = {}
+
+    def clone(self, offset=None):
+        """See Transport.clone()."""
+        if offset is None:
+            return copy(self)
+        segments = offset.split('/')
+        cwdsegments = self._cwd.split('/')[:-1]
+        while len(segments):
+            segment = segments.pop(0)
+            if segment == '.':
+                continue
+            if segment == '..':
+                if len(cwdsegments) > 1:
+                    cwdsegments.pop()
+                continue
+            cwdsegments.append(segment)
+        url = self.base[:self.base.find(':') + 1] + '/'.join(cwdsegments) + '/'
+        result = MemoryTransport(url)
+        result._dirs = self._dirs
+        result._files = self._files
+        return result
+
+    def abspath(self, relpath):
+        """See Transport.abspath()."""
+        return self.base[:-1] + self._abspath(relpath)
+
+    def append(self, relpath, f):
+        """See Transport.append()."""
+        _abspath = self._abspath(relpath)
+        self._check_parent(_abspath)
+        orig_content, orig_mode = self._files.get(_abspath, ("", None))
+        self._files[_abspath] = (orig_content + f.read(), orig_mode)
+
+    def _check_parent(self, _abspath):
+        dir = os.path.dirname(_abspath)
+        if dir != '/':
+            if not dir in self._dirs:
+                raise NoSuchFile(_abspath)
+
+    def has(self, relpath):
+        """See Transport.has()."""
+        _abspath = self._abspath(relpath)
+        return _abspath in self._files or _abspath in self._dirs
+
+    def delete(self, relpath):
+        """See Transport.delete()."""
+        _abspath = self._abspath(relpath)
+        if not _abspath in self._files:
+            raise NoSuchFile(relpath)
+        del self._files[_abspath]
+
+    def get(self, relpath):
+        """See Transport.get()."""
+        _abspath = self._abspath(relpath)
+        if not _abspath in self._files:
+            raise NoSuchFile(relpath)
+        return StringIO(self._files[_abspath][0])
+
+    def put(self, relpath, f, mode=None):
+        """See Transport.put()."""
+        _abspath = self._abspath(relpath)
+        self._check_parent(_abspath)
+        self._files[_abspath] = (f.read(), mode)
+
+    def mkdir(self, relpath, mode=None):
+        """See Transport.mkdir()."""
+        _abspath = self._abspath(relpath)
+        self._check_parent(_abspath)
+        if _abspath in self._dirs:
+            raise FileExists(relpath)
+        self._dirs[_abspath]=mode
+
+    def listable(self):
+        """See Transport.listable."""
+        return True
+
+    def iter_files_recursive(self):
+        for file in self._files:
+            if file.startswith(self._cwd):
+                yield file[len(self._cwd):]
+    
+    def list_dir(self, relpath):
+        """See Transport.list_dir()."""
+        _abspath = self._abspath(relpath)
+        if _abspath != '/' and _abspath not in self._dirs:
+            raise NoSuchFile(relpath)
+        result = []
+        for path in self._files:
+            if (path.startswith(_abspath) and 
+                path[len(_abspath) + 1:].find('/') == -1 and
+                len(path) > len(_abspath)):
+                result.append(path[len(_abspath) + 1:])
+        for path in self._dirs:
+            if (path.startswith(_abspath) and 
+                path[len(_abspath) + 1:].find('/') == -1 and
+                len(path) > len(_abspath)):
+                result.append(path[len(_abspath) + 1:])
+        return result
+    
+    def stat(self, relpath):
+        """See Transport.stat()."""
+        _abspath = self._abspath(relpath)
+        if _abspath in self._files:
+            return MemoryStat(len(self._files[_abspath][0]), False, 
+                              self._files[_abspath][1])
+        elif _abspath == '':
+            return MemoryStat(0, True, None)
+        elif _abspath in self._dirs:
+            return MemoryStat(0, True, self._dirs[_abspath])
+        else:
+            raise NoSuchFile(relpath)
+
+#    def lock_read(self, relpath):
+#   TODO if needed
+#
+#    def lock_write(self, relpath):
+#   TODO if needed
+
+    def _abspath(self, relpath):
+        """Generate an internal absolute path."""
+        if relpath.find('..') != -1:
+            raise AssertionError('relpath contains ..')
+        if relpath == '.':
+            return self._cwd[:-1]
+        if relpath.endswith('/'):
+            relpath = relpath[:-1]
+        return self._cwd + relpath
+
+
+class MemoryServer(Server):
+    """Server for the MemoryTransport for testing with."""
+
+    def setUp(self):
+        """See bzrlib.transport.Server.setUp."""
+        self._scheme = "memory+%s:" % id(self)
+        register_transport(self._scheme, MemoryTransport)
+
+    def tearDown(self):
+        """See bzrlib.transport.Server.tearDown."""
+        # unregister this server
+
+    def get_url(self):
+        """See bzrlib.transport.Server.get_url."""
+        return self._scheme
+
+
+def get_test_permutations():
+    """Return the permutations to be used in testing."""
+    return [(MemoryTransport, MemoryServer),
+            ]

=== added file 'bzrlib/transport/sftp.py'
--- bzrlib/transport/sftp.py	1970-01-01 00:00:00 +0000
+++ bzrlib/transport/sftp.py	2006-01-10 01:10:19 +0000
@@ -0,0 +1,970 @@
+# Copyright (C) 2005 Robey Pointer <robey@lag.net>, Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Implementation of Transport over SFTP, using paramiko."""
+
+import errno
+import getpass
+import os
+import re
+import stat
+import sys
+import urllib
+import urlparse
+import time
+import random
+import subprocess
+import weakref
+
+from bzrlib.config import config_dir, ensure_config_dir_exists
+from bzrlib.errors import (ConnectionError,
+                           FileExists, 
+                           TransportNotPossible, NoSuchFile, PathNotChild,
+                           TransportError,
+                           LockError
+                           )
+from bzrlib.osutils import pathjoin, fancy_rename
+from bzrlib.trace import mutter, warning, error
+from bzrlib.transport import Transport, Server, urlescape
+import bzrlib.ui
+
+try:
+    import paramiko
+except ImportError:
+    error('The SFTP transport requires paramiko.')
+    raise
+else:
+    from paramiko.sftp import (SFTP_FLAG_WRITE, SFTP_FLAG_CREATE,
+                               SFTP_FLAG_EXCL, SFTP_FLAG_TRUNC,
+                               CMD_HANDLE, CMD_OPEN)
+    from paramiko.sftp_attr import SFTPAttributes
+    from paramiko.sftp_file import SFTPFile
+    from paramiko.sftp_client import SFTPClient
+
+if 'sftp' not in urlparse.uses_netloc: urlparse.uses_netloc.append('sftp')
+
+
+_close_fds = True
+if sys.platform == 'win32':
+    # close_fds not supported on win32
+    _close_fds = False
+
+_ssh_vendor = None
+def _get_ssh_vendor():
+    """Find out what version of SSH is on the system."""
+    global _ssh_vendor
+    if _ssh_vendor is not None:
+        return _ssh_vendor
+
+    _ssh_vendor = 'none'
+
+    try:
+        p = subprocess.Popen(['ssh', '-V'],
+                             close_fds=_close_fds,
+                             stdin=subprocess.PIPE,
+                             stdout=subprocess.PIPE,
+                             stderr=subprocess.PIPE)
+        returncode = p.returncode
+        stdout, stderr = p.communicate()
+    except OSError:
+        returncode = -1
+        stdout = stderr = ''
+    if 'OpenSSH' in stderr:
+        mutter('ssh implementation is OpenSSH')
+        _ssh_vendor = 'openssh'
+    elif 'SSH Secure Shell' in stderr:
+        mutter('ssh implementation is SSH Corp.')
+        _ssh_vendor = 'ssh'
+
+    if _ssh_vendor != 'none':
+        return _ssh_vendor
+
+    # XXX: 20051123 jamesh
+    # A check for putty's plink or lsh would go here.
+
+    mutter('falling back to paramiko implementation')
+    return _ssh_vendor
+
+
+class SFTPSubprocess:
+    """A socket-like object that talks to an ssh subprocess via pipes."""
+    def __init__(self, hostname, vendor, port=None, user=None):
+        assert vendor in ['openssh', 'ssh']
+        if vendor == 'openssh':
+            args = ['ssh',
+                    '-oForwardX11=no', '-oForwardAgent=no',
+                    '-oClearAllForwardings=yes', '-oProtocol=2',
+                    '-oNoHostAuthenticationForLocalhost=yes']
+            if port is not None:
+                args.extend(['-p', str(port)])
+            if user is not None:
+                args.extend(['-l', user])
+            args.extend(['-s', hostname, 'sftp'])
+        elif vendor == 'ssh':
+            args = ['ssh', '-x']
+            if port is not None:
+                args.extend(['-p', str(port)])
+            if user is not None:
+                args.extend(['-l', user])
+            args.extend(['-s', 'sftp', hostname])
+
+        self.proc = subprocess.Popen(args, close_fds=_close_fds,
+                                     stdin=subprocess.PIPE,
+                                     stdout=subprocess.PIPE)
+
+    def send(self, data):
+        return os.write(self.proc.stdin.fileno(), data)
+
+    def recv_ready(self):
+        # TODO: jam 20051215 this function is necessary to support the
+        # pipelined() function. In reality, it probably should use
+        # poll() or select() to actually return if there is data
+        # available, otherwise we probably don't get any benefit
+        return True
+
+    def recv(self, count):
+        return os.read(self.proc.stdout.fileno(), count)
+
+    def close(self):
+        self.proc.stdin.close()
+        self.proc.stdout.close()
+        self.proc.wait()
+
+
+SYSTEM_HOSTKEYS = {}
+BZR_HOSTKEYS = {}
+
+# This is a weakref dictionary, so that we can reuse connections
+# that are still active. Long term, it might be nice to have some
+# sort of expiration policy, such as disconnect if inactive for
+# X seconds. But that requires a lot more fanciness.
+_connected_hosts = weakref.WeakValueDictionary()
+
+def load_host_keys():
+    """
+    Load system host keys (probably doesn't work on windows) and any
+    "discovered" keys from previous sessions.
+    """
+    global SYSTEM_HOSTKEYS, BZR_HOSTKEYS
+    try:
+        SYSTEM_HOSTKEYS = paramiko.util.load_host_keys(os.path.expanduser('~/.ssh/known_hosts'))
+    except Exception, e:
+        mutter('failed to load system host keys: ' + str(e))
+    bzr_hostkey_path = pathjoin(config_dir(), 'ssh_host_keys')
+    try:
+        BZR_HOSTKEYS = paramiko.util.load_host_keys(bzr_hostkey_path)
+    except Exception, e:
+        mutter('failed to load bzr host keys: ' + str(e))
+        save_host_keys()
+
+def save_host_keys():
+    """
+    Save "discovered" host keys in $(config)/ssh_host_keys/.
+    """
+    global SYSTEM_HOSTKEYS, BZR_HOSTKEYS
+    bzr_hostkey_path = pathjoin(config_dir(), 'ssh_host_keys')
+    ensure_config_dir_exists()
+
+    try:
+        f = open(bzr_hostkey_path, 'w')
+        f.write('# SSH host keys collected by bzr\n')
+        for hostname, keys in BZR_HOSTKEYS.iteritems():
+            for keytype, key in keys.iteritems():
+                f.write('%s %s %s\n' % (hostname, keytype, key.get_base64()))
+        f.close()
+    except IOError, e:
+        mutter('failed to save bzr host keys: ' + str(e))
+
+
+class SFTPLock(object):
+    """This fakes a lock in a remote location."""
+    __slots__ = ['path', 'lock_path', 'lock_file', 'transport']
+    def __init__(self, path, transport):
+        assert isinstance(transport, SFTPTransport)
+
+        self.lock_file = None
+        self.path = path
+        self.lock_path = path + '.write-lock'
+        self.transport = transport
+        try:
+            # RBC 20060103 FIXME should we be using private methods here ?
+            abspath = transport._remote_path(self.lock_path)
+            self.lock_file = transport._sftp_open_exclusive(abspath)
+        except FileExists:
+            raise LockError('File %r already locked' % (self.path,))
+
+    def __del__(self):
+        """Should this warn, or actually try to cleanup?"""
+        if self.lock_file:
+            warn("SFTPLock %r not explicitly unlocked" % (self.path,))
+            self.unlock()
+
+    def unlock(self):
+        if not self.lock_file:
+            return
+        self.lock_file.close()
+        self.lock_file = None
+        try:
+            self.transport.delete(self.lock_path)
+        except (NoSuchFile,):
+            # What specific errors should we catch here?
+            pass
+
+
+
+class SFTPTransport (Transport):
+    """
+    Transport implementation for SFTP access.
+    """
+    _do_prefetch = False # Right now Paramiko's prefetch support causes things to hang
+
+    def __init__(self, base, clone_from=None):
+        assert base.startswith('sftp://')
+        self._parse_url(base)
+        base = self._unparse_url()
+        if base[-1] != '/':
+            base = base + '/'
+        super(SFTPTransport, self).__init__(base)
+        if clone_from is None:
+            self._sftp_connect()
+        else:
+            # use the same ssh connection, etc
+            self._sftp = clone_from._sftp
+        # super saves 'self.base'
+    
+    def should_cache(self):
+        """
+        Return True if the data pulled across should be cached locally.
+        """
+        return True
+
+    def clone(self, offset=None):
+        """
+        Return a new SFTPTransport with root at self.base + offset.
+        We share the same SFTP session between such transports, because it's
+        fairly expensive to set them up.
+        """
+        if offset is None:
+            return SFTPTransport(self.base, self)
+        else:
+            return SFTPTransport(self.abspath(offset), self)
+
+    def abspath(self, relpath):
+        """
+        Return the full url to the given relative path.
+        
+        @param relpath: the relative path or path components
+        @type relpath: str or list
+        """
+        return self._unparse_url(self._remote_path(relpath))
+    
+    def _remote_path(self, relpath):
+        """Return the path to be passed along the sftp protocol for relpath.
+        
+        relpath is a urlencoded string.
+        """
+        # FIXME: share the common code across transports
+        assert isinstance(relpath, basestring)
+        relpath = urllib.unquote(relpath).split('/')
+        basepath = self._path.split('/')
+        if len(basepath) > 0 and basepath[-1] == '':
+            basepath = basepath[:-1]
+
+        for p in relpath:
+            if p == '..':
+                if len(basepath) == 0:
+                    # In most filesystems, a request for the parent
+                    # of root, just returns root.
+                    continue
+                basepath.pop()
+            elif p == '.':
+                continue # No-op
+            else:
+                basepath.append(p)
+
+        path = '/'.join(basepath)
+        return path
+
+    def relpath(self, abspath):
+        username, password, host, port, path = self._split_url(abspath)
+        error = []
+        if (username != self._username):
+            error.append('username mismatch')
+        if (host != self._host):
+            error.append('host mismatch')
+        if (port != self._port):
+            error.append('port mismatch')
+        if (not path.startswith(self._path)):
+            error.append('path mismatch')
+        if error:
+            extra = ': ' + ', '.join(error)
+            raise PathNotChild(abspath, self.base, extra=extra)
+        pl = len(self._path)
+        return path[pl:].strip('/')
+
+    def has(self, relpath):
+        """
+        Does the target location exist?
+        """
+        try:
+            self._sftp.stat(self._remote_path(relpath))
+            return True
+        except IOError:
+            return False
+
+    def get(self, relpath, decode=False):
+        """
+        Get the file at the given relative path.
+
+        :param relpath: The relative path to the file
+        """
+        try:
+            path = self._remote_path(relpath)
+            f = self._sftp.file(path, mode='rb')
+            if self._do_prefetch and hasattr(f, 'prefetch'):
+                f.prefetch()
+            return f
+        except (IOError, paramiko.SSHException), e:
+            self._translate_io_exception(e, path, ': error retrieving')
+
+    def get_partial(self, relpath, start, length=None):
+        """
+        Get just part of a file.
+
+        :param relpath: Path to the file, relative to base
+        :param start: The starting position to read from
+        :param length: The length to read. A length of None indicates
+                       read to the end of the file.
+        :return: A file-like object containing at least the specified bytes.
+                 Some implementations may return objects which can be read
+                 past this length, but this is not guaranteed.
+        """
+        # TODO: implement get_partial_multi to help with knit support
+        f = self.get(relpath)
+        f.seek(start)
+        if self._do_prefetch and hasattr(f, 'prefetch'):
+            f.prefetch()
+        return f
+
+    def put(self, relpath, f, mode=None):
+        """
+        Copy the file-like or string object into the location.
+
+        :param relpath: Location to put the contents, relative to base.
+        :param f:       File-like or string object.
+        :param mode: The final mode for the file
+        """
+        final_path = self._remote_path(relpath)
+        self._put(final_path, f, mode=mode)
+
+    def _put(self, abspath, f, mode=None):
+        """Helper function so both put() and copy_abspaths can reuse the code"""
+        tmp_abspath = '%s.tmp.%.9f.%d.%d' % (abspath, time.time(),
+                        os.getpid(), random.randint(0,0x7FFFFFFF))
+        fout = self._sftp_open_exclusive(tmp_abspath, mode=mode)
+        closed = False
+        try:
+            try:
+                fout.set_pipelined(True)
+                self._pump(f, fout)
+            except (IOError, paramiko.SSHException), e:
+                self._translate_io_exception(e, tmp_abspath)
+            if mode is not None:
+                self._sftp.chmod(tmp_abspath, mode)
+            fout.close()
+            closed = True
+            self._rename(tmp_abspath, abspath)
+        except Exception, e:
+            # If we fail, try to clean up the temporary file
+            # before we throw the exception
+            # but don't let another exception mess things up
+            # Write out the traceback, because otherwise
+            # the catch and throw destroys it
+            import traceback
+            mutter(traceback.format_exc())
+            try:
+                if not closed:
+                    fout.close()
+                self._sftp.remove(tmp_abspath)
+            except:
+                # raise the saved except
+                raise e
+            # raise the original with its traceback if we can.
+            raise
+
+    def iter_files_recursive(self):
+        """Walk the relative paths of all files in this transport."""
+        queue = list(self.list_dir('.'))
+        while queue:
+            relpath = urllib.quote(queue.pop(0))
+            st = self.stat(relpath)
+            if stat.S_ISDIR(st.st_mode):
+                for i, basename in enumerate(self.list_dir(relpath)):
+                    queue.insert(i, relpath+'/'+basename)
+            else:
+                yield relpath
+
+    def mkdir(self, relpath, mode=None):
+        """Create a directory at the given path."""
+        try:
+            path = self._remote_path(relpath)
+            # In the paramiko documentation, it says that passing a mode flag 
+            # will filtered against the server umask.
+            # StubSFTPServer does not do this, which would be nice, because it is
+            # what we really want :)
+            # However, real servers do use umask, so we really should do it that way
+            self._sftp.mkdir(path)
+            if mode is not None:
+                self._sftp.chmod(path, mode=mode)
+        except (paramiko.SSHException, IOError), e:
+            self._translate_io_exception(e, path, ': unable to mkdir',
+                failure_exc=FileExists)
+
+    def _translate_io_exception(self, e, path, more_info='', failure_exc=NoSuchFile):
+        """Translate a paramiko or IOError into a friendlier exception.
+
+        :param e: The original exception
+        :param path: The path in question when the error is raised
+        :param more_info: Extra information that can be included,
+                          such as what was going on
+        :param failure_exc: Paramiko has the super fun ability to raise completely
+                           opaque errors that just set "e.args = ('Failure',)" with
+                           no more information.
+                           This sometimes means FileExists, but it also sometimes
+                           means NoSuchFile
+        """
+        # paramiko seems to generate detailless errors.
+        self._translate_error(e, path, raise_generic=False)
+        if hasattr(e, 'args'):
+            if (e.args == ('No such file or directory',) or
+                e.args == ('No such file',)):
+                raise NoSuchFile(path, str(e) + more_info)
+            if (e.args == ('mkdir failed',)):
+                raise FileExists(path, str(e) + more_info)
+            # strange but true, for the paramiko server.
+            if (e.args == ('Failure',)):
+                raise failure_exc(path, str(e) + more_info)
+            mutter('Raising exception with args %s', e.args)
+        if hasattr(e, 'errno'):
+            mutter('Raising exception with errno %s', e.errno)
+        raise e
+
+    def append(self, relpath, f):
+        """
+        Append the text in the file-like object into the final
+        location.
+        """
+        try:
+            path = self._remote_path(relpath)
+            fout = self._sftp.file(path, 'ab')
+            self._pump(f, fout)
+        except (IOError, paramiko.SSHException), e:
+            self._translate_io_exception(e, relpath, ': unable to append')
+
+    def copy(self, rel_from, rel_to):
+        """Copy the item at rel_from to the location at rel_to"""
+        path_from = self._remote_path(rel_from)
+        path_to = self._remote_path(rel_to)
+        self._copy_abspaths(path_from, path_to)
+
+    def _copy_abspaths(self, path_from, path_to, mode=None):
+        """Copy files given an absolute path
+
+        :param path_from: Path on remote server to read
+        :param path_to: Path on remote server to write
+        :return: None
+
+        TODO: Should the destination location be atomically created?
+              This has not been specified
+        TODO: This should use some sort of remote copy, rather than
+              pulling the data locally, and then writing it remotely
+        """
+        try:
+            fin = self._sftp.file(path_from, 'rb')
+            try:
+                self._put(path_to, fin, mode=mode)
+            finally:
+                fin.close()
+        except (IOError, paramiko.SSHException), e:
+            self._translate_io_exception(e, path_from, ': unable copy to: %r' % path_to)
+
+    def copy_to(self, relpaths, other, mode=None, pb=None):
+        """Copy a set of entries from self into another Transport.
+
+        :param relpaths: A list/generator of entries to be copied.
+        """
+        if isinstance(other, SFTPTransport) and other._sftp is self._sftp:
+            # Both from & to are on the same remote filesystem
+            # We can use a remote copy, instead of pulling locally, and pushing
+
+            total = self._get_total(relpaths)
+            count = 0
+            for path in relpaths:
+                path_from = self._remote_path(relpath)
+                path_to = other._remote_path(relpath)
+                self._update_pb(pb, 'copy-to', count, total)
+                self._copy_abspaths(path_from, path_to, mode=mode)
+                count += 1
+            return count
+        else:
+            return super(SFTPTransport, self).copy_to(relpaths, other, mode=mode, pb=pb)
+
+    def _rename(self, abs_from, abs_to):
+        """Do a fancy rename on the remote server.
+        
+        Using the implementation provided by osutils.
+        """
+        try:
+            fancy_rename(abs_from, abs_to,
+                    rename_func=self._sftp.rename,
+                    unlink_func=self._sftp.remove)
+        except (IOError, paramiko.SSHException), e:
+            self._translate_io_exception(e, abs_from, ': unable to rename to %r' % (abs_to))
+
+    def move(self, rel_from, rel_to):
+        """Move the item at rel_from to the location at rel_to"""
+        path_from = self._remote_path(rel_from)
+        path_to = self._remote_path(rel_to)
+        self._rename(path_from, path_to)
+
+    def delete(self, relpath):
+        """Delete the item at relpath"""
+        path = self._remote_path(relpath)
+        try:
+            self._sftp.remove(path)
+        except (IOError, paramiko.SSHException), e:
+            self._translate_io_exception(e, path, ': unable to delete')
+            
+    def listable(self):
+        """Return True if this store supports listing."""
+        return True
+
+    def list_dir(self, relpath):
+        """
+        Return a list of all files at the given location.
+        """
+        # does anything actually use this?
+        path = self._remote_path(relpath)
+        try:
+            return self._sftp.listdir(path)
+        except (IOError, paramiko.SSHException), e:
+            self._translate_io_exception(e, path, ': failed to list_dir')
+
+    def stat(self, relpath):
+        """Return the stat information for a file."""
+        path = self._remote_path(relpath)
+        try:
+            return self._sftp.stat(path)
+        except (IOError, paramiko.SSHException), e:
+            self._translate_io_exception(e, path, ': unable to stat')
+
+    def lock_read(self, relpath):
+        """
+        Lock the given file for shared (read) access.
+        :return: A lock object, which has an unlock() member function
+        """
+        # FIXME: there should be something clever i can do here...
+        class BogusLock(object):
+            def __init__(self, path):
+                self.path = path
+            def unlock(self):
+                pass
+        return BogusLock(relpath)
+
+    def lock_write(self, relpath):
+        """
+        Lock the given file for exclusive (write) access.
+        WARNING: many transports do not support this, so trying avoid using it
+
+        :return: A lock object, which has an unlock() member function
+        """
+        # This is a little bit bogus, but basically, we create a file
+        # which should not already exist, and if it does, we assume
+        # that there is a lock, and if it doesn't, the we assume
+        # that we have taken the lock.
+        return SFTPLock(relpath, self)
+
+
+    def _unparse_url(self, path=None):
+        if path is None:
+            path = self._path
+        path = urllib.quote(path)
+        if path.startswith('/'):
+            path = '/%2F' + path[1:]
+        else:
+            path = '/' + path
+        netloc = urllib.quote(self._host)
+        if self._username is not None:
+            netloc = '%s@%s' % (urllib.quote(self._username), netloc)
+        if self._port is not None:
+            netloc = '%s:%d' % (netloc, self._port)
+
+        return urlparse.urlunparse(('sftp', netloc, path, '', '', ''))
+
+    def _split_url(self, url):
+        if isinstance(url, unicode):
+            url = url.encode('utf-8')
+        (scheme, netloc, path, params,
+         query, fragment) = urlparse.urlparse(url, allow_fragments=False)
+        assert scheme == 'sftp'
+        username = password = host = port = None
+        if '@' in netloc:
+            username, host = netloc.split('@', 1)
+            if ':' in username:
+                username, password = username.split(':', 1)
+                password = urllib.unquote(password)
+            username = urllib.unquote(username)
+        else:
+            host = netloc
+
+        if ':' in host:
+            host, port = host.rsplit(':', 1)
+            try:
+                port = int(port)
+            except ValueError:
+                # TODO: Should this be ConnectionError?
+                raise TransportError('%s: invalid port number' % port)
+        host = urllib.unquote(host)
+
+        path = urllib.unquote(path)
+
+        # the initial slash should be removed from the path, and treated
+        # as a homedir relative path (the path begins with a double slash
+        # if it is absolute).
+        # see draft-ietf-secsh-scp-sftp-ssh-uri-03.txt
+        if path.startswith('/'):
+            path = path[1:]
+
+        return (username, password, host, port, path)
+
+    def _parse_url(self, url):
+        (self._username, self._password,
+         self._host, self._port, self._path) = self._split_url(url)
+
+    def _sftp_connect(self):
+        """Connect to the remote sftp server.
+        After this, self._sftp should have a valid connection (or
+        we raise an TransportError 'could not connect').
+
+        TODO: Raise a more reasonable ConnectionFailed exception
+        """
+        global _connected_hosts
+
+        idx = (self._host, self._port, self._username)
+        try:
+            self._sftp = _connected_hosts[idx]
+            return
+        except KeyError:
+            pass
+        
+        vendor = _get_ssh_vendor()
+        if vendor != 'none':
+            sock = SFTPSubprocess(self._host, vendor, self._port,
+                                  self._username)
+            self._sftp = SFTPClient(sock)
+        else:
+            self._paramiko_connect()
+
+        _connected_hosts[idx] = self._sftp
+
+    def _paramiko_connect(self):
+        global SYSTEM_HOSTKEYS, BZR_HOSTKEYS
+        
+        load_host_keys()
+
+        try:
+            t = paramiko.Transport((self._host, self._port or 22))
+            t.start_client()
+        except paramiko.SSHException, e:
+            raise ConnectionError('Unable to reach SSH host %s:%d' %
+                                  (self._host, self._port), e)
+            
+        server_key = t.get_remote_server_key()
+        server_key_hex = paramiko.util.hexify(server_key.get_fingerprint())
+        keytype = server_key.get_name()
+        if SYSTEM_HOSTKEYS.has_key(self._host) and SYSTEM_HOSTKEYS[self._host].has_key(keytype):
+            our_server_key = SYSTEM_HOSTKEYS[self._host][keytype]
+            our_server_key_hex = paramiko.util.hexify(our_server_key.get_fingerprint())
+        elif BZR_HOSTKEYS.has_key(self._host) and BZR_HOSTKEYS[self._host].has_key(keytype):
+            our_server_key = BZR_HOSTKEYS[self._host][keytype]
+            our_server_key_hex = paramiko.util.hexify(our_server_key.get_fingerprint())
+        else:
+            warning('Adding %s host key for %s: %s' % (keytype, self._host, server_key_hex))
+            if not BZR_HOSTKEYS.has_key(self._host):
+                BZR_HOSTKEYS[self._host] = {}
+            BZR_HOSTKEYS[self._host][keytype] = server_key
+            our_server_key = server_key
+            our_server_key_hex = paramiko.util.hexify(our_server_key.get_fingerprint())
+            save_host_keys()
+        if server_key != our_server_key:
+            filename1 = os.path.expanduser('~/.ssh/known_hosts')
+            filename2 = pathjoin(config_dir(), 'ssh_host_keys')
+            raise TransportError('Host keys for %s do not match!  %s != %s' % \
+                (self._host, our_server_key_hex, server_key_hex),
+                ['Try editing %s or %s' % (filename1, filename2)])
+
+        self._sftp_auth(t)
+        
+        try:
+            self._sftp = t.open_sftp_client()
+        except paramiko.SSHException, e:
+            raise ConnectionError('Unable to start sftp client %s:%d' %
+                                  (self._host, self._port), e)
+
+    def _sftp_auth(self, transport):
+        # paramiko requires a username, but it might be none if nothing was supplied
+        # use the local username, just in case.
+        # We don't override self._username, because if we aren't using paramiko,
+        # the username might be specified in ~/.ssh/config and we don't want to
+        # force it to something else
+        # Also, it would mess up the self.relpath() functionality
+        username = self._username or getpass.getuser()
+
+        # Paramiko tries to open a socket.AF_UNIX in order to connect
+        # to ssh-agent. That attribute doesn't exist on win32 (it does in cygwin)
+        # so we get an AttributeError exception. For now, just don't try to
+        # connect to an agent if we are on win32
+        if sys.platform != 'win32':
+            agent = paramiko.Agent()
+            for key in agent.get_keys():
+                mutter('Trying SSH agent key %s' % paramiko.util.hexify(key.get_fingerprint()))
+                try:
+                    transport.auth_publickey(username, key)
+                    return
+                except paramiko.SSHException, e:
+                    pass
+        
+        # okay, try finding id_rsa or id_dss?  (posix only)
+        if self._try_pkey_auth(transport, paramiko.RSAKey, username, 'id_rsa'):
+            return
+        if self._try_pkey_auth(transport, paramiko.DSSKey, username, 'id_dsa'):
+            return
+
+        if self._password:
+            try:
+                transport.auth_password(username, self._password)
+                return
+            except paramiko.SSHException, e:
+                pass
+
+            # FIXME: Don't keep a password held in memory if you can help it
+            #self._password = None
+
+        # give up and ask for a password
+        password = bzrlib.ui.ui_factory.get_password(
+                prompt='SSH %(user)s@%(host)s password',
+                user=username, host=self._host)
+        try:
+            transport.auth_password(username, password)
+        except paramiko.SSHException, e:
+            raise ConnectionError('Unable to authenticate to SSH host as %s@%s' %
+                                  (username, self._host), e)
+
+    def _try_pkey_auth(self, transport, pkey_class, username, filename):
+        filename = os.path.expanduser('~/.ssh/' + filename)
+        try:
+            key = pkey_class.from_private_key_file(filename)
+            transport.auth_publickey(username, key)
+            return True
+        except paramiko.PasswordRequiredException:
+            password = bzrlib.ui.ui_factory.get_password(
+                    prompt='SSH %(filename)s password',
+                    filename=filename)
+            try:
+                key = pkey_class.from_private_key_file(filename, password)
+                transport.auth_publickey(username, key)
+                return True
+            except paramiko.SSHException:
+                mutter('SSH authentication via %s key failed.' % (os.path.basename(filename),))
+        except paramiko.SSHException:
+            mutter('SSH authentication via %s key failed.' % (os.path.basename(filename),))
+        except IOError:
+            pass
+        return False
+
+    def _sftp_open_exclusive(self, abspath, mode=None):
+        """Open a remote path exclusively.
+
+        SFTP supports O_EXCL (SFTP_FLAG_EXCL), which fails if
+        the file already exists. However it does not expose this
+        at the higher level of SFTPClient.open(), so we have to
+        sneak away with it.
+
+        WARNING: This breaks the SFTPClient abstraction, so it
+        could easily break against an updated version of paramiko.
+
+        :param abspath: The remote absolute path where the file should be opened
+        :param mode: The mode permissions bits for the new file
+        """
+        path = self._sftp._adjust_cwd(abspath)
+        attr = SFTPAttributes()
+        if mode is not None:
+            attr.st_mode = mode
+        omode = (SFTP_FLAG_WRITE | SFTP_FLAG_CREATE 
+                | SFTP_FLAG_TRUNC | SFTP_FLAG_EXCL)
+        try:
+            t, msg = self._sftp._request(CMD_OPEN, path, omode, attr)
+            if t != CMD_HANDLE:
+                raise TransportError('Expected an SFTP handle')
+            handle = msg.get_string()
+            return SFTPFile(self._sftp, handle, 'wb', -1)
+        except (paramiko.SSHException, IOError), e:
+            self._translate_io_exception(e, abspath, ': unable to open',
+                failure_exc=FileExists)
+
+
+# ------------- server test implementation --------------
+import socket
+import threading
+
+from bzrlib.tests.stub_sftp import StubServer, StubSFTPServer
+
+STUB_SERVER_KEY = """
+-----BEGIN RSA PRIVATE KEY-----
+MIICWgIBAAKBgQDTj1bqB4WmayWNPB+8jVSYpZYk80Ujvj680pOTh2bORBjbIAyz
+oWGW+GUjzKxTiiPvVmxFgx5wdsFvF03v34lEVVhMpouqPAYQ15N37K/ir5XY+9m/
+d8ufMCkjeXsQkKqFbAlQcnWMCRnOoPHS3I4vi6hmnDDeeYTSRvfLbW0fhwIBIwKB
+gBIiOqZYaoqbeD9OS9z2K9KR2atlTxGxOJPXiP4ESqP3NVScWNwyZ3NXHpyrJLa0
+EbVtzsQhLn6rF+TzXnOlcipFvjsem3iYzCpuChfGQ6SovTcOjHV9z+hnpXvQ/fon
+soVRZY65wKnF7IAoUwTmJS9opqgrN6kRgCd3DASAMd1bAkEA96SBVWFt/fJBNJ9H
+tYnBKZGw0VeHOYmVYbvMSstssn8un+pQpUm9vlG/bp7Oxd/m+b9KWEh2xPfv6zqU
+avNwHwJBANqzGZa/EpzF4J8pGti7oIAPUIDGMtfIcmqNXVMckrmzQ2vTfqtkEZsA
+4rE1IERRyiJQx6EJsz21wJmGV9WJQ5kCQQDwkS0uXqVdFzgHO6S++tjmjYcxwr3g
+H0CoFYSgbddOT6miqRskOQF3DZVkJT3kyuBgU2zKygz52ukQZMqxCb1fAkASvuTv
+qfpH87Qq5kQhNKdbbwbmd2NxlNabazPijWuphGTdW0VfJdWfklyS2Kr+iqrs/5wV
+HhathJt636Eg7oIjAkA8ht3MQ+XSl9yIJIS8gVpbPxSw5OMfw0PjVE7tBdQruiSc
+nvuQES5C9BMHjF39LZiGH1iLQy7FgdHyoP+eodI7
+-----END RSA PRIVATE KEY-----
+"""
+    
+
+class SingleListener(threading.Thread):
+
+    def __init__(self, callback):
+        threading.Thread.__init__(self)
+        self._callback = callback
+        self._socket = socket.socket()
+        self._socket.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
+        self._socket.bind(('localhost', 0))
+        self._socket.listen(1)
+        self.port = self._socket.getsockname()[1]
+        self.stop_event = threading.Event()
+
+    def run(self):
+        s, _ = self._socket.accept()
+        # now close the listen socket
+        self._socket.close()
+        self._callback(s, self.stop_event)
+    
+    def stop(self):
+        self.stop_event.set()
+        # We should consider waiting for the other thread
+        # to stop, because otherwise we get spurious
+        #   bzr: ERROR: Socket exception: Connection reset by peer (54)
+        # because the test suite finishes before the thread has a chance
+        # to close. (Especially when only running a few tests)
+        
+        
+class SFTPServer(Server):
+    """Common code for SFTP server facilities."""
+
+    def _get_sftp_url(self, path):
+        """Calculate a sftp url to this server for path."""
+        return 'sftp://foo:bar@localhost:%d/%s' % (self._listener.port, path)
+
+    def __init__(self):
+        self._original_vendor = None
+        self._homedir = None
+        self._server_homedir = None
+        self._listener = None
+        self._root = None
+        # sftp server logs
+        self.logs = []
+
+    def log(self, message):
+        """What to do here? do we need this? Its for the StubServer.."""
+        self.logs.append(message)
+
+    def _run_server(self, s, stop_event):
+        ssh_server = paramiko.Transport(s)
+        key_file = os.path.join(self._homedir, 'test_rsa.key')
+        file(key_file, 'w').write(STUB_SERVER_KEY)
+        host_key = paramiko.RSAKey.from_private_key_file(key_file)
+        ssh_server.add_server_key(host_key)
+        server = StubServer(self)
+        ssh_server.set_subsystem_handler('sftp', paramiko.SFTPServer,
+                                         StubSFTPServer, root=self._root,
+                                         home=self._server_homedir)
+        event = threading.Event()
+        ssh_server.start_server(event, server)
+        event.wait(5.0)
+        stop_event.wait(30.0)
+
+    def setUp(self):
+        """See bzrlib.transport.Server.setUp."""
+        # XXX: 20051124 jamesh
+        # The tests currently pop up a password prompt when an external ssh
+        # is used.  This forces the use of the paramiko implementation.
+        global _ssh_vendor
+        self._original_vendor = _ssh_vendor
+        _ssh_vendor = 'none'
+        self._homedir = os.getcwdu()
+        if self._server_homedir is None:
+            self._server_homedir = self._homedir
+        self._root = '/'
+        # FIXME WINDOWS: _root should be _server_homedir[0]:/
+        self._listener = SingleListener(self._run_server)
+        self._listener.setDaemon(True)
+        self._listener.start()
+
+    def tearDown(self):
+        """See bzrlib.transport.Server.tearDown."""
+        global _ssh_vendor
+        self._listener.stop()
+        _ssh_vendor = self._original_vendor
+
+
+class SFTPAbsoluteServer(SFTPServer):
+    """A test server for sftp transports, using absolute urls."""
+
+    def get_url(self):
+        """See bzrlib.transport.Server.get_url."""
+        return self._get_sftp_url("%%2f%s" % 
+                urlescape(self._homedir[1:]))
+
+
+class SFTPHomeDirServer(SFTPServer):
+    """A test server for sftp transports, using homedir relative urls."""
+
+    def get_url(self):
+        """See bzrlib.transport.Server.get_url."""
+        return self._get_sftp_url("")
+
+
+class SFTPSiblingAbsoluteServer(SFTPAbsoluteServer):
+    """A test servere for sftp transports, using absolute urls to non-home."""
+
+    def setUp(self):
+        self._server_homedir = '/dev/noone/runs/tests/here'
+        super(SFTPSiblingAbsoluteServer, self).setUp()
+
+
+def get_test_permutations():
+    """Return the permutations to be used in testing."""
+    return [(SFTPTransport, SFTPAbsoluteServer),
+            (SFTPTransport, SFTPHomeDirServer),
+            (SFTPTransport, SFTPSiblingAbsoluteServer),
+            ]

=== modified file 'bzrlib/tree.py'
--- bzrlib/tree.py	2005-10-03 05:54:35 +0000
+++ bzrlib/tree.py	2005-12-19 20:31:01 +0000
@@ -26,9 +26,6 @@
 from bzrlib.inventory import Inventory
 from bzrlib.osutils import appendpath, fingerprint_file
 
-
-exporters = {}
-
 class Tree(object):
     """Abstract file tree.
 
@@ -58,6 +55,11 @@
     def has_id(self, file_id):
         return self.inventory.has_id(file_id)
 
+    def has_or_had_id(self, file_id):
+        if file_id == self.inventory.root.file_id:
+            return True
+        return self.inventory.has_id(file_id)
+
     __contains__ = has_id
 
     def __iter__(self):
@@ -66,6 +68,9 @@
     def id2path(self, file_id):
         return self.inventory.id2path(file_id)
 
+    def kind(self, file_id):
+        raise NotImplementedError("subclasses must implement kind")
+
     def _get_inventory(self):
         return self._inventory
     
@@ -101,17 +106,6 @@
         sys.stdout.write(self.get_file_text(file_id))
         
         
-    def export(self, dest, format='dir', root=None):
-        """Export this tree."""
-        try:
-            exporter = exporters[format]
-        except KeyError:
-            from bzrlib.errors import BzrCommandError
-            raise BzrCommandError("export format %r not supported" % format)
-        exporter(self, dest, root)
-
-
-
 class RevisionTree(Tree):
     """Tree viewing a previous revision.
 
@@ -122,25 +116,30 @@
            or at least passing a description to the constructor.
     """
     
-    def __init__(self, weave_store, inv, revision_id):
-        self._weave_store = weave_store
+    def __init__(self, branch, inv, revision_id):
+        self._branch = branch
+        self._weave_store = branch.weave_store
         self._inventory = inv
         self._revision_id = revision_id
 
     def get_weave(self, file_id):
-        return self._weave_store.get_weave(file_id)
+        import bzrlib.transactions as transactions
+        return self._weave_store.get_weave(file_id,
+                self._branch.get_transaction())
 
+    def get_weave_prelude(self, file_id):
+        import bzrlib.transactions as transactions
+        return self._weave_store.get_weave_prelude(file_id,
+                self._branch.get_transaction())
 
     def get_file_lines(self, file_id):
         ie = self._inventory[file_id]
         weave = self.get_weave(file_id)
         return weave.get(ie.revision)
-        
 
     def get_file_text(self, file_id):
         return ''.join(self.get_file_lines(file_id))
 
-
     def get_file(self, file_id):
         return StringIO(self.get_file_text(file_id))
 
@@ -153,6 +152,9 @@
             return ie.text_sha1
 
     def is_executable(self, file_id):
+        ie = self._inventory[file_id]
+        if ie.kind != "file":
+            return None 
         return self._inventory[file_id].executable
 
     def has_filename(self, filename):
@@ -167,6 +169,9 @@
         ie = self._inventory[file_id]
         return ie.symlink_target;
 
+    def kind(self, file_id):
+        return self._inventory[file_id].kind
+
 
 class EmptyTree(Tree):
     def __init__(self):
@@ -178,6 +183,10 @@
     def has_filename(self, filename):
         return False
 
+    def kind(self, file_id):
+        assert self._inventory[file_id].kind == "root_directory"
+        return "root_directory"
+
     def list_files(self):
         return iter([])
     
@@ -255,86 +264,3 @@
             
 
 
-######################################################################
-# export
-
-def dir_exporter(tree, dest, root):
-    """Export this tree to a new directory.
-
-    `dest` should not exist, and will be created holding the
-    contents of this tree.
-
-    TODO: To handle subdirectories we need to create the
-           directories first.
-
-    :note: If the export fails, the destination directory will be
-           left in a half-assed state.
-    """
-    import os
-    os.mkdir(dest)
-    mutter('export version %r' % tree)
-    inv = tree.inventory
-    for dp, ie in inv.iter_entries():
-        ie.put_on_disk(dest, dp, tree)
-
-exporters['dir'] = dir_exporter
-
-try:
-    import tarfile
-except ImportError:
-    pass
-else:
-    def get_root_name(dest):
-        """Get just the root name for a tarball.
-
-        >>> get_root_name('mytar.tar')
-        'mytar'
-        >>> get_root_name('mytar.tar.bz2')
-        'mytar'
-        >>> get_root_name('tar.tar.tar.tgz')
-        'tar.tar.tar'
-        >>> get_root_name('bzr-0.0.5.tar.gz')
-        'bzr-0.0.5'
-        >>> get_root_name('a/long/path/mytar.tgz')
-        'mytar'
-        >>> get_root_name('../parent/../dir/other.tbz2')
-        'other'
-        """
-        endings = ['.tar', '.tar.gz', '.tgz', '.tar.bz2', '.tbz2']
-        dest = os.path.basename(dest)
-        for end in endings:
-            if dest.endswith(end):
-                return dest[:-len(end)]
-
-    def tar_exporter(tree, dest, root, compression=None):
-        """Export this tree to a new tar file.
-
-        `dest` will be created holding the contents of this tree; if it
-        already exists, it will be clobbered, like with "tar -c".
-        """
-        from time import time
-        now = time()
-        compression = str(compression or '')
-        if root is None:
-            root = get_root_name(dest)
-        try:
-            ball = tarfile.open(dest, 'w:' + compression)
-        except tarfile.CompressionError, e:
-            raise BzrError(str(e))
-        mutter('export version %r' % tree)
-        inv = tree.inventory
-        for dp, ie in inv.iter_entries():
-            mutter("  export {%s} kind %s to %s" % (ie.file_id, ie.kind, dest))
-            item, fileobj = ie.get_tar_item(root, dp, now, tree)
-            ball.addfile(item, fileobj)
-        ball.close()
-
-    exporters['tar'] = tar_exporter
-
-    def tgz_exporter(tree, dest, root):
-        tar_exporter(tree, dest, root, compression='gz')
-    exporters['tgz'] = tgz_exporter
-
-    def tbz_exporter(tree, dest, root):
-        tar_exporter(tree, dest, root, compression='bz2')
-    exporters['tbz2'] = tbz_exporter

=== added file 'bzrlib/tsort.py'
--- bzrlib/tsort.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tsort.py	2005-10-26 05:49:06 +0000
@@ -0,0 +1,60 @@
+# (C) 2005 Canonical
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+import pdb
+
+from bzrlib.errors import GraphCycleError
+
+def topo_sort(graph):
+    """Topological sort a graph.
+
+    graph -- sequence of pairs of node->parents_list.
+
+    The result is a list of node names, such that all parents come before
+    their children.
+
+    Nodes at the same depth are returned in sorted order.
+
+    node identifiers can be any hashable object, and are typically strings.
+    """
+    parents = {}  # node -> list of parents
+    children = {} # node -> list of children
+    for node, node_parents in graph:
+        assert node not in parents, \
+            ('node %r repeated in graph' % node)
+        parents[node] = set(node_parents)
+        if node not in children:
+            children[node] = set()
+        for parent in node_parents:
+            if parent in children:
+                children[parent].add(node)
+            else:
+                children[parent] = set([node])
+    result = []
+    while parents:
+        # find nodes with no parents, and take them now
+        no_parents = [n for n in parents if len(parents[n]) == 0]
+        no_parents.sort()
+        if not no_parents:
+            raise GraphCycleError(parents)
+        for n in no_parents:
+            result.append(n)
+            for child in children[n]:
+                assert n in parents[child]
+                parents[child].remove(n)
+            del children[n]
+            del parents[n]
+    return result

=== added directory 'bzrlib/ui'
=== renamed file 'bzrlib/ui.py' => 'bzrlib/ui/__init__.py'
--- bzrlib/ui.py	2005-08-25 12:46:42 +0000
+++ bzrlib/ui/__init__.py	2005-11-30 16:41:08 +0000
@@ -28,24 +28,42 @@
 displays no output.
 """
 
-
-
-
-
 import bzrlib.progress
 
 
-class TextUIFactory(object):
+class UIFactory(object):
+    """UI abstraction.
+
+    This tells the library how to display things to the user.  Through this
+    layer different applications can choose the style of UI.
+    """
     def progress_bar(self):
-
-        # this in turn is abstract, and creates either a tty or dots
-        # bar depending on what we think of the terminal
-        return bzrlib.progress.ProgressBar()
-
-
-class SilentUIFactory(object):
+        """Return a progress bar object"""
+        raise NotImplementedError
+
+    def get_password(self, prompt='', **kwargs):
+        """Prompt the user for a password.
+
+        :param prompt: The prompt to present the user
+        :param kwargs: Arguments which will be expanded into the prompt.
+                       This lets front ends display different things if
+                       they so choose.
+        :return: The password string, return None if the user 
+                 canceled the request.
+        """
+        raise NotImplementedError
+        
+
+class SilentUIFactory(UIFactory):
+    """A UI Factory which never prints anything.
+
+    This is the default UI, if another one is never registered.
+    """
     def progress_bar(self):
         return bzrlib.progress.DummyProgress()
 
+    def get_password(self, prompt='', **kwargs):
+        return None
+
 
 ui_factory = SilentUIFactory()

=== added file 'bzrlib/ui/text.py'
--- bzrlib/ui/text.py	1970-01-01 00:00:00 +0000
+++ bzrlib/ui/text.py	2005-11-30 16:41:08 +0000
@@ -0,0 +1,52 @@
+# Copyright (C) 2005 Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+
+"""Text UI, write output to the console.
+"""
+
+import getpass
+import sys
+
+import bzrlib.progress
+from bzrlib.ui import UIFactory
+
+
+class TextUIFactory(UIFactory):
+    def progress_bar(self):
+
+        # this in turn is abstract, and creates either a tty or dots
+        # bar depending on what we think of the terminal
+        return bzrlib.progress.ProgressBar()
+
+    def get_password(self, prompt='', **kwargs):
+        """Prompt the user for a password.
+
+        :param prompt: The prompt to present the user
+        :param kwargs: Arguments which will be expanded into the prompt.
+                       This lets front ends display different things if
+                       they so choose.
+        :return: The password string, return None if the user 
+                 canceled the request.
+        """
+        prompt = (prompt % kwargs).encode(sys.stdout.encoding, 'replace')
+        prompt += ': '
+        try:
+            return getpass.getpass(prompt)
+        except KeyboardInterrupt:
+            return None
+

=== added file 'bzrlib/uncommit.py'
--- bzrlib/uncommit.py	1970-01-01 00:00:00 +0000
+++ bzrlib/uncommit.py	2005-11-23 15:32:07 +0000
@@ -0,0 +1,87 @@
+#!/usr/bin/env python
+"""\
+Remove the last revision from the history of the current branch.
+"""
+
+import os
+import bzrlib
+
+def test_remove(filename):
+    if os.path.exists(filename):
+        os.remove(filename)
+    else:
+        print '* file does not exist: %r' % filename
+
+
+def uncommit(branch, remove_files=False,
+        dry_run=False, verbose=False, revno=None):
+    """Remove the last revision from the supplied branch.
+
+    :param remove_files: If True, remove files from the stores
+        as well.
+    :param force: Don't ask any questions
+    """
+    from bzrlib.atomicfile import AtomicFile
+    rh = branch.revision_history()
+    if revno is None:
+        revno = len(rh)
+
+    files_to_remove = []
+    new_rev_history = AtomicFile(branch.controlfilename('revision-history'))
+    for r in range(revno-1, len(rh)):
+        rev_id = rh.pop()
+        if verbose:
+            print 'Removing revno %d: %s' % (len(rh)+1, rev_id)
+        rev = branch.get_revision(rev_id)
+        inv = branch.get_revision_inventory(rev_id)
+        inv_prev = []
+        for p in rev.parent_ids:
+            inv_prev.append(branch.get_revision_inventory(p))
+
+        if remove_files:
+            # Figure out what text-store entries are new
+
+            # In the future, when we have text_version instead of
+            # text_id, we can just check to see if the text_version
+            # equals the current revision id.
+            for file_id in inv:
+                ie = inv[file_id]
+                if not hasattr(ie, 'text_id'):
+                    continue
+                for other_inv in inv_prev:
+                    if file_id in other_inv:
+                        other_ie = other_inv[file_id]
+                        if other_ie.text_id == ie.text_id:
+                            break
+                else:
+                    # None of the previous ancestors used
+                    # the same inventory
+                    files_to_remove.append(branch.controlfilename(['text-store',
+                        ie.text_id + '.gz']))
+            rev_file = branch.controlfilename(['revision-store',
+                    rev_id + '.gz'])
+            files_to_remove.append(rev_file)
+            inv_file = branch.controlfilename(['inventory-store',
+                    rev.inventory_id + '.gz'])
+            files_to_remove.append(inv_file)
+
+    if verbose and files_to_remove:
+        print 'Removing files:'
+        for f in files_to_remove:
+            print '\t%s' % branch.relpath(f)
+
+    new_rev_history.write('\n'.join(rh))
+
+    # Committing before we start removing files, because
+    # once we have removed at least one, all the rest are invalid.
+    if not dry_run:
+        new_rev_history.commit()
+        if remove_files:
+            # Actually start removing files
+            for f in files_to_remove:
+                test_remove(f)
+
+    else:
+        new_rev_history.abort()
+
+

=== modified file 'bzrlib/upgrade.py'
--- bzrlib/upgrade.py	2005-10-06 06:31:33 +0000
+++ bzrlib/upgrade.py	2005-12-01 18:16:47 +0000
@@ -1,5 +1,3 @@
-#! /usr/bin/python
-#
 # Copyright (C) 2005 Canonical Ltd
 #
 # This program is free software; you can redistribute it and/or modify
@@ -68,33 +66,22 @@
 # versions.
 
 
-# TODO: Don't create a progress bar here, have it passed by the caller.  
-# At least do it from the UI factory.
-
-if False:
-    try:
-        import psyco
-        psyco.full()
-    except ImportError:
-        pass
-
-
 import os
 import tempfile
 import sys
-import logging
 import shutil
 
-from bzrlib.branch import Branch, find_branch, BZR_BRANCH_FORMAT_5
-from bzrlib.revfile import Revfile
+from bzrlib.branch import Branch, find_branch
+from bzrlib.branch import BZR_BRANCH_FORMAT_5, BZR_BRANCH_FORMAT_6
+import bzrlib.hashcache as hashcache
 from bzrlib.weave import Weave
 from bzrlib.weavefile import read_weave, write_weave
 from bzrlib.ui import ui_factory
 from bzrlib.atomicfile import AtomicFile
 from bzrlib.xml4 import serializer_v4
 from bzrlib.xml5 import serializer_v5
-from bzrlib.trace import mutter, note, warning, enable_default_logging
-from bzrlib.osutils import sha_strings, sha_string
+from bzrlib.trace import mutter, note, warning
+from bzrlib.osutils import sha_strings, sha_string, pathjoin, abspath
 
 
 class Convert(object):
@@ -112,9 +99,41 @@
             return
         note('starting upgrade of %s', os.path.abspath(self.base))
         self._backup_control_dir()
-        note('starting upgrade')
+        self.pb = ui_factory.progress_bar()
+        if self.old_format == 4:
+            note('starting upgrade from format 4 to 5')
+            self._convert_to_weaves()
+            self._open_branch()
+        if self.old_format == 5:
+            note('starting upgrade from format 5 to 6')
+            self._convert_to_prefixed()
+            self._open_branch()
+        cache = hashcache.HashCache(abspath(self.base))
+        cache.clear()
+        cache.write()
+        note("finished")
+
+
+    def _convert_to_prefixed(self):
+        from bzrlib.store import hash_prefix
+        for store_name in ["weaves", "revision-store"]:
+            note("adding prefixes to %s" % store_name) 
+            store_dir = pathjoin(self.base, ".bzr", store_name)
+            for filename in os.listdir(store_dir):
+                if filename.endswith(".weave") or filename.endswith(".gz"):
+                    file_id = os.path.splitext(filename)[0]
+                else:
+                    file_id = filename
+                prefix_dir = pathjoin(store_dir, hash_prefix(file_id))
+                if not os.path.isdir(prefix_dir):
+                    os.mkdir(prefix_dir)
+                os.rename(pathjoin(store_dir, filename),
+                          pathjoin(prefix_dir, filename))
+        self._set_new_format(BZR_BRANCH_FORMAT_6)
+
+
+    def _convert_to_weaves(self):
         note('note: upgrade may be faster if all store files are ungzipped first')
-        self.pb = ui_factory.progress_bar()
         if not os.path.isdir(self.base + '/.bzr/weaves'):
             os.mkdir(self.base + '/.bzr/weaves')
         self.inv_weave = Weave('inventory')
@@ -126,7 +145,7 @@
         # to_read is a stack holding the revisions we still need to process;
         # appending to it adds new highest-priority revisions
         self.known_revisions = set(rev_history)
-        self.to_read = [rev_history[-1]]
+        self.to_read = rev_history[-1:]
         while self.to_read:
             rev_id = self.to_read.pop()
             if (rev_id not in self.revisions
@@ -144,23 +163,24 @@
         note('  %6d texts' % self.text_count)
         self._write_all_weaves()
         self._write_all_revs()
-        self._set_new_format()
         self._cleanup_spare_files()
+        self._set_new_format(BZR_BRANCH_FORMAT_5)
 
 
     def _open_branch(self):
         self.branch = Branch.open_downlevel(self.base)
-        if self.branch._branch_format == 5:
-            note('this branch is already in the most current format')
+        self.old_format = self.branch._branch_format
+        if self.old_format == 6:
+            note('this branch is in the most current format')
             return False
-        if self.branch._branch_format != 4:
+        if self.old_format not in (4, 5):
             raise BzrError("cannot upgrade from branch format %r" %
                            self.branch._branch_format)
         return True
 
 
-    def _set_new_format(self):
-        self.branch.put_controlfile('branch-format', BZR_BRANCH_FORMAT_5)
+    def _set_new_format(self, format):
+        self.branch.put_controlfile('branch-format', format)
 
 
     def _cleanup_spare_files(self):
@@ -228,14 +248,14 @@
         self.pb.update('loading revision',
                        len(self.revisions),
                        len(self.known_revisions))
-        if rev_id not in self.branch.revision_store:
+        if not self.branch.revision_store.has_id(rev_id):
             self.pb.clear()
             note('revision {%s} not present in branch; '
                  'will be converted as a ghost',
                  rev_id)
             self.absent_revisions.add(rev_id)
         else:
-            rev_xml = self.branch.revision_store[rev_id].read()
+            rev_xml = self.branch.revision_store.get(rev_id).read()
             rev = serializer_v4.read_revision_from_string(rev_xml)
             for parent_id in rev.parent_ids:
                 self.known_revisions.add(parent_id)
@@ -245,7 +265,7 @@
 
     def _load_old_inventory(self, rev_id):
         assert rev_id not in self.converted_revs
-        old_inv_xml = self.branch.inventory_store[rev_id].read()
+        old_inv_xml = self.branch.inventory_store.get(rev_id).read()
         inv = serializer_v4.read_inventory_from_string(old_inv_xml)
         rev = self.revisions[rev_id]
         if rev.inventory_sha1:
@@ -340,7 +360,7 @@
                 return
         parent_indexes = map(w.lookup, previous_revisions)
         if ie.has_text():
-            file_lines = self.branch.text_store[ie.text_id].readlines()
+            file_lines = self.branch.text_store.get(ie.text_id).readlines()
             assert sha_strings(file_lines) == ie.text_sha1
             assert sum(map(len, file_lines)) == ie.text_size
             w.add(rev_id, parent_indexes, file_lines, ie.text_sha1)

=== added directory 'bzrlib/util/configobj'
=== added file 'bzrlib/util/configobj/__init__.py'
=== added file 'bzrlib/util/configobj/configobj.py'
--- bzrlib/util/configobj/configobj.py	1970-01-01 00:00:00 +0000
+++ bzrlib/util/configobj/configobj.py	2005-10-18 18:48:27 +0000
@@ -0,0 +1,2681 @@
+# configobj.py
+# A config file reader/writer that supports nested sections in config files.
+# Copyright (C) 2005 Michael Foord, Nicola Larosa
+# E-mail: fuzzyman AT voidspace DOT org DOT uk
+#         nico AT tekNico DOT net
+
+# ConfigObj 4
+
+# Released subject to the BSD License
+# Please see http://www.voidspace.org.uk/documents/BSD-LICENSE.txt
+
+# Scripts maintained at http://www.voidspace.org.uk/python/index.shtml
+# For information about bugfixes, updates and support, please join the
+# ConfigObj mailing list:
+# http://lists.sourceforge.net/lists/listinfo/configobj-develop
+# Comments, suggestions and bug reports welcome.
+
+"""
+    >>> z = ConfigObj()
+    >>> z['a'] = 'a'
+    >>> z['sect'] = {
+    ...    'subsect': {
+    ...         'a': 'fish',
+    ...         'b': 'wobble',
+    ...     },
+    ...     'member': 'value',
+    ... }
+    >>> x = ConfigObj(z.write())
+    >>> z == x
+    1
+"""
+
+import sys
+INTP_VER = sys.version_info[:2]
+if INTP_VER < (2, 2):
+    raise RuntimeError("Python v.2.2 or later needed")
+
+import os, re
+from types import StringTypes
+
+# the UTF8 BOM - from codecs module
+BOM_UTF8 = '\xef\xbb\xbf'
+
+__version__ = '4.0.0'
+
+__revision__ = '$Id: configobj.py 135 2005-10-12 14:52:28Z fuzzyman $'
+
+__docformat__ = "restructuredtext en"
+
+__all__ = (
+    '__version__',
+    'BOM_UTF8',
+    'DEFAULT_INDENT_TYPE',
+    'NUM_INDENT_SPACES',
+    'MAX_INTERPOL_DEPTH',
+    'ConfigObjError',
+    'NestingError',
+    'ParseError',
+    'DuplicateError',
+    'ConfigspecError',
+    'ConfigObj',
+    'SimpleVal',
+    'InterpolationError',
+    'InterpolationDepthError',
+    'MissingInterpolationOption',
+    'RepeatSectionError',
+    '__docformat__',
+)
+
+DEFAULT_INDENT_TYPE = ' '
+NUM_INDENT_SPACES = 4
+MAX_INTERPOL_DEPTH = 10
+
+OPTION_DEFAULTS = {
+    'interpolation': True,
+    'raise_errors': False,
+    'list_values': True,
+    'create_empty': False,
+    'file_error': False,
+    'configspec': None,
+    'stringify': True,
+    # option may be set to one of ('', ' ', '\t')
+    'indent_type': None,
+}
+
+class ConfigObjError(SyntaxError):
+    """
+    This is the base class for all errors that ConfigObj raises.
+    It is a subclass of SyntaxError.
+    
+    >>> raise ConfigObjError
+    Traceback (most recent call last):
+    ConfigObjError
+    """
+    def __init__(self, message='', line_number=None, line=''):
+        self.line = line
+        self.line_number = line_number
+        self.message = message
+        SyntaxError.__init__(self, message)
+
+class NestingError(ConfigObjError):
+    """
+    This error indicates a level of nesting that doesn't match.
+    
+    >>> raise NestingError
+    Traceback (most recent call last):
+    NestingError
+    """
+
+class ParseError(ConfigObjError):
+    """
+    This error indicates that a line is badly written.
+    It is neither a valid ``key = value`` line,
+    nor a valid section marker line.
+    
+    >>> raise ParseError
+    Traceback (most recent call last):
+    ParseError
+    """
+
+class DuplicateError(ConfigObjError):
+    """
+    The keyword or section specified already exists.
+    
+    >>> raise DuplicateError
+    Traceback (most recent call last):
+    DuplicateError
+    """
+
+class ConfigspecError(ConfigObjError):
+    """
+    An error occured whilst parsing a configspec.
+    
+    >>> raise ConfigspecError
+    Traceback (most recent call last):
+    ConfigspecError
+    """
+
+class InterpolationError(ConfigObjError):
+    """Base class for the two interpolation errors."""
+
+class InterpolationDepthError(InterpolationError):
+    """Maximum interpolation depth exceeded in string interpolation."""
+
+    def __init__(self, option):
+        """
+        >>> raise InterpolationDepthError('yoda')
+        Traceback (most recent call last):
+        InterpolationDepthError: max interpolation depth exceeded in value "yoda".
+        """
+        InterpolationError.__init__(
+            self,
+            'max interpolation depth exceeded in value "%s".' % option)
+
+class RepeatSectionError(ConfigObjError):
+    """
+    This error indicates additional sections in a section with a
+    ``__many__`` (repeated) section.
+    
+    >>> raise RepeatSectionError
+    Traceback (most recent call last):
+    RepeatSectionError
+    """
+
+class MissingInterpolationOption(InterpolationError):
+    """A value specified for interpolation was missing."""
+
+    def __init__(self, option):
+        """
+        >>> raise MissingInterpolationOption('yoda')
+        Traceback (most recent call last):
+        MissingInterpolationOption: missing option "yoda" in interpolation.
+        """
+        InterpolationError.__init__(
+            self,
+            'missing option "%s" in interpolation.' % option)
+
+class Section(dict):
+    """
+    A dictionary-like object that represents a section in a config file.
+    
+    It does string interpolation if the 'interpolate' attribute
+    of the 'main' object is set to True.
+    
+    Interpolation is tried first from the 'DEFAULT' section of this object,
+    next from the 'DEFAULT' section of the parent, lastly the main object.
+    
+    A Section will behave like an ordered dictionary - following the
+    order of the ``scalars`` and ``sections`` attributes.
+    You can use this to change the order of members.
+    
+    Iteration follows the order: scalars, then sections.
+    """
+
+    _KEYCRE = re.compile(r"%\(([^)]*)\)s|.")
+
+    def __init__(self, parent, depth, main, indict=None, name=None):
+        """
+        parent is the section above
+        depth is the depth level of this section
+        main is the main ConfigObj
+        indict is a dictionary to initialise the section with
+        """
+        if indict is None:
+            indict = {}
+        dict.__init__(self)
+        # used for nesting level *and* interpolation
+        self.parent = parent
+        # used for the interpolation attribute
+        self.main = main
+        # level of nesting depth of this Section
+        self.depth = depth
+        # the sequence of scalar values in this Section
+        self.scalars = []
+        # the sequence of sections in this Section
+        self.sections = []
+        # purely for information
+        self.name = name
+        # for comments :-)
+        self.comments = {}
+        self.inline_comments = {}
+        # for the configspec
+        self.configspec = {}
+        # for defaults
+        self.defaults = []
+        #
+        # we do this explicitly so that __setitem__ is used properly
+        # (rather than just passing to ``dict.__init__``)
+        for entry in indict:
+            self[entry] = indict[entry]
+
+    def _interpolate(self, value):
+        """Nicked from ConfigParser."""
+        depth = MAX_INTERPOL_DEPTH
+        # loop through this until it's done
+        while depth:
+            depth -= 1
+            if value.find("%(") != -1:
+                value = self._KEYCRE.sub(self._interpolation_replace, value)
+            else:
+                break
+        else:
+            raise InterpolationDepthError(value)
+        return value
+
+    def _interpolation_replace(self, match):
+        """ """
+        s = match.group(1)
+        if s is None:
+            return match.group()
+        else:
+            # switch off interpolation before we try and fetch anything !
+            self.main.interpolation = False
+            # try the 'DEFAULT' member of *this section* first
+            val = self.get('DEFAULT', {}).get(s)
+            # try the 'DEFAULT' member of the *parent section* next
+            if val is None:
+                val = self.parent.get('DEFAULT', {}).get(s)
+            # last, try the 'DEFAULT' member of the *main section*
+            if val is None:
+                val = self.main.get('DEFAULT', {}).get(s)
+            self.main.interpolation = True
+            if val is None:
+                raise MissingInterpolationOption(s)
+            return val
+
+    def __getitem__(self, key):
+        """Fetch the item and do string interpolation."""
+        val = dict.__getitem__(self, key)
+        if self.main.interpolation and isinstance(val, StringTypes):
+            return self._interpolate(val)
+        return val
+
+    def __setitem__(self, key, value):
+        """
+        Correctly set a value.
+        
+        Making dictionary values Section instances.
+        (We have to special case 'Section' instances - which are also dicts)
+        
+        Keys must be strings.
+        Values need only be strings (or lists of strings) if
+        ``main.stringify`` is set.
+        """
+        if not isinstance(key, StringTypes):
+            raise ValueError, 'The key "%s" is not a string.' % key
+##        if self.depth is None:
+##            self.depth = 0
+        # add the comment
+        if not self.comments.has_key(key):
+            self.comments[key] = []
+            self.inline_comments[key] = ''
+        # remove the entry from defaults
+        if key in self.defaults:
+            self.defaults.remove(key)
+        #
+        if isinstance(value, Section):
+            if not self.has_key(key):
+                self.sections.append(key)
+            dict.__setitem__(self, key, value)
+        elif isinstance(value, dict):
+            # First create the new depth level,
+            # then create the section
+            if not self.has_key(key):
+                self.sections.append(key)
+            new_depth = self.depth + 1
+            dict.__setitem__(
+                self,
+                key,
+                Section(
+                    self,
+                    new_depth,
+                    self.main,
+                    indict=value,
+                    name=key))
+        else:
+            if not self.has_key(key):
+                self.scalars.append(key)
+            if not self.main.stringify:
+                if isinstance(value, StringTypes):
+                    pass
+                elif isinstance(value, (list, tuple)):
+                    for entry in value:
+                        if not isinstance(entry, StringTypes):
+                            raise TypeError, (
+                                'Value is not a string "%s".' % entry)
+                else:
+                    raise TypeError, 'Value is not a string "%s".' % value
+            dict.__setitem__(self, key, value)
+
+    def __delitem__(self, key):
+        """Remove items from the sequence when deleting."""
+        dict. __delitem__(self, key)
+        if key in self.scalars:
+            self.scalars.remove(key)
+        else:
+            self.sections.remove(key)
+        del self.comments[key]
+        del self.inline_comments[key]
+
+    def get(self, key, default=None):
+        """A version of ``get`` that doesn't bypass string interpolation."""
+        try:
+            return self[key]
+        except KeyError:
+            return default
+
+    def update(self, indict):
+        """A version of update that uses our ``__setitem__``."""
+        for entry in indict:
+            self[entry] = indict[entry]
+
+    def pop(self, key, *args):
+        """ """
+        val = dict.pop(self, key, *args)
+        if key in self.scalars:
+            del self.comments[key]
+            del self.inline_comments[key]
+            self.scalars.remove(key)
+        elif key in self.sections:
+            del self.comments[key]
+            del self.inline_comments[key]
+            self.sections.remove(key)
+        if self.main.interpolation and isinstance(val, StringTypes):
+            return self._interpolate(val)
+        return val
+
+    def popitem(self):
+        """Pops the first (key,val)"""
+        sequence = (self.scalars + self.sections)
+        if not sequence:
+            raise KeyError, ": 'popitem(): dictionary is empty'"
+        key = sequence[0]
+        val =  self[key]
+        del self[key]
+        return key, val
+
+    def clear(self):
+        """
+        A version of clear that also affects scalars/sections
+        Also clears comments and configspec.
+        
+        Leaves other attributes alone :
+            depth/main/parent are not affected
+        """
+        dict.clear(self)
+        self.scalars = []
+        self.sections = []
+        self.comments = {}
+        self.inline_comments = {}
+        self.configspec = {}
+
+    def setdefault(self, key, default=None):
+        """A version of setdefault that sets sequence if appropriate."""
+        try:
+            return self[key]
+        except KeyError:
+            self[key] = default
+            return self[key]
+
+    def items(self):
+        """ """
+        return zip((self.scalars + self.sections), self.values())
+
+    def keys(self):
+        """ """
+        return (self.scalars + self.sections)
+
+    def values(self):
+        """ """
+        return [self[key] for key in (self.scalars + self.sections)]
+
+    def iteritems(self):
+        """ """
+        return iter(self.items())
+
+    def iterkeys(self):
+        """ """
+        return iter((self.scalars + self.sections))
+
+    __iter__ = iterkeys
+
+    def itervalues(self):
+        """ """
+        return iter(self.values())
+
+    def __repr__(self):
+        return '{%s}' % ', '.join([('%s: %s' % (repr(key), repr(self[key])))
+            for key in (self.scalars + self.sections)])
+
+    __str__ = __repr__
+
+    # Extra methods - not in a normal dictionary
+
+    def dict(self):
+        """
+        Return a deepcopy of self as a dictionary.
+        
+        All members that are ``Section`` instances are recursively turned to
+        ordinary dictionaries - by calling their ``dict`` method.
+        
+        >>> n = a.dict()
+        >>> n == a
+        1
+        >>> n is a
+        0
+        """
+        newdict = {}
+        for entry in self:
+            this_entry = self[entry]
+            if isinstance(this_entry, Section):
+                this_entry = this_entry.dict()
+            elif isinstance(this_entry, (list, tuple)):
+                # create a copy rather than a reference
+                this_entry = list(this_entry)
+            newdict[entry] = this_entry
+        return newdict
+
+    def rename(self, oldkey, newkey):
+        """
+        Change a keyname to another, without changing position in sequence.
+        
+        Implemented so that transformations can be made on keys,
+        as well as on values. (used by encode and decode)
+        
+        Also renames comments.
+        """
+        if oldkey in self.scalars:
+            the_list = self.scalars
+        elif oldkey in self.sections:
+            the_list = self.sections
+        else:
+            raise KeyError, 'Key "%s" not found.' % oldkey
+        pos = the_list.index(oldkey)
+        #
+        val = self[oldkey]
+        dict.__delitem__(self, oldkey)
+        dict.__setitem__(self, newkey, val)
+        the_list.remove(oldkey)
+        the_list.insert(pos, newkey)
+        comm = self.comments[oldkey]
+        inline_comment = self.inline_comments[oldkey]
+        del self.comments[oldkey]
+        del self.inline_comments[oldkey]
+        self.comments[newkey] = comm
+        self.inline_comments[newkey] = inline_comment
+
+    def walk(self, function, raise_errors=True,
+            call_on_sections=False, **keywargs):
+        """
+        Walk every member and call a function on the keyword and value.
+        
+        Return a dictionary of the return values
+        
+        If the function raises an exception, raise the errror
+        unless ``raise_errors=False``, in which case set the return value to
+        ``False``.
+        
+        Any unrecognised keyword arguments you pass to walk, will be pased on
+        to the function you pass in.
+        
+        Note: if ``call_on_sections`` is ``True`` then - on encountering a
+        subsection, *first* the function is called for the *whole* subsection,
+        and then recurses into it's members. This means your function must be
+        able to handle strings, dictionaries and lists. This allows you
+        to change the key of subsections as well as for ordinary members. The
+        return value when called on the whole subsection has to be discarded.
+        
+        See  the encode and decode methods for examples, including functions.
+        """
+        out = {}
+        # scalars first
+        for entry in self.scalars[:]:
+            try:
+                out[entry] = function(self, entry, **keywargs)
+            except Exception:
+                if raise_errors:
+                    raise
+                else:
+                    out[entry] = False
+        # then sections
+        for entry in self.sections[:]:
+            if call_on_sections:
+                try:
+                    function(self, entry, **keywargs)
+                except Exception:
+                    if raise_errors:
+                        raise
+                    else:
+                        out[entry] = False
+            # previous result is discarded
+            out[entry] = self[entry].walk(
+                function,
+                raise_errors=raise_errors,
+                call_on_sections=call_on_sections,
+                **keywargs)
+        return out
+
+    def decode(self, encoding):
+        """
+        Decode all strings and values to unicode, using the specified encoding.
+        
+        Works with subsections and list values.
+        
+        Uses the ``walk`` method.
+        
+        Testing ``encode`` and ``decode``.
+        >>> m = ConfigObj(a)
+        >>> m.decode('ascii')
+        >>> def testuni(val):
+        ...     for entry in val:
+        ...         if not isinstance(entry, unicode):
+        ...             print >> sys.stderr, type(entry)
+        ...             raise AssertionError, 'decode failed.'
+        ...         if isinstance(val[entry], dict):
+        ...             testuni(val[entry])
+        ...         elif not isinstance(val[entry], unicode):
+        ...             raise AssertionError, 'decode failed.'
+        >>> testuni(m)
+        >>> m.encode('ascii')
+        >>> a == m
+        1
+        """
+        def decode(section, key, encoding=encoding):
+            """ """
+            val = section[key]
+            if isinstance(val, (list, tuple)):
+                newval = []
+                for entry in val:
+                    newval.append(entry.decode(encoding))
+            elif isinstance(val, dict):
+                newval = val
+            else:
+                newval = val.decode(encoding)
+            newkey = key.decode(encoding)
+            section.rename(key, newkey)
+            section[newkey] = newval
+        # using ``call_on_sections`` allows us to modify section names
+        self.walk(decode, call_on_sections=True)
+
+    def encode(self, encoding):
+        """
+        Encode all strings and values from unicode,
+        using the specified encoding.
+        
+        Works with subsections and list values.
+        Uses the ``walk`` method.
+        """
+        def encode(section, key, encoding=encoding):
+            """ """
+            val = section[key]
+            if isinstance(val, (list, tuple)):
+                newval = []
+                for entry in val:
+                    newval.append(entry.encode(encoding))
+            elif isinstance(val, dict):
+                newval = val
+            else:
+                newval = val.encode(encoding)
+            newkey = key.encode(encoding)
+            section.rename(key, newkey)
+            section[newkey] = newval
+        self.walk(encode, call_on_sections=True)
+
+class ConfigObj(Section):
+    """
+    An object to read, create, and write config files.
+    
+    Testing with duplicate keys and sections.
+    
+    >>> c = '''
+    ... [hello]
+    ... member = value
+    ... [hello again]
+    ... member = value
+    ... [ "hello" ]
+    ... member = value
+    ... '''
+    >>> ConfigObj(c.split('\\n'), raise_errors = True)
+    Traceback (most recent call last):
+    DuplicateError: Duplicate section name at line 5.
+    
+    >>> d = '''
+    ... [hello]
+    ... member = value
+    ... [hello again]
+    ... member1 = value
+    ... member2 = value
+    ... 'member1' = value
+    ... [ "and again" ]
+    ... member = value
+    ... '''
+    >>> ConfigObj(d.split('\\n'), raise_errors = True)
+    Traceback (most recent call last):
+    DuplicateError: Duplicate keyword name at line 6.
+    """
+
+    _keyword = re.compile(r'''^ # line start
+        (\s*)                   # indentation
+        (                       # keyword
+            (?:".*?")|          # double quotes
+            (?:'.*?')|          # single quotes
+            (?:[^'"=].*?)       # no quotes
+        )
+        \s*=\s*                 # divider
+        (.*)                    # value (including list values and comments)
+        $   # line end
+        ''',
+        re.VERBOSE)
+
+    _sectionmarker = re.compile(r'''^
+        (\s*)                     # 1: indentation
+        ((?:\[\s*)+)              # 2: section marker open
+        (                         # 3: section name open
+            (?:"\s*\S.*?\s*")|    # at least one non-space with double quotes
+            (?:'\s*\S.*?\s*')|    # at least one non-space with single quotes
+            (?:[^'"\s].*?)        # at least one non-space unquoted
+        )                         # section name close
+        ((?:\s*\])+)              # 4: section marker close
+        \s*(\#.*)?                # 5: optional comment
+        $''',
+        re.VERBOSE)
+
+    # this regexp pulls list values out as a single string
+    # or single values and comments
+    _valueexp = re.compile(r'''^
+        (?:
+            (?:
+                (
+                    (?:
+                        (?:
+                            (?:".*?")|              # double quotes
+                            (?:'.*?')|              # single quotes
+                            (?:[^'",\#][^,\#]*?)       # unquoted
+                        )
+                        \s*,\s*                     # comma
+                    )*      # match all list items ending in a comma (if any)
+                )
+                (
+                    (?:".*?")|                      # double quotes
+                    (?:'.*?')|                      # single quotes
+                    (?:[^'",\#\s][^,]*?)             # unquoted
+                )?          # last item in a list - or string value
+            )|
+            (,)             # alternatively a single comma - empty list
+        )
+        \s*(\#.*)?          # optional comment
+        $''',
+        re.VERBOSE)
+
+    # use findall to get the members of a list value
+    _listvalueexp = re.compile(r'''
+        (
+            (?:".*?")|          # double quotes
+            (?:'.*?')|          # single quotes
+            (?:[^'",\#].*?)       # unquoted
+        )
+        \s*,\s*                 # comma
+        ''',
+        re.VERBOSE)
+
+    # this regexp is used for the value
+    # when lists are switched off
+    _nolistvalue = re.compile(r'''^
+        (
+            (?:".*?")|          # double quotes
+            (?:'.*?')|          # single quotes
+            (?:[^'"\#].*?)      # unquoted
+        )
+        \s*(\#.*)?              # optional comment
+        $''',
+        re.VERBOSE)
+
+    # regexes for finding triple quoted values on one line
+    _single_line_single = re.compile(r"^'''(.*?)'''\s*(#.*)?$")
+    _single_line_double = re.compile(r'^"""(.*?)"""\s*(#.*)?$')
+    _multi_line_single = re.compile(r"^(.*?)'''\s*(#.*)?$")
+    _multi_line_double = re.compile(r'^(.*?)"""\s*(#.*)?$')
+
+    _triple_quote = {
+        "'''": (_single_line_single, _multi_line_single),
+        '"""': (_single_line_double, _multi_line_double),
+    }
+
+    def __init__(self, infile=None, options=None, **kwargs):
+        """
+        Parse or create a config file object.
+        
+        ``ConfigObj(infile=None, options=None, **kwargs)``
+        """
+        if infile is None:
+            infile = []
+        if options is None:
+            options = {}
+        # keyword arguments take precedence over an options dictionary
+        options.update(kwargs)
+        # init the superclass
+        Section.__init__(self, self, 0, self)
+        #
+        defaults = OPTION_DEFAULTS.copy()
+        for entry in options.keys():
+            if entry not in defaults.keys():
+                raise TypeError, 'Unrecognised option "%s".' % entry
+        # TODO: check the values too
+        # add the explicit options to the defaults
+        defaults.update(options)
+        #
+        # initialise a few variables
+        self._errors = []
+        self.raise_errors = defaults['raise_errors']
+        self.interpolation = defaults['interpolation']
+        self.list_values = defaults['list_values']
+        self.create_empty = defaults['create_empty']
+        self.file_error = defaults['file_error']
+        self.stringify = defaults['stringify']
+        self.indent_type = defaults['indent_type']
+        # used by the write method
+        self.BOM = None
+        #
+        self.initial_comment = []
+        self.final_comment = []
+        #
+        if isinstance(infile, StringTypes):
+            self.filename = os.path.abspath(infile)
+            if os.path.isfile(self.filename):
+                infile = open(self.filename).readlines()
+            elif self.file_error:
+                # raise an error if the file doesn't exist
+                raise IOError, 'Config file not found: "%s".' % self.filename
+            else:
+                # file doesn't already exist
+                if self.create_empty:
+                    # this is a good test that the filename specified
+                    # isn't impossible - like on a non existent device
+                    h = open(self.filename)
+                    h.write('')
+                    h.close()
+                infile = []
+        elif isinstance(infile, (list, tuple)):
+            self.filename = None
+        elif isinstance(infile, dict):
+            # initialise self
+            # the Section class handles creating subsections
+            if isinstance(infile, ConfigObj):
+                # get a copy of our ConfigObj
+                infile = infile.dict()
+            for entry in infile:
+                self[entry] = infile[entry]
+            self.filename = None
+            del self._errors
+            if defaults['configspec'] is not None:
+                self._handle_configspec(defaults['configspec'])
+            else:
+                self.configspec = None
+            return
+        elif hasattr(infile, 'seek'):
+            # this supports StringIO instances and even file objects
+            self.filename = infile
+            infile.seek(0)
+            infile = infile.readlines()
+            self.filename.seek(0)
+        else:
+            raise TypeError, ('infile must be a filename,'
+                ' StringIO instance, or a file as a list.')
+        #
+        # strip trailing '\n' from lines
+        infile = [line.rstrip('\n') for line in infile]
+        #
+        # remove the UTF8 BOM if it is there
+        # FIXME: support other BOM
+        if infile and infile[0].startswith(BOM_UTF8):
+            infile[0] = infile[0][3:]
+            self.BOM = BOM_UTF8
+        else:
+            self.BOM = None
+        #
+        self._parse(infile)
+        # if we had any errors, now is the time to raise them
+        if self._errors:
+            error = ConfigObjError("Parsing failed.")
+            # set the errors attribute; it's a list of tuples:
+            # (error_type, message, line_number)
+            error.errors = self._errors
+            # set the config attribute
+            error.config = self
+            raise error
+        # delete private attributes
+        del self._errors
+        #
+        if defaults['configspec'] is None:
+            self.configspec = None
+        else:
+            self._handle_configspec(defaults['configspec'])
+
+    def _parse(self, infile):
+        """
+        Actually parse the config file
+        
+        Testing Interpolation
+        
+        >>> c = ConfigObj()
+        >>> c['DEFAULT'] = {
+        ...     'b': 'goodbye',
+        ...     'userdir': 'c:\\\\home',
+        ...     'c': '%(d)s',
+        ...     'd': '%(c)s'
+        ... }
+        >>> c['section'] = {
+        ...     'a': '%(datadir)s\\\\some path\\\\file.py',
+        ...     'b': '%(userdir)s\\\\some path\\\\file.py',
+        ...     'c': 'Yo %(a)s',
+        ...     'd': '%(not_here)s',
+        ...     'e': '%(c)s',
+        ... }
+        >>> c['section']['DEFAULT'] = {
+        ...     'datadir': 'c:\\\\silly_test',
+        ...     'a': 'hello - %(b)s',
+        ... }
+        >>> c['section']['a'] == 'c:\\\\silly_test\\\\some path\\\\file.py'
+        1
+        >>> c['section']['b'] == 'c:\\\\home\\\\some path\\\\file.py'
+        1
+        >>> c['section']['c'] == 'Yo hello - goodbye'
+        1
+        
+        Switching Interpolation Off
+        
+        >>> c.interpolation = False
+        >>> c['section']['a'] == '%(datadir)s\\\\some path\\\\file.py'
+        1
+        >>> c['section']['b'] == '%(userdir)s\\\\some path\\\\file.py'
+        1
+        >>> c['section']['c'] == 'Yo %(a)s'
+        1
+        
+        Testing the interpolation errors.
+        
+        >>> c.interpolation = True
+        >>> c['section']['d']
+        Traceback (most recent call last):
+        MissingInterpolationOption: missing option "not_here" in interpolation.
+        >>> c['section']['e']
+        Traceback (most recent call last):
+        InterpolationDepthError: max interpolation depth exceeded in value "%(c)s".
+        
+        Testing our quoting.
+        
+        >>> i._quote('\"""\'\'\'')
+        Traceback (most recent call last):
+        SyntaxError: EOF while scanning triple-quoted string
+        >>> try:
+        ...     i._quote('\\n', multiline=False)
+        ... except ConfigObjError, e:
+        ...    e.msg
+        'Value "\\n" cannot be safely quoted.'
+        >>> k._quote(' "\' ', multiline=False)
+        Traceback (most recent call last):
+        SyntaxError: EOL while scanning single-quoted string
+        
+        Testing with "stringify" off.
+        >>> c.stringify = False
+        >>> c['test'] = 1
+        Traceback (most recent call last):
+        TypeError: Value is not a string "1".
+        """
+        comment_list = []
+        done_start = False
+        this_section = self
+        maxline = len(infile) - 1
+        cur_index = -1
+        reset_comment = False
+        while cur_index < maxline:
+            if reset_comment:
+                comment_list = []
+            cur_index += 1
+            line = infile[cur_index]
+            sline = line.strip()
+            # do we have anything on the line ?
+            if not sline or sline.startswith('#'):
+                reset_comment = False
+                comment_list.append(line)
+                continue
+            if not done_start:
+                # preserve initial comment
+                self.initial_comment = comment_list
+                comment_list = []
+                done_start = True
+            reset_comment = True
+            # first we check if it's a section marker
+            mat = self._sectionmarker.match(line)
+##            print >> sys.stderr, sline, mat
+            if mat is not None:
+                # is a section line
+                (indent, sect_open, sect_name, sect_close, comment) = (
+                    mat.groups())
+                if indent and (self.indent_type is None):
+                    self.indent_type = indent[0]
+                cur_depth = sect_open.count('[')
+                if cur_depth != sect_close.count(']'):
+                    self._handle_error(
+                        "Cannot compute the section depth at line %s.",
+                        NestingError, infile, cur_index)
+                    continue
+                if cur_depth < this_section.depth:
+                    # the new section is dropping back to a previous level
+                    try:
+                        parent = self._match_depth(
+                            this_section,
+                            cur_depth).parent
+                    except SyntaxError:
+                        self._handle_error(
+                            "Cannot compute nesting level at line %s.",
+                            NestingError, infile, cur_index)
+                        continue
+                elif cur_depth == this_section.depth:
+                    # the new section is a sibling of the current section
+                    parent = this_section.parent
+                elif cur_depth == this_section.depth + 1:
+                    # the new section is a child the current section
+                    parent = this_section
+                else:
+                    self._handle_error(
+                        "Section too nested at line %s.",
+                        NestingError, infile, cur_index)
+                #
+                sect_name = self._unquote(sect_name)
+                if parent.has_key(sect_name):
+##                    print >> sys.stderr, sect_name
+                    self._handle_error(
+                        'Duplicate section name at line %s.',
+                        DuplicateError, infile, cur_index)
+                    continue
+                # create the new section
+                this_section = Section(
+                    parent,
+                    cur_depth,
+                    self,
+                    name=sect_name)
+                parent[sect_name] = this_section
+                parent.inline_comments[sect_name] = comment
+                parent.comments[sect_name] = comment_list
+##                print >> sys.stderr, parent[sect_name] is this_section
+                continue
+            #
+            # it's not a section marker,
+            # so it should be a valid ``key = value`` line
+            mat = self._keyword.match(line)
+##            print >> sys.stderr, sline, mat
+            if mat is not None:
+                # is a keyword value
+                # value will include any inline comment
+                (indent, key, value) = mat.groups()
+                if indent and (self.indent_type is None):
+                    self.indent_type = indent[0]
+                # check for a multiline value
+                if value[:3] in ['"""', "'''"]:
+                    try:
+                        (value, comment, cur_index) = self._multiline(
+                            value, infile, cur_index, maxline)
+                    except SyntaxError:
+                        self._handle_error(
+                            'Parse error in value at line %s.',
+                            ParseError, infile, cur_index)
+                        continue
+                else:
+                    # extract comment and lists
+                    try:
+                        (value, comment) = self._handle_value(value)
+                    except SyntaxError:
+                        self._handle_error(
+                            'Parse error in value at line %s.',
+                            ParseError, infile, cur_index)
+                        continue
+                #
+##                print >> sys.stderr, sline
+                key = self._unquote(key)
+                if this_section.has_key(key):
+                    self._handle_error(
+                        'Duplicate keyword name at line %s.',
+                        DuplicateError, infile, cur_index)
+                    continue
+                # add the key
+##                print >> sys.stderr, this_section.name
+                this_section[key] = value
+                this_section.inline_comments[key] = comment
+                this_section.comments[key] = comment_list
+##                print >> sys.stderr, key, this_section[key]
+##                if this_section.name is not None:
+##                    print >> sys.stderr, this_section
+##                    print >> sys.stderr, this_section.parent
+##                    print >> sys.stderr, this_section.parent[this_section.name]
+                continue
+            #
+            # it neither matched as a keyword
+            # or a section marker
+            self._handle_error(
+                'Invalid line at line "%s".',
+                ParseError, infile, cur_index)
+        if self.indent_type is None:
+            # no indentation used, set the type accordingly
+            self.indent_type = ''
+        # preserve the final comment
+        self.final_comment = comment_list
+
+    def _match_depth(self, sect, depth):
+        """
+        Given a section and a depth level, walk back through the sections
+        parents to see if the depth level matches a previous section.
+        
+        Return a reference to the right section,
+        or raise a SyntaxError.
+        """
+        while depth < sect.depth:
+            if sect is sect.parent:
+                # we've reached the top level already
+                raise SyntaxError
+            sect = sect.parent
+        if sect.depth == depth:
+            return sect
+        # shouldn't get here
+        raise SyntaxError
+
+    def _handle_error(self, text, ErrorClass, infile, cur_index):
+        """
+        Handle an error according to the error settings.
+        
+        Either raise the error or store it.
+        The error will have occured at ``cur_index``
+        """
+        line = infile[cur_index]
+        message = text % cur_index
+        error = ErrorClass(message, cur_index, line)
+        if self.raise_errors:
+            # raise the error - parsing stops here
+            raise error
+        # store the error
+        # reraise when parsing has finished
+        self._errors.append(error)
+
+    def _unquote(self, value):
+        """Return an unquoted version of a value"""
+        if (value[0] == value[-1]) and (value[0] in ('"', "'")):
+            value = value[1:-1]
+        return value
+
+    def _quote(self, value, multiline=True):
+        """
+        Return a safely quoted version of a value.
+        
+        Raise a ConfigObjError if the value cannot be safely quoted.
+        If multiline is ``True`` (default) then use triple quotes
+        if necessary.
+        
+        Don't quote values that don't need it.
+        Recursively quote members of a list and return a comma joined list.
+        Multiline is ``False`` for lists.
+        Obey list syntax for empty and single member lists.
+        """
+        if isinstance(value, (list, tuple)):
+            if not value:
+                return ','
+            elif len(value) == 1:
+                return self._quote(value[0], multiline=False) + ','
+            return ','.join([self._quote(val, multiline=False)
+                for val in value])
+        if not isinstance(value, StringTypes):
+            if self.stringify:
+                value = str(value)
+            else:
+                raise TypeError, 'Value "%s" is not a string.' % value
+        squot = "'%s'"
+        dquot = '"%s"'
+        noquot = "%s"
+        wspace_plus = ' \r\t\n\v\t\'"'
+        tsquot = '"""%s"""'
+        tdquot = "'''%s'''"
+        if not value:
+            return '""'
+        if not (multiline and
+                ((("'" in value) and ('"' in value)) or ('\n' in value))):
+            # for normal values either single or double quotes will do
+            if '\n' in value:
+                raise ConfigObjError, ('Value "%s" cannot be safely quoted.' %
+                    value)
+            if ((value[0] not in wspace_plus) and
+                    (value[-1] not in wspace_plus) and
+                    (',' not in value)):
+                quot = noquot
+            else:
+                if ("'" in value) and ('"' in value):
+                    raise ConfigObjError, (
+                        'Value "%s" cannot be safely quoted.' % value)
+                elif '"' in value:
+                    quot = squot
+                else:
+                    quot = dquot
+        else:
+            # if value has '\n' or "'" *and* '"', it will need triple quotes
+            if (value.find('"""') != -1) and (value.find("'''") != -1):
+                raise ConfigObjError, (
+                    'Value "%s" cannot be safely quoted.' % value)
+            if value.find('"""') == -1:
+                quot = tdquot
+            else:
+                quot = tsquot
+        return quot % value
+
+    def _handle_value(self, value):
+        """
+        Given a value string, unquote, remove comment,
+        handle lists. (including empty and single member lists)
+        
+        Testing list values.
+        
+        >>> testconfig3 = '''
+        ... a = ,
+        ... b = test,
+        ... c = test1, test2   , test3
+        ... d = test1, test2, test3,
+        ... '''
+        >>> d = ConfigObj(testconfig3.split('\\n'), raise_errors=True)
+        >>> d['a'] == []
+        1
+        >>> d['b'] == ['test']
+        1
+        >>> d['c'] == ['test1', 'test2', 'test3']
+        1
+        >>> d['d'] == ['test1', 'test2', 'test3']
+        1
+        
+        Testing with list values off.
+        
+        >>> e = ConfigObj(
+        ...     testconfig3.split('\\n'),
+        ...     raise_errors=True,
+        ...     list_values=False)
+        >>> e['a'] == ','
+        1
+        >>> e['b'] == 'test,'
+        1
+        >>> e['c'] == 'test1, test2   , test3'
+        1
+        >>> e['d'] == 'test1, test2, test3,'
+        1
+        
+        Testing creating from a dictionary.
+        
+        >>> f = {
+        ...     'key1': 'val1',
+        ...     'key2': 'val2',
+        ...     'section 1': {
+        ...         'key1': 'val1',
+        ...         'key2': 'val2',
+        ...         'section 1b': {
+        ...             'key1': 'val1',
+        ...             'key2': 'val2',
+        ...         },
+        ...     },
+        ...     'section 2': {
+        ...         'key1': 'val1',
+        ...         'key2': 'val2',
+        ...         'section 2b': {
+        ...             'key1': 'val1',
+        ...             'key2': 'val2',
+        ...         },
+        ...     },
+        ...      'key3': 'val3',
+        ... }
+        >>> g = ConfigObj(f)
+        >>> f == g
+        1
+        
+        Testing we correctly detect badly built list values (4 of them).
+        
+        >>> testconfig4 = '''
+        ... config = 3,4,,
+        ... test = 3,,4
+        ... fish = ,,
+        ... dummy = ,,hello, goodbye
+        ... '''
+        >>> try:
+        ...     ConfigObj(testconfig4.split('\\n'))
+        ... except ConfigObjError, e:
+        ...     len(e.errors)
+        4
+        
+        Testing we correctly detect badly quoted values (4 of them).
+        
+        >>> testconfig5 = '''
+        ... config = "hello   # comment
+        ... test = 'goodbye
+        ... fish = 'goodbye   # comment
+        ... dummy = "hello again
+        ... '''
+        >>> try:
+        ...     ConfigObj(testconfig5.split('\\n'))
+        ... except ConfigObjError, e:
+        ...     len(e.errors)
+        4
+        """
+        # do we look for lists in values ?
+        if not self.list_values:
+            mat = self._nolistvalue.match(value)
+            if mat is None:
+                raise SyntaxError
+            (value, comment) = mat.groups()
+            # FIXME: unquoting here can be a source of error
+            return (self._unquote(value), comment)
+        mat = self._valueexp.match(value)
+        if mat is None:
+            # the value is badly constructed, probably badly quoted,
+            # or an invalid list
+            raise SyntaxError
+        (list_values, single, empty_list, comment) = mat.groups()
+        if (list_values == '') and (single is None):
+            # change this if you want to accept empty values
+            raise SyntaxError
+        # NOTE: note there is no error handling from here if the regex
+        # is wrong: then incorrect values will slip through
+        if empty_list is not None:
+            # the single comma - meaning an empty list
+            return ([], comment)
+        if single is not None:
+            single = self._unquote(single)
+        if list_values == '':
+            # not a list value
+            return (single, comment)
+        the_list = self._listvalueexp.findall(list_values)
+        the_list = [self._unquote(val) for val in the_list]
+        if single is not None:
+            the_list += [single]
+        return (the_list, comment)
+
+    def _multiline(self, value, infile, cur_index, maxline):
+        """
+        Extract the value, where we are in a multiline situation
+        
+        Testing multiline values.
+        
+        >>> i == {
+        ...     'name4': ' another single line value ',
+        ...     'multi section': {
+        ...         'name4': '\\n        Well, this is a\\n        multiline '
+        ...             'value\\n        ',
+        ...         'name2': '\\n        Well, this is a\\n        multiline '
+        ...             'value\\n        ',
+        ...         'name3': '\\n        Well, this is a\\n        multiline '
+        ...             'value\\n        ',
+        ...         'name1': '\\n        Well, this is a\\n        multiline '
+        ...             'value\\n        ',
+        ...     },
+        ...     'name2': ' another single line value ',
+        ...     'name3': ' a single line value ',
+        ...     'name1': ' a single line value ',
+        ... }
+        1
+        """
+        quot = value[:3]
+        newvalue = value[3:]
+        single_line = self._triple_quote[quot][0]
+        multi_line = self._triple_quote[quot][1]
+        mat = single_line.match(value)
+        if mat is not None:
+            retval = list(mat.groups())
+            retval.append(cur_index)
+            return retval
+        elif newvalue.find(quot) != -1:
+            # somehow the triple quote is missing
+            raise SyntaxError
+        #
+        while cur_index < maxline:
+            cur_index += 1
+            newvalue += '\n'
+            line = infile[cur_index]
+            if line.find(quot) == -1:
+                newvalue += line
+            else:
+                # end of multiline, process it
+                break
+        else:
+            # we've got to the end of the config, oops...
+            raise SyntaxError
+        mat = multi_line.match(line)
+        if mat is None:
+            # a badly formed line
+            raise SyntaxError
+        (value, comment) = mat.groups()
+        return (newvalue + value, comment, cur_index)
+
+    def _handle_configspec(self, configspec):
+        """Parse the configspec."""
+        try:
+            configspec = ConfigObj(
+                configspec,
+                raise_errors=True,
+                file_error=True,
+                list_values=False)
+        except ConfigObjError, e:
+            # FIXME: Should these errors have a reference
+            # to the already parsed ConfigObj ?
+            raise ConfigspecError('Parsing configspec failed: %s' % e)
+        except IOError, e:
+            raise IOError('Reading configspec failed: %s' % e)
+        self._set_configspec_value(configspec, self)
+
+    def _set_configspec_value(self, configspec, section):
+        """Used to recursively set configspec values."""
+        if '__many__' in configspec.sections:
+            section.configspec['__many__'] = configspec['__many__']
+            if len(configspec.sections) > 1:
+                # FIXME: can we supply any useful information here ?
+                raise RepeatSectionError
+        for entry in configspec.scalars:
+            section.configspec[entry] = configspec[entry]
+        for entry in configspec.sections:
+            if entry == '__many__':
+                continue
+            if not section.has_key(entry):
+                section[entry] = {}
+            self._set_configspec_value(configspec[entry], section[entry])
+
+    def _handle_repeat(self, section, configspec):
+        """Dynamically assign configspec for repeated section."""
+        try:
+            section_keys = configspec.sections
+            scalar_keys = configspec.scalars
+        except AttributeError:
+            section_keys = [entry for entry in configspec 
+                                if isinstance(configspec[entry], dict)]
+            scalar_keys = [entry for entry in configspec 
+                                if not isinstance(configspec[entry], dict)]
+        if '__many__' in section_keys and len(section_keys) > 1:
+            # FIXME: can we supply any useful information here ?
+            raise RepeatSectionError
+        scalars = {}
+        sections = {}
+        for entry in scalar_keys:
+            val = configspec[entry]
+            scalars[entry] = val
+        for entry in section_keys:
+            val = configspec[entry]
+            if entry == '__many__':
+                scalars[entry] = val
+                continue
+            sections[entry] = val
+        #
+        section.configspec = scalars
+        for entry in sections:
+            if not section.has_key(entry):
+                section[entry] = {}
+            self._handle_repeat(section[entry], sections[entry])
+
+    def _write_line(self, indent_string, entry, this_entry, comment):
+        """Write an individual line, for the write method"""
+        return '%s%s = %s%s' % (
+            indent_string,
+            self._quote(entry, multiline=False),
+            self._quote(this_entry),
+            comment)
+
+    def _write_marker(self, indent_string, depth, entry, comment):
+        """Write a section marker line"""
+        return '%s%s%s%s%s' % (
+            indent_string,
+            '[' * depth,
+            self._quote(entry, multiline=False),
+            ']' * depth,
+            comment)
+
+    def _handle_comment(self, comment):
+        """
+        Deal with a comment.
+        
+        >>> filename = a.filename
+        >>> a.filename = None
+        >>> values = a.write()
+        >>> index = 0
+        >>> while index < 23:
+        ...     index += 1
+        ...     line = values[index-1]
+        ...     assert line.endswith('# comment ' + str(index))
+        >>> a.filename = filename
+        
+        >>> start_comment = ['# Initial Comment', '', '#']
+        >>> end_comment = ['', '#', '# Final Comment']
+        >>> newconfig = start_comment + testconfig1.split('\\n') + end_comment
+        >>> nc = ConfigObj(newconfig)
+        >>> nc.initial_comment
+        ['# Initial Comment', '', '#']
+        >>> nc.final_comment
+        ['', '#', '# Final Comment']
+        >>> nc.initial_comment == start_comment
+        1
+        >>> nc.final_comment == end_comment
+        1
+        """
+        if not comment:
+            return ''
+        if self.indent_type == '\t':
+            start = '\t'
+        else:
+            start = ' ' * NUM_INDENT_SPACES
+        if not comment.startswith('#'):
+            start += '# '
+        return (start + comment)
+
+    def _compute_indent_string(self, depth):
+        """
+        Compute the indent string, according to current indent_type and depth
+        """
+        if self.indent_type == '':
+            # no indentation at all
+            return ''
+        if self.indent_type == '\t':
+            return '\t' * depth
+        if self.indent_type == ' ':
+            return ' ' * NUM_INDENT_SPACES * depth
+        raise SyntaxError
+
+    # Public methods
+
+    def write(self, section=None):
+        """
+        Write the current ConfigObj as a file
+        
+        tekNico: FIXME: use StringIO instead of real files
+        
+        >>> filename = a.filename
+        >>> a.filename = 'test.ini'
+        >>> a.write()
+        >>> a.filename = filename
+        >>> a == ConfigObj('test.ini', raise_errors=True)
+        1
+        >>> os.remove('test.ini')
+        >>> b.filename = 'test.ini'
+        >>> b.write()
+        >>> b == ConfigObj('test.ini', raise_errors=True)
+        1
+        >>> os.remove('test.ini')
+        >>> i.filename = 'test.ini'
+        >>> i.write()
+        >>> i == ConfigObj('test.ini', raise_errors=True)
+        1
+        >>> os.remove('test.ini')
+        >>> a = ConfigObj()
+        >>> a['DEFAULT'] = {'a' : 'fish'}
+        >>> a['a'] = '%(a)s'
+        >>> a.write()
+        ['a = %(a)s', '[DEFAULT]', 'a = fish']
+        """
+        int_val = 'test'
+        if self.indent_type is None:
+            # this can be true if initialised from a dictionary
+            self.indent_type = DEFAULT_INDENT_TYPE
+        #
+        out = []
+        return_list = True
+        if section is None:
+            int_val = self.interpolation
+            self.interpolation = False
+            section = self
+            return_list = False
+            for line in self.initial_comment:
+                stripped_line = line.strip()
+                if stripped_line and not stripped_line.startswith('#'):
+                    line = '# ' + line
+                out.append(line)
+        #
+        indent_string = self._compute_indent_string(section.depth)
+        for entry in (section.scalars + section.sections):
+            if entry in section.defaults:
+                # don't write out default values
+                continue
+            for comment_line in section.comments[entry]:
+                comment_line = comment_line.lstrip()
+                if comment_line and not comment_line.startswith('#'):
+                    comment_line = '#' + comment_line
+                out.append(indent_string + comment_line)
+            this_entry = section[entry]
+            comment = self._handle_comment(section.inline_comments[entry])
+            #
+            if isinstance(this_entry, dict):
+                # a section
+                out.append(self._write_marker(
+                    indent_string,
+                    this_entry.depth,
+                    entry,
+                    comment))
+                out.extend(self.write(this_entry))
+            else:
+                out.append(self._write_line(
+                    indent_string,
+                    entry,
+                    this_entry,
+                    comment))
+        #
+        if not return_list:
+            for line in self.final_comment:
+                stripped_line = line.strip()
+                if stripped_line and not stripped_line.startswith('#'):
+                    line = '# ' + line
+                out.append(line)
+        #
+        if int_val != 'test':
+            self.interpolation = int_val
+        #
+        if (return_list) or (self.filename is None):
+            return out
+        #
+        if isinstance(self.filename, StringTypes):
+            h = open(self.filename, 'w')
+            h.write(self.BOM or '')
+            h.write('\n'.join(out))
+            h.close()
+        else:
+            self.filename.seek(0)
+            self.filename.write(self.BOM or '')
+            self.filename.write('\n'.join(out))
+            # if we have a stored file object (or StringIO)
+            # we *don't* close it
+
+    def validate(self, validator, section=None):
+        """
+        Test the ConfigObj against a configspec.
+        
+        It uses the ``validator`` object from *validate.py*.
+        
+        To run ``validate`` on the current ConfigObj, call: ::
+        
+            test = config.validate(validator)
+        
+        (Normally having previously passed in the configspec when the ConfigObj
+        was created - you can dynamically assign a dictionary of checks to the
+        ``configspec`` attribute of a section though).
+        
+        It returns ``True`` if everything passes, or a dictionary of
+        pass/fails (True/False). If every member of a subsection passes, it
+        will just have the value ``True``. (It also returns ``False`` if all
+        members fail).
+        
+        In addition, it converts the values from strings to their native
+        types if their checks pass (and ``stringify`` is set).
+        
+        >>> try:
+        ...     from validate import Validator
+        ... except ImportError:
+        ...     print >> sys.stderr, 'Cannot import the Validator object, skipping the realted tests'
+        ... else:
+        ...     config = '''
+        ...     test1=40
+        ...     test2=hello
+        ...     test3=3
+        ...     test4=5.0
+        ...     [section]
+        ...         test1=40
+        ...         test2=hello
+        ...         test3=3
+        ...         test4=5.0
+        ...         [[sub section]]
+        ...             test1=40
+        ...             test2=hello
+        ...             test3=3
+        ...             test4=5.0
+        ... '''.split('\\n')
+        ...     configspec = '''
+        ...     test1='integer(30,50)'
+        ...     test2='string'
+        ...     test3='integer'
+        ...     test4='float(6.0)'
+        ...     [section ]
+        ...         test1='integer(30,50)'
+        ...         test2='string'
+        ...         test3='integer'
+        ...         test4='float(6.0)'
+        ...         [[sub section]]
+        ...             test1='integer(30,50)'
+        ...             test2='string'
+        ...             test3='integer'
+        ...             test4='float(6.0)'
+        ...     '''.split('\\n')
+        ...     val = Validator()
+        ...     c1 = ConfigObj(config, configspec=configspec)
+        ...     test = c1.validate(val)
+        ...     test == {
+        ...         'test1': True,
+        ...         'test2': True,
+        ...         'test3': True,
+        ...         'test4': False,
+        ...         'section': {
+        ...             'test1': True,
+        ...             'test2': True,
+        ...             'test3': True,
+        ...             'test4': False,
+        ...             'sub section': {
+        ...                 'test1': True,
+        ...                 'test2': True,
+        ...                 'test3': True,
+        ...                 'test4': False,
+        ...             },
+        ...         },
+        ...     }
+        1
+        >>> val.check(c1.configspec['test4'], c1['test4'])
+        Traceback (most recent call last):
+        VdtValueTooSmallError: the value "5.0" is too small.
+        
+        >>> val_test_config = '''
+        ...     key = 0
+        ...     key2 = 1.1
+        ...     [section]
+        ...     key = some text
+        ...     key2 = 1.1, 3.0, 17, 6.8
+        ...         [[sub-section]]
+        ...         key = option1
+        ...         key2 = True'''.split('\\n')
+        >>> val_test_configspec = '''
+        ...     key = integer
+        ...     key2 = float
+        ...     [section]
+        ...     key = string
+        ...     key2 = float_list(4)
+        ...        [[sub-section]]
+        ...        key = option(option1, option2)
+        ...        key2 = boolean'''.split('\\n')
+        >>> val_test = ConfigObj(val_test_config, configspec=val_test_configspec)
+        >>> val_test.validate(val)
+        1
+        >>> val_test['key'] = 'text not a digit'
+        >>> val_res = val_test.validate(val)
+        >>> val_res == {'key2': True, 'section': True, 'key': False}
+        1
+        >>> configspec = '''
+        ...     test1='integer(30,50, default=40)'
+        ...     test2='string(default="hello")'
+        ...     test3='integer(default=3)'
+        ...     test4='float(6.0, default=6.0)'
+        ...     [section ]
+        ...         test1='integer(30,50, default=40)'
+        ...         test2='string(default="hello")'
+        ...         test3='integer(default=3)'
+        ...         test4='float(6.0, default=6.0)'
+        ...         [[sub section]]
+        ...             test1='integer(30,50, default=40)'
+        ...             test2='string(default="hello")'
+        ...             test3='integer(default=3)'
+        ...             test4='float(6.0, default=6.0)'
+        ...     '''.split('\\n')
+        >>> default_test = ConfigObj(['test1=30'], configspec=configspec)
+        >>> default_test
+        {'test1': '30', 'section': {'sub section': {}}}
+        >>> default_test.validate(val)
+        1
+        >>> default_test == {
+        ...     'test1': 30,
+        ...     'test2': 'hello',
+        ...     'test3': 3,
+        ...     'test4': 6.0,
+        ...     'section': {
+        ...         'test1': 40,
+        ...         'test2': 'hello',
+        ...         'test3': 3,
+        ...         'test4': 6.0,
+        ...         'sub section': {
+        ...             'test1': 40,
+        ...             'test3': 3,
+        ...             'test2': 'hello',
+        ...             'test4': 6.0,
+        ...         },
+        ...     },
+        ... }
+        1
+        
+        Now testing with repeated sections : BIG TEST
+        
+        >>> repeated_1 = '''
+        ... [dogs]
+        ...     [[__many__]] # spec for a dog
+        ...         fleas = boolean(default=True)
+        ...         tail = option(long, short, default=long)
+        ...         name = string(default=rover)
+        ...         [[[__many__]]]  # spec for a puppy
+        ...             name = string(default="son of rover")
+        ...             age = float(default=0.0)
+        ... [cats]
+        ...     [[__many__]] # spec for a cat
+        ...         fleas = boolean(default=True)
+        ...         tail = option(long, short, default=short)
+        ...         name = string(default=pussy)
+        ...         [[[__many__]]] # spec for a kitten
+        ...             name = string(default="son of pussy")
+        ...             age = float(default=0.0)
+        ...         '''.split('\\n')
+        >>> repeated_2 = '''
+        ... [dogs]
+        ... 
+        ...     # blank dogs with puppies
+        ...     # should be filled in by the configspec
+        ...     [[dog1]]
+        ...         [[[puppy1]]]
+        ...         [[[puppy2]]]
+        ...         [[[puppy3]]]
+        ...     [[dog2]]
+        ...         [[[puppy1]]]
+        ...         [[[puppy2]]]
+        ...         [[[puppy3]]]
+        ...     [[dog3]]
+        ...         [[[puppy1]]]
+        ...         [[[puppy2]]]
+        ...         [[[puppy3]]]
+        ... [cats]
+        ... 
+        ...     # blank cats with kittens
+        ...     # should be filled in by the configspec
+        ...     [[cat1]]
+        ...         [[[kitten1]]]
+        ...         [[[kitten2]]]
+        ...         [[[kitten3]]]
+        ...     [[cat2]]
+        ...         [[[kitten1]]]
+        ...         [[[kitten2]]]
+        ...         [[[kitten3]]]
+        ...     [[cat3]]
+        ...         [[[kitten1]]]
+        ...         [[[kitten2]]]
+        ...         [[[kitten3]]]
+        ... '''.split('\\n')
+        >>> repeated_3 = '''
+        ... [dogs]
+        ... 
+        ...     [[dog1]]
+        ...     [[dog2]]
+        ...     [[dog3]]
+        ... [cats]
+        ... 
+        ...     [[cat1]]
+        ...     [[cat2]]
+        ...     [[cat3]]
+        ... '''.split('\\n')
+        >>> repeated_4 = '''
+        ... [__many__]
+        ... 
+        ...     name = string(default=Michael)
+        ...     age = float(default=0.0)
+        ...     sex = option(m, f, default=m)
+        ... '''.split('\\n')
+        >>> repeated_5 = '''
+        ... [cats]
+        ... [[__many__]]
+        ...     fleas = boolean(default=True)
+        ...     tail = option(long, short, default=short)
+        ...     name = string(default=pussy)
+        ...     [[[description]]]
+        ...         height = float(default=3.3)
+        ...         weight = float(default=6)
+        ...         [[[[coat]]]]
+        ...             fur = option(black, grey, brown, "tortoise shell", default=black)
+        ...             condition = integer(0,10, default=5)
+        ... '''.split('\\n')
+        >>> from validate import Validator
+        >>> val= Validator()
+        >>> repeater = ConfigObj(repeated_2, configspec=repeated_1)
+        >>> repeater.validate(val)
+        1
+        >>> repeater == {
+        ...     'dogs': {
+        ...         'dog1': {
+        ...             'fleas': True,
+        ...             'tail': 'long',
+        ...             'name': 'rover',
+        ...             'puppy1': {'name': 'son of rover', 'age': 0.0},
+        ...             'puppy2': {'name': 'son of rover', 'age': 0.0},
+        ...             'puppy3': {'name': 'son of rover', 'age': 0.0},
+        ...         },
+        ...         'dog2': {
+        ...             'fleas': True,
+        ...             'tail': 'long',
+        ...             'name': 'rover',
+        ...             'puppy1': {'name': 'son of rover', 'age': 0.0},
+        ...             'puppy2': {'name': 'son of rover', 'age': 0.0},
+        ...             'puppy3': {'name': 'son of rover', 'age': 0.0},
+        ...         },
+        ...         'dog3': {
+        ...             'fleas': True,
+        ...             'tail': 'long',
+        ...             'name': 'rover',
+        ...             'puppy1': {'name': 'son of rover', 'age': 0.0},
+        ...             'puppy2': {'name': 'son of rover', 'age': 0.0},
+        ...             'puppy3': {'name': 'son of rover', 'age': 0.0},
+        ...         },
+        ...     },
+        ...     'cats': {
+        ...         'cat1': {
+        ...             'fleas': True,
+        ...             'tail': 'short',
+        ...             'name': 'pussy',
+        ...             'kitten1': {'name': 'son of pussy', 'age': 0.0},
+        ...             'kitten2': {'name': 'son of pussy', 'age': 0.0},
+        ...             'kitten3': {'name': 'son of pussy', 'age': 0.0},
+        ...         },
+        ...         'cat2': {
+        ...             'fleas': True,
+        ...             'tail': 'short',
+        ...             'name': 'pussy',
+        ...             'kitten1': {'name': 'son of pussy', 'age': 0.0},
+        ...             'kitten2': {'name': 'son of pussy', 'age': 0.0},
+        ...             'kitten3': {'name': 'son of pussy', 'age': 0.0},
+        ...         },
+        ...         'cat3': {
+        ...             'fleas': True,
+        ...             'tail': 'short',
+        ...             'name': 'pussy',
+        ...             'kitten1': {'name': 'son of pussy', 'age': 0.0},
+        ...             'kitten2': {'name': 'son of pussy', 'age': 0.0},
+        ...             'kitten3': {'name': 'son of pussy', 'age': 0.0},
+        ...         },
+        ...     },
+        ... }
+        1
+        >>> repeater = ConfigObj(repeated_3, configspec=repeated_1)
+        >>> repeater.validate(val)
+        1
+        >>> repeater == {
+        ...     'cats': {
+        ...         'cat1': {'fleas': True, 'tail': 'short', 'name': 'pussy'},
+        ...         'cat2': {'fleas': True, 'tail': 'short', 'name': 'pussy'},
+        ...         'cat3': {'fleas': True, 'tail': 'short', 'name': 'pussy'},
+        ...     },
+        ...     'dogs': {
+        ...         'dog1': {'fleas': True, 'tail': 'long', 'name': 'rover'},
+        ...         'dog2': {'fleas': True, 'tail': 'long', 'name': 'rover'},
+        ...         'dog3': {'fleas': True, 'tail': 'long', 'name': 'rover'},
+        ...     },
+        ... }
+        1
+        >>> repeater = ConfigObj(configspec=repeated_4)
+        >>> repeater['Michael'] = {}
+        >>> repeater.validate(val)
+        1
+        >>> repeater == {
+        ...     'Michael': {'age': 0.0, 'name': 'Michael', 'sex': 'm'},
+        ... }
+        1
+        >>> repeater = ConfigObj(repeated_3, configspec=repeated_5)
+        >>> repeater == {
+        ...     'dogs': {'dog1': {}, 'dog2': {}, 'dog3': {}},
+        ...     'cats': {'cat1': {}, 'cat2': {}, 'cat3': {}},
+        ... }
+        1
+        >>> repeater.validate(val)
+        1
+        >>> repeater == {
+        ...     'dogs': {'dog1': {}, 'dog2': {}, 'dog3': {}},
+        ...     'cats': {
+        ...         'cat1': {
+        ...             'fleas': True,
+        ...             'tail': 'short',
+        ...             'name': 'pussy',
+        ...             'description': {
+        ...                 'weight': 6.0,
+        ...                 'height': 3.2999999999999998,
+        ...                 'coat': {'fur': 'black', 'condition': 5},
+        ...             },
+        ...         },
+        ...         'cat2': {
+        ...             'fleas': True,
+        ...             'tail': 'short',
+        ...             'name': 'pussy',
+        ...             'description': {
+        ...                 'weight': 6.0,
+        ...                 'height': 3.2999999999999998,
+        ...                 'coat': {'fur': 'black', 'condition': 5},
+        ...             },
+        ...         },
+        ...         'cat3': {
+        ...             'fleas': True,
+        ...             'tail': 'short',
+        ...             'name': 'pussy',
+        ...             'description': {
+        ...                 'weight': 6.0,
+        ...                 'height': 3.2999999999999998,
+        ...                 'coat': {'fur': 'black', 'condition': 5},
+        ...             },
+        ...         },
+        ...     },
+        ... }
+        1
+        
+        Test that interpolation is preserved for validated string values.
+        >>> t = ConfigObj()
+        >>> t['DEFAULT'] = {}
+        >>> t['DEFAULT']['test'] = 'a'
+        >>> t['test'] = '%(test)s'
+        >>> t['test']
+        'a'
+        >>> v = Validator()
+        >>> t.configspec = {'test': 'string'}
+        >>> t.validate(v)
+        1
+        >>> t.interpolation = False
+        >>> t
+        {'test': '%(test)s', 'DEFAULT': {'test': 'a'}}
+        
+        FIXME: Above tests will fail if we couldn't import Validator (the ones
+        that don't raise errors will produce different output and still fail as
+        tests)
+        """
+        if section is None:
+            if self.configspec is None:
+                raise ValueError, 'No configspec supplied.'
+            section = self
+        #
+        spec_section = section.configspec
+        if '__many__' in section.configspec:
+            many = spec_section['__many__']
+            # dynamically assign the configspecs
+            # for the sections below
+            for entry in section.sections:
+                self._handle_repeat(section[entry], many)
+        #
+        out = {}
+        ret_true = True
+        ret_false = True
+        for entry in spec_section:
+            if entry == '__many__':
+                continue
+            if (not entry in section.scalars) or (entry in section.defaults):
+                # missing entries
+                # or entries from defaults
+                missing = True
+                val = None
+            else:
+                missing = False
+                val = section[entry]
+            try:
+                check = validator.check(spec_section[entry],
+                                        val,
+                                        missing=missing)
+            except validator.baseErrorClass:
+                out[entry] = False
+                ret_true = False
+            # MIKE: we want to raise all other exceptions, not just print ?
+##            except Exception, err:
+##                print err
+            else:
+                ret_false = False
+                out[entry] = True
+                if self.stringify or missing:
+                    # if we are doing type conversion
+                    # or the value is a supplied default
+                    if not self.stringify:
+                        if isinstance(check, (list, tuple)):
+                            # preserve lists
+                            check = [str(item) for item in check]
+                        elif missing and check is None:
+                            # convert the None from a default to a ''
+                            check = ''
+                        else:
+                            check = str(check)
+                    if (check != val) or missing:
+                        section[entry] = check
+                if missing and entry not in section.defaults:
+                    section.defaults.append(entry)
+        #
+        for entry in section.sections:
+            check = self.validate(validator, section[entry])
+            out[entry] = check
+            if check == False:
+                ret_true = False
+            elif check == True:
+                ret_false = False
+            else:
+                ret_true = False
+                ret_false = False
+        #
+        if ret_true:
+            return True
+        elif ret_false:
+            return False
+        else:
+            return out
+
+class SimpleVal(object):
+    """
+    A simple validator.
+    Can be used to check that all members expected are present.
+    
+    To use it, provide a configspec with all your members in (the value given
+    will be ignored). Pass an instance of ``SimpleVal`` to the ``validate``
+    method of your ``ConfigObj``. ``validate`` will return ``True`` if all
+    members are present, or a dictionary with True/False meaning
+    present/missing. (Whole missing sections will be replaced with ``False``)
+    
+    >>> val = SimpleVal()
+    >>> config = '''
+    ... test1=40
+    ... test2=hello
+    ... test3=3
+    ... test4=5.0
+    ... [section]
+    ... test1=40
+    ... test2=hello
+    ... test3=3
+    ... test4=5.0
+    ...     [[sub section]]
+    ...     test1=40
+    ...     test2=hello
+    ...     test3=3
+    ...     test4=5.0
+    ... '''.split('\\n')
+    >>> configspec = '''
+    ... test1=''
+    ... test2=''
+    ... test3=''
+    ... test4=''
+    ... [section]
+    ... test1=''
+    ... test2=''
+    ... test3=''
+    ... test4=''
+    ...     [[sub section]]
+    ...     test1=''
+    ...     test2=''
+    ...     test3=''
+    ...     test4=''
+    ... '''.split('\\n')
+    >>> o = ConfigObj(config, configspec=configspec)
+    >>> o.validate(val)
+    1
+    >>> o = ConfigObj(configspec=configspec)
+    >>> o.validate(val)
+    0
+    """
+    
+    def __init__(self):
+        self.baseErrorClass = ConfigObjError
+    
+    def check(self, check, member, missing=False):
+        """A dummy check method, always returns the value unchanged."""
+        if missing:
+            raise self.baseErrorClass
+        return member
+
+# FIXME: test error code for badly built multiline values
+# FIXME: test handling of StringIO
+# FIXME: test interpolation with writing
+
+def _doctest():
+    """
+    Dummy function to hold some of the doctests.
+    
+    >>> a.depth
+    0
+    >>> a == {
+    ...     'key2': 'val',
+    ...     'key1': 'val',
+    ...     'lev1c': {
+    ...         'lev2c': {
+    ...             'lev3c': {
+    ...                 'key1': 'val',
+    ...             },
+    ...         },
+    ...     },
+    ...     'lev1b': {
+    ...         'key2': 'val',
+    ...         'key1': 'val',
+    ...         'lev2ba': {
+    ...             'key1': 'val',
+    ...         },
+    ...         'lev2bb': {
+    ...             'key1': 'val',
+    ...         },
+    ...     },
+    ...     'lev1a': {
+    ...         'key2': 'val',
+    ...         'key1': 'val',
+    ...     },
+    ... }
+    1
+    >>> b.depth
+    0
+    >>> b == {
+    ...     'key3': 'val3',
+    ...     'key2': 'val2',
+    ...     'key1': 'val1',
+    ...     'section 1': {
+    ...         'keys11': 'val1',
+    ...         'keys13': 'val3',
+    ...         'keys12': 'val2',
+    ...     },
+    ...     'section 2': {
+    ...         'section 2 sub 1': {
+    ...             'fish': '3',
+    ...     },
+    ...     'keys21': 'val1',
+    ...     'keys22': 'val2',
+    ...     'keys23': 'val3',
+    ...     },
+    ... }
+    1
+    >>> t = '''
+    ... 'a' = b # !"$%^&*(),::;'@~#= 33
+    ... "b" = b #= 6, 33
+    ... ''' .split('\\n')
+    >>> t2 = ConfigObj(t)
+    >>> assert t2 == {'a': 'b', 'b': 'b'}
+    >>> t2.inline_comments['b'] = ''
+    >>> del t2['a']
+    >>> assert t2.write() == ['','b = b', '']
+    """
+
+if __name__ == '__main__':
+    # run the code tests in doctest format
+    #
+    testconfig1 = """\
+    key1= val    # comment 1
+    key2= val    # comment 2
+    # comment 3
+    [lev1a]     # comment 4
+    key1= val    # comment 5
+    key2= val    # comment 6
+    # comment 7
+    [lev1b]    # comment 8
+    key1= val    # comment 9
+    key2= val    # comment 10
+    # comment 11
+        [[lev2ba]]    # comment 12
+        key1= val    # comment 13
+        # comment 14
+        [[lev2bb]]    # comment 15
+        key1= val    # comment 16
+    # comment 17
+    [lev1c]    # comment 18
+    # comment 19
+        [[lev2c]]    # comment 20
+        # comment 21
+            [[[lev3c]]]    # comment 22
+            key1 = val    # comment 23"""
+    #
+    testconfig2 = """\
+                        key1 = 'val1'
+                        key2 =   "val2"
+                        key3 = val3
+                        ["section 1"] # comment
+                        keys11 = val1
+                        keys12 = val2
+                        keys13 = val3
+                        [section 2]
+                        keys21 = val1
+                        keys22 = val2
+                        keys23 = val3
+                        
+                            [['section 2 sub 1']]
+                            fish = 3
+    """
+    #
+    testconfig6 = '''
+    name1 = """ a single line value """ # comment
+    name2 = \''' another single line value \''' # comment
+    name3 = """ a single line value """
+    name4 = \''' another single line value \'''
+        [ "multi section" ]
+        name1 = """
+        Well, this is a
+        multiline value
+        """
+        name2 = \'''
+        Well, this is a
+        multiline value
+        \'''
+        name3 = """
+        Well, this is a
+        multiline value
+        """     # a comment
+        name4 = \'''
+        Well, this is a
+        multiline value
+        \'''  # I guess this is a comment too
+    '''
+    #
+    import doctest
+    m = sys.modules.get('__main__')
+    globs = m.__dict__.copy()
+    a = ConfigObj(testconfig1.split('\n'), raise_errors=True)
+    b = ConfigObj(testconfig2.split('\n'), raise_errors=True)
+    i = ConfigObj(testconfig6.split('\n'), raise_errors=True)
+    globs.update({
+        'INTP_VER': INTP_VER,
+        'a': a,
+        'b': b,
+        'i': i,
+    })
+    doctest.testmod(m, globs=globs)
+
+"""
+    BUGS
+    ====
+    
+    With list values off, ConfigObj can incorrectly unquote values. (This makes
+    it impossible to use listquote to handle your list values for you - for
+    nested lists. Not handling quotes at all would be better for this)
+    
+    TODO
+    ====
+    
+    A method to optionally remove uniform indentation from multiline values.
+    (do as an example of using ``walk`` - along with string-escape)
+    
+    INCOMPATIBLE CHANGES
+    ====================
+    
+    (I have removed a lot of needless complications - this list is probably not
+    conclusive, many option/attribute/method names have changed)
+    
+    Case sensitive
+    
+    The only valid divider is '='
+    
+    We've removed line continuations with '\'
+    
+    No recursive lists in values
+    
+    No empty section
+    
+    No distinction between flatfiles and non flatfiles
+    
+    Change in list syntax - use commas to indicate list, not parentheses
+    (square brackets and parentheses are no longer recognised as lists)
+    
+    ';' is no longer valid for comments and no multiline comments
+    
+    No attribute access
+    
+    We don't allow empty values - have to use '' or ""
+    
+    In ConfigObj 3 - setting a non-flatfile member to ``None`` would
+    initialise it as an empty section.
+    
+    The escape entities '&mjf-lf;' and '&mjf-quot;' have gone
+    replaced by triple quote, multiple line values.
+    
+    The ``newline``, ``force_return``, and ``default`` options have gone
+    
+    The ``encoding`` and ``backup_encoding`` methods have gone - replaced
+    with the ``encode`` and ``decode`` methods.
+    
+    ``fileerror`` and ``createempty`` options have become ``file_error`` and
+    ``create_empty``
+    
+    Partial configspecs (for specifying the order members should be written
+    out and which should be present) have gone. The configspec is no longer
+    used to specify order for the ``write`` method.
+    
+    Exceeding the maximum depth of recursion in string interpolation now
+    raises an error ``InterpolationDepthError``.
+    
+    Specifying a value for interpolation which doesn't exist now raises an
+    error ``MissingInterpolationOption`` (instead of merely being ignored).
+    
+    The ``writein`` method has been removed.
+    
+    The comments attribute is now a list (``inline_comments`` equates to the
+    old comments attribute)
+    
+    ISSUES
+    ======
+    
+    You can't have a keyword with the same name as a section (in the same
+    section). They are both dictionary keys - so they would overlap.
+    
+    Interpolation checks first the 'DEFAULT' subsection of the current
+    section, next it checks the 'DEFAULT' section of the parent section,
+    last it checks the 'DEFAULT' section of the main section.
+    
+    Logically a 'DEFAULT' section should apply to all subsections of the *same
+    parent* - this means that checking the 'DEFAULT' subsection in the
+    *current section* is not necessarily logical ?
+    
+    In order to simplify unicode support (which is possibly of limited value
+    in a config file) I have removed automatic support and added the
+    ``encode`` and ``decode methods, which can be used to transform keys and
+    entries. Because the regex looks for specific values on inital parsing
+    (i.e. the quotes and the equals signs) it can only read ascii compatible
+    encodings. For unicode use ``UTF8``, which is ASCII compatible.
+    
+    Does it matter that we don't support the ':' divider, which is supported
+    by ``ConfigParser`` ?
+    
+    Following error with "list_values=False" : ::
+    
+        >>> a = ["a='hello', 'goodbye'"]
+        >>>
+        >>> c(a, list_values=False)
+        {'a': "hello', 'goodbye"}
+    
+    The regular expression correctly removes the value -
+    ``"'hello', 'goodbye'"`` and then unquote just removes the front and
+    back quotes (called from ``_handle_value``). What should we do ??
+    (*ought* to raise exception because it's an invalid value if lists are
+    off *sigh*. This is not what you want if you want to do your own list
+    processing - would be *better* in this case not to unquote.)
+    
+    String interpolation and validation don't play well together. When
+    validation changes type it sets the value. This will correctly fetch the
+    value using interpolation - but then overwrite the interpolation reference.
+    If the value is unchanged by validation (it's a string) - but other types
+    will be.
+    
+    List Value Syntax
+    =================
+    
+    List values allow you to specify multiple values for a keyword. This
+    maps to a list as the resulting Python object when parsed.
+    
+    The syntax for lists is easy. A list is a comma separated set of values.
+    If these values contain quotes, the hash mark, or commas, then the values
+    can be surrounded by quotes. e.g. : ::
+    
+        keyword = value1, 'value 2', "value 3"
+    
+    If a value needs to be a list, but only has one member, then you indicate
+    this with a trailing comma. e.g. : ::
+    
+        keyword = "single value",
+    
+    If a value needs to be a list, but it has no members, then you indicate
+    this with a single comma. e.g. : ::
+    
+        keyword = ,     # an empty list
+    
+    Using triple quotes it will be possible for single values to contain
+    newlines and *both* single quotes and double quotes. Triple quotes aren't
+    allowed in list values. This means that the members of list values can't
+    contain carriage returns (or line feeds :-) or both quote values.
+      
+    CHANGELOG
+    =========
+    
+    2005/10/09
+    ----------
+    
+    Fixed typo in ``write`` method. (Testing for the wrong value when resetting
+    ``interpolation``).
+    
+    2005/09/16
+    ----------
+    
+    Fixed bug in ``setdefault`` - creating a new section *wouldn't* return
+    a reference to the new section.
+    
+    2005/09/09
+    ----------
+    
+    Removed ``PositionError``.
+    
+    Allowed quotes around keys as documented.
+    
+    Fixed bug with commas in comments. (matched as a list value)
+    
+    Beta 5
+    
+    2005/09/07
+    ----------
+    
+    Fixed bug in initialising ConfigObj from a ConfigObj.
+    
+    Changed the mailing list address.
+    
+    Beta 4
+    
+    2005/09/03
+    ----------
+    
+    Fixed bug in ``Section__delitem__`` oops.
+    
+    2005/08/28
+    ----------
+    
+    Interpolation is switched off before writing out files.
+    
+    Fixed bug in handling ``StringIO`` instances. (Thanks to report from
+    "Gustavo Niemeyer" <gustavo@niemeyer.net>)
+    
+    Moved the doctests from the ``__init__`` method to a separate function.
+    (For the sake of IDE calltips).
+    
+    Beta 3
+    
+    2005/08/26
+    ----------
+    
+    String values unchanged by validation *aren't* reset. This preserves
+    interpolation in string values.
+    
+    2005/08/18
+    ----------
+    
+    None from a default is turned to '' if stringify is off - because setting 
+    a value to None raises an error.
+    
+    Version 4.0.0-beta2
+    
+    2005/08/16
+    ----------
+    
+    By Nicola Larosa
+    
+    Actually added the RepeatSectionError class ;-)
+    
+    2005/08/15
+    ----------
+    
+    If ``stringify`` is off - list values are preserved by the ``validate``
+    method. (Bugfix)
+    
+    2005/08/14
+    ----------
+    
+    By Michael Foord
+    
+    Fixed ``simpleVal``.
+    
+    Added ``RepeatSectionError`` error if you have additional sections in a
+    section with a ``__many__`` (repeated) section.
+    
+    By Nicola Larosa
+    
+    Reworked the ConfigObj._parse, _handle_error and _multiline methods:
+    mutated the self._infile, self._index and self._maxline attributes into
+    local variables and method parameters
+    
+    Reshaped the ConfigObj._multiline method to better reflect its semantics
+    
+    Changed the "default_test" test in ConfigObj.validate to check the fix for
+    the bug in validate.Validator.check
+    
+    2005/08/13
+    ----------
+    
+    By Nicola Larosa
+    
+    Updated comments at top
+    
+    2005/08/11
+    ----------
+    
+    By Michael Foord
+    
+    Implemented repeated sections.
+    
+    By Nicola Larosa
+    
+    Added test for interpreter version: raises RuntimeError if earlier than
+    2.2
+    
+    2005/08/10
+    ----------
+   
+    By Michael Foord
+     
+    Implemented default values in configspecs.
+    
+    By Nicola Larosa
+    
+    Fixed naked except: clause in validate that was silencing the fact
+    that Python2.2 does not have dict.pop
+    
+    2005/08/08
+    ----------
+    
+    By Michael Foord
+    
+    Bug fix causing error if file didn't exist.
+    
+    2005/08/07
+    ----------
+    
+    By Nicola Larosa
+    
+    Adjusted doctests for Python 2.2.3 compatibility
+    
+    2005/08/04
+    ----------
+    
+    By Michael Foord
+    
+    Added the inline_comments attribute
+    
+    We now preserve and rewrite all comments in the config file
+    
+    configspec is now a section attribute
+    
+    The validate method changes values in place
+    
+    Added InterpolationError
+    
+    The errors now have line number, line, and message attributes. This
+    simplifies error handling
+    
+    Added __docformat__
+    
+    2005/08/03
+    ----------
+    
+    By Michael Foord
+    
+    Fixed bug in Section.pop (now doesn't raise KeyError if a default value
+    is specified)
+    
+    Replaced ``basestring`` with ``types.StringTypes``
+    
+    Removed the ``writein`` method
+    
+    Added __version__
+    
+    2005/07/29
+    ----------
+    
+    By Nicola Larosa
+    
+    Indentation in config file is not significant anymore, subsections are
+    designated by repeating square brackets
+    
+    Adapted all tests and docs to the new format
+    
+    2005/07/28
+    ----------
+    
+    By Nicola Larosa
+    
+    Added more tests
+    
+    2005/07/23
+    ----------
+    
+    By Nicola Larosa
+    
+    Reformatted final docstring in ReST format, indented it for easier folding
+    
+    Code tests converted to doctest format, and scattered them around
+    in various docstrings
+    
+    Walk method rewritten using scalars and sections attributes
+    
+    2005/07/22
+    ----------
+    
+    By Nicola Larosa
+    
+    Changed Validator and SimpleVal "test" methods to "check"
+    
+    More code cleanup
+    
+    2005/07/21
+    ----------
+    
+    Changed Section.sequence to Section.scalars and Section.sections
+    
+    Added Section.configspec
+    
+    Sections in the root section now have no extra indentation
+    
+    Comments now better supported in Section and preserved by ConfigObj
+    
+    Comments also written out
+    
+    Implemented initial_comment and final_comment
+    
+    A scalar value after a section will now raise an error
+    
+    2005/07/20
+    ----------
+    
+    Fixed a couple of bugs
+    
+    Can now pass a tuple instead of a list
+    
+    Simplified dict and walk methods
+    
+    Added __str__ to Section
+    
+    2005/07/10
+    ----------
+    
+    By Nicola Larosa
+    
+    More code cleanup
+    
+    2005/07/08
+    ----------
+    
+    The stringify option implemented. On by default.
+    
+    2005/07/07
+    ----------
+    
+    Renamed private attributes with a single underscore prefix.
+    
+    Changes to interpolation - exceeding recursion depth, or specifying a
+    missing value, now raise errors.
+    
+    Changes for Python 2.2 compatibility. (changed boolean tests - removed
+    ``is True`` and ``is False``)
+    
+    Added test for duplicate section and member (and fixed bug)
+    
+    2005/07/06
+    ----------
+    
+    By Nicola Larosa
+    
+    Code cleanup
+    
+    2005/07/02
+    ----------
+    
+    Version 0.1.0
+    
+    Now properly handles values including comments and lists.
+    
+    Better error handling.
+    
+    String interpolation.
+    
+    Some options implemented.
+    
+    You can pass a Section a dictionary to initialise it.
+    
+    Setting a Section member to a dictionary will create a Section instance.
+    
+    2005/06/26
+    ----------
+    
+    Version 0.0.1
+    
+    Experimental reader.
+    
+    A reasonably elegant implementation - a basic reader in 160 lines of code.
+    
+    *A programming language is a medium of expression.* - Paul Graham
+"""
+

=== added directory 'bzrlib/util/configobj/docs'
=== added file 'bzrlib/util/configobj/docs/BSD-LICENSE.txt'
--- bzrlib/util/configobj/docs/BSD-LICENSE.txt	1970-01-01 00:00:00 +0000
+++ bzrlib/util/configobj/docs/BSD-LICENSE.txt	2005-10-18 18:48:27 +0000
@@ -0,0 +1,60 @@
+===================================
+ The Voidspace Open Source License
+===================================
+
+.. note::
+
+    See this document online at http://www.voidspace.org.uk/python/license.shtml
+
+This is the OSI_ Approved license that the Voidspace modules and programs are
+available under [#]_. It's often reffered to as the *BSD License*.
+
+It is a very unrestrictive license but it comes with the usual disclaimer. 
+This is free software: test it, break it, just don't blame me if it eats your 
+data ! Of course if it does, let me know and I'll fix the problem so that it 
+doesn't happen to anyone else {sm;:-)}.
+
+.. _osi: http://www.opensource.org/
+
+::
+
+    Copyright (c) 2003-2005, Michael Foord
+    All rights reserved.
+    E-mail : fuzzyman AT voidspace DOT org DOT uk
+    
+    Redistribution and use in source and binary forms, with or without
+    modification, are permitted provided that the following conditions are
+    met:
+    
+    
+        * Redistributions of source code must retain the above copyright
+          notice, this list of conditions and the following disclaimer.
+    
+        * Redistributions in binary form must reproduce the above
+          copyright notice, this list of conditions and the following
+          disclaimer in the documentation and/or other materials provided
+          with the distribution.
+    
+        * Neither the name of Michael Foord nor the name of Voidspace 
+          may be used to endorse or promote products derived from this
+          software without specific prior written permission.
+    
+    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+    A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+    OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+    SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+    LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+    DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+    THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+    OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+
+If you use any of my code in your projects, then a link back would be nice. If
+you let me know about any of your projects that use them - then I can link to
+you as examples.
+
+-----------
+
+.. [#] Unless specifically stated otherwise of course.

=== added file 'bzrlib/util/configobj/docs/configobj.txt'
--- bzrlib/util/configobj/docs/configobj.txt	1970-01-01 00:00:00 +0000
+++ bzrlib/util/configobj/docs/configobj.txt	2005-10-18 18:48:27 +0000
@@ -0,0 +1,1981 @@
+==================================
+ Reading and Writing Config Files
+==================================
+
+----------------------------------------
+ ConfigObj 4 Introduction and Reference
+----------------------------------------
+
+:Authors: Michael Foord, Nicola Larosa
+:Version: ConfigObj 4.0.0
+:Date: 2005/10/17
+:Homepage: `ConfigObj Homepage`_
+:Sourceforge: Sourceforge_
+:License: `BSD License`_
+:Support: `Mailing List`_
+
+.. _Mailing List: http://lists.sourceforge.net/lists/listinfo/configobj-develop
+
+.. meta::
+   :description: ConfigObj - a Python module for easy reading and writing of 
+                 config files.
+   :keywords: python, script, module, config, configuration, data, persistence,
+              developer, configparser
+
+.. contents:: ConfigObj Manual
+.. sectnum::
+
+Introduction
+============
+
+**ConfigObj** is a simple but powerful config file reader and writer: an *ini
+file round tripper*. Its main feature is that it is very easy to use, with a
+straightforward programmer's interface and a simple syntax for config files.
+It has lots of other features though :
+
+* Nested sections (subsections), to any level
+* List values
+* Multiple line values
+* String interpolation (substitution)
+* Integrated with a powerful validation system
+
+    * including automatic type checking/conversion
+    * repeated sections
+    * and allowing default values
+
+* All comments in the file are preserved
+* The order of keys/sections is preserved
+* No external dependencies
+
+ConfigObj 4 is a complete rewrite of ConfigObj. A great deal has been
+simplified and improved [#]_ since ConfigObj 3. If you have used ConfigObj
+before then you need to read the section on `backwards compatibility`_.
+
+ConfigObj now has a barrage [#]_ of doctests built into it, testing almost every
+feature. Run ``python configobj.py -v`` to see them in action. Despite the tests,
+ConfigObj 4 is actually smaller than version 3 and has no external dependencies.
+
+For support and bug reports please use the ConfigObj `Mailing List`_.
+
+.. hint::
+
+    There is an article on using `ConfigObj for Data Persistence`_.
+    
+    The code from that article is available as ConfigPersist.py_.
+
+.. _ConfigObj for Data Persistence: http://www.voidspace.org.uk/python/articles/configobj_for_data_persistence.shtml
+.. _ConfigPersist.py: http://www.voidspace.org.uk/python/configpersist.html
+
+Downloading
+===========
+
+The current version is **4.0.0**, dated 17th October 2005. ConfigObj 4 is
+now stable. We may still expect to pick up a few bugs along the way though [#]_.
+{sm;:-)}
+
+You can get ConfigObj in the following ways :
+
+Files
+-----
+
+* configobj.py_ from Voidspace
+
+    ConfigObj has no external dependencies. This file is sufficient to access
+    all the functionality except Validation_.
+
+* configobj.zip_ from Voidspace
+
+    This also contains validate.py_ , the `API Docs`_ and `this document`_.
+
+* The latest development version can be obtained from the `Subversion
+  Repository`_.
+
+* validate.py_ from Voidspace
+
+* You can also download *configobj.zip* from Sourceforge_
+
+Documentation
+-------------
+
+*configobj.zip* contains `this document`_ and full `API Docs`_, generated by
+the EpyDoc_ program.
+
+* You can view `this document`_ online as the `ConfigObj Homepage`_.
+
+* You can also browse the `API Docs`_ online.
+
+Pythonutils
+-----------
+
+ConfigObj is also part of the Pythonutils_ set of modules. This contains
+various other useful modules, and is required by many of the `Voidspace Python
+Projects`_.
+
+.. _configobj.py: http://www.voidspace.org.uk/cgi-bin/voidspace/downman.py?file=configobj.py
+.. _configobj.zip: http://www.voidspace.org.uk/cgi-bin/voidspace/downman.py?file=configobj-4.0.0.zip
+.. _validate.py: http://www.voidspace.org.uk/cgi-bin/voidspace/downman.py?file=validate.py
+.. _API Docs: http://www.voidspace.org.uk/python/configobj-api/
+.. _this document:
+.. _configobj homepage: http://www.voidspace.org.uk/python/configobj.html
+.. _Subversion Repository: http://svn.rest2web.python-hosting.com/branches/configobj4
+.. _Sourceforge: http://sourceforge.net/projects/configobj
+.. _EpyDoc: http://epydoc.sourceforge.net
+.. _pythonutils: http://www.voidspace.org.uk/python/pythonutils.html
+.. _Voidspace Python Projects: http://www.voidspace.org.uk/python
+
+Getting Started
+===============
+
+The outstanding feature of using ConfigObj is simplicity. Most functions can be
+performed with single line commands.
+
+Reading a Config File
+---------------------
+
+The normal way to read a config file, is to give ConfigObj the filename :
+
+.. raw:: html
+
+    {+coloring}
+
+    from configobj import ConfigObj
+    config = ConfigObj(filename)
+
+    {-coloring}
+
+You can also pass the config file in as a list of lines, or a ``StringIO``
+instance, so it doesn't matter where your config data comes from.
+
+You can then access members of your config file as a dictionary. Subsections
+will also be dictionaries.
+
+.. raw:: html
+
+    {+coloring}
+
+    from configobj import ConfigObj
+    config = ConfigObj(filename)
+    #
+    value1 = config['keyword1']
+    value2 = config['keyword2']
+    #
+    section1 = config['section1']
+    value3 = section1['keyword3']
+    value4 = section1['keyword4']
+    #
+    # you could also write
+    value3 = config['section1']['keyword3']
+    value4 = config['section1']['keyword4']
+
+    {-coloring}
+
+Writing a Config File
+---------------------
+
+Creating a new config file is just as easy as reading one. You can specify a
+filename when you create the ConfigObj, or do it later [#]_.
+
+If you *don't* set a filename, then the ``write`` method will return a list of
+lines instead of writing to file. See the write_ method for more details.
+
+Here we show creating an empty ConfigObj, setting a filename and some values,
+and then writing to file :
+
+.. raw:: html
+
+    {+coloring}
+
+    from configobj import ConfigObj
+    config = ConfigObj()
+    config.filename = filename
+    #
+    config['keyword1'] = value1
+    config['keyword2'] = value2
+    #
+    config['section1'] = {}
+    config['section1']['keyword3'] = value3
+    config['section1']['keyword4'] = value4
+    #
+    section2 = {
+        'keyword5': value5,
+        'keyword6': value6,
+        'sub-section': {
+            'keyword7': value7
+            }
+    }
+    config['section2'] = section2
+    #
+    config['section3'] = {}
+    config['section3']['keyword 8'] = [value8, value9, value10]
+    config['section3']['keyword 9'] = [value11, value12, value13]
+    #
+    config.write()
+
+    {-coloring}
+
+.. caution::
+
+    Keywords and section names can only be strings [#]_. Attempting to set
+    anything else will raise a ``ValueError``.
+
+Config Files
+------------
+
+The config files that ConfigObj will read and write are based on the 'INI'
+format. This means it will read and write files created for ``ConfigParser``
+[#]_.
+
+Keywords and values are separated by an ``'='``, and section markers are
+between square brackets. Keywords, values, and section names can be surrounded
+by single or double quotes. Indentation is not significant, but can be
+preserved.
+
+Subsections are indicated by repeating the square brackets in the section
+marker. You nest levels by using more brackets.
+
+You can have list values by separating items with a comma, and values spanning
+multiple lines by using triple quotes (single or double).
+
+For full details on all these see `the config file format`_. Here's an example
+to illustrate : ::
+
+    # This is the 'initial_comment'
+    # Which may be several lines
+    keyword1 = value1
+    'keyword 2' = 'value 2'
+
+    [ "section 1" ]
+    # This comment goes with keyword 3
+    keyword 3 = value 3
+    'keyword 4' = value4, value 5, 'value 6'
+
+        [[ sub-section ]]    # an inline comment
+        # sub-section is inside "section 1"
+        'keyword 5' = 'value 7'
+        'keyword 6' = '''A multiline value,
+    that spans more than one line :-)
+    The line breaks are included in the value.'''
+
+            [[[ sub-sub-section ]]]
+            # sub-sub-section is *in* 'sub-section'
+            # which is in 'section 1'
+            'keyword 7' = 'value 8'
+
+    [section 2]    # an inline comment
+    keyword8 = "value 9"
+    keyword9 = value10     # an inline comment
+    # The 'final_comment'
+    # Which also may be several lines
+
+ConfigObj specifications
+========================
+
+.. raw:: html
+
+    {+coloring}
+
+    config = ConfigObj(infile=None, options=None, **keywargs)
+
+    {-coloring}
+
+infile
+------
+
+You don't need to specify an infile. If you omit it, an empty ConfigObj will be
+created. ``infile`` *can* be :
+
+* Nothing. In which case the ``filename`` attribute of your ConfigObj will be
+  ``None``. You can set a filename at any time.
+
+* A filename. What happens if the file doesn't already exist is determined by
+  the options_ ``file_error`` and ``create_empty``. The filename will be
+  preserved as the ``filename`` attribute. This can be changed at any time.
+
+* A list of lines. Any trailing ``\n`` will be removed from the lines. The
+  ``filename`` attribute of your ConfigObj will be ``None``.
+
+* A ``StringIO`` instance or file object, or any object with ``seek`` and
+  ``read`` methods. The object you pass in will be preserved as the
+  ``filename`` attribute of your ConfigObj. If it has a ``write`` method [#]_
+  then you can use the ConfigObj ``write`` method. Note that a file object
+  passed in won't have its ``close`` method called by ConfigObj.
+
+* A dictionary. You can initialise a ConfigObj from a dictionary [#]_. The
+  ``filename`` attribute of your ConfigObj will be ``None``. All keys must be
+  strings. In this case, the order of values and sections is arbitrary.
+
+options
+-------
+
+There are various options that control the way ConfigObj behaves. They can be
+passed in as a dictionary of options, or as keyword arguments. Explicit keyword
+arguments override the dictionary.
+
+All of the options are available as attributes after the config file has been
+parsed.
+
+ConfigObj has the following options (with the default values shown) :
+
+* 'raise_errors': ``False``
+
+    When parsing, it is possible that the config file will be badly formed. The
+    default is to parse the whole file and raise a single error at the end. You
+    can set ``raise_errors = True`` to have errors raised immediately. See the
+    exceptions_ section for more details.
+
+    Altering this value after initial parsing has no effect.
+
+* 'list_values': ``True``
+
+    If ``True`` (the default) then list values are possible. If ``False``, the
+    values are not parsed for lists.
+
+    Altering this value after initial parsing has no effect.
+
+* 'create_empty': ``False``
+
+    If this value is ``True`` and the file specified by ``infile`` doesn't
+    exist, ConfigObj will create an empty file. This can be a useful test that
+    the filename makes sense: an impossible filename will cause an error.
+
+    Altering this value after initial parsing has no effect.
+
+* 'file_error': ``False``
+
+    If this value is ``True`` and the file specified by ``infile`` doesn't
+    exist, ConfigObj will raise an ``IOError``.
+
+    Altering this value after initial parsing has no effect.
+
+* 'interpolation': ``True``
+
+    Whether string interpolation is switched on or not. It is on (``True``) by
+    default.
+
+    You can set this attribute to change whether string interpolation is done
+    when values are fetched. See the interpolation_ section for more details.
+
+* 'configspec': ``None``
+
+    If you want to use the validation system, you supply a configspec. This is
+    effectively a type of config file that specifies a check for each member.
+    This check can be used to do type conversion as well as check that the
+    value is within your required parameters.
+
+    You provide a configspec in the same way as you do the initial file: a
+    filename, or list of lines, etc. See the validation_ section for full
+    details on how to use the system.
+
+    When parsed, every section has a ``configspec`` with a dictionary of
+    configspec checks for *that section*.
+
+* 'stringify': ``True``
+
+    If you use the validation scheme, it can do type checking *and* conversion
+    for you. This means you may want to set members to integers, or other
+    non-string values.
+
+    If 'stringify' is set to ``True`` (default) then non-string values will
+    be converted to strings when you write the config file. The validation_
+    process converts values from strings to the required type.
+
+    If 'stringify' is set to ``False``, attempting to set a member to a
+    non-string value [#]_ will raise a ``TypeError`` (no type conversion is
+    done by validation).
+
+* 'indent_type': ``' '``
+
+    Indentation is not significant; it can however be present in the output
+    config. Allowable values are: ``''`` (no indentation), ``' '`` (indentation
+    with spaces, fixed at four per level), or ``'\t'`` (indentation with tabs,
+    one tab per level).
+
+    If this option is not specified, and the ConfigObj is initialised with a
+    dictionary, the indentation used in the output is the default one, that is,
+    spaces.
+
+    If this option is not specified, and the ConfigObj is initialised with a
+    list of lines or a file, the indentation used in the first indented line is
+    selected and used in all output lines. If no input line is indented, no
+    output line will be either.
+
+    If this option *is* specified, the option value is used in the output
+    config, overriding the type of indentation in the input config (if any).
+
+Methods
+-------
+
+The ConfigObj is a subclass of an object called ``Section``, which is itself a
+subclass of ``dict``, the builtin dictionary type. This means it also has
+**all** the normal dictionary methods.
+
+In addition, the *encode*, *decode*, *walk*, and *dict* methods of section may
+be useful. The sections and subsections are also instances of ``Section``. Read
+about Sections_ for details of all the methods.
+
+The public methods available on ConfigObj are :
+
+* 'write'
+* 'validate'
+
+write
+~~~~~
+
+This method takes no arguments [#]_. It writes the current ConfigObj. What that
+means depends on the ``filename`` attribute of the ConfigObj.
+
+filename
+    ConfigObj will write the configuration to file.
+
+``None``
+    ``write`` returns a list of lines. (Not ``'\n'`` terminated)
+
+``StringIO``
+    If ``filename`` is an object with a seek attribute, then the config file is
+    written to that object.
+
+First the 'initial_comment' is written, then the config file, followed by the
+'final_comment'. Comment lines and inline comments are written with each
+key/value.
+
+validate
+~~~~~~~~
+
+.. raw:: html
+
+    {+coloring}
+
+    # filename is the config file
+    # filename2 is the configspec
+    # (which could also be hardcoded into your program)
+    config = ConfigObj(filename, configspec=filename2)
+    #
+    from validate import Validator
+    val = Validator()
+    test = config.validate(val)
+    if test == True:
+        print 'Succeeded.'
+
+    {-coloring}
+
+This method validates the ConfigObj against the configspec. By doing type
+conversion as well, it can abstract away the config file altogether and present
+the config *data* to your application (in the types it expects it to be).
+
+If the ``configspec`` attribute of the ConfigObj is ``None``, it raises a
+``ValueError``.
+
+If the stringify_ attribute is set, this process will convert values to the
+type defined in the configspec.
+
+The validate method uses checks specified in the configspec and defined in the
+``Validator`` object. It is very easy to extend.
+
+The configspec looks like the config file, but instead of the value, you
+specify the check (and any default value). See the validation_ section for
+details.
+
+.. hint::
+
+    If your ConfigObj is only comprised of basic data types, then you can use
+    a function from the ConfigPersist.py_ module to auto-generate your
+    configspec.
+    
+    See `ConfigObj for Data Persistence`_.
+
+Return Value
+############
+
+The validate method either returns ``True`` (everything passed) or a dictionary 
+of ``True``/``False`` representing pass/fail. The dictionary follows the 
+structure of the ConfigObj
+
+If a whole section passes then it is replaced with the value ``True``. If a 
+whole section fails, then it is replaced with the value ``False``.
+
+If a value is missing, and there is no default in the check, then the check 
+automatically fails.
+
+Mentioning Default Values
+#########################
+
+In the check in your configspec, you can specify a default to be used - by 
+using the ``default`` keyword. E.g. ::
+
+    key1 = integer(0, 30, default=15)
+    key2 = integer(default=15)
+    key3 = boolean(default=True)
+    key4 = option('Hello', 'Goodbye', 'Not Today', default='Not Today')
+
+If the configspec check supplies a default and the value is missing in the
+config, then the default will be set in your ConfigObj. (It is still passed to
+the ``Validator`` so that type conversion can be done: this means the default
+value must still pass the check.)
+
+ConfigObj keeps a record of which values come from defaults, using the
+``defaults`` attribute of sections_. Any key in this list isn't written out by
+the ``write`` method. If a key is set from outside (even to the same value)
+then it is removed from the ``defaults`` list.
+
+.. note:
+
+    Even if all the keys in a section are in the defaults list, the section
+    marker is still written out.
+
+There is additionally a special case default value of ``None``. If you set the
+default value to ``None`` and the value is missing, the value will always be
+set to ``None``. As the other checks don't return ``None`` (unless you
+implement your own that do), you can tell that this value came from a default
+value (and was missing from the config file). It allows an easy way of
+implementing optional values. Simply check (and ignore) members that are set
+to ``None``.
+
+.. note::
+
+    If stringify_ is ``False`` then ``default=None`` returns ``''`` instead of
+    ``None``. This is because setting a value to a non-string raises an error
+    if stringify is unset.
+
+Mentioning Repeated Sections
+############################
+
+In the configspec it is possible to cause *every* sub-section in a section to
+be validated using the same configspec. You do this with a section in the
+configspec  called ``__many__``. Every sub-section in that section has the
+``__many__`` configspec applied to it (without you having to explicitly name
+them in advance).
+
+If you define a ``__many__`` type section it must the only sub-section in that
+section. Having a ``__many__`` *and* other sub-sections defined in the same
+section will raise a ``RepeatSectionError``.
+
+Your ``__many__`` section can have nested subsections, which can also include
+``__many__`` type sections.
+
+See `Repeated Sections`_ for examples.
+
+Mentioning SimpleVal
+####################
+
+If you just want to check if all members are present, then you can use the
+``SimpleVal`` object that comes with ConfigObj. It only fails members if they
+are missing.
+
+Write a configspec that has all the members you want to check for, but set
+every section to ``''``.
+
+.. raw:: html
+
+    {+coloring}
+
+    val = SimpleVal()
+    test = config.validate(val)
+    if test is True:
+        print 'Succeeded.'
+
+    {-coloring}
+
+Attributes
+----------
+
+A ConfigObj has the following attributes :
+
+* indent_type
+* interpolate
+* stringify
+* BOM
+* initial_comment
+* final_comment
+
+.. note::
+
+    This doesn't include *comments*, *inline_comments*, *defaults*, or
+    *configspec*. These are actually attributes of Sections_.
+
+It also has the following attributes as a result of parsing. They correspond to
+options_ when the ConfigObj was created, but changing them has no effect.
+
+* raise_errors
+* create_empty
+* file_error
+* list_values
+
+interpolate
+~~~~~~~~~~~
+
+ConfigObj can perform string interpolation in a *similar* way to
+``ConfigParser``. See the interpolation_ section for full details.
+
+If ``interpolate`` is set to ``False``, then interpolation is *not* done when
+you fetch values.
+
+stringify
+~~~~~~~~~
+
+If this attribute is set (``True``) then the validate_ method changes the
+values in the ConfigObj. These are turned back into strings when write_ is
+called.
+
+If stringify is unset (``False``) then attempting to set a value to a non
+string (or a list of strings) will raise a ``TypeError``.
+
+BOM
+~~~
+
+If the initial config file *started* with the UTF8 Unicode signature (known
+slightly incorrectly as the {acro;BOM;Byte Order Mark}), then this value will
+be set to the UTF8 BOM. Otherwise it is ``None``.
+
+If it is set, then it is  written out by the ``write`` method.
+
+initial_comment
+~~~~~~~~~~~~~~~
+
+This is a list of lines. If the ConfigObj is created from an existing file, it
+will contain any lines of comments before the start of the members.
+
+If you create a new ConfigObj, this will be an empty list.
+
+The write method puts these lines before it starts writing out the members.
+
+final_comment
+~~~~~~~~~~~~~
+
+This is a list of lines. If the ConfigObj is created from an existing file, it
+will contain any lines of comments after the last member.
+
+If you create a new ConfigObj, this will be an empty list.
+
+The ``write`` method puts these lines after it finishes writing out the
+members.
+
+The Config File Format
+======================
+
+You saw an example config file in the `Config Files`_ section. Here is a fuller
+specification of the config files used and created by ConfigObj.
+
+The basic pattern for keywords is : ::
+
+    # comment line
+    # comment line
+    keyword = value # inline comment
+
+Both keyword and value can optionally be surrounded in quotes. The equals sign
+is the only valid divider.
+
+Values can have comments on the lines above them, and an inline comment after
+them. This, of course, is optional. See the comments_ section for details.
+
+If a keyword or value starts or ends with whitespace, or contains a quote mark
+or comma, then it should be surrounded by quotes. Quotes are not necessary if
+whitespace is surrounded by non-whitespace.
+
+Values can also be lists. Lists are comma separated. You indicate a single
+member list by a trailing comma. An empty list is shown by a single comma : ::
+
+    keyword1 = value1, value2, value3
+    keyword2 = value1, # a single member list
+    keyword3 = , # an empty list
+
+Values that contain line breaks (multi-line values) can be surrounded by triple
+quotes. These can also be used if a value contains both types of quotes. List
+members cannot be surrounded by triple quotes : ::
+
+    keyword1 = ''' A multi line value
+    on several
+    lines'''     # with a comment
+    keyword2 = '''I won't be "afraid".'''
+    #
+    keyword3 = """ A multi line value
+    on several
+    lines"""     # with a comment
+    keyword4 = """I won't be "afraid"."""
+
+.. warning::
+
+    There is no way of safely quoting values that contain both types of triple
+    quotes.
+
+A line that starts with a '#', possibly preceded by whitespace, is a comment.
+
+New sections are indicated by a section marker line. That is the section name
+in square brackets. Whitespace around the section name is ignored. The name can
+be quoted with single or double quotes. The marker can have comments before it
+and an inline comment after it : ::
+
+    # The First Section
+    [ section name 1 ] # first section
+    keyword1 = value1
+
+    # The Second Section
+    [ "section name 2" ] # second section
+    keyword2 = value2
+
+Any subsections (sections that are *inside* the current section) are
+designated by repeating the square brackets before and after the section name.
+The number of square brackets represents the nesting level of the sub-section.
+Square brackets may be separated by whitespace; such whitespace, however, will
+not be present in the output config written by the ``write`` method.
+
+Indentation is not significant, but can be preserved. See the description of
+the ``indent_type`` option, in the `ConfigObj specifications`_ chapter, for the
+details.
+
+A *NestingError* will be raised if the number of the opening and the closing
+brackets in a section marker is not the same, or if a sub-section's nesting
+level is greater than the nesting level of it parent plus one.
+
+In the outer section, single values can only appear before any sub-section.
+Otherwise they will belong to the sub-section immediately before them. ::
+
+    # initial comment
+    keyword1 = value1
+    keyword2 = value2
+
+    [section 1]
+    keyword1 = value1
+    keyword2 = value2
+
+        [[sub-section]]
+        # this is in section 1
+        keyword1 = value1
+        keyword2 = value2
+
+            [[[nested section]]]
+            # this is in sub section
+            keyword1 = value1
+            keyword2 = value2
+
+        [[sub-section2]]
+        # this is in section 1 again
+        keyword1 = value1
+        keyword2 = value2
+
+    [[sub-section3]]
+    # this is also in section 1, indentation is misleading here
+    keyword1 = value1
+    keyword2 = value2
+
+    # final comment
+
+When parsed, the above config file produces the following data structure :
+
+.. raw:: html
+
+    {+coloring}
+
+    {
+        'keyword1': 'value1',
+        'keyword2': 'value2',
+        'section 1': {
+            'keyword1': 'value1',
+            'keyword2': 'value2',
+            'sub-section': {
+                'keyword1': 'value1',
+                'keyword2': 'value2',
+                'nested section': {
+                    'keyword1': 'value1',
+                    'keyword2': 'value2',
+                },
+            },
+            'sub-section2': {
+                'keyword1': 'value1',
+                'keyword2': 'value2',
+            },
+            'sub-section3': {
+                'keyword1': 'value1',
+                'keyword2': 'value2',
+            },
+        },
+    }
+
+    {-coloring}
+
+Sections are ordered: note how the structure of the resulting ConfigObj is in
+the same order as the original file.
+
+Sections
+========
+
+Every section in a ConfigObj has certain properties. The ConfigObj itself also
+has these properties, because it too is a section (sometimes called the *root
+section*).
+
+``Section`` is a subclass of the standard new-class dictionary, therefore it
+has **all** the methods of a normal dictionary. This means you can ``update``
+and ``clear`` sections.
+
+.. note::
+
+    You create a new section by assigning a member to be a dictionary.
+    
+    The new ``Section`` is created *from* the dictionary, but isn't the same
+    thing as the dictionary. (So references to the dictionary you use to create
+    the section *aren't* references to the new section).
+    
+    Note the following.
+
+    .. raw:: html
+    
+        {+coloring}
+        
+        config = ConfigObj()
+        vals = {'key1': 'value 1', 
+                'key2': 'value 2'
+               }
+        config['vals'] = vals
+        config['vals'] == vals
+        True
+        config['vals'] is vals
+        False
+        
+        {-coloring}
+     
+    If you now change ``vals``, the changes won't be reflected in ``config['vals']``.
+
+A section is ordered, following its ``scalars`` and ``sections``
+attributes documented below. This means that the following dictionary
+attributes return their results in order.
+
+* '__iter__'
+
+    More commonly known as ``for member in section:``.
+
+* '__repr__' and '__str__'
+
+    Any time you print or display the ConfigObj.
+
+* 'items'
+
+* 'iteritems'
+
+* 'iterkeys'
+
+* 'itervalues'
+
+* 'keys'
+
+* 'popitem'
+
+* 'values'
+
+Section Attributes
+------------------
+
+* main
+
+    A reference to the main ConfigObj.
+
+* parent
+
+    A reference to the 'parent' section, the section that this section is a
+    member of.
+
+    On the ConfigObj this attribute is a reference to itself. You can use this
+    to walk up the sections, stopping when ``section.parent is section``.
+
+* depth
+
+    The nesting level of the current section.
+
+    If you create a new ConfigObj and add sections, 1 will be added to the
+    depth level between sections.
+
+* defaults
+
+    This attribute is a list of scalars that came from default values. Values
+    that came from defaults aren't written out by the ``write`` method.
+    Setting any of these values in the section removes them from the defaults
+    list.
+
+* scalars, sections
+
+    These attributes are normal lists, representing the order that members,
+    single values and subsections appear in the section. The order will either
+    be the order of the original config file, *or* the order that you added
+    members.
+
+    The order of members in this lists is the order that ``write`` creates in
+    the config file. The ``scalars`` list is output before the ``sections``
+    list.
+
+    Adding or removing members also alters these lists. You can manipulate the
+    lists directly to alter the order of members.
+
+    .. warning::
+
+        If you alter the ``scalars``, ``sections``, or ``defaults`` attributes
+        so that they no longer reflect the contents of the section, you will
+        break your ConfigObj.
+
+    See also the ``rename`` method.
+
+* comments
+
+    This is a dictionary of comments associated with each member. Each entry is
+    a list of lines. These lines are written out before the member.
+
+* inline_comments
+
+    This is *another* dictionary of comments associated with each member. Each
+    entry is a string that is put inline with the member.
+
+* configspec
+
+    The configspec attribute is a dictionary mapping scalars to *checks*. A
+    check defines the expected type and possibly the allowed values for a
+    member.
+
+    The configspec has the same format as a config file, but instead of values
+    it has a specification for the value (which may include a default value).
+    The validate_ method uses it to check the config file makes sense. If a
+    configspec is passed in when the ConfigObj is created, then it is parsed
+    and broken up to become the ``configspec`` attribute of each section.
+
+    If you didn't pass in a configspec, this attribute will be ``None`` on the
+    root section (the main ConfigObj).
+
+    You can set the configspec attribute directly on a section.
+
+    See the validation_ section for full details of how to write configspecs.
+
+Section Methods
+---------------
+
+* 'dict'
+
+    This method takes no arguments. It returns a deep copy of the section as a
+    dictionary. All subsections will also be dictionaries, and list values will
+    be copies, rather than references to the original [#]_.
+
+* rename
+
+    ``rename(oldkey, newkey)``
+
+    This method renames a key, without affecting its position in the sequence.
+
+    It is mainly implemented for the ``encode`` and ``decode`` methods, which
+    provide some Unicode support.
+
+* walk
+
+    This method can be used to transform values and names. See `walking a
+    section`_ for examples and explanation.
+
+* decode
+
+    ``decode(encoding)``
+
+    This method decodes names and values into Unicode objects, using the
+    supplied encoding.
+
+    Because of the way ConfigObj reads files, config files should be in an
+    ASCII compatible encoding. See encodings_ for more details.
+
+* encode
+
+    ``encode(encoding)``
+
+    This method is the opposite of ``decode`` {sm;:!:}.
+
+    It encodes names and values using the supplied encoding. If any of your
+    names/values are strings rather than Unicode, Python will have to do an
+    implicit decode first.
+
+    See encodings_ for more details.
+
+Walking a Section
+-----------------
+
+.. note::
+
+    The walk method allows you to call a function on every member/name.
+
+.. raw:: html
+
+    {+coloring}
+
+        walk(function, raise_errors=True,
+            call_on_sections=False, **keywargs):
+
+    {-coloring}
+
+``walk`` is a method of the ``Section`` object. This means it is also a method
+of ConfigObj.
+
+It walks through every member and calls a function on the keyword and value. It
+walks recursively through subsections.
+
+It returns a dictionary of all the computed values.
+
+If the function raises an exception, the default is to propagate the error, and
+stop. If ``raise_errors=False`` then it sets the return value for that keyword
+to ``False`` instead, and continues. This is similar to the way validation_
+works.
+
+Your function receives the arguments ``(section, key)``. The current value is
+then ``section[key]`` [#]_. Any unrecognised keyword arguments you pass to
+walk, are passed on to the function.
+
+Normally ``walk`` just recurses into subsections. If you are transforming (or
+checking) names as well as values, then you want to be able to change the names
+of sections. In this case set ``call_on_sections`` to ``True``. Now, on
+encountering a sub-section, *first* the function is called for the *whole*
+sub-section, and *then* it recurses into it's members. This means your function
+must be able to handle receiving dictionaries as well as strings and lists.
+
+If you are using the return value from ``walk`` *and* ``call_on_sections``,
+note that walk discards the return value when it calls your function.
+
+Examples
+--------
+
+Examples that use the walk method are the ``encode`` and ``decode`` methods.
+They both define a function and pass it to walk. Because these functions
+transform names as well as values (from byte strings to Unicode) they set
+``call_on_sections=True``.
+
+To see how they do it, *read the source Luke* {sm;:cool:}.
+
+You can use this for transforming all values in your ConfigObj. For example
+you might like the nested lists from ConfigObj 3. This was provided by the
+listquote_ module [#]_. You could switch off the parsing for list values
+(``list_values=False``) and use listquote to parse every value.
+
+Another thing you might want to do is use the Python escape codes in your
+values. You might be *used* to using ``\n`` for line feed and ``\t`` for tab.
+Obviously we'd need to decode strings that come from the config file (using the
+escape codes). Before writing out we'll need to put the escape codes back in
+encode.
+
+As an example we'll write a function to use with walk, that encodes or decodes
+values using the ``string-escape`` codec.
+
+The function has to take each value and set the new value. As a bonus we'll
+create one function that will do decode *or* encode depending on a keyword
+argument.
+
+We don't want to work with section names, we're only transforming values, so
+we can leave ``call_on_sections`` as ``False``. This means the two datatypes we
+have to handle are strings and lists, we can ignore everything else. (We'll
+treat tuples as lists as well).
+
+We're not using the return values, so it doesn't need to return anything, just
+change the values if appropriate.
+
+.. raw:: html
+
+    {+coloring}
+
+    def string_escape(section, key, encode=False):
+        """
+        A function to encode or decode using the 'string-escape' codec.
+        To be passed to the walk method of a ConfigObj.
+        By default it decodes.
+        To encode, pass in the keyword argument ``encode=True``.
+        """
+        val = section[key]
+        # is it a type we can work with
+        # NOTE: for platforms where Python > 2.2
+        # you can use basestring instead of (str, unicode)
+        if not isinstance(val, (str, unicode, list, tuple)):
+            # no !
+            return
+        elif isinstance(val, (str, unicode)):
+            # it's a string !
+            if not encode:
+                section[key] = val.decode('string-escape')
+            else:
+                section[key] = val.encode('string-escape')
+        else:
+            # it must be a list or tuple!
+            # we'll be lazy and create a new list
+            newval = []
+            # we'll check every member of the list
+            for entry in val:
+                if isinstance(entry, (str, unicode)):
+                    if not encode:
+                        newval.append(entry.decode('string-escape'))
+                    else:
+                       newval.append(entry.encode('string-escape'))
+                else:
+                    newval.append(entry)
+            # done !
+            section[key] =  newval
+
+    # assume we have a ConfigObj called ``config``
+    #
+    # To decode
+    config.walk(string_escape)
+    #
+    # To encode.
+    # Because ``walk`` doesn't recognise the ``encode`` argument
+    # it passes it to our function.
+    config.walk(string_escape, encode=True)
+
+    {-coloring}
+
+Exceptions
+==========
+
+There are several places where ConfigObj may raise exceptions (other than
+because of bugs).
+
+1) If a configspec filename you pass in doesn't exist, or a config file
+    filename doesn't exist *and* ``file_error=True``, an ``IOError`` will be
+    raised.
+
+2) If you try to set a non-string key, or a non string value when
+    ``stringify=False`, a ``TypeError`` will be raised.
+
+3) A badly built config file will cause parsing errors.
+
+4) A parsing error can also occur when reading a configspec.
+
+5) In string interpolation you can specify a value that doesn't exist, or
+    create circular references (recursion).
+
+6) If you have a ``__many__`` repeated section with other section definitions
+    (in a configspec), a ``RepeatSectionError`` will be raised.
+
+Number 5 (which is actually two different types of exceptions) is documented
+    in interpolation_.
+
+Number 6 is explained in the validation_ section.
+
+*This* section is about errors raised during parsing.
+
+The base error class is ``ConfigObjError``. This is a subclass of
+``SyntaxError``, so you can trap for ``SyntaxError`` without needing to
+directly import any of the ConfigObj exceptions.
+
+The following other exceptions are defined (all deriving from
+``ConfigObjError``) :
+
+* ``NestingError``
+
+    This error indicates either a mismatch in the brackets in a section marker,
+    or an excessive level of nesting.
+
+* ``ParseError``
+
+    This error indicates that a line is badly written. It is neither a valid
+    ``key = value`` line, nor a valid section marker line, nor a comment line.
+
+* ``DuplicateError``
+
+    The keyword or section specified already exists.
+
+* ``ConfigspecError``
+
+    An error occured whilst parsing a configspec.
+
+When parsing a configspec, ConfigObj will stop on the first error it
+encounters.  It will raise a ``ConfigspecError``. This will have an ``error``
+attribute, which is the actual error that was raised.
+
+Behavior when parsing a config file depends on the option ``raise_errors``.
+If ConfigObj encounters an error while parsing a config file:
+
+    If ``raise_errors=True`` then ConfigObj will raise the appropriate error
+    and parsing will stop.
+
+    If ``raise_errors=False`` (the default) then parsing will continue to the
+    end and *all* errors will be collected.
+
+In the second case a ``ConfigObjError`` is raised after parsing has stopped.
+The error raised has a ``config`` attribute, which is the parts of the
+ConfigObj that parsed successfully. It also has an attribute ``errors``, which
+is a list of *all* the errors raised. Each entry in the list is an instance of
+the appropriate error type. Each one has the following attributes (useful for
+delivering a sensible error message to your user) :
+
+* ``line``: the original line that caused the error.
+
+* ``line_number``: its number in the config file.
+
+* ``message``: the error message that accompanied the error.
+
+.. note::
+
+    One wrongly written line could break the basic structure of your config
+    file. This could cause every line after it to flag an error, so having a
+    list of all the lines that caused errors may not be as useful as it sounds.
+    {sm;:-(}.
+
+Validation
+==========
+
+Validation is done through a combination of the configspec_ and a ``Validator``
+object. For this you need *validate.py* [#]_. See downloading_ if you don't
+have a copy.
+
+Validation can perform two different operations :
+
+1) Check that a value meets a specification. For example, check that a value
+    is an integer between one and six, or is a choice from a specific set of
+    options.
+
+2) It can convert the value into the type required. For example, if one of
+    your values is a port number, validation will turn it into an integer for
+    you.
+
+So validation can act as a transparent layer between the datatypes of your
+application configuration (boolean, integers, floats, etc) and the text format
+of your config file.
+
+configspec
+----------
+
+The ``validate`` method checks members against an entry in the configspec. Your
+configspec therefore resembles your config file, with a check for every member.
+
+In order to perform validation you need a ``Validator`` object. This has
+several useful built-in check functions. You can also create your own custom
+functions and register them with your Validator object.
+
+Each check is the name of one of these functions, including any parameters and
+keyword arguments. The configspecs look like function calls, and they map to
+function calls.
+
+The basic datatypes that an un-extended Validator can test for are :
+
+* boolean values (True and False)
+* integers (including minimum and maximum values)
+* floats (including min and max)
+* strings (including min and max length)
+* IP addresses (v4 only)
+
+It can also handle lists of these types and restrict a value to being one from
+a set of options.
+
+An example configspec is going to look something like : ::
+
+    port = integer(0, 100)
+    user = string(max=25)
+    mode = option('quiet', 'loud', 'silent')
+
+You can specify default values, and also have the same configspec applied to
+several sections. This is called `repeated sections`_.
+
+For full details on writing configspecs, please refer to the `validate.py
+documentation`_.
+
+.. _validate.py documentation: http://www.voidspace.org.uk/python/validate.html
+
+Type Conversion
+---------------
+
+By default, validation does type conversion. This means that if you specify
+``integer`` as the check, then calling validate_ will actually change the value
+to an integer (so long as the check succeeds).
+
+It also means that when you call the write_ method, the value will be converted
+back into a string using the ``str`` function.
+
+To switch this off, and leave values as strings after validation, you need to
+set the stringify_ attribute to ``False``. If this is the case, attempting to
+set a value to a non-string will raise an error.
+
+Default Values
+--------------
+
+You can set a default value in your check. If the value is missing from the
+config file then this value will be used instead. This means that your user
+only has to supply values that differ from the defaults.
+
+If you *don't* supply a default then for a value to be missing is an error,
+and this will show in the `return value`_ from validate.
+
+Additionally you can set the default to be ``None``. This means the value will
+be set to ``None`` (the object) *whichever check is used*. (It will be set to
+``''`` rather than ``None`` if stringify_ is ``False``). You can use this
+to easily implement optional values in your config files. ::
+
+    port = integer(0, 100, default=80)
+    user = string(max=25, default=0)
+    mode = option('quiet', 'loud', 'silent', default='loud')
+    nick = string(default=None)
+
+.. note::
+
+    Because the default goes through type conversion, it also has to pass the
+    check.
+
+    Note that ``default=None`` is case sensitive.
+
+Repeated Sections
+-----------------
+
+Repeated sections are a way of specifying a configspec for a section that
+should be applied to *all* subsections in the same section.
+
+The easiest way of explaining this is to give an example. Suppose you have a
+config file that describes a dog. That dog has various attributes, but it can
+also have many fleas. You don't know in advance how many fleas there will be,
+or what they will be called, but you want each flea validated against the same
+configspec.
+
+We can define a section called *fleas*. We want every flea in that section
+(every sub-section) to have the same configspec applied to it. We do this by
+defining a single section called ``__many__``. ::
+
+    [dog]
+    name = string(default=Rover)
+    age = float(0, 99, default=0)
+
+        [[fleas]]
+
+            [[[__many__]]]
+            bloodsucker = boolean(default=True)
+            children = integer(default=10000)
+            size = option(small, tiny, micro, default=tiny)
+
+Every flea on our dog will now be validated using the ``__many__`` configspec.
+
+If you define another sub-section in a section *as well as* a ``__many__`` then
+you will get an error.
+
+``__many__`` sections can have sub-sections, including their own ``__many__``
+sub-sections. Defaults work in the normal way in repeated sections.
+
+
+Validation and Interpolation
+----------------------------
+
+String interpolation and validation don't play well together. When validation
+changes type it sets the value. If the value uses interpolation, then the 
+interpolation reference would normally be overwritten. Calling ``write`` would
+then use the absolute value and the interpolation reference would be lost.
+
+As a compromise - if the value is unchanged by validation then it is not reset.
+This means strings that pass through validation unmodified will not be 
+overwritten. If validation changes type - the value has to be overwritten, and
+any interpolation references are lost {sm;:-(}.
+
+SimpleVal
+---------
+
+You may not need a full validation process, but still want to check if all the
+expected values are present.
+
+Provided as part of the ConfigObj module is the ``SimpleVal`` object. This has
+a dummy ``test`` method that always passes.
+
+The only reason a test will fail is if the value is missing. The return value
+from ``validate`` will either be ``True``, meaning all present, or a dictionary
+with ``False`` for all missing values/sections.
+
+To use it, you still need to pass in a valid configspec when you create the
+ConfigObj, but just set all the values to ``''``. Then create an instance of
+``SimpleVal`` and pass it to the ``validate`` method.
+
+As a trivial example if you had the following config file : ::
+
+    # config file for an application
+    port = 80
+    protocol = http
+    domain = voidspace
+    top_level_domain = org.uk
+
+You would write the following configspec : ::
+
+    port = ''
+    protocol = ''
+    domain = ''
+    top_level_domain = ''
+
+.. raw:: html
+
+    {+coloring}
+
+    config = Configobj(filename, configspec=configspec)
+    val = SimpleVal()
+    test = config.validate(val)
+    if test == True:
+        print 'All values present.'
+    elif test == False:
+        print 'No values present!'
+    else:
+        for entry in test:
+            if test[entry] == False:
+                print '"%s" missing.' % entry
+
+    {-coloring}
+
+Interpolation
+=============
+
+ConfigObj allows string interpolation *similar* to the way ``ConfigParser``
+
+You specify a value to be substituted by including ``%(name)s`` in the value.
+
+Interpolation checks first the 'DEFAULT' sub-section of the current section to
+see if ``name`` is the key to a value. ('name' is case sensitive).
+
+If it doesn't find it, next it checks the 'DEFAULT' section of the parent
+section, last it checks the 'DEFAULT' section of the main section.
+
+If the value specified isn't found then a ``MissingInterpolationOption`` error
+is raised (a subclass of ``ConfigObjError``).
+
+If it is found then the returned value is also checked for substitutions. This
+allows you to make up compound values (for example directory paths) that use
+more than one default value. It also means it's possible to create circular
+references. If after ten replacements there are still values to substitute, an
+``InterpolationDepthError`` is raised.
+
+Both of these errors are subclasses of ``InterpolationError``, which is a
+subclass of ``ConfigObjError``.
+
+String interpolation and validation don't play well together. This is because 
+validation overwrites values - and so may erase the interpolation references.
+See `Validation and Interpolation`_. (This can only happen if validation
+has to *change* the value).
+
+Comments
+========
+
+Any line that starts with a '#', possibly preceded by whitespace, is a comment.
+
+If a config file starts with comments then these are preserved as the
+initial_comment_.
+
+If a config file ends with comments then these are preserved as the
+final_comment_.
+
+Every key or section marker may have lines of comments immediately above it.
+These are saved as the ``comments`` attribute of the section. Each member is a
+list of lines.
+
+You can also have a comment inline with a value. These are saved as the
+``inline_comments`` attribute of the section, with one entry per member of the
+section.
+
+Subsections (section markers in the config file) can also have comments.
+
+See `Section Attributes`_ for more on these attributes.
+
+These comments are all written back out by the ``write`` method.
+
+Encodings
+=========
+
+ConfigObj 4 is designed to work with ASCII compatible encodings [#]_. If you
+need support for other character sets, then I suggest you use the UTF8
+encoding for your config files.
+
+By default ConfigObj leaves keys/members as encoded byte strings (ordinary
+strings). If you want to access the config file members as Unicode objects
+rather than strings, you can use the ``decode`` method. This takes an encoding
+as its argument and decodes all members and keys into Unicode. It will only
+work if *all* members are byte strings (or lists of strings) , so you should do
+it before calling ``validate``.
+
+If you want to turn the Unicode strings back into byte strings, you can call
+the ``encode`` method. This also takes an encoding as its argument and assumes
+*all* keys/members are Unicode.
+
+If you start working with Unicode strings, you may find you get
+``UnicodeDecodeError`` or ``UnicodeEncodeError`` in unexpected places. This is
+because you have forced Python to do an *implicit* encode or decode.
+
+Implicit decodes (and encodes) use the encoding returned by
+``sys.getdefaultencoding()``. This is *usually* ASCII. This means that if you
+have any non-ASCII characters, Python doesn't know how to treat them and will
+raise an error.
+
+This happens if you add a byte string to a Unicode string, compare a byte
+string to a Unicode string, print a Unicode string, or write it to a file. If
+you work with Unicode, you should do the appropriate encode or decode first.
+
+Backwards Compatibility
+=======================
+
+There have been a lot of changes since ConfigObj 3. The core parser is now
+based on regular expressions, and is a lot faster and smaller. There is now no
+difference in the way we treat flat files and non-flatfiles, that is, no empty
+sections. This means some of the code can be a lot simpler, less code does
+more of the work [#]_.
+
+There have been other simplifications: for example we only have eight options
+instead of seventeen.
+
+Most config files created for ConfigObj 3 will be read with no changes and many
+programs will work without having to alter code. Some of the changes do break
+backwards compatibility: for example, code that uses the previous options will
+now raise an error. It should be very easy to fix these, though.
+
+Below is a list of all the changes that affect backwards compatibility. This
+doesn't include details of method signatures that have changed, because almost
+all of them have.
+
+Incompatible Changes
+--------------------
+
+(I have removed a lot of needless complications: this list is probably not
+conclusive, many option/attribute/method names have changed.)
+
+Case sensitive.
+
+The only valid divider is '='.
+
+Line continuations with ``\`` removed.
+
+No recursive lists in values.
+
+No empty sections.
+
+No distinction between flatfiles and non flatfiles.
+
+Change in list syntax: use commas to indicate list, not parentheses (square
+brackets and parentheses are no longer recognised as lists).
+
+';' is no longer valid for comments, and no multiline comments.
+
+No attribute-style access to values.
+
+Empty values not allowed: use '' or "".
+
+In ConfigObj 3, setting a non-flatfile member to ``None`` would initialise it
+as an empty section.
+
+The escape entities '&mjf-lf;' and '&mjf-quot;' have gone, replaced by triple
+quote, multiple line values.
+
+The ``newline``, ``force_return``, and ``default`` options have gone.
+
+The ``encoding`` and ``backup_encoding`` methods have gone, replaced with the
+``encode`` and ``decode`` methods.
+
+``fileerror`` and ``createempty`` options have become ``file_error`` and
+``create_empty``.
+
+Partial configspecs (for specifying the order members should be written out,
+and which should be present) have gone. The configspec is no longer used to
+specify order for the ``write`` method.
+
+Exceeding the maximum depth of recursion in string interpolation now raises an
+error ``InterpolationDepthError``.
+
+Specifying a value for interpolation which doesn't exist now raises a
+``MissingInterpolationOption`` error (instead of merely being ignored).
+
+The ``writein`` method has been removed.
+
+The comments attribute is now a list (``inline_comments`` equates to the old
+comments attribute).
+
+ConfigObj 3
+-----------
+
+ConfigObj 3 is now deprecated in favour of ConfigObj 4. I can fix bugs in
+ConfigObj 3 if needed, though.
+
+For anyone who still needs it, you can download it here: `ConfigObj 3.3.1`_
+
+You can read the old docs at : `ConfigObj 3 Docs`_
+
+.. _ConfigObj 3.3.1: http://www.voidspace.org.uk/cgi-bin/voidspace/downman.py?file=configobj3.zip
+.. _ConfigObj 3 Docs: http://www.voidspace.org.uk/python/configobj3.html
+
+CREDITS
+=======
+
+ConfigObj 4 is written by (and copyright) `Michael Foord`_ and 
+`Nicola Larosa`_.
+
+Particularly thanks to Nicola Larosa for help on the config file spec, the
+validation system and the doctests.
+
+*validate.py* was originally written by Michael Foord and `Mark Andrews`_.
+
+Thanks to others for input and bugfixes.
+
+LICENSE
+=======
+
+ConfigObj, and related files, are licensed under the BSD license. This is a
+very unrestrictive license, but it comes with the usual disclaimer. This is
+free software: test it, break it, just don't blame us if it eats your data !
+Of course if it does, let us know and we'll fix the problem so it doesn't
+happen to anyone else {sm;:-)}. ::
+
+    Copyright (c) 2004 & 2005, Michael Foord & Nicola Larosa
+    All rights reserved.
+
+    Redistribution and use in source and binary forms, with or without
+    modification, are permitted provided that the following conditions are
+    met:
+
+
+        * Redistributions of source code must retain the above copyright
+          notice, this list of conditions and the following disclaimer.
+
+        * Redistributions in binary form must reproduce the above
+          copyright notice, this list of conditions and the following
+          disclaimer in the documentation and/or other materials provided
+          with the distribution.
+
+        * Neither the name of Michael Foord nor Nicola Larosa
+          may be used to endorse or promote products derived from this
+          software without specific prior written permission.
+
+    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+    "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+    LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+    A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+    OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+    SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
+    LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
+    DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
+    THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
+    OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+
+You should also be able to find a copy of this license at : `BSD License`_
+
+.. _BSD License: http://www.voidspace.org.uk/documents/BSD-LICENSE.txt
+
+TODO
+====
+
+Fix any bugs (and resolvable issues).
+
+Do an example for the 'walk' which removes uniform indentation in multiline
+values.
+
+When initialising a section from a ConfigObj *or* an ``OrderedDictionary``
+we could preserve ordering.
+
+Add an *odict* method which returns an `OrderedDictionary``.
+
+ISSUES
+======
+
+.. note::
+
+    Please file any bug reports to `Michael Foord`_ or the ConfigObj
+    `Mailing List`_.
+
+You can't have a keyword with the same name as a section (in the same section).
+They are both dictionary keys, so they would overlap.
+
+Interpolation checks first the 'DEFAULT' sub-section of the current section,
+next it checks the 'DEFAULT' section of the parent section, last it checks the
+'DEFAULT' section of the main section.
+
+Logically a 'DEFAULT' section should apply to all subsections of the *same
+parent*: this means that checking the 'DEFAULT' sub-section in the *current
+section* is not necessarily logical ?
+
+In order to simplify Unicode support (which is possibly of limited value
+in a config file ?) I have removed automatic support, and added thev``encode``
+and ``decode`` methods. These can be used to transform keys and entries.
+Because the regex looks for specific values on inital parsing (i.e. the quotes
+and the equals signs) it can only read ASCII compatible encodings. For Unicode
+I suggest ``UTF8``, which is ASCII compatible.
+
+.. note::
+
+    There is no reason why you shouldn't decode your config file into Unicode
+    before passing them to ConfigObj (as a list of lines). This should give you
+    Unicode keys and values.
+
+Does it matter that we don't support the ':' divider, which is supported by
+``ConfigParser`` ?
+
+Following error with "list_values=False" : ::
+
+    >>> a = ["a='hello', 'goodbye'"]
+    >>>
+    >>> c(a, list_values=False)
+    {'a': "hello', 'goodbye"}
+
+The regular expression correctly removes the value - ``"'hello', 'goodbye'"`` -
+and then unquote just removes the front and back quotes (called by
+``_handle_value``). What should we do ?? We *ought* to raise an exception - 
+because if lists are off it's an invalid value. This is not desired if you want 
+to do your own list processing e.g. using listquote for nested lists. It would
+be *better* in this case not to unquote. Raising an exception would require a
+new regex.
+
+String interpolation and validation don't play well together. See 
+`Validation and Interpolation`_.
+
+CHANGELOG
+=========
+
+This is an abbreviated changelog showing the major releases up to version 4.
+From version 4 it lists all releases and changes. *More* data on individual
+changes may be found in the source code.
+
+2005/10/17 - Version 4.0.0
+--------------------------
+
+**ConfigObj 4.0.0 Final**
+
+Fixed bug in ``setdefault``. When creating a new section with setdefault the
+reference returned would be to the dictionary passed in *not* to the new 
+section. Bug fixed and behaviour documented.
+
+Obscure typo/bug fixed in ``write``. Wouldn't have affected anyone though.
+
+2005/09/09 - Version 4.0.0 beta 5
+---------------------------------
+
+Removed ``PositionError``.
+
+Allowed quotes around keys as documented.
+
+Fixed bug with commas in comments. (matched as a list value)
+
+2005/09/07 - Version 4.0.0 beta 4
+---------------------------------
+
+Fixed bug in ``__delitem__``. Deleting an item no longer deletes the 
+``inline_comments`` attribute.
+
+Fixed bug in initialising ConfigObj from a ConfigObj.
+
+Changed the mailing list address.
+
+2005/08/28 - Version 4.0.0 beta 3
+---------------------------------
+
+Interpolation is switched off before writing out files.
+
+Fixed bug in handling ``StringIO`` instances. (Thanks to report from
+"Gustavo Niemeyer" <gustavo@niemeyer.net>)
+
+Moved the doctests from the ``__init__`` method to a separate function.
+(For the sake of IDE calltips).
+
+2005/08/25 - Version 4.0.0 beta 2
+---------------------------------
+
+Amendments to *validate.py*.
+
+Official release.
+
+2005/08/21 - Version 4.0.0 beta 1
+---------------------------------
+
+Reads nested subsections to any depth.
+
+Multiline values.
+
+Simplified options and methods.
+
+New list syntax.
+
+Faster, smaller, and better parser.
+
+Validation greatly improved. Includes:
+
+    * type conversion
+    * default values
+    * repeated sections
+
+Improved error handling.
+
+Plus lots of other improvements {sm;:grin:}.
+
+2004/05/24 - Version 3.0.0
+--------------------------
+
+Several incompatible changes: another major overhaul and change. (Lots of
+improvements though).
+
+Added support for standard config files with sections. This has an entirely
+new interface: each section is a dictionary of values.
+
+Changed the update method to be called writein: update clashes with a dict
+method.
+
+Made various attributes keyword arguments, added several.
+
+Configspecs and orderlists have changed a great deal.
+
+Removed support for adding dictionaries: use update instead.
+
+Now subclasses a new class called caselessDict. This should add various
+dictionary methods that could have caused errors before.
+
+It also preserves the original casing of keywords when writing them back out.
+
+Comments are also saved using a ``caselessDict``.
+
+Using a non-string key will now raise a ``TypeError`` rather than converting 
+the key.
+
+Added an exceptions keyword for *much* better handling of errors.
+
+Made ``creatempty=False`` the default.
+
+Now checks indict *and* any keyword args. Keyword args take precedence over
+indict.
+
+``' ', ':', '=', ','`` and ``'\t'`` are now all valid dividers where the 
+keyword is unquoted.
+
+ConfigObj now does no type checking against configspec when you set items.
+
+delete and add methods removed (they were unnecessary).
+
+Docs rewritten to include all this gumph and more; actually ConfigObj is
+*really* easy to use.
+
+Support for stdout was removed.
+
+A few new methods added.
+
+Charmap is now incorporated into ConfigObj.
+
+2004/03/14 - Version 2.0.0 beta
+-------------------------------
+
+Re-written it to subclass dict. My first forays into inheritance and operator
+overloading.
+
+The config object now behaves like a dictionary.
+
+I've completely broken the interface, but I don't think anyone was really
+ using it anyway.
+
+This new version is much more 'classy' {sm;:wink:}
+
+It will also read straight from/to a filename and completely parse a config
+file without you *having* to supply a config spec.
+
+Uses listparse, so can handle nested list items as values.
+
+No longer has getval and setval methods: use normal dictionary methods, or add
+and delete.
+
+2004/01/29 - Version 1.0.5
+--------------------------
+
+Version 1.0.5 has a couple of bugfixes as well as a couple of useful additions
+over previous versions.
+
+Since 1.0.0 the buildconfig function has been moved into this distribution,
+and the methods reset, verify, getval and setval have been added.
+
+A couple of bugs have been fixed.
+
+Origins
+-------
+
+ConfigObj originated in a set of functions for reading config files in the
+atlantibots_ project. The original functions were written by Rob McNeur...
+
+.. _atlantibots: http://www.voidspace.org.uk/atlantibots
+
+----------
+
+Footnotes
+=========
+
+.. [#] The core parser is now based on regular expressions, so it's a lot
+    faster.
+
+.. [#] 134 of them, at the time of writing.
+
+.. [#] And if you discover any bugs, let us know. We'll fix them quickly.
+
+.. [#] If you specify a filename that doesn't exist, ConfigObj will assume you
+    are creating a new one. See the *create_empty* and *file_error* options_.
+
+.. [#] They can be byte strings ('ordinary' strings) or Unicode. If they are
+    Unicode then ConfigObj will have to do an implicit encode before writing.
+    See the encodings_ section for more details.
+
+.. [#] Except we don't support the RFC822 style line continuations, nor ':' as
+    a divider.
+
+.. [#] For a file object that will depend what mode it was opened with. You
+    can read *and* write to a ``StringIO`` instance, but not always to a
+    ``cStringIO`` instance.
+
+.. [#] A side effect of this is that it enables you to copy a ConfigObj :
+
+.. raw:: html
+
+    {+coloring}
+
+    # only copies members
+    # not attributes/comments
+    config2 = ConfigObj(config1)
+
+    {-coloring}
+
+..
+
+    The order of values and sections will not be preserved, though.
+
+.. [#] Other than lists of strings.
+
+.. [#] The method signature in the API docs will show that it does in fact
+    take one argument: the section to be written. This is because the
+    ``write`` method is called recursively. Using this argument *forces* write
+    to return a list of lines, so it's probably not very useful to you.
+
+.. [#] The dict method doesn't actually use the deepcopy mechanism. This means
+    if you add nested lists (etc) to your ConfigObj, then the dictionary
+    returned by dict may contain some references. For all *normal* ConfigObjs
+    it will return a deepcopy.
+
+.. [#] Passing ``(section, key)`` rather than ``(value, key)`` allows you to
+    change the value by setting ``section[key] = newval``. It also gives you
+    access to the *rename* method of the section.
+
+.. [#] A current bug in unquoting when ``list_values=False`` currently makes
+    this impossible. We are considering the best way to fix this.
+
+.. [#] Minimum required version of *validate.py* 0.2.0 .
+
+.. [#] There's nothing to stop you decoding the whole config file to Unicode
+    *first*.
+
+.. [#] It also makes ConfigObj a lot simpler to *use*.
+
+.. note::
+
+    Rendering this document with docutils also needs the
+    textmacros module and the PySrc CSS stuff. See
+    http://www.voidspace.org.uk/python/firedrop2/textmacros.shtml
+
+.. raw:: html
+
+    <div align="center">
+        <a href="http://sourceforge.net/donate/index.php?group_id=123265">
+            <img src="http://images.sourceforge.net/images/project-support.jpg" width="88" height="32" border="0" alt="Support This Project" /> 
+        </a>
+        <a href="http://sourceforge.net">
+            <img src="http://sourceforge.net/sflogo.php?group_id=123265&amp;type=1" width="88" height="31" border="0" alt="SourceForge.net Logo" />
+        </a>
+        <br />
+        <a href="http://www.python.org">
+            <img src="images/powered_by_python.jpg" width="602" height="186" border="0" />
+        </a>
+        <a href="http://www.opensource.org">
+            <img src="images/osi-certified-120x100.gif" width="120" height="100" border="0" />
+            <br /><strong>Certified Open Source</strong>
+        </a>
+        <br /><br />
+        <script type="text/javascript" language="JavaScript">var site="s16atlantibots"</script>
+        <script type="text/javascript" language="JavaScript1.2" src="http://s16.sitemeter.com/js/counter.js?site=s16atlantibots"></script>
+        <noscript>
+            <a href="http://s16.sitemeter.com/stats.asp?site=s16atlantibots">
+                <img src="http://s16.sitemeter.com/meter.asp?site=s16atlantibots" alt="Site Meter" border=0 />
+            </a>
+        </noscript>
+        <br />
+    </div>
+
+.. _listquote: http://www.voidspace.org.uk/python/modules.shtml#listquote
+.. _Michael Foord: http://www.voidspace.org.uk
+.. _Nicola Larosa: http://www.teknico.net
+.. _Mark Andrews: http://www.la-la.com

=== added file 'bzrlib/util/configobj/docs/validate.txt'
--- bzrlib/util/configobj/docs/validate.txt	1970-01-01 00:00:00 +0000
+++ bzrlib/util/configobj/docs/validate.txt	2005-10-18 18:48:27 +0000
@@ -0,0 +1,599 @@
+===================================
+ Validation Schema with validate.py
+===================================
+
+--------------------------
+ Using the Validator class
+--------------------------
+
+
+:Authors: `Michael Foord`_, `Nicola Larosa`_, `Mark Andrews`_
+:Version: Validate 0.2.0
+:Date: 2005/08/25
+:Homepage: `Validate Homepage`_
+:License: `BSD License`_
+:Support: `Mailing List`_
+
+.. _Mailing List: http://lists.sourceforge.net/lists/listinfo/configobj-develop
+.. _Michael Foord: fuzzyman@voidspace.org.uk
+.. _Nicola Larosa: nico@teknico.net
+.. _Mark Andrews: mark@la-la.com
+.. _This Document:
+.. _Validate Homepage: http://www.voidspace.org.uk/python/validate.html
+.. _BSD License: http://www.voidspace.org.uk/documents/BSD-LICENSE.txt
+
+
+.. contents:: Validate Manual
+
+Introduction
+============
+
+Validation is used to check that supplied values conform to a specification.
+
+The value can be supplied as a string, e.g. from a config file. In this case
+the check will also *convert* the value to the required type. This allows you
+to add validation as a transparent layer to access data stored as strings. The
+validation checks that the data is correct *and* converts it to the expected
+type.
+
+Checks are also strings, and are easy to write. One generic system can be used
+to validate information from different sources, via a single consistent
+mechanism.
+
+Checks look like function calls, and map to function calls. They can include
+parameters and keyword arguments. These arguments are passed to the relevant
+function by the ``Validator`` instance, along with the value being checked.
+
+The syntax for checks also allows for specifying a default value. This default
+value can be ``None``, no matter what the type of the check. This can be used
+to indicate that a value was missing, and so holds no useful value.
+
+Functions either return a new value, or raise an exception. See `Validator
+Exceptions`_ for the low down on the exception classes that ``validate.py``
+defines.
+
+Some standard functions are provided, for basic data types; these come built
+into every validator. Additional checks are easy to write: they can be provided
+when the ``Validator`` is instantiated, or added afterwards.
+
+Validate was primarily written to support ConfigObj_, but was designed to be 
+applicable to many other situations.
+
+For support and bug reports please use the ConfigObj `Mailing List`_.
+
+.. _ConfigObj: http://www.voidspace.org.uk/python/configobj.html
+
+Downloading
+===========
+
+The current version is **0.2.0**, dated 25th August 2005. 
+
+You can get obtain validate in the following ways :
+
+Files
+-----
+
+* validate.py_ from Voidspace
+
+* configobj.zip from Voidspace - See the homepage of ConfigObj_ for the latest 
+  version and downlaod links.
+
+    This contains validate.py and `this document`_. (As well as ConfigObj_ and 
+    the ConfigObj documentation).
+    
+* The latest development version can be obtained from the `Subversion Repository`_.
+
+Documentation
+-------------
+
+*configobj.zip* contains `this document`_ and full `API Docs`_, generated 
+automatically by EpyDoc_.
+
+* You can view `this document`_ online as the `Validate Homepage`_.
+
+* You can also browse the `API Docs`_ online.
+
+Pythonutils
+-----------
+
+Validate_ is also part of the Pythonutils_ set of modules. This contains 
+various other useful helper modules, and is required by many of the `Voidspace 
+Python Projects`_.
+
+.. _configobj.py: http://www.voidspace.org.uk/cgi-bin/voidspace/downman.py?file=configobj.py
+.. _configobj.zip: http://www.voidspace.org.uk/cgi-bin/voidspace/downman.py?file=configobj-4.0.0b4.zip
+.. _validate.py: http://www.voidspace.org.uk/cgi-bin/voidspace/downman.py?file=validate.py
+.. _API Docs: http://www.voidspace.org.uk/python/configobj-api/
+.. _Subversion Repository: http://svn.rest2web.python-hosting.com/branches/configobj4/
+.. _Sourceforge: http://sourceforge.net/projects/configobj
+.. _EpyDoc: http://epydoc.sourceforge.net
+.. _pythonutils: http://www.voidspace.org.uk/python/pythonutils.html
+.. _Voidspace Python Projects: http://www.voidspace.org.uk/python
+.. _validate: http://www.voidspace.org.uk/python/validate.html
+
+
+The standard functions
+======================
+
+The standard functions come built-in to every ``Validator`` instance. They work
+with the following basic data types :
+
+* integer
+* float
+* boolean
+* string
+* ip_addr
+
+plus lists of these datatypes.
+
+Adding additional checks is done through coding simple functions.
+
+The full set of standard checks are :
+
+:'integer': matches integer values (including negative). Takes optional 'min'
+            and 'max' arguments : ::
+
+                integer()
+                integer(3, 9)    # any value from 3 to 9
+                integer(min=0) # any positive value
+                integer(max=9)
+
+:'float': matches float values
+          Has the same parameters as the integer check.
+
+:'bool': matches boolean values: ``True`` or ``False``.
+         Acceptable string values for True are : ::
+
+             true, on, yes, 1
+
+         Acceptable string values for False are : ::
+
+             false, off, no, 0
+
+         Any other value raises an error.
+
+:'string': matches any string. Takes optional keyword args 'min' and 'max' to
+           specify min and max length of string.
+
+:'ip_addr': matches an Internet Protocol address, v.4, represented by a
+            dotted-quad string, i.e. '1.2.3.4'.
+
+:'list': matches any list. Takes optional keyword args 'min', and 'max' to
+         specify min and max sizes of the list.
+
+:'int_list': Matches a list of integers. Takes the same arguments as list.
+
+:'float_list': Matches a list of floats. Takes the same arguments as list.
+
+:'bool_list': Matches a list of boolean values. Takes the same arguments as
+              list.
+
+:'string_list': Matches a list of strings. Takes the same arguments as list.
+
+:'ip_addr_list': Matches a list of IP addresses. Takes the same arguments as
+                 list.
+
+:'mixed_list': Matches a list with different types in specific positions.
+               List size must match the number of arguments.
+
+               Each position can be one of : ::
+
+                   int, str, bool, float, ip_addr
+
+               So to specify a list with two strings followed by two integers,
+               you write the check as : ::
+
+                   mixed_list(str, str, int, int)
+
+:'pass': matches everything: it never fails and the value is unchanged. It is
+         also the default if no check is specified.
+
+:'option': matches any from a list of options.
+           You specify this test with : ::
+
+               option('option 1', 'option 2', 'option 3')
+
+The following code will work without you having to specifically add the
+functions yourself.
+
+.. raw:: html
+
+    {+coloring}
+
+    from validate import Validator
+    #
+    vtor = Validator()
+    newval1 = vtor.check(value1, 'integer')
+    newval2 = vtor.check(value2, 'bool')
+    # etc ...
+
+    {-coloring}
+
+.. note::
+
+    Of course, if these checks fail they raise exceptions. So you should wrap
+    them in ``try...except`` blocks. Better still,  use ConfigObj for a higher
+    level interface.
+
+Using Validator
+===============
+
+Using ``Validator`` is very easy. It has one public attribute and one public
+method.
+
+Shown below are the different steps in using ``Validator``.
+
+The only additional thing you need to know, is about `Writing check
+functions`_.
+
+Instantiate
+-----------
+
+.. raw:: html
+
+    {+coloring}
+
+    from validate import Validator
+    vtor = Validator()
+
+    {-coloring}
+
+or even :
+
+.. raw:: html
+
+    {+coloring}
+
+    from validate import Validator
+    #
+    fdict = {
+        'check_name1': function1,
+        'check_name2': function2,
+        'check_name3': function3,
+    }
+    #
+    vtor = Validator(fdict)
+
+    {-coloring}
+
+The second method adds a set of your functions as soon as your validator is
+created. They are stored in the ``vtor.functions`` dictionary. The 'key' you
+give them in this dictionary is the name you use in your checks (not the
+original function name).
+
+Dictionary keys/functions you pass in can override the built-in ones if you
+want.
+
+Adding functions
+----------------
+
+The code shown above, for adding functions on instantiation, has exactly the
+same effect as the following code :
+
+.. raw:: html
+
+    {+coloring}
+
+    from validate import Validator
+    #
+    vtor = Validator()
+    vtor.functions['check_name1'] = function1
+    vtor.functions['check_name2'] = function2
+    vtor.functions['check_name3'] = function3
+
+    {-coloring}
+
+``vtor.functions``is just a dictionary that maps names to functions, so we
+could also have called ``vtor.functions.update(fdict)``.
+
+Writing the check
+-----------------
+
+As we've heard, the checks map to the names in the ``functions`` dictionary.
+You've got a full list of `The standard functions`_ and the arguments they
+take.
+
+If you're using ``Validator`` from ConfigObj, then your checks will look like
+: ::
+
+    keyword = int_list(max=6)
+
+but the check part will be identical .
+
+The check method
+----------------
+
+If you're not using ``Validator`` from ConfigObj, then you'll need to call the
+``check`` method yourself.
+
+If the check fails then it will raise an exception, so you'll want to trap
+that. Here's the basic example :
+
+.. raw:: html
+
+    {+coloring}
+
+    from validate import Validator, ValidateError
+    #
+    vtor = Validator()
+    check = "integer(0, 9)"
+    value = 3
+    try:
+        newvalue = vtor.check(check, value)
+    except ValidateError:
+        print 'Check Failed.'
+    else:
+        print 'Check passed.'
+
+    {-coloring}
+
+.. caution::
+
+    Although the value can be a string, if it represents a list it should
+    already have been turned into a list of strings.
+
+Default Values
+~~~~~~~~~~~~~~
+
+Some values may not be available, and you may want to be able to specify a
+default as part of the check.
+
+You do this by passing the keyword ``missing=True`` to the ``check`` method, as
+well as a ``default=value`` in the check. (Constructing these checks is done
+automatically by ConfigObj: you only need to know about the ``default=value``
+part) :
+
+.. raw:: html
+
+    {+coloring}
+
+    check1 = 'integer(default=50)'
+    check2 = 'option("val 1", "val 2", "val 3", default="val 1")'
+
+    assert vtor.check('', check1, missing=True) == 50
+    assert vtor.check('', check2, missing=True) == "val 1"
+
+    {-coloring}
+
+If you pass in ``missing=True`` to the check method, then the actual value is
+ignored. If no default is specified in the check, a ``ValidateMissingValue``
+exception is raised. If a default is specified then that is passed to the
+check instead.
+
+If the check has ``default=None`` (case sensitive) then ``vtor.check`` will
+*always* return ``None`` (the object). This makes it easy to tell your program
+that this check contains no useful value when missing, i.e. the value is
+optional, and may be omitted without harm.
+
+Validator Exceptions
+====================
+
+.. note::
+
+    If you only use Validator through ConfigObj, it traps these Exceptions for
+    you. You will still need to know about them for writing your own check
+    functions.
+
+``vtor.check`` indicates that the check has failed by raising an exception.
+The appropriate error should be raised in the check function.
+
+The base error class is ``ValidateError``. All errors (except for ``VdtParamError``) 
+raised are sub-classes of this.
+
+If an unrecognised check is specified then ``VdtUnknownCheckError`` is
+raised.
+
+There are also ``VdtTypeError`` and ``VdtValueError``.
+
+If incorrect parameters are passed to a check function then it will (or should)
+raise ``VdtParamError``. As this indicates *programmer* error, rather than an error
+in the value, it is a subclass of ``SyntaxError`` instead of ``ValidateError``. 
+
+.. note::
+
+    This means it *won't* be caught by ConfigObj - but propagated instead.
+
+If the value supplied is the wrong type, then the check should raise
+``VdtTypeError``. e.g. the check requires the value to be an integer (or
+representation of an integer) and something else was supplied.
+
+If the value supplied is the right type, but an unacceptable value, then the
+check should raise ``VdtValueError``. e.g. the check requires the value to
+be an integer (or representation of an integer) less than ten and a higher
+value was supplied.
+
+Both ``VdtTypeError`` and ``VdtValueError`` are initialised with the
+incorrect value. In other words you raise them like this :
+
+.. raw:: html
+
+    {+coloring}
+
+    raise VdtTypeError(value)
+    #
+    raise VdtValueError(value)
+
+    {-coloring}
+
+``VdtValueError`` has the following subclasses, which should be raised if
+they are more appropriate.
+
+* ``VdtValueTooSmallError``
+* ``VdtValueTooBigError``
+* ``VdtValueTooShortError``
+* ``VdtValueTooLongError``
+
+Writing check functions
+=======================
+
+Writing check functions is easy.
+
+The check function will receive the value as its first argument, followed by
+any other parameters and keyword arguments.
+
+If the check fails, it should raise a ``VdtTypeError`` or a
+``VdtValueError`` (or an appropriate subclass).
+
+All parameters and keyword arguments are *always* passed as strings. (Parsed
+from the check string).
+
+The value might be a string (or list of strings) and need
+converting to the right type - alternatively it might already be a list of 
+integers. Our function needs to be able to handle either.
+
+If the check passes then it should return the value (possibly converted to the
+right type).
+
+And that's it !
+
+Example
+-------
+
+Here is an example function that requires a list of integers. Each integer
+must be between 0 and 99.
+
+It takes a single argument specifying the length of the list. (Which allows us
+to use the same check in more than one place). If the length can't be converted
+to an integer then we need to raise ``VdtParamError``.
+
+Next we check that the value is a list. Anything else should raise a
+``VdtTypeError``. The list should also have 'length' entries. If the list
+has more or less entries then we will need to raise a
+``VdtValueTooShortError`` or a ``VdtValueTooLongError``.
+
+Then we need to check every entry in the list. Each entry should be an integer
+between 0 and 99, or a string representation of an integer between 0 and 99.
+Any other type is a ``VdtTypeError``, any other value is a
+``VdtValueError`` (either too big, or too small).
+
+.. raw:: html
+
+    {+coloring}
+
+    def special_list(value, length):
+        """
+        Check that the supplied value is a list of integers,
+        with 'length' entries, and each entry between 0 and 99.
+        """
+        # length is supplied as a string
+        # we need to convert it to an integer
+        try:
+            length = int(length)
+        except ValueError:
+            raise VdtParamError('length', length)
+        #
+        # Check the supplied value is a list
+        if not isinstance(value, list):
+            raise VdtTypeError(value)
+        #
+        # check the length of the list is correct
+        if len(value) > length:
+            raise VdtValueTooLongError(value)
+        elif len(value) < length:
+            raise VdtValueTooShortError(value)
+        #
+        # Next, check every member in the list
+        # converting strings as necessary
+        out = []
+        for entry in value:
+            if not isinstance(entry, (str, unicode, int)):
+                # a value in the list
+                # is neither an integer nor a string
+                raise VdtTypeError(value)
+            elif isinstance(entry, (str, unicode)):
+                if not entry.isdigit():
+                    raise VdtTypeError(value)
+                else:
+                    entry = int(entry)
+            if entry < 0:
+                raise VdtValueTooSmallError(value)
+            elif entry > 99:
+                raise VdtValueTooBigError(value)
+            out.append(entry)
+        #
+        # if we got this far, all is well
+        # return the new list
+        return out
+
+    {-coloring}
+
+If you are only using validate from ConfigObj then the error type (*TooBig*, 
+*TooSmall*, etc) is lost - so you may only want to raise ``VdtValueError``.
+
+.. caution::
+
+    If your function raises an exception that isn't a subclass of 
+    ``ValidateError``, then ConfigObj won't trap it. This means validation will 
+    fail.
+    
+    This is why our function starts by checking the type of the value. If we 
+    are passed the wrong type (e.g. an integer rather than a list) we get a 
+    ``VdtTypeError`` rather than bombing out when we try to iterate over 
+    the value.
+
+If you are using validate in another circumstance you may want to create your 
+own subclasses of ``ValidateError``, that convey more specific information.
+
+TODO
+====
+
+* A regex check function ?
+* A timestamp check function ? (Using the ``parse`` function from ``DateUtil``).
+* Allow triple quotes ? (getting a bit heavy for a single regex)
+
+ISSUES
+======
+
+.. note::
+
+    Please file any bug reports to `Michael Foord`_ or the ConfigObj
+    `Mailing List`_.
+
+Lists passed as function arguments need additional quotes. Ugly, could do
+with fixing this.
+
+If we could pull tuples out of arguments, it would be easier
+to specify arguments for 'mixed_lists'.
+
+CHANGELOG
+=========
+
+
+2005/08/18      Version 0.2.0
+-----------------------------
+
+Updated by `Michael Foord`_ and `Nicola Larosa`_
+
+Does type conversion as well.
+
+2005/02/01      Version 0.1.0
+-----------------------------
+
+Initial version developed by `Michael Foord`_
+and `Mark Andrews`_
+
+.. note::
+
+    Rendering this document with docutils also needs the
+    textmacros module and the PySrc CSS stuff. See
+    http://www.voidspace.org.uk/python/firedrop2/textmacros.shtml
+
+.. raw:: html
+
+    <div align="center">
+        <a href="http://www.python.org">
+            <img src="images/powered_by_python.jpg" width="602" height="186" border="0" />
+        </a>
+        <a href="http://www.opensource.org">
+            <img src="images/osi-certified-120x100.gif" width="120" height="100" border="0" />
+            <br /><strong>Certified Open Source</strong>
+        </a>
+        <br /><br />
+        <script type="text/javascript" language="JavaScript">var site="s16atlantibots"</script>
+        <script type="text/javascript" language="JavaScript1.2" src="http://s16.sitemeter.com/js/counter.js?site=s16atlantibots"></script>
+        <noscript>
+            <a href="http://s16.sitemeter.com/stats.asp?site=s16atlantibots">
+                <img src="http://s16.sitemeter.com/meter.asp?site=s16atlantibots" alt="Site Meter" border=0 />
+            </a>
+        </noscript>
+    </div>

=== added file 'bzrlib/util/configobj/validate.py'
--- bzrlib/util/configobj/validate.py	1970-01-01 00:00:00 +0000
+++ bzrlib/util/configobj/validate.py	2005-10-18 18:48:27 +0000
@@ -0,0 +1,1263 @@
+# validate.py
+# A Validator object
+# Copyright (C) 2005 Michael Foord, Mark Andrews, Nicola Larosa
+# E-mail: fuzzyman AT voidspace DOT org DOT uk
+#         mark AT la-la DOT com
+#         nico AT tekNico DOT net
+
+# This software is licensed under the terms of the BSD license.
+# http://www.voidspace.org.uk/documents/BSD-LICENSE.txt
+# Basically you're free to copy, modify, distribute and relicense it,
+# So long as you keep a copy of the license with it.
+
+# Scripts maintained at http://www.voidspace.org.uk/python/index.shtml
+# For information about bugfixes, updates and support, please join the
+# ConfigObj mailing list:
+# http://lists.sourceforge.net/lists/listinfo/configobj-develop
+# Comments, suggestions and bug reports welcome.
+
+"""
+    The Validator object is used to check that supplied values 
+    conform to a specification.
+    
+    The value can be supplied as a string - e.g. from a config file.
+    In this case the check will also *convert* the value to
+    the required type. This allows you to add validation
+    as a transparent layer to access data stored as strings.
+    The validation checks that the data is correct *and*
+    converts it to the expected type.
+    
+    Some standard checks are provided for basic data types.
+    Additional checks are easy to write. They can be
+    provided when the ``Validator`` is instantiated or
+    added afterwards.
+    
+    The standard functions work with the following basic data types :
+    
+    * integers
+    * floats
+    * booleans
+    * strings
+    * ip_addr
+    
+    plus lists of these datatypes
+    
+    Adding additional checks is done through coding simple functions.
+    
+    The full set of standard checks are : 
+    
+    * 'integer': matches integer values (including negative)
+                 Takes optional 'min' and 'max' arguments : ::
+    
+                   integer()
+                   integer(3, 9)  # any value from 3 to 9
+                   integer(min=0) # any positive value
+                   integer(max=9)
+    
+    * 'float': matches float values
+               Has the same parameters as the integer check.
+    
+    * 'boolean': matches boolean values - ``True`` or ``False``
+                 Acceptable string values for True are :
+                   true, on, yes, 1
+                 Acceptable string values for False are :
+                   false, off, no, 0
+    
+                 Any other value raises an error.
+    
+    * 'ip_addr': matches an Internet Protocol address, v.4, represented
+                 by a dotted-quad string, i.e. '1.2.3.4'.
+    
+    * 'string': matches any string.
+                Takes optional keyword args 'min' and 'max'
+                to specify min and max lengths of the string.
+    
+    * 'list': matches any list.
+              Takes optional keyword args 'min', and 'max' to specify min and
+              max sizes of the list.
+    
+    * 'int_list': Matches a list of integers.
+                  Takes the same arguments as list.
+    
+    * 'float_list': Matches a list of floats.
+                    Takes the same arguments as list.
+    
+    * 'bool_list': Matches a list of boolean values.
+                   Takes the same arguments as list.
+    
+    * 'ip_addr_list': Matches a list of IP addresses.
+                     Takes the same arguments as list.
+    
+    * 'string_list': Matches a list of strings.
+                     Takes the same arguments as list.
+    
+    * 'mixed_list': Matches a list with different types in 
+                    specific positions. List size must match
+                    the number of arguments.
+    
+                    Each position can be one of :
+                    'integer', 'float', 'ip_addr', 'string', 'boolean'
+    
+                    So to specify a list with two strings followed
+                    by two integers, you write the check as : ::
+    
+                      mixed_list('string', 'string', 'integer', 'integer')
+    
+    * 'pass': This check matches everything ! It never fails
+              and the value is unchanged.
+    
+              It is also the default if no check is specified.
+    
+    * 'option': This check matches any from a list of options.
+                You specify this check with : ::
+    
+                  option('option 1', 'option 2', 'option 3')
+"""
+
+__docformat__ = "restructuredtext en"
+
+__version__ = '0.2.0'
+
+__revision__ = '$Id: validate.py 123 2005-09-08 08:54:28Z fuzzyman $'
+
+__all__ = (
+    '__version__',
+    'dottedQuadToNum',
+    'numToDottedQuad',
+    'ValidateError',
+    'VdtUnknownCheckError',
+    'VdtParamError',
+    'VdtTypeError',
+    'VdtValueError',
+    'VdtValueTooSmallError',
+    'VdtValueTooBigError',
+    'VdtValueTooShortError',
+    'VdtValueTooLongError',
+    'VdtMissingValue',
+    'Validator',
+    'is_integer',
+    'is_float',
+    'is_bool',
+    'is_list',
+    'is_ip_addr',
+    'is_string',
+    'is_int_list',
+    'is_bool_list',
+    'is_float_list',
+    'is_string_list',
+    'is_ip_addr_list',
+    'is_mixed_list',
+    'is_option',
+    '__docformat__',
+)
+
+import sys
+INTP_VER = sys.version_info[:2]
+if INTP_VER < (2, 2):
+    raise RuntimeError("Python v.2.2 or later needed")
+
+import re
+StringTypes = (str, unicode)
+
+# Python pre 2.2.1 doesn't have bool
+try:
+    bool
+except NameError:
+    def bool(val):
+        """Simple boolean equivalent function. """
+        if val:
+            return 1
+        else:
+            return 0
+
+def dottedQuadToNum(ip):
+    """
+    Convert decimal dotted quad string to long integer
+    
+    >>> dottedQuadToNum('1 ')
+    1L
+    >>> dottedQuadToNum(' 1.2')
+    16777218L
+    >>> dottedQuadToNum(' 1.2.3 ')
+    16908291L
+    >>> dottedQuadToNum('1.2.3.4')
+    16909060L
+    >>> dottedQuadToNum('1.2.3. 4')
+    Traceback (most recent call last):
+    ValueError: Not a good dotted-quad IP: 1.2.3. 4
+    >>> dottedQuadToNum('255.255.255.255')
+    4294967295L
+    >>> dottedQuadToNum('255.255.255.256')
+    Traceback (most recent call last):
+    ValueError: Not a good dotted-quad IP: 255.255.255.256
+    """
+    
+    # import here to avoid it when ip_addr values are not used
+    import socket, struct
+    
+    try:
+        return struct.unpack('!L',
+            socket.inet_aton(ip.strip()))[0]
+    except socket.error:
+        # bug in inet_aton, corrected in Python 2.3
+        if ip.strip() == '255.255.255.255':
+            return 0xFFFFFFFFL
+        else:
+            raise ValueError('Not a good dotted-quad IP: %s' % ip)
+    return
+
+def numToDottedQuad(num):
+    """
+    Convert long int to dotted quad string
+    
+    >>> numToDottedQuad(-1L)
+    Traceback (most recent call last):
+    ValueError: Not a good numeric IP: -1
+    >>> numToDottedQuad(1L)
+    '0.0.0.1'
+    >>> numToDottedQuad(16777218L)
+    '1.0.0.2'
+    >>> numToDottedQuad(16908291L)
+    '1.2.0.3'
+    >>> numToDottedQuad(16909060L)
+    '1.2.3.4'
+    >>> numToDottedQuad(4294967295L)
+    '255.255.255.255'
+    >>> numToDottedQuad(4294967296L)
+    Traceback (most recent call last):
+    ValueError: Not a good numeric IP: 4294967296
+    """
+    
+    # import here to avoid it when ip_addr values are not used
+    import socket, struct
+    
+    # no need to intercept here, 4294967295L is fine
+    try:
+        return socket.inet_ntoa(
+            struct.pack('!L', long(num)))
+    except (socket.error, struct.error, OverflowError):
+        raise ValueError('Not a good numeric IP: %s' % num)
+
+class ValidateError(Exception):
+    """
+    This error indicates that the check failed.
+    It can be the base class for more specific errors.
+    
+    Any check function that fails ought to raise this error.
+    (or a subclass)
+    
+    >>> raise ValidateError
+    Traceback (most recent call last):
+    ValidateError
+    """
+
+class VdtMissingValue(ValidateError):
+    """No value was supplied to a check that needed one."""
+
+class VdtUnknownCheckError(ValidateError):
+    """An unknown check function was requested"""
+
+    def __init__(self, value):
+        """
+        >>> raise VdtUnknownCheckError('yoda')
+        Traceback (most recent call last):
+        VdtUnknownCheckError: the check "yoda" is unknown.
+        """
+        ValidateError.__init__(
+            self,
+            'the check "%s" is unknown.' % value)
+
+class VdtParamError(SyntaxError):
+    """An incorrect parameter was passed"""
+
+    def __init__(self, name, value):
+        """
+        >>> raise VdtParamError('yoda', 'jedi')
+        Traceback (most recent call last):
+        VdtParamError: passed an incorrect value "jedi" for parameter "yoda".
+        """
+        SyntaxError.__init__(
+            self,
+            'passed an incorrect value "%s" for parameter "%s".' % (
+                value, name))
+
+class VdtTypeError(ValidateError):
+    """The value supplied was of the wrong type"""
+
+    def __init__(self, value):
+        """
+        >>> raise VdtTypeError('jedi')
+        Traceback (most recent call last):
+        VdtTypeError: the value "jedi" is of the wrong type.
+        """
+        ValidateError.__init__(
+            self,
+            'the value "%s" is of the wrong type.' % value)
+
+class VdtValueError(ValidateError):
+    """
+    The value supplied was of the correct type, but was not an allowed value.
+    """
+
+    def __init__(self, value):
+        """
+        >>> raise VdtValueError('jedi')
+        Traceback (most recent call last):
+        VdtValueError: the value "jedi" is unacceptable.
+        """
+        ValidateError.__init__(
+            self,
+            'the value "%s" is unacceptable.' % value)
+
+class VdtValueTooSmallError(VdtValueError):
+    """The value supplied was of the correct type, but was too small."""
+
+    def __init__(self, value):
+        """
+        >>> raise VdtValueTooSmallError('0')
+        Traceback (most recent call last):
+        VdtValueTooSmallError: the value "0" is too small.
+        """
+        ValidateError.__init__(
+            self,
+            'the value "%s" is too small.' % value)
+
+class VdtValueTooBigError(VdtValueError):
+    """The value supplied was of the correct type, but was too big."""
+
+    def __init__(self, value):
+        """
+        >>> raise VdtValueTooBigError('1')
+        Traceback (most recent call last):
+        VdtValueTooBigError: the value "1" is too big.
+        """
+        ValidateError.__init__(
+            self,
+            'the value "%s" is too big.' % value)
+
+class VdtValueTooShortError(VdtValueError):
+    """The value supplied was of the correct type, but was too short."""
+
+    def __init__(self, value):
+        """
+        >>> raise VdtValueTooShortError('jed')
+        Traceback (most recent call last):
+        VdtValueTooShortError: the value "jed" is too short.
+        """
+        ValidateError.__init__(
+            self,
+            'the value "%s" is too short.' % (value,))
+
+class VdtValueTooLongError(VdtValueError):
+    """The value supplied was of the correct type, but was too long."""
+
+    def __init__(self, value):
+        """
+        >>> raise VdtValueTooLongError('jedie')
+        Traceback (most recent call last):
+        VdtValueTooLongError: the value "jedie" is too long.
+        """
+        ValidateError.__init__(
+            self,
+            'the value "%s" is too long.' %  (value,))
+
+class Validator(object):
+    """
+        Validator is an object that allows you to register a set of 'checks'.
+        These checks take input and test that it conforms to the check.
+        
+        This can also involve converting the value from a string into
+        the correct datatype.
+        
+        The ``check`` method takes an input string which configures which
+        check is to be used and applies that check to a supplied value.
+        
+        An example input string would be:
+        'int_range(param1, param2)'
+        
+        You would then provide something like:
+        
+        >>> def int_range_check(value, min, max):
+        ...     # turn min and max from strings to integers
+        ...     min = int(min)
+        ...     max = int(max)
+        ...     # check that value is of the correct type.
+        ...     # possible valid inputs are integers or strings
+        ...     # that represent integers
+        ...     if not isinstance(value, (int, long, StringTypes)):
+        ...         raise VdtTypeError(value)
+        ...     elif isinstance(value, StringTypes):
+        ...         # if we are given a string
+        ...         # attempt to convert to an integer
+        ...         try:
+        ...             value = int(value)
+        ...         except ValueError:
+        ...             raise VdtValueError(value)
+        ...     # check the value is between our constraints
+        ...     if not min <= value:
+        ...          raise VdtValueTooSmallError(value)
+        ...     if not value <= max:
+        ...          raise VdtValueTooBigError(value)
+        ...     return value
+        
+        >>> fdict = {'int_range': int_range_check}
+        >>> vtr1 = Validator(fdict)
+        >>> vtr1.check('int_range(20, 40)', '30')
+        30
+        >>> vtr1.check('int_range(20, 40)', '60')
+        Traceback (most recent call last):
+        VdtValueTooBigError: the value "60" is too big.
+        
+        New functions can be added with : ::
+        
+        >>> vtr2 = Validator()       
+        >>> vtr2.functions['int_range'] = int_range_check
+        
+        Or by passing in a dictionary of functions when Validator 
+        is instantiated.
+        
+        Your functions *can* use keyword arguments,
+        but the first argument should always be 'value'.
+        
+        If the function doesn't take additional arguments,
+        the parentheses are optional in the check.
+        It can be written with either of : ::
+        
+            keyword = function_name
+            keyword = function_name()
+        
+        The first program to utilise Validator() was Michael Foord's
+        ConfigObj, an alternative to ConfigParser which supports lists and
+        can validate a config file using a config schema.
+        For more details on using Validator with ConfigObj see:
+        http://www.voidspace.org.uk/python/configobj.html
+    """
+
+    # this regex pulls values out of a comma separated line
+    _paramfinder = re.compile(r'''(?:'.*?')|(?:".*?")|(?:[^'",\s][^,]*)''')
+    # this regex is used for finding keyword arguments
+    _key_arg = re.compile(r'^([a-zA-Z_][a-zA-Z0-9_]*)\s*=\s*(.*)$')
+    # this regex does the initial parsing of the checks
+    _func_re = re.compile(r'(.+?)\((.*)\)')
+
+    def __init__(self, functions=None):
+        """
+        >>> vtri = Validator()
+        """
+        self.functions = {
+            '': self._pass,
+            'integer': is_integer,
+            'float': is_float,
+            'boolean': is_bool,
+            'ip_addr': is_ip_addr,
+            'string': is_string,
+            'list': is_list,
+            'int_list': is_int_list,
+            'float_list': is_float_list,
+            'bool_list': is_bool_list,
+            'ip_addr_list': is_ip_addr_list,
+            'string_list': is_string_list,
+            'mixed_list': is_mixed_list,
+            'pass': self._pass,
+            'option': is_option,
+        }
+        if functions is not None:
+            self.functions.update(functions)
+        # tekNico: for use by ConfigObj
+        self.baseErrorClass = ValidateError
+
+    def check(self, check, value, missing=False):
+        """
+        Usage: check(check, value)
+        
+        Arguments:
+            check: string representing check to apply (including arguments)
+            value: object to be checked
+        Returns value, converted to correct type if necessary
+        
+        If the check fails, raises a ``ValidateError`` subclass.
+        
+        >>> vtor.check('yoda', '')
+        Traceback (most recent call last):
+        VdtUnknownCheckError: the check "yoda" is unknown.
+        >>> vtor.check('yoda()', '')
+        Traceback (most recent call last):
+        VdtUnknownCheckError: the check "yoda" is unknown.
+        """
+        fun_match = self._func_re.match(check)
+        if fun_match:
+            fun_name = fun_match.group(1)
+            fun_args = []
+            fun_kwargs = {}
+            # pull out args of group 2
+            for arg in self._paramfinder.findall(fun_match.group(2)):
+                # args may need whitespace removing (before removing quotes)
+                arg = arg.strip()
+                keymatch = self._key_arg.match(arg)
+                if keymatch:
+                    val = keymatch.group(2)
+                    if (val[0] in ("'", '"')) and (val[0] == val[-1]):
+                        val = val[1:-1]
+                    fun_kwargs[keymatch.group(1)] = val
+                    continue
+                #
+                if (arg[0] in ("'", '"')) and (arg[0] == arg[-1]):
+                    arg = arg[1:-1]
+                fun_args.append(arg)
+        else:
+            # allows for function names without (args)
+            (fun_name, fun_args, fun_kwargs) = (check, (), {})
+        #
+        if missing:
+            try:
+                value = fun_kwargs['default']
+            except KeyError:
+                raise VdtMissingValue
+            if value == 'None':
+                value = None
+        if value is None:
+            return None
+# tekNico: default must be deleted if the value is specified too,
+# otherwise the check function will get a spurious "default" keyword arg
+        try:
+            del fun_kwargs['default']
+        except KeyError:
+            pass
+        try:
+            return self.functions[fun_name](value, *fun_args, **fun_kwargs)
+        except KeyError:
+            raise VdtUnknownCheckError(fun_name)
+
+    def _pass(self, value):
+        """
+        Dummy check that always passes
+        
+        >>> vtor.check('', 0)
+        0
+        >>> vtor.check('', '0')
+        '0'
+        """
+        return value
+
+
+def _is_num_param(names, values, to_float=False):
+    """
+    Return numbers from inputs or raise VdtParamError.
+    
+    Lets ``None`` pass through.
+    Pass in keyword argument ``to_float=True`` to
+    use float for the conversion rather than int.
+    
+    >>> _is_num_param(('', ''), (0, 1.0))
+    [0, 1]
+    >>> _is_num_param(('', ''), (0, 1.0), to_float=True)
+    [0.0, 1.0]
+    >>> _is_num_param(('a'), ('a'))
+    Traceback (most recent call last):
+    VdtParamError: passed an incorrect value "a" for parameter "a".
+    """
+    fun = to_float and float or int
+    out_params = []
+    for (name, val) in zip(names, values):
+        if val is None:
+            out_params.append(val)
+        elif isinstance(val, (int, long, float, StringTypes)):
+            try:
+                out_params.append(fun(val))
+            except ValueError, e:
+                raise VdtParamError(name, val)
+        else:
+            raise VdtParamError(name, val)
+    return out_params
+
+# built in checks
+# you can override these by setting the appropriate name
+# in Validator.functions
+# note: if the params are specified wrongly in your input string,
+#       you will also raise errors.
+
+def is_integer(value, min=None, max=None):
+    """
+    A check that tests that a given value is an integer (int, or long)
+    and optionally, between bounds. A negative value is accepted, while
+    a float will fail.
+    
+    If the value is a string, then the conversion is done - if possible.
+    Otherwise a VdtError is raised.
+    
+    >>> vtor.check('integer', '-1')
+    -1
+    >>> vtor.check('integer', '0')
+    0
+    >>> vtor.check('integer', 9)
+    9
+    >>> vtor.check('integer', 'a')
+    Traceback (most recent call last):
+    VdtTypeError: the value "a" is of the wrong type.
+    >>> vtor.check('integer', '2.2')
+    Traceback (most recent call last):
+    VdtTypeError: the value "2.2" is of the wrong type.
+    >>> vtor.check('integer(10)', '20')
+    20
+    >>> vtor.check('integer(max=20)', '15')
+    15
+    >>> vtor.check('integer(10)', '9')
+    Traceback (most recent call last):
+    VdtValueTooSmallError: the value "9" is too small.
+    >>> vtor.check('integer(10)', 9)
+    Traceback (most recent call last):
+    VdtValueTooSmallError: the value "9" is too small.
+    >>> vtor.check('integer(max=20)', '35')
+    Traceback (most recent call last):
+    VdtValueTooBigError: the value "35" is too big.
+    >>> vtor.check('integer(max=20)', 35)
+    Traceback (most recent call last):
+    VdtValueTooBigError: the value "35" is too big.
+    >>> vtor.check('integer(0, 9)', False)
+    0
+    """
+#    print value, type(value)
+    (min_val, max_val) = _is_num_param(('min', 'max'), (min, max))
+    if not isinstance(value, (int, long, StringTypes)):
+        raise VdtTypeError(value)
+    if isinstance(value, StringTypes):
+        # if it's a string - does it represent an integer ?
+        try:
+            value = int(value)
+        except ValueError:
+            raise VdtTypeError(value)
+    if (min_val is not None) and (value < min_val):
+        raise VdtValueTooSmallError(value)
+    if (max_val is not None) and (value > max_val):
+        raise VdtValueTooBigError(value)
+    return value
+
+def is_float(value, min=None, max=None):
+    """
+    A check that tests that a given value is a float
+    (an integer will be accepted), and optionally - that it is between bounds.
+    
+    If the value is a string, then the conversion is done - if possible.
+    Otherwise a VdtError is raised.
+    
+    This can accept negative values.
+    
+    >>> vtor.check('float', '2')
+    2.0
+    
+    From now on we multiply the value to avoid comparing decimals
+    
+    >>> vtor.check('float', '-6.8') * 10
+    -68.0
+    >>> vtor.check('float', '12.2') * 10
+    122.0
+    >>> vtor.check('float', 8.4) * 10
+    84.0
+    >>> vtor.check('float', 'a')
+    Traceback (most recent call last):
+    VdtTypeError: the value "a" is of the wrong type.
+    >>> vtor.check('float(10.1)', '10.2') * 10
+    102.0
+    >>> vtor.check('float(max=20.2)', '15.1') * 10
+    151.0
+    >>> vtor.check('float(10.0)', '9.0')
+    Traceback (most recent call last):
+    VdtValueTooSmallError: the value "9.0" is too small.
+    >>> vtor.check('float(max=20.0)', '35.0')
+    Traceback (most recent call last):
+    VdtValueTooBigError: the value "35.0" is too big.
+    """
+    (min_val, max_val) = _is_num_param(
+        ('min', 'max'), (min, max), to_float=True)
+    if not isinstance(value, (int, long, float, StringTypes)):
+        raise VdtTypeError(value)
+    if not isinstance(value, float):
+        # if it's a string - does it represent a float ?
+        try:
+            value = float(value)
+        except ValueError:
+            raise VdtTypeError(value)
+    if (min_val is not None) and (value < min_val):
+        raise VdtValueTooSmallError(value)
+    if (max_val is not None) and (value > max_val):
+        raise VdtValueTooBigError(value)
+    return value
+
+bool_dict = {
+    True: True, 'on': True, '1': True, 'true': True, 'yes': True, 
+    False: False, 'off': False, '0': False, 'false': False, 'no': False,
+}
+
+def is_bool(value):
+    """
+    Check if the value represents a boolean.
+    
+    >>> vtor.check('boolean', 0)
+    0
+    >>> vtor.check('boolean', False)
+    0
+    >>> vtor.check('boolean', '0')
+    0
+    >>> vtor.check('boolean', 'off')
+    0
+    >>> vtor.check('boolean', 'false')
+    0
+    >>> vtor.check('boolean', 'no')
+    0
+    >>> vtor.check('boolean', 'nO')
+    0
+    >>> vtor.check('boolean', 'NO')
+    0
+    >>> vtor.check('boolean', 1)
+    1
+    >>> vtor.check('boolean', True)
+    1
+    >>> vtor.check('boolean', '1')
+    1
+    >>> vtor.check('boolean', 'on')
+    1
+    >>> vtor.check('boolean', 'true')
+    1
+    >>> vtor.check('boolean', 'yes')
+    1
+    >>> vtor.check('boolean', 'Yes')
+    1
+    >>> vtor.check('boolean', 'YES')
+    1
+    >>> vtor.check('boolean', '')
+    Traceback (most recent call last):
+    VdtTypeError: the value "" is of the wrong type.
+    >>> vtor.check('boolean', 'up')
+    Traceback (most recent call last):
+    VdtTypeError: the value "up" is of the wrong type.
+    
+    """
+    if isinstance(value, StringTypes):
+        try:
+            return bool_dict[value.lower()]
+        except KeyError:
+            raise VdtTypeError(value)
+    # we do an equality test rather than an identity test
+    # this ensures Python 2.2 compatibilty
+    # and allows 0 and 1 to represent True and False
+    if value == False:
+        return False
+    elif value == True:
+        return True
+    else:
+        raise VdtTypeError(value)
+
+
+def is_ip_addr(value):
+    """
+    Check that the supplied value is an Internet Protocol address, v.4,
+    represented by a dotted-quad string, i.e. '1.2.3.4'.
+    
+    >>> vtor.check('ip_addr', '1 ')
+    '1'
+    >>> vtor.check('ip_addr', ' 1.2')
+    '1.2'
+    >>> vtor.check('ip_addr', ' 1.2.3 ')
+    '1.2.3'
+    >>> vtor.check('ip_addr', '1.2.3.4')
+    '1.2.3.4'
+    >>> vtor.check('ip_addr', '0.0.0.0')
+    '0.0.0.0'
+    >>> vtor.check('ip_addr', '255.255.255.255')
+    '255.255.255.255'
+    >>> vtor.check('ip_addr', '255.255.255.256')
+    Traceback (most recent call last):
+    VdtValueError: the value "255.255.255.256" is unacceptable.
+    >>> vtor.check('ip_addr', '1.2.3.4.5')
+    Traceback (most recent call last):
+    VdtValueError: the value "1.2.3.4.5" is unacceptable.
+    >>> vtor.check('ip_addr', '1.2.3. 4')
+    Traceback (most recent call last):
+    VdtValueError: the value "1.2.3. 4" is unacceptable.
+    >>> vtor.check('ip_addr', 0)
+    Traceback (most recent call last):
+    VdtTypeError: the value "0" is of the wrong type.
+    """
+    if not isinstance(value, StringTypes):
+        raise VdtTypeError(value)
+    value = value.strip()
+    try:
+        dottedQuadToNum(value)
+    except ValueError:
+        raise VdtValueError(value)
+    return value
+
+def is_list(value, min=None, max=None):
+    """
+    Check that the value is a list of values.
+    
+    You can optionally specify the minimum and maximum number of members.
+    
+    It does no check on list members.
+    
+    >>> vtor.check('list', ())
+    ()
+    >>> vtor.check('list', [])
+    []
+    >>> vtor.check('list', (1, 2))
+    (1, 2)
+    >>> vtor.check('list', [1, 2])
+    [1, 2]
+    >>> vtor.check('list', '12')
+    '12'
+    >>> vtor.check('list(3)', (1, 2))
+    Traceback (most recent call last):
+    VdtValueTooShortError: the value "(1, 2)" is too short.
+    >>> vtor.check('list(max=5)', (1, 2, 3, 4, 5, 6))
+    Traceback (most recent call last):
+    VdtValueTooLongError: the value "(1, 2, 3, 4, 5, 6)" is too long.
+    >>> vtor.check('list(min=3, max=5)', (1, 2, 3, 4))
+    (1, 2, 3, 4)
+    >>> vtor.check('list', 0)
+    Traceback (most recent call last):
+    VdtTypeError: the value "0" is of the wrong type.
+    """
+    (min_len, max_len) = _is_num_param(('min', 'max'), (min, max))
+    try:
+        num_members = len(value)
+    except TypeError:
+        raise VdtTypeError(value)
+    if min_len is not None and num_members < min_len:
+        raise VdtValueTooShortError(value)
+    if max_len is not None and num_members > max_len:
+        raise VdtValueTooLongError(value)
+    return value
+
+def is_string(value, min=None, max=None):
+    """
+    Check that the supplied value is a string.
+    
+    You can optionally specify the minimum and maximum number of members.
+    
+    >>> vtor.check('string', '0')
+    '0'
+    >>> vtor.check('string', 0)
+    Traceback (most recent call last):
+    VdtTypeError: the value "0" is of the wrong type.
+    >>> vtor.check('string(2)', '12')
+    '12'
+    >>> vtor.check('string(2)', '1')
+    Traceback (most recent call last):
+    VdtValueTooShortError: the value "1" is too short.
+    >>> vtor.check('string(min=2, max=3)', '123')
+    '123'
+    >>> vtor.check('string(min=2, max=3)', '1234')
+    Traceback (most recent call last):
+    VdtValueTooLongError: the value "1234" is too long.
+    """
+    if not isinstance(value, StringTypes):
+        raise VdtTypeError(value)
+    return is_list(value, min, max)
+
+def is_int_list(value, min=None, max=None):
+    """
+    Check that the value is a list of integers.
+    
+    You can optionally specify the minimum and maximum number of members.
+    
+    Each list member is checked that it is an integer.
+    
+    >>> vtor.check('int_list', ())
+    []
+    >>> vtor.check('int_list', [])
+    []
+    >>> vtor.check('int_list', (1, 2))
+    [1, 2]
+    >>> vtor.check('int_list', [1, 2])
+    [1, 2]
+    >>> vtor.check('int_list', [1, 'a'])
+    Traceback (most recent call last):
+    VdtTypeError: the value "a" is of the wrong type.
+    """
+    return [is_integer(mem) for mem in is_list(value, min, max)]
+
+def is_bool_list(value, min=None, max=None):
+    """
+    Check that the value is a list of booleans.
+    
+    You can optionally specify the minimum and maximum number of members.
+    
+    Each list member is checked that it is a boolean.
+    
+    >>> vtor.check('bool_list', ())
+    []
+    >>> vtor.check('bool_list', [])
+    []
+    >>> check_res = vtor.check('bool_list', (True, False))
+    >>> check_res == [True, False]
+    1
+    >>> check_res = vtor.check('bool_list', [True, False])
+    >>> check_res == [True, False]
+    1
+    >>> vtor.check('bool_list', [True, 'a'])
+    Traceback (most recent call last):
+    VdtTypeError: the value "a" is of the wrong type.
+    """
+    return [is_bool(mem) for mem in is_list(value, min, max)]
+
+def is_float_list(value, min=None, max=None):
+    """
+    Check that the value is a list of floats.
+    
+    You can optionally specify the minimum and maximum number of members.
+    
+    Each list member is checked that it is a float.
+    
+    >>> vtor.check('float_list', ())
+    []
+    >>> vtor.check('float_list', [])
+    []
+    >>> vtor.check('float_list', (1, 2.0))
+    [1.0, 2.0]
+    >>> vtor.check('float_list', [1, 2.0])
+    [1.0, 2.0]
+    >>> vtor.check('float_list', [1, 'a'])
+    Traceback (most recent call last):
+    VdtTypeError: the value "a" is of the wrong type.
+    """
+    return [is_float(mem) for mem in is_list(value, min, max)]
+
+def is_string_list(value, min=None, max=None):
+    """
+    Check that the value is a list of strings.
+    
+    You can optionally specify the minimum and maximum number of members.
+    
+    Each list member is checked that it is a string.
+    
+    >>> vtor.check('string_list', ())
+    []
+    >>> vtor.check('string_list', [])
+    []
+    >>> vtor.check('string_list', ('a', 'b'))
+    ['a', 'b']
+    >>> vtor.check('string_list', ['a', 1])
+    Traceback (most recent call last):
+    VdtTypeError: the value "1" is of the wrong type.
+    """
+    return [is_string(mem) for mem in is_list(value, min, max)]
+
+def is_ip_addr_list(value, min=None, max=None):
+    """
+    Check that the value is a list of IP addresses.
+    
+    You can optionally specify the minimum and maximum number of members.
+    
+    Each list member is checked that it is an IP address.
+    
+    >>> vtor.check('ip_addr_list', ())
+    []
+    >>> vtor.check('ip_addr_list', [])
+    []
+    >>> vtor.check('ip_addr_list', ('1.2.3.4', '5.6.7.8'))
+    ['1.2.3.4', '5.6.7.8']
+    >>> vtor.check('ip_addr_list', ['a'])
+    Traceback (most recent call last):
+    VdtValueError: the value "a" is unacceptable.
+    """
+    return [is_ip_addr(mem) for mem in is_list(value, min, max)]
+
+fun_dict = {
+    'integer': is_integer,
+    'float': is_float,
+    'ip_addr': is_ip_addr,
+    'string': is_string,
+    'boolean': is_bool,
+}
+
+def is_mixed_list(value, *args):
+    """
+    Check that the value is a list.
+    Allow specifying the type of each member.
+    Work on lists of specific lengths.
+    
+    You specify each member as a positional argument specifying type
+    
+    Each type should be one of the following strings :
+      'integer', 'float', 'ip_addr', 'string', 'boolean'
+    
+    So you can specify a list of two strings, followed by
+    two integers as :
+    
+      mixed_list('string', 'string', 'integer', 'integer')
+    
+    The length of the list must match the number of positional
+    arguments you supply.
+    
+    >>> mix_str = "mixed_list('integer', 'float', 'ip_addr', 'string', 'boolean')"
+    >>> check_res = vtor.check(mix_str, (1, 2.0, '1.2.3.4', 'a', True))
+    >>> check_res == [1, 2.0, '1.2.3.4', 'a', True]
+    1
+    >>> check_res = vtor.check(mix_str, ('1', '2.0', '1.2.3.4', 'a', 'True'))
+    >>> check_res == [1, 2.0, '1.2.3.4', 'a', True]
+    1
+    >>> vtor.check(mix_str, ('b', 2.0, '1.2.3.4', 'a', True))
+    Traceback (most recent call last):
+    VdtTypeError: the value "b" is of the wrong type.
+    >>> vtor.check(mix_str, (1, 2.0, '1.2.3.4', 'a'))
+    Traceback (most recent call last):
+    VdtValueTooShortError: the value "(1, 2.0, '1.2.3.4', 'a')" is too short.
+    >>> vtor.check(mix_str, (1, 2.0, '1.2.3.4', 'a', 1, 'b'))
+    Traceback (most recent call last):
+    VdtValueTooLongError: the value "(1, 2.0, '1.2.3.4', 'a', 1, 'b')" is too long.
+    >>> vtor.check(mix_str, 0)
+    Traceback (most recent call last):
+    VdtTypeError: the value "0" is of the wrong type.
+    
+    This test requires an elaborate setup, because of a change in error string
+    output from the interpreter between Python 2.2 and 2.3 .
+    
+    >>> res_seq = (
+    ...     'passed an incorrect value "',
+    ...     'yoda',
+    ...     '" for parameter "mixed_list".',
+    ... )
+    >>> if INTP_VER == (2, 2):
+    ...     res_str = "".join(res_seq)
+    ... else:
+    ...     res_str = "'".join(res_seq)
+    >>> try:
+    ...     vtor.check('mixed_list("yoda")', ('a'))
+    ... except VdtParamError, err:
+    ...     str(err) == res_str
+    1
+    """
+    try:
+        length = len(value)
+    except TypeError:
+        raise VdtTypeError(value)
+    if length < len(args):
+        raise VdtValueTooShortError(value)
+    elif length > len(args):
+        raise VdtValueTooLongError(value)
+    try:
+        return [fun_dict[arg](val) for arg, val in zip(args, value)]
+    except KeyError, e:
+        raise VdtParamError('mixed_list', e)
+
+def is_option(value, *options):
+    """
+    This check matches the value to any of a set of options.
+    
+    >>> vtor.check('option("yoda", "jedi")', 'yoda')
+    'yoda'
+    >>> vtor.check('option("yoda", "jedi")', 'jed')
+    Traceback (most recent call last):
+    VdtValueError: the value "jed" is unacceptable.
+    >>> vtor.check('option("yoda", "jedi")', 0)
+    Traceback (most recent call last):
+    VdtTypeError: the value "0" is of the wrong type.
+    """
+    if not isinstance(value, StringTypes):
+        raise VdtTypeError(value)
+    if not value in options:
+        raise VdtValueError(value)
+    return value
+
+if __name__ == '__main__':
+    # run the code tests in doctest format
+    import doctest
+    m = sys.modules.get('__main__')
+    globs = m.__dict__.copy()
+    globs.update({
+        'INTP_VER': INTP_VER,
+        'vtor': Validator(),
+    })
+    doctest.testmod(m, globs=globs)
+
+"""
+    TODO
+    ====
+    
+    Consider which parts of the regex stuff to put back in
+    
+    Can we implement a timestamp datatype ? (check DateUtil module)
+    
+    ISSUES
+    ======
+    
+    Lists passed as function arguments need additional quotes. Ugly, could do
+    with fixing this.
+    
+    If we could pull tuples out of arguments, it would be easier
+    to specify arguments for 'mixed_lists'.
+    
+    CHANGELOG
+    =========
+    
+    2005/08/25
+    ----------
+    
+    Most errors now prefixed ``Vdt``
+    
+    ``VdtParamError`` no longer derives from ``VdtError``
+    
+    Finalised as version 0.2.0
+    
+    2005/08/21
+    ----------
+    
+    By Nicola Larosa
+    
+    Removed the "length" argument for lists and strings, and related tests
+    
+    2005/08/16
+    ----------
+    
+    By Nicola Larosa
+    
+    Deleted the "none" and "multiple" types and checks
+    
+    Added the None value for all types in Validation.check
+    
+    2005/08/14
+    ----------
+    
+    By Michael Foord
+    
+    Removed timestamp.
+    
+    By Nicola Larosa
+    
+    Fixed bug in Validator.check: when a value that has a default is also
+    specified in the config file, the default must be deleted from fun_kwargs
+    anyway, otherwise the check function will get a spurious "default" keyword
+    argument
+    
+    Added "ip_addr_list" check
+    
+    2005/08/13
+    ----------
+    
+    By Nicola Larosa
+    
+    Updated comments at top
+    
+    2005/08/11
+    ----------
+    
+    By Nicola Larosa
+    
+    Added test for interpreter version: raises RuntimeError if earlier than
+    2.2
+    
+    Fixed last is_mixed_list test to work on Python 2.2 too
+    
+    2005/08/10
+    ----------
+    
+    By Nicola Larosa
+    
+    Restored Python2.2 compatibility by avoiding usage of dict.pop
+    
+    2005/08/07
+    ----------
+    
+    By Nicola Larosa
+    
+    Adjusted doctests for Python 2.2.3 compatibility, one test still fails
+    for trivial reasons (string output delimiters)
+    
+    2005/08/05
+    ----------
+    
+    By Michael Foord
+    
+    Added __version__, __all__, and __docformat__
+    
+    Replaced ``basestring`` with ``types.StringTypes``
+    
+    2005/07/28
+    ----------
+    
+    By Nicola Larosa
+    
+    Reformatted final docstring in ReST format, indented it for easier folding
+    
+    2005/07/20
+    ----------
+    
+    By Nicola Larosa
+    
+    Added an 'ip_addr' IPv4 address value check, with tests
+    
+    Updated the tests for mixed_list to include IP addresses
+    
+    Changed all references to value "tests" into value "checks", including
+    the main Validator method, and all code tests
+    
+    2005/07/19
+    ----------
+    
+    By Nicola Larosa
+    
+    Added even more code tests
+    
+    Refined the mixed_list check
+    
+    2005/07/18
+    ----------
+    
+    By Nicola Larosa
+    
+    Introduced more VdtValueError subclasses
+    
+    Collapsed the ``_function_test`` and ``_function_parse`` methods into the
+    ``check`` one
+    
+    Refined the value checks, using the new VdtValueError subclasses
+    
+    Changed "is_string" to use "is_list"
+    
+    Added many more code tests
+    
+    Changed the "bool" value type to "boolean"
+    
+    Some more code cleanup
+    
+    2005/07/17
+    ----------
+    
+    By Nicola Larosa
+    
+    Code tests converted to doctest format and placed in the respective
+    docstrings, so they are automatically checked, and easier to update
+    
+    Changed local vars "min" and "max" to "min_len", "max_len", "min_val" and
+    "max_val", to avoid shadowing the builtin functions (but left function
+    parameters alone)
+    
+    Uniformed value check function names to is_* convention
+    
+    ``date`` type name changed to ``timestamp``
+    
+    Avoided some code duplication in list check functions
+    
+    Some more code cleanup
+    
+    2005/07/09
+    ----------
+    
+    Recoded the standard functions
+    
+    2005/07/08
+    ----------
+    
+    Improved paramfinder regex
+    
+    Ripped out all the regex stuff, checks, and the example functions
+    (to be replaced !)
+    
+    2005/07/06
+    ----------
+    
+    By Nicola Larosa
+    
+    Code cleanup
+"""
+

=== modified file 'bzrlib/weave.py' (properties changed: -x to +x)
--- bzrlib/weave.py	2005-10-06 08:11:46 +0000
+++ bzrlib/weave.py	2005-12-20 02:45:27 +0000
@@ -50,40 +50,33 @@
 # have slight specializations for different ways its used: annotate,
 # basis for add, get, etc.
 
-# TODO: Perhaps the API should work only in names to hide the integer
-# indexes from the user?
+# TODO: Probably the API should work only in names to hide the integer
+# indexes from the user.
 
 # TODO: Is there any potential performance win by having an add()
 # variant that is passed a pre-cooked version of the single basis
 # version?
 
-# TODO: Have a way to go back and insert a revision V1 that is a parent 
-# of an already-stored revision V2.  This means some lines previously 
-# counted as new in V2 will be discovered to have actually come from V1.  
-# It is probably necessary to insert V1, then compute a whole new diff 
-# from the mashed ancestors to V2.  This must be repeated for every 
-# direct child of V1.  The deltas from V2 to its descendents won't change,
-# although their location within the weave may change.  It may be possible
-# to just adjust the location of those instructions rather than 
-# re-weaving the whole thing.  This is expected to be a fairly rare
-# operation, only used when inserting data that was previously a ghost.
-
-
-
-
+# TODO: Reweave can possibly be made faster by remembering diffs
+# where the basis and destination are unchanged.
+
+# FIXME: Sometimes we will be given a parents list for a revision
+# that includes some redundant parents (i.e. already a parent of 
+# something in the list.)  We should eliminate them.  This can 
+# be done fairly efficiently because the sequence numbers constrain
+# the possible relationships.
+
+
+import os
 import sha
 from difflib import SequenceMatcher
 
-
-
-
-class WeaveError(Exception):
-    """Exception in processing weave"""
-
-
-class WeaveFormatError(WeaveError):
-    """Weave invariant violated"""
-    
+from bzrlib.trace import mutter
+from bzrlib.errors import (WeaveError, WeaveFormatError, WeaveParentMismatch,
+        WeaveRevisionNotPresent, WeaveRevisionAlreadyPresent)
+import bzrlib.errors as errors
+from bzrlib.tsort import topo_sort
+
 
 class Weave(object):
     """weave - versioned text file storage.
@@ -185,6 +178,9 @@
         self._name_map = {}
         self._weave_name = weave_name
 
+    def __repr__(self):
+        return "Weave(%r)" % self._weave_name
+
 
     def copy(self):
         """Return a deep copy of self.
@@ -210,6 +206,8 @@
     def __ne__(self, other):
         return not self.__eq__(other)
 
+    def __contains__(self, name):
+        return self._name_map.has_key(name)
 
     def maybe_lookup(self, name_or_index):
         """Convert possible symbolic name to index, or pass through indexes."""
@@ -224,9 +222,10 @@
         try:
             return self._name_map[name]
         except KeyError:
-            raise WeaveError("name %r not present in weave %r" %
-                             (name, self._weave_name))
+            raise WeaveRevisionNotPresent(name, self)
 
+    def names(self):
+        return self._names[:]
 
     def iter_names(self):
         """Yield a list of all names in this weave."""
@@ -235,23 +234,17 @@
     def idx_to_name(self, version):
         return self._names[version]
 
-
     def _check_repeated_add(self, name, parents, text, sha1):
         """Check that a duplicated add is OK.
 
         If it is, return the (old) index; otherwise raise an exception.
         """
         idx = self.lookup(name)
-        if sorted(self._parents[idx]) != sorted(parents):
-            raise WeaveError("name \"%s\" already present in weave "
-                             "with different parents" % name)
-        if sha1 != self._sha1s[idx]:
-            raise WeaveError("name \"%s\" already present in weave "
-                             "with different text" % name)            
+        if sorted(self._parents[idx]) != sorted(parents) \
+            or sha1 != self._sha1s[idx]:
+            raise WeaveRevisionAlreadyPresent(name, self)
         return idx
         
-
-        
     def add(self, name, parents, text, sha1=None):
         """Add a single text on top of the weave.
   
@@ -398,6 +391,11 @@
         return self._parents[version]
 
 
+    def parent_names(self, version):
+        """Return version names for parents of a version."""
+        return map(self.idx_to_name, self._parents[self.lookup(version)])
+
+
     def minimal_parents(self, version):
         """Find the minimal set of parents for the version."""
         included = self._parents[version]
@@ -453,7 +451,6 @@
         for origin, lineno, text in self._extract(incls):
             yield origin, text
 
-
     def _walk(self):
         """Walk the weave.
 
@@ -481,15 +478,19 @@
                 elif c == ']':
                     dset.remove(v)
                 else:
-                    raise WeaveFormatError('unexpected instruction %r'
-                                           % v)
+                    raise WeaveFormatError('unexpected instruction %r' % v)
             else:
                 assert isinstance(l, basestring)
                 assert istack
                 yield lineno, istack[-1], dset, l
             lineno += 1
 
-
+        if istack:
+            raise WeaveFormatError("unclosed insertion blocks "
+                    "at end of weave: %s" % istack)
+        if dset:
+            raise WeaveFormatError("unclosed deletion blocks at end of weave: %s"
+                                   % dset)
 
     def _extract(self, versions):
         """Yield annotation of lines in included set.
@@ -542,23 +543,36 @@
                 if isactive:
                     result.append((istack[-1], lineno, l))
             lineno += 1
-
         if istack:
-            raise WFE("unclosed insertion blocks at end of weave",
-                                   istack)
+            raise WeaveFormatError("unclosed insertion blocks "
+                    "at end of weave: %s" % istack)
         if dset:
-            raise WFE("unclosed deletion blocks at end of weave",
-                                   dset)
-
+            raise WeaveFormatError("unclosed deletion blocks at end of weave: %s"
+                                   % dset)
         return result
-    
 
 
     def get_iter(self, name_or_index):
         """Yield lines for the specified version."""
         incls = [self.maybe_lookup(name_or_index)]
+        if len(incls) == 1:
+            index = incls[0]
+            cur_sha = sha.new()
+        else:
+            # We don't have sha1 sums for multiple entries
+            cur_sha = None
         for origin, lineno, line in self._extract(incls):
+            if cur_sha:
+                cur_sha.update(line)
             yield line
+        if cur_sha:
+            expected_sha1 = self._sha1s[index]
+            measured_sha1 = cur_sha.hexdigest() 
+            if measured_sha1 != expected_sha1:
+                raise errors.WeaveInvalidChecksum(
+                        'file %s, revision %s, expected: %s, measured %s' 
+                        % (self._weave_name, self._names[index],
+                           expected_sha1, measured_sha1))
 
 
     def get_text(self, name_or_index):
@@ -573,6 +587,13 @@
     get = get_lines
 
 
+    def get_sha1(self, name):
+        """Get the stored sha1 sum for the given revision.
+        
+        :param name: The name of the version to lookup
+        """
+        return self._sha1s[self.lookup(name)]
+
     def mash_iter(self, included):
         """Return composed version of multiple included versions."""
         included = map(self.maybe_lookup, included)
@@ -598,7 +619,6 @@
     def __len__(self):
         return self.numversions()
 
-
     def check(self, progress_bar=None):
         # check no circular inclusions
         for version in range(self.numversions()):
@@ -609,44 +629,53 @@
                     raise WeaveFormatError("invalid included version %d for index %d"
                                            % (inclusions[-1], version))
 
-        # try extracting all versions; this is a bit slow and parallel
-        # extraction could be used
+        # try extracting all versions; parallel extraction is used
         nv = self.numversions()
+        sha1s = [sha.new() for i in range(nv)]
+        texts = [[] for i in range(nv)]
+        inclusions = []
+        for i in range(nv):
+            # For creating the ancestry, IntSet is much faster (3.7s vs 0.17s)
+            # The problem is that set membership is much more expensive
+            new_inc = set([i])
+            for p in self._parents[i]:
+                new_inc.update(inclusions[p])
+
+            #assert set(new_inc) == self.inclusions([i]), 'failed %s != %s' % (new_inc, self.inclusions([i]))
+            inclusions.append(new_inc)
+
+        nlines = len(self._weave)
+
+        update_text = 'checking weave'
+        if self._weave_name:
+            short_name = os.path.basename(self._weave_name)
+            update_text = 'checking %s' % (short_name,)
+            update_text = update_text[:25]
+
+        for lineno, insert, deleteset, line in self._walk():
+            if progress_bar:
+                progress_bar.update(update_text, lineno, nlines)
+
+            for j, j_inc in enumerate(inclusions):
+                # The active inclusion must be an ancestor,
+                # and no ancestors must have deleted this line,
+                # because we don't support resurrection.
+                if (insert in j_inc) and not (deleteset & j_inc):
+                    sha1s[j].update(line)
+
         for version in range(nv):
-            if progress_bar:
-                progress_bar.update('checking text', version, nv)
-            s = sha.new()
-            for l in self.get_iter(version):
-                s.update(l)
-            hd = s.hexdigest()
+            hd = sha1s[version].hexdigest()
             expected = self._sha1s[version]
             if hd != expected:
-                raise WeaveError("mismatched sha1 for version %d; "
-                                 "got %s, expected %s"
-                                 % (version, hd, expected))
+                raise errors.WeaveInvalidChecksum(
+                        "mismatched sha1 for version %s: "
+                        "got %s, expected %s"
+                        % (self._names[version], hd, expected))
 
         # TODO: check insertions are properly nested, that there are
         # no lines outside of insertion blocks, that deletions are
         # properly paired, etc.
 
-
-
-    def merge(self, merge_versions):
-        """Automerge and mark conflicts between versions.
-
-        This returns a sequence, each entry describing alternatives
-        for a chunk of the file.  Each of the alternatives is given as
-        a list of lines.
-
-        If there is a chunk of the file where there's no diagreement,
-        only one alternative is given.
-        """
-        # approach: find the included versions common to all the
-        # merged versions
-        raise NotImplementedError()
-
-
-
     def _delta(self, included, lines):
         """Return changes from basis to new revision.
 
@@ -721,7 +750,11 @@
         lines_a = []
         lines_b = []
         ch_a = ch_b = False
-
+        # TODO: Return a structured form of the conflicts (e.g. 2-tuples for
+        # conflicted regions), rather than just inserting the markers.
+        # 
+        # TODO: Show some version information (e.g. author, date) on 
+        # conflicted regions.
         for state, line in plan:
             if state == 'unchanged' or state == 'killed-both':
                 # resync and flush queued conflicts changes if any
@@ -735,11 +768,11 @@
                 elif lines_a == lines_b:
                     for l in lines_a: yield l
                 else:
-                    yield '<<<<\n'
+                    yield '<<<<<<<\n'
                     for l in lines_a: yield l
-                    yield '====\n'
+                    yield '=======\n'
                     for l in lines_b: yield l
-                    yield '>>>>\n'
+                    yield '>>>>>>>\n'
 
                 del lines_a[:]
                 del lines_b[:]
@@ -765,8 +798,9 @@
                                  'killed-both'), \
                        state
 
-                
+
     def join(self, other):
+        import sys, time
         """Integrate versions from other into this weave.
 
         The resulting weave contains all the history of both weaves; 
@@ -774,23 +808,45 @@
         retrieved from self after this call.
 
         It is illegal for the two weaves to contain different values 
-        for any version.
+        or different parents for any version.  See also reweave().
         """
         if other.numversions() == 0:
             return          # nothing to update, easy
+        # two loops so that we do not change ourselves before verifying it
+        # will be ok
         # work through in index order to make sure we get all dependencies
         for other_idx, name in enumerate(other._names):
-            # TODO: If all the parents of the other version are already 
+            self._check_version_consistent(other, other_idx, name)
+
+        merged = 0
+        processed = 0
+        time0 = time.time( )
+        for other_idx, name in enumerate(other._names):
+            # TODO: If all the parents of the other version are already
             # present then we can avoid some work by just taking the delta
             # and adjusting the offsets.
-            if self._check_version_consistent(other, other_idx, name):
-                continue
             new_parents = self._imported_parents(other, other_idx)
+            sha1 = other._sha1s[other_idx]
+
+            processed += 1
+           
+            if name in self._names:
+                idx = self.lookup(name)
+                n1 = map(other.idx_to_name, other._parents[other_idx] )
+                n2 = map(self.idx_to_name, self._parents[other_idx] )
+                if sha1 ==  self._sha1s[idx] and n1 == n2:
+                        continue
+
+            merged += 1
             lines = other.get_lines(other_idx)
-            sha1 = other._sha1s[other_idx]
             self.add(name, new_parents, lines, sha1)
 
-
+        mutter("merged = %d, processed = %d, file_id=%s; deltat=%d"%(
+                merged,processed,self._weave_name, time.time( )-time0))
+
+
+
+ 
     def _imported_parents(self, other, other_idx):
         """Return list of parents in self corresponding to indexes in other."""
         new_parents = []
@@ -828,14 +884,64 @@
             n1.sort()
             n2.sort()
             if n1 != n2:
-                raise WeaveError("inconsistent parents for version {%s}: "
-                                 "%s vs %s"
-                                 % (name, n1, n2))
+                raise WeaveParentMismatch("inconsistent parents "
+                    "for version {%s}: %s vs %s" % (name, n1, n2))
             else:
                 return True         # ok!
         else:
             return False
 
+    def reweave(self, other):
+        """Reweave self with other."""
+        new_weave = reweave(self, other)
+        for attr in self.__slots__:
+            setattr(self, attr, getattr(new_weave, attr))
+
+
+def reweave(wa, wb):
+    """Combine two weaves and return the result.
+
+    This works even if a revision R has different parents in 
+    wa and wb.  In the resulting weave all the parents are given.
+
+    This is done by just building up a new weave, maintaining ordering 
+    of the versions in the two inputs.  More efficient approaches
+    might be possible but it should only be necessary to do 
+    this operation rarely, when a new previously ghost version is 
+    inserted.
+    """
+    wr = Weave()
+    ia = ib = 0
+    queue_a = range(wa.numversions())
+    queue_b = range(wb.numversions())
+    # first determine combined parents of all versions
+    # map from version name -> all parent names
+    combined_parents = _reweave_parent_graphs(wa, wb)
+    mutter("combined parents: %r", combined_parents)
+    order = topo_sort(combined_parents.iteritems())
+    mutter("order to reweave: %r", order)
+    for name in order:
+        if name in wa._name_map:
+            lines = wa.get_lines(name)
+            if name in wb._name_map:
+                assert lines == wb.get_lines(name)
+        else:
+            lines = wb.get_lines(name)
+        wr.add(name, combined_parents[name], lines)
+    return wr
+
+
+def _reweave_parent_graphs(wa, wb):
+    """Return combined parent ancestry for two weaves.
+    
+    Returned as a list of (version_name, set(parent_names))"""
+    combined = {}
+    for weave in [wa, wb]:
+        for idx, name in enumerate(weave._names):
+            p = combined.setdefault(name, set())
+            p.update(map(weave.idx_to_name, weave._parents[idx]))
+    return combined
+
 
 def weave_toc(w):
     """Show the weave's table-of-contents"""
@@ -1083,12 +1189,24 @@
     return ret
 
 
+def lsprofile_main(argv): 
+    from bzrlib.lsprof import profile
+    ret,stats = profile(main, argv)
+    stats.sort()
+    stats.pprint()
+    return ret
+
+
 if __name__ == '__main__':
     import sys
     if '--profile' in sys.argv:
         args = sys.argv[:]
         args.remove('--profile')
         sys.exit(profile_main(args))
+    elif '--lsprof' in sys.argv:
+        args = sys.argv[:]
+        args.remove('--lsprof')
+        sys.exit(lsprofile_main(args))
     else:
         sys.exit(main(sys.argv))
 

=== modified file 'bzrlib/weavefile.py'
--- bzrlib/weavefile.py	2005-09-27 07:24:40 +0000
+++ bzrlib/weavefile.py	2005-12-08 07:35:29 +0000
@@ -92,11 +92,11 @@
 
 
 
-def read_weave(f):
-    return read_weave_v5(f)
-
-
-def read_weave_v5(f):
+def read_weave(f,prelude=False):
+    return read_weave_v5(f,prelude=prelude)
+
+
+def read_weave_v5(f,prelude=False):
     from weave import Weave, WeaveFormatError
     w = Weave(getattr(f, 'name', None))
 
@@ -133,6 +133,9 @@
         else:
             raise WeaveFormatError('unexpected line %r' % l)
 
+    if prelude:
+        return w
+
     while True:
         l = f.readline()
         if l == 'W\n':

=== added file 'bzrlib/win32console.py'
--- bzrlib/win32console.py	1970-01-01 00:00:00 +0000
+++ bzrlib/win32console.py	2005-10-21 03:33:18 +0000
@@ -0,0 +1,40 @@
+
+"""
+Set of functions to work with console on Windows.
+Author: Alexander Belchenko (e-mail: bialix AT ukr.net)
+License: Public domain
+"""
+
+import struct
+
+try:
+   import ctypes
+except ImportError:
+   ctypes = None
+
+
+def get_console_size(defaultx=80, defaulty=25):
+   """ Return size of current console.
+
+   This function try to determine actual size of current working
+   console window and return tuple (sizex, sizey) if success,
+   or default size (defaultx, defaulty) otherwise.
+
+   Dependencies: ctypes should be installed.
+   """
+   if ctypes is None:
+       # no ctypes is found
+       return (defaultx, defaulty)
+
+   h = ctypes.windll.kernel32.GetStdHandle(-11)
+   csbi = ctypes.create_string_buffer(22)
+   res = ctypes.windll.kernel32.GetConsoleScreenBufferInfo(h, csbi)
+
+   if res:
+       (bufx, bufy, curx, cury, wattr,
+        left, top, right, bottom, maxx, maxy) = struct.unpack("hhhhHhhhhhh", csbi.raw)
+       sizex = right - left + 1
+       sizey = bottom - top + 1
+       return (sizex, sizey)
+   else:
+       return (defaultx, defaulty)

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2005-10-06 04:29:10 +0000
+++ bzrlib/workingtree.py	2006-01-11 19:18:38 +0000
@@ -14,20 +14,104 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-# TODO: Don't allow WorkingTrees to be constructed for remote branches.
+"""WorkingTree object and friends.
+
+A WorkingTree represents the editable working copy of a branch.
+Operations which represent the WorkingTree are also done here, 
+such as renaming or adding files.  The WorkingTree has an inventory 
+which is updated by these operations.  A commit produces a 
+new revision based on the workingtree and its inventory.
+
+At the moment every WorkingTree has its own branch.  Remote
+WorkingTrees aren't supported.
+
+To get a WorkingTree, call Branch.working_tree():
+"""
+
+
+# TODO: Don't allow WorkingTrees to be constructed for remote branches if 
+# they don't work.
 
 # FIXME: I don't know if writing out the cache from the destructor is really a
-# good idea, because destructors are considered poor taste in Python, and
-# it's not predictable when it will be written out.
-
+# good idea, because destructors are considered poor taste in Python, and it's
+# not predictable when it will be written out.
+
+# TODO: Give the workingtree sole responsibility for the working inventory;
+# remove the variable and references to it from the branch.  This may require
+# updating the commit code so as to update the inventory within the working
+# copy, and making sure there's only one WorkingTree for any directory on disk.
+# At the momenthey may alias the inventory and have old copies of it in memory.
+
+from copy import deepcopy
 import os
 import stat
 import fnmatch
-        
+ 
+from bzrlib.branch import (Branch,
+                           is_control_file,
+                           needs_read_lock,
+                           needs_write_lock,
+                           quotefn)
+from bzrlib.errors import (BzrCheckError,
+                           BzrError,
+                           DivergedBranches,
+                           WeaveRevisionNotPresent,
+                           NotBranchError,
+                           NotVersionedError)
+from bzrlib.inventory import InventoryEntry
+from bzrlib.osutils import (appendpath,
+                            compact_date,
+                            file_kind,
+                            isdir,
+                            getcwd,
+                            pathjoin,
+                            pumpfile,
+                            splitpath,
+                            rand_bytes,
+                            abspath,
+                            normpath,
+                            realpath,
+                            relpath,
+                            rename)
+from bzrlib.textui import show_status
 import bzrlib.tree
-from bzrlib.osutils import appendpath, file_kind, isdir, splitpath
-from bzrlib.errors import BzrCheckError
 from bzrlib.trace import mutter
+import bzrlib.xml5
+
+
+def gen_file_id(name):
+    """Return new file id.
+
+    This should probably generate proper UUIDs, but for the moment we
+    cope with just randomness because running uuidgen every time is
+    slow."""
+    import re
+    from binascii import hexlify
+    from time import time
+
+    # get last component
+    idx = name.rfind('/')
+    if idx != -1:
+        name = name[idx+1 : ]
+    idx = name.rfind('\\')
+    if idx != -1:
+        name = name[idx+1 : ]
+
+    # make it not a hidden file
+    name = name.lstrip('.')
+
+    # remove any wierd characters; we don't escape them but rather
+    # just pull them out
+    name = re.sub(r'[^\w.]', '', name)
+
+    s = hexlify(rand_bytes(8))
+    return '-'.join((name, compact_date(time()), s))
+
+
+def gen_root_id():
+    """Return a new tree-root file id."""
+    return gen_file_id('TREE_ROOT')
+
 
 class TreeEntry(object):
     """An entry that implements the minium interface used by commands.
@@ -93,16 +177,32 @@
     It is possible for a `WorkingTree` to have a filename which is
     not listed in the Inventory and vice versa.
     """
-    def __init__(self, basedir, inv):
+
+    def __init__(self, basedir=u'.', branch=None):
+        """Construct a WorkingTree for basedir.
+
+        If the branch is not supplied, it is opened automatically.
+        If the branch is supplied, it must be the branch for this basedir.
+        (branch.base is not cross checked, because for remote branches that
+        would be meaningless).
+        """
         from bzrlib.hashcache import HashCache
         from bzrlib.trace import note, mutter
-
-        self._inventory = inv
-        self.basedir = basedir
-        self.path2id = inv.path2id
+        assert isinstance(basedir, basestring), \
+            "base directory %r is not a string" % basedir
+        if branch is None:
+            branch = Branch.open(basedir)
+        assert isinstance(branch, Branch), \
+            "branch %r is not a Branch" % branch
+        self.branch = branch
+        self.basedir = realpath(basedir)
 
         # update the whole cache up front and write to disk if anything changed;
         # in the future we might want to do this more selectively
+        # two possible ways offer themselves : in self._unlock, write the cache
+        # if needed, or, when the cache sees a change, append it to the hash
+        # cache file, and have the parser take the most recent entry for a
+        # given path only.
         hc = self._hashcache = HashCache(basedir)
         hc.read()
         hc.scan()
@@ -110,12 +210,46 @@
         if hc.needs_write:
             mutter("write hc")
             hc.write()
-            
-            
-    def __del__(self):
-        if self._hashcache.needs_write:
-            self._hashcache.write()
-
+
+        self._set_inventory(self.read_working_inventory())
+
+    def _set_inventory(self, inv):
+        self._inventory = inv
+        self.path2id = self._inventory.path2id
+
+    @staticmethod
+    def open_containing(path=None):
+        """Open an existing working tree which has its root about path.
+        
+        This probes for a working tree at path and searches upwards from there.
+
+        Basically we keep looking up until we find the control directory or
+        run into /.  If there isn't one, raises NotBranchError.
+        TODO: give this a new exception.
+        If there is one, it is returned, along with the unused portion of path.
+        """
+        if path is None:
+            path = getcwd()
+        else:
+            # sanity check.
+            if path.find('://') != -1:
+                raise NotBranchError(path=path)
+        path = abspath(path)
+        tail = u''
+        while True:
+            try:
+                return WorkingTree(path), tail
+            except NotBranchError:
+                pass
+            if tail:
+                tail = pathjoin(os.path.basename(path), tail)
+            else:
+                tail = os.path.basename(path)
+            lastpath = path
+            path = os.path.dirname(path)
+            if lastpath == path:
+                # reached the root, whatever that may be
+                raise NotBranchError(path=path)
 
     def __iter__(self):
         """Iterate through file_ids for this tree.
@@ -128,15 +262,16 @@
             if bzrlib.osutils.lexists(self.abspath(path)):
                 yield ie.file_id
 
-
     def __repr__(self):
         return "<%s of %s>" % (self.__class__.__name__,
                                getattr(self, 'basedir', None))
 
-
-
     def abspath(self, filename):
-        return os.path.join(self.basedir, filename)
+        return pathjoin(self.basedir, filename)
+
+    def relpath(self, abs):
+        """Return the local path portion from a given absolute path."""
+        return relpath(self.basedir, abs)
 
     def has_filename(self, filename):
         return bzrlib.osutils.lexists(self.abspath(filename))
@@ -147,15 +282,24 @@
     def get_file_byname(self, filename):
         return file(self.abspath(filename), 'rb')
 
+    def get_root_id(self):
+        """Return the id of this trees root"""
+        inv = self.read_working_inventory()
+        return inv.root.file_id
+        
     def _get_store_filename(self, file_id):
-        ## XXX: badly named; this isn't in the store at all
+        ## XXX: badly named; this is not in the store at all
         return self.abspath(self.id2path(file_id))
 
+    @needs_write_lock
+    def commit(self, *args, **kw):
+        from bzrlib.commit import Commit
+        Commit().commit(self.branch, *args, **kw)
+        self._set_inventory(self.read_working_inventory())
 
     def id2abspath(self, file_id):
         return self.abspath(self.id2path(file_id))
 
-                
     def has_id(self, file_id):
         # files that have been deleted are excluded
         inv = self._inventory
@@ -164,18 +308,21 @@
         path = inv.id2path(file_id)
         return bzrlib.osutils.lexists(self.abspath(path))
 
+    def has_or_had_id(self, file_id):
+        if file_id == self.inventory.root.file_id:
+            return True
+        return self.inventory.has_id(file_id)
 
     __contains__ = has_id
-    
 
     def get_file_size(self, file_id):
         return os.path.getsize(self.id2abspath(file_id))
 
+    @needs_read_lock
     def get_file_sha1(self, file_id):
         path = self._inventory.id2path(file_id)
         return self._hashcache.get_sha1(path)
 
-
     def is_executable(self, file_id):
         if os.name == "nt":
             return self._inventory[file_id].executable
@@ -184,8 +331,105 @@
             mode = os.lstat(self.abspath(path)).st_mode
             return bool(stat.S_ISREG(mode) and stat.S_IEXEC&mode)
 
+    @needs_write_lock
+    def add(self, files, ids=None):
+        """Make files versioned.
+
+        Note that the command line normally calls smart_add instead,
+        which can automatically recurse.
+
+        This adds the files to the inventory, so that they will be
+        recorded by the next commit.
+
+        files
+            List of paths to add, relative to the base of the tree.
+
+        ids
+            If set, use these instead of automatically generated ids.
+            Must be the same length as the list of files, but may
+            contain None for ids that are to be autogenerated.
+
+        TODO: Perhaps have an option to add the ids even if the files do
+              not (yet) exist.
+
+        TODO: Perhaps callback with the ids and paths as they're added.
+        """
+        # TODO: Re-adding a file that is removed in the working copy
+        # should probably put it back with the previous ID.
+        if isinstance(files, basestring):
+            assert(ids is None or isinstance(ids, basestring))
+            files = [files]
+            if ids is not None:
+                ids = [ids]
+
+        if ids is None:
+            ids = [None] * len(files)
+        else:
+            assert(len(ids) == len(files))
+
+        inv = self.read_working_inventory()
+        for f,file_id in zip(files, ids):
+            if is_control_file(f):
+                raise BzrError("cannot add control file %s" % quotefn(f))
+
+            fp = splitpath(f)
+
+            if len(fp) == 0:
+                raise BzrError("cannot add top-level %r" % f)
+
+            fullpath = normpath(self.abspath(f))
+
+            try:
+                kind = file_kind(fullpath)
+            except OSError:
+                # maybe something better?
+                raise BzrError('cannot add: not a regular file, symlink or directory: %s' % quotefn(f))
+
+            if not InventoryEntry.versionable_kind(kind):
+                raise BzrError('cannot add: not a versionable file ('
+                               'i.e. regular file, symlink or directory): %s' % quotefn(f))
+
+            if file_id is None:
+                file_id = gen_file_id(f)
+            inv.add_path(f, kind=kind, file_id=file_id)
+
+            mutter("add file %s file_id:{%s} kind=%r" % (f, file_id, kind))
+        self._write_inventory(inv)
+
+    @needs_write_lock
+    def add_pending_merge(self, *revision_ids):
+        # TODO: Perhaps should check at this point that the
+        # history of the revision is actually present?
+        p = self.pending_merges()
+        updated = False
+        for rev_id in revision_ids:
+            if rev_id in p:
+                continue
+            p.append(rev_id)
+            updated = True
+        if updated:
+            self.set_pending_merges(p)
+
+    def pending_merges(self):
+        """Return a list of pending merges.
+
+        These are revisions that have been merged into the working
+        directory but not yet committed.
+        """
+        cfn = self.branch._rel_controlfilename('pending-merges')
+        if not self.branch._transport.has(cfn):
+            return []
+        p = []
+        for l in self.branch.controlfile('pending-merges', 'r').readlines():
+            p.append(l.rstrip('\n'))
+        return p
+
+    @needs_write_lock
+    def set_pending_merges(self, rev_list):
+        self.branch.put_controlfile('pending-merges', '\n'.join(rev_list))
+
     def get_symlink_target(self, file_id):
-        return os.readlink(self.id2path(file_id))
+        return os.readlink(self.id2abspath(file_id))
 
     def file_class(self, filename):
         if self.path2id(filename):
@@ -265,12 +509,145 @@
                 for ff in descend(fp, f_ie.file_id, fap):
                     yield ff
 
-        for f in descend('', inv.root.file_id, self.basedir):
+        for f in descend(u'', inv.root.file_id, self.basedir):
             yield f
-            
-
-
+
+    @needs_write_lock
+    def move(self, from_paths, to_name):
+        """Rename files.
+
+        to_name must exist in the inventory.
+
+        If to_name exists and is a directory, the files are moved into
+        it, keeping their old names.  
+
+        Note that to_name is only the last component of the new name;
+        this doesn't change the directory.
+
+        This returns a list of (from_path, to_path) pairs for each
+        entry that is moved.
+        """
+        result = []
+        ## TODO: Option to move IDs only
+        assert not isinstance(from_paths, basestring)
+        inv = self.inventory
+        to_abs = self.abspath(to_name)
+        if not isdir(to_abs):
+            raise BzrError("destination %r is not a directory" % to_abs)
+        if not self.has_filename(to_name):
+            raise BzrError("destination %r not in working directory" % to_abs)
+        to_dir_id = inv.path2id(to_name)
+        if to_dir_id == None and to_name != '':
+            raise BzrError("destination %r is not a versioned directory" % to_name)
+        to_dir_ie = inv[to_dir_id]
+        if to_dir_ie.kind not in ('directory', 'root_directory'):
+            raise BzrError("destination %r is not a directory" % to_abs)
+
+        to_idpath = inv.get_idpath(to_dir_id)
+
+        for f in from_paths:
+            if not self.has_filename(f):
+                raise BzrError("%r does not exist in working tree" % f)
+            f_id = inv.path2id(f)
+            if f_id == None:
+                raise BzrError("%r is not versioned" % f)
+            name_tail = splitpath(f)[-1]
+            dest_path = appendpath(to_name, name_tail)
+            if self.has_filename(dest_path):
+                raise BzrError("destination %r already exists" % dest_path)
+            if f_id in to_idpath:
+                raise BzrError("can't move %r to a subdirectory of itself" % f)
+
+        # OK, so there's a race here, it's possible that someone will
+        # create a file in this interval and then the rename might be
+        # left half-done.  But we should have caught most problems.
+        orig_inv = deepcopy(self.inventory)
+        try:
+            for f in from_paths:
+                name_tail = splitpath(f)[-1]
+                dest_path = appendpath(to_name, name_tail)
+                result.append((f, dest_path))
+                inv.rename(inv.path2id(f), to_dir_id, name_tail)
+                try:
+                    rename(self.abspath(f), self.abspath(dest_path))
+                except OSError, e:
+                    raise BzrError("failed to rename %r to %r: %s" %
+                                   (f, dest_path, e[1]),
+                            ["rename rolled back"])
+        except:
+            # restore the inventory on error
+            self._set_inventory(orig_inv)
+            raise
+        self._write_inventory(inv)
+        return result
+
+    @needs_write_lock
+    def rename_one(self, from_rel, to_rel):
+        """Rename one file.
+
+        This can change the directory or the filename or both.
+        """
+        inv = self.inventory
+        if not self.has_filename(from_rel):
+            raise BzrError("can't rename: old working file %r does not exist" % from_rel)
+        if self.has_filename(to_rel):
+            raise BzrError("can't rename: new working file %r already exists" % to_rel)
+
+        file_id = inv.path2id(from_rel)
+        if file_id == None:
+            raise BzrError("can't rename: old name %r is not versioned" % from_rel)
+
+        entry = inv[file_id]
+        from_parent = entry.parent_id
+        from_name = entry.name
+        
+        if inv.path2id(to_rel):
+            raise BzrError("can't rename: new name %r is already versioned" % to_rel)
+
+        to_dir, to_tail = os.path.split(to_rel)
+        to_dir_id = inv.path2id(to_dir)
+        if to_dir_id == None and to_dir != '':
+            raise BzrError("can't determine destination directory id for %r" % to_dir)
+
+        mutter("rename_one:")
+        mutter("  file_id    {%s}" % file_id)
+        mutter("  from_rel   %r" % from_rel)
+        mutter("  to_rel     %r" % to_rel)
+        mutter("  to_dir     %r" % to_dir)
+        mutter("  to_dir_id  {%s}" % to_dir_id)
+
+        inv.rename(file_id, to_dir_id, to_tail)
+
+        from_abs = self.abspath(from_rel)
+        to_abs = self.abspath(to_rel)
+        try:
+            rename(from_abs, to_abs)
+        except OSError, e:
+            inv.rename(file_id, from_parent, from_name)
+            raise BzrError("failed to rename %r to %r: %s"
+                    % (from_abs, to_abs, e[1]),
+                    ["rename rolled back"])
+        self._write_inventory(inv)
+
+    @needs_read_lock
     def unknowns(self):
+        """Return all unknown files.
+
+        These are files in the working directory that are not versioned or
+        control files or ignored.
+        
+        >>> from bzrlib.branch import ScratchBranch
+        >>> b = ScratchBranch(files=['foo', 'foo~'])
+        >>> tree = WorkingTree(b.base, b)
+        >>> map(str, tree.unknowns())
+        ['foo']
+        >>> tree.add('foo')
+        >>> list(b.unknowns())
+        []
+        >>> tree.remove('foo')
+        >>> list(b.unknowns())
+        [u'foo']
+        """
         for subp in self.extras():
             if not self.is_ignored(subp):
                 yield subp
@@ -285,6 +662,26 @@
                 conflicted.add(stem)
                 yield stem
 
+    @needs_write_lock
+    def pull(self, source, overwrite=False):
+        from bzrlib.merge import merge_inner
+        source.lock_read()
+        try:
+            old_revision_history = self.branch.revision_history()
+            count = self.branch.pull(source, overwrite)
+            new_revision_history = self.branch.revision_history()
+            if new_revision_history != old_revision_history:
+                if len(old_revision_history):
+                    other_revision = old_revision_history[-1]
+                else:
+                    other_revision = None
+                merge_inner(self.branch,
+                            self.branch.basis_tree(), 
+                            self.branch.revision_tree(other_revision))
+            return count
+        finally:
+            source.unlock()
+
     def extras(self):
         """Yield all unknown files in this WorkingTree.
 
@@ -296,7 +693,7 @@
         """
         ## TODO: Work from given directory downwards
         for path, dir_entry in self.inventory.directories():
-            mutter("search for unknowns in %r" % path)
+            mutter("search for unknowns in %r", path)
             dirabs = self.abspath(path)
             if not isdir(dirabs):
                 # e.g. directory deleted
@@ -376,6 +773,169 @@
         else:
             return None
 
+    def kind(self, file_id):
+        return file_kind(self.id2abspath(file_id))
+
+    def lock_read(self):
+        """See Branch.lock_read, and WorkingTree.unlock."""
+        return self.branch.lock_read()
+
+    def lock_write(self):
+        """See Branch.lock_write, and WorkingTree.unlock."""
+        return self.branch.lock_write()
+
+    def _basis_inventory_name(self, revision_id):
+        return 'basis-inventory.%s' % revision_id
+
+    def set_last_revision(self, new_revision, old_revision=None):
+        if old_revision:
+            try:
+                path = self._basis_inventory_name(old_revision)
+                path = self.branch._rel_controlfilename(path)
+                self.branch._transport.delete(path)
+            except:
+                pass
+        try:
+            xml = self.branch.get_inventory_xml(new_revision)
+            path = self._basis_inventory_name(new_revision)
+            self.branch.put_controlfile(path, xml)
+        except WeaveRevisionNotPresent:
+            pass
+
+    def read_basis_inventory(self, revision_id):
+        """Read the cached basis inventory."""
+        path = self._basis_inventory_name(revision_id)
+        return self.branch.controlfile(path, 'r').read()
+        
+    @needs_read_lock
+    def read_working_inventory(self):
+        """Read the working inventory."""
+        # ElementTree does its own conversion from UTF-8, so open in
+        # binary.
+        f = self.branch.controlfile('inventory', 'rb')
+        return bzrlib.xml5.serializer_v5.read_inventory(f)
+
+    @needs_write_lock
+    def remove(self, files, verbose=False):
+        """Remove nominated files from the working inventory..
+
+        This does not remove their text.  This does not run on XXX on what? RBC
+
+        TODO: Refuse to remove modified files unless --force is given?
+
+        TODO: Do something useful with directories.
+
+        TODO: Should this remove the text or not?  Tough call; not
+        removing may be useful and the user can just use use rm, and
+        is the opposite of add.  Removing it is consistent with most
+        other tools.  Maybe an option.
+        """
+        ## TODO: Normalize names
+        ## TODO: Remove nested loops; better scalability
+        if isinstance(files, basestring):
+            files = [files]
+
+        inv = self.inventory
+
+        # do this before any modifications
+        for f in files:
+            fid = inv.path2id(f)
+            if not fid:
+                # TODO: Perhaps make this just a warning, and continue?
+                # This tends to happen when 
+                raise NotVersionedError(path=f)
+            mutter("remove inventory entry %s {%s}", quotefn(f), fid)
+            if verbose:
+                # having remove it, it must be either ignored or unknown
+                if self.is_ignored(f):
+                    new_status = 'I'
+                else:
+                    new_status = '?'
+                show_status(new_status, inv[fid].kind, quotefn(f))
+            del inv[fid]
+
+        self._write_inventory(inv)
+
+    @needs_write_lock
+    def revert(self, filenames, old_tree=None, backups=True):
+        from bzrlib.merge import merge_inner
+        if old_tree is None:
+            old_tree = self.branch.basis_tree()
+        merge_inner(self.branch, old_tree,
+                    self, ignore_zero=True,
+                    backup_files=backups, 
+                    interesting_files=filenames)
+        if not len(filenames):
+            self.set_pending_merges([])
+
+    @needs_write_lock
+    def set_inventory(self, new_inventory_list):
+        from bzrlib.inventory import (Inventory,
+                                      InventoryDirectory,
+                                      InventoryEntry,
+                                      InventoryFile,
+                                      InventoryLink)
+        inv = Inventory(self.get_root_id())
+        for path, file_id, parent, kind in new_inventory_list:
+            name = os.path.basename(path)
+            if name == "":
+                continue
+            # fixme, there should be a factory function inv,add_?? 
+            if kind == 'directory':
+                inv.add(InventoryDirectory(file_id, name, parent))
+            elif kind == 'file':
+                inv.add(InventoryFile(file_id, name, parent))
+            elif kind == 'symlink':
+                inv.add(InventoryLink(file_id, name, parent))
+            else:
+                raise BzrError("unknown kind %r" % kind)
+        self._write_inventory(inv)
+
+    @needs_write_lock
+    def set_root_id(self, file_id):
+        """Set the root id for this tree."""
+        inv = self.read_working_inventory()
+        orig_root_id = inv.root.file_id
+        del inv._byid[inv.root.file_id]
+        inv.root.file_id = file_id
+        inv._byid[inv.root.file_id] = inv.root
+        for fid in inv:
+            entry = inv[fid]
+            if entry.parent_id in (None, orig_root_id):
+                entry.parent_id = inv.root.file_id
+        self._write_inventory(inv)
+
+    def unlock(self):
+        """See Branch.unlock.
+        
+        WorkingTree locking just uses the Branch locking facilities.
+        This is current because all working trees have an embedded branch
+        within them. IF in the future, we were to make branch data shareable
+        between multiple working trees, i.e. via shared storage, then we 
+        would probably want to lock both the local tree, and the branch.
+        """
+        if self._hashcache.needs_write and self.branch._lock_count==1:
+            self._hashcache.write()
+        return self.branch.unlock()
+
+    @needs_write_lock
+    def _write_inventory(self, inv):
+        """Write inventory as the current inventory."""
+        from cStringIO import StringIO
+        from bzrlib.atomicfile import AtomicFile
+        sio = StringIO()
+        bzrlib.xml5.serializer_v5.write_inventory(inv, sio)
+        sio.seek(0)
+        f = AtomicFile(self.branch.controlfilename('inventory'))
+        try:
+            pumpfile(sio, f)
+            f.commit()
+        finally:
+            f.close()
+        self._set_inventory(inv)
+        mutter('wrote working inventory')
+            
+
 CONFLICT_SUFFIXES = ('.THIS', '.BASE', '.OTHER')
 def get_conflicted_stem(path):
     for suffix in CONFLICT_SUFFIXES:

=== modified file 'bzrlib/xml.py'
--- bzrlib/xml.py	2005-10-03 14:28:37 +0000
+++ bzrlib/xml.py	2005-12-01 00:33:31 +0000
@@ -1,4 +1,3 @@
-#! /usr/bin/env python
 # -*- coding: UTF-8 -*-
 
 # This program is free software; you can redistribute it and/or modify
@@ -29,8 +28,8 @@
     from cElementTree import (ElementTree, SubElement, Element,
                               XMLTreeBuilder, fromstring, tostring)
 except ImportError:
-    ## from warnings import warn
-    ## warn('using slower ElementTree; consider installing cElementTree')
+    mutter('WARNING: using slower ElementTree; consider installing cElementTree'
+           " and make sure it's on your PYTHONPATH")
     from util.elementtree.ElementTree import (ElementTree, SubElement,
                                               Element, XMLTreeBuilder,
                                               fromstring, tostring)
@@ -46,7 +45,7 @@
         self._write_element(elt, f)
 
     def write_inventory_to_string(self, inv):
-        return tostring(self._pack_inventory(inv))
+        return tostring(self._pack_inventory(inv)) + '\n'
 
     def read_inventory_from_string(self, xml_string):
         return self._unpack_inventory(fromstring(xml_string))
@@ -58,7 +57,7 @@
         self._write_element(self._pack_revision(rev), f)
 
     def write_revision_to_string(self, rev):
-        return tostring(self._pack_revision(rev), f)
+        return tostring(self._pack_revision(rev)) + '\n'
 
     def read_revision(self, f):
         return self._unpack_revision(self._read_element(f))

=== modified file 'bzrlib/xml4.py'
--- bzrlib/xml4.py	2005-10-03 15:19:25 +0000
+++ bzrlib/xml4.py	2005-11-10 03:03:53 +0000
@@ -1,5 +1,3 @@
-#! /usr/bin/env python
-
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
@@ -116,7 +114,7 @@
         else:
             raise BzrError("unknown kind %r" % kind)
 
-        ## mutter("read inventoryentry: %r" % (elt.attrib))
+        ## mutter("read inventoryentry: %r", elt.attrib)
 
         return ie
 

=== modified file 'bzrlib/xml5.py'
--- bzrlib/xml5.py	2005-10-06 06:31:16 +0000
+++ bzrlib/xml5.py	2005-11-04 14:33:19 +0000
@@ -1,5 +1,3 @@
-#! /usr/bin/env python
-
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
@@ -22,9 +20,6 @@
 from bzrlib.errors import BzrError
 
 
-
-
-
 class Serializer_v5(Serializer):
     """Version 5 serializer
 
@@ -99,8 +94,22 @@
                 p = SubElement(pelts, 'revision_ref')
                 p.tail = '\n'
                 p.set('revision_id', parent_id)
+        if rev.properties:
+            self._pack_revision_properties(rev, root)
         return root
-    
+
+
+    def _pack_revision_properties(self, rev, under_element):
+        top_elt = SubElement(under_element, 'properties')
+        for prop_name, prop_value in sorted(rev.properties.items()):
+            assert isinstance(prop_name, basestring) 
+            assert isinstance(prop_value, basestring) 
+            prop_elt = SubElement(top_elt, 'property')
+            prop_elt.set('name', prop_name)
+            prop_elt.text = prop_value
+            prop_elt.tail = '\n'
+        top_elt.tail = '\n'
+
 
     def _unpack_inventory(self, elt):
         """Construct from XML Element
@@ -173,12 +182,27 @@
             assert p.tag == 'revision_ref', \
                    "bad parent node tag %r" % p.tag
             rev.parent_ids.append(p.get('revision_id'))
-
+        self._unpack_revision_properties(elt, rev)
         v = elt.get('timezone')
         rev.timezone = v and int(v)
-
         rev.message = elt.findtext('message') # text of <message>
         return rev
 
 
+    def _unpack_revision_properties(self, elt, rev):
+        """Unpack properties onto a revision."""
+        props_elt = elt.find('properties')
+        assert len(rev.properties) == 0
+        if not props_elt:
+            return
+        for prop_elt in props_elt:
+            assert prop_elt.tag == 'property', \
+                "bad tag under properties list: %r" % p.tag
+            name = prop_elt.get('name')
+            value = prop_elt.text
+            assert name not in rev.properties, \
+                "repeated property %r" % p.name
+            rev.properties[name] = value
+
+
 serializer_v5 = Serializer_v5()

=== modified file 'contrib/zsh/_bzr'
--- contrib/zsh/_bzr	2005-05-09 04:30:00 +0000
+++ contrib/zsh/_bzr	2005-11-18 08:10:07 +0000
@@ -9,13 +9,308 @@
 # how to get zsh to reload this file when it changes, other than by
 # starting a new zsh.
 
-# This is not very useful at the moment because it only completes on
-# commands and breaks filename expansion for other arguments.
-
-
-_arguments -S "1::bzr command:($(bzr help commands | grep -v '^   '))"
-
-
-# Local variables:
-# mode: shell-script
-# End:
+local _bzr_subcommands expl curcontext="$curcontext" state line
+local bzrRoot helpArgs parentFile
+typeset -A opt_args
+
+helpArgs=( '(--help)-h[show help message]' '(-h)--help[show help message]' )
+bzrRoot=$(bzr root 2> /dev/null)
+parentFile=$bzrRoot/.bzr/parent
+
+if [[ $service == "bzr" ]]; then
+    _arguments -C -A "-*" \
+    '*::command:->subcmd' && return 0
+
+    if (( CURRENT == 1 )); then
+      _bzr_subcommands=(${(f)"$(_call_program bzr bzr shell-complete)"})
+      _describe -t subcommand 'subcommand' _bzr_subcommands
+      return
+    fi
+
+    service="$words[1]"
+    curcontext="${curcontext%:*}=$service:"
+fi
+
+_unknownFiles() {
+    local fileList;
+    fileList=(${(ps:\0:)"$(bzr ls --null --unknown)"})
+    compadd -af fileList
+    return 0
+}
+
+_unknownRoot() {
+    local fileList;
+    fileList=(${(ps:\0:)"$(bzr ls --null --from-root --unknown)"})
+    compadd -af fileList
+    return 0
+}
+
+_versionedFiles() {
+    local fileList;
+    fileList=(${(ps:\0:)"$(bzr ls --null --versioned)"})
+    compadd -af fileList
+    return 0
+}
+
+case $service in
+    (add)
+       _arguments $helpArgs \
+       '--no-recurse[do not recurse into subdirectories]' \
+       '(-q)--quiet' \
+       '(--quiet)-q' \
+       '*:unknown files:_unknownFiles'
+    ;;
+
+    (annotate|praise|blame)
+        _arguments $helpArgs '*:files:_versionedFiles'
+    ;;
+
+    (clone|branch|get)
+        if (( CURRENT == 2 )); then
+            _arguments $helpArgs \
+            '(-r)--revision:rev:' \
+            '(--revision)-r:rev:' \
+            '--basis[specify basis branch]:basis:' \
+            '*:FROM_LOCATION:_files -/'
+        elif (( CURRENT == 3 )); then
+            _arguments $helpArgs \
+            '(-r)--revision:rev:' \
+            '(--revision)-r:rev:' \
+            '--basis[specify basis branch]:basis:' \
+            '*:TO_LOCATION:_files -/'
+        fi
+    ;;
+
+    (rename|mv)
+        if (( CURRENT == 2 )); then
+            _arguments $helpArgs '*:old name:_versionedFiles'
+        else
+            _arguments $helpArgs '*:new name:'
+        fi
+    ;;
+
+    (cat)
+        _arguments $helpArgs \
+        '(-r)--revision:rev:' \
+        '(--revision)-r:rev:' \
+        '*:file:_versionedFiles'
+    ;;
+
+    (root)
+        _arguments $helpArgs '*:file:_files'
+    ;;
+
+    (log)
+        _arguments $helpArgs \
+        '(-r)--revision[revision or range]:rev or rev range:' \
+        '(--revision)-r[revision or range]:rev or rev range:' \
+        '(--verbose)-v[show revision manifest]' \
+        '(-v)--verbose[show revision manifest]' \
+        '(--short --long)-l[use long format (default)]' \
+        '(--short -l)--long[use long format (default)]' \
+        '(-l --long)--short[use short format]' \
+        '(--message)-m[specify regexp]:regexp:' \
+        '(-m)--message[specify regexp]:regexp:' \
+        '--show-ids[show file IDs]' \
+        '--forward[reverse direction of revisions]' \
+        '--timezoe[specify timezone for dates]:timezone:' \
+        '*:file:_versionedFiles'
+    ;;
+
+    (resolve)
+        _arguments $helpArgs '--all' '*:file:_versionedFiles'
+    ;;
+
+    (status|st|stat)
+        _arguments $helpArgs \
+        '--all[include unchanged versioned files]' \
+        '--show-ids[show file IDs]' \
+        '*:file:_versionedFiles'
+    ;;
+
+    (check)
+        _arguments $helpArgs \
+        '(--verbose)-v' \
+        '(-v)--verbose' \
+        '*:DIR:_files -/'
+    ;;
+
+    (mkdir|upgrade|renames)
+        _arguments $helpArgs '*:DIR:_files -/'
+    ;;
+
+    (remove|rm)
+        _arguments $helpArgs \
+        '(--verbose)-v' \
+        '(-v)--verbose' \
+        '*:file:_versionedFiles'
+    ;;
+
+    (pull)
+        _arguments $helpArgs \
+        '--remember[Remember the specified location as a default]' \
+        '--overwrite[Ignore differences, overwrite unconditionally]' \
+        '*:local repository:_files -/'
+        [[ -r $parentFile ]] && compadd -X "Completing parents" $(cat $parentFile)
+    ;;
+
+    (missing|miss|mis)
+        _arguments $helpArgs \
+        '(--verbose -q --quiet)-v' \
+        '(-v -q --quiet)--verbose' \
+        '(--verbose -v --quiet)-q' \
+        '(-v -q --verbose)--quiet' \
+        '*:local repository:_files -/'
+        [[ -r $parentFile ]] && compadd -X "Completing parents" $(cat $parentFile)
+    ;;
+
+    (commit|checkin|ci)
+        _arguments $helpArgs \
+        '(--message)-m[commit message]:message text:' \
+        '(-m)--message[commit message]:message text:' \
+        '(--file)-F[commit message from file]:message file:' \
+        '(-F)--file[commit message from file]:message file:' \
+        '--unchanged[include unchanged files]' \
+        '(--verbose)-v' \
+        '(-v)--verbose' \
+        '*:modified files:_versionedFiles'
+    ;;
+
+    (conflicts|added|deleted|modified|unknowns|directories|ignored)
+        _arguments $helpArgs
+    ;;
+
+    (revno|init|version)
+        _arguments $helpArgs
+    ;;
+
+    (whoami)
+        _arguments $helpArgs '--email[only show e-mail address]' 
+    ;;
+
+    (inventory)
+        _arguments $helpArgs \
+        '(-r)--revision[show inventory of a revision]:revision:' \
+        '(--revision)-r[show inventory of a revision]:revision:' \
+        '--show-ids[show file IDs]'
+    ;;
+
+    (diff|dif|di)
+        _arguments $helpArgs \
+        '(-r)--revision:revision:' \
+        '(--revision)-r:revision:' \
+        '--diff-options[options to pass to gdiff]:diff options:' \
+        '*:files:_files'
+    ;;
+
+    (export)
+        _arguments $helpArgs \
+        '(-r)--revision:revision:' \
+        '(--revision)-r:revision:' \
+        '--format=[format of exported file]:format:(dir tar tgz tbz2)' \
+        '--root[root directory of patch]:_files -/' \
+        '*:destination:_files'
+    ;;
+
+    (ignore)
+        _arguments $helpArgs '*:NAME_PATTERN:_unknownRoot'
+    ;;
+
+    (info)
+        _arguments $helpArgs '*:branch:_files -/'
+    ;;
+
+    (testament)
+        _arguments $helpArgs \
+        '(-r)--revision:revision:' \
+        '(--revision)-r:revision:' \
+        '(--long)-l' \
+        '(-l)--long' \
+        '*:branch:_files -/'
+    ;;
+
+    (revert|merge-revert)
+        _arguments $helpArgs \
+        '(-r)--revision:revision:' \
+        '(--revision)-r:revision:' \
+        '--no-backup[skip generation of backup~ files]' \
+        '*:file:_versionedFiles'
+    ;;
+
+    (merge)
+        _arguments $helpArgs \
+        '(-r)--revision:revision:' \
+        '(--revision)-r:revision:' \
+        '--reprocess[Reprocess to reduce spurious conflicts]' \
+        '--show-base[Show base revision text in conflicts]' \
+        '--force[ignore uncommitted changes]' \
+        '--merge-type:merge type:(diff3 merge3 weave)' \
+        '*:local repository:_files -/'
+        [[ -r $parentFile ]] && compadd -X "Completing parents" $(cat $parentFile)
+    ;;
+
+    (ls)
+        _arguments $helpArgs \
+        '(--verbose)-v[display more information]' \
+        '(-v)--verbose[display more information]' \
+        '(-r)--revision:revision:' \
+        '(--revision)-r:revision:' \
+        '--from-root[Print all paths from the root of the branch]' \
+        '--non-recursive[Do not recurse into subdirectories]' \
+        '--null[Null separate the files]' \
+        '--ignored[Print ignored files]' \
+        '--unknown[Print unknown files]' \
+        '--versioned[Print versioned files]'
+    ;;
+
+    (move)
+        if (( CURRENT == 2 )); then
+            _arguments $helpArgs '*:files:_versionedFiles'
+        else
+            _arguments '*:destination dir:_files -/'
+        fi
+    ;;
+
+    (help)
+        _bzr_subcommands=(${(f)"$(_call_program bzr bzr shell-complete)"})
+        _arguments $helpArgs \
+        '(--long)-l' \
+        '(-l)--long' \
+        '*:subcmds:->cmds'
+        _describe -t subcommand 'subcommand' _bzr_subcommands
+    ;;
+
+    # Plugins
+
+    (visualize|visualise|viz|vis)
+        _arguments $helpArgs \
+            '(-r)--revision[starting revision]:rev:' \
+            '(--revision)-r[starting revision]:rev:'
+    ;;
+
+    (gannotate|gblame|gpraise)
+        _arguments $helpArgs \
+            '--all[show annotations on all lines]' \
+            '--plain[do not hightlight annotation lines]' \
+            '*:files:_versionedFiles'
+    ;;
+
+    (push)
+        _arguments $helpArgs \
+        '--overwrite[Ignore differences, overwrite unconditionally]' \
+        '*:local repository:_files -/'
+        [[ -r $parentFile ]] && compadd -X "Completing parents" $(cat $parentFile)
+    ;;
+
+    (clean-tree)
+        _arguments $helpArgs \
+        '--dry-run[show files to delete instead of deleting them]' \
+        '--ignored[delete all ignored files]' \
+        '--detritus[delete conflict files, merge backups, failed self-tests, *~, *.tmp, etc]'
+    ;;
+
+
+    (*)
+        _message "unknown bzr command completion: $service"
+    ;;
+esac

=== modified file 'doc/random.txt'
--- doc/random.txt	2005-04-26 05:20:17 +0000
+++ doc/random.txt	2005-11-18 08:10:07 +0000
@@ -14,7 +14,7 @@
 
 * It is possibly nice that tla by default forbids you from committing
   if emacs autosave or lock files exist -- I find it confusing to
-  commit somethin other than what is shown in the editor window
+  commit something other than what is shown in the editor window
   because there are unsaved changes. 
 
   However, grumbling about unknown files is annoying, and requiring

=== removed file 'patches/cache-remote-revisions.diff'
--- patches/cache-remote-revisions.diff	2005-06-15 04:17:12 +0000
+++ patches/cache-remote-revisions.diff	1970-01-01 00:00:00 +0000
@@ -1,105 +0,0 @@
-***************
-*** 738,777 ****
-          revisions = []
-          pb = ProgressBar(show_spinner=True)
-          total = len(revision_ids)
--         for i,f in enumerate(revision_ids):
--             revisions.append(other.get_revision(f))
--             pb.update('retrieving revisions', i+1, total)
--         pb.clear()
--         
--         needed_texts = sets.Set()
-- 
--         for index, rev in enumerate(revisions):
--             pb.update('Scanning revisions for file contents', index, total)
--             inv = other.get_inventory(str(rev.inventory_id))
--             for key, entry in inv.iter_entries():
--                 if entry.text_id is None:
--                     continue
--                 if entry.text_id not in self.text_store:
--                     needed_texts.add(entry.text_id)
--         pb.clear()
--         count = self.text_store.copy_multi(other.text_store, needed_texts, pb,
--                                            "Copying file contents")
--         pb.clear()
--         print "Added %d file contents." % count 
--         inventory_ids = [ f.inventory_id for f in revisions ]
--         count = self.inventory_store.copy_multi(other.inventory_store, 
--                                                 inventory_ids, pb,
--                                                 "Copying inventories")
--         pb.clear()
--         print "Added %d inventories." % count 
--         revision_ids = [ f.revision_id for f in revisions]
--         count = self.revision_store.copy_multi(other.revision_store, 
--                                                revision_ids, pb, 
--                                                "Copying revisions")
--         pb.clear()
--         for revision_id in revision_ids:
--             self.append_revision(revision_id)
--         print "Added %d revisions." % count
-                      
-          
-      def commit(self, *args, **kw):
---- 738,799 ----
-          revisions = []
-          pb = ProgressBar(show_spinner=True)
-          total = len(revision_ids)
-+         tmp_dir = tempfile.mkdtemp(prefix = "temp-stores-")
-+         try:
-+             tmp_rev_dir = os.path.join(tmp_dir, "revisions")
-+             os.mkdir(tmp_rev_dir)
-+             tmp_revs = ImmutableStore(tmp_rev_dir)
-+             count = tmp_revs.copy_multi(other.revision_store, revision_ids, pb, 
-+                                         "Caching revisions")
-+             #EVIL! Substituting a local partial store for a complete one
-+             #This is a significant performance boost when complete one is
-+             #a remote store. 
-+             other.revision_store = tmp_revs
-+             pb.clear()
-+ 
-+             for i,f in enumerate(revision_ids):
-+                 revisions.append(other.get_revision(f))
-+                 pb.update("Parsing revisions", i, len(revision_ids))
-+             
-+             needed_texts = sets.Set()
-+ 
-+             #Again with the EVIL.
-+             tmp_rev_dir = os.path.join(tmp_dir, "inventories")
-+             os.mkdir(tmp_rev_dir)
-+             inv_ids = [r.inventory_id for r in revisions]
-+             tmp_revs = ImmutableStore(tmp_rev_dir)
-+             count = tmp_revs.copy_multi(other.inventory_store, inv_ids, pb, 
-+                                         "Caching inventories")
-+             other.inventory_store = tmp_revs
-+             pb.clear()
-+             for index, rev in enumerate(revisions):
-+                 pb.update('Scanning revisions for file contents', index, total)
-+                 inv = other.get_inventory(str(rev.inventory_id))
-+                 for key, entry in inv.iter_entries():
-+                     if entry.text_id is None:
-+                         continue
-+                     if entry.text_id not in self.text_store:
-+                         needed_texts.add(entry.text_id)
-+             pb.clear()
-+             count = self.text_store.copy_multi(other.text_store, needed_texts, pb,
-+                                                "Copying file contents")
-+             pb.clear()
-+             print "Added %d file contents." % count 
-+             inventory_ids = [ f.inventory_id for f in revisions ]
-+             count = self.inventory_store.copy_multi(other.inventory_store, 
-+                                                     inventory_ids, pb,
-+                                                     "Copying inventories")
-+             pb.clear()
-+             print "Added %d inventories." % count 
-+             revision_ids = [ f.revision_id for f in revisions]
-+             count = self.revision_store.copy_multi(other.revision_store, 
-+                                                    revision_ids)
-+             pb.clear()
-+             for revision_id in revision_ids:
-+                 self.append_revision(revision_id)
-+             print "Added %d revisions." % count
-+         finally:
-+             shutil.rmtree(tmp_dir)
-                      
-          
-      def commit(self, *args, **kw):

=== removed file 'patches/cache_weave_inclusions.diff'
--- patches/cache_weave_inclusions.diff	2005-09-19 10:46:28 +0000
+++ patches/cache_weave_inclusions.diff	1970-01-01 00:00:00 +0000
@@ -1,85 +0,0 @@
-=== modified file 'bzrlib/weave.py'
---- bzrlib/weave.py
-+++ bzrlib/weave.py
-@@ -83,6 +83,10 @@
- 
- # TODO: Perhaps the API should work only in names to hide the integer
- # indexes from the user?
-+
-+# TODO: Is there any potential performance win by having an add()
-+# variant that is passed a pre-cooked version of the single basis
-+# version?
- 
- 
- 
-@@ -191,8 +195,8 @@
-         Set by read_weave.
-     """
- 
--    __slots__ = ['_weave', '_parents', '_sha1s', '_names', '_name_map',
--                 '_weave_name']
-+    ##__slots__ = ['_weave', '_parents', '_sha1s', '_names', '_name_map',
-+    ##             '_weave_name', '_inclusion_cache']
-     
-     def __init__(self, weave_name=None):
-         self._weave = []
-@@ -201,7 +205,7 @@
-         self._names = []
-         self._name_map = {}
-         self._weave_name = weave_name
--
-+        self._inclusion_cache = {}
- 
-     def __eq__(self, other):
-         if not isinstance(other, Weave):
-@@ -300,7 +304,6 @@
-                 self._weave.append(('{', new_version))
-                 self._weave.extend(text)
-                 self._weave.append(('}', None))
--        
-             return new_version
- 
-         if len(parents) == 1:
-@@ -308,10 +311,9 @@
-             if sha1 == self._sha1s[pv]:
-                 # special case: same as the single parent
-                 return new_version
--            
- 
-         ancestors = self.inclusions(parents)
--
-+        self._inclusion_cache[(new_version,)] = ancestors | set([new_version])
-         l = self._weave
- 
-         # basis a list of (origin, lineno, line)
-@@ -379,17 +381,19 @@
- 
-     def inclusions(self, versions):
-         """Return set of all ancestors of given version(s)."""
-+        tv = tuple(sorted(versions))
-+        cached_val = self._inclusion_cache.get(tv)
-+        if cached_val is not None:
-+            return cached_val
-         i = set(versions)
--        v = max(versions)
--        try:
--            while v >= 0:
--                if v in i:
--                    # include all its parents
--                    i.update(self._parents[v])
--                v -= 1
--            return i
--        except IndexError:
--            raise ValueError("version %d not present in weave" % v)
-+        for v in xrange(max(versions), 0, -1):
-+            if v in i:
-+                # include all its parents
-+                i.update(self._parents[v])
-+        self._inclusion_cache[tv] = i
-+        return i
-+        ## except IndexError:
-+        ##     raise ValueError("version %d not present in weave" % v)
- 
- 
-     def parents(self, version):
-

=== removed file 'patches/find-touching-from-seq.diff'
--- patches/find-touching-from-seq.diff	2005-06-06 12:33:11 +0000
+++ patches/find-touching-from-seq.diff	1970-01-01 00:00:00 +0000
@@ -1,83 +0,0 @@
-*** modified file 'bzrlib/commands.py'
---- bzrlib/commands.py 
-+++ bzrlib/commands.py 
-@@ -610,11 +610,13 @@
-     hidden = True
-     takes_args = ["filename"]
-     def run(self, filename):
-+        from bzrlib.log import find_touching_revisions
-         b = Branch(filename, lock_mode='r')
-         inv = b.read_working_inventory()
-         file_id = inv.path2id(b.relpath(filename))
--        for revno, revision_id, what in bzrlib.log.find_touching_revisions(b, file_id):
--            print "%6d %s" % (revno, what)
-+        rh = b.revision_history()
-+        for revision_id, what in find_touching_revisions(b, file_id, rh):
-+            print "%-40s %s" % (revision_id, what)
- 
- 
- class cmd_ls(Command):
-
-*** modified file 'bzrlib/log.py'
---- bzrlib/log.py 
-+++ bzrlib/log.py 
-@@ -17,7 +17,7 @@
- 
- 
- 
--def find_touching_revisions(branch, file_id):
-+def find_touching_revisions(branch, file_id, revisions):
-     """Yield a description of revisions which affect the file_id.
- 
-     Each returned element is (revno, revision_id, description)
-@@ -25,13 +25,20 @@
-     This is the list of revisions where the file is either added,
-     modified, renamed or deleted.
- 
--    TODO: Perhaps some way to limit this to only particular revisions,
--    or to traverse a non-mainline set of revisions?
-+    branch
-+        Branch to examine
-+
-+    file_id
-+        File to consider
-+
-+    revisions
-+        Sequence of revisions to search, can be
-+        branch.revision_history() or a filtered version of that
-+        or some sequence of non-mailine revisions.
-     """
-     last_ie = None
-     last_path = None
--    revno = 1
--    for revision_id in branch.revision_history():
-+    for revision_id in revisions:
-         this_inv = branch.get_revision_inventory(revision_id)
-         if file_id in this_inv:
-             this_ie = this_inv[file_id]
-@@ -46,19 +53,19 @@
-             # not present in either
-             pass
-         elif this_ie and not last_ie:
--            yield revno, revision_id, "added " + this_path
-+            yield revision_id, "added " + this_path
-         elif not this_ie and last_ie:
-             # deleted here
--            yield revno, revision_id, "deleted " + last_path
-+            yield revision_id, "deleted " + last_path
-         elif this_path != last_path:
--            yield revno, revision_id, ("renamed %s => %s" % (last_path, this_path))
-+            yield revision_id, ("renamed %s => %s" % (last_path, this_path))
-         elif (this_ie.text_size != last_ie.text_size
-               or this_ie.text_sha1 != last_ie.text_sha1):
--            yield revno, revision_id, "modified " + this_path
-+            yield revision_id, "modified " + this_path
- 
-         last_ie = this_ie
-         last_path = this_path
--        revno += 1
-+
- 
- 
- def show_log(branch,
-

=== removed file 'patches/meta-data-in-inventory.patch'
--- patches/meta-data-in-inventory.patch	2005-06-13 12:14:52 +0000
+++ patches/meta-data-in-inventory.patch	1970-01-01 00:00:00 +0000
@@ -1,300 +0,0 @@
-*** modified file 'bzrlib/commands.py'
---- bzrlib/commands.py 
-+++ bzrlib/commands.py 
-@@ -1175,7 +1175,68 @@
-         b = Branch('.')
-         statcache.update_cache(b.base, b.read_working_inventory())
- 
--
-+class cmd_meta(Command):
-+    """Get or set the meta information properties about a file.
-+
-+    bzr meta FILENAME # Display all meta information
-+    bzr meta FILENAME prop # Display the value of meta-info named prop, return error if not found
-+    bzr meta FILENAME --set prop value # Set the value of prop to value
-+    echo "the value" | bzr meta FILENAME --set prop # Set the value to whatever is read from stdin
-+    bzr meta FILENAME --unset prop # Remove the property
-+
-+    bzr meta FILENAME --revision=10 # Display the meta information for a given revision
-+        (Not supported yet)
-+    """
-+    hidden = True
-+    takes_args = ['filename', 'property?', 'value?']
-+    takes_options = ['revision', 'set', 'unset']
-+
-+    def run(self, filename, property=None, value=None, revision=None, set=False, unset=False):
-+        if isinstance(revision, list) and len(revision) > 1:
-+            raise BzrCommandError('bzr meta takes at most 1 revision.')
-+        if revision is not None and (set or unset):
-+            raise BzrCommandError('Cannot set/unset meta information in an old version.')
-+        if set and unset:
-+            raise BzrCommandError('Cannot set and unset at the same time')
-+        if not set and value:
-+            raise BzrCommandError('You must supply --set if you want to set the value of a property.')
-+
-+        b = Branch(filename)
-+        inv = b.inventory
-+        file_id = inv.path2id(b.relpath(filename))
-+        inv_entry = inv[file_id]
-+        if not property:
-+            meta = inv_entry.meta
-+            if meta: # Not having meta is the same as having an empty meta
-+                keys = meta.properties.keys()
-+                keys.sort()
-+                # The output really needs to be parseable
-+                for key in keys:
-+                    print '%s: %r' % (key, meta.properties[key])
-+        else:
-+            meta = inv_entry.meta
-+            if set:
-+                if value is None:
-+                    value = sys.stdin.read()
-+                if not meta:
-+                    from bzrlib.inventory import Meta
-+                    inv_entry.meta = Meta({property:value})
-+                else:
-+                    inv_entry.meta.properties[property] = value
-+                b.inventory = inv # This should cause it to be saved
-+            elif unset:
-+                if not meta or not meta.properties.has_key(property):
-+                    return 3 # Cannot unset a property that doesn't exist
-+                    # I wonder if this should be a different return code
-+                del inv_entry.meta.properties[property]
-+                b.inventory = inv
-+            else:
-+                if not meta or not meta.properties.has_key(property):
-+                    return 3 # Missing property
-+                # Probably this should not be print, but 
-+                # sys.stdout.write() so that you get back exactly
-+                # what was given. But I'm leaving it this way for now
-+                print meta.properties[property]
- 
- # list of all available options; the rhs can be either None for an
- # option that takes no argument, or a constructor function that checks
-@@ -1196,6 +1257,8 @@
-     'verbose':                None,
-     'version':                None,
-     'email':                  None,
-+    'set':                    None,
-+    'unset':                  None,
-     }
- 
- SHORT_OPTIONS = {
-
-*** modified file 'bzrlib/diff.py'
---- bzrlib/diff.py 
-+++ bzrlib/diff.py 
-@@ -226,8 +226,10 @@
-                       new_tree.get_file(file_id).readlines(),
-                       to_file)
- 
--    for old_path, new_path, file_id, kind, text_modified in delta.renamed:
-+    for old_path, new_path, file_id, kind, text_modified, meta_modified in delta.renamed:
-         print '*** renamed %s %r => %r' % (kind, old_path, new_path)
-+        if meta_modified:
-+            print '## Meta-info modified'
-         if text_modified:
-             diff_file(old_label + old_path,
-                       old_tree.get_file(file_id).readlines(),
-@@ -235,9 +237,11 @@
-                       new_tree.get_file(file_id).readlines(),
-                       to_file)
- 
--    for path, file_id, kind in delta.modified:
-+    for path, file_id, kind, text_modified, meta_modified in delta.modified:
-         print '*** modified %s %r' % (kind, path)
--        if kind == 'file':
-+        if meta_modified:
-+            print '## Meta-info modified'
-+        if kind == 'file' and text_modified:
-             diff_file(old_label + path,
-                       old_tree.get_file(file_id).readlines(),
-                       new_label + path,
-@@ -316,7 +320,7 @@
- 
-         if self.renamed:
-             print >>to_file, 'renamed:'
--            for oldpath, newpath, fid, kind, text_modified in self.renamed:
-+            for oldpath, newpath, fid, kind, text_modified, meta_modified in self.renamed:
-                 if show_ids:
-                     print >>to_file, '  %s => %s %s' % (oldpath, newpath, fid)
-                 else:
-@@ -324,7 +328,16 @@
-                     
-         if self.modified:
-             print >>to_file, 'modified:'
--            show_list(self.modified)
-+            for path, fid, kind, text_modified, meta_modified in self.modified:
-+                if kind == 'directory':
-+                    path += '/'
-+                elif kind == 'symlink':
-+                    path += '@'
-+                    
-+                if show_ids:
-+                    print >>to_file, '  %-30s %s' % (path, fid)
-+                else:
-+                    print >>to_file, ' ', path
-             
-         if show_unchanged and self.unchanged:
-             print >>to_file, 'unchanged:'
-@@ -388,6 +401,14 @@
-                 ## mutter("no text to check for %r %r" % (file_id, kind))
-                 text_modified = False
- 
-+            old_meta = old_inv[file_id].meta
-+            new_meta = new_inv[file_id].meta
-+
-+            if old_meta != new_meta:
-+                meta_modified = True
-+            else:
-+                meta_modified = False
-+
-             # TODO: Can possibly avoid calculating path strings if the
-             # two files are unchanged and their names and parents are
-             # the same and the parents are unchanged all the way up.
-@@ -395,9 +416,10 @@
-             
-             if old_path != new_path:
-                 delta.renamed.append((old_path, new_path, file_id, kind,
--                                      text_modified))
--            elif text_modified:
--                delta.modified.append((new_path, file_id, kind))
-+                                      text_modified, meta_modified))
-+            elif text_modified or meta_modified:
-+                delta.modified.append((new_path, file_id, kind,
-+                                       text_modified, meta_modified))
-             elif want_unchanged:
-                 delta.unchanged.append((new_path, file_id, kind))
-         else:
-
-*** modified file 'bzrlib/inventory.py'
---- bzrlib/inventory.py 
-+++ bzrlib/inventory.py 
-@@ -33,6 +33,67 @@
- import bzrlib
- from bzrlib.osutils import uuid, quotefn, splitpath, joinpath, appendpath
- from bzrlib.trace import mutter
-+
-+class Meta(XMLMixin):
-+    """Meta information about a single inventory entry.
-+
-+    This is basically just a set of key->value pairs.
-+
-+    In general, bzr does not handle this information, it only provides
-+    a location for plugins and other data sources to store this data.
-+
-+    """
-+    def __init__(self, properties):
-+        self.properties = properties
-+
-+    def __eq__(self, other):
-+        if other is None:
-+            return False
-+        if not isinstance(other, Meta):
-+            return NotImplemented
-+
-+        return (self.properties == other.properties)
-+
-+    def __ne__(self, other):
-+        return not (self == other)
-+
-+    def __hash__(self):
-+        raise ValueError('not hashable')
-+
-+    def from_element(cls, elt):
-+        assert elt.tag == 'meta'
-+
-+        properties = {}
-+        for child in elt:
-+            if child.tag == 'property':
-+                if child.text is None:
-+                    properties[child.get('name')] = ''
-+                else:
-+                    properties[child.get('name')] = child.text
-+        self = cls(properties)
-+        return self
-+
-+    from_element = classmethod(from_element)
-+
-+    def to_element(self):
-+        e = Element('meta')
-+        keys = self.properties.keys()
-+        keys.sort()
-+
-+        # The blah.text is just to make things look pretty in the files
-+        # We may want to remove it
-+        if len(keys) > 0:
-+            e.text = '\n'
-+
-+        for key in keys:
-+            prop = Element('property')
-+            prop.set('name', key)
-+            prop.text=self.properties[key]
-+            prop.tail='\n'
-+            e.append(prop)
-+
-+        return e
-+
- 
- class InventoryEntry(XMLMixin):
-     """Description of a versioned file.
-@@ -100,6 +161,7 @@
- 
-     text_sha1 = None
-     text_size = None
-+    meta = None
-     
-     def __init__(self, file_id, name, kind, parent_id, text_id=None):
-         """Create an InventoryEntry
-@@ -124,6 +186,7 @@
-         self.kind = kind
-         self.text_id = text_id
-         self.parent_id = parent_id
-+        self.meta = None
-         if kind == 'directory':
-             self.children = {}
-         elif kind == 'file':
-@@ -146,6 +209,10 @@
-         other.text_size = self.text_size
-         # note that children are *not* copied; they're pulled across when
-         # others are added
-+        if self.meta:
-+            other.meta = Meta(self.meta.properties)
-+        else:
-+            other.meta = None
-         return other
- 
- 
-@@ -182,6 +249,9 @@
-             e.set('parent_id', self.parent_id)
- 
-         e.tail = '\n'
-+
-+        if self.meta:
-+            e.append(self.meta.to_element())
-             
-         return e
- 
-@@ -205,6 +275,11 @@
-         v = elt.get('text_size')
-         self.text_size = v and int(v)
- 
-+        self.meta = None
-+        for child in elt:
-+            if child.tag == 'meta':
-+                self.meta = Meta.from_element(child)
-+
-         return self
-             
- 
-@@ -220,7 +295,8 @@
-                and (self.text_size == other.text_size) \
-                and (self.text_id == other.text_id) \
-                and (self.parent_id == other.parent_id) \
--               and (self.kind == other.kind)
-+               and (self.kind == other.kind) \
-+               and (self.meta == other.meta)
- 
- 
-     def __ne__(self, other):
-

=== removed file 'patches/ndiff.patch'
--- patches/ndiff.patch	2005-07-04 12:11:40 +0000
+++ patches/ndiff.patch	1970-01-01 00:00:00 +0000
@@ -1,149 +0,0 @@
-*** modified file 'bzrlib/commands.py'
---- bzrlib/commands.py 
-+++ bzrlib/commands.py 
-@@ -721,10 +721,11 @@
-     """
-     
-     takes_args = ['file*']
--    takes_options = ['revision', 'diff-options']
-+    takes_options = ['revision', 'diff-options', 'ndiff']
-     aliases = ['di', 'dif']
- 
--    def run(self, revision=None, file_list=None, diff_options=None):
-+    def run(self, revision=None, file_list=None, diff_options=None,
-+            ndiff=None):
-         from bzrlib.diff import show_diff
- 
-         if file_list:
-@@ -735,9 +736,15 @@
-                 file_list = None
-         else:
-             b = find_branch('.')
--    
-+
-+        if ndiff:
-+            format = 'ndiff'
-+        else:
-+            format = None
-+
-         show_diff(b, revision, specific_files=file_list,
--                  external_diff_options=diff_options)
-+                  external_diff_options=diff_options,
-+                  format=format)
- 
- 
-         
-@@ -1344,6 +1351,7 @@
-     'format':                 unicode,
-     'forward':                None,
-     'message':                unicode,
-+    'ndiff':                  None,
-     'no-recurse':             None,
-     'profile':                None,
-     'revision':               _parse_revision_str,
-
-*** modified file 'bzrlib/diff.py'
---- bzrlib/diff.py 
-+++ bzrlib/diff.py 
-@@ -70,6 +70,13 @@
-     print >>to_file
- 
- 
-+
-+def internal_ndiff(old_label, oldlines,
-+                   new_label, newlines,
-+                   to_file):
-+    """Show diff in python-specific ndiff format."""
-+    from difflib import ndiff
-+    to_file.writelines(ndiff(oldlines, newlines))
- 
- 
- def external_diff(old_label, oldlines, new_label, newlines, to_file,
-@@ -152,7 +159,8 @@
-     
- 
- 
--def show_diff(b, revision, specific_files, external_diff_options=None):
-+def show_diff(b, revision, specific_files, external_diff_options=None,
-+              format=None):
-     """Shortcut for showing the diff to the working tree.
- 
-     b
-@@ -160,6 +168,9 @@
- 
-     revision
-         None for each, or otherwise the old revision to compare against.
-+
-+    format
-+        'unified', 'context', 'ndiff', 'external'
-     
-     The more general form is show_diff_trees(), where the caller
-     supplies any two trees.
-@@ -174,12 +185,13 @@
-     new_tree = b.working_tree()
- 
-     show_diff_trees(old_tree, new_tree, sys.stdout, specific_files,
--                    external_diff_options)
-+                    external_diff_options, format)
- 
- 
- 
- def show_diff_trees(old_tree, new_tree, to_file, specific_files=None,
--                    external_diff_options=None):
-+                    external_diff_options=None,
-+                    format=None):
-     """Show in text form the changes from one tree to another.
- 
-     to_files
-@@ -204,10 +216,12 @@
-     if external_diff_options:
-         assert isinstance(external_diff_options, basestring)
-         opts = external_diff_options.split()
--        def diff_file(olab, olines, nlab, nlines, to_file):
-+        def diff_fn(olab, olines, nlab, nlines, to_file):
-             external_diff(olab, olines, nlab, nlines, to_file, opts)
-+    elif format == 'ndiff':
-+        diff_fn = internal_ndiff
-     else:
--        diff_file = internal_diff
-+        diff_fn = internal_diff
-     
- 
-     delta = compare_trees(old_tree, new_tree, want_unchanged=False,
-@@ -216,7 +230,7 @@
-     for path, file_id, kind in delta.removed:
-         print >>to_file, '*** removed %s %r' % (kind, path)
-         if kind == 'file':
--            diff_file(old_label + path,
-+            diff_fn(old_label + path,
-                       old_tree.get_file(file_id).readlines(),
-                       DEVNULL, 
-                       [],
-@@ -225,7 +239,7 @@
-     for path, file_id, kind in delta.added:
-         print >>to_file, '*** added %s %r' % (kind, path)
-         if kind == 'file':
--            diff_file(DEVNULL,
-+            diff_fn(DEVNULL,
-                       [],
-                       new_label + path,
-                       new_tree.get_file(file_id).readlines(),
-@@ -234,7 +248,7 @@
-     for old_path, new_path, file_id, kind, text_modified in delta.renamed:
-         print >>to_file, '*** renamed %s %r => %r' % (kind, old_path, new_path)
-         if text_modified:
--            diff_file(old_label + old_path,
-+            diff_fn(old_label + old_path,
-                       old_tree.get_file(file_id).readlines(),
-                       new_label + new_path,
-                       new_tree.get_file(file_id).readlines(),
-@@ -243,7 +257,7 @@
-     for path, file_id, kind in delta.modified:
-         print >>to_file, '*** modified %s %r' % (kind, path)
-         if kind == 'file':
--            diff_file(old_label + path,
-+            diff_fn(old_label + path,
-                       old_tree.get_file(file_id).readlines(),
-                       new_label + path,
-                       new_tree.get_file(file_id).readlines(),
-

=== modified file 'setup.py' (properties changed: -x to +x)
--- setup.py	2005-10-06 12:34:22 +0000
+++ setup.py	2005-12-05 00:22:26 +0000
@@ -4,66 +4,103 @@
 # './setup.py install', or
 # './setup.py --help' for more options
 
+# Reinvocation stolen from bzr, we need python2.4 by virtue of bzr_man
+# including bzrlib.help
+
+import os, sys
+
+try:
+    version_info = sys.version_info
+except AttributeError:
+    version_info = 1, 5 # 1.5 or older
+
+REINVOKE = "__BZR_REINVOKE"
+NEED_VERS = (2, 4)
+KNOWN_PYTHONS = ('python2.4',)
+
+if version_info < NEED_VERS:
+    if not os.environ.has_key(REINVOKE):
+        # mutating os.environ doesn't work in old Pythons
+        os.putenv(REINVOKE, "1")
+        for python in KNOWN_PYTHONS:
+            try:
+                os.execvp(python, [python] + sys.argv)
+            except OSError:
+                pass
+    print >>sys.stderr, "bzr: error: cannot find a suitable python interpreter"
+    print >>sys.stderr, "  (need %d.%d or later)" % NEED_VERS
+    sys.exit(1)
+if hasattr(os, "unsetenv"):
+    os.unsetenv(REINVOKE)
+
+
 from distutils.core import setup
-
-# more sophisticated setup script, based on pychecker setup.py
-import sys, os
-from distutils.command.build_scripts import build_scripts
-
+from distutils.command.install_scripts import install_scripts
+from distutils.command.build import build
 
 ###############################
 # Overridden distutils actions
 ###############################
 
-class my_build_scripts(build_scripts):
-    """Customized build_scripts distutils action.
-
+class my_install_scripts(install_scripts):
+    """ Customized install_scripts distutils action.
     Create bzr.bat for win32.
     """
-
     def run(self):
+        import os
+        import sys
+
+        install_scripts.run(self)   # standard action
+
         if sys.platform == "win32":
-            bat_path = os.path.join(self.build_dir, "bzr.bat")
-            self.scripts.append(bat_path)
-            self.mkpath(self.build_dir)
-            scripts_dir = self.distribution.get_command_obj("install").\
-                                                            install_scripts
-            self.execute(func=self._create_bat,
-                         args=[bat_path, scripts_dir],
-                         msg="Create %s" % bat_path)
-        build_scripts.run(self) # invoke "standard" action
-
-    def _create_bat(self, bat_path, scripts_dir):
-        """ Creates the batch file for bzr on win32.
-        """
-        try:
-            script_path = os.path.join(scripts_dir, "bzr")
-            bat_str = "@%s %s %%*\n" % (sys.executable, script_path)
-            file(bat_path, "w").write(bat_str)
-            print "file written"
-        except Exception, e:
-            print "ERROR: Unable to create %s: %s" % (bat_path, e)
-            raise e
-
+            try:
+                scripts_dir = self.install_dir
+                script_path = os.path.join(scripts_dir, "bzr")
+                batch_str = "@%s %s %%*\n" % (sys.executable, script_path)
+                batch_path = script_path + ".bat"
+                f = file(batch_path, "w")
+                f.write(batch_str)
+                f.close()
+                print "Created:", batch_path
+            except Exception, e:
+                print "ERROR: Unable to create %s: %s" % (batch_path, e)
+
+
+class bzr_build(build):
+    """Customized build distutils action.
+    Generate bzr.1.
+    """
+    def run(self):
+        build.run(self)
+
+        import bzr_man
+        bzr_man.main()
 
 ########################
 ## Setup
 ########################
 
 setup(name='bzr',
-      version='0.1',
+      version='0.7pre',
       author='Martin Pool',
       author_email='mbp@sourcefrog.net',
       url='http://www.bazaar-ng.org/',
       description='Friendly distributed version control system',
       license='GNU GPL v2',
       packages=['bzrlib',
+                'bzrlib.export',
                 'bzrlib.plugins',
-                'bzrlib.selftest',
+                'bzrlib.store',
+                'bzrlib.tests',
+                'bzrlib.tests.blackbox',
+                'bzrlib.transport',
+                'bzrlib.ui',
                 'bzrlib.util',
-                'bzrlib.transport',
-                'bzrlib.store',
                 'bzrlib.util.elementtree',
                 'bzrlib.util.effbot.org',
+                'bzrlib.util.configobj',
                 ],
-      scripts=['bzr'])
+      scripts=['bzr'],
+      cmdclass={'install_scripts': my_install_scripts, 'build': bzr_build},
+      data_files=[('man/man1', ['bzr.1'])],
+     )

=== removed file 'testbzr'
--- testbzr	2005-06-22 06:37:43 +0000
+++ testbzr	1970-01-01 00:00:00 +0000
@@ -1,23 +0,0 @@
-#! /usr/bin/python
-# -*- coding: utf-8 -*-
-
-# Copyright (C) 2005 Canonical Ltd
-
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-
-print 'please use "bzr selftest" instead'
-import sys
-sys.exit(1)
-

=== added file 'tools/biobench.py'
--- tools/biobench.py	1970-01-01 00:00:00 +0000
+++ tools/biobench.py	2005-11-04 01:46:31 +0000
@@ -0,0 +1,92 @@
+#! /usr/bin/env python
+
+# (C) 2005 Canonical Ltd
+
+"""Benchmark for basicio
+
+Tries serializing an inventory to basic_io repeatedly.
+"""
+
+if True:
+    import psyco
+    psyco.full()
+
+
+import sys
+from timeit import Timer
+from tempfile import TemporaryFile, NamedTemporaryFile
+
+from bzrlib.branch import Branch
+from bzrlib.xml5 import serializer_v5
+from bzrlib.basicio import write_inventory, BasicWriter, \
+        read_inventory
+from bzrlib.inventory import Inventory, InventoryEntry, InventoryFile, ROOT_ID
+
+## b = Branch.open('.')
+## inv = b.get_inventory(b.last_revision())
+
+nrepeats = 3
+ntimes = 5
+NFILES = 30000
+
+def make_inventory():
+    inv = Inventory()
+    for i in range(NFILES):
+        ie = InventoryFile('%08d-id' % i, '%08d-file' % i, ROOT_ID)
+        ie.text_sha1='1212121212121212121212121212121212121212'
+        ie.text_size=12312
+        inv.add(ie)
+    return inv
+
+inv = make_inventory()
+
+bio_tmp = NamedTemporaryFile()
+xml_tmp = NamedTemporaryFile()
+
+def bio_test():
+    bio_tmp.seek(0)
+    w = BasicWriter(bio_tmp)
+    write_inventory(w, inv)
+    bio_tmp.seek(0)
+    new_inv = read_inventory(bio_tmp)
+
+def xml_test():
+    xml_tmp.seek(0)
+    serializer_v5.write_inventory(inv, xml_tmp)
+    xml_tmp.seek(0)
+    new_inv = serializer_v5.read_inventory(xml_tmp)
+
+def run_benchmark(function_name, tmp_file):
+    t = Timer(function_name + '()', 
+              'from __main__ import ' + function_name)
+    times = t.repeat(nrepeats, ntimes)
+    tmp_file.seek(0, 2)
+    size = tmp_file.tell()
+    print 'wrote inventory to %10s %5d times, each %6d bytes, total %6dkB' \
+            % (function_name, ntimes, size, (size * ntimes)>>10), 
+    each = (min(times)/ntimes*1000)
+    print 'in %.1fms each' % each 
+    return each
+
+def profileit(fn): 
+    import hotshot, hotshot.stats
+    prof_f = NamedTemporaryFile()
+    prof = hotshot.Profile(prof_f.name)
+    prof.runcall(fn) 
+    prof.close()
+    stats = hotshot.stats.load(prof_f.name)
+    #stats.strip_dirs()
+    stats.sort_stats('time')
+    ## XXX: Might like to write to stderr or the trace file instead but
+    ## print_stats seems hardcoded to stdout
+    stats.print_stats(20)
+
+if '-p' in sys.argv[1:]:
+    profileit(bio_test)
+else:
+    bio_each = run_benchmark('bio_test', bio_tmp)
+    xml_each = run_benchmark('xml_test', xml_tmp)
+    print 'so bio is %.1f%% faster' % (100 * ((xml_each / bio_each) - 1))
+
+# make sure it was a fair comparison
+# assert 'cElementTree' in sys.modules

=== added file 'tools/riodemo.py'
--- tools/riodemo.py	1970-01-01 00:00:00 +0000
+++ tools/riodemo.py	2005-11-28 08:03:42 +0000
@@ -0,0 +1,60 @@
+
+
+
+# \subsection{Example usage}
+# 
+# \textbf{XXX:} Move these to object serialization code. 
+
+def write_revision(writer, revision):
+    s = Stanza(revision=revision.revision_id,
+               committer=revision.committer, 
+               timezone=long(revision.timezone),
+               timestamp=long(revision.timestamp),
+               inventory_sha1=revision.inventory_sha1,
+               message=revision.message)
+    for parent_id in revision.parent_ids:
+        s.add('parent', parent_id)
+    for prop_name, prop_value in revision.properties.items():
+        s.add(prop_name, prop_value)
+    writer.write_stanza(s)
+
+def write_inventory(writer, inventory):
+    s = Stanza(inventory_version=7)
+    writer.write_stanza(s)
+
+    for path, ie in inventory.iter_entries():
+        s = Stanza()
+        s.add(ie.kind, ie.file_id)
+        for attr in ['name', 'parent_id', 'revision', \
+                     'text_sha1', 'text_size', 'executable', \
+                     'symlink_target', \
+                     ]:
+            attr_val = getattr(ie, attr, None)
+            if attr == 'executable' and attr_val == 0:
+                continue
+            if attr == 'parent_id' and attr_val == 'TREE_ROOT':
+                continue
+            if attr_val is not None:
+                s.add(attr, attr_val)
+        writer.write_stanza(s)
+
+
+def read_inventory(inv_file):
+    """Read inventory object from rio formatted inventory file"""
+    from bzrlib.inventory import Inventory, InventoryFile
+    s = read_stanza(inv_file)
+    assert s['inventory_version'] == 7
+    inv = Inventory()
+    for s in read_stanzas(inv_file):
+        kind, file_id = s.items[0]
+        parent_id = None
+        if 'parent_id' in s:
+            parent_id = s['parent_id']
+        if kind == 'file':
+            ie = InventoryFile(file_id, s['name'], parent_id)
+            ie.text_sha1 = s['text_sha1']
+            ie.text_size = s['text_size']
+        else:
+            raise NotImplementedError()
+        inv.add(ie)
+    return inv

=== added file 'tools/trace-revisions'
--- tools/trace-revisions	1970-01-01 00:00:00 +0000
+++ tools/trace-revisions	2005-11-07 16:21:12 +0000
@@ -0,0 +1,19 @@
+#! /usr/bin/env python2.4
+
+import sys
+
+from bzrlib.branch import Branch
+
+b = Branch.open('.')
+rev_id = sys.argv[1]
+rh = []
+while True:
+    rh.append(rev_id)
+    rev = b.get_revision(rev_id)
+    if rev.parent_ids:
+        rev_id = rev.parent_ids[0]
+    else:
+        break
+rh.reverse()
+for rev_id in rh:
+    print rev_id

=== modified file 'tutorial.txt'
--- tutorial.txt	2005-09-14 09:03:27 +0000
+++ tutorial.txt	2006-01-06 12:18:04 +0000
@@ -1,76 +1,84 @@
+.. This file is in Python ReStructuredText format - it can be formatted
+.. into HTML or text.  In the future we plan to extract the example commands
+.. and automatically test them.
+
+.. This text was previously on the wiki at http://bazaar.canonical.com/IntroductionToBzr
+.. but has been moved into the source tree so it can be kept in sync with
+.. the source and possibly automatically checked.
+
 ==================
 Bazaar-NG Tutorial
 ==================
 
-current for bzr 0.0.6pre, July 2005
-
-
-*NOTE* For a more current and user-editable version of this 
-document, see the wiki at http://bazaar.canonical.com/IntroductionToBzr
+Current for bzr-0.7pre, 2006-01-06.
 
 
 Introduction
 ============
 
-Bazaar-NG is a version control tool.  It manages trees of files and subdirectories.  In particular, it records *revisions* of trees, representing their state at a particular point in time, and information about those revisions and their relationships.  Recording and retrieving tree revisions is useful in several ways if you are writing software or documents or doing similar creative work.
-
- * Keeping previous revisions lets you go back if you make a mistake or want to check your work.  It acts as a high-level unlimited undo.
-
- * By recording comments on every revision, you produce an annotated history of the project, describing what, who, why, and when.
-
- * Using a version control tool can be an aid to thinking about a project: getting to a stable state at regular intervals and then writing a description of what you did is an easy way to stay organized and on track.
-
-Bazaar-NG remembers the *ancestry* of a revision: the previous revisions that it is based upon.  A single revision may have more than one direct descendant, each with different changes, representing a divergence in the evolution of the tree.  
-By branching, Bazaar-NG allows multiple people to cooperate on the evolution of a project, without all needing to work in strict
-lock-step.  Branching can be useful even for a single developer.
-
-Bazaar-NG installs a single new command,
-*bzr*.  Everything else is a subcommand of this.  You can get
-some help with ``bzr help``.  There will be more in the future.
-
-
-
-Introducing yourself to bzr
-===========================
-
-One function of a version control system is to keep track of who
-changed what.  In a distributed system that requires an identifier for
-each author that is globally unique.  Most people already have one of
-these: an email address.
-
-[after 0.0.4] To tell bzr which email address to use, put it in the file
-``$HOME/.bzr.conf/email``, or the environment variable ``$BZREMAIL``.
-If neither of these are set, bzr will use the ``$EMAIL``
-variable, or use your username and hostname.
-
-To check this has taken effect, or if you forget your own name, use
-the ``whoami`` ("who am i?") command::
-
-  % bzr whoami
-
-Some people want to avoid sharing their email address so as not to
-get spam.  bzr will never
-disclose your email address unless you tell it to by publishing an
-archive or transmitting a changeset.  It's recommended that you do use
-a real address, so that people can contact you about your work, but
-it's not required.  You can use an address which is obfuscated, which
-bounces, or which goes through an anti-spam service such as spamgourmet.com.
-
-
+If you are already familiar with decentralized revision control, then please feel free to skip ahead to "Introducing Yourself to Bazaar-NG". If, on the other hand, you are familiar with revision control but not decentralized revision control, then please start at "How DRCS is different." Otherwise, get some coffee or tea, get comfortable and get ready to catch up. 
+
+The Purposes of Revision Control
+================================
+
+Odds are that you have worked on some sort of textual data -- the sources to a program, web sites or the config files that Unix system administrators have to deal with in /etc. The chances are also good that you have made some sort of mistake that you deeply regretted. Perhaps you deleted the configuration file for your mailserver or perhaps mauled the source code for a pet project. Whatever happened, you have just deleted important information that you would desperately like to get back. If this has ever happened to you, then you are probably ready for Bazaar-NG.
+
+Revision control systems (which I'll henceforth call RCS) such as Bazaar-NG give you the ability to track changes for a directory by turning it into something slightly more complicated than a directory that we call a **branch**. The branch not only stores how the directory looks right now, but also how it looked at various points in the past. Then, when you do something you wish you hadn't, you can restore the directory to the way it looked at some point in the past.
+
+Revision control systems give users the ability to save changes to a branch by "committing a **revision**". The revision created is essentially a summary of the changes that were made since the last time the tree was saved. 
+
+These revisions have other uses as well. For example, one can comment revisions to record what the recent set of changes meant by providing an optional log message. Real life log messages include things like "Fixed the web template to close the table" and "Added sftp suppport. Fixes #595"
+	
+We keep these logs so that if later there is some sort of problem with sftp, we can figure out when the problem probably happened. 
+
+How DRCS is Different
+---------------------
+
+Many Revision Control Systems (RCS) are stored on servers. If one wants to work on the code stored within an RCS, then one needs to connect to the server and "checkout" the code. Doing so gives one a directory in which a person can make changes and then commit. The RCS client then connects to the RCS server and stores the changes. This method is known as the centralized model. 
+
+The centralized model can have some drawbacks. A centralized RCS requires that one is able to connect to the server whenever one wants to do version control work. This can be a bit of a problem if your server on some other machine on the internet and you are not. Or, worse yet, you ''are'' on the internet but the server is missing!
+
+Decentralized Revision Control Systems (which I'll call DRCS after this point) deal with this problem by keeping branches on the same machine as the client. In Bazaar-NG's case, the branch is kept in the same place as the code that is being version controlled. This allows the user to save his changes (**commit**) whenever he wants -- even if he is offline. The user only needs internet access when he wants to access the changes in someone else's branch that are somewhere else.
+
+ 
+A common requirement that many people have is the need to keep track of the changes for a directory such as file and subdirectory changes. Performing this tracking by hand is a awkward process that over time becomes unwieldy. That is, until one considers version control tools such as Bazaar-NG. These tools automate the process of storing data by creating a **revision** of the directory tree whenever the user asks. 
+
+Revision control software such as Bazaar-NG can do much more than just storage and performing undo.  For example, with Bazaar-NG developer can take the modifications in one branch of software and apply them to another, related, branch -- even if those changes exist in a branch owned by somebody else. This allows developers to cooperate without giving write access to repository.
+
+Bazaar-NG remembers the ''ancestry'' of a revision: the previous revisions that it is based upon.  A single revision may have more than one direct descendant, each with different changes, representing a divergence in the evolution of the tree. By branching, Bazaar-NG allows multiple people to cooperate on the evolution of a project, without all needing to work in strict lock-step.  Branching can be useful even for a single developer.
+
+Introducing yourself to Bazaar-NG
+=================================
+
+Bazaar-NG installs a single new command, **bzr**.  Everything else is a subcommand of this.  You can get some help with `bzr help`.  There will be more in the future.
+
+One function of a version control system is to keep track of who changed what.  In a decentralized system, that requires an identifier for each author that is globally unique.  Most people already have one of these: an email address. Bzr is smart enough to automatically generate an email address by looking up your username and hostname. If you don't like the guess that Bazaar-NG makes, then three options exist:
+
+ 1. (**Bazaar-NG 0.6 and later**). Setting the email address in the
+ ``~/.bazaar/bazaar.conf`` by adding the following lines. Please note that
+ ``[DEFAULT]`` is case sensitive::
+
+    [DEFAULT]
+    email= Your Name <email@isp.com>
+ 
+ 1. (**Bazaar-NG 0.6 and later**) Override the previous setting on a
+ branch by branch basis by creating a branch section in
+ ``~/.bazaar/branches.conf`` by adding the following lines::
+
+    [/the/directory/to/the/branch]
+    email=Your Name <email@isp.com>
+ 
+ 1. Overriding the two previous options by setting the global environment
+ variable ``$BZREMAIL`` or ``$EMAIL`` (``$BZREMAIL`` will take precedence) to your full email address.
 
 Creating a branch
-==================
-
-
-History is by default stored in the .bzr directory of the branch.
-There will be a facility to store it in a separate repository, which
-may be remote.  We create a new branch by running *bzr init* in
-an existing directory::
+=================
+
+History is by default stored in the .bzr directory of the branch. There will be a facility to store it in a separate repository, which may be remote.  We create a new branch by running **bzr init** in an existing directory::
 
     % mkdir tutorial
     % cd tutorial
     % ls -a
-    ls -a
     ./  ../
     % pwd
     /home/mbp/work/bzr.test/tutorial
@@ -80,51 +88,59 @@
     ./  ../  .bzr/
     %
 
-As for CVS, there are three classes of file: unknown, ignored, and
-versioned.  The *add* command makes a file versioned: that is,
-changes to it will be recorded by the system::
+As for CVS, there are three classes of file: unknown, ignored, and versioned.  The **add** command makes a file versioned: that is, changes to it will be recorded by the system::
 
     % echo 'hello world' > hello.txt
     % bzr status
-    ?       hello.txt
+    unknown:
+      hello.txt
     % bzr unknowns
     hello.txt
-    % bzr add -v hello.txt
-    A       hello.txt
+    % bzr add hello.txt
+    added hello.txt
     % bzr unknowns
 
 
-If you add the wrong file, simply use ``bzr remove`` to make
-it unversioned again.  This does not delete the working copy.
-
+If you add the wrong file, simply use **bzr remove** to make it unversioned again.  This does not delete the working copy.
+
+Branch locations
+================
+
+All history is stored in a branch, which is just an on-disk directory containing control files.  There is no repository or database as used in svn or svk.
+
+  (There are proposals to add shared storage between related branches.)
+
+You'll usually refer to branches on your computer's filesystem just by
+giving the name of the directory containing the branch.  bzr also supports
+accessing branches over http, for example::
+
+    % bzr log http://bazaar-ng.org/bzr/bzr.dev/
+
+By installing bzr plugins you can also access branches over the sftp or rsync protocols.
 
 Reviewing changes
 =================
 
-Once you have completed some work, you will want to *commit*
-it to the version history.  It is good to commit fairly often:
-whenever you get a new feature working, fix a bug, or improve some
-code or documentation.  It's also a good practice to make sure that
-the code compiles and passes its test suite before committing, to make
-sure that every revision is a known-good state.  You can also review
-your changes, to make sure you're committing what you intend to, and
-as a chance to rethink your work before you permanently record it. 
-
-Two bzr commands are particularly useful here: *status* and
-*diff*.  The *status* command
-shows a listing with one line per file, indicating whether it has been
-Added, Deleted, Modified, or Renamed in the current revision.  Unknown
-files are shown as '?'.  With the ``--all`` option, the status
-command also shows unmodified versioned files as '.', and ignored
-files as 'I'::
+Once you have completed some work, you will want to **commit** it to the version history.  It is good to commit fairly often: whenever you get a new feature working, fix a bug, or improve some code or documentation.  It's also a good practice to make sure that the code compiles and passes its test suite before committing, to make sure that every revision is a known-good state.  You can also review your changes, to make sure you're committing what you intend to, and as a chance to rethink your work before you permanently record it. 
+
+Two bzr commands are particularly useful here: **status** and **diff**.  
+
+bzr status
+----------
+
+The **status** command tells you what changes have been made to the
+working directory since the last revision::
 
     % bzr status
-    A       hello.txt
-
-The *diff* command shows the full text of changes to all
-files as a standard unified diff.  This can be piped through many
-programs such as ``patch``, ``diffstat``,
-``filterdiff`` and ``colordiff``::
+    modified:
+       foo
+
+By default **bzr status** hides "boring" files that are either unchanged or ignored.  To see them too, use the --all option.  The status command can optionally be given the name of some files or directories to check.
+
+bzr diff
+--------
+
+The **diff** command shows the full text of changes to all files as a standard unified diff.  This can be piped through many programs such as ''patch'', ''diffstat'', ''filterdiff'' and ''colordiff''::
 
     % bzr diff
     *** added file 'hello.txt'
@@ -133,93 +149,88 @@
     @@ -1,0 +1,1 @@
     +hello world
 
-With the ``-r`` option, the tree is compared to an earlier
-revision.
-
-[TODO: options to run external diff; to get context diff or other
-formats; to diff only selected files; to compare two historical
-revisions.]
-
-
+
+With the ''-r'' option, the tree is compared to an earlier revision, or
+the differences between two versions are shown::
+
+    % bzr diff -r 1000..          # everything since r1000
+    % bzr diff -r 1000..1100      # changes from 1000 to 1100
+
+The --diff-options option causes bzr to run the external diff program,
+passing options.  For example::
+
+    % bzr diff --diff-options --side-by-side foo
 
 Committing changes
 ==================
 
-When the working tree state is satisfactory, it can be
-*committed* to the branch, creating a new revision holding a
-snapshot of that state.  
-
-The ``commit`` command takes a message describing the changes
-in the revision.  It also records your userid, the current time and
-timezone, and the inventory and contents of the tree.  The commit
-message is specified by the ``-m`` or ``--message`` option.
-You can enter a multi-line commit message; in most shells you can
-enter this just by leaving the quotes open at the end of the line. ::
+When the working tree state is satisfactory, it can be **committed** to the branch, creating a new revision holding a snapshot of that state.  
+
+bzr commit
+----------
+
+The **commit** command takes a message describing the changes in the revision.  It also records your userid, the current time and timezone, and the inventory and contents of the tree.  The commit message is specified by the ''-m'' or ''--message'' option. You can enter a multi-line commit message; in most shells you can enter this just by leaving the quotes open at the end of the line.
+
+::
 
     % bzr commit -m "added my first file"
 
-[TODO: commit message interactively, through an editor or from a
-file.]
-
-[TODO: commit only selected files, including renamed/added/deleted
-files.]
-
+You can also use the -F option to take the message from a file.  Some people like to make notes for a commit message while they work, then review the diff to make sure they did what they said they did.  (This file can also be useful when you pick up your work after a break.)
+
+Message from an editor
+======================
+
+If you use neither the `-m` nor the `-F` option then bzr will open an editor for you to enter a message.  The editor to run is controlled by your `$EDITOR` environment variable or (Post Bazaar-NG 0.6) email setting in .  If you quit the editor without making any changes, the commit will be cancelled.
+
+Selective commit
+----------------
+
+If you give file or directory names on the commit command line then only
+the changes to those files will be committed.  For example::
+
+    % bzr commit -m "documentation fix" commit.py
+
+By default bzr always commits all changes to the tree, even if run from a
+subdirectory.  To commit from only the current directory down, use::
+
+    % bzr commit .
 
 
 Removing uncommitted changes
 ============================
 
-If you've made some changes and don't want to keep them, use the
-``revert`` command to go back to the previous head version.  It's a
-good idea to use ``bzr diff`` first to see what will be removed. 
-By default the revert command reverts the whole tree; if file or
-directory names are given then only those ones will be affected.
-revert also clears the list of pending merges revisions.
-
-
-
-
+If you've made some changes and don't want to keep them, use the **revert** command to go back to the previous head version.  It's a good idea to use **bzr diff** first to see what will be removed. By default the revert command reverts the whole tree; if file or directory names are given then only those ones will be affected. **revert** also clears the list of pending merges revisions.
 
 Ignoring files
 ==============
 
-Many source trees contain some files that do not need to be
-versioned, such as editor backups, object or bytecode files, and built
-programs.  You can simply not add them, but then they'll always crop
-up as unknown files.  You can also tell bzr to ignore these files by
-adding them to a file called ``.bzrignore`` at the top of the
-tree.
+Many source trees contain some files that do not need to be versioned, such as editor backups, object or bytecode files, and built programs.  You can simply not add them, but then they'll always crop up as unknown files.  You can also tell bzr to ignore these files by adding them to a file called ''.bzrignore'' at the top of the tree.
 
-This file contains a list of file wildcards (or "globs"), one
-per line.  Typical contents are like this::
+This file contains a list of file wildcards (or "globs"), one per line.  Typical contents are like this::
 
     *.o
     *~
     *.tmp
     *.py[co]
 
-If a glob contains a slash, it is matched against the whole path
-from the top of the tree; otherwise it is matched against only the
-filename.  So the previous example ignores ``*.o`` in all
-subdirectories, but this example ignores only config.h at the top
-level and HTML files in ``doc/``::
+If a glob contains a slash, it is matched against the whole path from the
+top of the tree; otherwise it is matched against only the filename.  So
+the previous example ignores files with extension ``.o`` in all
+subdirectories, but this example ignores only config.h at the top level
+and HTML files in ``doc/``::
 
     ./config.h
     doc/*.html
 
-To get a list of which files are ignored and what pattern they matched, use ``bzr ignored``::
+To get a list of which files are ignored and what pattern they matched, use ''bzr ignored''::
 
     % bzr ignored
-    config.h                                           ./config.h
-    configure.in~                                      *~
-
-It is OK to have an ignore pattern match a versioned file, or to
-add an ignored file.  Ignore patterns have no effect on versioned
-files; they only determine whether unversioned files are reported as
-unknown or ignored.
-
-The ``.bzrignore`` file should normally be versioned, so that new
-copies of the branch see the same patterns::
+    config.h                 ./config.h
+    configure.in~            *~
+
+It is OK to have either an ignore pattern match a versioned file, or to add an ignored file.  Ignore patterns have no effect on versioned files; they only determine whether unversioned files are reported as unknown or ignored.
+
+The ''.bzrignore'' file should normally be versioned, so that new copies of the branch see the same patterns::
 
     % bzr add .bzrignore
     % bzr commit -m "Add ignore patterns"
@@ -231,21 +242,26 @@
 bzr log
 -------
 
-The ``log`` command shows a list of previous revisions.
+The **bzr log** command shows a list of previous revisions. The **bzr log --forward** command does the same in chronological order to get most recent revisions printed at last.
+
+As with bzr diff, bzr log supports the -r argument::
+
+    % bzr log -r 1000..          # Revision 1000 and everything after it
+    % bzr log -r ..1000          # Everything up to and including revision % 1000
+    % bzr log -r 1000..1100      # changes from 1000 to 1100
+    % bzr log -r 1000            # The changes in only revision 1000
 
 
 Branch statistics
 =================
 
-The ``bzr info`` command shows some summary information about
-the working tree and the branch history.  
+The **bzr info** command shows some summary information about the working tree and the branch history.  
 
 
 Versioning directories
 ======================
 
-bzr versions files and directories in a way that can keep track of
-renames and intelligently merge them::
+bzr versions files and directories in a way that can keep track of renames and intelligently merge them::
 
     % mkdir src
     % echo 'int main() {}' > src/simple.c
@@ -261,14 +277,11 @@
 
 Deleting and removing files
 ===========================
-
-You can delete files or directories by just deleting them from the
-working directory.  This is a bit different to CVS, which requires
-that you also do *cvs remove*.
-
-*bzr remove* makes the file un-versioned, but does not
-delete the working copy.  This is useful when you add the wrong file,
-or decide that a file should actually not be versioned. ::
+You can delete files or directories by just deleting them from the working directory.  This is a bit different to CVS, which requires that you also do **cvs remove**.
+
+	**bzr remove** makes the file un-versioned, but does not delete the working copy.  This is useful when you add the wrong file, or decide that a file should actually not be versioned. 
+
+::
 
     % rm -r src
     % bzr remove -v hello.txt
@@ -278,35 +291,53 @@
     D       src/
     D       src/simple.c
 
+If you remove the wrong file by accident, you can use **bzr revert** to restore it.
+
 
 Branching
 =========
 
-Often rather than starting your own project, you will want to
-submit a change to an existing project.  You can get a copy of an
-existing branch by copying its directory, expanding a tarball, or by a
-remote copy using something like rsync.  You can also use bzr to fetch
-a copy.  Because this new copy is potentially a new branch, the
-command is called *branch*::
+Often rather than starting your own project, you will want to submit a change to an existing project.  You can get a copy of an existing branch by copying its directory, expanding a tarball, or by a remote copy using something like rsync.  You can also use bzr to fetch a copy.  Because this new copy is potentially a new branch, the command is called *branch*::
 
-    % bzr branch http://bazaar-ng.org/bzr/bzr.dev
+    % bzr branch http://bazaar-ng.org/bzr/bzr.dev 
     % cd bzr.dev
 
-This copies down the complete history of this branch, so we can
-do all operations on it locally: log, annotate, making and merging
-branches.  There will be an option to get only part of the history if
-you wish.
-
-
+This copies down the complete history of this branch, so we can do all operations on it locally: log, annotate, making and merging branches.  There will be an option to get only part of the history if you wish.
 
 Following upstream changes
 ==========================
 
-You can stay up-to-date with the parent branch by *pulling*
-in their changes::
+You can stay up-to-date with the parent branch by "pulling" in their changes::
 
     % bzr pull
 
-This only works if your local branch includes only changes from the
-parent branch.  Otherwise, the branches are said to have *diverged*,
-and they must be merged instead.
+After this change, the local directory will be a mirror of the source.
+
+This command only works if your local (destination) branch includes only
+changes from the parent branch and no commits of its own.  Otherwise, the
+branches are said to have ''diverged'', and they must be merged instead.
+
+Merging from related branches
+=============================
+
+If two branches have diverged (both have unique changes) then **bzr merge** is the appropriate command to use. Merge will automatically calculate the changes that exist in the branch you're merging from that are not in your branch and attempt to apply them in your branch.
+
+::
+
+  % bzr merge URL
+
+
+If there is a conflict during a merge, 3 files with the same basename are created. The filename of the common base is appended with .BASE, the filename of the file containing your changes is appended .THIS and the filename with the changes from the other tree is appended .OTHER.
+Using a program such as kdiff3, you can now comfortably merge them into one file. To commit you have to rename it to the original basename and delete the other two files. As long as there exist files with .BASE, .THIS or .OTHER the commit command will complain.
+
+[**TODO**: explain conflict markers within files]
+
+
+Publishing your branch
+======================
+You don't need a special server to publish a bzr branch, just a normal web server.  Just mirror the files to your server, including the .bzr directory.  One can push a branch (or the changes for a branch) by one of the following three methods:
+
+ * Rsync:   rsync -avrz LOCALBRANCH  servername.com/this/directory/here
+ * (Post 0.6): bzr push sftp://servername.com/this/directory/here (sftp://servername.com/this/directory/ must already exist)
+ * The push plugin that comes with BzrTools
+

